@@ -4,19 +4,20 @@
 
 #include "base/i18n/case_conversion.h"
 
+#include "base/string16.h"
 #include "unicode/unistr.h"
 
 namespace base {
 namespace i18n {
 
-string16 ToLower(const string16& string) {
-  icu::UnicodeString unicode_string(string.c_str(), string.size());
+string16 ToLower(const StringPiece16& string) {
+  icu::UnicodeString unicode_string(string.data(), string.size());
   unicode_string.toLower();
   return string16(unicode_string.getBuffer(), unicode_string.length());
 }
 
-string16 ToUpper(const string16& string) {
-  icu::UnicodeString unicode_string(string.c_str(), string.size());
+string16 ToUpper(const StringPiece16& string) {
+  icu::UnicodeString unicode_string(string.data(), string.size());
   unicode_string.toUpper();
   return string16(unicode_string.getBuffer(), unicode_string.length());
 }
