@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,11 +6,13 @@
 #define BASE_I18N_ICU_UTIL_H_
 #pragma once
 
+#include "base/i18n/base_i18n_export.h"
+
 namespace icu_util {
 
 // Call this function to load ICU's data tables for the current process.  This
 // function should be called before ICU is used.
-bool Initialize();
+BASE_I18N_EXPORT bool Initialize();
 
 }  // namespace icu_util
 
