@@ -120,7 +120,8 @@ TEST(MD5, ContextWithLongData) {
     if (len > length - total)
       len = length - total;
 
-    MD5Update(&ctx, data.get() + total, len);
+    MD5Update(&ctx,
+              StringPiece(reinterpret_cast<char*>(data.get() + total), len));
     total += len;
   }
 
@@ -188,4 +189,19 @@ TEST(MD5, MD5StringTestSuite7) {
   EXPECT_EQ(expected, actual);
 }
 
+TEST(MD5, ContextWithStringData) {
+  MD5Context ctx;
+  MD5Init(&ctx);
+
+  MD5Update(&ctx, "abc");
+
+  MD5Digest digest;
+  MD5Final(&digest, &ctx);
+
+  std::string actual = MD5DigestToBase16(digest);
+  std::string expected = "900150983cd24fb0d6963f7d28e17f72";
+
+  EXPECT_EQ(expected, actual);
+}
+
 }  // namespace base
