@@ -8,6 +8,7 @@
 
 #include <fcntl.h>
 #include <sys/types.h>
+#include <time64.h>
 #include <utime.h>
 
 // Not implemented in Bionic. See platform_file_android.cc.
@@ -23,4 +24,9 @@ inline int lockf(int fd, int cmd, off_t ignored_len) {
   return flock(fd, cmd);
 }
 
+// Android has only timegm64() and no timegm().
+inline time_t timegm(struct tm* const tmp) {
+  return static_cast<time_t>(timegm64(tmp));
+}
+
 #endif  // BASE_OS_COMPAT_ANDROID_H_
