@@ -10,6 +10,7 @@
 #include "base/logging.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/safe_strerror_posix.h"
+#include "base/threading/thread_local.h"
 #include "base/threading/thread_restrictions.h"
 
 #if defined(OS_MACOSX)
@@ -25,6 +26,10 @@
 #include <unistd.h>
 #endif
 
+#if defined(OS_ANDROID)
+#include "base/android/jni_android.h"
+#endif
+
 #if defined(OS_NACL)
 #include <sys/nacl_syscalls.h>
 #endif
@@ -37,6 +42,8 @@ void InitThreading();
 
 namespace {
 
+static ThreadLocalPointer<char> current_thread_name;
+
 struct ThreadParams {
   PlatformThread::Delegate* delegate;
   bool joinable;
@@ -49,6 +56,9 @@ void* ThreadFunc(void* params) {
     base::ThreadRestrictions::SetSingletonAllowed(false);
   delete thread_params;
   delegate->ThreadMain();
+#if defined(OS_ANDROID)
+  base::android::DetachFromVM();
+#endif
   return NULL;
 }
 
@@ -124,6 +134,8 @@ PlatformThreadId PlatformThread::CurrentId() {
   return mach_thread_self();
 #elif defined(OS_LINUX)
   return syscall(__NR_gettid);
+#elif defined(OS_ANDROID)
+  return gettid();
 #elif defined(OS_FREEBSD)
   // TODO(BSD): find a better thread ID
   return reinterpret_cast<int64>(pthread_self());
@@ -159,6 +171,10 @@ void PlatformThread::Sleep(int duration_ms) {
 #if 0 && defined(OS_LINUX)
 // static
 void PlatformThread::SetName(const char* name) {
+  // have to cast away const because ThreadLocalPointer does not support const
+  // void*
+  current_thread_name.Set(const_cast<char*>(name));
+
   // http://0pointer.de/blog/projects/name-your-threads.html
 
   // glibc recently added support for pthread_setname_np, but it's not
@@ -189,14 +205,25 @@ void PlatformThread::SetName(const char* name) {
 // Mac is implemented in platform_thread_mac.mm.
 #else
 // static
-void PlatformThread::SetName(const char* /*name*/) {
-  // Leave it unimplemented.
+void PlatformThread::SetName(const char* name) {
+  // have to cast away const because ThreadLocalPointer does not support const
+  // void*
+  current_thread_name.Set(const_cast<char*>(name));
 
   // (This should be relatively simple to implement for the BSDs; I
   // just don't have one handy to test the code on.)
 }
 #endif  // defined(OS_LINUX)
 
+
+#if !defined(OS_MACOSX)
+// Mac is implemented in platform_thread_mac.mm.
+// static
+const char* PlatformThread::GetName() {
+  return current_thread_name.Get();
+}
+#endif
+
 // static
 bool PlatformThread::Create(size_t stack_size, Delegate* delegate,
                             PlatformThreadHandle* thread_handle) {
