@@ -5,6 +5,7 @@
 #include "chrome/browser/alternate_nav_url_fetcher.h"
 
 #include "base/utf_string_conversions.h"
+#include "chrome/browser/infobars/infobar_tab_helper.h"
 #include "chrome/browser/intranet_redirect_detector.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
@@ -61,7 +62,7 @@ void AlternateNavURLFetcher::Observe(int type,
         fetcher_.reset(new URLFetcher(GURL(alternate_nav_url_),
                                       URLFetcher::HEAD, this));
         fetcher_->set_request_context(
-            controller_->profile()->GetRequestContext());
+            controller_->browser_context()->GetRequestContext());
         fetcher_->Start();
       }
       break;
@@ -166,5 +167,5 @@ void AlternateNavURLFetcher::ShowInfobarIfPossible() {
   infobar_contents_ = controller_->tab_contents();
   StoreActiveEntryUniqueID(infobar_contents_);
   TabContentsWrapper::GetCurrentWrapperForContents(infobar_contents_)->
-      AddInfoBar(this);
+      infobar_tab_helper()->AddInfoBar(this);
 }
