@@ -12,16 +12,19 @@
 #include "base/message_loop.h"
 #include "base/string_number_conversions.h"
 #include "base/sys_string_conversions.h"
+#include "base/utf_string_conversions.h"
 #include "chrome/app/chrome_command_ids.h"
 #include "chrome/browser/background/background_application_list_model.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/browser_shutdown.h"
 #include "chrome/browser/command_updater.h"
-#include "chrome/browser/download/download_manager.h"
 #include "chrome/browser/instant/instant_confirm_dialog.h"
 #include "chrome/browser/prefs/pref_service.h"
+#include "chrome/browser/printing/cloud_print/virtual_driver_install_helper.h"
+#include "chrome/browser/printing/print_dialog_cloud.h"
 #include "chrome/browser/printing/print_job_manager.h"
 #include "chrome/browser/profiles/profile_manager.h"
+#include "chrome/browser/service/service_process_control.h"
 #include "chrome/browser/sessions/session_service.h"
 #include "chrome/browser/sessions/session_service_factory.h"
 #include "chrome/browser/sessions/tab_restore_service.h"
@@ -37,7 +40,6 @@
 #import "chrome/browser/ui/cocoa/bookmarks/bookmark_menu_bridge.h"
 #import "chrome/browser/ui/cocoa/browser_window_cocoa.h"
 #import "chrome/browser/ui/cocoa/browser_window_controller.h"
-#import "chrome/browser/ui/cocoa/bug_report_window_controller.h"
 #import "chrome/browser/ui/cocoa/confirm_quit_panel_controller.h"
 #import "chrome/browser/ui/cocoa/encoding_menu_controller_delegate_mac.h"
 #import "chrome/browser/ui/cocoa/history_menu_bridge.h"
@@ -46,14 +48,17 @@
 #import "chrome/browser/ui/cocoa/tabs/tab_window_controller.h"
 #include "chrome/browser/ui/cocoa/task_manager_mac.h"
 #include "chrome/common/chrome_notification_types.h"
-#include "chrome/common/app_mode_common_mac.h"
 #include "chrome/common/chrome_paths_internal.h"
 #include "chrome/common/chrome_switches.h"
+#include "chrome/common/mac/app_mode_common.h"
 #include "chrome/common/pref_names.h"
+#include "chrome/common/service_messages.h"
 #include "chrome/common/url_constants.h"
 #include "content/browser/browser_thread.h"
+#include "content/browser/download/download_manager.h"
 #include "content/browser/tab_contents/tab_contents.h"
 #include "content/browser/user_metrics.h"
+#include "content/common/cloud_print_class_mac.h"
 #include "content/common/content_notification_types.h"
 #include "content/common/notification_service.h"
 #include "grit/chromium_strings.h"
@@ -153,6 +158,9 @@ void RecordLastRunAppBundlePath() {
 
 }  // anonymous namespace
 
+const AEEventClass kAECloudPrintInstallClass = 'GCPi';
+const AEEventClass kAECloudPrintUninstallClass = 'GCPu';
+
 @interface AppController (Private)
 - (void)initMenuState;
 - (void)initProfileMenu;
@@ -161,6 +169,9 @@ void RecordLastRunAppBundlePath() {
 - (void)openUrls:(const std::vector<GURL>&)urls;
 - (void)getUrl:(NSAppleEventDescriptor*)event
      withReply:(NSAppleEventDescriptor*)reply;
+- (void)submitCloudPrintJob:(NSAppleEventDescriptor*)event;
+- (void)installCloudPrint:(NSAppleEventDescriptor*)event;
+- (void)uninstallCloudPrint:(NSAppleEventDescriptor*)event;
 - (void)windowLayeringDidChange:(NSNotification*)inNotification;
 - (void)windowChangedToProfile:(Profile*)profile;
 - (void)checkForAnyKeyWindows;
@@ -183,6 +194,19 @@ void RecordLastRunAppBundlePath() {
           andSelector:@selector(getUrl:withReply:)
         forEventClass:kInternetEventClass
            andEventID:kAEGetURL];
+  [em setEventHandler:self
+          andSelector:@selector(submitCloudPrintJob:)
+        forEventClass:content::kAECloudPrintClass
+           andEventID:content::kAECloudPrintClass];
+  // Install and uninstall handlers for virtual drivers.
+  [em setEventHandler:self
+          andSelector:@selector(installCloudPrint:)
+        forEventClass:kAECloudPrintInstallClass
+           andEventID:kAECloudPrintInstallClass];
+  [em setEventHandler:self
+          andSelector:@selector(uninstallCloudPrint:)
+        forEventClass:kAECloudPrintUninstallClass
+           andEventID:kAECloudPrintUninstallClass];
   [em setEventHandler:self
           andSelector:@selector(getUrl:withReply:)
         forEventClass:'WWW!'    // A particularly ancient AppleEvent that dates
@@ -718,6 +742,9 @@ void RecordLastRunAppBundlePath() {
           sync_ui_util::UpdateSyncItem(item, enable, lastProfile);
           break;
         }
+        case IDC_FEEDBACK:
+          enable = NO;
+          break;
         default:
           enable = menuState_->IsCommandEnabled(tag) ?
                    [self keyWindowIsNotModal] : NO;
@@ -836,17 +863,6 @@ void RecordLastRunAppBundlePath() {
       else
         Browser::OpenHelpWindow(lastProfile);
       break;
-    case IDC_FEEDBACK: {
-      Browser* browser = BrowserList::GetLastActive();
-      TabContents* currentTab =
-          browser ? browser->GetSelectedTabContents() : NULL;
-      BugReportWindowController* controller =
-          [[BugReportWindowController alloc]
-              initWithTabContents:currentTab
-                          profile:[self lastProfile]];
-      [controller runModalDialog];
-      break;
-    }
     case IDC_SYNC_BOOKMARKS:
       // The profile may be NULL during shutdown -- see
       // http://code.google.com/p/chromium/issues/detail?id=43048 .
@@ -1070,6 +1086,40 @@ void RecordLastRunAppBundlePath() {
   [self openUrls:gurlVector];
 }
 
+// Apple Event handler that receives print event from service
+// process, gets the required data and launches Print dialog.
+- (void)submitCloudPrintJob:(NSAppleEventDescriptor*)event {
+  // Pull parameter list out of Apple Event.
+  NSAppleEventDescriptor *paramList =
+      [event paramDescriptorForKeyword:content::kAECloudPrintClass];
+
+  if (paramList != nil) {
+    // Pull required fields out of parameter list.
+    NSString* mime = [[paramList descriptorAtIndex:1] stringValue];
+    NSString* inputPath = [[paramList descriptorAtIndex:2] stringValue];
+    NSString* printTitle = [[paramList descriptorAtIndex:3] stringValue];
+    NSString* printTicket = [[paramList descriptorAtIndex:4] stringValue];
+    // Convert the title to UTF 16 as required.
+    string16 title16 = base::SysNSStringToUTF16(printTitle);
+    string16 printTicket16 = base::SysNSStringToUTF16(printTicket);
+    print_dialog_cloud::CreatePrintDialogForFile(
+        FilePath([inputPath UTF8String]), title16,
+        printTicket16, [mime UTF8String], /*modal=*/false);
+  }
+}
+
+// Calls the helper class to install the virtual driver to the
+// service process.
+- (void)installCloudPrint:(NSAppleEventDescriptor*)event {
+  cloud_print::VirtualDriverInstallHelper::SetUpInstall();
+}
+
+// Calls the helper class to uninstall the virtual driver to the
+// service process.
+- (void)uninstallCloudPrint:(NSAppleEventDescriptor*)event {
+  cloud_print::VirtualDriverInstallHelper::SetUpUninstall();
+}
+
 - (void)application:(NSApplication*)sender
           openFiles:(NSArray*)filenames {
   std::vector<GURL> gurlVector;
