@@ -25,11 +25,9 @@
 
 AutocompletePopupModel::AutocompletePopupModel(
     AutocompletePopupView* popup_view,
-    AutocompleteEditModel* edit_model,
-    Profile* profile)
+    AutocompleteEditModel* edit_model)
     : view_(popup_view),
       edit_model_(edit_model),
-      profile_(profile),
       hovered_line_(kNoMatch),
       selected_line_(kNoMatch) {
   edit_model->set_popup_model(this);
@@ -134,27 +132,12 @@ bool AutocompletePopupModel::GetKeywordForMatch(const AutocompleteMatch& match,
   // Assume we have no keyword until we find otherwise.
   keyword->clear();
 
-  if (match.template_url) {
-    TemplateURLService* url_service =
-        TemplateURLServiceFactory::GetForProfile(profile_);
-    if (!url_service)
-      return false;
-
-    // Only show the keyword for the default provider if the user typed in
-    // the keyword and it isn't SEARCH_WHAT_YOU_TYPED.
-    const TemplateURL* default_url = url_service->GetDefaultSearchProvider();
-    if (default_url && (default_url->id() == match.template_url->id())) {
-      if (StartsWith(autocomplete_controller()->input().text(),
-                     default_url->keyword(), false) &&
-          (match.type != AutocompleteMatch::SEARCH_WHAT_YOU_TYPED)) {
-        keyword->assign(match.template_url->keyword());
-        return false;
-      }
-    } else if (TemplateURL::SupportsReplacement(match.template_url)) {
-      // The current match is a keyword, return that as the selected keyword.
-      keyword->assign(match.template_url->keyword());
-      return false;
-    }
+  if (match.template_url &&
+      TemplateURL::SupportsReplacement(match.template_url) &&
+      match.transition == PageTransition::KEYWORD) {
+    // The current match is a keyword, return that as the selected keyword.
+    keyword->assign(match.template_url->keyword());
+    return false;
   }
 
   // See if the current match's fill_into_edit corresponds to a keyword.
@@ -171,8 +154,9 @@ bool AutocompletePopupModel::GetKeywordForText(const string16& text,
 
   if (keyword_hint.empty())
     return false;
+  Profile* profile = edit_model_->profile();
   TemplateURLService* url_service =
-      TemplateURLServiceFactory::GetForProfile(profile_);
+      TemplateURLServiceFactory::GetForProfile(profile);
   if (!url_service)
     return false;
   url_service->Load();
@@ -185,12 +169,10 @@ bool AutocompletePopupModel::GetKeywordForText(const string16& text,
 
   // Don't provide a hint for inactive/disabled extension keywords.
   if (template_url->IsExtensionKeyword()) {
-    const Extension* extension = profile_->GetExtensionService()->
+    const Extension* extension = profile->GetExtensionService()->
         GetExtensionById(template_url->GetExtensionId(), false);
-    if (!extension ||
-        (profile_->IsOffTheRecord() &&
-         !profile_->GetExtensionService()->
-             IsIncognitoEnabled(extension->id())))
+    if (!extension || (profile->IsOffTheRecord() &&
+        !profile->GetExtensionService()->IsIncognitoEnabled(extension->id())))
       return false;
   }
 
@@ -250,7 +232,7 @@ const SkBitmap* AutocompletePopupModel::GetIconIfExtensionMatch(
   if (!match.template_url || !match.template_url->IsExtensionKeyword())
     return NULL;
 
-  return &profile_->GetExtensionService()->GetOmniboxPopupIcon(
+  return &edit_model_->profile()->GetExtensionService()->GetOmniboxPopupIcon(
       match.template_url->GetExtensionId());
 }
 
