@@ -8,12 +8,11 @@
 #include "chrome/browser/autocomplete/keyword_provider.h"
 #include "chrome/browser/search_engines/template_url.h"
 #include "chrome/browser/search_engines/template_url_service.h"
-#include "chrome/test/testing_browser_process.h"
-#include "chrome/test/testing_browser_process_test.h"
+#include "chrome/test/base/testing_browser_process.h"
 #include "googleurl/src/gurl.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
-class KeywordProviderTest : public TestingBrowserProcessTest {
+class KeywordProviderTest : public testing::Test {
  protected:
   template<class ResultType>
   struct test_data {
