@@ -21,15 +21,15 @@
 #include "chrome/browser/history/url_database.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/common/pref_names.h"
-#include "chrome/test/testing_browser_process.h"
-#include "chrome/test/testing_browser_process_test.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/testing_profile.h"
 #include "content/browser/browser_thread.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 using base::Time;
 using base::TimeDelta;
 
+namespace {
+
 struct TestShortcutInfo {
   std::string url;
   std::string title;  // The text that orginally was searched for.
@@ -95,7 +95,9 @@ struct TestShortcutInfo {
     "Test - site", "0,0", 1, 4},
 };
 
-class ShortcutsProviderTest : public TestingBrowserProcessTest,
+}  // end namespace
+
+class ShortcutsProviderTest : public testing::Test,
                               public ACProviderListener {
  public:
   ShortcutsProviderTest();
