@@ -10,9 +10,8 @@
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/autocomplete_history_manager.h"
 #include "chrome/browser/webdata/web_data_service.h"
-#include "chrome/test/testing_browser_process.h"
-#include "chrome/test/testing_browser_process_test.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/testing_browser_process.h"
+#include "chrome/test/base/testing_profile.h"
 #include "content/browser/renderer_host/test_render_view_host.h"
 #include "content/browser/tab_contents/test_tab_contents.h"
 #include "testing/gmock/include/gmock/gmock.h"
@@ -58,12 +57,11 @@ TEST_F(AutocompleteHistoryManagerTest, CreditCardNumberValue) {
   form.user_submitted = true;
 
   // Valid Visa credit card number pulled from the paypal help site.
-  webkit_glue::FormField valid_cc(ASCIIToUTF16("Credit Card"),
-                                  ASCIIToUTF16("ccnum"),
-                                  ASCIIToUTF16("4012888888881881"),
-                                  ASCIIToUTF16("text"),
-                                  20,
-                                  false);
+  webkit_glue::FormField valid_cc;
+  valid_cc.label = ASCIIToUTF16("Credit Card");
+  valid_cc.name = ASCIIToUTF16("ccnum");
+  valid_cc.value = ASCIIToUTF16("4012888888881881");
+  valid_cc.form_control_type = ASCIIToUTF16("text");
   form.fields.push_back(valid_cc);
 
   EXPECT_CALL(*web_data_service_, AddFormFields(_)).Times(0);
@@ -82,12 +80,11 @@ TEST_F(AutocompleteHistoryManagerTest, NonCreditCardNumberValue) {
   form.user_submitted = true;
 
   // Invalid credit card number.
-  webkit_glue::FormField invalid_cc(ASCIIToUTF16("Credit Card"),
-                                    ASCIIToUTF16("ccnum"),
-                                    ASCIIToUTF16("4580123456789012"),
-                                    ASCIIToUTF16("text"),
-                                    20,
-                                    false);
+  webkit_glue::FormField invalid_cc;
+  invalid_cc.label = ASCIIToUTF16("Credit Card");
+  invalid_cc.name = ASCIIToUTF16("ccnum");
+  invalid_cc.value = ASCIIToUTF16("4580123456789012");
+  invalid_cc.form_control_type = ASCIIToUTF16("text");
   form.fields.push_back(invalid_cc);
 
   EXPECT_CALL(*(web_data_service_.get()), AddFormFields(_)).Times(1);
@@ -103,12 +100,11 @@ TEST_F(AutocompleteHistoryManagerTest, SSNValue) {
   form.action = GURL("http://myform.com/submit.html");
   form.user_submitted = true;
 
-  webkit_glue::FormField ssn(ASCIIToUTF16("Social Security Number"),
-                             ASCIIToUTF16("ssn"),
-                             ASCIIToUTF16("078-05-1120"),
-                             ASCIIToUTF16("text"),
-                             20,
-                             false);
+  webkit_glue::FormField ssn;
+  ssn.label = ASCIIToUTF16("Social Security Number");
+  ssn.name = ASCIIToUTF16("ssn");
+  ssn.value = ASCIIToUTF16("078-05-1120");
+  ssn.form_control_type = ASCIIToUTF16("text");
   form.fields.push_back(ssn);
 
   EXPECT_CALL(*web_data_service_, AddFormFields(_)).Times(0);
@@ -124,13 +120,12 @@ TEST_F(AutocompleteHistoryManagerTest, SearchField) {
   form.action = GURL("http://myform.com/submit.html");
   form.user_submitted = true;
 
-  // Invalid credit card number.
-  webkit_glue::FormField search_field(ASCIIToUTF16("Search"),
-                                      ASCIIToUTF16("search"),
-                                      ASCIIToUTF16("my favorite query"),
-                                      ASCIIToUTF16("search"),
-                                      20,
-                                      false);
+  // Search field.
+  webkit_glue::FormField search_field;
+  search_field.label = ASCIIToUTF16("Search");
+  search_field.name = ASCIIToUTF16("search");
+  search_field.value = ASCIIToUTF16("my favorite query");
+  search_field.form_control_type = ASCIIToUTF16("search");
   form.fields.push_back(search_field);
 
   EXPECT_CALL(*(web_data_service_.get()), AddFormFields(_)).Times(1);
