@@ -12,11 +12,10 @@
 #include "chrome/browser/autofill/autofill_metrics.h"
 #include "chrome/browser/autofill/autofill_type.h"
 #include "chrome/browser/autofill/form_structure.h"
-#include "chrome/test/test_url_request_context_getter.h"
-#include "chrome/test/testing_browser_process.h"
-#include "chrome/test/testing_browser_process_test.h"
-#include "chrome/test/testing_profile.h"
-#include "content/common/test_url_fetcher_factory.h"
+#include "chrome/test/base/test_url_request_context_getter.h"
+#include "chrome/test/base/testing_browser_process.h"
+#include "chrome/test/base/testing_profile.h"
+#include "content/test/test_url_fetcher_factory.h"
 #include "net/url_request/url_request_status.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -24,6 +23,7 @@
 #include "webkit/glue/form_data.h"
 
 using webkit_glue::FormData;
+using webkit_glue::FormField;
 using WebKit::WebInputElement;
 
 namespace {
@@ -68,19 +68,18 @@ class AutofillDownloadTestHelper : public AutofillDownloadManager::Observer {
   }
 
   // AutofillDownloadManager::Observer overridables:
-  virtual void OnLoadedAutofillHeuristics(
-      const std::string& heuristic_xml) {
+  virtual void OnLoadedServerPredictions(const std::string& response_xml) {
     ResponseData response;
-    response.response = heuristic_xml;
+    response.response = response_xml;
     response.type_of_response = QUERY_SUCCESSFULL;
     responses_.push_back(response);
   };
-  virtual void OnUploadedAutofillHeuristics(const std::string& form_signature) {
+  virtual void OnUploadedPossibleFieldTypes() {
     ResponseData response;
     response.type_of_response = UPLOAD_SUCCESSFULL;
     responses_.push_back(response);
   }
-  virtual void OnHeuristicsRequestError(
+  virtual void OnServerRequestError(
       const std::string& form_signature,
       AutofillDownloadManager::AutofillRequestType request_type,
       int http_error) {
@@ -115,89 +114,79 @@ class AutofillDownloadTestHelper : public AutofillDownloadManager::Observer {
   scoped_refptr<net::URLRequestContextGetter> request_context_getter;
 };
 
-typedef TestingBrowserProcessTest AutofillDownloadTest;
+typedef testing::Test AutofillDownloadTest;
 
 TEST_F(AutofillDownloadTest, QueryAndUploadTest) {
   MessageLoopForUI message_loop;
   // Create and register factory.
   AutofillDownloadTestHelper helper;
   TestURLFetcherFactory factory;
-  URLFetcher::set_factory(&factory);
 
   FormData form;
   form.method = ASCIIToUTF16("post");
-  form.fields.push_back(webkit_glue::FormField(ASCIIToUTF16("username"),
-                                               ASCIIToUTF16("username"),
-                                               string16(),
-                                               ASCIIToUTF16("text"),
-                                               0,
-                                               false));
-  form.fields.push_back(webkit_glue::FormField(ASCIIToUTF16("First Name"),
-                                               ASCIIToUTF16("firstname"),
-                                               string16(),
-                                               ASCIIToUTF16("text"),
-                                               0,
-                                               false));
-  form.fields.push_back(webkit_glue::FormField(ASCIIToUTF16("Last Name"),
-                                               ASCIIToUTF16("lastname"),
-                                               string16(),
-                                               ASCIIToUTF16("text"),
-                                               0,
-                                               false));
-  form.fields.push_back(webkit_glue::FormField(ASCIIToUTF16("email"),
-                                               ASCIIToUTF16("email"),
-                                               string16(),
-                                               ASCIIToUTF16("text"),
-                                               0,
-                                               false));
-  form.fields.push_back(webkit_glue::FormField(ASCIIToUTF16("email2"),
-                                               ASCIIToUTF16("email2"),
-                                               string16(),
-                                               ASCIIToUTF16("text"),
-                                               0,
-                                               false));
-  form.fields.push_back(webkit_glue::FormField(ASCIIToUTF16("password"),
-                                               ASCIIToUTF16("password"),
-                                               string16(),
-                                               ASCIIToUTF16("password"),
-                                               0,
-                                               false));
-  form.fields.push_back(webkit_glue::FormField(string16(),
-                                               ASCIIToUTF16("Submit"),
-                                               string16(),
-                                               ASCIIToUTF16("submit"),
-                                               0,
-                                               false));
+
+  FormField field;
+  field.label = ASCIIToUTF16("username");
+  field.name = ASCIIToUTF16("username");
+  field.form_control_type = ASCIIToUTF16("text");
+  form.fields.push_back(field);
+
+  field.label = ASCIIToUTF16("First Name");
+  field.name = ASCIIToUTF16("firstname");
+  field.form_control_type = ASCIIToUTF16("text");
+  form.fields.push_back(field);
+
+  field.label = ASCIIToUTF16("Last Name");
+  field.name = ASCIIToUTF16("lastname");
+  field.form_control_type = ASCIIToUTF16("text");
+  form.fields.push_back(field);
+
+  field.label = ASCIIToUTF16("email");
+  field.name = ASCIIToUTF16("email");
+  field.form_control_type = ASCIIToUTF16("text");
+  form.fields.push_back(field);
+
+  field.label = ASCIIToUTF16("email2");
+  field.name = ASCIIToUTF16("email2");
+  field.form_control_type = ASCIIToUTF16("text");
+  form.fields.push_back(field);
+
+  field.label = ASCIIToUTF16("password");
+  field.name = ASCIIToUTF16("password");
+  field.form_control_type = ASCIIToUTF16("password");
+  form.fields.push_back(field);
+
+  field.label = string16();
+  field.name = ASCIIToUTF16("Submit");
+  field.form_control_type = ASCIIToUTF16("submit");
+  form.fields.push_back(field);
 
   FormStructure *form_structure = new FormStructure(form);
   ScopedVector<FormStructure> form_structures;
   form_structures.push_back(form_structure);
 
   form.fields.clear();
-  form.fields.push_back(webkit_glue::FormField(ASCIIToUTF16("address"),
-                                               ASCIIToUTF16("address"),
-                                               string16(),
-                                               ASCIIToUTF16("text"),
-                                               0,
-                                               false));
-  form.fields.push_back(webkit_glue::FormField(ASCIIToUTF16("address2"),
-                                               ASCIIToUTF16("address2"),
-                                               string16(),
-                                               ASCIIToUTF16("text"),
-                                               0,
-                                               false));
-  form.fields.push_back(webkit_glue::FormField(ASCIIToUTF16("city"),
-                                               ASCIIToUTF16("city"),
-                                               string16(),
-                                               ASCIIToUTF16("text"),
-                                               0,
-                                               false));
-  form.fields.push_back(webkit_glue::FormField(string16(),
-                                               ASCIIToUTF16("Submit"),
-                                               string16(),
-                                               ASCIIToUTF16("submit"),
-                                               0,
-                                               false));
+
+  field.label = ASCIIToUTF16("address");
+  field.name = ASCIIToUTF16("address");
+  field.form_control_type = ASCIIToUTF16("text");
+  form.fields.push_back(field);
+
+  field.label = ASCIIToUTF16("address2");
+  field.name = ASCIIToUTF16("address2");
+  field.form_control_type = ASCIIToUTF16("text");
+  form.fields.push_back(field);
+
+  field.label = ASCIIToUTF16("city");
+  field.name = ASCIIToUTF16("city");
+  field.form_control_type = ASCIIToUTF16("text");
+  form.fields.push_back(field);
+
+  field.label = string16();
+  field.name = ASCIIToUTF16("Submit");
+  field.form_control_type = ASCIIToUTF16("submit");
+  form.fields.push_back(field);
+
   form_structure = new FormStructure(form);
   form_structures.push_back(form_structure);
 
@@ -302,12 +291,10 @@ TEST_F(AutofillDownloadTest, QueryAndUploadTest) {
   EXPECT_EQ(NULL, fetcher);
 
   // Modify form structures to miss the cache.
-  form.fields.push_back(webkit_glue::FormField(ASCIIToUTF16("Address line 2"),
-                                               ASCIIToUTF16("address2"),
-                                               string16(),
-                                               ASCIIToUTF16("text"),
-                                               0,
-                                               false));
+  field.label = ASCIIToUTF16("Address line 2");
+  field.name = ASCIIToUTF16("address2");
+  field.form_control_type = ASCIIToUTF16("text");
+  form.fields.push_back(field);
   form_structure = new FormStructure(form);
   form_structures.push_back(form_structure);
 
@@ -339,8 +326,8 @@ TEST_F(AutofillDownloadTest, QueryAndUploadTest) {
   fetcher = factory.GetFetcherByID(4);
   EXPECT_EQ(NULL, fetcher);
 
-  // Set upload to 100% so requests happen.
-  helper.download_manager.SetPositiveUploadRate(1.0);
+  // Set upload required to true so requests happen.
+  form_structures[0]->upload_required_ = UPLOAD_REQUIRED;
   // Request with id 4.
   EXPECT_TRUE(helper.download_manager.StartUploadRequest(*(form_structures[0]),
                                                          true,
@@ -364,9 +351,6 @@ TEST_F(AutofillDownloadTest, QueryAndUploadTest) {
                                                           FieldTypeSet()));
   fetcher = factory.GetFetcherByID(5);
   EXPECT_EQ(NULL, fetcher);
-
-  // Make sure consumer of URLFetcher does the right thing.
-  URLFetcher::set_factory(NULL);
 }
 
 TEST_F(AutofillDownloadTest, CacheQueryTest) {
@@ -374,51 +358,43 @@ TEST_F(AutofillDownloadTest, CacheQueryTest) {
   AutofillDownloadTestHelper helper;
   // Create and register factory.
   TestURLFetcherFactory factory;
-  URLFetcher::set_factory(&factory);
   helper.InitContextGetter();
 
   FormData form;
   form.method = ASCIIToUTF16("post");
-  form.fields.push_back(webkit_glue::FormField(ASCIIToUTF16("username"),
-                                               ASCIIToUTF16("username"),
-                                               string16(),
-                                               ASCIIToUTF16("text"),
-                                               0,
-                                               false));
-  form.fields.push_back(webkit_glue::FormField(ASCIIToUTF16("First Name"),
-                                               ASCIIToUTF16("firstname"),
-                                               string16(),
-                                               ASCIIToUTF16("text"),
-                                               0,
-                                               false));
-  form.fields.push_back(webkit_glue::FormField(ASCIIToUTF16("Last Name"),
-                                               ASCIIToUTF16("lastname"),
-                                               string16(),
-                                               ASCIIToUTF16("text"),
-                                               0,
-                                               false));
+
+  FormField field;
+  field.form_control_type = ASCIIToUTF16("text");
+
+  field.label = ASCIIToUTF16("username");
+  field.name = ASCIIToUTF16("username");
+  form.fields.push_back(field);
+
+  field.label = ASCIIToUTF16("First Name");
+  field.name = ASCIIToUTF16("firstname");
+  form.fields.push_back(field);
+
+  field.label = ASCIIToUTF16("Last Name");
+  field.name = ASCIIToUTF16("lastname");
+  form.fields.push_back(field);
+
   FormStructure *form_structure = new FormStructure(form);
   ScopedVector<FormStructure> form_structures0;
   form_structures0.push_back(form_structure);
 
-  form.fields.push_back(webkit_glue::FormField(ASCIIToUTF16("email"),
-                                               ASCIIToUTF16("email"),
-                                               string16(),
-                                               ASCIIToUTF16("text"),
-                                               0,
-                                               false));
-  // Slightly different form - so different request.
+  // Add a slightly different form, which should result in a different request.
+  field.label = ASCIIToUTF16("email");
+  field.name = ASCIIToUTF16("email");
+  form.fields.push_back(field);
   form_structure = new FormStructure(form);
   ScopedVector<FormStructure> form_structures1;
   form_structures1.push_back(form_structure);
 
-  form.fields.push_back(webkit_glue::FormField(ASCIIToUTF16("email2"),
-                                               ASCIIToUTF16("email2"),
-                                               string16(),
-                                               ASCIIToUTF16("text"),
-                                               0,
-                                               false));
-  // Slightly different form - so different request.
+  // Add another slightly different form, which should also result in a
+  // different request.
+  field.label = ASCIIToUTF16("email2");
+  field.name = ASCIIToUTF16("email2");
+  form.fields.push_back(field);
   form_structure = new FormStructure(form);
   ScopedVector<FormStructure> form_structures2;
   form_structures2.push_back(form_structure);
@@ -546,8 +522,5 @@ TEST_F(AutofillDownloadTest, CacheQueryTest) {
                                           std::string(responses[0]));
   ASSERT_EQ(static_cast<size_t>(1), helper.responses_.size());
   EXPECT_EQ(responses[0], helper.responses_.front().response);
-
-  // Make sure consumer of URLFetcher does the right thing.
-  URLFetcher::set_factory(NULL);
 }
 
