@@ -7,6 +7,7 @@
 #include "base/memory/ref_counted.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/string16.h"
+#include "base/time.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/autofill/autofill_cc_infobar_delegate.h"
 #include "chrome/browser/autofill/autofill_common_test.h"
@@ -14,13 +15,19 @@
 #include "chrome/browser/autofill/autofill_metrics.h"
 #include "chrome/browser/autofill/personal_data_manager.h"
 #include "chrome/browser/webdata/web_data_service.h"
-#include "content/browser/tab_contents/test_tab_contents.h"
 #include "chrome/browser/ui/tab_contents/test_tab_contents_wrapper.h"
+#include "content/browser/browser_thread.h"
+#include "content/browser/tab_contents/test_tab_contents.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "webkit/glue/form_data.h"
 #include "webkit/glue/form_field.h"
 
+using ::testing::_;
+using ::testing::AnyNumber;
+using ::testing::Mock;
+using base::TimeTicks;
+using base::TimeDelta;
 using webkit_glue::FormData;
 using webkit_glue::FormField;
 
@@ -45,10 +52,23 @@ class MockAutofillMetrics : public AutofillMetrics {
   MOCK_CONST_METHOD2(LogQualityMetric, void(QualityMetric metric,
                                             const std::string& experiment_id));
   MOCK_CONST_METHOD1(LogServerQueryMetric, void(ServerQueryMetric metric));
+  MOCK_CONST_METHOD1(LogUserHappinessMetric, void(UserHappinessMetric metric));
+  MOCK_CONST_METHOD1(LogFormFillDurationFromLoadWithAutofill,
+                     void(const TimeDelta& duration));
+  MOCK_CONST_METHOD1(LogFormFillDurationFromLoadWithoutAutofill,
+                     void(const TimeDelta& duration));
+  MOCK_CONST_METHOD1(LogFormFillDurationFromInteractionWithAutofill,
+                     void(const TimeDelta& duration));
+  MOCK_CONST_METHOD1(LogFormFillDurationFromInteractionWithoutAutofill,
+                     void(const TimeDelta& duration));
   MOCK_CONST_METHOD1(LogIsAutofillEnabledAtPageLoad, void(bool enabled));
   MOCK_CONST_METHOD1(LogIsAutofillEnabledAtStartup, void(bool enabled));
   MOCK_CONST_METHOD1(LogStoredProfileCount, void(size_t num_profiles));
   MOCK_CONST_METHOD1(LogAddressSuggestionsCount, void(size_t num_suggestions));
+  MOCK_CONST_METHOD1(LogServerExperimentIdForQuery,
+                     void(const std::string& experiment_id));
+  MOCK_CONST_METHOD1(LogServerExperimentIdForUpload,
+                     void(const std::string& experiment_id));
 
  private:
   DISALLOW_COPY_AND_ASSIGN(MockAutofillMetrics);
@@ -115,37 +135,6 @@ class TestPersonalDataManager : public PersonalDataManager {
   DISALLOW_COPY_AND_ASSIGN(TestPersonalDataManager);
 };
 
-class TestAutofillManager : public AutofillManager {
- public:
-  TestAutofillManager(TabContentsWrapper* tab_contents,
-                      TestPersonalDataManager* personal_manager)
-      : AutofillManager(tab_contents, personal_manager),
-        autofill_enabled_(true) {
-    set_metric_logger(new MockAutofillMetrics);
-  }
-  virtual ~TestAutofillManager() {}
-
-  virtual bool IsAutofillEnabled() const { return autofill_enabled_; }
-
-  void set_autofill_enabled(bool autofill_enabled) {
-    autofill_enabled_ = autofill_enabled;
-  }
-
-  const MockAutofillMetrics* metric_logger() const {
-    return static_cast<const MockAutofillMetrics*>(
-        AutofillManager::metric_logger());
-  }
-
-  void AddSeenForm(FormStructure* form) {
-    form_structures()->push_back(form);
-  }
-
- private:
-  bool autofill_enabled_;
-
-  DISALLOW_COPY_AND_ASSIGN(TestAutofillManager);
-};
-
 class TestFormStructure : public FormStructure {
  public:
   explicit TestFormStructure(const FormData& form) : FormStructure(form) {}
@@ -178,6 +167,49 @@ class TestFormStructure : public FormStructure {
   DISALLOW_COPY_AND_ASSIGN(TestFormStructure);
 };
 
+class TestAutofillManager : public AutofillManager {
+ public:
+  TestAutofillManager(TabContentsWrapper* tab_contents,
+                      TestPersonalDataManager* personal_manager)
+      : AutofillManager(tab_contents, personal_manager),
+        autofill_enabled_(true) {
+    set_metric_logger(new MockAutofillMetrics);
+  }
+  virtual ~TestAutofillManager() {}
+
+  virtual bool IsAutofillEnabled() const { return autofill_enabled_; }
+
+  void set_autofill_enabled(bool autofill_enabled) {
+    autofill_enabled_ = autofill_enabled;
+  }
+
+  MockAutofillMetrics* metric_logger() {
+    return static_cast<MockAutofillMetrics*>(const_cast<AutofillMetrics*>(
+        AutofillManager::metric_logger()));
+  }
+
+  void AddSeenForm(const FormData& form,
+                   const std::vector<AutofillFieldType>& heuristic_types,
+                   const std::vector<AutofillFieldType>& server_types,
+                   const std::string& experiment_id) {
+    FormData empty_form = form;
+    for (size_t i = 0; i < empty_form.fields.size(); ++i) {
+      empty_form.fields[i].value = string16();
+    }
+
+    // |form_structure| will be owned by |form_structures()|.
+    TestFormStructure* form_structure = new TestFormStructure(empty_form);
+    form_structure->SetFieldTypes(heuristic_types, server_types);
+    form_structure->set_server_experiment_id(experiment_id);
+    form_structures()->push_back(form_structure);
+  }
+
+ private:
+  bool autofill_enabled_;
+
+  DISALLOW_COPY_AND_ASSIGN(TestAutofillManager);
+};
+
 }  // namespace
 
 class AutofillMetricsTest : public TabContentsWrapperTestHarness {
@@ -195,10 +227,14 @@ class AutofillMetricsTest : public TabContentsWrapperTestHarness {
   scoped_refptr<TestPersonalDataManager> test_personal_data_;
 
  private:
+  BrowserThread browser_thread_;
+
   DISALLOW_COPY_AND_ASSIGN(AutofillMetricsTest);
 };
 
-AutofillMetricsTest::AutofillMetricsTest() {
+AutofillMetricsTest::AutofillMetricsTest()
+  : TabContentsWrapperTestHarness(),
+    browser_thread_(BrowserThread::UI, &message_loop_) {
 }
 
 AutofillMetricsTest::~AutofillMetricsTest() {
@@ -242,7 +278,7 @@ TEST_F(AutofillMetricsTest, QualityMetrics) {
   FormField field;
 
   autofill_test::CreateTestFormField(
-      "Autofilled", "autofilled", "Elvis Presley", "text", &field);
+      "Autofilled", "autofilled", "Elvis Aaron Presley", "text", &field);
   field.is_autofilled = true;
   form.fields.push_back(field);
   heuristic_types.push_back(NAME_FULL);
@@ -250,24 +286,28 @@ TEST_F(AutofillMetricsTest, QualityMetrics) {
 
   autofill_test::CreateTestFormField(
       "Autofill Failed", "autofillfailed", "buddy@gmail.com", "text", &field);
+  field.is_autofilled = false;
   form.fields.push_back(field);
   heuristic_types.push_back(PHONE_HOME_NUMBER);
   server_types.push_back(EMAIL_ADDRESS);
 
   autofill_test::CreateTestFormField(
       "Empty", "empty", "", "text", &field);
+  field.is_autofilled = false;
   form.fields.push_back(field);
   heuristic_types.push_back(NAME_FULL);
   server_types.push_back(NAME_FIRST);
 
   autofill_test::CreateTestFormField(
       "Unknown", "unknown", "garbage", "text", &field);
+  field.is_autofilled = false;
   form.fields.push_back(field);
   heuristic_types.push_back(PHONE_HOME_NUMBER);
   server_types.push_back(EMAIL_ADDRESS);
 
   autofill_test::CreateTestFormField(
       "Select", "select", "USA", "select-one", &field);
+  field.is_autofilled = false;
   form.fields.push_back(field);
   heuristic_types.push_back(UNKNOWN_TYPE);
   server_types.push_back(NO_SERVER_DATA);
@@ -280,26 +320,26 @@ TEST_F(AutofillMetricsTest, QualityMetrics) {
   server_types.push_back(PHONE_HOME_WHOLE_NUMBER);
 
   // Simulate having seen this form on page load.
-  // |form_structure| will be owned by |autofill_manager_|.
-  TestFormStructure* form_structure = new TestFormStructure(form);
-  form_structure->SetFieldTypes(heuristic_types, server_types);
-  autofill_manager_->AddSeenForm(form_structure);
+  autofill_manager_->AddSeenForm(form, heuristic_types, server_types,
+                                 std::string());
 
   // Establish our expectations.
   ::testing::InSequence dummy;
+  EXPECT_CALL(*autofill_manager_->metric_logger(),
+              LogServerExperimentIdForUpload(std::string()));
   // Autofilled field
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogQualityMetric(AutofillMetrics::FIELD_SUBMITTED,
                                std::string()));
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogHeuristicTypePrediction(AutofillMetrics::TYPE_MATCH,
-                  UNKNOWN_TYPE, std::string()));
+                  NAME_FULL, std::string()));
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogServerTypePrediction(AutofillMetrics::TYPE_MISMATCH,
-                  UNKNOWN_TYPE, std::string()));
+                  NAME_FULL, std::string()));
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogOverallTypePrediction(AutofillMetrics::TYPE_MISMATCH,
-                  UNKNOWN_TYPE, std::string()));
+                  NAME_FULL, std::string()));
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogQualityMetric(AutofillMetrics::FIELD_AUTOFILLED,
                                std::string()));
@@ -364,9 +404,13 @@ TEST_F(AutofillMetricsTest, QualityMetrics) {
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogQualityMetric(AutofillMetrics::FIELD_AUTOFILLED,
                                std::string()));
+  EXPECT_CALL(*autofill_manager_->metric_logger(),
+              LogUserHappinessMetric(
+                  AutofillMetrics::SUBMITTED_FILLABLE_FORM_AUTOFILLED_SOME));
 
   // Simulate form submission.
-  EXPECT_NO_FATAL_FAILURE(autofill_manager_->OnFormSubmitted(form));
+  EXPECT_NO_FATAL_FAILURE(autofill_manager_->OnFormSubmitted(form,
+                                                             TimeTicks::Now()));
 }
 
 // Test that we log the appropriate additional metrics when Autofill failed.
@@ -458,13 +502,15 @@ TEST_F(AutofillMetricsTest, QualityMetricsForFailure) {
 
   // Simulate having seen this form with the desired heuristic and server types.
   // |form_structure| will be owned by |autofill_manager_|.
-  TestFormStructure* form_structure = new TestFormStructure(form);
-  form_structure->SetFieldTypes(heuristic_types, server_types);
-  autofill_manager_->AddSeenForm(form_structure);
+  autofill_manager_->AddSeenForm(form, heuristic_types, server_types,
+                                 std::string());
+
 
   // Establish our expectations.
   ::testing::FLAGS_gmock_verbose = "error";
   ::testing::InSequence dummy;
+  EXPECT_CALL(*autofill_manager_->metric_logger(),
+              LogServerExperimentIdForUpload(std::string()));
   for (size_t i = 0; i < ARRAYSIZE_UNSAFE(failure_cases); ++i) {
     EXPECT_CALL(*autofill_manager_->metric_logger(),
                 LogQualityMetric(AutofillMetrics::FIELD_SUBMITTED,
@@ -481,7 +527,8 @@ TEST_F(AutofillMetricsTest, QualityMetricsForFailure) {
   }
 
   // Simulate form submission.
-  EXPECT_NO_FATAL_FAILURE(autofill_manager_->OnFormSubmitted(form));
+  EXPECT_NO_FATAL_FAILURE(autofill_manager_->OnFormSubmitted(form,
+                                                             TimeTicks::Now()));
 }
 
 // Test that we behave sanely when the cached form differs from the submitted
@@ -499,32 +546,35 @@ TEST_F(AutofillMetricsTest, SaneMetricsWithCacheMismatch) {
 
   FormField field;
   autofill_test::CreateTestFormField(
-      "Both match", "match", "Elvis Presley", "text", &field);
+      "Both match", "match", "Elvis Aaron Presley", "text", &field);
   field.is_autofilled = true;
   form.fields.push_back(field);
   heuristic_types.push_back(NAME_FULL);
   server_types.push_back(NAME_FULL);
   autofill_test::CreateTestFormField(
       "Both mismatch", "mismatch", "buddy@gmail.com", "text", &field);
+  field.is_autofilled = false;
   form.fields.push_back(field);
   heuristic_types.push_back(PHONE_HOME_NUMBER);
   server_types.push_back(PHONE_HOME_NUMBER);
   autofill_test::CreateTestFormField(
       "Only heuristics match", "mixed", "Memphis", "text", &field);
+  field.is_autofilled = false;
   form.fields.push_back(field);
   heuristic_types.push_back(ADDRESS_HOME_CITY);
   server_types.push_back(PHONE_HOME_NUMBER);
   autofill_test::CreateTestFormField(
       "Unknown", "unknown", "garbage", "text", &field);
+  field.is_autofilled = false;
   form.fields.push_back(field);
   heuristic_types.push_back(UNKNOWN_TYPE);
   server_types.push_back(UNKNOWN_TYPE);
 
   // Simulate having seen this form with the desired heuristic and server types.
   // |form_structure| will be owned by |autofill_manager_|.
-  TestFormStructure* form_structure = new TestFormStructure(form);
-  form_structure->SetFieldTypes(heuristic_types, server_types);
-  autofill_manager_->AddSeenForm(form_structure);
+  autofill_manager_->AddSeenForm(form, heuristic_types, server_types,
+                                 std::string());
+
 
   // Add a field and re-arrange the remaining form fields before submitting.
   std::vector<FormField> cached_fields = form.fields;
@@ -540,6 +590,8 @@ TEST_F(AutofillMetricsTest, SaneMetricsWithCacheMismatch) {
   // Establish our expectations.
   ::testing::InSequence dummy;
   // New field
+  EXPECT_CALL(*autofill_manager_->metric_logger(),
+              LogServerExperimentIdForUpload(std::string()));
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogQualityMetric(AutofillMetrics::FIELD_SUBMITTED,
                                std::string()));
@@ -621,19 +673,20 @@ TEST_F(AutofillMetricsTest, SaneMetricsWithCacheMismatch) {
                                std::string()));
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogHeuristicTypePrediction(AutofillMetrics::TYPE_MATCH,
-                  UNKNOWN_TYPE, std::string()));
+                  NAME_FULL, std::string()));
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogServerTypePrediction(AutofillMetrics::TYPE_MATCH,
-                  UNKNOWN_TYPE, std::string()));
+                  NAME_FULL, std::string()));
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogOverallTypePrediction(AutofillMetrics::TYPE_MATCH,
-                  UNKNOWN_TYPE, std::string()));
+                  NAME_FULL, std::string()));
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogQualityMetric(AutofillMetrics::FIELD_AUTOFILLED,
                                std::string()));
 
   // Simulate form submission.
-  EXPECT_NO_FATAL_FAILURE(autofill_manager_->OnFormSubmitted(form));
+  EXPECT_NO_FATAL_FAILURE(autofill_manager_->OnFormSubmitted(form,
+                                                             TimeTicks::Now()));
 }
 
 // Test that we don't log quality metrics for non-autofillable forms.
@@ -659,7 +712,8 @@ TEST_F(AutofillMetricsTest, NoQualityMetricsForNonAutofillableForms) {
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogQualityMetric(AutofillMetrics::FIELD_SUBMITTED,
                                std::string())).Times(0);
-  EXPECT_NO_FATAL_FAILURE(autofill_manager_->OnFormSubmitted(form));
+  EXPECT_NO_FATAL_FAILURE(autofill_manager_->OnFormSubmitted(form,
+                                                             TimeTicks::Now()));
 
   // Search forms are not auto-fillable.
   form.action = GURL("http://example.com/search?q=Elvis%20Presley");
@@ -671,7 +725,8 @@ TEST_F(AutofillMetricsTest, NoQualityMetricsForNonAutofillableForms) {
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogQualityMetric(AutofillMetrics::FIELD_SUBMITTED,
                                std::string())).Times(0);
-  EXPECT_NO_FATAL_FAILURE(autofill_manager_->OnFormSubmitted(form));
+  EXPECT_NO_FATAL_FAILURE(autofill_manager_->OnFormSubmitted(form,
+                                                             TimeTicks::Now()));
 }
 
 // Test that we recored the experiment id appropriately.
@@ -688,7 +743,7 @@ TEST_F(AutofillMetricsTest, QualityMetricsWithExperimentId) {
   FormField field;
 
   autofill_test::CreateTestFormField(
-      "Autofilled", "autofilled", "Elvis Presley", "text", &field);
+      "Autofilled", "autofilled", "Elvis Aaron Presley", "text", &field);
   field.is_autofilled = true;
   form.fields.push_back(field);
   heuristic_types.push_back(NAME_FULL);
@@ -696,24 +751,28 @@ TEST_F(AutofillMetricsTest, QualityMetricsWithExperimentId) {
 
   autofill_test::CreateTestFormField(
       "Autofill Failed", "autofillfailed", "buddy@gmail.com", "text", &field);
+  field.is_autofilled = false;
   form.fields.push_back(field);
   heuristic_types.push_back(PHONE_HOME_NUMBER);
   server_types.push_back(EMAIL_ADDRESS);
 
   autofill_test::CreateTestFormField(
       "Empty", "empty", "", "text", &field);
+  field.is_autofilled = false;
   form.fields.push_back(field);
   heuristic_types.push_back(NAME_FULL);
   server_types.push_back(NAME_FIRST);
 
   autofill_test::CreateTestFormField(
       "Unknown", "unknown", "garbage", "text", &field);
+  field.is_autofilled = false;
   form.fields.push_back(field);
   heuristic_types.push_back(PHONE_HOME_NUMBER);
   server_types.push_back(EMAIL_ADDRESS);
 
   autofill_test::CreateTestFormField(
       "Select", "select", "USA", "select-one", &field);
+  field.is_autofilled = false;
   form.fields.push_back(field);
   heuristic_types.push_back(UNKNOWN_TYPE);
   server_types.push_back(NO_SERVER_DATA);
@@ -722,26 +781,26 @@ TEST_F(AutofillMetricsTest, QualityMetricsWithExperimentId) {
 
   // Simulate having seen this form on page load.
   // |form_structure| will be owned by |autofill_manager_|.
-  TestFormStructure* form_structure = new TestFormStructure(form);
-  form_structure->SetFieldTypes(heuristic_types, server_types);
-  form_structure->set_server_experiment_id(experiment_id);
-  autofill_manager_->AddSeenForm(form_structure);
+  autofill_manager_->AddSeenForm(form, heuristic_types, server_types,
+                                 experiment_id);
 
   // Establish our expectations.
   ::testing::InSequence dummy;
+  EXPECT_CALL(*autofill_manager_->metric_logger(),
+              LogServerExperimentIdForUpload(experiment_id));
   // Autofilled field
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogQualityMetric(AutofillMetrics::FIELD_SUBMITTED,
                                experiment_id));
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogHeuristicTypePrediction(AutofillMetrics::TYPE_MATCH,
-                                         UNKNOWN_TYPE, experiment_id));
+                                         NAME_FULL, experiment_id));
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogServerTypePrediction(AutofillMetrics::TYPE_MISMATCH,
-                                      UNKNOWN_TYPE, experiment_id));
+                                      NAME_FULL, experiment_id));
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogOverallTypePrediction(AutofillMetrics::TYPE_MISMATCH,
-                                       UNKNOWN_TYPE, experiment_id));
+                                       NAME_FULL, experiment_id));
   EXPECT_CALL(*autofill_manager_->metric_logger(),
               LogQualityMetric(AutofillMetrics::FIELD_AUTOFILLED,
                                experiment_id));
@@ -792,7 +851,8 @@ TEST_F(AutofillMetricsTest, QualityMetricsWithExperimentId) {
                                        ADDRESS_HOME_COUNTRY, experiment_id));
 
   // Simulate form submission.
-  EXPECT_NO_FATAL_FAILURE(autofill_manager_->OnFormSubmitted(form));
+  EXPECT_NO_FATAL_FAILURE(autofill_manager_->OnFormSubmitted(form,
+                                                             TimeTicks::Now()));
 }
 
 // Test that the profile count is logged correctly.
@@ -845,9 +905,8 @@ TEST_F(AutofillMetricsTest, AddressSuggestionsCount) {
 
   // Simulate having seen this form on page load.
   // |form_structure| will be owned by |autofill_manager_|.
-  TestFormStructure* form_structure = new TestFormStructure(form);
-  form_structure->SetFieldTypes(field_types, field_types);
-  autofill_manager_->AddSeenForm(form_structure);
+  autofill_manager_->AddSeenForm(form, field_types, field_types,
+                                 std::string());
 
   // Establish our expectations.
   ::testing::FLAGS_gmock_verbose = "error";
@@ -865,9 +924,8 @@ TEST_F(AutofillMetricsTest, AddressSuggestionsCount) {
 
   // Reset the autofill manager state.
   autofill_manager_->Reset();
-  form_structure = new TestFormStructure(form);
-  form_structure->SetFieldTypes(field_types, field_types);
-  autofill_manager_->AddSeenForm(form_structure);
+  autofill_manager_->AddSeenForm(form, field_types, field_types,
+                                 std::string());
 
   // Establish our expectations.
   EXPECT_CALL(*autofill_manager_->metric_logger(),
@@ -878,9 +936,8 @@ TEST_F(AutofillMetricsTest, AddressSuggestionsCount) {
 
   // Reset the autofill manager state again.
   autofill_manager_->Reset();
-  form_structure = new TestFormStructure(form);
-  form_structure->SetFieldTypes(field_types, field_types);
-  autofill_manager_->AddSeenForm(form_structure);
+  autofill_manager_->AddSeenForm(form, field_types, field_types,
+                                 std::string());
 
   // Establish our expectations.
   EXPECT_CALL(*autofill_manager_->metric_logger(),
@@ -900,7 +957,7 @@ TEST_F(AutofillMetricsTest, AutofillIsEnabledAtPageLoad) {
               LogIsAutofillEnabledAtPageLoad(true)).Times(1);
 
   autofill_manager_->set_autofill_enabled(true);
-  autofill_manager_->OnFormsSeen(std::vector<FormData>());
+  autofill_manager_->OnFormsSeen(std::vector<FormData>(), TimeTicks());
 
   // Reset the autofill manager state.
   autofill_manager_->Reset();
@@ -910,7 +967,7 @@ TEST_F(AutofillMetricsTest, AutofillIsEnabledAtPageLoad) {
               LogIsAutofillEnabledAtPageLoad(false)).Times(1);
 
   autofill_manager_->set_autofill_enabled(false);
-  autofill_manager_->OnFormsSeen(std::vector<FormData>());
+  autofill_manager_->OnFormsSeen(std::vector<FormData>(), TimeTicks());
 }
 
 // Test that credit card infobar metrics are logged correctly.
@@ -959,3 +1016,394 @@ TEST_F(AutofillMetricsTest, CreditCardInfoBar) {
         LogCreditCardInfoBarMetric(AutofillMetrics::INFOBAR_IGNORED)).Times(1);
   }
 }
+
+// Test that server query response experiment id metrics are logged correctly.
+TEST_F(AutofillMetricsTest, ServerQueryExperimentIdForQuery) {
+  MockAutofillMetrics metric_logger;
+  ::testing::InSequence dummy;
+
+  // No experiment specified.
+  EXPECT_CALL(metric_logger,
+              LogServerQueryMetric(AutofillMetrics::QUERY_RESPONSE_RECEIVED));
+  EXPECT_CALL(metric_logger,
+              LogServerQueryMetric(AutofillMetrics::QUERY_RESPONSE_PARSED));
+  EXPECT_CALL(metric_logger,
+              LogServerExperimentIdForQuery(std::string()));
+  EXPECT_CALL(metric_logger,
+              LogServerQueryMetric(
+                  AutofillMetrics::QUERY_RESPONSE_MATCHED_LOCAL_HEURISTICS));
+  FormStructure::ParseQueryResponse(
+      "<autofillqueryresponse></autofillqueryresponse>",
+      std::vector<FormStructure*>(), metric_logger);
+
+  // Experiment "ar1" specified.
+  EXPECT_CALL(metric_logger,
+              LogServerQueryMetric(AutofillMetrics::QUERY_RESPONSE_RECEIVED));
+  EXPECT_CALL(metric_logger,
+              LogServerQueryMetric(AutofillMetrics::QUERY_RESPONSE_PARSED));
+  EXPECT_CALL(metric_logger,
+              LogServerExperimentIdForQuery("ar1"));
+  EXPECT_CALL(metric_logger,
+              LogServerQueryMetric(
+                  AutofillMetrics::QUERY_RESPONSE_MATCHED_LOCAL_HEURISTICS));
+  FormStructure::ParseQueryResponse(
+      "<autofillqueryresponse experimentid=\"ar1\"></autofillqueryresponse>",
+      std::vector<FormStructure*>(), metric_logger);
+}
+
+// Verify that we correctly log user happiness metrics dealing with form loading
+// and form submission.
+TEST_F(AutofillMetricsTest, UserHappinessFormLoadAndSubmission) {
+  // Start with a form with insufficiently many fields.
+  FormData form;
+  form.name = ASCIIToUTF16("TestForm");
+  form.method = ASCIIToUTF16("POST");
+  form.origin = GURL("http://example.com/form.html");
+  form.action = GURL("http://example.com/submit.html");
+  form.user_submitted = true;
+
+  FormField field;
+  autofill_test::CreateTestFormField("Name", "name", "", "text", &field);
+  form.fields.push_back(field);
+  autofill_test::CreateTestFormField("Email", "email", "", "text", &field);
+  form.fields.push_back(field);
+
+  std::vector<FormData> forms(1, form);
+
+  // Expect no notifications when the form is first seen.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(AutofillMetrics::FORMS_LOADED)).Times(0);
+    autofill_manager_->OnFormsSeen(forms, TimeTicks());
+  }
+
+
+  // Expect no notifications when the form is submitted.
+  {
+    EXPECT_CALL(
+        *autofill_manager_->metric_logger(),
+        LogUserHappinessMetric(
+            AutofillMetrics::SUBMITTED_FILLABLE_FORM_AUTOFILLED_ALL)).Times(0);
+    EXPECT_CALL(
+        *autofill_manager_->metric_logger(),
+        LogUserHappinessMetric(
+            AutofillMetrics::SUBMITTED_FILLABLE_FORM_AUTOFILLED_SOME)).Times(0);
+    EXPECT_CALL(
+        *autofill_manager_->metric_logger(),
+        LogUserHappinessMetric(
+            AutofillMetrics::SUBMITTED_FILLABLE_FORM_AUTOFILLED_NONE)).Times(0);
+    EXPECT_CALL(
+        *autofill_manager_->metric_logger(),
+        LogUserHappinessMetric(
+            AutofillMetrics::SUBMITTED_NON_FILLABLE_FORM)).Times(0);
+    autofill_manager_->OnFormSubmitted(form, TimeTicks::Now());
+  }
+
+  // Add more fields to the form.
+  autofill_test::CreateTestFormField("Phone", "phone", "", "text", &field);
+  form.fields.push_back(field);
+  autofill_test::CreateTestFormField("Unknown", "unknown", "", "text", &field);
+  form.fields.push_back(field);
+  forms.front() = form;
+
+  // Expect a notification when the form is first seen.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(AutofillMetrics::FORMS_LOADED));
+    autofill_manager_->OnFormsSeen(forms, TimeTicks());
+  }
+
+  // Expect a notification when the form is submitted.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(
+                    AutofillMetrics::SUBMITTED_NON_FILLABLE_FORM));
+    autofill_manager_->OnFormSubmitted(form, TimeTicks::Now());
+  }
+
+  // Fill in two of the fields.
+  form.fields[0].value = ASCIIToUTF16("Elvis Aaron Presley");
+  form.fields[1].value = ASCIIToUTF16("theking@gmail.com");
+  forms.front() = form;
+
+  // Expect a notification when the form is submitted.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(
+                    AutofillMetrics::SUBMITTED_NON_FILLABLE_FORM));
+    autofill_manager_->OnFormSubmitted(form, TimeTicks::Now());
+  }
+
+  // Fill in the third field.
+  form.fields[2].value = ASCIIToUTF16("12345678901");
+  forms.front() = form;
+
+  // Expect notifications when the form is submitted.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(
+                    AutofillMetrics::SUBMITTED_FILLABLE_FORM_AUTOFILLED_NONE));
+    autofill_manager_->OnFormSubmitted(form, TimeTicks::Now());
+  }
+
+
+  // Mark one of the fields as autofilled.
+  form.fields[1].is_autofilled = true;
+  forms.front() = form;
+
+  // Expect notifications when the form is submitted.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(
+                    AutofillMetrics::SUBMITTED_FILLABLE_FORM_AUTOFILLED_SOME));
+    autofill_manager_->OnFormSubmitted(form, TimeTicks::Now());
+  }
+
+  // Mark all of the fillable fields as autofilled.
+  form.fields[0].is_autofilled = true;
+  form.fields[2].is_autofilled = true;
+  forms.front() = form;
+
+  // Expect notifications when the form is submitted.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(
+                    AutofillMetrics::SUBMITTED_FILLABLE_FORM_AUTOFILLED_ALL));
+    autofill_manager_->OnFormSubmitted(form, TimeTicks::Now());
+  }
+
+  // Clear out the third field's value.
+  form.fields[2].value = string16();
+  forms.front() = form;
+
+  // Expect notifications when the form is submitted.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(
+                    AutofillMetrics::SUBMITTED_NON_FILLABLE_FORM));
+    autofill_manager_->OnFormSubmitted(form, TimeTicks::Now());
+  }
+}
+
+// Verify that we correctly log user happiness metrics dealing with form
+// interaction.
+TEST_F(AutofillMetricsTest, UserHappinessFormInteraction) {
+  // Load a fillable form.
+  FormData form;
+  form.name = ASCIIToUTF16("TestForm");
+  form.method = ASCIIToUTF16("POST");
+  form.origin = GURL("http://example.com/form.html");
+  form.action = GURL("http://example.com/submit.html");
+  form.user_submitted = true;
+
+  FormField field;
+  autofill_test::CreateTestFormField("Name", "name", "", "text", &field);
+  form.fields.push_back(field);
+  autofill_test::CreateTestFormField("Email", "email", "", "text", &field);
+  form.fields.push_back(field);
+  autofill_test::CreateTestFormField("Phone", "phone", "", "text", &field);
+  form.fields.push_back(field);
+
+  std::vector<FormData> forms(1, form);
+
+  // Expect a notification when the form is first seen.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(AutofillMetrics::FORMS_LOADED));
+    autofill_manager_->OnFormsSeen(forms, TimeTicks());
+  }
+
+  // Simulate typing.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(AutofillMetrics::USER_DID_TYPE));
+    autofill_manager_->OnTextFieldDidChange(form, form.fields.front(),
+                                            TimeTicks());
+  }
+
+  // Simulate suggestions shown twice for a single edit (i.e. multiple
+  // keystrokes in a single field).
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(
+                    AutofillMetrics::SUGGESTIONS_SHOWN)).Times(1);
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(
+                    AutofillMetrics::SUGGESTIONS_SHOWN_ONCE)).Times(1);
+    autofill_manager_->OnDidShowAutofillSuggestions(true);
+    autofill_manager_->OnDidShowAutofillSuggestions(false);
+  }
+
+  // Simulate suggestions shown for a different field.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(AutofillMetrics::SUGGESTIONS_SHOWN));
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(
+                    AutofillMetrics::SUGGESTIONS_SHOWN_ONCE)).Times(0);
+    autofill_manager_->OnDidShowAutofillSuggestions(true);
+  }
+
+  // Simulate invoking autofill.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(AutofillMetrics::USER_DID_AUTOFILL));
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(
+                    AutofillMetrics::USER_DID_AUTOFILL_ONCE));
+    autofill_manager_->OnDidFillAutofillFormData(TimeTicks());
+  }
+
+  // Simulate editing an autofilled field.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(
+                    AutofillMetrics::USER_DID_EDIT_AUTOFILLED_FIELD));
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(
+                    AutofillMetrics::USER_DID_EDIT_AUTOFILLED_FIELD_ONCE));
+    AutofillManager::GUIDPair guid("00000000-0000-0000-0000-000000000001", 0);
+    AutofillManager::GUIDPair empty(std::string(), 0);
+    autofill_manager_->OnFillAutofillFormData(
+        0, form, form.fields.front(),
+        autofill_manager_->PackGUIDs(empty, guid));
+    autofill_manager_->OnTextFieldDidChange(form, form.fields.front(),
+                                            TimeTicks());
+    // Simulate a second keystroke; make sure we don't log the metric twice.
+    autofill_manager_->OnTextFieldDidChange(form, form.fields.front(),
+                                            TimeTicks());
+  }
+
+  // Simulate invoking autofill again.
+  EXPECT_CALL(*autofill_manager_->metric_logger(),
+              LogUserHappinessMetric(AutofillMetrics::USER_DID_AUTOFILL));
+  EXPECT_CALL(*autofill_manager_->metric_logger(),
+              LogUserHappinessMetric(
+                  AutofillMetrics::USER_DID_AUTOFILL_ONCE)).Times(0);
+  autofill_manager_->OnDidFillAutofillFormData(TimeTicks());
+
+  // Simulate editing another autofilled field.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogUserHappinessMetric(
+                    AutofillMetrics::USER_DID_EDIT_AUTOFILLED_FIELD));
+    autofill_manager_->OnTextFieldDidChange(form, form.fields[1], TimeTicks());
+  }
+}
+
+// Verify that we correctly log metrics tracking the duration of form fill.
+TEST_F(AutofillMetricsTest, FormFillDuration) {
+  // Load a fillable form.
+  FormData form;
+  form.name = ASCIIToUTF16("TestForm");
+  form.method = ASCIIToUTF16("POST");
+  form.origin = GURL("http://example.com/form.html");
+  form.action = GURL("http://example.com/submit.html");
+  form.user_submitted = true;
+
+  FormField field;
+  autofill_test::CreateTestFormField("Name", "name", "", "text", &field);
+  form.fields.push_back(field);
+  autofill_test::CreateTestFormField("Email", "email", "", "text", &field);
+  form.fields.push_back(field);
+  autofill_test::CreateTestFormField("Phone", "phone", "", "text", &field);
+  form.fields.push_back(field);
+
+  std::vector<FormData> forms(1, form);
+
+  // Fill the field values for form submission.
+  form.fields[0].value = ASCIIToUTF16("Elvis Aaron Presley");
+  form.fields[1].value = ASCIIToUTF16("theking@gmail.com");
+  form.fields[2].value = ASCIIToUTF16("12345678901");
+
+  // Ignore any non-timing metrics.
+  // CAUTION: This is a global variable.  So as to not affect other tests, this
+  // _must_ be restored to "warning" at the end of the test.
+  ::testing::FLAGS_gmock_verbose = "error";
+
+  // Expect only form load metrics to be logged if the form is submitted without
+  // user interaction.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogFormFillDurationFromLoadWithAutofill(_)).Times(0);
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogFormFillDurationFromLoadWithoutAutofill(
+                    TimeDelta::FromInternalValue(16)));
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogFormFillDurationFromInteractionWithAutofill(_)).Times(0);
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogFormFillDurationFromInteractionWithoutAutofill(_)).Times(0);
+    autofill_manager_->OnFormsSeen(forms, TimeTicks::FromInternalValue(1));
+    autofill_manager_->OnFormSubmitted(form, TimeTicks::FromInternalValue(17));
+    autofill_manager_->Reset();
+    Mock::VerifyAndClearExpectations(autofill_manager_->metric_logger());
+  }
+
+  // Expect metric to be logged if the user manually edited a form field.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogFormFillDurationFromLoadWithAutofill(_)).Times(0);
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogFormFillDurationFromLoadWithoutAutofill(
+                    TimeDelta::FromInternalValue(16)));
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogFormFillDurationFromInteractionWithAutofill(_)).Times(0);
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogFormFillDurationFromInteractionWithoutAutofill(
+                    TimeDelta::FromInternalValue(14)));
+    autofill_manager_->OnFormsSeen(forms, TimeTicks::FromInternalValue(1));
+    autofill_manager_->OnTextFieldDidChange(form, form.fields.front(),
+                                            TimeTicks::FromInternalValue(3));
+    autofill_manager_->OnFormSubmitted(form, TimeTicks::FromInternalValue(17));
+    autofill_manager_->Reset();
+    Mock::VerifyAndClearExpectations(autofill_manager_->metric_logger());
+  }
+
+  // Expect metric to be logged if the user autofilled the form.
+  form.fields[0].is_autofilled = true;
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogFormFillDurationFromLoadWithAutofill(
+                    TimeDelta::FromInternalValue(16)));
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogFormFillDurationFromLoadWithoutAutofill(_)).Times(0);
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogFormFillDurationFromInteractionWithAutofill(
+                    TimeDelta::FromInternalValue(12)));
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogFormFillDurationFromInteractionWithoutAutofill(_)).Times(0);
+    autofill_manager_->OnFormsSeen(forms, TimeTicks::FromInternalValue(1));
+    autofill_manager_->OnDidFillAutofillFormData(
+        TimeTicks::FromInternalValue(5));
+    autofill_manager_->OnFormSubmitted(form, TimeTicks::FromInternalValue(17));
+    autofill_manager_->Reset();
+    Mock::VerifyAndClearExpectations(autofill_manager_->metric_logger());
+  }
+
+  // Expect metric to be logged if the user both manually filled some fields
+  // and autofilled others.  Messages can arrive out of order, so make sure they
+  // take precedence appropriately.
+  {
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogFormFillDurationFromLoadWithAutofill(
+                    TimeDelta::FromInternalValue(16)));
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogFormFillDurationFromLoadWithoutAutofill(_)).Times(0);
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogFormFillDurationFromInteractionWithAutofill(
+                    TimeDelta::FromInternalValue(14)));
+    EXPECT_CALL(*autofill_manager_->metric_logger(),
+                LogFormFillDurationFromInteractionWithoutAutofill(_)).Times(0);
+    autofill_manager_->OnFormsSeen(forms, TimeTicks::FromInternalValue(1));
+    autofill_manager_->OnDidFillAutofillFormData(
+        TimeTicks::FromInternalValue(5));
+    autofill_manager_->OnTextFieldDidChange(form, form.fields.front(),
+                                            TimeTicks::FromInternalValue(3));
+    autofill_manager_->OnFormSubmitted(form, TimeTicks::FromInternalValue(17));
+    autofill_manager_->Reset();
+    Mock::VerifyAndClearExpectations(autofill_manager_->metric_logger());
+  }
+
+  // Restore the global Gmock verbosity level to its default value.
+  ::testing::FLAGS_gmock_verbose = "warning";
+}
