@@ -4,23 +4,15 @@
 
 #include "chrome/browser/autofill/email_field.h"
 
-#include "chrome/browser/autofill/autofill_ecml.h"
 #include "chrome/browser/autofill/autofill_scanner.h"
 #include "grit/autofill_resources.h"
 #include "ui/base/l10n/l10n_util.h"
 
-using autofill::GetEcmlPattern;
-
 // static
-FormField* EmailField::Parse(AutofillScanner* scanner, bool is_ecml) {
-  string16 pattern;
-  if (is_ecml)
-    pattern = GetEcmlPattern(kEcmlShipToEmail, kEcmlBillToEmail, '|');
-  else
-    pattern = l10n_util::GetStringUTF16(IDS_AUTOFILL_EMAIL_RE);
-
+FormField* EmailField::Parse(AutofillScanner* scanner) {
   const AutofillField* field;
-  if (ParseFieldSpecifics(scanner, pattern,
+  if (ParseFieldSpecifics(scanner,
+                          l10n_util::GetStringUTF16(IDS_AUTOFILL_EMAIL_RE),
                           MATCH_DEFAULT | MATCH_EMAIL, &field)) {
     return new EmailField(field);
   }
