@@ -4,20 +4,30 @@
 
 #include "chrome/browser/autofill/form_structure.h"
 
+#include <utility>
+
 #include "base/basictypes.h"
 #include "base/logging.h"
 #include "base/sha1.h"
 #include "base/stringprintf.h"
 #include "base/string_number_conversions.h"
+#include "base/string_util.h"
+#include "base/time.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/autofill/autofill_metrics.h"
+#include "chrome/browser/autofill/autofill_type.h"
 #include "chrome/browser/autofill/autofill_xml_parser.h"
 #include "chrome/browser/autofill/field_types.h"
 #include "chrome/browser/autofill/form_field.h"
 #include "third_party/libjingle/source/talk/xmllite/xmlelement.h"
+#include "webkit/glue/form_data.h"
+#include "webkit/glue/form_data_predictions.h"
 #include "webkit/glue/form_field.h"
+#include "webkit/glue/form_field_predictions.h"
 
 using webkit_glue::FormData;
+using webkit_glue::FormDataPredictions;
+using webkit_glue::FormFieldPredictions;
 
 namespace {
 
@@ -78,13 +88,196 @@ std::string EncodeFieldTypes(const FieldTypeSet& available_field_types) {
   return data_presence;
 }
 
+bool UpdateFromAutocompleteType(const string16& autocomplete_type,
+                                AutofillField* field) {
+  if (autocomplete_type == ASCIIToUTF16("given-name")) {
+    field->set_heuristic_type(NAME_FIRST);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("middle-name")) {
+    field->set_heuristic_type(NAME_MIDDLE);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("middle-initial")) {
+    field->set_heuristic_type(NAME_MIDDLE_INITIAL);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("surname")) {
+    field->set_heuristic_type(NAME_LAST);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("full-name")) {
+    field->set_heuristic_type(NAME_FULL);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("street-address") ||
+      autocomplete_type == ASCIIToUTF16("address-line1")) {
+    field->set_heuristic_type(ADDRESS_HOME_LINE1);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("address-line2")) {
+    field->set_heuristic_type(ADDRESS_HOME_LINE2);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("locality") ||
+      autocomplete_type == ASCIIToUTF16("city")) {
+    field->set_heuristic_type(ADDRESS_HOME_CITY);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("administrative-area") ||
+      autocomplete_type == ASCIIToUTF16("state") ||
+      autocomplete_type == ASCIIToUTF16("province") ||
+      autocomplete_type == ASCIIToUTF16("region")) {
+    field->set_heuristic_type(ADDRESS_HOME_STATE);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("postal-code")) {
+    field->set_heuristic_type(ADDRESS_HOME_ZIP);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("country")) {
+    field->set_heuristic_type(ADDRESS_HOME_COUNTRY);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("organization")) {
+    field->set_heuristic_type(COMPANY_NAME);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("email")) {
+    field->set_heuristic_type(EMAIL_ADDRESS);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("phone-full")) {
+    field->set_heuristic_type(PHONE_HOME_WHOLE_NUMBER);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("phone-country-code")) {
+    field->set_heuristic_type(PHONE_HOME_COUNTRY_CODE);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("phone-national")) {
+    field->set_heuristic_type(PHONE_HOME_CITY_AND_NUMBER);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("phone-area-code")) {
+    field->set_heuristic_type(PHONE_HOME_CITY_CODE);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("phone-local")) {
+    field->set_heuristic_type(PHONE_HOME_NUMBER);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("phone-local-prefix")) {
+    field->set_heuristic_type(PHONE_HOME_NUMBER);
+    field->set_phone_part(AutofillField::PHONE_PREFIX);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("phone-local-suffix")) {
+    field->set_heuristic_type(PHONE_HOME_NUMBER);
+    field->set_phone_part(AutofillField::PHONE_SUFFIX);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("fax-full")) {
+    field->set_heuristic_type(PHONE_FAX_WHOLE_NUMBER);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("fax-country-code")) {
+    field->set_heuristic_type(PHONE_FAX_COUNTRY_CODE);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("fax-national")) {
+    field->set_heuristic_type(PHONE_FAX_CITY_AND_NUMBER);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("fax-area-code")) {
+    field->set_heuristic_type(PHONE_FAX_CITY_CODE);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("fax-local")) {
+    field->set_heuristic_type(PHONE_FAX_NUMBER);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("fax-local-prefix")) {
+    field->set_heuristic_type(PHONE_FAX_NUMBER);
+    field->set_phone_part(AutofillField::PHONE_PREFIX);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("fax-local-suffix")) {
+    field->set_heuristic_type(PHONE_FAX_NUMBER);
+    field->set_phone_part(AutofillField::PHONE_SUFFIX);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("cc-full-name")) {
+    field->set_heuristic_type(CREDIT_CARD_NAME);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("cc-number")) {
+    field->set_heuristic_type(CREDIT_CARD_NUMBER);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("cc-exp-month")) {
+    field->set_heuristic_type(CREDIT_CARD_EXP_MONTH);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("cc-exp-year")) {
+    if (field->max_length == 2)
+      field->set_heuristic_type(CREDIT_CARD_EXP_2_DIGIT_YEAR);
+    else
+      field->set_heuristic_type(CREDIT_CARD_EXP_4_DIGIT_YEAR);
+    return true;
+  }
+
+  if (autocomplete_type == ASCIIToUTF16("cc-exp")) {
+    if (field->max_length == 5)
+      field->set_heuristic_type(CREDIT_CARD_EXP_DATE_2_DIGIT_YEAR);
+    else
+      field->set_heuristic_type(CREDIT_CARD_EXP_DATE_4_DIGIT_YEAR);
+    return true;
+  }
+
+  return false;
+}
+
 }  // namespace
 
 FormStructure::FormStructure(const FormData& form)
     : form_name_(form.name),
       source_url_(form.origin),
       target_url_(form.action),
-      autofill_count_(0) {
+      autofill_count_(0),
+      upload_required_(USE_UPLOAD_RATES),
+      server_experiment_id_("no server response"),
+      has_author_specified_types_(false) {
   // Copy the form fields.
   std::vector<webkit_glue::FormField>::const_iterator field;
   for (field = form.fields.begin();
@@ -113,26 +306,39 @@ FormStructure::FormStructure(const FormData& form)
 FormStructure::~FormStructure() {}
 
 void FormStructure::DetermineHeuristicTypes() {
-  autofill_count_ = 0;
+  // First, try to detect field types based on the fields' |autocompletetype|
+  // attributes.  If there is at least one form field with this attribute, don't
+  // try to apply other heuristics to match fields in this form.
+  bool found_sections;
+  ParseAutocompletetypeAttributes(&has_author_specified_types_,
+                                  &found_sections);
 
-  FieldTypeMap field_type_map;
-  FormField::ParseFormFields(fields_.get(), &field_type_map);
-
-  for (size_t index = 0; index < field_count(); index++) {
-    AutofillField* field = fields_[index];
-    FieldTypeMap::iterator iter = field_type_map.find(field->unique_name());
-
-    AutofillFieldType heuristic_autofill_type;
-    if (iter == field_type_map.end()) {
-      heuristic_autofill_type = UNKNOWN_TYPE;
-    } else {
-      heuristic_autofill_type = iter->second;
-      ++autofill_count_;
+  if (!has_author_specified_types_) {
+    FieldTypeMap field_type_map;
+    FormField::ParseFormFields(fields_.get(), &field_type_map);
+    for (size_t index = 0; index < field_count(); index++) {
+      AutofillField* field = fields_[index];
+      FieldTypeMap::iterator iter = field_type_map.find(field->unique_name());
+      if (iter != field_type_map.end())
+        field->set_heuristic_type(iter->second);
     }
+  }
 
-    field->set_heuristic_type(heuristic_autofill_type);
+  UpdateAutofillCount();
 
-    AutofillType autofill_type(field->type());
+  if (!found_sections)
+    IdentifySections();
+
+  // Ensure that credit card and address fields are in separate sections.
+  // This simplifies the section-aware logic in autofill_manager.cc.
+  for (std::vector<AutofillField*>::iterator field = fields_->begin();
+       field != fields_->end(); ++field) {
+    AutofillType::FieldTypeGroup field_type_group =
+        AutofillType((*field)->type()).group();
+    if (field_type_group == AutofillType::CREDIT_CARD)
+      (*field)->set_section((*field)->section() + ASCIIToUTF16("-cc"));
+    else
+      (*field)->set_section((*field)->section() + ASCIIToUTF16("-default"));
   }
 }
 
@@ -140,8 +346,8 @@ bool FormStructure::EncodeUploadRequest(
     const FieldTypeSet& available_field_types,
     bool form_was_autofilled,
     std::string* encoded_xml) const {
-  if (!ShouldBeParsed(true)) {
-    NOTREACHED();  // Caller should've checked for search pages.
+  if (!ShouldBeCrowdsourced()) {
+    NOTREACHED();
     return false;
   }
 
@@ -183,7 +389,8 @@ bool FormStructure::EncodeUploadRequest(
 }
 
 // static
-bool FormStructure::EncodeQueryRequest(const ScopedVector<FormStructure>& forms,
+bool FormStructure::EncodeQueryRequest(
+    const ScopedVector<FormStructure>& forms,
     std::vector<std::string>* encoded_signatures,
     std::string* encoded_xml) {
   DCHECK(encoded_signatures);
@@ -236,14 +443,14 @@ bool FormStructure::EncodeQueryRequest(const ScopedVector<FormStructure>& forms,
 // static
 void FormStructure::ParseQueryResponse(const std::string& response_xml,
                                        const std::vector<FormStructure*>& forms,
-                                       UploadRequired* upload_required,
                                        const AutofillMetrics& metric_logger) {
   metric_logger.LogServerQueryMetric(AutofillMetrics::QUERY_RESPONSE_RECEIVED);
 
   // Parse the field types from the server response to the query.
   std::vector<AutofillFieldType> field_types;
+  UploadRequired upload_required;
   std::string experiment_id;
-  AutofillQueryXmlParser parse_handler(&field_types, upload_required,
+  AutofillQueryXmlParser parse_handler(&field_types, &upload_required,
                                        &experiment_id);
   buzz::XmlParser parser(&parse_handler);
   parser.Parse(response_xml.c_str(), response_xml.length(), true);
@@ -251,6 +458,7 @@ void FormStructure::ParseQueryResponse(const std::string& response_xml,
     return;
 
   metric_logger.LogServerQueryMetric(AutofillMetrics::QUERY_RESPONSE_PARSED);
+  metric_logger.LogServerExperimentIdForQuery(experiment_id);
 
   bool heuristics_detected_fillable_field = false;
   bool query_response_overrode_heuristics = false;
@@ -260,6 +468,7 @@ void FormStructure::ParseQueryResponse(const std::string& response_xml,
   for (std::vector<FormStructure*>::const_iterator iter = forms.begin();
        iter != forms.end(); ++iter) {
     FormStructure* form = *iter;
+    form->upload_required_ = upload_required;
     form->server_experiment_id_ = experiment_id;
 
     for (std::vector<AutofillField*>::iterator field = form->fields_.begin();
@@ -297,6 +506,43 @@ void FormStructure::ParseQueryResponse(const std::string& response_xml,
   metric_logger.LogServerQueryMetric(metric);
 }
 
+// static
+void FormStructure::GetFieldTypePredictions(
+    const std::vector<FormStructure*>& form_structures,
+    std::vector<FormDataPredictions>* forms) {
+  forms->clear();
+  forms->reserve(form_structures.size());
+  for (size_t i = 0; i < form_structures.size(); ++i) {
+    FormStructure* form_structure = form_structures[i];
+    FormDataPredictions form;
+    form.data.name = form_structure->form_name_;
+    form.data.method =
+        ASCIIToUTF16((form_structure->method_ == POST) ? "POST" : "GET");
+    form.data.origin = form_structure->source_url_;
+    form.data.action = form_structure->target_url_;
+    form.signature = form_structure->FormSignature();
+    form.experiment_id = form_structure->server_experiment_id_;
+
+    for (std::vector<AutofillField*>::const_iterator field =
+             form_structure->fields_.begin();
+         field != form_structure->fields_.end(); ++field) {
+      form.data.fields.push_back(webkit_glue::FormField(**field));
+
+      FormFieldPredictions annotated_field;
+      annotated_field.signature = (*field)->FieldSignature();
+      annotated_field.heuristic_type =
+          AutofillType::FieldTypeToString((*field)->heuristic_type());
+      annotated_field.server_type =
+          AutofillType::FieldTypeToString((*field)->server_type());
+      annotated_field.overall_type =
+          AutofillType::FieldTypeToString((*field)->type());
+      form.fields.push_back(annotated_field);
+    }
+
+    forms->push_back(form);
+  }
+}
+
 std::string FormStructure::FormSignature() const {
   std::string scheme(target_url_.scheme());
   std::string host(target_url_.host());
@@ -342,9 +588,22 @@ bool FormStructure::ShouldBeParsed(bool require_method_post) const {
   if (target_url_.path() == "/search")
     return false;
 
+  // Make sure there as at least one text field.
+  bool has_text_field = false;
+  for (std::vector<AutofillField*>::const_iterator it = begin();
+       it != end() && !has_text_field; ++it) {
+    has_text_field |= (*it)->form_control_type != ASCIIToUTF16("select-one");
+  }
+  if (!has_text_field)
+    return false;
+
   return !require_method_post || (method_ == POST);
 }
 
+bool FormStructure::ShouldBeCrowdsourced() const {
+  return !has_author_specified_types_ && ShouldBeParsed(true);
+}
+
 void FormStructure::UpdateFromCache(const FormStructure& cached_form) {
   // Map from field signatures to cached fields.
   std::map<std::string, const AutofillField*> cached_fields;
@@ -360,6 +619,13 @@ void FormStructure::UpdateFromCache(const FormStructure& cached_form) {
     std::map<std::string, const AutofillField*>::const_iterator
         cached_field = cached_fields.find(field->FieldSignature());
     if (cached_field != cached_fields.end()) {
+      if (field->form_control_type != ASCIIToUTF16("select-one") &&
+          field->value == cached_field->second->value) {
+        // From the perspective of learning user data, text fields containing
+        // default values are equivalent to empty fields.
+        field->value = string16();
+      }
+
       field->set_heuristic_type(cached_field->second->heuristic_type());
       field->set_server_type(cached_field->second->server_type());
     }
@@ -368,11 +634,29 @@ void FormStructure::UpdateFromCache(const FormStructure& cached_form) {
   UpdateAutofillCount();
 
   server_experiment_id_ = cached_form.server_experiment_id();
+
+  // The form signature should match between query and upload requests to the
+  // server. On many websites, form elements are dynamically added, removed, or
+  // rearranged via JavaScript between page load and form submission, so we
+  // copy over the |form_signature_field_names_| corresponding to the query
+  // request.
+  DCHECK_EQ(cached_form.form_name_, form_name_);
+  DCHECK_EQ(cached_form.source_url_, source_url_);
+  DCHECK_EQ(cached_form.target_url_, target_url_);
+  form_signature_field_names_ = cached_form.form_signature_field_names_;
 }
 
 void FormStructure::LogQualityMetrics(
-    const AutofillMetrics& metric_logger) const {
+    const AutofillMetrics& metric_logger,
+    const base::TimeTicks& load_time,
+    const base::TimeTicks& interaction_time,
+    const base::TimeTicks& submission_time) const {
   std::string experiment_id = server_experiment_id();
+  metric_logger.LogServerExperimentIdForUpload(experiment_id);
+
+  size_t num_detected_field_types = 0;
+  bool did_autofill_all_possible_fields = true;
+  bool did_autofill_some_possible_fields = false;
   for (size_t i = 0; i < field_count(); ++i) {
     const AutofillField* field = this->field(i);
     metric_logger.LogQualityMetric(AutofillMetrics::FIELD_SUBMITTED,
@@ -385,6 +669,12 @@ void FormStructure::LogQualityMetrics(
     if (field_types.count(EMPTY_TYPE) || field_types.count(UNKNOWN_TYPE))
       continue;
 
+    ++num_detected_field_types;
+    if (field->is_autofilled)
+      did_autofill_some_possible_fields = true;
+    else
+      did_autofill_all_possible_fields = false;
+
     // Collapse field types that Chrome treats as identical, e.g. home and
     // billing address fields.
     FieldTypeSet collapsed_field_types;
@@ -488,16 +778,51 @@ void FormStructure::LogQualityMetrics(
       }
     }
   }
-}
 
-void FormStructure::set_possible_types(size_t index,
-                                       const FieldTypeSet& types) {
-  if (index >= fields_.size()) {
-    NOTREACHED();
-    return;
+  if (num_detected_field_types < kRequiredFillableFields) {
+    metric_logger.LogUserHappinessMetric(
+        AutofillMetrics::SUBMITTED_NON_FILLABLE_FORM);
+  } else {
+    if (did_autofill_all_possible_fields) {
+      metric_logger.LogUserHappinessMetric(
+          AutofillMetrics::SUBMITTED_FILLABLE_FORM_AUTOFILLED_ALL);
+    } else if (did_autofill_some_possible_fields) {
+      metric_logger.LogUserHappinessMetric(
+          AutofillMetrics::SUBMITTED_FILLABLE_FORM_AUTOFILLED_SOME);
+    } else {
+      metric_logger.LogUserHappinessMetric(
+          AutofillMetrics::SUBMITTED_FILLABLE_FORM_AUTOFILLED_NONE);
+    }
+
+    // Unlike the other times, the |submission_time| should always be available.
+    DCHECK(!submission_time.is_null());
+
+    // The |load_time| might be unset, in the case that the form was dynamically
+    // added to the DOM.
+    if (!load_time.is_null()) {
+      // Submission should always chronologically follow form load.
+      DCHECK(submission_time > load_time);
+      base::TimeDelta elapsed = submission_time - load_time;
+      if (did_autofill_some_possible_fields)
+        metric_logger.LogFormFillDurationFromLoadWithAutofill(elapsed);
+      else
+        metric_logger.LogFormFillDurationFromLoadWithoutAutofill(elapsed);
+    }
+
+    // The |interaction_time| might be unset, in the case that the user
+    // submitted a blank form.
+    if (!interaction_time.is_null()) {
+      // Submission should always chronologically follow interaction.
+      DCHECK(submission_time > interaction_time);
+      base::TimeDelta elapsed = submission_time - interaction_time;
+      if (did_autofill_some_possible_fields) {
+        metric_logger.LogFormFillDurationFromInteractionWithAutofill(elapsed);
+      } else {
+        metric_logger.LogFormFillDurationFromInteractionWithoutAutofill(
+            elapsed);
+      }
+    }
   }
-
-  fields_[index]->set_possible_types(types);
 }
 
 const AutofillField* FormStructure::field(size_t index) const {
@@ -509,6 +834,11 @@ const AutofillField* FormStructure::field(size_t index) const {
   return fields_[index];
 }
 
+AutofillField* FormStructure::field(size_t index) {
+  return const_cast<AutofillField*>(
+      static_cast<const FormStructure*>(this)->field(index));
+}
+
 size_t FormStructure::field_count() const {
   return fields_.size();
 }
@@ -592,3 +922,83 @@ bool FormStructure::EncodeFormRequest(
   }
   return true;
 }
+
+void FormStructure::ParseAutocompletetypeAttributes(bool* found_attribute,
+                                                    bool* found_sections) {
+  *found_attribute = false;
+  *found_sections = false;
+  for (std::vector<AutofillField*>::iterator field = fields_->begin();
+       field != fields_->end(); ++field) {
+    if ((*field)->autocomplete_type.empty())
+      continue;
+
+    *found_attribute = true;
+    std::vector<string16> types;
+    Tokenize((*field)->autocomplete_type, ASCIIToUTF16(" "), &types);
+
+    // Look for a named section.
+    const string16 kSectionPrefix = ASCIIToUTF16("section-");
+    if (!types.empty() && StartsWith(types.front(), kSectionPrefix, true)) {
+      *found_sections = true;
+      (*field)->set_section(types.front().substr(kSectionPrefix.size()));
+    }
+
+    // Look for specified types.
+    for (std::vector<string16>::const_iterator type = types.begin();
+         type != types.end(); ++type) {
+      if (UpdateFromAutocompleteType(*type, *field))
+        break;
+    }
+  }
+}
+
+void FormStructure::IdentifySections() {
+  if (fields_.empty())
+    return;
+
+  // Name sections after the first field in the section.
+  string16 current_section = fields_->front()->unique_name();
+
+  // Keep track of the types we've seen in this section.
+  std::set<AutofillFieldType> seen_types;
+  AutofillFieldType previous_type = UNKNOWN_TYPE;
+
+  for (std::vector<AutofillField*>::iterator field = fields_->begin();
+       field != fields_->end(); ++field) {
+    const AutofillFieldType current_type =
+        AutofillType::GetEquivalentFieldType((*field)->type());
+
+    bool already_saw_current_type = seen_types.count(current_type) > 0;
+
+    // Forms often ask for multiple phone numbers -- e.g. both a daytime and
+    // evening phone number.  Our phone and fax number detection is also
+    // generally a little off.  Hence, ignore both field types as a signal here.
+    AutofillType::FieldTypeGroup current_type_group =
+        AutofillType(current_type).group();
+    if (current_type_group == AutofillType::PHONE_HOME ||
+        current_type_group == AutofillType::PHONE_FAX)
+      already_saw_current_type = false;
+
+    // Some forms have adjacent fields of the same type.  Two common examples:
+    //  * Forms with two email fields, where the second is meant to "confirm"
+    //    the first.
+    //  * Forms with a <select> menu for states in some countries, and a
+    //    freeform <input> field for states in other countries.  (Usually, only
+    //    one of these two will be visible for any given choice of country.)
+    // Generally, adjacent fields of the same type belong in the same logical
+    // section.
+    if (current_type == previous_type)
+      already_saw_current_type = false;
+
+    previous_type = current_type;
+
+    if (current_type != UNKNOWN_TYPE && already_saw_current_type) {
+      // We reached the end of a section, so start a new section.
+      seen_types.clear();
+      current_section = (*field)->unique_name();
+    }
+
+    seen_types.insert(current_type);
+    (*field)->set_section(current_section);
+  }
+}
