@@ -13,8 +13,9 @@
 #include "chrome/browser/autofill/autofill-inl.h"
 #include "chrome/browser/autofill/autofill_field.h"
 #include "chrome/browser/autofill/autofill_metrics.h"
-#include "chrome/browser/autofill/autofill_scanner.h"
+#include "chrome/browser/autofill/autofill_regexes.h"
 #include "chrome/browser/autofill/form_structure.h"
+#include "chrome/browser/autofill/personal_data_manager_observer.h"
 #include "chrome/browser/autofill/phone_number.h"
 #include "chrome/browser/autofill/phone_number_i18n.h"
 #include "chrome/browser/autofill/select_control_handler.h"
@@ -27,10 +28,6 @@
 
 namespace {
 
-// The minimum number of fields that must contain relevant user data before
-// Autofill will attempt to import the data into a credit card.
-const int kMinCreditCardImportSize = 2;
-
 template<typename T>
 class FormGroupMatchesByGUIDFunctor {
  public:
@@ -76,14 +73,14 @@ bool IsValidEmail(const string16& value) {
   // This regex is more permissive than the official rfc2822 spec on the
   // subject, but it does reject obvious non-email addresses.
   const string16 kEmailPattern = ASCIIToUTF16("^[^@]+@[^@]+\\.[a-z]{2,6}$");
-  return autofill::MatchString(value, kEmailPattern);
+  return autofill::MatchesPattern(value, kEmailPattern);
 }
 
 // Valid for US zip codes only.
 bool IsValidZip(const string16& value) {
   // Basic US zip code matching.
   const string16 kZipPattern = ASCIIToUTF16("^\\d{5}(-\\d{4})?$");
-  return autofill::MatchString(value, kZipPattern);
+  return autofill::MatchesPattern(value, kZipPattern);
 }
 
 // Returns true if minimum requirements for import of a given |profile| have
@@ -162,22 +159,20 @@ void PersonalDataManager::OnWebDataServiceRequestDone(
     std::copy(web_profiles_.begin(), web_profiles_.end(),
               profile_pointers.begin());
     AutofillProfile::AdjustInferredLabels(&profile_pointers);
-    FOR_EACH_OBSERVER(Observer, observers_, OnPersonalDataChanged());
+    FOR_EACH_OBSERVER(PersonalDataManagerObserver, observers_,
+                      OnPersonalDataChanged());
   }
 }
 
-/////////////////////////////////////////////////////////////////////////////
-// PersonalDataManager,
-// views::ButtonListener implementations
-void PersonalDataManager::SetObserver(PersonalDataManager::Observer* observer) {
-  // TODO: RemoveObserver is for compatibility with old code, it should be
-  // nuked.
+void PersonalDataManager::SetObserver(PersonalDataManagerObserver* observer) {
+  // TODO(dhollowa): RemoveObserver is for compatibility with old code, it
+  // should be nuked.
   observers_.RemoveObserver(observer);
   observers_.AddObserver(observer);
 }
 
 void PersonalDataManager::RemoveObserver(
-    PersonalDataManager::Observer* observer) {
+    PersonalDataManagerObserver* observer) {
   observers_.RemoveObserver(observer);
 }
 
@@ -248,17 +243,11 @@ bool PersonalDataManager::ImportFormData(
     types_seen.insert(field_type);
 
     if (group == AutofillType::CREDIT_CARD) {
-      // If the user has a password set, we have no way of setting credit
-      // card numbers.
       if (LowerCaseEqualsASCII(field->form_control_type, "month")) {
         DCHECK_EQ(CREDIT_CARD_EXP_MONTH, field_type);
         local_imported_credit_card->SetInfoForMonthInputType(value);
       } else {
-        if (field_type == CREDIT_CARD_NUMBER) {
-          // Clean up any imported credit card numbers.
-          value = CreditCard::StripSeparators(value);
-        }
-        local_imported_credit_card->SetInfo(field_type, value);
+        local_imported_credit_card->SetCanonicalizedInfo(field_type, value);
       }
       ++importable_credit_card_fields;
     } else {
@@ -267,7 +256,7 @@ bool PersonalDataManager::ImportFormData(
       // If the fields are not the phone fields in question both home.SetInfo()
       // and fax.SetInfo() are going to return false.
       if (!home.SetInfo(field_type, value) && !fax.SetInfo(field_type, value))
-        imported_profile->SetInfo(field_type, value);
+        imported_profile->SetCanonicalizedInfo(field_type, value);
 
       // Reject profiles with invalid country information.
       if (field_type == ADDRESS_HOME_COUNTRY &&
@@ -278,34 +267,25 @@ bool PersonalDataManager::ImportFormData(
     }
   }
 
-  // Build phone numbers if they are from parts.
-  if (imported_profile.get()) {
+  // Construct the phone and fax numbers.  Reject the profile if either number
+  // is invalid.
+  if (imported_profile.get() && !home.IsEmpty()) {
     string16 constructed_number;
-    if (!home.empty()) {
-      if (!home.ParseNumber(imported_profile->CountryCode(),
-                            &constructed_number)) {
-        imported_profile.reset();
-      } else {
-        imported_profile->SetInfo(PHONE_HOME_WHOLE_NUMBER, constructed_number);
-      }
-    }
-  }
-  if (imported_profile.get()) {
-    string16 constructed_number;
-    if (!fax.empty()) {
-      if (!fax.ParseNumber(imported_profile->CountryCode(),
-                           &constructed_number)) {
-        imported_profile.reset();
-      } else {
-        imported_profile->SetInfo(PHONE_FAX_WHOLE_NUMBER, constructed_number);
-      }
-    }
-  }
-  // Normalize phone numbers.
-  if (imported_profile.get()) {
-    // Reject profile if even one of the phones is invalid.
-    if (!imported_profile->NormalizePhones())
+    if (!home.ParseNumber(imported_profile->CountryCode(),
+                          &constructed_number) ||
+        !imported_profile->SetCanonicalizedInfo(PHONE_HOME_WHOLE_NUMBER,
+                                                constructed_number)) {
       imported_profile.reset();
+    }
+  }
+  if (imported_profile.get() && !fax.IsEmpty()) {
+    string16 constructed_number;
+    if (!fax.ParseNumber(imported_profile->CountryCode(),
+                         &constructed_number) ||
+        !imported_profile->SetCanonicalizedInfo(PHONE_FAX_WHOLE_NUMBER,
+                                                constructed_number)) {
+      imported_profile.reset();
+    }
   }
 
   // Reject the profile if minimum address and validation requirements are not
@@ -316,9 +296,7 @@ bool PersonalDataManager::ImportFormData(
   // Reject the credit card if we did not detect enough filled credit card
   // fields or if the credit card number does not seem to be valid.
   if (local_imported_credit_card.get() &&
-      (importable_credit_card_fields < kMinCreditCardImportSize ||
-       !CreditCard::IsValidCreditCardNumber(
-           local_imported_credit_card->GetInfo(CREDIT_CARD_NUMBER)))) {
+      !local_imported_credit_card->IsComplete()) {
     local_imported_credit_card.reset();
   }
 
@@ -340,7 +318,13 @@ bool PersonalDataManager::ImportFormData(
   }
   *imported_credit_card = local_imported_credit_card.release();
 
-  return imported_profile.get() || *imported_credit_card;
+  if (imported_profile.get() || *imported_credit_card) {
+    return true;
+  } else {
+    FOR_EACH_OBSERVER(PersonalDataManagerObserver, observers_,
+                      OnInsufficientFormData());
+    return false;
+  }
 }
 
 void PersonalDataManager::AddProfile(const AutofillProfile& profile) {
@@ -495,41 +479,6 @@ CreditCard* PersonalDataManager::GetCreditCardByGUID(const std::string& guid) {
   return NULL;
 }
 
-void PersonalDataManager::GetMatchingTypes(const string16& text,
-                                           FieldTypeSet* matching_types) const {
-  string16 clean_info = StringToLowerASCII(CollapseWhitespace(text, false));
-  if (clean_info.empty()) {
-    matching_types->insert(EMPTY_TYPE);
-    return;
-  }
-
-  const std::vector<AutofillProfile*>& profiles = this->profiles();
-  for (std::vector<AutofillProfile*>::const_iterator iter = profiles.begin();
-       iter != profiles.end(); ++iter) {
-    const FormGroup* profile = *iter;
-    if (!profile) {
-      DLOG(ERROR) << "NULL information in profiles list";
-      continue;
-    }
-
-    profile->GetMatchingTypes(clean_info, matching_types);
-  }
-
-  for (ScopedVector<CreditCard>::const_iterator iter = credit_cards_.begin();
-       iter != credit_cards_.end(); ++iter) {
-    const FormGroup* credit_card = *iter;
-    if (!credit_card) {
-      DLOG(ERROR) << "NULL information in credit cards list";
-      continue;
-    }
-
-    credit_card->GetMatchingTypes(clean_info, matching_types);
-  }
-
-  if (matching_types->empty())
-    matching_types->insert(UNKNOWN_TYPE);
-}
-
 void PersonalDataManager::GetNonEmptyTypes(
     FieldTypeSet* non_empty_types) const {
   const std::vector<AutofillProfile*>& profiles = this->profiles();
