@@ -13,16 +13,17 @@
 #include "chrome/browser/autofill/autofill_profile.h"
 #include "chrome/browser/autofill/form_structure.h"
 #include "chrome/browser/autofill/personal_data_manager.h"
+#include "chrome/browser/autofill/personal_data_manager_observer.h"
 #include "chrome/browser/password_manager/encryptor.h"
 #include "chrome/common/guid.h"
-#include "chrome/test/testing_browser_process.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/testing_browser_process.h"
+#include "chrome/test/base/testing_profile.h"
 #include "content/browser/browser_thread.h"
+#include "content/common/content_notification_types.h"
 #include "content/common/notification_details.h"
 #include "content/common/notification_observer_mock.h"
 #include "content/common/notification_registrar.h"
 #include "content/common/notification_source.h"
-#include "content/common/notification_type.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "webkit/glue/form_data.h"
@@ -34,7 +35,7 @@ ACTION(QuitUIMessageLoop) {
   MessageLoop::current()->Quit();
 }
 
-class PersonalDataLoadedObserverMock : public PersonalDataManager::Observer {
+class PersonalDataLoadedObserverMock : public PersonalDataManagerObserver {
  public:
   PersonalDataLoadedObserverMock() {}
   virtual ~PersonalDataLoadedObserverMock() {}
@@ -80,7 +81,6 @@ class PersonalDataManagerTest : public testing::Test {
     MessageLoop::current()->Run();
   }
 
-  ScopedTestingBrowserProcess browser_process_;
   MessageLoopForUI message_loop_;
   BrowserThread ui_thread_;
   BrowserThread db_thread_;
@@ -1543,64 +1543,73 @@ TEST_F(PersonalDataManagerTest, AggregateCreditCardWithMissingInfoInNew) {
   const std::vector<CreditCard*>& results2 = personal_data_->credit_cards();
   ASSERT_EQ(1U, results2.size());
   EXPECT_EQ(0, expected2.Compare(*results2[0]));
+
+  // Add a third credit card where the expiration date is missing.
+  FormData form3;
+  autofill_test::CreateTestFormField(
+      "Name on card:", "name_on_card", "Johnny McEnroe", "text", &field);
+  form3.fields.push_back(field);
+  autofill_test::CreateTestFormField(
+      "Card Number:", "card_number", "5555555555554444", "text", &field);
+  form3.fields.push_back(field);
+  // Note missing expiration month and year..
+
+  FormStructure form_structure3(form3);
+  form_structure3.DetermineHeuristicTypes();
+  EXPECT_FALSE(personal_data_->ImportFormData(form_structure3,
+                                              &imported_credit_card));
+  ASSERT_FALSE(imported_credit_card);
+
+  // Note: no refresh here.
+
+  // No change is expected.
+  CreditCard expected3;
+  autofill_test::SetCreditCardInfo(&expected3,
+      "Biggie Smalls", "4111111111111111", "01", "2011");
+  const std::vector<CreditCard*>& results3 = personal_data_->credit_cards();
+  ASSERT_EQ(1U, results3.size());
+  EXPECT_EQ(0, expected3.Compare(*results2[0]));
 }
 
 TEST_F(PersonalDataManagerTest, AggregateCreditCardWithMissingInfoInOld) {
-  FormData form1;
-
-  // Start with a single valid credit card form.
-  webkit_glue::FormField field;
-  // Note missing name.
-  autofill_test::CreateTestFormField(
-      "Card Number:", "card_number", "4111-1111-1111-1111", "text", &field);
-  form1.fields.push_back(field);
-  autofill_test::CreateTestFormField(
-      "Exp Month:", "exp_month", "01", "text", &field);
-  form1.fields.push_back(field);
-  autofill_test::CreateTestFormField(
-      "Exp Year:", "exp_year", "2011", "text", &field);
-  form1.fields.push_back(field);
-
-  FormStructure form_structure1(form1);
-  form_structure1.DetermineHeuristicTypes();
-  const CreditCard* imported_credit_card;
-  EXPECT_TRUE(personal_data_->ImportFormData(form_structure1,
-                                             &imported_credit_card));
-  ASSERT_TRUE(imported_credit_card);
-  personal_data_->SaveImportedCreditCard(*imported_credit_card);
-  delete imported_credit_card;
+  // Start with a single valid credit card stored via the preferences.
+  // Note the empty name.
+  CreditCard saved_credit_card;
+  autofill_test::SetCreditCardInfo(&saved_credit_card,
+      "", "4111111111111111" /* Visa */, "01", "2011");
+  personal_data_->AddCreditCard(saved_credit_card);
 
   // Verify that the web database has been updated and the notification sent.
   EXPECT_CALL(personal_data_observer_,
               OnPersonalDataChanged()).WillOnce(QuitUIMessageLoop());
   MessageLoop::current()->Run();
 
-  CreditCard expected;
-  autofill_test::SetCreditCardInfo(&expected,
-      NULL, "4111111111111111", "01", "2011");
-  const std::vector<CreditCard*>& results = personal_data_->credit_cards();
-  ASSERT_EQ(1U, results.size());
-  EXPECT_EQ(0, expected.Compare(*results[0]));
+  const std::vector<CreditCard*>& results1 = personal_data_->credit_cards();
+  ASSERT_EQ(1U, results1.size());
+  EXPECT_EQ(saved_credit_card, *results1[0]);
+
 
   // Add a second different valid credit card where the year is different but
   // the credit card number matches.
-  FormData form2;
+  FormData form;
+  webkit_glue::FormField field;
   autofill_test::CreateTestFormField(
       "Name on card:", "name_on_card", "Biggie Smalls", "text", &field);
-  form2.fields.push_back(field);
+  form.fields.push_back(field);
   autofill_test::CreateTestFormField(
       "Card Number:", "card_number", "4111-1111-1111-1111", "text", &field);
-  form2.fields.push_back(field);
+  form.fields.push_back(field);
   autofill_test::CreateTestFormField(
       "Exp Month:", "exp_month", "01", "text", &field);
-  form2.fields.push_back(field);
+  form.fields.push_back(field);
   autofill_test::CreateTestFormField(
-      "Exp Year:", "exp_year", "2011", "text", &field);
-  form2.fields.push_back(field);
+      "Exp Year:", "exp_year", "2012", "text", &field);
+  form.fields.push_back(field);
 
-  FormStructure form_structure2(form2);
-  form_structure2.DetermineHeuristicTypes();
-  EXPECT_TRUE(personal_data_->ImportFormData(form_structure2,
+  FormStructure form_structure(form);
+  form_structure.DetermineHeuristicTypes();
+  const CreditCard* imported_credit_card;
+  EXPECT_TRUE(personal_data_->ImportFormData(form_structure,
                                              &imported_credit_card));
   ASSERT_TRUE(imported_credit_card);
   personal_data_->SaveImportedCreditCard(*imported_credit_card);
@@ -1615,7 +1624,7 @@ TEST_F(PersonalDataManagerTest, AggregateCreditCardWithMissingInfoInOld) {
   // added to the existing credit card.
   CreditCard expected2;
   autofill_test::SetCreditCardInfo(&expected2,
-      "Biggie Smalls", "4111111111111111", "01", "2011");
+      "Biggie Smalls", "4111111111111111", "01", "2012");
   const std::vector<CreditCard*>& results2 = personal_data_->credit_cards();
   ASSERT_EQ(1U, results2.size());
   EXPECT_EQ(0, expected2.Compare(*results2[0]));
@@ -1750,3 +1759,101 @@ TEST_F(PersonalDataManagerTest, GetNonEmptyTypes) {
   EXPECT_TRUE(non_empty_types.count(CREDIT_CARD_EXP_DATE_2_DIGIT_YEAR));
   EXPECT_TRUE(non_empty_types.count(CREDIT_CARD_EXP_DATE_4_DIGIT_YEAR));
 }
+
+TEST_F(PersonalDataManagerTest, CaseInsensitiveMultiValueAggregation) {
+  FormData form1;
+  webkit_glue::FormField field;
+  autofill_test::CreateTestFormField(
+      "First name:", "first_name", "George", "text", &field);
+  form1.fields.push_back(field);
+  autofill_test::CreateTestFormField(
+      "Last name:", "last_name", "Washington", "text", &field);
+  form1.fields.push_back(field);
+  autofill_test::CreateTestFormField(
+      "Email:", "email", "theprez@gmail.com", "text", &field);
+  form1.fields.push_back(field);
+  autofill_test::CreateTestFormField(
+      "Address:", "address1", "21 Laussat St", "text", &field);
+  form1.fields.push_back(field);
+  autofill_test::CreateTestFormField(
+      "City:", "city", "San Francisco", "text", &field);
+  form1.fields.push_back(field);
+  autofill_test::CreateTestFormField(
+      "State:", "state", "California", "text", &field);
+  form1.fields.push_back(field);
+  autofill_test::CreateTestFormField(
+      "Zip:", "zip", "94102", "text", &field);
+  form1.fields.push_back(field);
+  autofill_test::CreateTestFormField(
+      "Phone number:", "phone_number", "817-555-6789", "text", &field);
+  form1.fields.push_back(field);
+
+  FormStructure form_structure1(form1);
+  form_structure1.DetermineHeuristicTypes();
+  const CreditCard* imported_credit_card;
+  EXPECT_TRUE(personal_data_->ImportFormData(form_structure1,
+                                             &imported_credit_card));
+  ASSERT_FALSE(imported_credit_card);
+
+  // Verify that the web database has been updated and the notification sent.
+  EXPECT_CALL(personal_data_observer_,
+              OnPersonalDataChanged()).WillOnce(QuitUIMessageLoop());
+  MessageLoop::current()->Run();
+
+  AutofillProfile expected;
+  autofill_test::SetProfileInfo(&expected, "George", NULL,
+      "Washington", "theprez@gmail.com", NULL, "21 Laussat St", NULL,
+      "San Francisco", "California", "94102", NULL, "817-555-6789", NULL);
+  const std::vector<AutofillProfile*>& results1 = personal_data_->profiles();
+  ASSERT_EQ(1U, results1.size());
+  EXPECT_EQ(0, expected.Compare(*results1[0]));
+
+  // Upper-case the first name and change the phone number.
+  FormData form2;
+  autofill_test::CreateTestFormField(
+      "First name:", "first_name", "GEORGE", "text", &field);
+  form2.fields.push_back(field);
+  autofill_test::CreateTestFormField(
+      "Last name:", "last_name", "Washington", "text", &field);
+  form2.fields.push_back(field);
+  autofill_test::CreateTestFormField(
+      "Email:", "email", "theprez@gmail.com", "text", &field);
+  form2.fields.push_back(field);
+  autofill_test::CreateTestFormField(
+      "Address:", "address1", "21 Laussat St", "text", &field);
+  form2.fields.push_back(field);
+  autofill_test::CreateTestFormField(
+      "City:", "city", "San Francisco", "text", &field);
+  form2.fields.push_back(field);
+  autofill_test::CreateTestFormField(
+      "State:", "state", "California", "text", &field);
+  form2.fields.push_back(field);
+  autofill_test::CreateTestFormField(
+      "Zip:", "zip", "94102", "text", &field);
+  form2.fields.push_back(field);
+  autofill_test::CreateTestFormField(
+      "Phone number:", "phone_number", "214-555-1234", "text", &field);
+  form2.fields.push_back(field);
+
+  FormStructure form_structure2(form2);
+  form_structure2.DetermineHeuristicTypes();
+  EXPECT_TRUE(personal_data_->ImportFormData(form_structure2,
+                                             &imported_credit_card));
+  ASSERT_FALSE(imported_credit_card);
+
+  // Verify that the web database has been updated and the notification sent.
+  EXPECT_CALL(personal_data_observer_,
+              OnPersonalDataChanged()).WillOnce(QuitUIMessageLoop());
+  MessageLoop::current()->Run();
+
+  const std::vector<AutofillProfile*>& results2 = personal_data_->profiles();
+
+  // Modify expected to include multi-valued fields.
+  std::vector<string16> values;
+  expected.GetMultiInfo(PHONE_HOME_CITY_AND_NUMBER, &values);
+  values.push_back(ASCIIToUTF16("214-555-1234"));
+  expected.SetMultiInfo(PHONE_HOME_CITY_AND_NUMBER, values);
+
+  ASSERT_EQ(1U, results2.size());
+  EXPECT_EQ(0, expected.CompareMulti(*results2[0]));
+}
