@@ -7,6 +7,7 @@
 #include <set>
 
 #include "base/callback.h"
+#include "base/command_line.h"
 #include "base/debug/trace_event.h"
 #include "base/file_path.h"
 #include "base/json/json_reader.h"
@@ -15,7 +16,7 @@
 #include "base/message_loop.h"
 #include "base/path_service.h"
 #include "base/process_util.h"
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
 #include "base/string_number_conversions.h"
 #include "base/string_util.h"
 #include "base/synchronization/waitable_event.h"
@@ -28,7 +29,6 @@
 #include "chrome/browser/autofill/autofill_manager.h"
 #include "chrome/browser/automation/automation_browser_tracker.h"
 #include "chrome/browser/automation/automation_extension_tracker.h"
-#include "chrome/browser/automation/automation_omnibox_tracker.h"
 #include "chrome/browser/automation/automation_provider_list.h"
 #include "chrome/browser/automation/automation_provider_observers.h"
 #include "chrome/browser/automation/automation_resource_message_filter.h"
@@ -41,10 +41,7 @@
 #include "chrome/browser/browsing_data_remover.h"
 #include "chrome/browser/character_encoding.h"
 #include "chrome/browser/content_settings/host_content_settings_map.h"
-#include "chrome/browser/debugger/devtools_manager.h"
 #include "chrome/browser/dom_operation_notification_details.h"
-#include "chrome/browser/download/download_item.h"
-#include "chrome/browser/download/save_package.h"
 #include "chrome/browser/extensions/crx_installer.h"
 #include "chrome/browser/extensions/extension_browser_event_router.h"
 #include "chrome/browser/extensions/extension_host.h"
@@ -55,17 +52,14 @@
 #include "chrome/browser/extensions/extension_toolbar_model.h"
 #include "chrome/browser/extensions/user_script_master.h"
 #include "chrome/browser/net/url_request_mock_util.h"
-#include "chrome/browser/platform_util.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/printing/print_job.h"
 #include "chrome/browser/profiles/profile_manager.h"
 #include "chrome/browser/ssl/ssl_blocking_page.h"
-#include "chrome/browser/ssl/ssl_manager.h"
 #include "chrome/browser/ui/app_modal_dialogs/app_modal_dialog.h"
 #include "chrome/browser/ui/app_modal_dialogs/app_modal_dialog_queue.h"
 #include "chrome/browser/ui/browser_list.h"
 #include "chrome/browser/ui/browser_window.h"
-#include "chrome/browser/ui/download/download_tab_helper.h"
 #include "chrome/browser/ui/find_bar/find_bar.h"
 #include "chrome/browser/ui/find_bar/find_bar_controller.h"
 #include "chrome/browser/ui/find_bar/find_notification_details.h"
@@ -83,10 +77,13 @@
 #include "chrome/common/pref_names.h"
 #include "chrome/common/render_messages.h"
 #include "chrome/common/url_constants.h"
-#include "chrome/test/automation/tab_proxy.h"
 #include "content/browser/browser_thread.h"
+#include "content/browser/debugger/devtools_manager.h"
+#include "content/browser/download/download_item.h"
+#include "content/browser/download/save_package.h"
 #include "content/browser/renderer_host/render_process_host.h"
 #include "content/browser/renderer_host/render_view_host.h"
+#include "content/browser/ssl/ssl_manager.h"
 #include "content/browser/tab_contents/navigation_entry.h"
 #include "content/browser/tab_contents/tab_contents.h"
 #include "content/browser/tab_contents/tab_contents_view.h"
@@ -99,20 +96,27 @@
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebFindOptions.h"
 #include "webkit/glue/password_form.h"
 
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
 #include "chrome/browser/external_tab_container_win.h"
 #endif  // defined(OS_WIN)
 
+#if defined(OS_CHROMEOS)
+#include "chrome/browser/chromeos/login/user_manager.h"
+#endif  // defined(OS_CHROMEOS)
+
 using WebKit::WebFindOptions;
 using base::Time;
 
 AutomationProvider::AutomationProvider(Profile* profile)
     : profile_(profile),
       reply_message_(NULL),
-      reinitialize_on_channel_error_(false),
+      reinitialize_on_channel_error_(
+          CommandLine::ForCurrentProcess()->HasSwitch(
+              switches::kAutomationReinitializeOnChannelError)),
       is_connected_(false),
       initial_tab_loads_complete_(false),
-      network_library_initialized_(true) {
+      network_library_initialized_(true),
+      login_webui_ready_(true) {
   TRACE_EVENT_BEGIN_ETW("AutomationProvider::AutomationProvider", 0, "");
 
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
@@ -121,8 +125,7 @@ AutomationProvider::AutomationProvider(Profile* profile)
   extension_tracker_.reset(new AutomationExtensionTracker(this));
   tab_tracker_.reset(new AutomationTabTracker(this));
   window_tracker_.reset(new AutomationWindowTracker(this));
-  automation_omnibox_tracker_.reset(new AutomationOmniboxTracker(this));
-  new_tab_ui_load_observer_.reset(new NewTabUILoadObserver(this));
+  new_tab_ui_load_observer_.reset(new NewTabUILoadObserver(this, profile));
   metric_event_duration_observer_.reset(new MetricEventDurationObserver());
   extension_test_result_observer_.reset(
       new ExtensionTestResultNotificationObserver(this));
@@ -161,15 +164,22 @@ bool AutomationProvider::InitializeChannel(const std::string& channel_id) {
     automation_resource_message_filter_ = new AutomationResourceMessageFilter;
   }
 
-  channel_.reset(new IPC::SyncChannel(
+  channel_.reset(new IPC::ChannelProxy(
       effective_channel_id,
       GetChannelMode(use_named_interface),
       this,
-      BrowserThread::GetMessageLoopProxyForThread(BrowserThread::IO),
-      true, g_browser_process->shutdown_event()));
+      BrowserThread::GetMessageLoopProxyForThread(BrowserThread::IO)));
   channel_->AddFilter(automation_resource_message_filter_);
 
 #if defined(OS_CHROMEOS)
+  // Wait for webui login to be ready.
+  // Observer will delete itself.
+  if (CommandLine::ForCurrentProcess()->HasSwitch(switches::kWebUILogin) &&
+      !chromeos::UserManager::Get()->user_is_logged_in()) {
+    login_webui_ready_ = false;
+    new LoginWebuiReadyObserver(this);
+  }
+
   // Wait for the network manager to initialize.
   // The observer will delete itself when done.
   network_library_initialized_ = false;
@@ -205,13 +215,20 @@ void AutomationProvider::SetExpectedTabCount(size_t expected_tabs) {
 
 void AutomationProvider::OnInitialTabLoadsComplete() {
   initial_tab_loads_complete_ = true;
-  if (is_connected_ && network_library_initialized_)
+  if (is_connected_ && network_library_initialized_ && login_webui_ready_)
     Send(new AutomationMsg_InitialLoadsComplete());
 }
 
 void AutomationProvider::OnNetworkLibraryInit() {
   network_library_initialized_ = true;
-  if (is_connected_ && initial_tab_loads_complete_)
+  if (is_connected_ && initial_tab_loads_complete_ && login_webui_ready_)
+    Send(new AutomationMsg_InitialLoadsComplete());
+}
+
+void AutomationProvider::OnLoginWebuiReady() {
+  login_webui_ready_ = true;
+  if (is_connected_ && initial_tab_loads_complete_ &&
+      network_library_initialized_)
     Send(new AutomationMsg_InitialLoadsComplete());
 }
 
@@ -263,8 +280,6 @@ DictionaryValue* AutomationProvider::GetDictionaryFromDownloadItem(
   dl_item_value->SetBoolean("is_paused", download->is_paused());
   dl_item_value->SetBoolean("open_when_complete",
                             download->open_when_complete());
-  dl_item_value->SetBoolean("is_extension_install",
-                            download->is_extension_install());
   dl_item_value->SetBoolean("is_temporary", download->is_temporary());
   dl_item_value->SetBoolean("is_otr", download->is_otr());  // incognito
   dl_item_value->SetString("state", state_to_string[download->state()]);
@@ -307,7 +322,8 @@ void AutomationProvider::OnChannelConnected(int pid) {
 
   // Send a hello message with our current automation protocol version.
   channel_->Send(new AutomationMsg_Hello(GetProtocolVersion()));
-  if (initial_tab_loads_complete_ && network_library_initialized_)
+  if (initial_tab_loads_complete_ && network_library_initialized_ &&
+      login_webui_ready_)
     Send(new AutomationMsg_InitialLoadsComplete());
 }
 
@@ -331,13 +347,10 @@ bool AutomationProvider::OnMessageReceived(const IPC::Message& message) {
     IPC_MESSAGE_HANDLER(AutomationMsg_ReloadAsync, ReloadAsync)
     IPC_MESSAGE_HANDLER(AutomationMsg_StopAsync, StopAsync)
     IPC_MESSAGE_HANDLER(AutomationMsg_SetPageFontSize, OnSetPageFontSize)
-    IPC_MESSAGE_HANDLER_DELAY_REPLY(AutomationMsg_InstallExtension,
-                                    InstallExtension)
     IPC_MESSAGE_HANDLER_DELAY_REPLY(AutomationMsg_WaitForExtensionTestResult,
                                     WaitForExtensionTestResult)
-    IPC_MESSAGE_HANDLER_DELAY_REPLY(
-        AutomationMsg_InstallExtensionAndGetHandle,
-        InstallExtensionAndGetHandle)
+    IPC_MESSAGE_HANDLER_DELAY_REPLY(AutomationMsg_InstallExtension,
+                                    InstallExtension)
     IPC_MESSAGE_HANDLER(AutomationMsg_UninstallExtension,
                         UninstallExtension)
     IPC_MESSAGE_HANDLER_DELAY_REPLY(AutomationMsg_EnableExtension,
@@ -355,7 +368,7 @@ bool AutomationProvider::OnMessageReceived(const IPC::Message& message) {
     IPC_MESSAGE_HANDLER(AutomationMsg_RemoveBrowsingData, RemoveBrowsingData)
     IPC_MESSAGE_HANDLER(AutomationMsg_JavaScriptStressTestControl,
                         JavaScriptStressTestControl)
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
     // These are for use with external tabs.
     IPC_MESSAGE_HANDLER(AutomationMsg_CreateExternalTab, CreateExternalTab)
     IPC_MESSAGE_HANDLER(AutomationMsg_ProcessUnhandledAccelerator,
@@ -582,16 +595,7 @@ class SetProxyConfigTask : public Task {
 
 void AutomationProvider::SetProxyConfig(const std::string& new_proxy_config) {
   net::URLRequestContextGetter* context_getter =
-      Profile::GetDefaultRequestContext();
-  if (!context_getter) {
-    FilePath user_data_dir;
-    PathService::Get(chrome::DIR_USER_DATA, &user_data_dir);
-    ProfileManager* profile_manager = g_browser_process->profile_manager();
-    DCHECK(profile_manager);
-    Profile* profile = profile_manager->GetDefaultProfile(user_data_dir);
-    DCHECK(profile);
-    context_getter = profile->GetRequestContext();
-  }
+      profile_->GetRequestContext();
   DCHECK(context_getter);
 
   BrowserThread::PostTask(
@@ -727,9 +731,10 @@ void AutomationProvider::OnSetPageFontSize(int tab_handle,
     NavigationController* tab = tab_tracker_->GetResource(tab_handle);
     DCHECK(tab != NULL);
     if (tab && tab->tab_contents()) {
-      DCHECK(tab->tab_contents()->profile() != NULL);
-      tab->tab_contents()->profile()->GetPrefs()->SetInteger(
-          prefs::kWebKitDefaultFontSize, font_size);
+      DCHECK(tab->tab_contents()->browser_context() != NULL);
+      Profile* profile =
+          Profile::FromBrowserContext(tab->tab_contents()->browser_context());
+      profile->GetPrefs()->SetInteger(prefs::kWebKitDefaultFontSize, font_size);
     }
   }
 }
@@ -752,7 +757,7 @@ void AutomationProvider::JavaScriptStressTestControl(int tab_handle,
     return;
   }
 
-  view->Send(new ViewMsg_JavaScriptStressTestControl(
+  view->Send(new ChromeViewMsg_JavaScriptStressTestControl(
       view->routing_id(), cmd, param));
 }
 
@@ -777,26 +782,6 @@ RenderViewHost* AutomationProvider::GetViewForTab(int tab_handle) {
   return NULL;
 }
 
-void AutomationProvider::InstallExtension(const FilePath& crx_path,
-                                          IPC::Message* reply_message) {
-  ExtensionService* service = profile_->GetExtensionService();
-  if (service) {
-    // The observer will delete itself when done.
-    new ExtensionInstallNotificationObserver(this,
-                                             AutomationMsg_InstallExtension::ID,
-                                             reply_message);
-
-    // Pass NULL for a silent install with no UI.
-    scoped_refptr<CrxInstaller> installer(service->MakeCrxInstaller(NULL));
-    installer->set_install_cause(extension_misc::INSTALL_CAUSE_AUTOMATION);
-    installer->InstallCrx(crx_path);
-  } else {
-    AutomationMsg_InstallExtension::WriteReplyParams(
-        reply_message, AUTOMATION_MSG_EXTENSION_INSTALL_FAILED);
-    Send(reply_message);
-  }
-}
-
 void AutomationProvider::WaitForExtensionTestResult(
     IPC::Message* reply_message) {
   DCHECK(!reply_message_);
@@ -806,26 +791,33 @@ void AutomationProvider::WaitForExtensionTestResult(
   extension_test_result_observer_->MaybeSendResult();
 }
 
-void AutomationProvider::InstallExtensionAndGetHandle(
-    const FilePath& crx_path, bool with_ui, IPC::Message* reply_message) {
+void AutomationProvider::InstallExtension(
+    const FilePath& extension_path, bool with_ui,
+    IPC::Message* reply_message) {
   ExtensionService* service = profile_->GetExtensionService();
   ExtensionProcessManager* manager = profile_->GetExtensionProcessManager();
   if (service && manager) {
     // The observer will delete itself when done.
     new ExtensionReadyNotificationObserver(
         manager,
+        service,
         this,
-        AutomationMsg_InstallExtensionAndGetHandle::ID,
+        AutomationMsg_InstallExtension::ID,
         reply_message);
 
-    ExtensionInstallUI* client =
-        (with_ui ? new ExtensionInstallUI(profile_) : NULL);
-    scoped_refptr<CrxInstaller> installer(service->MakeCrxInstaller(client));
-    installer->set_install_cause(extension_misc::INSTALL_CAUSE_AUTOMATION);
-    installer->InstallCrx(crx_path);
+    if (extension_path.MatchesExtension(FILE_PATH_LITERAL(".crx"))) {
+      ExtensionInstallUI* client =
+          (with_ui ? new ExtensionInstallUI(profile_) : NULL);
+      scoped_refptr<CrxInstaller> installer(service->MakeCrxInstaller(client));
+      if (!with_ui)
+        installer->set_allow_silent_install(true);
+      installer->set_install_cause(extension_misc::INSTALL_CAUSE_AUTOMATION);
+      installer->InstallCrx(extension_path);
+    } else {
+      service->LoadExtension(extension_path, with_ui);
+    }
   } else {
-    AutomationMsg_InstallExtensionAndGetHandle::WriteReplyParams(
-        reply_message, 0);
+    AutomationMsg_InstallExtension::WriteReplyParams(reply_message, 0);
     Send(reply_message);
   }
 }
@@ -854,6 +846,7 @@ void AutomationProvider::EnableExtension(int extension_handle,
     // The observer will delete itself when done.
     new ExtensionReadyNotificationObserver(
         manager,
+        service,
         this,
         AutomationMsg_EnableExtension::ID,
         reply_message);
@@ -976,9 +969,6 @@ void AutomationProvider::GetExtensionProperty(
 void AutomationProvider::SaveAsAsync(int tab_handle) {
   NavigationController* tab = NULL;
   TabContents* tab_contents = GetTabContentsForHandle(tab_handle, &tab);
-  if (tab_contents) {
-    TabContentsWrapper* wrapper =
-        TabContentsWrapper::GetCurrentWrapperForContents(tab_contents);
-    wrapper->download_tab_helper()->OnSavePage();
-  }
+  if (tab_contents)
+    tab_contents->OnSavePage();
 }
