@@ -6,19 +6,15 @@
 
 #include "base/path_service.h"
 #include "base/metrics/histogram.h"
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
 #include "chrome/browser/automation/url_request_automation_job.h"
 #include "chrome/browser/content_settings/tab_specific_content_settings.h"
-#include "chrome/browser/net/url_request_failed_dns_job.h"
-#include "chrome/browser/net/url_request_mock_http_job.h"
 #include "chrome/browser/net/url_request_mock_util.h"
-#include "chrome/browser/net/url_request_slow_download_job.h"
-#include "chrome/browser/net/url_request_slow_http_job.h"
 #include "chrome/common/automation_messages.h"
 #include "chrome/common/chrome_paths.h"
+#include "chrome/common/render_messages.h"
 #include "content/browser/browser_message_filter.h"
 #include "content/browser/browser_thread.h"
-#include "content/common/view_messages.h"
 #include "googleurl/src/gurl.h"
 #include "net/base/net_errors.h"
 #include "net/url_request/url_request_context.h"
@@ -35,63 +31,6 @@ base::LazyInstance<AutomationResourceMessageFilter::CompletionCallbackMap>
 int AutomationResourceMessageFilter::unique_request_id_ = 1;
 int AutomationResourceMessageFilter::next_completion_callback_id_ = 0;
 
-// CookieStore specialization to enable fetching and setting cookies over the
-// automation channel. This cookie store is transient i.e. it maintains cookies
-// for the duration of the current request to set or get cookies from the
-// renderer.
-class AutomationCookieStore : public net::CookieStore {
- public:
-  AutomationCookieStore(AutomationResourceMessageFilter* automation_client,
-                        int tab_handle)
-      : automation_client_(automation_client),
-        tab_handle_(tab_handle) {
-  }
-
-  virtual ~AutomationCookieStore() {
-    DVLOG(1) << "In " << __FUNCTION__;
-  }
-
-  // CookieStore implementation.
-  virtual bool SetCookieWithOptions(const GURL& url,
-                                    const std::string& cookie_line,
-                                    const net::CookieOptions& options) {
-    // The cookie_string_ is available only once, i.e. once it is read by
-    // it is invalidated.
-    cookie_string_ = cookie_line;
-    return true;
-  }
-
-  virtual std::string GetCookiesWithOptions(const GURL& url,
-                                            const net::CookieOptions& options) {
-    return cookie_string_;
-  }
-
-  virtual void GetCookiesWithInfo(const GURL& url,
-                                  const net::CookieOptions& options,
-                                  std::string* cookie_line,
-                                  std::vector<CookieInfo>* cookie_infos) {
-    NOTREACHED() << "Should not get called for an automation profile";
-  }
-
-  virtual void DeleteCookie(const GURL& url,
-                            const std::string& cookie_name) {
-    NOTREACHED() << "Should not get called for an automation profile";
-  }
-
-  virtual net::CookieMonster* GetCookieMonster() {
-    NOTREACHED() << "Should not get called for an automation profile";
-    return NULL;
-  }
-
- protected:
-  scoped_refptr<AutomationResourceMessageFilter> automation_client_;
-  int tab_handle_;
-  std::string cookie_string_;
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(AutomationCookieStore);
-};
-
 AutomationResourceMessageFilter::AutomationDetails::AutomationDetails()
     : tab_handle(0),
       ref_count(1),
@@ -113,7 +52,7 @@ struct AutomationResourceMessageFilter::CookieCompletionInfo {
   scoped_refptr<net::URLRequestContext> context;
   int render_process_id;
   IPC::Message* reply_msg;
-  scoped_refptr<net::CookieStore> cookie_store;
+  scoped_refptr<AutomationResourceMessageFilter> automation_message_filter;
 };
 
 AutomationResourceMessageFilter::AutomationResourceMessageFilter()
@@ -162,6 +101,18 @@ void AutomationResourceMessageFilter::OnChannelClosing() {
       index++;
     }
   }
+
+  CompletionCallbackMap::iterator callback_index =
+      completion_callback_map_.Get().begin();
+  while (callback_index != completion_callback_map_.Get().end()) {
+    const CookieCompletionInfo& cookie_completion_info =
+        (*callback_index).second;
+    if (cookie_completion_info.automation_message_filter.get() == this) {
+      completion_callback_map_.Get().erase(callback_index++);
+    } else {
+      callback_index++;
+    }
+  }
 }
 
 // Called on the IPC thread:
@@ -187,7 +138,10 @@ bool AutomationResourceMessageFilter::OnMessageReceived(
   }
 
   bool handled = true;
-  IPC_BEGIN_MESSAGE_MAP(AutomationResourceMessageFilter, message)
+  bool deserialize_success = true;
+  IPC_BEGIN_MESSAGE_MAP_EX(AutomationResourceMessageFilter,
+                           message,
+                           deserialize_success)
     IPC_MESSAGE_HANDLER(AutomationMsg_SetFilteredInet,
                         OnSetFilteredInet)
     IPC_MESSAGE_HANDLER(AutomationMsg_GetFilteredInetHitCount,
@@ -197,7 +151,13 @@ bool AutomationResourceMessageFilter::OnMessageReceived(
     IPC_MESSAGE_HANDLER(AutomationMsg_GetCookiesHostResponse,
                         OnGetCookiesHostResponse)
     IPC_MESSAGE_UNHANDLED(handled = false)
-  IPC_END_MESSAGE_MAP()
+  IPC_END_MESSAGE_MAP_EX()
+
+  if (!deserialize_success) {
+    LOG(ERROR) << "Failed to deserialize IPC message. "
+               << "Closing the automation channel.";
+    channel_->Close();
+  }
 
   return handled;
 }
@@ -307,20 +267,15 @@ void AutomationResourceMessageFilter::RegisterRenderViewInIOThread(
     bool pending_view) {
   RendererId renderer_key(renderer_pid, renderer_id);
 
-  scoped_refptr<net::CookieStore> cookie_store(
-      new AutomationCookieStore(filter, tab_handle));
-
   RenderViewMap::iterator automation_details_iter(
       filtered_render_views_.Get().find(renderer_key));
   if (automation_details_iter != filtered_render_views_.Get().end()) {
-    DCHECK(automation_details_iter->second.ref_count > 0);
+    DCHECK_GT(automation_details_iter->second.ref_count, 0);
     automation_details_iter->second.ref_count++;
   } else {
     filtered_render_views_.Get()[renderer_key] =
         AutomationDetails(tab_handle, filter, pending_view);
   }
-
-  filtered_render_views_.Get()[renderer_key].set_cookie_store(cookie_store);
 }
 
 // static
@@ -364,9 +319,6 @@ bool AutomationResourceMessageFilter::ResumePendingRenderViewInIOThread(
 
   DCHECK(automation_details_iter->second.is_pending_render_view);
 
-  scoped_refptr<net::CookieStore> cookie_store(
-      new AutomationCookieStore(filter, tab_handle));
-
   AutomationResourceMessageFilter* old_filter =
       automation_details_iter->second.filter;
   DCHECK(old_filter != NULL);
@@ -374,8 +326,6 @@ bool AutomationResourceMessageFilter::ResumePendingRenderViewInIOThread(
   filtered_render_views_.Get()[renderer_key] =
       AutomationDetails(tab_handle, filter, false);
 
-  filtered_render_views_.Get()[renderer_key].set_cookie_store(cookie_store);
-
   ResumeJobsForPendingView(tab_handle, old_filter, filter);
   return true;
 }
@@ -461,7 +411,6 @@ void AutomationResourceMessageFilter::GetCookiesForUrl(
 
   DCHECK(automation_details_iter != filtered_render_views_.Get().end());
   DCHECK(automation_details_iter->second.filter != NULL);
-  DCHECK(automation_details_iter->second.cookie_store_.get() != NULL);
 
   int completion_callback_id = GetNextCompletionCallbackId();
   DCHECK(!ContainsKey(completion_callback_map_.Get(), completion_callback_id));
@@ -471,7 +420,8 @@ void AutomationResourceMessageFilter::GetCookiesForUrl(
   cookie_info.context = context;
   cookie_info.render_process_id = render_process_id;
   cookie_info.reply_msg = reply_msg;
-  cookie_info.cookie_store = automation_details_iter->second.cookie_store_;
+  cookie_info.automation_message_filter =
+      automation_details_iter->second.filter;
 
   completion_callback_map_.Get()[completion_callback_id] = cookie_info;
 
@@ -496,26 +446,10 @@ void AutomationResourceMessageFilter::OnGetCookiesHostResponse(
     return;
   }
 
-  scoped_refptr<net::CookieStore> cookie_store = index->second.cookie_store;
-  DCHECK(cookie_store.get() != NULL);
-  cookie_store->SetCookieWithOptions(url, cookies, net::CookieOptions());
-
-  int render_view_id = index->second.reply_msg->routing_id();
-  ViewHostMsg_GetCookies::WriteReplyParams(index->second.reply_msg, cookies);
+  ChromeViewHostMsg_GetCookies::WriteReplyParams(index->second.reply_msg,
+                                                 cookies);
   index->second.filter->Send(index->second.reply_msg);
-  net::CookieMonster* cookie_monster = index->second.context->cookie_store()->
-      GetCookieMonster();
-  net::CookieList cookie_list = cookie_monster->GetAllCookiesForURLWithOptions(
-      url, net::CookieOptions());
-  BrowserThread::PostTask(
-      BrowserThread::UI, FROM_HERE,
-      NewRunnableFunction(
-          &TabSpecificContentSettings::CookiesRead,
-          index->second.render_process_id, render_view_id, url, cookie_list,
-          !success));
 
-  // The cookie for this URL is only valid until it is read by the callback.
-  cookie_store->SetCookieWithOptions(url, "", net::CookieOptions());
   completion_callback_map_.Get().erase(index);
 }
 
