@@ -81,6 +81,10 @@ int BrowserX11IOErrorHandler(Display* d) {
 
 }  // namespace
 
+BrowserMainPartsGtk::BrowserMainPartsGtk(const MainFunctionParams& parameters)
+    : BrowserMainPartsPosix(parameters) {
+}
+
 void BrowserMainPartsGtk::PreEarlyInitialization() {
   DetectRunningAsRoot();
 
@@ -163,9 +167,13 @@ void BrowserMainPartsGtk::SetupSandbox() {
   zhost->Init(sandbox_cmd);
 }
 
+namespace content {
+
 void DidEndMainMessageLoop() {
 }
 
+}
+
 void RecordBreakpadStatusUMA(MetricsService* metrics) {
 #if defined(USE_LINUX_BREAKPAD)
   metrics->RecordBreakpadRegistration(IsCrashReporterEnabled());
@@ -179,6 +187,22 @@ void WarnAboutMinimumSystemRequirements() {
   // Nothing to warn about on GTK right now.
 }
 
+void ShowMissingLocaleMessageBox() {
+  GtkWidget* dialog = gtk_message_dialog_new(
+      NULL,
+      static_cast<GtkDialogFlags>(0),
+      GTK_MESSAGE_ERROR,
+      GTK_BUTTONS_CLOSE,
+      "%s",
+      chrome_browser::kMissingLocaleDataMessage);
+
+  gtk_window_set_title(GTK_WINDOW(dialog),
+                       chrome_browser::kMissingLocaleDataTitle);
+
+  gtk_dialog_run(GTK_DIALOG(dialog));
+  gtk_widget_destroy(dialog);
+}
+
 void RecordBrowserStartupTime() {
   // Not implemented on GTK for now.
 }
@@ -205,11 +229,3 @@ void SetBrowserX11ErrorHandlers() {
   // goes away.
   ui::SetX11ErrorHandlers(BrowserX11ErrorHandler, BrowserX11IOErrorHandler);
 }
-
-#if !defined(OS_CHROMEOS)
-// static
-BrowserMainParts* BrowserMainParts::CreateBrowserMainParts(
-    const MainFunctionParams& parameters) {
-  return new BrowserMainPartsGtk(parameters);
-}
-#endif
