@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -11,9 +11,9 @@ namespace chromeos {
 
 void BrightnessObserver::BrightnessChanged(int level, bool user_initiated) {
   if (user_initiated)
-    BrightnessBubble::GetInstance()->ShowBubble(level);
+    BrightnessBubble::GetInstance()->ShowBubble(level, true);
   else
-    BrightnessBubble::GetInstance()->UpdateWithoutShowingBubble(level);
+    BrightnessBubble::GetInstance()->UpdateWithoutShowingBubble(level, true);
 
   VolumeBubble::GetInstance()->HideBubble();
 }
