@@ -6,8 +6,6 @@
 #define CHROME_BROWSER_CHROMEOS_CROS_BRIGHTNESS_LIBRARY_H_
 #pragma once
 
-#include "third_party/cros/chromeos_brightness.h"
-
 namespace chromeos {
 
 class BrightnessLibrary {
@@ -19,12 +17,14 @@ class BrightnessLibrary {
 
   virtual ~BrightnessLibrary() {}
 
-  virtual void DecreaseScreenBrightness(bool allow_off) = 0;
-  virtual void IncreaseScreenBrightness() = 0;
+  virtual void Init() = 0;
 
   virtual void AddObserver(Observer* observer) = 0;
   virtual void RemoveObserver(Observer* observer) = 0;
 
+  virtual void DecreaseScreenBrightness(bool allow_off) = 0;
+  virtual void IncreaseScreenBrightness() = 0;
+
   // Factory function, creates a new instance and returns ownership.
   // For normal usage, access the singleton via CrosLibrary::Get().
   static BrightnessLibrary* GetImpl(bool stub);
