@@ -8,7 +8,7 @@
 
 #include "base/memory/scoped_ptr.h"
 #include "chrome/browser/chromeos/cros/cros_mock.h"
-#include "chrome/test/in_process_browser_test.h"
+#include "chrome/test/base/in_process_browser_test.h"
 
 namespace chromeos {
 
