@@ -93,6 +93,7 @@ class MockNetworkLibrary : public NetworkLibrary {
   MOCK_METHOD0(RequestCellularScan, void());
   MOCK_METHOD1(RequestCellularRegister, void(const std::string&));
   MOCK_METHOD1(SetCellularDataRoamingAllowed, void(bool));
+  MOCK_METHOD0(IsCellularAlwaysInRoaming, bool());
 
   MOCK_METHOD0(RequestNetworkScan, void(void));
   MOCK_CONST_METHOD1(HasProfileType, bool(NetworkProfileType));
@@ -153,6 +154,7 @@ class MockNetworkLibrary : public NetworkLibrary {
                                                    std::string*,
                                                    HardwareAddressFormat));
   MOCK_METHOD1(SetIPConfig, void(const NetworkIPConfig&));
+  MOCK_METHOD0(SwitchToPreferredNetwork, void(void));
 };
 
 }  // namespace chromeos
