@@ -5,6 +5,7 @@
 #include "chrome/browser/chromeos/cros/mount_library.h"
 
 #include <set>
+#include <vector>
 
 #include "base/message_loop.h"
 #include "base/string_util.h"
@@ -33,6 +34,20 @@ std::string MountLibrary::MountTypeToString(MountType type) {
   return "";
 }
 
+std::string MountLibrary::MountConditionToString(MountCondition condition) {
+  switch (condition) {
+    case MOUNT_CONDITION_NONE:
+      return "";
+    case MOUNT_CONDITION_UNKNOWN_FILESYSTEM:
+      return "unknown_filesystem";
+    case MOUNT_CONDITION_UNSUPPORTED_FILESYSTEM:
+      return "unsupported_filesystem";
+    default:
+      NOTREACHED();
+  }
+  return "";
+}
+
 // static
 MountType MountLibrary::MountTypeFromString(
     const std::string& type_str) {
@@ -48,18 +63,19 @@ MountType MountLibrary::MountTypeFromString(
 }
 
 MountLibrary::Disk::Disk(const std::string& device_path,
-     const std::string& mount_path,
-     const std::string& system_path,
-     const std::string& file_path,
-     const std::string& device_label,
-     const std::string& drive_label,
-     const std::string& parent_path,
-     DeviceType device_type,
-     uint64 total_size,
-     bool is_parent,
-     bool is_read_only,
-     bool has_media,
-     bool on_boot_device)
+    const std::string& mount_path,
+    const std::string& system_path,
+    const std::string& file_path,
+    const std::string& device_label,
+    const std::string& drive_label,
+    const std::string& parent_path,
+    const std::string& system_path_prefix,
+    DeviceType device_type,
+    uint64 total_size,
+    bool is_parent,
+    bool is_read_only,
+    bool has_media,
+    bool on_boot_device)
     : device_path_(device_path),
       mount_path_(mount_path),
       system_path_(system_path),
@@ -67,6 +83,7 @@ MountLibrary::Disk::Disk(const std::string& device_path,
       device_label_(device_label),
       drive_label_(drive_label),
       parent_path_(parent_path),
+      system_path_prefix_(system_path_prefix),
       device_type_(device_type),
       total_size_(total_size),
       is_parent_(is_parent),
@@ -77,6 +94,53 @@ MountLibrary::Disk::Disk(const std::string& device_path,
 
 MountLibrary::Disk::~Disk() {}
 
+class MountLibcrosProxyImpl : public MountLibcrosProxy {
+ public:
+  virtual void CallMountPath(const char* source_path,
+                             MountType type,
+                             const MountPathOptions& options,
+                             MountCompletedMonitor callback,
+                             void* object) OVERRIDE {
+    MountSourcePath(source_path, type, options, callback, object);
+  }
+
+  virtual void CallUnmountPath(const char* path,
+                               UnmountRequestCallback callback,
+                               void* object) OVERRIDE {
+    UnmountMountPoint(path, callback, object);
+  }
+
+  virtual void CallRequestMountInfo(RequestMountInfoCallback callback,
+                                    void* object) OVERRIDE {
+    RequestMountInfo(callback, object);
+  }
+
+  virtual void CallFormatDevice(const char* file_path,
+                                const char* filesystem,
+                                FormatRequestCallback callback,
+                                void* object) OVERRIDE {
+    FormatDevice(file_path, filesystem, callback, object);
+  }
+
+  virtual void CallGetDiskProperties(const char* device_path,
+                                     GetDiskPropertiesCallback callback,
+                                     void* object) OVERRIDE {
+    GetDiskProperties(device_path, callback, object);
+  }
+
+  virtual MountEventConnection MonitorCrosDisks(MountEventMonitor monitor,
+      MountCompletedMonitor mount_completed_monitor,
+      void* object) OVERRIDE {
+    return MonitorAllMountEvents(monitor, mount_completed_monitor, object);
+  }
+
+  virtual void DisconnectCrosDisksMonitorIfSet(MountEventConnection conn)
+      OVERRIDE {
+    if (conn)
+      DisconnectMountEventMonitor(conn);
+  }
+};
+
 class MountLibraryImpl : public MountLibrary {
 
   struct UnmountDeviceRecursiveCallbackData {
@@ -97,7 +161,8 @@ class MountLibraryImpl : public MountLibrary {
   };
 
  public:
-  MountLibraryImpl() : mount_status_connection_(NULL) {
+  MountLibraryImpl() : libcros_proxy_(new MountLibcrosProxyImpl()),
+                       mount_status_connection_(NULL) {
     if (CrosLibrary::Get()->EnsureLoaded())
       Init();
     else
@@ -105,8 +170,8 @@ class MountLibraryImpl : public MountLibrary {
   }
 
   virtual ~MountLibraryImpl() {
-    if (mount_status_connection_)
-      DisconnectMountEventMonitor(mount_status_connection_);
+      libcros_proxy_->DisconnectCrosDisksMonitorIfSet(
+          mount_status_connection_);
   }
 
   // MountLibrary overrides.
@@ -124,10 +189,14 @@ class MountLibraryImpl : public MountLibrary {
     CHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
     if (!CrosLibrary::Get()->EnsureLoaded()) {
       OnMountCompleted(MOUNT_ERROR_LIBRARY_NOT_LOADED,
-                       MountPointInfo(source_path, NULL, type));
+                       MountPointInfo(source_path,
+                                      NULL,
+                                      type,
+                                      MOUNT_CONDITION_NONE));
       return;
     }
-    MountSourcePath(source_path, type, options, &MountCompletedHandler, this);
+    libcros_proxy_->CallMountPath(source_path, type, options,
+                                  &MountCompletedHandler, this);
   }
 
   virtual void UnmountPath(const char* mount_path) OVERRIDE {
@@ -139,8 +208,63 @@ class MountLibraryImpl : public MountLibrary {
       return;
     }
 
-    UnmountMountPoint(mount_path, &MountLibraryImpl::UnmountMountPointCallback,
-                      this);
+    libcros_proxy_->CallUnmountPath(mount_path, &UnmountMountPointCallback,
+                                    this);
+  }
+
+  virtual void FormatUnmountedDevice(const char* file_path) OVERRIDE {
+    DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+    if (!CrosLibrary::Get()->EnsureLoaded()) {
+      OnFormatDevice(file_path,
+                     false,
+                     MOUNT_METHOD_ERROR_LOCAL,
+                     kLibraryNotLoaded);
+      return;
+    }
+    for (MountLibrary::DiskMap::iterator it = disks_.begin();
+        it != disks_.end(); ++it) {
+      if (it->second->file_path().compare(file_path) == 0 &&
+          !it->second->mount_path().empty()) {
+        OnFormatDevice(file_path,
+                       false,
+                       MOUNT_METHOD_ERROR_LOCAL,
+                       "Device is still mounted.");
+        return;
+      }
+    }
+    libcros_proxy_->CallFormatDevice(file_path, "vfat", &FormatDeviceCallback,
+                                     this);
+  }
+
+  virtual void FormatMountedDevice(const char* mount_path) OVERRIDE {
+    DCHECK(mount_path);
+    Disk* disk = NULL;
+    for (MountLibrary::DiskMap::iterator it = disks_.begin();
+         it != disks_.end(); ++it) {
+      if (it->second->mount_path().compare(mount_path) == 0) {
+        disk = it->second;
+        break;
+      }
+    }
+    if (!disk) {
+      OnFormatDevice(mount_path,
+                     false,
+                     MOUNT_METHOD_ERROR_LOCAL,
+                     "Device with this mount path not found.");
+      return;
+    }
+
+    if (formatting_pending_.find(disk->device_path()) !=
+        formatting_pending_.end()) {
+      OnFormatDevice(mount_path,
+                     false,
+                     MOUNT_METHOD_ERROR_LOCAL,
+                     "Formatting is already pending.");
+      return;
+    }
+    // Formatting process continues, after unmounting.
+    formatting_pending_[disk->device_path()] = disk->file_path();
+    UnmountPath(disk->mount_path().c_str());
   }
 
   virtual void UnmountDeviceRecursive(const char* device_path,
@@ -186,10 +310,8 @@ class MountLibraryImpl : public MountLibrary {
       for (std::vector<const char*>::iterator it = devices_to_unmount.begin();
            it != devices_to_unmount.end();
            ++it) {
-        UnmountMountPoint(
-            *it,
-            &MountLibraryImpl::UnmountDeviceRecursiveCallback,
-            cb_data);
+        libcros_proxy_->CallUnmountPath(*it, &UnmountDeviceRecursiveCallback,
+                                        cb_data);
       }
     } else {
       LOG(WARNING) << "Unmount recursive request failed for device "
@@ -207,13 +329,18 @@ class MountLibraryImpl : public MountLibrary {
                          kLibraryNotLoaded);
       return;
     }
-    RequestMountInfo(&MountLibraryImpl::RequestMountInfoCallback,
-                     this);
+    libcros_proxy_->CallRequestMountInfo(RequestMountInfoCallback, this);
   }
 
   const DiskMap& disks() const OVERRIDE { return disks_; }
   const MountPointMap& mount_points() const OVERRIDE { return mount_points_; }
 
+  virtual void SetLibcrosProxy(MountLibcrosProxy* proxy) OVERRIDE {
+    libcros_proxy_->DisconnectCrosDisksMonitorIfSet(mount_status_connection_);
+    libcros_proxy_.reset(proxy);
+    mount_status_connection_ = libcros_proxy_->MonitorCrosDisks(
+        &MonitorMountEventsHandler, &MountCompletedHandler, this);
+  }
  private:
   // Callback for MountComplete signal and MountSourcePath method.
   static void MountCompletedHandler(void* object,
@@ -222,9 +349,20 @@ class MountLibraryImpl : public MountLibrary {
                                     MountType type,
                                     const char* mount_path) {
     DCHECK(object);
+    MountCondition mount_condition = MOUNT_CONDITION_NONE;
+    if (type == MOUNT_TYPE_DEVICE) {
+      if (error_code == MOUNT_ERROR_UNKNOWN_FILESYSTEM)
+          mount_condition = MOUNT_CONDITION_UNKNOWN_FILESYSTEM;
+      if (error_code == MOUNT_ERROR_UNSUPORTED_FILESYSTEM)
+          mount_condition = MOUNT_CONDITION_UNSUPPORTED_FILESYSTEM;
+    }
+
     MountLibraryImpl* self = static_cast<MountLibraryImpl*>(object);
     self->OnMountCompleted(static_cast<MountError>(error_code),
-                           MountPointInfo(source_path, mount_path, type));
+                           MountPointInfo(source_path,
+                                          mount_path,
+                                          type,
+                                          mount_condition));
   }
 
   // Callback for UnmountRemovableDevice method.
@@ -237,6 +375,23 @@ class MountLibraryImpl : public MountLibrary {
     self->OnUnmountPath(mount_path, error, error_message);
   }
 
+  // Callback for FormatRemovableDevice method.
+  static void FormatDeviceCallback(void* object,
+                                   const char* file_path,
+                                   bool success,
+                                   MountMethodErrorType error,
+                                   const char* error_message) {
+    DCHECK(object);
+    MountLibraryImpl* self = static_cast<MountLibraryImpl*>(object);
+    const char* device_path = self->FilePathToDevicePath(file_path);
+    if (!device_path) {
+          LOG(ERROR) << "Error while handling disks metadata. Cannot find "
+                     << "device that is being formatted.";
+      return;
+    }
+    self->OnFormatDevice(device_path, success, error, error_message);
+  }
+
   // Callback for UnmountDeviceRecursive.
   static void UnmountDeviceRecursiveCallback(void* object,
                                              const char* mount_path,
@@ -254,7 +409,7 @@ class MountLibraryImpl : public MountLibrary {
       cb_data->success = false;
     } else if (error == MOUNT_METHOD_ERROR_NONE) {
       LOG(INFO) << mount_path <<  " unmounted.";
-   }
+    }
 
     // This is safe as long as all callbacks are called on the same thread as
     // UnmountDeviceRecursive.
@@ -310,19 +465,20 @@ class MountLibraryImpl : public MountLibrary {
 
     FireMountCompleted(MOUNTING, error_code, mount_info);
 
-    if (error_code == MOUNT_ERROR_NONE &&
+    // If the device is corrupted but it's still possible to format it, it will
+    // be fake mounted.
+    if ((error_code == MOUNT_ERROR_NONE || mount_info.mount_condition) &&
         mount_points_.find(mount_info.mount_path) == mount_points_.end()) {
       mount_points_.insert(MountPointMap::value_type(
                                mount_info.mount_path.c_str(),
                                mount_info));
     }
 
-    if (error_code == MOUNT_ERROR_NONE &&
+    if ((error_code == MOUNT_ERROR_NONE || mount_info.mount_condition) &&
         mount_info.mount_type == MOUNT_TYPE_DEVICE &&
         !mount_info.source_path.empty() &&
         !mount_info.mount_path.empty()) {
-      std::string path(mount_info.source_path);
-      DiskMap::iterator iter = disks_.find(path);
+      DiskMap::iterator iter = disks_.find(mount_info.source_path);
       if (iter == disks_.end()) {
         // disk might have been removed by now?
         return;
@@ -348,19 +504,25 @@ class MountLibraryImpl : public MountLibrary {
           MOUNT_ERROR_NONE,
           MountPointInfo(mount_points_it->second.source_path.c_str(),
                          mount_points_it->second.mount_path.c_str(),
-                         mount_points_it->second.mount_type));
+                         mount_points_it->second.mount_type,
+                         mount_points_it->second.mount_condition));
       std::string path(mount_points_it->second.source_path);
       mount_points_.erase(mount_points_it);
-
       DiskMap::iterator iter = disks_.find(path);
       if (iter == disks_.end()) {
-        // disk might have been removed by now?
+        // disk might have been removed by now.
         return;
       }
       Disk* disk = iter->second;
       DCHECK(disk);
       disk->clear_mount_path();
-      FireDiskStatusUpdate(MOUNT_DISK_UNMOUNTED, disk);
+      // Check if there is a formatting scheduled
+      PathMap::iterator it = formatting_pending_.find(disk->device_path());
+      if (it != formatting_pending_.end()) {
+        const std::string file_path = it->second;
+        formatting_pending_.erase(it);
+        FormatUnmountedDevice(file_path.c_str());
+      }
     } else {
       LOG(WARNING) << "Unmount request failed for device "
                    << mount_path << ", with error: "
@@ -368,6 +530,23 @@ class MountLibraryImpl : public MountLibrary {
     }
   }
 
+  void OnFormatDevice(const char* device_path,
+                      bool success,
+                      MountMethodErrorType error,
+                      const char* error_message) {
+    DCHECK(device_path);
+    if (error == MOUNT_METHOD_ERROR_NONE && device_path && success) {
+      FireDeviceStatusUpdate(MOUNT_FORMATTING_STARTED, device_path);
+    } else {
+      FireDeviceStatusUpdate(MOUNT_FORMATTING_STARTED,
+                             std::string("!") + device_path);
+      LOG(WARNING) << "Format request failed for device "
+                   << device_path << ", with error: "
+                   << (error_message ? error_message : "Unknown");
+    }
+  }
+
+
   void OnGetDiskProperties(const char* device_path,
                            const DiskInfo* disk1,
                            MountMethodErrorType error,
@@ -395,6 +574,7 @@ class MountLibraryImpl : public MountLibrary {
         disks_.erase(iter);
         is_new = false;
       }
+
       std::string path;
       std::string mountpath;
       std::string systempath;
@@ -431,6 +611,7 @@ class MountLibraryImpl : public MountLibrary {
                                 devicelabel,
                                 drivelabel,
                                 parentpath,
+                                FindSystemPathPrefix(systempath),
                                 disk->device_type(),
                                 disk->size(),
                                 disk->is_drive(),
@@ -464,9 +645,8 @@ class MountLibraryImpl : public MountLibrary {
         current_device_set.insert(std::string(devices[i]));
         found_disk = true;
         // Initiate disk property retrieval for each relevant device path.
-        GetDiskProperties(devices[i],
-                          &MountLibraryImpl::GetDiskPropertiesCallback,
-                          this);
+        libcros_proxy_->CallGetDiskProperties(devices[i],
+            &GetDiskPropertiesCallback, this);
       }
     } else if (error != MOUNT_METHOD_ERROR_NONE) {
       LOG(WARNING) << "Request mount info retrieval request failed with error: "
@@ -492,11 +672,9 @@ class MountLibraryImpl : public MountLibrary {
       return;
     MountLibraryEventType type = MOUNT_DEVICE_ADDED;
     switch (evt) {
-      case DISK_ADDED:
-      case DISK_CHANGED: {
-        GetDiskProperties(device_path,
-                          &MountLibraryImpl::GetDiskPropertiesCallback,
-                          this);
+      case DISK_ADDED: {
+        libcros_proxy_->CallGetDiskProperties(device_path,
+            &MountLibraryImpl::GetDiskPropertiesCallback, this);
         return;
       }
       case DISK_REMOVED: {
@@ -513,16 +691,30 @@ class MountLibraryImpl : public MountLibrary {
       }
       case DEVICE_ADDED: {
         type = MOUNT_DEVICE_ADDED;
+        system_path_prefixes_.insert(device_path);
         break;
       }
       case DEVICE_REMOVED: {
         type = MOUNT_DEVICE_REMOVED;
+        system_path_prefixes_.erase(device_path);
         break;
       }
       case DEVICE_SCANNED: {
         type = MOUNT_DEVICE_SCANNED;
         break;
       }
+      case FORMATTING_FINISHED: {
+        // FORMATTING_FINISHED actually returns file path instead of device
+        // path.
+        device_path = FilePathToDevicePath(device_path);
+        if (!device_path) {
+          LOG(ERROR) << "Error while handling disks metadata. Cannot find "
+                     << "device that is being formatted.";
+          return;
+        }
+        type = MOUNT_FORMATTING_FINISHED;
+        break;
+      }
       default: {
         return;
       }
@@ -532,7 +724,7 @@ class MountLibraryImpl : public MountLibrary {
 
   void Init() {
     // Getting the monitor status so that the daemon starts up.
-    mount_status_connection_ = MonitorAllMountEvents(
+    mount_status_connection_ = libcros_proxy_->MonitorCrosDisks(
         &MonitorMountEventsHandler, &MountCompletedHandler, this);
   }
 
@@ -563,9 +755,38 @@ class MountLibraryImpl : public MountLibrary {
                                              mount_info));
   }
 
+  const char* FilePathToDevicePath(const char* file_path) {
+    int failed = (file_path[0] == '!') ? 1 : 0;
+    for (MountLibrary::DiskMap::iterator it = disks_.begin();
+         it != disks_.end(); ++it) {
+      if (it->second->file_path().compare(file_path + failed) == 0) {
+        if (failed) {
+          return (std::string("!") + it->second->device_path()).c_str();
+        } else {
+          return it->second->device_path().c_str();
+        }
+      }
+    }
+    return NULL;
+  }
+
+  const std::string& FindSystemPathPrefix(const std::string& system_path) {
+    if (system_path.empty())
+      return EmptyString();
+    for (SystemPathPrefixSet::const_iterator it = system_path_prefixes_.begin();
+         it != system_path_prefixes_.end();
+         ++it) {
+      if (system_path.find(*it, 0) == 0)
+        return *it;
+    }
+    return EmptyString();
+  }
+
   // Mount event change observers.
   ObserverList<Observer> observers_;
 
+  scoped_ptr<MountLibcrosProxy> libcros_proxy_;
+
   // A reference to the  mount api, to allow callbacks when the mount
   // status changes.
   MountEventConnection mount_status_connection_;
@@ -575,6 +796,14 @@ class MountLibraryImpl : public MountLibrary {
 
   MountLibrary::MountPointMap mount_points_;
 
+  typedef std::set<std::string> SystemPathPrefixSet;
+  SystemPathPrefixSet system_path_prefixes_;
+
+  // Set of devices that are supposed to be formated, but are currently waiting
+  // to be unmounted. When device is in this map, the formatting process HAVEN'T
+  // started yet.
+  PathMap formatting_pending_;
+
   DISALLOW_COPY_AND_ASSIGN(MountLibraryImpl);
 };
 
@@ -594,10 +823,11 @@ class MountLibraryStubImpl : public MountLibrary {
   virtual void MountPath(const char* source_path, MountType type,
                          const MountPathOptions& options) OVERRIDE {}
   virtual void UnmountPath(const char* mount_path) OVERRIDE {}
+  virtual void FormatUnmountedDevice(const char* device_path) OVERRIDE {}
+  virtual void FormatMountedDevice(const char* mount_path) OVERRIDE {}
   virtual void UnmountDeviceRecursive(const char* device_path,
       UnmountDeviceRecursiveCallbackType callback, void* user_data)
       OVERRIDE {}
-
  private:
   // The list of disks found.
   DiskMap disks_;
