@@ -24,9 +24,44 @@ typedef enum MountLibraryEventType {
   MOUNT_DISK_UNMOUNTED,
   MOUNT_DEVICE_ADDED,
   MOUNT_DEVICE_REMOVED,
-  MOUNT_DEVICE_SCANNED
+  MOUNT_DEVICE_SCANNED,
+  MOUNT_FORMATTING_STARTED,
+  MOUNT_FORMATTING_FINISHED
 } MountLibraryEventType;
 
+typedef enum MountCondition {
+  MOUNT_CONDITION_NONE,
+  MOUNT_CONDITION_UNKNOWN_FILESYSTEM,
+  MOUNT_CONDITION_UNSUPPORTED_FILESYSTEM
+} MountCondition;
+
+class MountLibcrosProxy {
+ public:
+  virtual ~MountLibcrosProxy() {}
+  virtual void CallMountPath(const char* source_path,
+                             MountType type,
+                             const MountPathOptions& options,
+                             MountCompletedMonitor callback,
+                             void* object) = 0;
+  virtual void CallUnmountPath(const char* path,
+                               UnmountRequestCallback callback,
+                               void* object) = 0;
+  virtual void CallRequestMountInfo(RequestMountInfoCallback callback,
+                                    void* object) = 0;
+  virtual void CallFormatDevice(const char* device_path,
+                                const char* filesystem,
+                                FormatRequestCallback callback,
+                                void* object) = 0;
+  virtual void CallGetDiskProperties(const char* device_path,
+                                     GetDiskPropertiesCallback callback,
+                                     void* object) = 0;
+  virtual MountEventConnection MonitorCrosDisks(MountEventMonitor monitor,
+      MountCompletedMonitor mount_complete_monitor,
+      void* object) = 0;
+  virtual void DisconnectCrosDisksMonitorIfSet(MountEventConnection connection)
+      = 0;
+};
+
 // This class handles the interaction with the ChromeOS mount library APIs.
 // Classes can add themselves as observers. Users can get an instance of this
 // library class like this: chromeos::CrosLibrary::Get()->GetMountLibrary()
@@ -46,6 +81,7 @@ class MountLibrary {
          const std::string& device_label,
          const std::string& drive_label,
          const std::string& parent_path,
+         const std::string& system_path_prefix,
          DeviceType device_type,
          uint64 total_size,
          bool is_parent,
@@ -68,6 +104,10 @@ class MountLibrary {
     const std::string& drive_label() const { return drive_label_; }
     // Parents device path. If device has no parent, then empty string.
     const std::string& parent_path() const { return parent_path_; }
+    // Path of the system device this device's block is a part of.
+    const std::string& system_path_prefix() const {
+      return system_path_prefix_;
+    }
     // Device type.
     DeviceType device_type() const { return device_type_; }
     // Total size of the device.
@@ -92,6 +132,7 @@ class MountLibrary {
     std::string device_label_;
     std::string drive_label_;
     std::string parent_path_;
+    std::string system_path_prefix_;
     DeviceType device_type_;
     uint64 total_size_;
     bool is_parent_;
@@ -100,16 +141,20 @@ class MountLibrary {
     bool on_boot_device_;
   };
   typedef std::map<std::string, Disk*> DiskMap;
+  typedef std::map<std::string, std::string> PathMap;
 
   struct MountPointInfo {
     std::string source_path;
     std::string mount_path;
     MountType mount_type;
+    MountCondition mount_condition;
 
-    MountPointInfo(const char* source, const char* mount, const MountType type)
+    MountPointInfo(const char* source, const char* mount, const MountType type,
+        MountCondition condition)
         : source_path(source ? source : ""),
           mount_path(mount ? mount : ""),
-          mount_type(type) {
+          mount_type(type),
+          mount_condition(condition) {
     }
   };
 
@@ -125,7 +170,7 @@ class MountLibrary {
     virtual void DiskChanged(MountLibraryEventType event,
                              const Disk* disk) = 0;
     virtual void DeviceChanged(MountLibraryEventType event,
-                               const std::string& device_path ) = 0;
+                               const std::string& device_path) = 0;
     virtual void MountCompleted(MountEvent event_type,
                                 MountError error_code,
                                 const MountPointInfo& mount_info) = 0;
@@ -144,6 +189,14 @@ class MountLibrary {
   // |path| is device's mount path.
   virtual void UnmountPath(const char* path) = 0;
 
+  // Formats device given its file path.
+  // Example: file_path: /dev/sdb1
+  virtual void FormatUnmountedDevice(const char* file_path) = 0;
+
+  // Formats Device given its mount path. Unmount's the device
+  // Example: mount_path: /media/VOLUME_LABEL
+  virtual void FormatMountedDevice(const char* mount_path) = 0;
+
   // Unmounts device_poath and all of its known children.
   virtual void UnmountDeviceRecursive(const char* device_path,
       UnmountDeviceRecursiveCallbackType callback, void* user_data) = 0;
@@ -151,6 +204,10 @@ class MountLibrary {
   // Helper functions for parameter conversions.
   static std::string MountTypeToString(MountType type);
   static MountType MountTypeFromString(const std::string& type_str);
+  static std::string MountConditionToString(MountCondition type);
+
+  // Used in testing. Enables mocking libcros.
+  virtual void SetLibcrosProxy(MountLibcrosProxy* proxy) {}
 
   // Factory function, creates a new instance and returns ownership.
   // For normal usage, access the singleton via CrosLibrary::Get().
