@@ -5,42 +5,53 @@
 #include "chrome/browser/chromeos/extensions/file_browser_event_router.h"
 
 #include "base/json/json_writer.h"
+#include "base/message_loop.h"
 #include "base/stl_util.h"
-#include "base/utf_string_conversions.h"
 #include "base/values.h"
 #include "chrome/browser/chromeos/cros/cros_library.h"
+#include "chrome/browser/chromeos/extensions/file_browser_notifications.h"
 #include "chrome/browser/chromeos/login/user_manager.h"
-#include "chrome/browser/chromeos/notifications/system_notification.h"
 #include "chrome/browser/extensions/extension_event_names.h"
 #include "chrome/browser/extensions/extension_event_router.h"
 #include "chrome/browser/extensions/extension_service.h"
 #include "chrome/browser/extensions/file_manager_util.h"
 #include "chrome/browser/profiles/profile.h"
-#include "content/browser/browser_thread.h"
 #include "grit/generated_resources.h"
-#include "grit/theme_resources.h"
-#include "ui/base/l10n/l10n_util.h"
 #include "webkit/fileapi/file_system_types.h"
 #include "webkit/fileapi/file_system_util.h"
 
-const char kDiskAddedEventType[] = "added";
-const char kDiskRemovedEventType[] = "removed";
+namespace {
+  const char kDiskAddedEventType[] = "added";
+  const char kDiskRemovedEventType[] = "removed";
 
-const char kPathChanged[] = "changed";
-const char kPathWatchError[] = "error";
+  const char kPathChanged[] = "changed";
+  const char kPathWatchError[] = "error";
 
-const char* DeviceTypeToString(chromeos::DeviceType type) {
-  switch (type) {
-    case chromeos::FLASH:
-      return "flash";
-    case chromeos::HDD:
-      return "hdd";
-    case chromeos::OPTICAL:
-      return "optical";
-    default:
-      break;
+  const char* DeviceTypeToString(chromeos::DeviceType type) {
+    switch (type) {
+      case chromeos::FLASH:
+        return "flash";
+      case chromeos::HDD:
+        return "hdd";
+      case chromeos::OPTICAL:
+        return "optical";
+      default:
+        break;
+    }
+    return "undefined";
+  }
+
+  DictionaryValue* DiskToDictionaryValue(
+      const chromeos::MountLibrary::Disk* disk) {
+    DictionaryValue* result = new DictionaryValue();
+    result->SetString("mountPath", disk->mount_path());
+    result->SetString("devicePath", disk->device_path());
+    result->SetString("label", disk->device_label());
+    result->SetString("deviceType", DeviceTypeToString(disk->device_type()));
+    result->SetInteger("totalSizeKB", disk->total_size() / 1024);
+    result->SetBoolean("readOnly", disk->is_read_only());
+    return result;
   }
-  return "undefined";
 }
 
 const char* MountErrorToString(chromeos::MountError error) {
@@ -65,21 +76,10 @@ const char* MountErrorToString(chromeos::MountError error) {
   return "";
 }
 
-DictionaryValue* DiskToDictionaryValue(
-    const chromeos::MountLibrary::Disk* disk) {
-  DictionaryValue* result = new DictionaryValue();
-  result->SetString("mountPath", disk->mount_path());
-  result->SetString("devicePath", disk->device_path());
-  result->SetString("label", disk->device_label());
-  result->SetString("deviceType", DeviceTypeToString(disk->device_type()));
-  result->SetInteger("totalSizeKB", disk->total_size() / 1024);
-  result->SetBoolean("readOnly", disk->is_read_only());
-  return result;
-}
-
 ExtensionFileBrowserEventRouter::ExtensionFileBrowserEventRouter(
     Profile* profile)
     : delegate_(new ExtensionFileBrowserEventRouter::FileWatcherDelegate(this)),
+      notifications_(new FileBrowserNotifications(profile)),
       profile_(profile) {
 }
 
@@ -91,12 +91,12 @@ ExtensionFileBrowserEventRouter::~ExtensionFileBrowserEventRouter() {
     NOTREACHED();
     return;
   }
+  profile_ = NULL;
   if (!chromeos::CrosLibrary::Get()->EnsureLoaded())
     return;
   chromeos::MountLibrary* lib =
       chromeos::CrosLibrary::Get()->GetMountLibrary();
   lib->RemoveObserver(this);
-  profile_ = NULL;
 }
 
 void ExtensionFileBrowserEventRouter::ObserveFileSystemEvents() {
@@ -171,48 +171,43 @@ void ExtensionFileBrowserEventRouter::DeviceChanged(
     OnDeviceRemoved(device_path);
   } else if (event == chromeos::MOUNT_DEVICE_SCANNED) {
     OnDeviceScanned(device_path);
+  } else if (event == chromeos::MOUNT_FORMATTING_STARTED) {
+    if (device_path[0] == '!') {
+      OnFormattingStarted(device_path.substr(1), false);
+    } else {
+      OnFormattingStarted(device_path, true);
+    }
+  } else if (event == chromeos::MOUNT_FORMATTING_FINISHED) {
+    if (device_path[0] == '!') {
+      OnFormattingFinished(device_path.substr(1), false);
+    } else {
+      OnFormattingFinished(device_path, true);
+    }
   }
 }
+
 void ExtensionFileBrowserEventRouter::MountCompleted(
     chromeos::MountLibrary::MountEvent event_type,
     chromeos::MountError error_code,
     const chromeos::MountLibrary::MountPointInfo& mount_info) {
+  DispatchMountCompletedEvent(event_type, error_code, mount_info);
+
   if (mount_info.mount_type == chromeos::MOUNT_TYPE_DEVICE &&
       event_type == chromeos::MountLibrary::MOUNTING) {
     chromeos::MountLibrary* mount_lib =
         chromeos::CrosLibrary::Get()->GetMountLibrary();
+    if (mount_lib->disks().find(mount_info.source_path) ==
+        mount_lib->disks().end()) {
+      return;
+    }
     chromeos::MountLibrary::Disk* disk =
         mount_lib->disks().find(mount_info.source_path)->second;
 
-    std::string notification_sys_path =
-        disk->system_path().substr(0, disk->system_path().rfind("/block"));
-
-    if (!error_code) {
-      HideDeviceNotification(disk->system_path());
-    } else {
-      HideDeviceNotification(disk->system_path());
-      if (!disk->drive_label().empty()) {
-        ShowDeviceNotification(notification_sys_path,
-            IDR_PAGEINFO_INFO,
-            // TODO(tbarzic): Find more suitable message.
-            l10n_util::GetStringFUTF16(
-                IDS_FILE_BROWSER_ARCHIVE_MOUNT_FAILED,
-                ASCIIToUTF16(disk->drive_label()),
-                ASCIIToUTF16(MountErrorToString(error_code))));
-      } else {
-        ShowDeviceNotification(notification_sys_path,
-            IDR_PAGEINFO_INFO,
-            // TODO(tbarzic): Find more suitable message.
-            l10n_util::GetStringFUTF16(
-                IDS_FILE_BROWSER_ARCHIVE_MOUNT_FAILED,
-                l10n_util::GetStringUTF16(
-                    IDS_FILE_BROWSER_DEVICE_TYPE_UNDEFINED),
-                ASCIIToUTF16(MountErrorToString(error_code))));
-      }
-    }
+     notifications_->ManageNotificationsOnMountCompleted(
+        disk->system_path_prefix(), disk->drive_label(), disk->is_parent(),
+        error_code == chromeos::MOUNT_ERROR_NONE,
+        error_code == chromeos::MOUNT_ERROR_UNSUPORTED_FILESYSTEM);
   }
-
-  DispatchMountCompletedEvent(event_type, error_code, mount_info);
 }
 
 void ExtensionFileBrowserEventRouter::HandleFileWatchNotification(
@@ -315,8 +310,11 @@ void ExtensionFileBrowserEventRouter::DispatchMountCompletedEvent(
 
   FilePath relative_mount_path;
   bool relative_mount_path_set = false;
-  // If there were no error, add mountPath to the event.
-  if (error_code == chromeos::MOUNT_ERROR_NONE) {
+
+  // If there were no error or some special conditions occured, add mountPath
+  // to the event.
+  if (error_code == chromeos::MOUNT_ERROR_NONE ||
+      mount_info.mount_condition) {
     // Convert mount point path to relative path with the external file system
     // exposed within File API.
     if (FileManagerUtil::ConvertFileToRelativeFileSystemPath(profile_,
@@ -336,8 +334,9 @@ void ExtensionFileBrowserEventRouter::DispatchMountCompletedEvent(
 
   if (relative_mount_path_set &&
       mount_info.mount_type == chromeos::MOUNT_TYPE_DEVICE &&
+      !mount_info.mount_condition &&
       event == chromeos::MountLibrary::MOUNTING) {
-    FileManagerUtil::ShowFullTabUrl(profile_, relative_mount_path);
+    FileManagerUtil::ShowFullTabUrl(profile_, FilePath(mount_info.mount_path));
   }
 }
 
@@ -364,7 +363,6 @@ void ExtensionFileBrowserEventRouter::OnDiskAdded(
 void ExtensionFileBrowserEventRouter::OnDiskRemoved(
     const chromeos::MountLibrary::Disk* disk) {
   VLOG(1) << "Disk removed: " << disk->device_path();
-  HideDeviceNotification(disk->system_path());
 
   if (!disk->mount_path().empty()) {
     chromeos::MountLibrary* lib =
@@ -377,15 +375,21 @@ void ExtensionFileBrowserEventRouter::OnDiskRemoved(
 void ExtensionFileBrowserEventRouter::OnDeviceAdded(
     const std::string& device_path) {
   VLOG(1) << "Device added : " << device_path;
-  // TODO(zelidrag): Find better icon here.
-  ShowDeviceNotification(device_path, IDR_PAGEINFO_INFO,
-      l10n_util::GetStringUTF16(IDS_REMOVABLE_DEVICE_SCANNING_MESSAGE));
 
+  notifications_->RegisterDevice(device_path);
+  notifications_->ShowNotificationDelayed(FileBrowserNotifications::DEVICE,
+                                          device_path,
+                                          4000);
 }
 
 void ExtensionFileBrowserEventRouter::OnDeviceRemoved(
-    const std::string& system_path) {
-  HideDeviceNotification(system_path);
+    const std::string& device_path) {
+  VLOG(1) << "Device removed : " << device_path;
+  notifications_->HideNotification(FileBrowserNotifications::DEVICE,
+                                   device_path);
+  notifications_->HideNotification(FileBrowserNotifications::DEVICE_FAIL,
+                                   device_path);
+  notifications_->UnregisterDevice(device_path);
 }
 
 void ExtensionFileBrowserEventRouter::OnDeviceScanned(
@@ -393,53 +397,39 @@ void ExtensionFileBrowserEventRouter::OnDeviceScanned(
   VLOG(1) << "Device scanned : " << device_path;
 }
 
-void ExtensionFileBrowserEventRouter::ShowDeviceNotification(
-    const std::string& system_path, int icon_resource_id,
-    const string16& message) {
-  NotificationMap::iterator iter = FindNotificationForPath(system_path);
-  std::string mount_path;
-  if (iter != notifications_.end()) {
-    iter->second->Show(message, false, false);
+void ExtensionFileBrowserEventRouter::OnFormattingStarted(
+    const std::string& device_path, bool success) {
+  if (success) {
+    notifications_->ShowNotification(FileBrowserNotifications::FORMAT_START,
+                                     device_path);
   } else {
-    if (!profile_) {
-      NOTREACHED();
-      return;
-    }
-    chromeos::SystemNotification* notification =
-        new chromeos::SystemNotification(
-            profile_,
-            system_path,
-            icon_resource_id,
-            l10n_util::GetStringUTF16(IDS_REMOVABLE_DEVICE_DETECTION_TITLE));
-    notifications_.insert(NotificationMap::value_type(system_path,
-        linked_ptr<chromeos::SystemNotification>(notification)));
-    notification->Show(message, false, false);
+    notifications_->ShowNotification(
+        FileBrowserNotifications::FORMAT_START_FAIL, device_path);
   }
 }
 
-void ExtensionFileBrowserEventRouter::HideDeviceNotification(
-    const std::string& system_path) {
-  NotificationMap::iterator iter = FindNotificationForPath(system_path);
-  if (iter != notifications_.end()) {
-    iter->second->Hide();
-    notifications_.erase(iter);
-  }
-}
+void ExtensionFileBrowserEventRouter::OnFormattingFinished(
+    const std::string& device_path, bool success) {
+  if (success) {
+    notifications_->HideNotification(FileBrowserNotifications::FORMAT_START,
+                                     device_path);
+    notifications_->ShowNotification(FileBrowserNotifications::FORMAT_SUCCESS,
+                                     device_path);
+    // Hide it after a couple of seconds.
+    notifications_->HideNotificationDelayed(
+        FileBrowserNotifications::FORMAT_SUCCESS, device_path, 4000);
 
-ExtensionFileBrowserEventRouter::NotificationMap::iterator
-    ExtensionFileBrowserEventRouter::FindNotificationForPath(
-        const std::string& system_path) {
-  for (NotificationMap::iterator iter = notifications_.begin();
-       iter != notifications_.end();
-       ++iter) {
-    const std::string& notification_device_path = iter->first;
-    // Doing a sub string match so that we find if this new one is a subdevice
-    // of another already inserted device.
-    if (StartsWithASCII(system_path, notification_device_path, true)) {
-      return iter;
-    }
+    chromeos::MountLibrary* lib =
+        chromeos::CrosLibrary::Get()->GetMountLibrary();
+    lib->MountPath(device_path.c_str(),
+                   chromeos::MOUNT_TYPE_DEVICE,
+                   chromeos::MountPathOptions());  // Unused.
+  } else {
+    notifications_->HideNotification(FileBrowserNotifications::FORMAT_START,
+                                     device_path);
+    notifications_->ShowNotification(FileBrowserNotifications::FORMAT_FAIL,
+                                     device_path);
   }
-  return notifications_.end();
 }
 
 // ExtensionFileBrowserEventRouter::WatcherDelegate methods.
