@@ -28,6 +28,7 @@ class ImageButton;
 class ImageView;
 class MenuDelegate;
 class MenuItemView;
+class MenuRunner;
 }  // namespace views
 
 namespace chromeos {
@@ -60,7 +61,9 @@ class BrowserView : public ::BrowserView,
   virtual void FocusChromeOSStatus() OVERRIDE;
   virtual views::LayoutManager* CreateLayoutManager() const OVERRIDE;
   virtual void ChildPreferredSizeChanged(View* child) OVERRIDE;
-  virtual bool GetSavedWindowBounds(gfx::Rect* bounds) const OVERRIDE;
+  virtual bool GetSavedWindowPlacement(
+      gfx::Rect* bounds,
+      ui::WindowShowState* show_state) const OVERRIDE;
   virtual void Cut() OVERRIDE;
   virtual void Copy() OVERRIDE;
   virtual void Paste() OVERRIDE;
@@ -129,7 +132,7 @@ class BrowserView : public ::BrowserView,
   LayoutModeButton* layout_mode_button_;
 
   // System menu.
-  scoped_ptr<views::MenuItemView> system_menu_;
+  scoped_ptr<views::MenuRunner> system_menu_runner_;
   scoped_ptr<views::MenuDelegate> system_menu_delegate_;
 
   // Focused native widget before wrench menu shows up. We need this to properly
