@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -32,11 +32,6 @@ class AuthResponseHandler {
   // the object lifetime.
   virtual URLFetcher* Handle(const std::string& to_process,
                              URLFetcher::Delegate* catcher) = 0;
-
-  // The URLs for different calls in the Google Accounts programmatic login API.
-  static const char kClientLoginUrl[];
-  static const char kIssueAuthTokenUrl[];
-  static const char kTokenAuthUrl[];
 };
 
 }  // namespace chromeos
