@@ -26,7 +26,7 @@
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/net/gaia/gaia_auth_fetcher_unittest.h"
 #include "chrome/common/net/gaia/gaia_urls.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/testing_profile.h"
 #include "content/browser/browser_thread.h"
 #include "content/common/url_fetcher.h"
 #include "googleurl/src/gurl.h"
@@ -50,7 +50,7 @@ using ::testing::_;
 
 namespace chromeos {
 
-class GoogleAuthenticatorTest : public ::testing::Test {
+class GoogleAuthenticatorTest : public testing::Test {
  public:
   GoogleAuthenticatorTest()
       : message_loop_ui_(MessageLoop::TYPE_UI),
@@ -162,6 +162,10 @@ class GoogleAuthenticatorTest : public ::testing::Test {
   std::string username_;
   std::string password_;
   GaiaAuthConsumer::ClientLoginResult result_;
+
+  // Initializes / shuts down a stub CrosLibrary.
+  chromeos::ScopedStubCrosEnabler stub_cros_enabler_;
+
   // Mocks, destroyed by CrosLibrary class.
   MockCryptohomeLibrary* mock_library_;
   MockLibraryLoader* loader_;
@@ -565,7 +569,6 @@ TEST_F(GoogleAuthenticatorTest, FullLogin) {
   TestingProfile profile;
 
   MockFactory<MockFetcher> factory;
-  URLFetcher::set_factory(&factory);
 
   scoped_refptr<GoogleAuthenticator> auth(new GoogleAuthenticator(&consumer));
   EXPECT_CALL(*user_manager_.get(), IsKnownUser(username_))
@@ -575,7 +578,6 @@ TEST_F(GoogleAuthenticatorTest, FullLogin) {
   auth->AuthenticateToLogin(
       &profile, username_, password_, std::string(), std::string());
 
-  URLFetcher::set_factory(NULL);
   message_loop_ui_.RunAllPending();
 }
 
@@ -602,7 +604,6 @@ TEST_F(GoogleAuthenticatorTest, FullHostedLoginFailure) {
   TestingProfile profile;
 
   MockFactory<HostedFetcher> factory_invalid;
-  URLFetcher::set_factory(&factory_invalid);
 
   scoped_refptr<GoogleAuthenticator> auth(new GoogleAuthenticator(&consumer));
   auth->set_user_manager(user_manager_.get());
@@ -619,7 +620,6 @@ TEST_F(GoogleAuthenticatorTest, FullHostedLoginFailure) {
 
   // Run the UI thread until we exit it gracefully.
   message_loop_ui_.Run();
-  URLFetcher::set_factory(NULL);
 }
 
 TEST_F(GoogleAuthenticatorTest, CancelLogin) {
@@ -650,7 +650,6 @@ TEST_F(GoogleAuthenticatorTest, CancelLogin) {
   // and then come back on the UI thread after a small delay.  They expect to
   // be canceled before they come back, and the test will fail if they are not.
   MockFactory<ExpectCanceledFetcher> factory;
-  URLFetcher::set_factory(&factory);
 
   scoped_refptr<GoogleAuthenticator> auth(new GoogleAuthenticator(&consumer));
   // For when |auth| tries to load the localaccount file.
@@ -665,8 +664,6 @@ TEST_F(GoogleAuthenticatorTest, CancelLogin) {
   // Post a task to cancel the login attempt.
   CancelLogin(auth.get());
 
-  URLFetcher::set_factory(NULL);
-
   // Run the UI thread until we exit it gracefully.
   message_loop_ui_.Run();
 }
@@ -699,7 +696,6 @@ TEST_F(GoogleAuthenticatorTest, CancelLoginAlreadyGotLocalaccount) {
   // and then come back on the UI thread after a small delay.  They expect to
   // be canceled before they come back, and the test will fail if they are not.
   MockFactory<ExpectCanceledFetcher> factory;
-  URLFetcher::set_factory(&factory);
 
   scoped_refptr<GoogleAuthenticator> auth(new GoogleAuthenticator(&consumer));
   // This time, instead of allowing |auth| to go get the localaccount file
@@ -715,8 +711,6 @@ TEST_F(GoogleAuthenticatorTest, CancelLoginAlreadyGotLocalaccount) {
   // Post a task to cancel the login attempt.
   CancelLogin(auth.get());
 
-  URLFetcher::set_factory(NULL);
-
   // Run the UI thread until we exit it gracefully.
   message_loop_ui_.Run();
 }
