@@ -83,8 +83,8 @@ class MockAuthenticator : public Authenticator {
 
 class MockLoginUtils : public LoginUtils {
  public:
-  explicit MockLoginUtils(const std::string& expected_username,
-                          const std::string& expected_password);
+  MockLoginUtils(const std::string& expected_username,
+                 const std::string& expected_password);
   virtual ~MockLoginUtils();
 
   virtual bool ShouldWaitForWifi();
@@ -97,6 +97,8 @@ class MockLoginUtils : public LoginUtils {
                               bool has_cookies,
                               Delegate* delegate);
 
+  virtual void DelegateDeleted(Delegate* delegate);
+
   virtual void CompleteOffTheRecordLogin(const GURL& start_url) {}
 
   virtual void SetFirstLoginPrefs(PrefService* prefs) {}
