@@ -10,9 +10,12 @@
 #include "chrome/browser/chromeos/login/login_utils.h"
 #include "chrome/browser/chromeos/login/screen_observer.h"
 #include "chrome/browser/chromeos/login/user_manager.h"
+#include "chrome/browser/chromeos/login/wizard_accessibility_helper.h"
 #include "chrome/common/chrome_notification_types.h"
 #include "content/common/notification_service.h"
+#include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
+#include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
 
 namespace chromeos {
@@ -61,10 +64,14 @@ void UserImageScreen::Show() {
       user_manager->GetUserDefaultImageIndex(logged_in_user);
   // The image must have been assigned by UserManager on new user login but
   // under some circumstances (i.e. the data is not written to Local State
-  // or the file was corrupt) -1 could still be returned.
-  if (selected_image_index == -1)
+  // or the file was corrupt) |kExternalImageIndex| could still be returned.
+  if (selected_image_index == UserManager::User::kInvalidImageIndex)
     selected_image_index = 0;
   actor_->SelectImage(selected_image_index);
+
+  WizardAccessibilityHelper::GetInstance()->MaybeSpeak(
+      l10n_util::GetStringUTF8(IDS_OPTIONS_CHANGE_PICTURE_DIALOG_TEXT).c_str(),
+      false, false);
 }
 
 void UserImageScreen::Hide() {
@@ -109,13 +116,14 @@ void UserImageScreen::OnPhotoTaken(const SkBitmap& image) {
   const UserManager::User& user = user_manager->logged_in_user();
   DCHECK(!user.email().empty());
 
-  user_manager->SetLoggedInUserImage(image);
+  user_manager->SetLoggedInUserImage(image,
+                                     UserManager::User::kExternalImageIndex);
   user_manager->SaveUserImage(user.email(), image);
   get_screen_observer()->OnExit(ScreenObserver::USER_IMAGE_SELECTED);
 
   UMA_HISTOGRAM_ENUMERATION("UserImage.FirstTimeChoice",
-                            kDefaultImagesCount,
-                            kDefaultImagesCount + 1);
+                            kHistogramImageFromCamera,
+                            kHistogramImagesCount);
 }
 
 void UserImageScreen::OnDefaultImageSelected(int index) {
@@ -129,7 +137,7 @@ void UserImageScreen::OnDefaultImageSelected(int index) {
 
   const SkBitmap* image = ResourceBundle::GetSharedInstance().GetBitmapNamed(
       kDefaultImageResources[index]);
-  user_manager->SetLoggedInUserImage(*image);
+  user_manager->SetLoggedInUserImage(*image, index);
   user_manager->SaveUserImagePath(
       user.email(),
       GetDefaultImagePath(static_cast<size_t>(index)));
@@ -137,7 +145,7 @@ void UserImageScreen::OnDefaultImageSelected(int index) {
 
   UMA_HISTOGRAM_ENUMERATION("UserImage.FirstTimeChoice",
                             index,
-                            kDefaultImagesCount + 1);
+                            kHistogramImagesCount);
 }
 
 void UserImageScreen::OnActorDestroyed(UserImageScreenActor* actor) {
@@ -159,4 +167,3 @@ void UserImageScreen::Observe(int type,
 }
 
 }  // namespace chromeos
-
