@@ -12,6 +12,7 @@
 #include "base/basictypes.h"
 #include "base/hash_tables.h"
 #include "base/memory/ref_counted.h"
+#include "base/observer_list.h"
 #include "base/synchronization/lock.h"
 #include "chrome/browser/chromeos/login/user_image_loader.h"
 #include "content/common/notification_observer.h"
@@ -43,6 +44,11 @@ class UserManager : public UserImageLoader::Delegate,
   // A class representing information about a previously logged in user.
   class User {
    public:
+    // Returned as default_image_index when user-selected file or photo
+    // is used as user image.
+    static const int kExternalImageIndex = -1;
+    static const int kInvalidImageIndex = -2;
+
     User();
     ~User();
 
@@ -82,8 +88,8 @@ class UserManager : public UserImageLoader::Delegate,
     // Cached flag of whether any users has same display name.
     bool is_displayname_unique_;
 
-    // Index of the default image the user has set. -1 if it's some other
-    // image.
+    // Index of the default image the user has set. |kExternalImageIndex|
+    // if it's some other image.
     int default_image_index_;
   };
 
@@ -121,14 +127,15 @@ class UserManager : public UserImageLoader::Delegate,
   // Returns the logged-in user.
   virtual const User& logged_in_user() const;
 
-  // Sets image for logged-in user and sends LOGIN_USER_IMAGE_CHANGED
-  // notification about the image changed via NotificationService.
-  void SetLoggedInUserImage(const SkBitmap& image);
+  // Sets image for logged-in user.
+  void SetLoggedInUserImage(const SkBitmap& image, int default_image_index);
 
   // Tries to load logged-in user image from disk and sets it for the user.
   void LoadLoggedInUserImage(const FilePath& path);
 
-  // Saves image to file and saves image path in local state preferences.
+  // Saves image to file, saves image path in local state preferences
+  // and sends LOGIN_USER_IMAGE_CHANGED notification about the image
+  // changed via NotificationService.
   void SaveUserImage(const std::string& username,
                      const SkBitmap& image);
 
@@ -136,12 +143,17 @@ class UserManager : public UserImageLoader::Delegate,
   void SaveUserOAuthStatus(const std::string& username,
                            OAuthTokenStatus oauth_token_status);
 
+  // Gets user's oauth token status in local state preferences.
+  OAuthTokenStatus GetUserOAuthStatus(const std::string& username);
+
   // Saves user image path for the user. Can be used to set default images.
+  // Sends LOGIN_USER_IMAGE_CHANGED notification about the image changed
+  // via NotificationService.
   void SaveUserImagePath(const std::string& username,
                          const std::string& image_path);
 
-  // Returns the index of user's default image or -1 if the image is not
-  // default.
+  // Returns the index of user's default image or |kInvalidImageIndex|
+  // if some error occurs (like Local State corruption).
   int GetUserDefaultImageIndex(const std::string& username);
 
   // chromeos::UserImageLoader::Delegate implementation.
@@ -165,9 +177,28 @@ class UserManager : public UserImageLoader::Delegate,
 
   bool user_is_logged_in() const { return user_is_logged_in_; }
 
+  void set_offline_login(bool value) { offline_login_ = value; }
+  bool offline_login() { return offline_login_; }
+
   // Returns true if we're logged in as a Guest.
   bool IsLoggedInAsGuest() const;
 
+  // Interface that observers of UserManager must implement in order
+  // to receive notification when local state preferences is changed
+  class Observer {
+   public:
+    // Called when the local state preferences is changed
+    virtual void LocalStateChanged(UserManager* user_manager) = 0;
+
+   protected:
+    virtual ~Observer() {}
+  };
+
+  void AddObserver(Observer* obs);
+  void RemoveObserver(Observer* obs);
+
+  void NotifyLocalStateChanged();
+
  protected:
   UserManager();
   virtual ~UserManager();
@@ -183,6 +214,12 @@ class UserManager : public UserImageLoader::Delegate,
   // setting in local state.
   void SetDefaultUserImage(const std::string& username);
 
+  // Sets image for user |username|.
+  void SetUserImage(const std::string& username,
+                    const SkBitmap& image,
+                    int default_image_index,
+                    bool save_image);
+
   // Loads user image from its file.
   scoped_refptr<UserImageLoader> image_loader_;
 
@@ -193,6 +230,9 @@ class UserManager : public UserImageLoader::Delegate,
   // The logged-in user.
   User logged_in_user_;
 
+  // Current user is logged in offline. Valid only for WebUI login flow.
+  bool offline_login_;
+
   // Cached flag of whether currently logged-in user is owner or not.
   // May be accessed on different threads, requires locking.
   bool current_user_is_owner_;
@@ -209,6 +249,8 @@ class UserManager : public UserImageLoader::Delegate,
 
   friend struct base::DefaultLazyInstanceTraits<UserManager>;
 
+  ObserverList<Observer> observer_list_;
+
   DISALLOW_COPY_AND_ASSIGN(UserManager);
 };
 
