@@ -4,12 +4,10 @@
 
 #include "chrome/browser/chromeos/proxy_cros_settings_provider.h"
 
-#include "base/command_line.h"
 #include "base/string_util.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/chromeos/cros_settings.h"
 #include "chrome/browser/ui/browser_list.h"
-#include "chrome/common/chrome_switches.h"
 
 namespace chromeos {
 
