@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -22,21 +22,24 @@ class SettingLevelBubbleView : public views::View {
  public:
   SettingLevelBubbleView();
 
-  // Initialize the view, setting the progress bar to the specified position.
-  // Ownership of |icon| remains with the caller (it's probably a shared
-  // instance from ResourceBundle).
-  void Init(SkBitmap* icon, int level_percent);
+  // Initialize the view, setting the progress bar to the specified level in the
+  // range [0.0, 100.0] and state.  Ownership of |icon| remains with the caller
+  // (it's probably a shared instance from ResourceBundle).
+  void Init(SkBitmap* icon, double level, bool enabled);
 
   // Change the icon that we're currently displaying.
   void SetIcon(SkBitmap* icon);
 
-  // Set the progress bar to the specified position and redraw it.
-  void Update(int level_percent);
+  // Set the progress bar to the specified level and redraw it.
+  void SetLevel(double level);
+
+  // Draw the progress bar in an enabled or disabled state.
+  void SetEnabled(bool enabled);
 
   // views::View implementation:
-  virtual void OnPaint(gfx::Canvas* canvas);
-  virtual void Layout();
-  virtual gfx::Size GetPreferredSize();
+  virtual void OnPaint(gfx::Canvas* canvas) OVERRIDE;
+  virtual void Layout() OVERRIDE;
+  virtual gfx::Size GetPreferredSize() OVERRIDE;
 
  private:
   views::ProgressBar* progress_bar_;
