@@ -31,7 +31,7 @@ class SimDialogDelegate : public HtmlDialogUIDelegate {
 
   // Overridden from HtmlDialogUI::Delegate:
   virtual bool IsDialogModal() const OVERRIDE;
-  virtual std::wstring GetDialogTitle() const OVERRIDE;
+  virtual string16 GetDialogTitle() const OVERRIDE;
   virtual GURL GetDialogContentURL() const OVERRIDE;
   virtual void GetWebUIMessageHandlers(
       std::vector<WebUIMessageHandler*>* handlers) const OVERRIDE;
