@@ -42,6 +42,7 @@ class UserCrosSettingsProvider : public CrosSettingsProvider {
   bool RequestTrustedDataRoamingEnabled(Task* callback);
   bool RequestTrustedShowUsersOnSignin(Task* callback);
   bool RequestTrustedOwner(Task* callback);
+  bool RequestTrustedReportingEnabled(Task* callback);
 
   // Reloads values from device settings.
   void Reload();
@@ -51,6 +52,7 @@ class UserCrosSettingsProvider : public CrosSettingsProvider {
   static bool cached_allow_new_user();
   static bool cached_data_roaming_enabled();
   static bool cached_show_users_on_signin();
+  static bool cached_reporting_enabled();
   static const base::ListValue* cached_whitelist();
   static std::string cached_owner();
 
