@@ -5,11 +5,19 @@
 #ifndef CHROME_BROWSER_CONTENT_SETTINGS_CONTENT_SETTINGS_UTILS_H_
 #define CHROME_BROWSER_CONTENT_SETTINGS_CONTENT_SETTINGS_UTILS_H_
 
+#include <string>
+#include <utility>
+
+#include "base/logging.h"
+#include "base/values.h"
+#include "chrome/browser/content_settings/content_settings_pattern.h"
 #include "chrome/common/content_settings.h"
 #include "chrome/common/content_settings_types.h"
 
 namespace content_settings {
 
+typedef std::pair<ContentSettingsPattern, ContentSettingsPattern> PatternPair;
+
 // Returns true if the |content_type| requires a resource identifier.
 bool RequiresResourceIdentifier(ContentSettingsType content_type);
 
@@ -18,6 +26,15 @@ bool RequiresResourceIdentifier(ContentSettingsType content_type);
 ContentSetting ClickToPlayFixup(ContentSettingsType content_type,
                                 ContentSetting setting);
 
+// Converts |Value| to |ContentSetting|.
+ContentSetting ValueToContentSetting(Value* value);
+
+PatternPair ParsePatternString(const std::string& pattern_str);
+
+std::string CreatePatternString(
+    const ContentSettingsPattern& item_pattern,
+    const ContentSettingsPattern& top_level_frame_pattern);
+
 }  // namespace content_settings
 
 #endif  // CHROME_BROWSER_CONTENT_SETTINGS_CONTENT_SETTINGS_UTILS_H_
