@@ -4,25 +4,31 @@
 
 #include "chrome/browser/content_settings/host_content_settings_map.h"
 
+#include <list>
+
 #include "base/command_line.h"
 #include "base/string_util.h"
 #include "base/utf_string_conversions.h"
+#include "chrome/browser/content_settings/content_settings_details.h"
 #include "chrome/browser/content_settings/content_settings_extension_provider.h"
+#include "chrome/browser/content_settings/content_settings_observable_provider.h"
 #include "chrome/browser/content_settings/content_settings_policy_provider.h"
 #include "chrome/browser/content_settings/content_settings_pref_provider.h"
 #include "chrome/browser/content_settings/content_settings_provider.h"
+#include "chrome/browser/content_settings/content_settings_utils.h"
 #include "chrome/browser/extensions/extension_service.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/prefs/scoped_user_pref_update.h"
 #include "chrome/browser/profiles/profile.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/url_constants.h"
 #include "content/browser/browser_thread.h"
 #include "content/browser/user_metrics.h"
+#include "content/common/content_switches.h"
 #include "content/common/notification_service.h"
 #include "content/common/notification_source.h"
-#include "content/common/notification_type.h"
 #include "googleurl/src/gurl.h"
 #include "net/base/net_errors.h"
 #include "net/base/net_util.h"
@@ -33,12 +39,14 @@ namespace {
 // Returns true if we should allow all content types for this URL.  This is
 // true for various internal objects like chrome:// URLs, so UI and other
 // things users think of as "not webpages" don't break.
-static bool ShouldAllowAllContent(const GURL& url) {
+static bool ShouldAllowAllContent(const GURL& url,
+                                  ContentSettingsType content_type) {
+  if (content_type == CONTENT_SETTINGS_TYPE_NOTIFICATIONS)
+    return false;
   return url.SchemeIs(chrome::kChromeDevToolsScheme) ||
          url.SchemeIs(chrome::kChromeInternalScheme) ||
          url.SchemeIs(chrome::kChromeUIScheme) ||
-         url.SchemeIs(chrome::kExtensionScheme) ||
-         url.SchemeIs(chrome::kUserScriptScheme);
+         url.SchemeIs(chrome::kExtensionScheme);
 }
 
 typedef linked_ptr<content_settings::DefaultProviderInterface>
@@ -53,38 +61,55 @@ typedef std::vector<ProviderPtr>::iterator ProviderIterator;
 typedef std::vector<ProviderPtr>::const_iterator ConstProviderIterator;
 
 typedef content_settings::ProviderInterface::Rules Rules;
-typedef content_settings::ProviderInterface::Rules::iterator
-    rules_iterator;
-typedef content_settings::ProviderInterface::Rules::const_iterator
-    const_rules_iterator;
+
+typedef std::pair<std::string, std::string> StringPair;
+
+const char* kProviderNames[] = {
+  "policy",
+  "extension",
+  "preference"
+};
+
+bool ContentTypeHasCompoundValue(ContentSettingsType type) {
+  // Values for content type CONTENT_SETTINGS_TYPE_AUTO_SELECT_CERTIFICATE are
+  // of type dictionary/map. Compound types like dictionaries can't be mapped to
+  // the type |ContentSetting|.
+  return type == CONTENT_SETTINGS_TYPE_AUTO_SELECT_CERTIFICATE;
+}
 
 }  // namespace
 
-HostContentSettingsMap::HostContentSettingsMap(Profile* profile)
-    : profile_(profile),
-      is_off_the_record_(profile_->IsOffTheRecord()),
+HostContentSettingsMap::HostContentSettingsMap(
+    PrefService* prefs,
+    ExtensionService* extension_service,
+    bool incognito)
+    : prefs_(prefs),
+      is_off_the_record_(incognito),
       updating_preferences_(false),
       block_third_party_cookies_(false),
       is_block_third_party_cookies_managed_(false) {
   // The order in which the default content settings providers are created is
   // critical, as providers that are further down in the list (i.e. added later)
   // override providers further up.
+  content_settings::PrefDefaultProvider* default_provider =
+      new content_settings::PrefDefaultProvider(prefs_, is_off_the_record_);
+  default_provider->AddObserver(this);
   default_content_settings_providers_.push_back(
-      make_linked_ptr(new content_settings::PrefDefaultProvider(profile)));
-  content_settings::DefaultProviderInterface* policy_default_provider =
-      new content_settings::PolicyDefaultProvider(profile);
+      make_linked_ptr(default_provider));
+
+  content_settings::PolicyDefaultProvider* policy_default_provider =
+      new content_settings::PolicyDefaultProvider(prefs_);
+  policy_default_provider->AddObserver(this);
   default_content_settings_providers_.push_back(
       make_linked_ptr(policy_default_provider));
 
-  PrefService* prefs = profile_->GetPrefs();
-
   // TODO(markusheintz): Discuss whether it is sensible to move migration code
   // to PrefContentSettingsProvider.
-  MigrateObsoleteCookiePref(prefs);
+  MigrateObsoleteCookiePref();
 
   // Read misc. global settings.
   block_third_party_cookies_ =
-      prefs->GetBoolean(prefs::kBlockThirdPartyCookies);
+      prefs_->GetBoolean(prefs::kBlockThirdPartyCookies);
   if (block_third_party_cookies_) {
     UserMetrics::RecordAction(
         UserMetricsAction("ThirdPartyCookieBlockingEnabled"));
@@ -93,30 +118,31 @@ HostContentSettingsMap::HostContentSettingsMap(Profile* profile)
         UserMetricsAction("ThirdPartyCookieBlockingDisabled"));
   }
   is_block_third_party_cookies_managed_ =
-      prefs->IsManagedPreference(prefs::kBlockThirdPartyCookies);
+      prefs_->IsManagedPreference(prefs::kBlockThirdPartyCookies);
 
   // User defined non default content settings are provided by the PrefProvider.
   // The order in which the content settings providers are created is critical,
   // as providers that are further up in the list (i.e. added earlier) override
   // providers further down.
-  content_settings_providers_.push_back(make_linked_ptr(
-      new content_settings::PolicyProvider(profile, policy_default_provider)));
-  ExtensionService* extension_service = profile->GetExtensionService();
+  content_settings::ObservableProvider* provider =
+      new content_settings::PolicyProvider(prefs_, policy_default_provider);
+  provider->AddObserver(this);
+  content_settings_providers_.push_back(make_linked_ptr(provider));
+
   if (extension_service) {
     // |extension_service| can be NULL in unit tests.
-    content_settings_providers_.push_back(make_linked_ptr(
-        new content_settings::ExtensionProvider(
-            profile,
-            extension_service->GetExtensionContentSettingsStore(),
-            is_off_the_record_)));
+    provider = new content_settings::ExtensionProvider(
+        extension_service->GetExtensionContentSettingsStore(),
+        is_off_the_record_);
+    provider->AddObserver(this);
+    content_settings_providers_.push_back(make_linked_ptr(provider));
   }
-  content_settings_providers_.push_back(
-      make_linked_ptr(new content_settings::PrefProvider(profile)));
+  provider = new content_settings::PrefProvider(prefs_, is_off_the_record_);
+  provider->AddObserver(this);
+  content_settings_providers_.push_back(make_linked_ptr(provider));
 
-  pref_change_registrar_.Init(prefs);
+  pref_change_registrar_.Init(prefs_);
   pref_change_registrar_.Add(prefs::kBlockThirdPartyCookies, this);
-  notification_registrar_.Add(this, NotificationType::PROFILE_DESTROYED,
-                              Source<Profile>(profile_));
 }
 
 // static
@@ -142,6 +168,7 @@ void HostContentSettingsMap::RegisterUserPrefs(PrefService* prefs) {
 
 ContentSetting HostContentSettingsMap::GetDefaultContentSetting(
     ContentSettingsType content_type) const {
+  DCHECK_NE(content_type, CONTENT_SETTINGS_TYPE_AUTO_SELECT_CERTIFICATE);
   ContentSetting setting = CONTENT_SETTING_DEFAULT;
   for (ConstDefaultProviderIterator provider =
            default_content_settings_providers_.begin();
@@ -160,39 +187,27 @@ ContentSetting HostContentSettingsMap::GetDefaultContentSetting(
 
 ContentSettings HostContentSettingsMap::GetDefaultContentSettings() const {
   ContentSettings output(CONTENT_SETTING_DEFAULT);
-  for (int i = 0; i < CONTENT_SETTINGS_NUM_TYPES; ++i)
-    output.settings[i] = GetDefaultContentSetting(ContentSettingsType(i));
+  for (int i = 0; i < CONTENT_SETTINGS_NUM_TYPES; ++i) {
+    if (!ContentTypeHasCompoundValue(ContentSettingsType(i)))
+      output.settings[i] = GetDefaultContentSetting(ContentSettingsType(i));
+  }
   return output;
 }
 
-ContentSetting HostContentSettingsMap::GetContentSetting(
-    const GURL& url,
-    ContentSettingsType content_type,
-    const std::string& resource_identifier) const {
-  DCHECK_NE(CONTENT_SETTINGS_TYPE_COOKIES, content_type);
-  return GetContentSettingInternal(url, content_type, resource_identifier);
-}
-
-ContentSetting HostContentSettingsMap::GetContentSettingInternal(
-    const GURL& url,
-    ContentSettingsType content_type,
-    const std::string& resource_identifier) const {
-  ContentSetting setting = GetNonDefaultContentSetting(url,
-                                                       content_type,
-                                                       resource_identifier);
-  if (setting == CONTENT_SETTING_DEFAULT)
-    return GetDefaultContentSetting(content_type);
-  return setting;
-}
-
 ContentSetting HostContentSettingsMap::GetCookieContentSetting(
     const GURL& url,
     const GURL& first_party_url,
     bool setting_cookie) const {
-  if (ShouldAllowAllContent(first_party_url))
+  if (ShouldAllowAllContent(first_party_url, CONTENT_SETTINGS_TYPE_COOKIES))
     return CONTENT_SETTING_ALLOW;
-  ContentSetting setting = CONTENT_SETTING_ALLOW;
-  if (BlockThirdPartyCookies()) {
+
+  // First get any host-specific settings.
+  ContentSetting setting = GetNonDefaultContentSetting(url,
+      first_party_url, CONTENT_SETTINGS_TYPE_COOKIES, "");
+
+  // If no explicit exception has been made and third-party cookies are blocked
+  // by default, apply that rule.
+  if (setting == CONTENT_SETTING_DEFAULT && BlockThirdPartyCookies()) {
     bool strict = CommandLine::ForCurrentProcess()->HasSwitch(
         switches::kBlockReadingThirdPartyCookies);
     net::StaticCookiePolicy policy(strict ?
@@ -200,25 +215,81 @@ ContentSetting HostContentSettingsMap::GetCookieContentSetting(
         net::StaticCookiePolicy::BLOCK_SETTING_THIRD_PARTY_COOKIES);
     int rv;
     if (setting_cookie)
-     rv = policy.CanSetCookie(url, first_party_url);
+      rv = policy.CanSetCookie(url, first_party_url);
     else
-     rv = policy.CanGetCookies(url, first_party_url);
+      rv = policy.CanGetCookies(url, first_party_url);
     DCHECK_NE(net::ERR_IO_PENDING, rv);
     if (rv != net::OK)
       setting = CONTENT_SETTING_BLOCK;
   }
 
-  if (setting == CONTENT_SETTING_ALLOW)
-    setting = GetContentSettingInternal(url, CONTENT_SETTINGS_TYPE_COOKIES, "");
+  // If no other policy has changed the setting, use the default.
+  if (setting == CONTENT_SETTING_DEFAULT)
+    setting = GetDefaultContentSetting(CONTENT_SETTINGS_TYPE_COOKIES);
 
   return setting;
 }
 
-ContentSetting HostContentSettingsMap::GetNonDefaultContentSetting(
-    const GURL& url,
+ContentSetting HostContentSettingsMap::GetContentSetting(
+    const GURL& primary_url,
+    const GURL& secondary_url,
     ContentSettingsType content_type,
     const std::string& resource_identifier) const {
-  if (ShouldAllowAllContent(url))
+  DCHECK_NE(CONTENT_SETTINGS_TYPE_COOKIES, content_type);
+  DCHECK_NE(content_settings::RequiresResourceIdentifier(content_type),
+            resource_identifier.empty());
+  return GetContentSettingInternal(
+      primary_url, secondary_url, content_type, resource_identifier);
+}
+
+Value* HostContentSettingsMap::GetContentSettingValue(
+    const GURL& primary_url,
+    const GURL& secondary_url,
+    ContentSettingsType content_type,
+    const std::string& resource_identifier) const {
+  // Check if the scheme of the requesting url is whitelisted.
+  if (ShouldAllowAllContent(secondary_url, content_type))
+    return Value::CreateIntegerValue(CONTENT_SETTING_ALLOW);
+
+  // First check if there are specific settings for the |primary_url| and
+  // |secondary_url|. The list of |content_settings_providers_| is ordered
+  // according to their priority.
+  for (ConstProviderIterator provider = content_settings_providers_.begin();
+       provider != content_settings_providers_.end();
+       ++provider) {
+    Value* value = (*provider)->GetContentSettingValue(
+        primary_url, secondary_url, content_type, resource_identifier);
+    if (value)
+      return value;
+  }
+
+  // If no specific settings were found for the |primary_url|, |secondary_url|
+  // pair, then the default value for the given |content_type| should be
+  // returned. For CONTENT_SETTINGS_TYPE_AUTO_SELECT_CERTIFICATE the default is
+  // 'no filter available'. That's why we return |NULL| for this content type.
+  if (content_type == CONTENT_SETTINGS_TYPE_AUTO_SELECT_CERTIFICATE)
+    return NULL;
+  return Value::CreateIntegerValue(GetDefaultContentSetting(content_type));
+}
+
+ContentSetting HostContentSettingsMap::GetContentSettingInternal(
+      const GURL& primary_url,
+      const GURL& secondary_url,
+      ContentSettingsType content_type,
+      const std::string& resource_identifier) const {
+  ContentSetting setting = GetNonDefaultContentSetting(
+      primary_url, secondary_url, content_type, resource_identifier);
+  if (setting == CONTENT_SETTING_DEFAULT)
+    return GetDefaultContentSetting(content_type);
+  return setting;
+}
+
+ContentSetting HostContentSettingsMap::GetNonDefaultContentSetting(
+      const GURL& primary_url,
+      const GURL& secondary_url,
+      ContentSettingsType content_type,
+      const std::string& resource_identifier) const {
+  if (ShouldAllowAllContent(secondary_url, content_type))
     return CONTENT_SETTING_ALLOW;
 
   // Iterate through the list of providers and break when the first non default
@@ -228,7 +299,7 @@ ContentSetting HostContentSettingsMap::GetNonDefaultContentSetting(
        provider != content_settings_providers_.end();
        ++provider) {
     provided_setting = (*provider)->GetContentSetting(
-        url, url, content_type, resource_identifier);
+        primary_url, secondary_url, content_type, resource_identifier);
     if (provided_setting != CONTENT_SETTING_DEFAULT)
       return provided_setting;
   }
@@ -236,16 +307,21 @@ ContentSetting HostContentSettingsMap::GetNonDefaultContentSetting(
 }
 
 ContentSettings HostContentSettingsMap::GetContentSettings(
-    const GURL& url) const {
-  ContentSettings output = GetNonDefaultContentSettings(url);
+      const GURL& primary_url,
+      const GURL& secondary_url) const {
+  ContentSettings output = GetNonDefaultContentSettings(
+      primary_url, secondary_url);
 
   // If we require a resource identifier, set the content settings to default,
-  // otherwise make the defaults explicit.
+  // otherwise make the defaults explicit. Values for content type
+  // CONTENT_SETTINGS_TYPE_AUTO_SELECT_CERTIFICATE can't be mapped to the type
+  // |ContentSetting|. So we ignore them here.
   for (int j = 0; j < CONTENT_SETTINGS_NUM_TYPES; ++j) {
     // A managed default content setting has the highest priority and hence
     // will overwrite any previously set value.
     if (output.settings[j] == CONTENT_SETTING_DEFAULT &&
-        j != CONTENT_SETTINGS_TYPE_PLUGINS) {
+        j != CONTENT_SETTINGS_TYPE_PLUGINS &&
+        !ContentTypeHasCompoundValue(ContentSettingsType(j))) {
       output.settings[j] = GetDefaultContentSetting(ContentSettingsType(j));
     }
   }
@@ -253,14 +329,17 @@ ContentSettings HostContentSettingsMap::GetContentSettings(
 }
 
 ContentSettings HostContentSettingsMap::GetNonDefaultContentSettings(
-    const GURL& url) const {
-  if (ShouldAllowAllContent(url))
-    return ContentSettings(CONTENT_SETTING_ALLOW);
-
+    const GURL& primary_url,
+    const GURL& secondary_url) const {
   ContentSettings output(CONTENT_SETTING_DEFAULT);
   for (int j = 0; j < CONTENT_SETTINGS_NUM_TYPES; ++j) {
-    output.settings[j] = GetNonDefaultContentSetting(
-        url, ContentSettingsType(j) , "");
+    if (!ContentTypeHasCompoundValue(ContentSettingsType(j))) {
+      output.settings[j] = GetNonDefaultContentSetting(
+          primary_url,
+          secondary_url,
+          ContentSettingsType(j),
+          "");
+    }
   }
   return output;
 }
@@ -269,41 +348,47 @@ void HostContentSettingsMap::GetSettingsForOneType(
     ContentSettingsType content_type,
     const std::string& resource_identifier,
     SettingsForOneType* settings) const {
+  DCHECK_NE(content_settings::RequiresResourceIdentifier(content_type),
+            resource_identifier.empty());
   DCHECK(settings);
+
   settings->clear();
-
-  // Collect content_settings::Rules for the given content_type and
-  // resource_identifier from the content settings providers.
-  Rules content_settings_rules;
-  for (ConstProviderIterator provider = content_settings_providers_.begin();
-       provider != content_settings_providers_.end();
-       ++provider) {
-    // TODO(markusheintz): Only the rules that are applied should be collected.
-    // Merge rules.
-    // TODO(markusheintz): GetAllContentSettingsRules should maybe not clear the
-    // passed vector in case rule sets are just unified.
+  for (size_t i = 0; i < content_settings_providers_.size(); ++i) {
+    // Get rules from the content settings provider.
     Rules rules;
-    (*provider)->GetAllContentSettingsRules(
+    content_settings_providers_[i]->GetAllContentSettingsRules(
         content_type, resource_identifier, &rules);
-    content_settings_rules.insert(content_settings_rules.end(),
-                                  rules.begin(),
-                                  rules.end());
-  }
 
-  // convert Rules to SettingsForOneType
-  for (const_rules_iterator rule_iterator =
-           content_settings_rules.begin();
-       rule_iterator != content_settings_rules.end();
-       ++rule_iterator) {
-    settings->push_back(std::make_pair(ContentSettingsPattern(
-        rule_iterator->requesting_url_pattern),
-        rule_iterator->content_setting));
+    // Sort rules according to their primary-secondary pattern string pairs
+    // using a map.
+    std::map<StringPair, PatternSettingSourceTuple> settings_map;
+    for (Rules::iterator rule = rules.begin();
+         rule != rules.end();
+         ++rule) {
+      StringPair sort_key(rule->primary_pattern.ToString(),
+                          rule->secondary_pattern.ToString());
+      settings_map[sort_key] = PatternSettingSourceTuple(
+          rule->primary_pattern,
+          rule->secondary_pattern,
+          rule->content_setting,
+          kProviderNames[i]);
+    }
+
+    // TODO(markusheintz): Only the rules that are applied should be added.
+    for (std::map<StringPair, PatternSettingSourceTuple>::iterator i(
+             settings_map.begin());
+         i != settings_map.end();
+         ++i) {
+      settings->push_back(i->second);
+    }
   }
 }
 
 void HostContentSettingsMap::SetDefaultContentSetting(
     ContentSettingsType content_type,
     ContentSetting setting) {
+  DCHECK_NE(content_type, CONTENT_SETTINGS_TYPE_AUTO_SELECT_CERTIFICATE);
+  DCHECK(IsSettingAllowedForType(setting, content_type));
   for (DefaultProviderIterator provider =
            default_content_settings_providers_.begin();
        provider != default_content_settings_providers_.end(); ++provider) {
@@ -312,30 +397,47 @@ void HostContentSettingsMap::SetDefaultContentSetting(
 }
 
 void HostContentSettingsMap::SetContentSetting(
-    const ContentSettingsPattern& pattern,
+    const ContentSettingsPattern& primary_pattern,
+    const ContentSettingsPattern& secondary_pattern,
     ContentSettingsType content_type,
     const std::string& resource_identifier,
     ContentSetting setting) {
+  DCHECK_NE(content_type, CONTENT_SETTINGS_TYPE_AUTO_SELECT_CERTIFICATE);
+  DCHECK(IsSettingAllowedForType(setting, content_type));
+  DCHECK_NE(content_settings::RequiresResourceIdentifier(content_type),
+            resource_identifier.empty());
   for (ProviderIterator provider = content_settings_providers_.begin();
        provider != content_settings_providers_.end();
        ++provider) {
     (*provider)->SetContentSetting(
-        pattern, pattern, content_type, resource_identifier, setting);
+        primary_pattern,
+        secondary_pattern,
+        content_type,
+        resource_identifier,
+        setting);
   }
 }
 
 void HostContentSettingsMap::AddExceptionForURL(
-    const GURL& url,
+    const GURL& primary_url,
+    const GURL& secondary_url,
     ContentSettingsType content_type,
     const std::string& resource_identifier,
     ContentSetting setting) {
+  // TODO(markusheintz): Until the UI supports pattern pairs, both urls must
+  // match.
+  DCHECK(primary_url == secondary_url);
+
   // Make sure there is no entry that would override the pattern we are about
   // to insert for exactly this URL.
-  SetContentSetting(ContentSettingsPattern::LegacyFromURLNoWildcard(url),
+  SetContentSetting(ContentSettingsPattern::FromURLNoWildcard(primary_url),
+                    ContentSettingsPattern::Wildcard(),
                     content_type,
                     resource_identifier,
                     CONTENT_SETTING_DEFAULT);
-  SetContentSetting(ContentSettingsPattern::LegacyFromURL(url),
+
+  SetContentSetting(ContentSettingsPattern::FromURL(primary_url),
+                    ContentSettingsPattern::Wildcard(),
                     content_type,
                     resource_identifier,
                     setting);
@@ -350,8 +452,40 @@ void HostContentSettingsMap::ClearSettingsForOneType(
   }
 }
 
+// static
+bool HostContentSettingsMap::IsSettingAllowedForType(
+    ContentSetting setting, ContentSettingsType content_type) {
+  // Intents content settings are hidden behind a switch for now.
+  if (content_type == CONTENT_SETTINGS_TYPE_INTENTS &&
+      !CommandLine::ForCurrentProcess()->HasSwitch(
+          switches::kEnableWebIntents))
+    return false;
+
+  // DEFAULT, ALLOW and BLOCK are always allowed.
+  if (setting == CONTENT_SETTING_DEFAULT ||
+      setting == CONTENT_SETTING_ALLOW ||
+      setting == CONTENT_SETTING_BLOCK) {
+    return true;
+  }
+  switch (content_type) {
+    case CONTENT_SETTINGS_TYPE_COOKIES:
+      return (setting == CONTENT_SETTING_SESSION_ONLY);
+    case CONTENT_SETTINGS_TYPE_PLUGINS:
+      return (setting == CONTENT_SETTING_ASK &&
+              CommandLine::ForCurrentProcess()->HasSwitch(
+                  switches::kEnableClickToPlay));
+    case CONTENT_SETTINGS_TYPE_GEOLOCATION:
+    case CONTENT_SETTINGS_TYPE_NOTIFICATIONS:
+    case CONTENT_SETTINGS_TYPE_INTENTS:
+      return (setting == CONTENT_SETTING_ASK);
+    default:
+      return false;
+  }
+}
+
 void HostContentSettingsMap::SetBlockThirdPartyCookies(bool block) {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  DCHECK(prefs_);
 
   // This setting may not be directly modified for OTR sessions.  Instead, it
   // is synced to the main profile's setting.
@@ -360,10 +494,9 @@ void HostContentSettingsMap::SetBlockThirdPartyCookies(bool block) {
     return;
   }
 
-  PrefService* prefs = profile_->GetPrefs();
   // If the preference block-third-party-cookies is managed then do not allow to
   // change it.
-  if (prefs->IsManagedPreference(prefs::kBlockThirdPartyCookies)) {
+  if (prefs_->IsManagedPreference(prefs::kBlockThirdPartyCookies)) {
     NOTREACHED();
     return;
   }
@@ -373,74 +506,53 @@ void HostContentSettingsMap::SetBlockThirdPartyCookies(bool block) {
     block_third_party_cookies_ = block;
   }
 
-  profile_->GetPrefs()->SetBoolean(prefs::kBlockThirdPartyCookies, block);
+  prefs_->SetBoolean(prefs::kBlockThirdPartyCookies, block);
 }
 
-void HostContentSettingsMap::ResetToDefaults() {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-
-  {
-    base::AutoLock auto_lock(lock_);
-    for (DefaultProviderIterator provider =
-             default_content_settings_providers_.begin();
-         provider != default_content_settings_providers_.end(); ++provider) {
-      (*provider)->ResetToDefaults();
-    }
-
-    for (ProviderIterator provider = content_settings_providers_.begin();
-         provider != content_settings_providers_.end();
-         ++provider) {
-      (*provider)->ResetToDefaults();
-    }
-
-    // Don't reset block third party cookies if they are managed.
-    if (!IsBlockThirdPartyCookiesManaged())
-      block_third_party_cookies_ = false;
-  }
-
-  if (!is_off_the_record_) {
-    PrefService* prefs = profile_->GetPrefs();
-    updating_preferences_ = true;
-    // If the block third party cookies preference is managed we still must
-    // clear it in order to restore the default value for later when the
-    // preference is not managed anymore.
-    prefs->ClearPref(prefs::kBlockThirdPartyCookies);
-    updating_preferences_ = false;
-  }
+void HostContentSettingsMap::OnContentSettingChanged(
+    ContentSettingsPattern primary_pattern,
+    ContentSettingsPattern secondary_pattern,
+    ContentSettingsType content_type,
+    std::string resource_identifier) {
+  const ContentSettingsDetails details(primary_pattern,
+                                       secondary_pattern,
+                                       content_type,
+                                       resource_identifier);
+  NotificationService::current()->Notify(
+      chrome::NOTIFICATION_CONTENT_SETTINGS_CHANGED,
+      Source<HostContentSettingsMap>(this),
+      Details<const ContentSettingsDetails>(&details));
 }
 
-void HostContentSettingsMap::Observe(NotificationType type,
+void HostContentSettingsMap::Observe(int type,
                                      const NotificationSource& source,
                                      const NotificationDetails& details) {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
 
-  if (type == NotificationType::PREF_CHANGED) {
-    DCHECK_EQ(profile_->GetPrefs(), Source<PrefService>(source).ptr());
+  if (type == chrome::NOTIFICATION_PREF_CHANGED) {
+    DCHECK_EQ(prefs_, Source<PrefService>(source).ptr());
     if (updating_preferences_)
       return;
 
     std::string* name = Details<std::string>(details).ptr();
     if (*name == prefs::kBlockThirdPartyCookies) {
       base::AutoLock auto_lock(lock_);
-      block_third_party_cookies_ = profile_->GetPrefs()->GetBoolean(
+      block_third_party_cookies_ = prefs_->GetBoolean(
           prefs::kBlockThirdPartyCookies);
       is_block_third_party_cookies_managed_ =
-          profile_->GetPrefs()->IsManagedPreference(
+          prefs_->IsManagedPreference(
               prefs::kBlockThirdPartyCookies);
     } else {
       NOTREACHED() << "Unexpected preference observed";
       return;
     }
-  } else if (type == NotificationType::PROFILE_DESTROYED) {
-    DCHECK_EQ(profile_, Source<Profile>(source).ptr());
-    UnregisterObservers();
   } else {
     NOTREACHED() << "Unexpected notification";
   }
 }
 
 HostContentSettingsMap::~HostContentSettingsMap() {
-  UnregisterObservers();
+  DCHECK(!prefs_);
 }
 
 bool HostContentSettingsMap::IsDefaultContentSettingManaged(
@@ -454,26 +566,33 @@ bool HostContentSettingsMap::IsDefaultContentSettingManaged(
   return false;
 }
 
-void HostContentSettingsMap::UnregisterObservers() {
+void HostContentSettingsMap::ShutdownOnUIThread() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  if (!profile_)
-    return;
+  DCHECK(prefs_);
   pref_change_registrar_.RemoveAll();
-  notification_registrar_.Remove(this, NotificationType::PROFILE_DESTROYED,
-                                 Source<Profile>(profile_));
-  profile_ = NULL;
+  prefs_ = NULL;
+  for (ProviderIterator it = content_settings_providers_.begin();
+       it != content_settings_providers_.end();
+       ++it) {
+    (*it)->ShutdownOnUIThread();
+  }
+  for (DefaultProviderIterator it = default_content_settings_providers_.begin();
+       it != default_content_settings_providers_.end();
+       ++it) {
+    (*it)->ShutdownOnUIThread();
+  }
 }
 
-void HostContentSettingsMap::MigrateObsoleteCookiePref(PrefService* prefs) {
-  if (prefs->HasPrefPath(prefs::kCookieBehavior)) {
-    int cookie_behavior = prefs->GetInteger(prefs::kCookieBehavior);
-    prefs->ClearPref(prefs::kCookieBehavior);
-    if (!prefs->HasPrefPath(prefs::kDefaultContentSettings)) {
+void HostContentSettingsMap::MigrateObsoleteCookiePref() {
+  if (prefs_->HasPrefPath(prefs::kCookieBehavior)) {
+    int cookie_behavior = prefs_->GetInteger(prefs::kCookieBehavior);
+    prefs_->ClearPref(prefs::kCookieBehavior);
+    if (!prefs_->HasPrefPath(prefs::kDefaultContentSettings)) {
         SetDefaultContentSetting(CONTENT_SETTINGS_TYPE_COOKIES,
             (cookie_behavior == net::StaticCookiePolicy::BLOCK_ALL_COOKIES) ?
                 CONTENT_SETTING_BLOCK : CONTENT_SETTING_ALLOW);
     }
-    if (!prefs->HasPrefPath(prefs::kBlockThirdPartyCookies)) {
+    if (!prefs_->HasPrefPath(prefs::kBlockThirdPartyCookies)) {
       SetBlockThirdPartyCookies(cookie_behavior ==
           net::StaticCookiePolicy::BLOCK_SETTING_THIRD_PARTY_COOKIES);
     }
