@@ -17,8 +17,10 @@
 #include "base/basictypes.h"
 #include "base/memory/linked_ptr.h"
 #include "base/memory/ref_counted.h"
+#include "base/values.h"
 #include "base/synchronization/lock.h"
 #include "chrome/browser/content_settings/content_settings_pattern.h"
+#include "chrome/browser/content_settings/content_settings_observer.h"
 #include "chrome/browser/prefs/pref_change_registrar.h"
 #include "chrome/common/content_settings.h"
 #include "content/browser/browser_thread.h"
@@ -31,20 +33,27 @@ class ProviderInterface;
 }  // namespace content_settings
 
 class ContentSettingsDetails;
-class DictionaryValue;
+class ExtensionService;
 class GURL;
 class PrefService;
 class Profile;
 
 class HostContentSettingsMap
-    : public NotificationObserver,
-      public base::RefCountedThreadSafe<HostContentSettingsMap,
-                                        BrowserThread::DeleteOnUIThread> {
+    : public content_settings::Observer,
+      public NotificationObserver,
+      public base::RefCountedThreadSafe<HostContentSettingsMap> {
  public:
-  typedef std::pair<ContentSettingsPattern, ContentSetting> PatternSettingPair;
-  typedef std::vector<PatternSettingPair> SettingsForOneType;
+  // TODO(markusheintz): I sold my soul to the devil on order to add this tuple.
+  // I really want my soul back, so I really will change this ASAP.
+  typedef Tuple4<ContentSettingsPattern,
+                 ContentSettingsPattern,
+                 ContentSetting,
+                 std::string> PatternSettingSourceTuple;
+  typedef std::vector<PatternSettingSourceTuple> SettingsForOneType;
 
-  explicit HostContentSettingsMap(Profile* profile);
+  HostContentSettingsMap(PrefService* prefs,
+                         ExtensionService* extension_service,
+                         bool incognito);
 
   static void RegisterUserPrefs(PrefService* prefs);
 
@@ -59,14 +68,28 @@ class HostContentSettingsMap
   // This may be called on any thread.
   ContentSettings GetDefaultContentSettings() const;
 
-  // Returns a single ContentSetting which applies to a given URL. Note that
+  // Returns a single ContentSetting which applies to the given URLs. Note that
   // certain internal schemes are whitelisted. For ContentSettingsTypes that
-  // require an resource identifier to be specified, the |resource_identifier|
+  // require a resource identifier to be specified, the |resource_identifier|
   // must be non-empty.
   //
   // This may be called on any thread.
   ContentSetting GetContentSetting(
-      const GURL& url,
+      const GURL& primary_url,
+      const GURL& secondary_url,
+      ContentSettingsType content_type,
+      const std::string& resource_identifier) const;
+
+  // Returns a content setting |Value| which applies to the given URLs. Note
+  // that certain internal schemes are whitelisted. For ContentSettingsTypes
+  // that require a resource identifier to be specified, the
+  // |resource_identifier| must be non-empty. Ownership of the returned |Value|
+  // is transfered to the caller.
+  //
+  // This may be called on any thread.
+  Value* GetContentSettingValue(
+      const GURL& primary_url,
+      const GURL& secondary_url,
       ContentSettingsType content_type,
       const std::string& resource_identifier) const;
 
@@ -80,31 +103,14 @@ class HostContentSettingsMap
       const GURL& first_party_url,
       bool setting_cookie) const;
 
-  // Returns a single ContentSetting which applies to a given URL or
-  // CONTENT_SETTING_DEFAULT, if no exception applies. Note that certain
-  // internal schemes are whitelisted. For ContentSettingsTypes that require an
-  // resource identifier to be specified, the |resource_identifier| must be
-  // non-empty.
-  //
-  // This may be called on any thread.
-  ContentSetting GetNonDefaultContentSetting(
-      const GURL& url,
-      ContentSettingsType content_type,
-      const std::string& resource_identifier) const;
-
-  // Returns all ContentSettings which apply to a given URL. For content
+  // Returns all ContentSettings which apply to the given URLs. For content
   // setting types that require an additional resource identifier, the default
   // content setting is returned.
   //
   // This may be called on any thread.
-  ContentSettings GetContentSettings(const GURL& url) const;
-
-  // Returns all non-default ContentSettings which apply to a given URL. For
-  // content setting types that require an additional resource identifier,
-  // CONTENT_SETTING_DEFAULT is returned.
-  //
-  // This may be called on any thread.
-  ContentSettings GetNonDefaultContentSettings(const GURL& url) const;
+  ContentSettings GetContentSettings(
+      const GURL& primary_url,
+      const GURL& secondary_url) const;
 
   // For a given content type, returns all patterns with a non-default setting,
   // mapped to their actual settings, in lexicographical order.  |settings|
@@ -125,25 +131,27 @@ class HostContentSettingsMap
   void SetDefaultContentSetting(ContentSettingsType content_type,
                                 ContentSetting setting);
 
-  // Sets the blocking setting for a particular pattern and content type.
+  // Sets the content setting for the given patterns and content type.
   // Setting the value to CONTENT_SETTING_DEFAULT causes the default setting
   // for that type to be used when loading pages matching this pattern. For
   // ContentSettingsTypes that require an resource identifier to be specified,
   // the |resource_identifier| must be non-empty.
   //
   // This should only be called on the UI thread.
-  void SetContentSetting(const ContentSettingsPattern& pattern,
+  void SetContentSetting(const ContentSettingsPattern& primary_pattern,
+                         const ContentSettingsPattern& secondary_pattern,
                          ContentSettingsType content_type,
                          const std::string& resource_identifier,
                          ContentSetting setting);
 
-  // Convenience method to add a content setting for a given URL, making sure
+  // Convenience method to add a content setting for the given URLs, making sure
   // that there is no setting overriding it. For ContentSettingsTypes that
   // require an resource identifier to be specified, the |resource_identifier|
   // must be non-empty.
   //
   // This should only be called on the UI thread.
-  void AddExceptionForURL(const GURL& url,
+  void AddExceptionForURL(const GURL& primary_url,
+                          const GURL& secondary_url,
                           ContentSettingsType content_type,
                           const std::string& resource_identifier,
                           ContentSetting setting);
@@ -153,6 +161,9 @@ class HostContentSettingsMap
   // This should only be called on the UI thread.
   void ClearSettingsForOneType(ContentSettingsType content_type);
 
+  static bool IsSettingAllowedForType(ContentSetting setting,
+                                      ContentSettingsType content_type);
+
   // This setting trumps any host-specific settings.
   bool BlockThirdPartyCookies() const { return block_third_party_cookies_; }
   bool IsBlockThirdPartyCookiesManaged() const {
@@ -165,40 +176,68 @@ class HostContentSettingsMap
   // This should only be called on the UI thread.
   void SetBlockThirdPartyCookies(bool block);
 
-  // Resets all settings levels.
-  //
-  // This should only be called on the UI thread.
-  void ResetToDefaults();
-
   // Returns true if the default setting for the |content_type| is managed.
   bool IsDefaultContentSettingManaged(ContentSettingsType content_type) const;
 
+  // Detaches the HostContentSettingsMap from all Profile-related objects like
+  // PrefService. This methods needs to be called before destroying the Profile.
+  // Afterwards, none of the methods above that should only be called on the UI
+  // thread should be called anymore.
+  void ShutdownOnUIThread();
+
+  // content_settings::Observer implementation.
+  virtual void OnContentSettingChanged(
+      ContentSettingsPattern primary_pattern,
+      ContentSettingsPattern secondary_pattern,
+      ContentSettingsType content_type,
+      std::string resource_identifier);
+
   // NotificationObserver implementation.
-  virtual void Observe(NotificationType type,
+  virtual void Observe(int type,
                        const NotificationSource& source,
                        const NotificationDetails& details);
 
  private:
-  friend struct BrowserThread::DeleteOnThread<BrowserThread::UI>;
-  friend class DeleteTask<HostContentSettingsMap>;
+  friend class base::RefCountedThreadSafe<HostContentSettingsMap>;
+  friend class HostContentSettingsMapTest_NonDefaultSettings_Test;
 
   virtual ~HostContentSettingsMap();
 
-  ContentSetting GetContentSettingInternal(
-      const GURL& url,
+  // Returns all non-default ContentSettings which apply to the given URLs. For
+  // content setting types that require an additional resource identifier,
+  // CONTENT_SETTING_DEFAULT is returned.
+  //
+  // This may be called on any thread.
+  ContentSettings GetNonDefaultContentSettings(
+      const GURL& primary_url,
+      const GURL& secondary_url) const;
+
+  // Returns a single ContentSetting which applies to the given URLs or
+  // CONTENT_SETTING_DEFAULT, if no exception applies. Note that certain
+  // internal schemes are whitelisted. For ContentSettingsTypes that require an
+  // resource identifier to be specified, the |resource_identifier| must be
+  // non-empty.
+  //
+  // This may be called on any thread.
+  ContentSetting GetNonDefaultContentSetting(
+      const GURL& primary_url,
+      const GURL& secondary_url,
       ContentSettingsType content_type,
       const std::string& resource_identifier) const;
 
-  void UnregisterObservers();
+  ContentSetting GetContentSettingInternal(
+      const GURL& primary_url,
+      const GURL& secondary_url,
+      ContentSettingsType content_type,
+      const std::string& resource_identifier) const;
 
   // Various migration methods (old cookie, popup and per-host data gets
   // migrated to the new format).
-  void MigrateObsoleteCookiePref(PrefService* prefs);
+  void MigrateObsoleteCookiePref();
 
-  // The profile we're associated with.
-  Profile* profile_;
+  // Weak; owned by the Profile.
+  PrefService* prefs_;
 
-  NotificationRegistrar notification_registrar_;
   PrefChangeRegistrar pref_change_registrar_;
 
   // Whether this settings map is for an OTR session.
