@@ -9,6 +9,7 @@
 #include "base/lazy_instance.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/browsing_data_appcache_helper.h"
+#include "chrome/browser/browsing_data_cookie_helper.h"
 #include "chrome/browser/browsing_data_database_helper.h"
 #include "chrome/browser/browsing_data_file_system_helper.h"
 #include "chrome/browser/browsing_data_indexed_db_helper.h"
@@ -17,6 +18,7 @@
 #include "chrome/browser/content_settings/host_content_settings_map.h"
 #include "chrome/browser/cookies_tree_model.h"
 #include "chrome/browser/profiles/profile.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/render_messages.h"
 #include "content/browser/renderer_host/render_process_host.h"
 #include "content/browser/renderer_host/render_view_host.h"
@@ -25,7 +27,6 @@
 #include "content/browser/tab_contents/tab_contents_delegate.h"
 #include "content/common/notification_service.h"
 #include "content/common/view_messages.h"
-#include "net/base/cookie_monster.h"
 #include "webkit/fileapi/file_system_types.h"
 
 namespace {
@@ -35,8 +36,8 @@ static base::LazyInstance<TabSpecificList> g_tab_specific(
 }
 
 bool TabSpecificContentSettings::LocalSharedObjectsContainer::empty() const {
-  return cookies_->GetAllCookies().empty() &&
-      appcaches_->empty() &&
+  return appcaches_->empty() &&
+      cookies_->empty() &&
       databases_->empty() &&
       file_systems_->empty() &&
       indexed_dbs_->empty() &&
@@ -46,16 +47,18 @@ bool TabSpecificContentSettings::LocalSharedObjectsContainer::empty() const {
 
 TabSpecificContentSettings::TabSpecificContentSettings(TabContents* tab)
     : TabContentsObserver(tab),
-      allowed_local_shared_objects_(tab->profile()),
-      blocked_local_shared_objects_(tab->profile()),
-      geolocation_settings_state_(tab->profile()),
+      profile_(Profile::FromBrowserContext(tab->browser_context())),
+      allowed_local_shared_objects_(profile_),
+      blocked_local_shared_objects_(profile_),
+      geolocation_settings_state_(profile_),
       load_plugins_link_enabled_(true) {
   ClearBlockedContentSettingsExceptForCookies();
   ClearCookieSpecificContentSettings();
   g_tab_specific.Get().push_back(this);
 
-  registrar_.Add(this, NotificationType::CONTENT_SETTINGS_CHANGED,
-                 NotificationService::AllSources());
+  registrar_.Add(this, chrome::NOTIFICATION_CONTENT_SETTINGS_CHANGED,
+                 Source<HostContentSettingsMap>(
+                     profile_->GetHostContentSettingsMap()));
 }
 
 TabSpecificContentSettings::~TabSpecificContentSettings() {
@@ -151,9 +154,6 @@ bool TabSpecificContentSettings::IsContentBlocked(
   DCHECK(content_type != CONTENT_SETTINGS_TYPE_NOTIFICATIONS)
       << "Notifications settings handled by "
       << "ContentSettingsNotificationsImageModel";
-  DCHECK(content_type != CONTENT_SETTINGS_TYPE_PRERENDER)
-      << "Prerendering settings handled by "
-      << "ContentSettingPrerenderImageModel";
 
   if (content_type == CONTENT_SETTINGS_TYPE_IMAGES ||
       content_type == CONTENT_SETTINGS_TYPE_JAVASCRIPT ||
@@ -215,7 +215,7 @@ void TabSpecificContentSettings::OnContentBlocked(
     content_blocked_[type] = true;
     // TODO: it would be nice to have a way of mocking this in tests.
     NotificationService::current()->Notify(
-        NotificationType::TAB_CONTENT_SETTINGS_CHANGED,
+        chrome::NOTIFICATION_TAB_CONTENT_SETTINGS_CHANGED,
         Source<TabContents>(tab_contents()),
         NotificationService::NoDetails());
   }
@@ -227,7 +227,7 @@ void TabSpecificContentSettings::OnContentAccessed(ContentSettingsType type) {
   if (!content_accessed_[type]) {
     content_accessed_[type] = true;
     NotificationService::current()->Notify(
-        NotificationType::TAB_CONTENT_SETTINGS_CHANGED,
+        chrome::NOTIFICATION_TAB_CONTENT_SETTINGS_CHANGED,
         Source<TabContents>(tab_contents()),
         NotificationService::NoDetails());
   }
@@ -239,24 +239,15 @@ void TabSpecificContentSettings::OnCookiesRead(
     bool blocked_by_policy) {
   if (cookie_list.empty())
     return;
-  LocalSharedObjectsContainer& container = blocked_by_policy ?
-      blocked_local_shared_objects_ : allowed_local_shared_objects_;
-  typedef net::CookieList::const_iterator cookie_iterator;
-  for (cookie_iterator cookie = cookie_list.begin();
-       cookie != cookie_list.end(); ++cookie) {
-    container.cookies()->SetCookieWithDetails(url,
-                                              cookie->Name(),
-                                              cookie->Value(),
-                                              cookie->Domain(),
-                                              cookie->Path(),
-                                              cookie->ExpiryDate(),
-                                              cookie->IsSecure(),
-                                              cookie->IsHttpOnly());
-  }
-  if (blocked_by_policy)
+  if (blocked_by_policy) {
+    blocked_local_shared_objects_.cookies()->AddReadCookies(
+        url, cookie_list);
     OnContentBlocked(CONTENT_SETTINGS_TYPE_COOKIES, std::string());
-  else
+  } else {
+    allowed_local_shared_objects_.cookies()->AddReadCookies(
+        url, cookie_list);
     OnContentAccessed(CONTENT_SETTINGS_TYPE_COOKIES);
+  }
 }
 
 void TabSpecificContentSettings::OnCookieChanged(
@@ -265,11 +256,11 @@ void TabSpecificContentSettings::OnCookieChanged(
     const net::CookieOptions& options,
     bool blocked_by_policy) {
   if (blocked_by_policy) {
-    blocked_local_shared_objects_.cookies()->SetCookieWithOptions(
+    blocked_local_shared_objects_.cookies()->AddChangedCookie(
         url, cookie_line, options);
     OnContentBlocked(CONTENT_SETTINGS_TYPE_COOKIES, std::string());
   } else {
-    allowed_local_shared_objects_.cookies()->SetCookieWithOptions(
+    allowed_local_shared_objects_.cookies()->AddChangedCookie(
         url, cookie_line, options);
     OnContentAccessed(CONTENT_SETTINGS_TYPE_COOKIES);
   }
@@ -353,7 +344,7 @@ void TabSpecificContentSettings::OnGeolocationPermissionSet(
   geolocation_settings_state_.OnGeolocationPermissionSet(requesting_origin,
                                                          allowed);
   NotificationService::current()->Notify(
-      NotificationType::TAB_CONTENT_SETTINGS_CHANGED,
+      chrome::NOTIFICATION_TAB_CONTENT_SETTINGS_CHANGED,
       Source<TabContents>(tab_contents()),
       NotificationService::NoDetails());
 }
@@ -369,7 +360,7 @@ void TabSpecificContentSettings::ClearBlockedContentSettingsExceptForCookies() {
   }
   load_plugins_link_enabled_ = true;
   NotificationService::current()->Notify(
-      NotificationType::TAB_CONTENT_SETTINGS_CHANGED,
+      chrome::NOTIFICATION_TAB_CONTENT_SETTINGS_CHANGED,
       Source<TabContents>(tab_contents()),
       NotificationService::NoDetails());
 }
@@ -381,7 +372,7 @@ void TabSpecificContentSettings::ClearCookieSpecificContentSettings() {
   content_accessed_[CONTENT_SETTINGS_TYPE_COOKIES] = false;
   content_blockage_indicated_to_user_[CONTENT_SETTINGS_TYPE_COOKIES] = false;
   NotificationService::current()->Notify(
-      NotificationType::TAB_CONTENT_SETTINGS_CHANGED,
+      chrome::NOTIFICATION_TAB_CONTENT_SETTINGS_CHANGED,
       Source<TabContents>(tab_contents()),
       NotificationService::NoDetails());
 }
@@ -390,7 +381,7 @@ void TabSpecificContentSettings::SetPopupsBlocked(bool blocked) {
   content_blocked_[CONTENT_SETTINGS_TYPE_POPUPS] = blocked;
   content_blockage_indicated_to_user_[CONTENT_SETTINGS_TYPE_POPUPS] = false;
   NotificationService::current()->Notify(
-      NotificationType::TAB_CONTENT_SETTINGS_CHANGED,
+      chrome::NOTIFICATION_TAB_CONTENT_SETTINGS_CHANGED,
       Source<TabContents>(tab_contents()),
       NotificationService::NoDetails());
 }
@@ -416,7 +407,7 @@ bool TabSpecificContentSettings::OnMessageReceived(
     const IPC::Message& message) {
   bool handled = true;
   IPC_BEGIN_MESSAGE_MAP(TabSpecificContentSettings, message)
-    IPC_MESSAGE_HANDLER(ViewHostMsg_ContentBlocked, OnContentBlocked)
+    IPC_MESSAGE_HANDLER(ChromeViewHostMsg_ContentBlocked, OnContentBlocked)
     IPC_MESSAGE_HANDLER(ViewHostMsg_AppCacheAccessed, OnAppCacheAccessed)
     IPC_MESSAGE_UNHANDLED(handled = false)
   IPC_END_MESSAGE_MAP()
@@ -435,9 +426,10 @@ void TabSpecificContentSettings::DidNavigateMainFramePostCommit(
 
 void TabSpecificContentSettings::RenderViewCreated(
     RenderViewHost* render_view_host) {
-  HostContentSettingsMap* map =
-      tab_contents()->profile()->GetHostContentSettingsMap();
-  render_view_host->Send(new ViewMsg_SetDefaultContentSettings(
+  Profile* profile =
+      Profile::FromBrowserContext(tab_contents()->browser_context());
+  HostContentSettingsMap* map = profile->GetHostContentSettingsMap();
+  render_view_host->Send(new ChromeViewMsg_SetDefaultContentSettings(
       map->GetDefaultContentSettings()));
 }
 
@@ -458,10 +450,10 @@ void TabSpecificContentSettings::DidStartProvisionalLoadForFrame(
   ClearGeolocationContentSettings();
 }
 
-void TabSpecificContentSettings::Observe(NotificationType type,
+void TabSpecificContentSettings::Observe(int type,
                                          const NotificationSource& source,
                                          const NotificationDetails& details) {
-  DCHECK(type.value == NotificationType::CONTENT_SETTINGS_CHANGED);
+  DCHECK(type == chrome::NOTIFICATION_CONTENT_SETTINGS_CHANGED);
 
   Details<const ContentSettingsDetails> settings_details(details);
   const NavigationController& controller = tab_contents()->controller();
@@ -470,29 +462,28 @@ void TabSpecificContentSettings::Observe(NotificationType type,
   if (entry)
     entry_url = entry->url();
   if (settings_details.ptr()->update_all() ||
-      settings_details.ptr()->pattern().Matches(entry_url)) {
-    HostContentSettingsMap* map =
-        tab_contents()->profile()->GetHostContentSettingsMap();
-    Send(new ViewMsg_SetDefaultContentSettings(
+      // The active NavigationEntry is the URL in the URL field of a tab.
+      // Currently this should be matched by the |primary_pattern|.
+      settings_details.ptr()->primary_pattern().Matches(entry_url)) {
+    Profile* profile =
+        Profile::FromBrowserContext(tab_contents()->browser_context());
+    HostContentSettingsMap* map = profile->GetHostContentSettingsMap();
+    Send(new ChromeViewMsg_SetDefaultContentSettings(
         map->GetDefaultContentSettings()));
-    Send(new ViewMsg_SetContentSettingsForCurrentURL(
-        entry_url, map->GetContentSettings(entry_url)));
+    Send(new ChromeViewMsg_SetContentSettingsForCurrentURL(
+        entry_url, map->GetContentSettings(entry_url, entry_url)));
   }
 }
 
 TabSpecificContentSettings::LocalSharedObjectsContainer::
     LocalSharedObjectsContainer(Profile* profile)
-    : cookies_(new net::CookieMonster(NULL, NULL)),
-      appcaches_(new CannedBrowsingDataAppCacheHelper(profile)),
+    : appcaches_(new CannedBrowsingDataAppCacheHelper(profile)),
+      cookies_(new CannedBrowsingDataCookieHelper(profile)),
       databases_(new CannedBrowsingDataDatabaseHelper(profile)),
       file_systems_(new CannedBrowsingDataFileSystemHelper(profile)),
-      indexed_dbs_(new CannedBrowsingDataIndexedDBHelper(profile)),
+      indexed_dbs_(new CannedBrowsingDataIndexedDBHelper()),
       local_storages_(new CannedBrowsingDataLocalStorageHelper(profile)),
       session_storages_(new CannedBrowsingDataLocalStorageHelper(profile)) {
-  cookies_->SetCookieableSchemes(
-      net::CookieMonster::kDefaultCookieableSchemes,
-      net::CookieMonster::kDefaultCookieableSchemesCount);
-  cookies_->SetKeepExpiredCookies();
 }
 
 TabSpecificContentSettings::LocalSharedObjectsContainer::
@@ -500,12 +491,8 @@ TabSpecificContentSettings::LocalSharedObjectsContainer::
 }
 
 void TabSpecificContentSettings::LocalSharedObjectsContainer::Reset() {
-  cookies_ = new net::CookieMonster(NULL, NULL);
-  cookies_->SetCookieableSchemes(
-      net::CookieMonster::kDefaultCookieableSchemes,
-      net::CookieMonster::kDefaultCookieableSchemesCount);
-  cookies_->SetKeepExpiredCookies();
   appcaches_->Reset();
+  cookies_->Reset();
   databases_->Reset();
   file_systems_->Reset();
   indexed_dbs_->Reset();
@@ -515,12 +502,13 @@ void TabSpecificContentSettings::LocalSharedObjectsContainer::Reset() {
 
 CookiesTreeModel*
 TabSpecificContentSettings::LocalSharedObjectsContainer::GetCookiesTreeModel() {
-  return new CookiesTreeModel(cookies_,
+  return new CookiesTreeModel(cookies_->Clone(),
                               databases_->Clone(),
                               local_storages_->Clone(),
                               session_storages_->Clone(),
                               appcaches_->Clone(),
                               indexed_dbs_->Clone(),
                               file_systems_->Clone(),
+                              NULL,
                               true);
 }
