@@ -5,15 +5,19 @@
 #ifndef CHROME_BROWSER_DEBUGGER_BROWSER_LIST_TABCONTENTS_PROVIDER_H_
 #define CHROME_BROWSER_DEBUGGER_BROWSER_LIST_TABCONTENTS_PROVIDER_H_
 
-#include "chrome/browser/debugger/devtools_http_protocol_handler.h"
+#include <string>
+#include "content/browser/debugger/devtools_http_protocol_handler.h"
 
-class BrowserListTabContentsProvider :
-    public DevToolsHttpProtocolHandler::TabContentsProvider {
+class BrowserListTabContentsProvider
+    : public DevToolsHttpProtocolHandler::Delegate {
  public:
   BrowserListTabContentsProvider() {}
   virtual ~BrowserListTabContentsProvider() {}
 
+  // DevToolsHttpProtocolHandler::Delegate overrides.
   virtual DevToolsHttpProtocolHandler::InspectableTabs GetInspectableTabs();
+  virtual std::string GetDiscoveryPageHTML();
+  virtual net::URLRequestContext* GetURLRequestContext();
  private:
   DISALLOW_COPY_AND_ASSIGN(BrowserListTabContentsProvider);
 };
