@@ -9,13 +9,28 @@
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_list.h"
+#include "chrome/browser/ui/browser_window.h"
+#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/common/extensions/extension.h"
+#include "chrome/test/base/ui_test_utils.h"
 #include "content/browser/renderer_host/render_view_host.h"
 #include "content/browser/tab_contents/tab_contents.h"
 #include "net/base/mock_host_resolver.h"
 
 class AppApiTest : public ExtensionApiTest {
+ protected:
+  // Gets the base URL for files for a specific test, making sure that it uses
+  // "localhost" as the hostname, since that is what the extent is declared
+  // as in the test apps manifests.
+  GURL GetTestBaseURL(std::string test_directory) {
+    GURL::Replacements replace_host;
+    std::string host_str("localhost");  // must stay in scope with replace_host
+    replace_host.SetHostStr(host_str);
+    GURL base_url = test_server()->GetURL(
+        "files/extensions/api_test/" + test_directory + "/");
+    return base_url.ReplaceComponents(replace_host);
+  }
 };
 
 // Simulates a page calling window.open on an URL, and waits for the navigation.
@@ -79,15 +94,7 @@ IN_PROC_BROWSER_TEST_F(AppApiTest, MAYBE_AppProcess) {
   ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII("app_process")));
 
   // Open two tabs in the app, one outside it.
-  GURL base_url = test_server()->GetURL(
-      "files/extensions/api_test/app_process/");
-
-  // The app under test acts on URLs whose host is "localhost",
-  // so the URLs we navigate to must have host "localhost".
-  GURL::Replacements replace_host;
-  std::string host_str("localhost");  // must stay in scope with replace_host
-  replace_host.SetHostStr(host_str);
-  base_url = base_url.ReplaceComponents(replace_host);
+  GURL base_url = GetTestBaseURL("app_process");
 
   // Test both opening a URL in a new tab, and opening a tab and then navigating
   // it.  Either way, app tabs should be considered extension processes, but
@@ -127,9 +134,12 @@ IN_PROC_BROWSER_TEST_F(AppApiTest, MAYBE_AppProcess) {
                    base_url.Resolve("path1/empty.html"), true);
   WindowOpenHelper(browser(), host,
                    base_url.Resolve("path2/empty.html"), true);
-  // This should open in a new process (i.e., false for the last argument).
+  // TODO(creis): This should open in a new process (i.e., false for the last
+  // argument), but we temporarily avoid swapping processes away from an app
+  // until we're able to support cross-process postMessage calls.
+  // See crbug.com/59285.
   WindowOpenHelper(browser(), host,
-                   base_url.Resolve("path3/empty.html"), false);
+                   base_url.Resolve("path3/empty.html"), true);
 
   // Now let's have these pages navigate, into or out of the extension web
   // extent. They should switch processes.
@@ -137,7 +147,10 @@ IN_PROC_BROWSER_TEST_F(AppApiTest, MAYBE_AppProcess) {
   const GURL& non_app_url(base_url.Resolve("path3/empty.html"));
   NavigateTabHelper(browser()->GetTabContentsAt(2), non_app_url);
   NavigateTabHelper(browser()->GetTabContentsAt(3), app_url);
-  EXPECT_NE(host->process(),
+  // TODO(creis): This should swap out of the app's process (i.e., EXPECT_NE),
+  // but we temporarily avoid swapping away from an app in case the window
+  // tries to send a postMessage to the app.  See crbug.com/59285.
+  EXPECT_EQ(host->process(),
             browser()->GetTabContentsAt(2)->render_view_host()->process());
   EXPECT_EQ(host->process(),
             browser()->GetTabContentsAt(3)->render_view_host()->process());
@@ -176,15 +189,7 @@ IN_PROC_BROWSER_TEST_F(AppApiTest, MAYBE_AppProcessInstances) {
       test_data_dir_.AppendASCII("app_process_instances")));
 
   // Open two tabs in the app, one outside it.
-  GURL base_url = test_server()->GetURL(
-      "files/extensions/api_test/app_process_instances/");
-
-  // The app under test acts on URLs whose host is "localhost",
-  // so the URLs we navigate to must have host "localhost".
-  GURL::Replacements replace_host;
-  std::string host_str("localhost");  // must stay in scope with replace_host
-  replace_host.SetHostStr(host_str);
-  base_url = base_url.ReplaceComponents(replace_host);
+  GURL base_url = GetTestBaseURL("app_process_instances");
 
   // Test both opening a URL in a new tab, and opening a tab and then navigating
   // it.  Either way, app tabs should be considered extension processes, but
@@ -234,15 +239,7 @@ IN_PROC_BROWSER_TEST_F(AppApiTest, AppProcessRedirectBack) {
   ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII("app_process")));
 
   // Open two tabs in the app.
-  GURL base_url = test_server()->GetURL(
-      "files/extensions/api_test/app_process/");
-
-  // The app under test acts on URLs whose host is "localhost",
-  // so the URLs we navigate to must have host "localhost".
-  GURL::Replacements replace_host;
-  std::string host_str("localhost");  // must stay in scope with replace_host
-  replace_host.SetHostStr(host_str);
-  base_url = base_url.ReplaceComponents(replace_host);
+  GURL base_url = GetTestBaseURL("app_process");
 
   browser()->NewTab();
   ui_test_utils::NavigateToURL(browser(), base_url.Resolve("path1/empty.html"));
@@ -273,12 +270,7 @@ IN_PROC_BROWSER_TEST_F(AppApiTest, ReloadIntoAppProcess) {
 
   // The app under test acts on URLs whose host is "localhost",
   // so the URLs we navigate to must have host "localhost".
-  GURL::Replacements replace_host;
-  std::string host_str("localhost");  // must stay in scope with replace_host
-  replace_host.SetHostStr(host_str);
-  GURL base_url = test_server()->GetURL(
-      "files/extensions/api_test/app_process/");
-  base_url = base_url.ReplaceComponents(replace_host);
+  GURL base_url = GetTestBaseURL("app_process");
 
   // Load an app URL before loading the app.
   ui_test_utils::NavigateToURL(browser(), base_url.Resolve("path1/empty.html"));
@@ -311,3 +303,136 @@ IN_PROC_BROWSER_TEST_F(AppApiTest, ReloadIntoAppProcess) {
   ui_test_utils::WaitForNavigation(&contents->controller());
   EXPECT_FALSE(contents->render_view_host()->process()->is_extension_process());
 }
+
+
+// Tests that if we have a non-app process (path3/container.html) that has an
+// iframe with  a URL in the app's extent (path1/iframe.html), then opening a
+// link from that iframe to a new window to a URL in the app's extent (path1/
+// empty.html) results in the new window being in an app process. See
+// http://crbug.com/89272 for more details.
+IN_PROC_BROWSER_TEST_F(AppApiTest, OpenAppFromIframe) {
+  CommandLine::ForCurrentProcess()->AppendSwitch(
+      switches::kDisablePopupBlocking);
+
+  host_resolver()->AddRule("*", "127.0.0.1");
+  ASSERT_TRUE(test_server()->Start());
+
+  GURL base_url = GetTestBaseURL("app_process");
+
+  // Load app and start URL (not in the app).
+  const Extension* app =
+      LoadExtension(test_data_dir_.AppendASCII("app_process"));
+  ASSERT_TRUE(app);
+  ui_test_utils::NavigateToURLWithDisposition(
+      browser(),
+      base_url.Resolve("path3/container.html"),
+      CURRENT_TAB,
+      ui_test_utils::BROWSER_TEST_WAIT_FOR_NAVIGATION |
+          ui_test_utils::BROWSER_TEST_WAIT_FOR_BROWSER);
+  EXPECT_FALSE(browser()->GetTabContentsAt(0)->render_view_host()->process()->
+      is_extension_process());
+
+  // Wait for popup window to appear.
+  GURL app_url = base_url.Resolve("path1/empty.html");
+  Browser* last_active_browser = BrowserList::GetLastActive();
+  EXPECT_TRUE(last_active_browser);
+  ASSERT_NE(browser(), last_active_browser);
+  TabContents* newtab = last_active_browser->GetSelectedTabContents();
+  EXPECT_TRUE(newtab);
+  if (!newtab->controller().GetLastCommittedEntry() ||
+      newtab->controller().GetLastCommittedEntry()->url() != app_url)
+    ui_test_utils::WaitForNavigation(&newtab->controller());
+
+  // Popup window should be in the app's process.
+  EXPECT_TRUE(last_active_browser->GetTabContentsAt(0)->render_view_host()->
+      process()->is_extension_process());
+}
+
+// Tests that if we have an app process (path1/container.html) with a non-app
+// iframe (path3/iframe.html), then opening a link from that iframe to a new
+// window to a same-origin non-app URL (path3/empty.html) should keep the window
+// in the app process.
+// This is in contrast to OpenAppFromIframe, since here the popup will not be
+// missing special permissions and should be scriptable from the iframe.
+// See http://crbug.com/92669 for more details.
+IN_PROC_BROWSER_TEST_F(AppApiTest, OpenWebPopupFromWebIframe) {
+  CommandLine::ForCurrentProcess()->AppendSwitch(
+      switches::kDisablePopupBlocking);
+
+  host_resolver()->AddRule("*", "127.0.0.1");
+  ASSERT_TRUE(test_server()->Start());
+
+  GURL base_url = GetTestBaseURL("app_process");
+
+  // Load app and start URL (in the app).
+  const Extension* app =
+      LoadExtension(test_data_dir_.AppendASCII("app_process"));
+  ASSERT_TRUE(app);
+  ui_test_utils::NavigateToURLWithDisposition(
+      browser(),
+      base_url.Resolve("path1/container.html"),
+      CURRENT_TAB,
+      ui_test_utils::BROWSER_TEST_WAIT_FOR_NAVIGATION |
+          ui_test_utils::BROWSER_TEST_WAIT_FOR_BROWSER);
+  RenderProcessHost* process =
+      browser()->GetTabContentsAt(0)->render_view_host()->process();
+  EXPECT_TRUE(process->is_extension_process());
+
+  // Wait for popup window to appear.  The new Browser may not have been
+  // added with SetLastActive, in which case we need to show it first.
+  // This is necessary for popup windows without a cross-site transition.
+  if (browser() == BrowserList::GetLastActive()) {
+    // Grab the second window and show it.
+    ASSERT_TRUE(BrowserList::size() == 2);
+    Browser* popup_browser = *(++BrowserList::begin());
+    popup_browser->window()->Show();
+  }
+  Browser* last_active_browser = BrowserList::GetLastActive();
+  EXPECT_TRUE(last_active_browser);
+  ASSERT_NE(browser(), last_active_browser);
+  TabContents* newtab = last_active_browser->GetSelectedTabContents();
+  EXPECT_TRUE(newtab);
+  GURL non_app_url = base_url.Resolve("path3/empty.html");
+  if (!newtab->controller().GetLastCommittedEntry() ||
+      newtab->controller().GetLastCommittedEntry()->url() != non_app_url)
+    ui_test_utils::WaitForNavigation(&newtab->controller());
+
+  // Popup window should be in the app's process.
+  RenderProcessHost* popup_process =
+      last_active_browser->GetTabContentsAt(0)->render_view_host()->process();
+  EXPECT_EQ(process, popup_process);
+}
+
+IN_PROC_BROWSER_TEST_F(AppApiTest, ReloadAppAfterCrash) {
+  host_resolver()->AddRule("*", "127.0.0.1");
+  ASSERT_TRUE(test_server()->Start());
+
+  ASSERT_TRUE(LoadExtension(test_data_dir_.AppendASCII("app_process")));
+
+  GURL base_url = GetTestBaseURL("app_process");
+
+  // Load the app, chrome.app.isInstalled should be true.
+  ui_test_utils::NavigateToURL(browser(), base_url.Resolve("path1/empty.html"));
+  TabContents* contents = browser()->GetTabContentsAt(0);
+  EXPECT_TRUE(contents->render_view_host()->process()->is_extension_process());
+  bool is_installed = false;
+  ASSERT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
+      contents->render_view_host(), L"",
+      L"window.domAutomationController.send(chrome.app.isInstalled)",
+      &is_installed));
+  ASSERT_TRUE(is_installed);
+
+  // Crash the tab and reload it, chrome.app.isInstalled should still be true.
+  ui_test_utils::CrashTab(browser()->GetSelectedTabContents());
+  ui_test_utils::WindowedNotificationObserver observer(
+      content::NOTIFICATION_LOAD_STOP,
+      Source<NavigationController>(
+          &browser()->GetSelectedTabContentsWrapper()->controller()));
+  browser()->Reload(CURRENT_TAB);
+  observer.Wait();
+  ASSERT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
+      contents->render_view_host(), L"",
+      L"window.domAutomationController.send(chrome.app.isInstalled)",
+      &is_installed));
+  ASSERT_TRUE(is_installed);
+}
