@@ -8,10 +8,19 @@
 
 #include <string>
 
+#include "chrome/browser/content_settings/content_settings_pattern.h"
 #include "chrome/common/content_settings.h"
 
 namespace extension_content_settings_helpers {
 
+// Parses an extension match pattern and returns a corresponding
+// content settings pattern object.
+// If |pattern_str| is invalid or can't be converted to a content settings
+// pattern, |error| is set to the parsing error and an invalid pattern
+// is returned.
+ContentSettingsPattern ParseExtensionPattern(const std::string& pattern_str,
+                                             std::string* error);
+
 // Converts a content settings type string to the corresponding
 // ContentSettingsType. Returns CONTENT_SETTINGS_TYPE_DEFAULT if the string
 // didn't specify a valid content settings type.
