@@ -4,9 +4,13 @@
 
 #include "chrome/browser/extensions/extension_data_deleter.h"
 
+#include "base/file_util.h"
 #include "chrome/browser/profiles/profile.h"
+#include "chrome/common/chrome_constants.h"
 #include "chrome/common/extensions/extension.h"
+#include "content/browser/appcache/chrome_appcache_service.h"
 #include "content/browser/in_process_webkit/webkit_context.h"
+#include "content/common/url_constants.h"
 #include "net/base/cookie_monster.h"
 #include "net/base/net_errors.h"
 #include "net/url_request/url_request_context.h"
@@ -16,15 +20,29 @@
 #include "webkit/fileapi/file_system_context.h"
 
 ExtensionDataDeleter::ExtensionDataDeleter(Profile* profile,
-                                           const GURL& extension_url) {
+                                           const std::string& extension_id,
+                                           const GURL& storage_origin,
+                                           bool is_storage_isolated) {
   DCHECK(profile);
+  appcache_service_ = profile->GetAppCacheService();
   webkit_context_ = profile->GetWebKitContext();
   database_tracker_ = profile->GetDatabaseTracker();
-  extension_request_context_ = profile->GetRequestContextForExtensions();
+  // Pick the right request context depending on whether it's an extension,
+  // isolated app, or regular app.
+  if (storage_origin.SchemeIs(chrome::kExtensionScheme)) {
+    extension_request_context_ = profile->GetRequestContextForExtensions();
+  } else if (is_storage_isolated) {
+    extension_request_context_ =
+        profile->GetRequestContextForIsolatedApp(extension_id);
+    isolated_app_path_ = profile->GetPath().
+        Append(chrome::kIsolatedAppStateDirname).AppendASCII(extension_id);
+  } else {
+    extension_request_context_ = profile->GetRequestContext();
+  }
   file_system_context_ = profile->GetFileSystemContext();
-  extension_url_ = extension_url;
+  storage_origin_ = storage_origin;
   origin_id_ =
-      webkit_database::DatabaseUtil::GetOriginIdentifier(extension_url_);
+      webkit_database::DatabaseUtil::GetOriginIdentifier(storage_origin_);
 }
 
 ExtensionDataDeleter::~ExtensionDataDeleter() {
@@ -56,16 +74,21 @@ void ExtensionDataDeleter::StartDeleting() {
       BrowserThread::FILE, FROM_HERE,
       NewRunnableMethod(
           this, &ExtensionDataDeleter::DeleteFileSystemOnFileThread));
+
+  BrowserThread::PostTask(
+      BrowserThread::IO, FROM_HERE,
+      NewRunnableMethod(
+          this, &ExtensionDataDeleter::DeleteAppcachesOnIOThread));
 }
 
 void ExtensionDataDeleter::DeleteCookiesOnIOThread() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
   net::CookieMonster* cookie_monster =
       extension_request_context_->GetURLRequestContext()->cookie_store()->
-      GetCookieMonster();
+          GetCookieMonster();
   if (cookie_monster)
     cookie_monster->DeleteAllForHostAsync(
-        extension_url_, net::CookieMonster::DeleteCallback());
+        storage_origin_, net::CookieMonster::DeleteCallback());
 }
 
 void ExtensionDataDeleter::DeleteDatabaseOnFileThread() {
@@ -83,10 +106,21 @@ void ExtensionDataDeleter::DeleteLocalStorageOnWebkitThread() {
 void ExtensionDataDeleter::DeleteIndexedDBOnWebkitThread() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::WEBKIT));
   webkit_context_->indexed_db_context()->DeleteIndexedDBForOrigin(
-      extension_url_);
+      storage_origin_);
 }
 
 void ExtensionDataDeleter::DeleteFileSystemOnFileThread() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::FILE));
-  file_system_context_->DeleteDataForOriginOnFileThread(extension_url_);
+  file_system_context_->DeleteDataForOriginOnFileThread(storage_origin_);
+
+  // TODO(creis): The following call fails because the request context is still
+  // around, and holding open file handles in this directory.
+  // See http://crbug.com/85127
+  if (!isolated_app_path_.empty())
+    file_util::Delete(isolated_app_path_, true);
+}
+
+void ExtensionDataDeleter::DeleteAppcachesOnIOThread() {
+  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
+  appcache_service_->DeleteAppCachesForOrigin(storage_origin_, NULL);
 }
