@@ -32,6 +32,11 @@ extern const char kOnExtensionDisabled[];
 
 // FileBrowser.
 extern const char kOnFileBrowserDiskChanged[];
+extern const char kOnFileChanged[];
+extern const char kOnFileBrowserMountCompleted[];
+
+// InputMethod.
+extern const char kOnInputMethodChanged[];
 
 };  // namespace extension_event_names
 
