@@ -132,7 +132,8 @@ ExtensionHost::ExtensionHost(const Extension* extension,
                              ViewType::Type host_type)
     : extension_(extension),
       extension_id_(extension->id()),
-      profile_(site_instance->browsing_instance()->profile()),
+      profile_(Profile::FromBrowserContext(
+          site_instance->browsing_instance()->browser_context())),
       did_stop_loading_(false),
       document_element_available_(false),
       url_(url),
@@ -346,8 +347,7 @@ void ExtensionHost::InsertInfobarCSS() {
       IDR_EXTENSIONS_INFOBAR_CSS));
 
   render_view_host()->Send(new ViewMsg_CSSInsertRequest(
-      render_view_host()->routing_id(), L"", css.as_string(),
-      "InfobarThemeCSS"));
+      render_view_host()->routing_id(), string16(), css.as_string()));
 }
 
 void ExtensionHost::DisableScrollbarsForSmallWindows(
@@ -489,7 +489,9 @@ void ExtensionHost::Close(RenderViewHost* render_view_host) {
   }
 }
 
-RendererPreferences ExtensionHost::GetRendererPrefs(Profile* profile) const {
+RendererPreferences ExtensionHost::GetRendererPrefs(
+    content::BrowserContext* browser_context) const {
+  Profile* profile = Profile::FromBrowserContext(browser_context);
   RendererPreferences preferences;
 
   TabContents* associated_contents = GetAssociatedTabContents();
@@ -503,7 +505,8 @@ RendererPreferences ExtensionHost::GetRendererPrefs(Profile* profile) const {
 }
 
 WebPreferences ExtensionHost::GetWebkitPrefs() {
-  Profile* profile = render_view_host()->process()->profile();
+  Profile* profile = Profile::FromBrowserContext(
+      render_view_host()->process()->browser_context());
   WebPreferences webkit_prefs =
       RenderViewHostDelegateHelper::GetWebkitPrefs(profile,
                                                    false);  // is_web_ui
@@ -580,17 +583,30 @@ void ExtensionHost::HandleMouseUp() {
 void ExtensionHost::HandleMouseActivate() {
 }
 
+void ExtensionHost::RunFileChooser(
+    RenderViewHost* render_view_host,
+    const ViewHostMsg_RunFileChooser_Params& params) {
+  // This object is destroyed when the file selection is performed or
+  // cancelled.
+  FileSelectHelper* file_select_helper = new FileSelectHelper(profile());
+  file_select_helper->RunFileChooser(render_view_host,
+                                     GetAssociatedTabContents(),
+                                     params);
+}
+
 void ExtensionHost::CreateNewWindow(
     int route_id,
     const ViewHostMsg_CreateWindow_Params& params) {
   // TODO(aa): Use the browser's profile if the extension is split mode
   // incognito.
+  Profile* profile = Profile::FromBrowserContext(
+      render_view_host()->process()->browser_context());
   TabContents* new_contents = delegate_view_helper_.CreateNewWindow(
       route_id,
-      render_view_host()->process()->profile(),
+      profile,
       site_instance(),
       ChromeWebUIFactory::GetInstance()->GetWebUIType(
-          render_view_host()->process()->profile(), url_),
+          render_view_host()->process()->browser_context(), url_),
       this,
       params.window_container_type,
       params.frame_name);
@@ -623,17 +639,17 @@ void ExtensionHost::ShowCreatedWindow(int route_id,
   TabContents* contents = delegate_view_helper_.GetCreatedWindow(route_id);
   if (!contents)
     return;
+  Profile* profile = Profile::FromBrowserContext(contents->browser_context());
 
   if (disposition == NEW_POPUP) {
     // Find a browser with a matching profile for creating a popup.
     // (If none is found, NULL argument to NavigateParams is valid.)
     Browser* browser = BrowserList::FindTabbedBrowser(
-        contents->profile(),
-        false);  // Match incognito exactly.
+        profile, false);  // Match incognito exactly.
     TabContentsWrapper* wrapper = new TabContentsWrapper(contents);
     browser::NavigateParams params(browser, wrapper);
     if (!browser)
-      params.profile = contents->profile();
+      params.profile = profile;
     // The extension_app_id parameter ends up as app_name in the Browser
     // which causes the Browser to return true for is_app().  This affects
     // among other things, whether the location bar gets displayed.
@@ -658,7 +674,7 @@ void ExtensionHost::ShowCreatedWindow(int route_id,
   // vice versa.
   TabContents* associated_contents = GetAssociatedTabContents();
   if (associated_contents &&
-      associated_contents->profile() == contents->profile()) {
+      associated_contents->browser_context() == contents->browser_context()) {
     associated_contents->AddNewContents(
         contents, disposition, initial_pos, user_gesture);
     return;
@@ -668,12 +684,11 @@ void ExtensionHost::ShowCreatedWindow(int route_id,
   // profile, try finding an open window. Again, we must make sure to find a
   // window with the correct profile.
   Browser* browser = BrowserList::FindTabbedBrowser(
-        contents->profile(),
-        false);  // Match incognito exactly.
+        profile, false);  // Match incognito exactly.
 
   // If there's no Browser open with the right profile, create a new one.
   if (!browser) {
-    browser = Browser::Create(contents->profile());
+    browser = Browser::Create(profile);
     browser->window()->Show();
   }
   browser->AddTabContents(contents, disposition, initial_pos, user_gesture);
@@ -759,7 +774,6 @@ ViewType::Type ExtensionHost::GetRenderViewType() const {
 bool ExtensionHost::OnMessageReceived(const IPC::Message& message) {
   bool handled = true;
   IPC_BEGIN_MESSAGE_MAP(ExtensionHost, message)
-    IPC_MESSAGE_HANDLER(ViewHostMsg_RunFileChooser, OnRunFileChooser)
     IPC_MESSAGE_HANDLER(ExtensionHostMsg_Request, OnRequest)
     IPC_MESSAGE_UNHANDLED(handled = false)
   IPC_END_MESSAGE_MAP()
@@ -785,11 +799,3 @@ void ExtensionHost::RenderViewCreated(RenderViewHost* render_view_host) {
         kPreferredSizeWidth | kPreferredSizeHeightThisIsSlow));
   }
 }
-
-void ExtensionHost::OnRunFileChooser(
-    const ViewHostMsg_RunFileChooser_Params& params) {
-  if (file_select_helper_.get() == NULL)
-    file_select_helper_.reset(new FileSelectHelper(profile()));
-  file_select_helper_->RunFileChooser(render_view_host_,
-                                      GetAssociatedTabContents(), params);
-}
