@@ -94,7 +94,8 @@ TEST_F(ExtensionInfoMapTest, RefCounting) {
   EXPECT_TRUE(weak_extension1->HasOneRef());
 
   // Remove extension2, and the extension2 object should have the only ref.
-  info_map->RemoveExtension(extension2->id(), UnloadedExtensionInfo::UNINSTALL);
+  info_map->RemoveExtension(
+      extension2->id(), extension_misc::UNLOAD_REASON_UNINSTALL);
   EXPECT_TRUE(extension2->HasOneRef());
 
   // Delete the info map, and the extension3 object should have the only ref.
