@@ -14,17 +14,20 @@
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/extensions/extension_install_dialog.h"
 #include "chrome/browser/extensions/theme_installed_infobar_delegate.h"
-#include "chrome/browser/platform_util.h"
+#include "chrome/browser/infobars/infobar_tab_helper.h"
 #include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/simple_message_box.h"
 #include "chrome/browser/tabs/tab_strip_model.h"
 #include "chrome/browser/themes/theme_service.h"
 #include "chrome/browser/themes/theme_service_factory.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_dialogs.h"
 #include "chrome/browser/ui/browser_list.h"
+#include "chrome/browser/ui/browser_navigator.h"
 #include "chrome/browser/ui/browser_window.h"
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
 #include "chrome/common/chrome_notification_types.h"
+#include "chrome/common/chrome_switches.h"
 #include "chrome/common/extensions/extension.h"
 #include "chrome/common/extensions/extension_icon_set.h"
 #include "chrome/common/extensions/extension_resource.h"
@@ -35,26 +38,37 @@
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
 #include "ui/base/l10n/l10n_util.h"
+#include "ui/base/resource/resource_bundle.h"
 
-// static
-const int ExtensionInstallUI::kTitleIds[NUM_PROMPT_TYPES] = {
+static const int kTitleIds[ExtensionInstallUI::NUM_PROMPT_TYPES] = {
   IDS_EXTENSION_INSTALL_PROMPT_TITLE,
-  IDS_EXTENSION_RE_ENABLE_PROMPT_TITLE
+  IDS_EXTENSION_INLINE_INSTALL_PROMPT_TITLE,
+  IDS_EXTENSION_RE_ENABLE_PROMPT_TITLE,
+  IDS_EXTENSION_PERMISSIONS_PROMPT_TITLE
 };
-// static
-const int ExtensionInstallUI::kHeadingIds[NUM_PROMPT_TYPES] = {
+static const int kHeadingIds[ExtensionInstallUI::NUM_PROMPT_TYPES] = {
   IDS_EXTENSION_INSTALL_PROMPT_HEADING,
-  IDS_EXTENSION_RE_ENABLE_PROMPT_HEADING
+  IDS_EXTENSION_INSTALL_PROMPT_HEADING,
+  IDS_EXTENSION_RE_ENABLE_PROMPT_HEADING,
+  IDS_EXTENSION_PERMISSIONS_PROMPT_HEADING
 };
-// static
-const int ExtensionInstallUI::kButtonIds[NUM_PROMPT_TYPES] = {
+static const int kAcceptButtonIds[ExtensionInstallUI::NUM_PROMPT_TYPES] = {
   IDS_EXTENSION_PROMPT_INSTALL_BUTTON,
-  IDS_EXTENSION_PROMPT_RE_ENABLE_BUTTON
+  IDS_EXTENSION_PROMPT_INLINE_INSTALL_BUTTON,
+  IDS_EXTENSION_PROMPT_RE_ENABLE_BUTTON,
+  IDS_EXTENSION_PROMPT_PERMISSIONS_BUTTON
 };
-// static
-const int ExtensionInstallUI::kWarningIds[NUM_PROMPT_TYPES] = {
+static const int kAbortButtonIds[ExtensionInstallUI::NUM_PROMPT_TYPES] = {
+  0,
+  0,
+  0,
+  IDS_EXTENSION_PROMPT_PERMISSIONS_ABORT_BUTTON
+};
+static const int kPermissionsHeaderIds[ExtensionInstallUI::NUM_PROMPT_TYPES] = {
   IDS_EXTENSION_PROMPT_WILL_HAVE_ACCESS_TO,
-  IDS_EXTENSION_PROMPT_WILL_NOW_HAVE_ACCESS_TO
+  IDS_EXTENSION_PROMPT_WILL_HAVE_ACCESS_TO,
+  IDS_EXTENSION_PROMPT_WILL_NOW_HAVE_ACCESS_TO,
+  IDS_EXTENSION_PROMPT_WANTS_ACCESS_TO,
 };
 
 namespace {
@@ -62,29 +76,116 @@ namespace {
 // Size of extension icon in top left of dialog.
 const int kIconSize = 69;
 
-// Shows the application install animation on the new tab page for the app
-// with |app_id|. If a NTP already exists on the active |browser|, this will
-// select that tab and show the animation there. Otherwise, it will create
-// a new NTP.
-void ShowAppInstalledAnimation(Browser* browser, const std::string& app_id) {
-  // Select an already open NTP, if there is one. Existing NTPs will
-  // automatically show the install animation for any new apps.
-  for (int i = 0; i < browser->tab_count(); ++i) {
-    GURL url = browser->GetTabContentsAt(i)->GetURL();
-    if (url.SchemeIs(chrome::kChromeUIScheme) &&
-        url.host() == chrome::kChromeUINewTabHost) {
-      browser->ActivateTabAt(i, false);
-      return;
-    }
-  }
+}  // namespace
 
-  // If there isn't an NTP, open one and pass it the ID of the installed app.
-  std::string url = base::StringPrintf(
-      "%s#app-id=%s", chrome::kChromeUINewTabURL, app_id.c_str());
-  browser->AddSelectedTabWithURL(GURL(url), PageTransition::TYPED);
+ExtensionInstallUI::Prompt::Prompt(PromptType type) : type_(type) {
 }
 
-}  // namespace
+ExtensionInstallUI::Prompt::~Prompt() {
+}
+
+void ExtensionInstallUI::Prompt::SetPermissions(
+    std::vector<string16> permissions) {
+  permissions_ = permissions;
+}
+
+void ExtensionInstallUI::Prompt::SetInlineInstallWebstoreData(
+    std::string localized_user_count,
+    double average_rating,
+    int rating_count) {
+  CHECK_EQ(INLINE_INSTALL_PROMPT, type_);
+  localized_user_count_ = localized_user_count;
+  average_rating_ = average_rating;
+  rating_count_ = rating_count;
+}
+
+string16 ExtensionInstallUI::Prompt::GetDialogTitle() const {
+  if (type_ == INLINE_INSTALL_PROMPT) {
+    return l10n_util::GetStringFUTF16(
+      kTitleIds[type_], l10n_util::GetStringUTF16(IDS_SHORT_PRODUCT_NAME));
+  } else {
+    return l10n_util::GetStringUTF16(kTitleIds[type_]);
+  }
+}
+
+string16 ExtensionInstallUI::Prompt::GetHeading(std::string extension_name)
+    const {
+  if (type_ == INLINE_INSTALL_PROMPT) {
+    return UTF8ToUTF16(extension_name);
+  } else {
+    return l10n_util::GetStringFUTF16(
+        kHeadingIds[type_], UTF8ToUTF16(extension_name));
+  }
+}
+
+string16 ExtensionInstallUI::Prompt::GetAcceptButtonLabel() const {
+  return l10n_util::GetStringUTF16(kAcceptButtonIds[type_]);
+}
+
+bool ExtensionInstallUI::Prompt::HasAbortButtonLabel() const {
+  return kAbortButtonIds[type_] > 0;
+}
+
+string16 ExtensionInstallUI::Prompt::GetAbortButtonLabel() const {
+  CHECK(HasAbortButtonLabel());
+  return l10n_util::GetStringUTF16(kAbortButtonIds[type_]);
+}
+
+string16 ExtensionInstallUI::Prompt::GetPermissionsHeader() const {
+  return l10n_util::GetStringUTF16(kPermissionsHeaderIds[type_]);
+}
+
+void ExtensionInstallUI::Prompt::AppendRatingStars(
+    StarAppender appender, void* data) const {
+  CHECK(appender);
+  CHECK_EQ(INLINE_INSTALL_PROMPT, type_);
+  int rating_integer = floor(average_rating_);
+  double rating_fractional = average_rating_ - rating_integer;
+
+  if (rating_fractional > 0.66) {
+    rating_integer++;
+  }
+
+  if (rating_fractional < 0.33 || rating_fractional > 0.66) {
+    rating_fractional = 0;
+  }
+
+  ResourceBundle& rb = ResourceBundle::GetSharedInstance();
+  int i;
+  for (i = 0; i < rating_integer; i++) {
+    appender(rb.GetBitmapNamed(IDR_EXTENSIONS_RATING_STAR_ON), data);
+  }
+  if (rating_fractional) {
+    appender(rb.GetBitmapNamed(IDR_EXTENSIONS_RATING_STAR_HALF_LEFT), data);
+    i++;
+  }
+  for (; i < kMaxExtensionRating; i++) {
+    appender(rb.GetBitmapNamed(IDR_EXTENSIONS_RATING_STAR_OFF), data);
+  }
+}
+
+string16 ExtensionInstallUI::Prompt::GetRatingCount() const {
+  CHECK_EQ(INLINE_INSTALL_PROMPT, type_);
+  return l10n_util::GetStringFUTF16(
+        IDS_EXTENSION_RATING_COUNT,
+        UTF8ToUTF16(base::IntToString(rating_count_)));
+}
+
+string16 ExtensionInstallUI::Prompt::GetUserCount() const {
+  CHECK_EQ(INLINE_INSTALL_PROMPT, type_);
+  return l10n_util::GetStringFUTF16(
+      IDS_EXTENSION_USER_COUNT,
+      UTF8ToUTF16(localized_user_count_));
+}
+
+size_t ExtensionInstallUI::Prompt::GetPermissionCount() const {
+  return permissions_.size();
+}
+
+string16 ExtensionInstallUI::Prompt::GetPermission(int index) const {
+  return l10n_util::GetStringFUTF16(
+      IDS_EXTENSION_PERMISSION_LINE, permissions_[index]);
+}
 
 ExtensionInstallUI::ExtensionInstallUI(Profile* profile)
     : profile_(profile),
@@ -93,7 +194,8 @@ ExtensionInstallUI::ExtensionInstallUI(Profile* profile)
       extension_(NULL),
       delegate_(NULL),
       prompt_type_(NUM_PROMPT_TYPES),
-      ALLOW_THIS_IN_INITIALIZER_LIST(tracker_(this)) {
+      ALLOW_THIS_IN_INITIALIZER_LIST(tracker_(this)),
+      use_app_installed_bubble_(false) {
   // Remember the current theme in case the user presses undo.
   if (profile_) {
     const Extension* previous_theme =
@@ -112,6 +214,7 @@ void ExtensionInstallUI::ConfirmInstall(Delegate* delegate,
                                         const Extension* extension) {
   DCHECK(ui_loop_ == MessageLoop::current());
   extension_ = extension;
+  permissions_ = extension->GetActivePermissions();
   delegate_ = delegate;
 
   // We special-case themes to not show any confirm UI. Instead they are
@@ -129,11 +232,24 @@ void ExtensionInstallUI::ConfirmReEnable(Delegate* delegate,
                                          const Extension* extension) {
   DCHECK(ui_loop_ == MessageLoop::current());
   extension_ = extension;
+  permissions_ = extension->GetActivePermissions();
   delegate_ = delegate;
 
   ShowConfirmation(RE_ENABLE_PROMPT);
 }
 
+void ExtensionInstallUI::ConfirmPermissions(
+    Delegate* delegate,
+    const Extension* extension,
+    const ExtensionPermissionSet* permissions) {
+  DCHECK(ui_loop_ == MessageLoop::current());
+  extension_ = extension;
+  permissions_ = permissions;
+  delegate_ = delegate;
+
+  ShowConfirmation(PERMISSIONS_PROMPT);
+}
+
 void ExtensionInstallUI::OnInstallSuccess(const Extension* extension,
                                           SkBitmap* icon) {
   extension_ = extension;
@@ -153,8 +269,16 @@ void ExtensionInstallUI::OnInstallSuccess(const Extension* extension,
     browser->AddBlankTab(true);
   browser->window()->Show();
 
-  if (extension->GetFullLaunchURL().is_valid()) {
-    ShowAppInstalledAnimation(browser, extension->id());
+  bool use_bubble_for_apps = false;
+
+#if defined(TOOLKIT_VIEWS)
+  CommandLine* cmdline = CommandLine::ForCurrentProcess();
+  use_bubble_for_apps = (use_app_installed_bubble_ ||
+                         cmdline->HasSwitch(switches::kAppsNewInstallBubble));
+#endif
+
+  if (extension->is_app() && !use_bubble_for_apps) {
+    ExtensionInstallUI::OpenAppInstalledNTP(browser, extension->id());
     return;
   }
 
@@ -173,7 +297,7 @@ void ExtensionInstallUI::OnInstallFailure(const std::string& error) {
   Browser* browser = BrowserList::GetLastActiveWithProfile(profile_);
   if (disable_failure_ui_for_tests)
     return;
-  platform_util::SimpleErrorBox(
+  browser::ShowErrorBox(
       browser ? browser->window()->GetNativeHandle() : NULL,
       l10n_util::GetStringUTF16(IDS_EXTENSION_INSTALL_FAILURE_TITLE),
       UTF8ToUTF16(error));
@@ -193,19 +317,18 @@ void ExtensionInstallUI::OnImageLoaded(
   SetIcon(image);
 
   switch (prompt_type_) {
+    case PERMISSIONS_PROMPT:
     case RE_ENABLE_PROMPT:
     case INSTALL_PROMPT: {
-      // TODO(jcivelli): http://crbug.com/44771 We should not show an install
-      //                 dialog when installing an app from the gallery.
       NotificationService* service = NotificationService::current();
       service->Notify(chrome::NOTIFICATION_EXTENSION_WILL_SHOW_CONFIRM_DIALOG,
           Source<ExtensionInstallUI>(this),
           NotificationService::NoDetails());
 
-      std::vector<string16> warnings =
-          extension_->GetPermissionMessageStrings();
+      Prompt prompt(prompt_type_);
+      prompt.SetPermissions(permissions_->GetWarningMessages());
       ShowExtensionInstallDialog(
-          profile_, delegate_, extension_, &icon_, warnings, prompt_type_);
+          profile_, delegate_, extension_, &icon_, prompt);
       break;
     }
     default:
@@ -214,6 +337,19 @@ void ExtensionInstallUI::OnImageLoaded(
   }
 }
 
+// static
+void ExtensionInstallUI::OpenAppInstalledNTP(Browser* browser,
+                                             const std::string& app_id) {
+  browser::NavigateParams params =
+      browser->GetSingletonTabNavigateParams(GURL(chrome::kChromeUINewTabURL));
+  browser::Navigate(&params);
+
+  NotificationService::current()->Notify(
+      chrome::NOTIFICATION_APP_INSTALLED_TO_NTP,
+      Source<TabContents>(params.target_contents->tab_contents()),
+      Details<const std::string>(&app_id));
+}
+
 // static
 void ExtensionInstallUI::DisableFailureUIForTests() {
   disable_failure_ui_for_tests = true;
@@ -234,11 +370,12 @@ void ExtensionInstallUI::ShowThemeInfoBar(const std::string& previous_theme_id,
   TabContentsWrapper* tab_contents = browser->GetSelectedTabContentsWrapper();
   if (!tab_contents)
     return;
+  InfoBarTabHelper* infobar_helper = tab_contents->infobar_tab_helper();
 
   // First find any previous theme preview infobars.
   InfoBarDelegate* old_delegate = NULL;
-  for (size_t i = 0; i < tab_contents->infobar_count(); ++i) {
-    InfoBarDelegate* delegate = tab_contents->GetInfoBarDelegateAt(i);
+  for (size_t i = 0; i < infobar_helper->infobar_count(); ++i) {
+    InfoBarDelegate* delegate = infobar_helper->GetInfoBarDelegateAt(i);
     ThemeInstalledInfoBarDelegate* theme_infobar =
         delegate->AsThemePreviewInfobarDelegate();
     if (theme_infobar) {
@@ -258,9 +395,9 @@ void ExtensionInstallUI::ShowThemeInfoBar(const std::string& previous_theme_id,
       previous_using_native_theme);
 
   if (old_delegate)
-    tab_contents->ReplaceInfoBar(old_delegate, new_delegate);
+    infobar_helper->ReplaceInfoBar(old_delegate, new_delegate);
   else
-    tab_contents->AddInfoBar(new_delegate);
+    infobar_helper->AddInfoBar(new_delegate);
 }
 
 void ExtensionInstallUI::ShowConfirmation(PromptType prompt_type) {
