@@ -25,9 +25,9 @@
 #include "chrome/common/extensions/extension.h"
 #include "chrome/common/extensions/extension_constants.h"
 #include "chrome/common/pref_names.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/testing_profile.h"
 #include "content/browser/browser_thread.h"
-#include "content/common/test_url_fetcher_factory.h"
+#include "content/test/test_url_fetcher_factory.h"
 #include "libxml/globals.h"
 #include "net/base/escape.h"
 #include "net/base/load_flags.h"
@@ -337,7 +337,6 @@ class ExtensionUpdaterTest : public testing::Test {
 
     // Set up and start the updater.
     TestURLFetcherFactory factory;
-    URLFetcher::set_factory(&factory);
     ExtensionUpdater updater(
         &service, service.extension_prefs(), service.pref_service(),
         service.profile(), 60*60*24);
@@ -390,7 +389,6 @@ class ExtensionUpdaterTest : public testing::Test {
     io_thread.Start();
 
     TestURLFetcherFactory factory;
-    URLFetcher::set_factory(&factory);
     ExtensionUpdater updater(
         &service, service.extension_prefs(), service.pref_service(),
         service.profile(), 60*60*24);
@@ -579,7 +577,6 @@ class ExtensionUpdaterTest : public testing::Test {
 
     TestURLFetcherFactory factory;
     TestURLFetcher* fetcher = NULL;
-    URLFetcher::set_factory(&factory);
     scoped_ptr<ServiceForDownloadTests> service(new ServiceForDownloadTests);
     ExtensionUpdater updater(service.get(),
                              service->extension_prefs(),
@@ -659,7 +656,6 @@ class ExtensionUpdaterTest : public testing::Test {
 
     TestURLFetcherFactory factory;
     TestURLFetcher* fetcher = NULL;
-    URLFetcher::set_factory(&factory);
     scoped_ptr<ServiceForDownloadTests> service(new ServiceForDownloadTests);
     ExtensionUpdater updater(service.get(), service->extension_prefs(),
                              service->pref_service(),
@@ -714,8 +710,6 @@ class ExtensionUpdaterTest : public testing::Test {
     // because of ImportantFileWriter.
     file_thread.Start();
     service.reset();
-
-    URLFetcher::set_factory(NULL);
   }
 
   static void TestBlacklistDownloading() {
@@ -727,7 +721,6 @@ class ExtensionUpdaterTest : public testing::Test {
 
     TestURLFetcherFactory factory;
     TestURLFetcher* fetcher = NULL;
-    URLFetcher::set_factory(&factory);
     ServiceForBlacklistTests service;
     ExtensionUpdater updater(
         &service, service.extension_prefs(), service.pref_service(),
@@ -764,8 +757,6 @@ class ExtensionUpdaterTest : public testing::Test {
 
     EXPECT_EQ(version, service.pref_service()->
       GetString(prefs::kExtensionBlacklistUpdateVersion));
-
-    URLFetcher::set_factory(NULL);
   }
 
   // Two extensions are updated.  If |updates_start_running| is true, the
@@ -781,7 +772,6 @@ class ExtensionUpdaterTest : public testing::Test {
 
     TestURLFetcherFactory factory;
     TestURLFetcher* fetcher = NULL;
-    URLFetcher::set_factory(&factory);
     ServiceForDownloadTests service;
     ExtensionUpdater updater(
         &service, service.extension_prefs(), service.pref_service(),
@@ -911,7 +901,6 @@ class ExtensionUpdaterTest : public testing::Test {
     BrowserThread file_thread(BrowserThread::FILE, &message_loop);
 
     TestURLFetcherFactory factory;
-    URLFetcher::set_factory(&factory);
 
     // Set up 2 mock extensions, one with a google.com update url and one
     // without.
@@ -1051,20 +1040,19 @@ class ExtensionUpdaterTest : public testing::Test {
 // actual test code to live in ExtenionUpdaterTest methods instead of TEST_F
 // subclasses where friendship with ExtenionUpdater is not inherited.
 
-TEST(ExtensionUpdaterTest, TestExtensionUpdateCheckRequests) {
+TEST_F(ExtensionUpdaterTest, TestExtensionUpdateCheckRequests) {
   ExtensionUpdaterTest::TestExtensionUpdateCheckRequests(false);
 }
 
-TEST(ExtensionUpdaterTest, TestExtensionUpdateCheckRequestsPending) {
+TEST_F(ExtensionUpdaterTest, TestExtensionUpdateCheckRequestsPending) {
   ExtensionUpdaterTest::TestExtensionUpdateCheckRequests(true);
 }
 
-// This test is disabled on Mac, see http://crbug.com/26035.
-TEST(ExtensionUpdaterTest, TestBlacklistUpdateCheckRequests) {
+TEST_F(ExtensionUpdaterTest, TestBlacklistUpdateCheckRequests) {
   ExtensionUpdaterTest::TestBlacklistUpdateCheckRequests();
 }
 
-TEST(ExtensionUpdaterTest, TestUpdateUrlData) {
+TEST_F(ExtensionUpdaterTest, TestUpdateUrlData) {
   MessageLoop message_loop;
   BrowserThread file_thread(BrowserThread::FILE, &message_loop);
 
@@ -1072,44 +1060,43 @@ TEST(ExtensionUpdaterTest, TestUpdateUrlData) {
   ExtensionUpdaterTest::TestUpdateUrlDataSimple();
   ExtensionUpdaterTest::TestUpdateUrlDataCompound();
   ExtensionUpdaterTest::TestUpdateUrlDataFromGallery(
-      Extension::GalleryUpdateUrl(false).spec());
+      extension_urls::GetWebstoreUpdateUrl(false).spec());
   ExtensionUpdaterTest::TestUpdateUrlDataFromGallery(
-      Extension::GalleryUpdateUrl(true).spec());
+      extension_urls::GetWebstoreUpdateUrl(true).spec());
 }
 
-TEST(ExtensionUpdaterTest, TestDetermineUpdates) {
+TEST_F(ExtensionUpdaterTest, TestDetermineUpdates) {
   ExtensionUpdaterTest::TestDetermineUpdates();
 }
 
-TEST(ExtensionUpdaterTest, TestDetermineUpdatesPending) {
+TEST_F(ExtensionUpdaterTest, TestDetermineUpdatesPending) {
   ExtensionUpdaterTest::TestDetermineUpdatesPending();
 }
 
-TEST(ExtensionUpdaterTest, TestMultipleManifestDownloading) {
+TEST_F(ExtensionUpdaterTest, TestMultipleManifestDownloading) {
   ExtensionUpdaterTest::TestMultipleManifestDownloading();
 }
 
-TEST(ExtensionUpdaterTest, TestSingleExtensionDownloading) {
+TEST_F(ExtensionUpdaterTest, TestSingleExtensionDownloading) {
   ExtensionUpdaterTest::TestSingleExtensionDownloading(false);
 }
 
-TEST(ExtensionUpdaterTest, TestSingleExtensionDownloadingPending) {
+TEST_F(ExtensionUpdaterTest, TestSingleExtensionDownloadingPending) {
   ExtensionUpdaterTest::TestSingleExtensionDownloading(true);
 }
 
-// This test is disabled on Mac, see http://crbug.com/26035.
-TEST(ExtensionUpdaterTest, TestBlacklistDownloading) {
+TEST_F(ExtensionUpdaterTest, TestBlacklistDownloading) {
   ExtensionUpdaterTest::TestBlacklistDownloading();
 }
 
-TEST(ExtensionUpdaterTest, TestMultipleExtensionDownloadingUpdatesFail) {
+TEST_F(ExtensionUpdaterTest, TestMultipleExtensionDownloadingUpdatesFail) {
   ExtensionUpdaterTest::TestMultipleExtensionDownloading(false);
 }
-TEST(ExtensionUpdaterTest, TestMultipleExtensionDownloadingUpdatesSucceed) {
+TEST_F(ExtensionUpdaterTest, TestMultipleExtensionDownloadingUpdatesSucceed) {
   ExtensionUpdaterTest::TestMultipleExtensionDownloading(true);
 }
 
-TEST(ExtensionUpdaterTest, TestGalleryRequests) {
+TEST_F(ExtensionUpdaterTest, TestGalleryRequests) {
   // We want to test a variety of combinations of expected ping conditions for
   // rollcall and active pings.
   int ping_cases[] = { ManifestFetchData::kNeverPinged, 0, 1, 5 };
@@ -1138,11 +1125,11 @@ TEST(ExtensionUpdaterTest, TestGalleryRequests) {
   }
 }
 
-TEST(ExtensionUpdaterTest, TestHandleManifestResults) {
+TEST_F(ExtensionUpdaterTest, TestHandleManifestResults) {
   ExtensionUpdaterTest::TestHandleManifestResults();
 }
 
-TEST(ExtensionUpdaterTest, TestManifestFetchesBuilderAddExtension) {
+TEST_F(ExtensionUpdaterTest, TestManifestFetchesBuilderAddExtension) {
   MessageLoop message_loop;
   BrowserThread file_thread(BrowserThread::FILE, &message_loop);
 
@@ -1191,14 +1178,13 @@ TEST(ExtensionUpdaterTest, TestManifestFetchesBuilderAddExtension) {
   EXPECT_FALSE(fetch->full_url().is_empty());
 }
 
-TEST(ExtensionUpdaterTest, TestStartUpdateCheckMemory) {
+TEST_F(ExtensionUpdaterTest, TestStartUpdateCheckMemory) {
     MessageLoop message_loop;
     BrowserThread ui_thread(BrowserThread::UI, &message_loop);
     BrowserThread file_thread(BrowserThread::FILE, &message_loop);
 
     ServiceForManifestTests service;
     TestURLFetcherFactory factory;
-    URLFetcher::set_factory(&factory);
     ExtensionUpdater updater(
         &service, service.extension_prefs(), service.pref_service(),
         service.profile(), kUpdateFrequencySecs);
@@ -1213,14 +1199,13 @@ TEST(ExtensionUpdaterTest, TestStartUpdateCheckMemory) {
     updater.Stop();
 }
 
-TEST(ExtensionUpdaterTest, TestCheckSoon) {
+TEST_F(ExtensionUpdaterTest, TestCheckSoon) {
     MessageLoop message_loop;
     BrowserThread ui_thread(BrowserThread::UI, &message_loop);
     BrowserThread file_thread(BrowserThread::FILE, &message_loop);
 
     ServiceForManifestTests service;
     TestURLFetcherFactory factory;
-    URLFetcher::set_factory(&factory);
     ExtensionUpdater updater(
         &service, service.extension_prefs(), service.pref_service(),
         service.profile(), kUpdateFrequencySecs);
