@@ -5,10 +5,11 @@
 #include "base/path_service.h"
 #include "chrome/browser/extensions/extension_apitest.h"
 #include "chrome/common/chrome_paths.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/test/base/ui_test_utils.h"
 #include "net/base/mock_host_resolver.h"
 
-IN_PROC_BROWSER_TEST_F(ExtensionApiTest, WebSocket) {
+// Disabled, http://crbug.com/91058.
+IN_PROC_BROWSER_TEST_F(ExtensionApiTest, DISABLED_WebSocket) {
   FilePath websocket_root_dir;
   PathService::Get(chrome::DIR_TEST_DATA, &websocket_root_dir);
   websocket_root_dir = websocket_root_dir.AppendASCII("layout_tests")
