@@ -8,9 +8,9 @@
 
 #include <string>
 
-#include "chrome/browser/browser_signin.h"
 #include "chrome/browser/extensions/extension_function.h"
 #include "chrome/browser/extensions/extension_install_ui.h"
+#include "chrome/browser/extensions/webstore_install_helper.h"
 #include "chrome/common/net/gaia/google_service_auth_error.h"
 #include "content/common/notification_observer.h"
 #include "content/common/notification_registrar.h"
@@ -22,10 +22,6 @@ class WebstorePrivateApi {
   // Allows you to set the ProfileSyncService the function will use for
   // testing purposes.
   static void SetTestingProfileSyncService(ProfileSyncService* service);
-
-  // Allows you to set the BrowserSignin the function will use for
-  // testing purposes.
-  static void SetTestingBrowserSignin(BrowserSignin* signin);
 };
 
 // TODO(asargent): this is being deprecated in favor of
@@ -40,8 +36,10 @@ class BeginInstallFunction : public SyncExtensionFunction {
   DECLARE_EXTENSION_FUNCTION_NAME("webstorePrivate.beginInstall");
 };
 
-class BeginInstallWithManifestFunction : public AsyncExtensionFunction,
-                                         public ExtensionInstallUI::Delegate {
+class BeginInstallWithManifestFunction
+    : public AsyncExtensionFunction,
+      public ExtensionInstallUI::Delegate,
+      public WebstoreInstallHelper::Delegate {
  public:
   BeginInstallWithManifestFunction();
 
@@ -71,25 +69,22 @@ class BeginInstallWithManifestFunction : public AsyncExtensionFunction,
 
     // The function was not called during a user gesture.
     NO_GESTURE,
+
+    // Invalid icon url.
+    INVALID_ICON_URL
   };
 
   // For use only in tests - sets a flag that can cause this function to ignore
   // the normal requirement that it is called during a user gesture.
   static void SetIgnoreUserGestureForTests(bool ignore);
 
-  // For use only in tests - sets a flag that makes invocations of
-  // beginInstallWithManifest skip putting up a real dialog, and instead act
-  // as if the dialog choice was to proceed or abort.
-  static void SetAutoConfirmForTests(bool should_proceed);
-
-  // Called when we've successfully parsed the manifest and icon data in the
-  // utility process. Ownership of parsed_manifest is transferred.
-  void OnParseSuccess(const SkBitmap& icon,
-                      base::DictionaryValue* parsed_manifest);
-
-  // Called to indicate a parse failure. The |result_code| parameter should
-  // indicate whether the problem was with the manifest or icon data.
-  void OnParseFailure(ResultCode result_code, const std::string& error_message);
+  // Implementing WebstoreInstallHelper::Delegate interface.
+  virtual void OnWebstoreParseSuccess(
+      const SkBitmap& icon,
+      base::DictionaryValue* parsed_manifest) OVERRIDE;
+  virtual void OnWebstoreParseFailure(
+      InstallHelperResultCode result_code,
+      const std::string& error_message) OVERRIDE;
 
   // Implementing ExtensionInstallUI::Delegate interface.
   virtual void InstallUIProceed() OVERRIDE;
@@ -108,6 +103,7 @@ class BeginInstallWithManifestFunction : public AsyncExtensionFunction,
   std::string manifest_;
   std::string icon_data_;
   std::string localized_name_;
+  bool use_app_installed_bubble_;
 
   // The results of parsing manifest_ and icon_data_ go into these two.
   scoped_ptr<base::DictionaryValue> parsed_manifest_;
@@ -116,6 +112,7 @@ class BeginInstallWithManifestFunction : public AsyncExtensionFunction,
   // A dummy Extension object we create for the purposes of using
   // ExtensionInstallUI to prompt for confirmation of the install.
   scoped_refptr<Extension> dummy_extension_;
+
   DECLARE_EXTENSION_FUNCTION_NAME("webstorePrivate.beginInstallWithManifest2");
 };
 
@@ -139,35 +136,4 @@ class SetStoreLoginFunction : public SyncExtensionFunction {
   DECLARE_EXTENSION_FUNCTION_NAME("webstorePrivate.setStoreLogin");
 };
 
-class PromptBrowserLoginFunction : public AsyncExtensionFunction,
-                                   public NotificationObserver,
-                                   public BrowserSignin::SigninDelegate {
- public:
-  PromptBrowserLoginFunction();
-  // Implements BrowserSignin::SigninDelegate interface.
-  virtual void OnLoginSuccess();
-  virtual void OnLoginFailure(const GoogleServiceAuthError& error);
-
-  // Implements the NotificationObserver interface.
-  virtual void Observe(int type,
-                       const NotificationSource& source,
-                       const NotificationDetails& details);
-
- protected:
-  virtual ~PromptBrowserLoginFunction();
-  virtual bool RunImpl();
-
- private:
-  // Creates the message for signing in.
-  virtual string16 GetLoginMessage();
-
-  // Are we waiting for a token available notification?
-  bool waiting_for_token_;
-
-  // Used for listening for TokenService notifications.
-  NotificationRegistrar registrar_;
-
-  DECLARE_EXTENSION_FUNCTION_NAME("webstorePrivate.promptBrowserLogin");
-};
-
 #endif  // CHROME_BROWSER_EXTENSIONS_EXTENSION_WEBSTORE_PRIVATE_API_H_
