@@ -19,7 +19,7 @@
 #include "chrome/common/extensions/extension.h"
 #include "chrome/common/extensions/extension_constants.h"
 #include "chrome/common/json_pref_store.h"
-#include "chrome/test/signaling_task.h"
+#include "chrome/test/base/signaling_task.h"
 #include "content/browser/browser_thread.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -101,20 +101,40 @@ scoped_refptr<Extension> TestExtensionPrefs::AddExtension(std::string name) {
   return AddExtensionWithManifest(dictionary, Extension::INTERNAL);
 }
 
+scoped_refptr<Extension> TestExtensionPrefs::AddApp(std::string name) {
+  DictionaryValue dictionary;
+  dictionary.SetString(extension_manifest_keys::kName, name);
+  dictionary.SetString(extension_manifest_keys::kVersion, "0.1");
+  dictionary.SetString(extension_manifest_keys::kApp, "true");
+  dictionary.SetString(extension_manifest_keys::kLaunchWebURL,
+                       "http://example.com");
+  return AddExtensionWithManifest(dictionary, Extension::INTERNAL);
+
+}
+
 scoped_refptr<Extension> TestExtensionPrefs::AddExtensionWithManifest(
     const DictionaryValue& manifest, Extension::Location location) {
+  return AddExtensionWithManifestAndFlags(manifest, location,
+                                          Extension::STRICT_ERROR_CHECKS);
+}
+
+scoped_refptr<Extension> TestExtensionPrefs::AddExtensionWithManifestAndFlags(
+    const DictionaryValue& manifest,
+    Extension::Location location,
+    int extra_flags) {
   std::string name;
   EXPECT_TRUE(manifest.GetString(extension_manifest_keys::kName, &name));
   FilePath path =  extensions_dir_.AppendASCII(name);
   std::string errors;
   scoped_refptr<Extension> extension = Extension::Create(
-      path, location, manifest, Extension::STRICT_ERROR_CHECKS, &errors);
-  EXPECT_TRUE(extension);
+      path, location, manifest, extra_flags, &errors);
+  EXPECT_TRUE(extension) << errors;
   if (!extension)
     return NULL;
 
   EXPECT_TRUE(Extension::IdIsValid(extension->id()));
-  prefs_->OnExtensionInstalled(extension, Extension::ENABLED, false);
+  prefs_->OnExtensionInstalled(extension, Extension::ENABLED,
+                               extra_flags & Extension::FROM_WEBSTORE, 0);
   return extension;
 }
 
