@@ -40,10 +40,20 @@ class TestExtensionPrefs {
   // our ExtensionPrefs, and returns it.
   scoped_refptr<Extension> AddExtension(std::string name);
 
+  // As above, but the extension is an app.
+  scoped_refptr<Extension> AddApp(std::string name);
+
   // Similar to AddExtension, but takes a dictionary with manifest values.
   scoped_refptr<Extension> AddExtensionWithManifest(
       const base::DictionaryValue& manifest, Extension::Location location);
 
+  // Similar to AddExtension, but takes a dictionary with manifest values
+  // and extension flags.
+  scoped_refptr<Extension> AddExtensionWithManifestAndFlags(
+      const base::DictionaryValue& manifest,
+      Extension::Location location,
+      int extra_flags);
+
   // Similar to AddExtension, this adds a new test Extension. This is useful for
   // cases when you don't need the Extension object, but just the id it was
   // assigned.
