@@ -10,9 +10,9 @@
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/extensions/extension_file_util.h"
 #include "content/browser/mock_resource_context.h"
+#include "content/browser/renderer_host/dummy_resource_handler.h"
 #include "content/browser/renderer_host/global_request_id.h"
 #include "content/browser/renderer_host/resource_dispatcher_host_request_info.h"
-#include "content/browser/renderer_host/resource_handler.h"
 #include "content/browser/renderer_host/resource_queue.h"
 #include "content/common/notification_service.h"
 #include "net/url_request/url_request.h"
@@ -22,76 +22,19 @@
 
 class Profile;
 
+using content::DummyResourceHandler;
+
 namespace {
 
 const char kMatchingUrl[] = "http://google.com/";
 const char kNotMatchingUrl[] = "http://example.com/";
 const char kTestData[] = "Hello, World!";
 
-// Dummy ResourceHandler required for ResourceDispatcherHostRequestInfo.
-class DummyResourceHandler : public ResourceHandler {
- public:
-  DummyResourceHandler() {
-  }
-
-  virtual bool OnUploadProgress(int request_id, uint64 position, uint64 size) {
-    NOTREACHED();
-    return true;
-  }
-
-
-  virtual bool OnRequestRedirected(int request_id, const GURL& url,
-                                   ResourceResponse* response,
-                                   bool* defer) {
-    NOTREACHED();
-    return true;
-  }
-
-  virtual bool OnResponseStarted(int request_id,
-                                 ResourceResponse* response) {
-    NOTREACHED();
-    return true;
-  }
-
-  virtual bool OnWillStart(int request_id,
-                           const GURL& url,
-                           bool* defer) {
-    NOTREACHED();
-    return true;
-  }
-
-  virtual bool OnWillRead(int request_id,
-                          net::IOBuffer** buf,
-                          int* buf_size,
-                          int min_size) {
-    NOTREACHED();
-    return true;
-  }
-
-  virtual bool OnReadCompleted(int request_id, int* bytes_read) {
-    NOTREACHED();
-    return true;
-  }
-
-  virtual bool OnResponseCompleted(int request_id,
-                                   const net::URLRequestStatus& status,
-                                   const std::string& security_info) {
-    NOTREACHED();
-    return true;
-  }
-
-  virtual void OnRequestClosed() {
-  }
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(DummyResourceHandler);
-};
-
 ResourceDispatcherHostRequestInfo* CreateRequestInfo(int request_id) {
   return new ResourceDispatcherHostRequestInfo(
       new DummyResourceHandler(), ChildProcessInfo::RENDER_PROCESS, 0, 0, 0,
-      request_id, false, -1, ResourceType::MAIN_FRAME, 0, false, false, false,
-      content::MockResourceContext::GetInstance());
+      request_id, false, -1, ResourceType::MAIN_FRAME, PageTransition::LINK, 0,
+      false, false, false, content::MockResourceContext::GetInstance());
 }
 
 // A simple test net::URLRequestJob. We don't care what it does, only that
@@ -168,7 +111,7 @@ class UserScriptListenerTest
   void UnloadTestExtension() {
     ASSERT_FALSE(service_->extensions()->empty());
     service_->UnloadExtension(service_->extensions()->at(0)->id(),
-                              UnloadedExtensionInfo::DISABLE);
+                              extension_misc::UNLOAD_REASON_DISABLE);
   }
 
   scoped_refptr<UserScriptListener> listener_;
