@@ -3,11 +3,13 @@
 // found in the LICENSE file.
 
 #include "chrome/browser/favicon/favicon_handler.h"
+#include "chrome/browser/profiles/profile.h"
 #include "content/browser/renderer_host/test_render_view_host.h"
 #include "content/browser/tab_contents/navigation_entry.h"
 #include "content/browser/tab_contents/test_tab_contents.h"
 #include "ui/gfx/codec/png_codec.h"
 #include "ui/gfx/favicon_size.h"
+#include "ui/gfx/image/image.h"
 
 class TestFaviconHandler;
 
@@ -121,17 +123,48 @@ class HistoryRequestHandler {
 
 }  // namespace
 
+
+// This class is used as a temporary hack to provide working implementations of
+// the various delegate methods.  Most of these methods are actually never
+// called.
+// TODO(rohitrao): Refactor the tests to override these delegate methods instead
+// of subclassing.
+class TestFaviconHandlerDelegate : public FaviconHandlerDelegate {
+ public:
+  explicit TestFaviconHandlerDelegate(TabContents* tab_contents)
+      : tab_contents_(tab_contents) {
+  }
+
+  virtual NavigationEntry* GetActiveEntry() {
+    ADD_FAILURE() << "TestFaviconHandlerDelegate::GetActiveEntry() "
+                  << "should never be called in tests.";
+    return NULL;
+  }
+
+  virtual void StartDownload(int id, const GURL& url, int image_size) {
+    ADD_FAILURE() << "TestFaviconHandlerDelegate::StartDownload() "
+                  << "should never be called in tests.";
+  }
+
+  virtual void NotifyFaviconUpdated() {
+    tab_contents_->NotifyNavigationStateChanged(TabContents::INVALIDATE_TAB);
+  }
+
+ private:
+  TabContents* tab_contents_;  // weak
+};
+
 // This class is used to catch the FaviconHandler's download and history
 // request, and also provide the methods to access the FaviconHandler internal.
 class TestFaviconHandler : public FaviconHandler {
  public:
   TestFaviconHandler(const GURL& page_url,
-                    TabContents* tab_contents,
-                    Type type)
-    : FaviconHandler(tab_contents, type),
-      download_image_size_(0),
-      download_id_(0),
-      tab_contents_(tab_contents){
+                     Profile* profile,
+                     FaviconHandlerDelegate* delegate,
+                     Type type)
+      : FaviconHandler(profile, delegate, type),
+        download_image_size_(0),
+        download_id_(0) {
     entry_.set_url(page_url);
   }
 
@@ -176,7 +209,7 @@ class TestFaviconHandler : public FaviconHandler {
   void OnDidDownloadFavicon(int id,
                             const GURL& image_url,
                             bool errored,
-                            const SkBitmap& image) {
+                            gfx::Image& image) {
     FaviconHandler::OnDidDownloadFavicon(id, image_url, errored, image);
   }
 
@@ -245,7 +278,6 @@ class TestFaviconHandler : public FaviconHandler {
   // FaviconHandler.
   int download_id_;
 
-  TabContents* tab_contents_;
   scoped_ptr<DownloadHandler> download_handler_;
   scoped_ptr<HistoryRequestHandler> history_handler_;
 
@@ -264,8 +296,9 @@ void DownloadHandler::UpdateFaviconURL(const std::vector<FaviconURL> urls) {
 }
 
 void DownloadHandler::InvokeCallback() {
+  gfx::Image image(new SkBitmap(bitmap_));
   favicon_helper_->OnDidDownloadFavicon(download_id_, image_url_, failed_,
-                                        bitmap_);
+                                        image);
 }
 
 void HistoryRequestHandler::InvokeCallback() {
@@ -279,7 +312,10 @@ TEST_F(FaviconHandlerTest, GetFaviconFromHistory) {
   const GURL page_url("http://www.google.com");
   const GURL icon_url("http://www.google.com/favicon");
 
-  TestFaviconHandler helper(page_url, contents(), FaviconHandler::FAVICON);
+  TestFaviconHandlerDelegate delegate(contents());
+  Profile* profile = Profile::FromBrowserContext(contents()->browser_context());
+  TestFaviconHandler helper(page_url, profile,
+                            &delegate, FaviconHandler::FAVICON);
 
   helper.FetchFavicon(page_url);
   HistoryRequestHandler* history_handler = helper.history_handler();
@@ -295,7 +331,7 @@ TEST_F(FaviconHandlerTest, GetFaviconFromHistory) {
   history_handler->favicon_data_.expired = false;
   history_handler->favicon_data_.icon_url = icon_url;
   scoped_refptr<RefCountedBytes> data = new RefCountedBytes();
-  FillBitmap(kFaviconSize, kFaviconSize, &data->data);
+  FillBitmap(kFaviconSize, kFaviconSize, &data->data());
   history_handler->favicon_data_.image_data = data;
 
   // Send history response.
@@ -324,7 +360,10 @@ TEST_F(FaviconHandlerTest, DownloadFavicon) {
   const GURL page_url("http://www.google.com");
   const GURL icon_url("http://www.google.com/favicon");
 
-  TestFaviconHandler helper(page_url, contents(), FaviconHandler::FAVICON);
+  TestFaviconHandlerDelegate delegate(contents());
+  Profile* profile = Profile::FromBrowserContext(contents()->browser_context());
+  TestFaviconHandler helper(page_url, profile,
+                            &delegate, FaviconHandler::FAVICON);
 
   helper.FetchFavicon(page_url);
   HistoryRequestHandler* history_handler = helper.history_handler();
@@ -388,7 +427,10 @@ TEST_F(FaviconHandlerTest, UpdateAndDownloadFavicon) {
   const GURL icon_url("http://www.google.com/favicon");
   const GURL new_icon_url("http://www.google.com/new_favicon");
 
-  TestFaviconHandler helper(page_url, contents(), FaviconHandler::FAVICON);
+  TestFaviconHandlerDelegate delegate(contents());
+  Profile* profile = Profile::FromBrowserContext(contents()->browser_context());
+  TestFaviconHandler helper(page_url, profile,
+                            &delegate, FaviconHandler::FAVICON);
 
   helper.FetchFavicon(page_url);
   HistoryRequestHandler* history_handler = helper.history_handler();
@@ -404,7 +446,7 @@ TEST_F(FaviconHandlerTest, UpdateAndDownloadFavicon) {
   history_handler->favicon_data_.expired = false;
   history_handler->favicon_data_.icon_url = icon_url;
   scoped_refptr<RefCountedBytes> data = new RefCountedBytes();
-  FillBitmap(kFaviconSize, kFaviconSize, &data->data);
+  FillBitmap(kFaviconSize, kFaviconSize, &data->data());
   history_handler->favicon_data_.image_data = data;
 
   // Send history response.
@@ -473,7 +515,10 @@ TEST_F(FaviconHandlerTest, UpdateFavicon) {
   const GURL icon_url("http://www.google.com/favicon");
   const GURL new_icon_url("http://www.google.com/new_favicon");
 
-  TestFaviconHandler helper(page_url, contents(), FaviconHandler::FAVICON);
+  TestFaviconHandlerDelegate delegate(contents());
+  Profile* profile = Profile::FromBrowserContext(contents()->browser_context());
+  TestFaviconHandler helper(page_url, profile,
+                            &delegate, FaviconHandler::FAVICON);
 
   helper.FetchFavicon(page_url);
   HistoryRequestHandler* history_handler = helper.history_handler();
@@ -489,7 +534,7 @@ TEST_F(FaviconHandlerTest, UpdateFavicon) {
   history_handler->favicon_data_.expired = false;
   history_handler->favicon_data_.icon_url = icon_url;
   scoped_refptr<RefCountedBytes> data = new RefCountedBytes();
-  FillBitmap(kFaviconSize, kFaviconSize, &data->data);
+  FillBitmap(kFaviconSize, kFaviconSize, &data->data());
   history_handler->favicon_data_.image_data = data;
 
   // Send history response.
@@ -544,7 +589,10 @@ TEST_F(FaviconHandlerTest, Download2ndFaviconURLCandidate) {
   const GURL icon_url("http://www.google.com/favicon");
   const GURL new_icon_url("http://www.google.com/new_favicon");
 
-  TestFaviconHandler helper(page_url, contents(), FaviconHandler::TOUCH);
+  TestFaviconHandlerDelegate delegate(contents());
+  Profile* profile = Profile::FromBrowserContext(contents()->browser_context());
+  TestFaviconHandler helper(page_url, profile,
+                            &delegate, FaviconHandler::TOUCH);
 
   helper.FetchFavicon(page_url);
   HistoryRequestHandler* history_handler = helper.history_handler();
@@ -629,7 +677,7 @@ TEST_F(FaviconHandlerTest, Download2ndFaviconURLCandidate) {
   history_handler->favicon_data_.expired = true;
   history_handler->favicon_data_.icon_url = new_icon_url;
   scoped_refptr<RefCountedBytes> data = new RefCountedBytes();
-  FillBitmap(kFaviconSize, kFaviconSize, &data->data);
+  FillBitmap(kFaviconSize, kFaviconSize, &data->data());
   history_handler->favicon_data_.image_data = data;
   history_handler->InvokeCallback();
 
@@ -658,7 +706,10 @@ TEST_F(FaviconHandlerTest, UpdateDuringDownloading) {
   const GURL icon_url("http://www.google.com/favicon");
   const GURL new_icon_url("http://www.google.com/new_favicon");
 
-  TestFaviconHandler helper(page_url, contents(), FaviconHandler::TOUCH);
+  TestFaviconHandlerDelegate delegate(contents());
+  Profile* profile = Profile::FromBrowserContext(contents()->browser_context());
+  TestFaviconHandler helper(page_url, profile,
+                            &delegate, FaviconHandler::TOUCH);
 
   helper.FetchFavicon(page_url);
   HistoryRequestHandler* history_handler = helper.history_handler();
@@ -756,7 +807,7 @@ TEST_F(FaviconHandlerTest, UpdateDuringDownloading) {
   handler->favicon_data_.icon_type = history::TOUCH_ICON;
   handler->favicon_data_.icon_url = latest_icon_url;
   scoped_refptr<RefCountedBytes> data = new RefCountedBytes();
-  FillBitmap(kFaviconSize, kFaviconSize, &data->data);
+  FillBitmap(kFaviconSize, kFaviconSize, &data->data());
   handler->favicon_data_.image_data = data;
 
   handler->InvokeCallback();
