@@ -2,22 +2,28 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <string>
+
 #include "base/compiler_specific.h"
 #include "base/stringprintf.h"
 #include "base/string_number_conversions.h"
 #include "base/utf_string_conversions.h"
+#include "chrome/browser/content_settings/content_settings_pattern.h"
+#include "chrome/browser/content_settings/host_content_settings_map.h"
 #include "chrome/browser/content_settings/tab_specific_content_settings.h"
 #include "chrome/browser/dom_operation_notification_details.h"
-#include "chrome/browser/geolocation/geolocation_content_settings_map.h"
 #include "chrome/browser/geolocation/geolocation_settings_state.h"
+#include "chrome/browser/infobars/infobar_tab_helper.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/tab_contents/confirm_infobar_delegate.h"
+#include "chrome/browser/tab_contents/infobar.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_list.h"
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/chrome_paths.h"
-#include "chrome/test/in_process_browser_test.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "chrome/test/base/ui_test_utils.h"
 #include "content/browser/geolocation/arbitrator_dependency_factories_for_test.h"
 #include "content/browser/geolocation/location_arbitrator.h"
 #include "content/browser/geolocation/mock_location_provider.h"
@@ -26,7 +32,6 @@
 #include "content/common/geoposition.h"
 #include "content/common/notification_details.h"
 #include "content/common/notification_service.h"
-#include "content/common/notification_type.h"
 #include "net/base/net_util.h"
 #include "net/test/test_server.h"
 
@@ -43,9 +48,9 @@ class IFrameLoader : public NotificationObserver {
         javascript_completed_(false) {
     NavigationController* controller =
         &browser->GetSelectedTabContents()->controller();
-    registrar_.Add(this, NotificationType::LOAD_STOP,
+    registrar_.Add(this, content::NOTIFICATION_LOAD_STOP,
                    Source<NavigationController>(controller));
-    registrar_.Add(this, NotificationType::DOM_OPERATION_RESPONSE,
+    registrar_.Add(this, chrome::NOTIFICATION_DOM_OPERATION_RESPONSE,
                    NotificationService::AllSources());
     std::string script = base::StringPrintf(
         "window.domAutomationController.setAutomationId(0);"
@@ -70,12 +75,12 @@ class IFrameLoader : public NotificationObserver {
 
   GURL iframe_url() const { return iframe_url_; }
 
-  virtual void Observe(NotificationType type,
+  virtual void Observe(int type,
                        const NotificationSource& source,
                        const NotificationDetails& details) {
-    if (type == NotificationType::LOAD_STOP) {
+    if (type == content::NOTIFICATION_LOAD_STOP) {
       navigation_completed_ = true;
-    } else if (type == NotificationType::DOM_OPERATION_RESPONSE) {
+    } else if (type == chrome::NOTIFICATION_DOM_OPERATION_RESPONSE) {
       Details<DomOperationNotificationDetails> dom_op_details(details);
       javascript_response_ = dom_op_details->json();
       javascript_completed_ = true;
@@ -111,17 +116,17 @@ class GeolocationNotificationObserver : public NotificationObserver {
       infobar_(NULL),
       navigation_started_(false),
       navigation_completed_(false) {
-    registrar_.Add(this, NotificationType::DOM_OPERATION_RESPONSE,
+    registrar_.Add(this, chrome::NOTIFICATION_DOM_OPERATION_RESPONSE,
                    NotificationService::AllSources());
     if (wait_for_infobar) {
-      registrar_.Add(this, NotificationType::TAB_CONTENTS_INFOBAR_ADDED,
+      registrar_.Add(this, chrome::NOTIFICATION_TAB_CONTENTS_INFOBAR_ADDED,
                      NotificationService::AllSources());
     } else {
-      registrar_.Add(this, NotificationType::NAV_ENTRY_COMMITTED,
+      registrar_.Add(this, content::NOTIFICATION_NAV_ENTRY_COMMITTED,
                      NotificationService::AllSources());
-      registrar_.Add(this, NotificationType::LOAD_START,
+      registrar_.Add(this, content::NOTIFICATION_LOAD_START,
                      NotificationService::AllSources());
-      registrar_.Add(this, NotificationType::LOAD_STOP,
+      registrar_.Add(this, content::NOTIFICATION_LOAD_STOP,
                      NotificationService::AllSources());
     }
   }
@@ -146,21 +151,21 @@ class GeolocationNotificationObserver : public NotificationObserver {
   }
 
   // NotificationObserver
-  virtual void Observe(NotificationType type,
+  virtual void Observe(int type,
                        const NotificationSource& source,
                        const NotificationDetails& details) {
-    if (type.value == NotificationType::TAB_CONTENTS_INFOBAR_ADDED) {
-      infobar_ = Details<InfoBarDelegate>(details).ptr();
+    if (type == chrome::NOTIFICATION_TAB_CONTENTS_INFOBAR_ADDED) {
+      infobar_ = Details<InfoBarAddedDetails>(details).ptr();
       ASSERT_TRUE(infobar_->GetIcon());
       ASSERT_TRUE(infobar_->AsConfirmInfoBarDelegate());
-    } else if (type == NotificationType::DOM_OPERATION_RESPONSE) {
+    } else if (type == chrome::NOTIFICATION_DOM_OPERATION_RESPONSE) {
       Details<DomOperationNotificationDetails> dom_op_details(details);
       javascript_response_ = dom_op_details->json();
       LOG(WARNING) << "javascript_response " << javascript_response_;
-    } else if (type == NotificationType::NAV_ENTRY_COMMITTED ||
-               type == NotificationType::LOAD_START) {
+    } else if (type == content::NOTIFICATION_NAV_ENTRY_COMMITTED ||
+               type == content::NOTIFICATION_LOAD_START) {
       navigation_started_ = true;
-    } else if (type == NotificationType::LOAD_STOP) {
+    } else if (type == content::NOTIFICATION_LOAD_STOP) {
       if (navigation_started_) {
         navigation_started_ = false;
         navigation_completed_ = true;
@@ -316,12 +321,18 @@ class GeolocationBrowserTest : public InProcessBrowserTest {
     size_t state_map_size = settings_state.state_map().size();
     ASSERT_TRUE(infobar_);
     LOG(WARNING) << "will set infobar response";
-    if (allowed)
-      infobar_->AsConfirmInfoBarDelegate()->Accept();
-    else
-      infobar_->AsConfirmInfoBarDelegate()->Cancel();
-    WaitForNavigation();
-    tab_contents_wrapper->RemoveInfoBar(infobar_);
+    {
+      ui_test_utils::WindowedNotificationObserver observer(
+          content::NOTIFICATION_LOAD_STOP,
+          Source<NavigationController>(&tab_contents_wrapper->controller()));
+      if (allowed)
+        infobar_->AsConfirmInfoBarDelegate()->Accept();
+      else
+        infobar_->AsConfirmInfoBarDelegate()->Cancel();
+      observer.Wait();
+    }
+
+    tab_contents_wrapper->infobar_tab_helper()->RemoveInfoBar(infobar_);
     LOG(WARNING) << "infobar response set";
     infobar_ = NULL;
     EXPECT_GT(settings_state.state_map().size(), state_map_size);
@@ -400,7 +411,7 @@ IN_PROC_BROWSER_TEST_F(GeolocationBrowserTest,
 }
 
 // http://crbug.com/44589. Hangs on Mac, crashes on Windows
-#if defined(OS_MACOSX) || defined(OS_WINDOWS)
+#if defined(OS_MACOSX) || defined(OS_WIN)
 #define MAYBE_NoInfobarForSecondTab DISABLED_NoInfobarForSecondTab
 #else
 #define MAYBE_NoInfobarForSecondTab NoInfobarForSecondTab
@@ -419,15 +430,19 @@ IN_PROC_BROWSER_TEST_F(GeolocationBrowserTest, MAYBE_NoInfobarForSecondTab) {
 }
 
 // http://crbug.com/44589. Hangs on Mac, crashes on Windows
-#if defined(OS_MACOSX) || defined(OS_WINDOWS)
+#if defined(OS_MACOSX) || defined(OS_WIN)
 #define MAYBE_NoInfobarForDeniedOrigin DISABLED_NoInfobarForDeniedOrigin
 #else
 #define MAYBE_NoInfobarForDeniedOrigin NoInfobarForDeniedOrigin
 #endif
 IN_PROC_BROWSER_TEST_F(GeolocationBrowserTest, MAYBE_NoInfobarForDeniedOrigin) {
   ASSERT_TRUE(Initialize(INITIALIZATION_NONE));
-  current_browser_->profile()->GetGeolocationContentSettingsMap()->
-      SetContentSetting(current_url_, current_url_, CONTENT_SETTING_BLOCK);
+  current_browser_->profile()->GetHostContentSettingsMap()->
+      SetContentSetting(ContentSettingsPattern::FromURLNoWildcard(current_url_),
+                        ContentSettingsPattern::FromURLNoWildcard(current_url_),
+                        CONTENT_SETTINGS_TYPE_GEOLOCATION,
+                        std::string(),
+                        CONTENT_SETTING_BLOCK);
   AddGeolocationWatch(false);
   // Checks we have an error for this denied origin.
   CheckStringValueFromJavascript("1", "geoGetLastError()");
@@ -439,8 +454,12 @@ IN_PROC_BROWSER_TEST_F(GeolocationBrowserTest, MAYBE_NoInfobarForDeniedOrigin) {
 
 IN_PROC_BROWSER_TEST_F(GeolocationBrowserTest, NoInfobarForAllowedOrigin) {
   ASSERT_TRUE(Initialize(INITIALIZATION_NONE));
-  current_browser_->profile()->GetGeolocationContentSettingsMap()->
-      SetContentSetting(current_url_, current_url_, CONTENT_SETTING_ALLOW);
+  current_browser_->profile()->GetHostContentSettingsMap()->
+      SetContentSetting(ContentSettingsPattern::FromURLNoWildcard(current_url_),
+                        ContentSettingsPattern::FromURLNoWildcard(current_url_),
+                        CONTENT_SETTINGS_TYPE_GEOLOCATION,
+                        std::string(),
+                        CONTENT_SETTING_ALLOW);
   // Checks no infobar will be created and there's no error callback.
   AddGeolocationWatch(false);
   CheckGeoposition(MockLocationProvider::instance_->position_);
@@ -461,7 +480,9 @@ IN_PROC_BROWSER_TEST_F(GeolocationBrowserTest, NoInfobarForOffTheRecord) {
   CheckGeoposition(MockLocationProvider::instance_->position_);
 }
 
-IN_PROC_BROWSER_TEST_F(GeolocationBrowserTest, IFramesWithFreshPosition) {
+// Test fails: http://crbug.com/90927
+IN_PROC_BROWSER_TEST_F(GeolocationBrowserTest,
+                       DISABLED_IFramesWithFreshPosition) {
   html_for_tests_ = "files/geolocation/iframes_different_origin.html";
   ASSERT_TRUE(Initialize(INITIALIZATION_IFRAMES));
   LoadIFrames(2);
@@ -501,7 +522,9 @@ IN_PROC_BROWSER_TEST_F(GeolocationBrowserTest, IFramesWithFreshPosition) {
   LOG(WARNING) << "...done.";
 }
 
-IN_PROC_BROWSER_TEST_F(GeolocationBrowserTest, IFramesWithCachedPosition) {
+// Test fails: http://crbug.com/90927
+IN_PROC_BROWSER_TEST_F(GeolocationBrowserTest,
+                       DISABLED_IFramesWithCachedPosition) {
   html_for_tests_ = "files/geolocation/iframes_different_origin.html";
   ASSERT_TRUE(Initialize(INITIALIZATION_IFRAMES));
   LoadIFrames(2);
@@ -554,12 +577,12 @@ IN_PROC_BROWSER_TEST_F(GeolocationBrowserTest,
   iframe_xpath_ = L"//iframe[@id='iframe_1']";
   AddGeolocationWatch(true);
 
-  size_t num_infobars_before_cancel =
-      current_browser_->GetSelectedTabContentsWrapper()->infobar_count();
+  InfoBarTabHelper* infobar_helper = current_browser_->
+      GetSelectedTabContentsWrapper()->infobar_tab_helper();
+  size_t num_infobars_before_cancel = infobar_helper->infobar_count();
   // Change the iframe, and ensure the infobar is gone.
   IFrameLoader change_iframe_1(current_browser_, 1, current_url_);
-  size_t num_infobars_after_cancel =
-      current_browser_->GetSelectedTabContentsWrapper()->infobar_count();
+  size_t num_infobars_after_cancel = infobar_helper->infobar_count();
   EXPECT_EQ(num_infobars_before_cancel, num_infobars_after_cancel + 1);
 }
 
