@@ -34,8 +34,6 @@
 #include "base/string_util.h"
 #include "base/task.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/download/download_item.h"
-#include "chrome/browser/history/download_history_info.h"
 #include "chrome/browser/history/history.h"
 #include "chrome/browser/history/history_backend.h"
 #include "chrome/browser/history/history_database.h"
@@ -46,6 +44,8 @@
 #include "chrome/common/chrome_paths.h"
 #include "chrome/common/thumbnail_score.h"
 #include "chrome/tools/profiles/thumbnail-inl.h"
+#include "content/browser/download/download_item.h"
+#include "content/browser/download/download_persistent_store_info.h"
 #include "content/common/notification_details.h"
 #include "content/common/notification_source.h"
 #include "sql/connection.h"
@@ -184,14 +184,15 @@ class HistoryTest : public testing::Test {
   }
 
   int64 AddDownload(int32 state, const Time& time) {
-    DownloadHistoryInfo download(FilePath(FILE_PATH_LITERAL("foo-path")),
-                                 GURL("foo-url"),
-                                 GURL(""),
-                                 time,
-                                 0,
-                                 512,
-                                 state,
-                                 0);
+    DownloadPersistentStoreInfo download(
+        FilePath(FILE_PATH_LITERAL("foo-path")),
+        GURL("foo-url"),
+        GURL(""),
+        time,
+        0,
+        512,
+        state,
+        0);
     return db_->CreateDownload(download);
   }
 
@@ -310,7 +311,7 @@ TEST_F(HistoryTest, ClearBrowsingData_Downloads) {
   Time month_ago = now - TimeDelta::FromDays(30);
 
   // Initially there should be nothing in the downloads database.
-  std::vector<DownloadHistoryInfo> downloads;
+  std::vector<DownloadPersistentStoreInfo> downloads;
   db_->QueryDownloads(&downloads);
   EXPECT_EQ(0U, downloads.size());
 
@@ -461,8 +462,8 @@ TEST_F(HistoryTest, AddRedirect) {
   ASSERT_TRUE(history->Init(history_dir_, NULL));
 
   const char* first_sequence[] = {
-    "http://first.page/",
-    "http://second.page/"};
+    "http://first.page.com/",
+    "http://second.page.com/"};
   int first_count = arraysize(first_sequence);
   history::RedirectList first_redirects;
   for (int i = 0; i < first_count; i++)
@@ -505,7 +506,7 @@ TEST_F(HistoryTest, AddRedirect) {
   // so we pass in a CLIENT_REDIRECT qualifier to mock that behavior.
   history::RedirectList second_redirects;
   second_redirects.push_back(first_redirects[1]);
-  second_redirects.push_back(GURL("http://last.page/"));
+  second_redirects.push_back(GURL("http://last.page.com/"));
   history->AddPage(second_redirects[1], MakeFakeHost(1), 1,
                    second_redirects[0],
                    static_cast<PageTransition::Type>(PageTransition::LINK |
@@ -527,6 +528,81 @@ TEST_F(HistoryTest, AddRedirect) {
   EXPECT_EQ(second_visit, query_url_visits_[0].referring_visit);
 }
 
+TEST_F(HistoryTest, MakeIntranetURLsTyped) {
+  scoped_refptr<HistoryService> history(new HistoryService);
+  history_service_ = history;
+  ASSERT_TRUE(history->Init(history_dir_, NULL));
+
+  // Add a non-typed visit to an intranet URL on an unvisited host.  This should
+  // get promoted to a typed visit.
+  const GURL test_url("http://intranet_host/path");
+  history->AddPage(test_url, NULL, 0, GURL(), PageTransition::LINK,
+                   history::RedirectList(), history::SOURCE_BROWSED, false);
+  EXPECT_TRUE(QueryURL(history, test_url));
+  EXPECT_EQ(1, query_url_row_.visit_count());
+  EXPECT_EQ(1, query_url_row_.typed_count());
+  ASSERT_EQ(1U, query_url_visits_.size());
+  EXPECT_EQ(PageTransition::TYPED,
+            PageTransition::StripQualifier(query_url_visits_[0].transition));
+
+  // Add more visits on the same host.  None of these should be promoted since
+  // there is already a typed visit.
+
+  // Different path.
+  const GURL test_url2("http://intranet_host/different_path");
+  history->AddPage(test_url2, NULL, 0, GURL(), PageTransition::LINK,
+                   history::RedirectList(), history::SOURCE_BROWSED, false);
+  EXPECT_TRUE(QueryURL(history, test_url2));
+  EXPECT_EQ(1, query_url_row_.visit_count());
+  EXPECT_EQ(0, query_url_row_.typed_count());
+  ASSERT_EQ(1U, query_url_visits_.size());
+  EXPECT_EQ(PageTransition::LINK,
+            PageTransition::StripQualifier(query_url_visits_[0].transition));
+
+  // No path.
+  const GURL test_url3("http://intranet_host/");
+  history->AddPage(test_url3, NULL, 0, GURL(), PageTransition::LINK,
+                   history::RedirectList(), history::SOURCE_BROWSED, false);
+  EXPECT_TRUE(QueryURL(history, test_url3));
+  EXPECT_EQ(1, query_url_row_.visit_count());
+  EXPECT_EQ(0, query_url_row_.typed_count());
+  ASSERT_EQ(1U, query_url_visits_.size());
+  EXPECT_EQ(PageTransition::LINK,
+            PageTransition::StripQualifier(query_url_visits_[0].transition));
+
+  // Different scheme.
+  const GURL test_url4("https://intranet_host/");
+  history->AddPage(test_url4, NULL, 0, GURL(), PageTransition::LINK,
+                   history::RedirectList(), history::SOURCE_BROWSED, false);
+  EXPECT_TRUE(QueryURL(history, test_url4));
+  EXPECT_EQ(1, query_url_row_.visit_count());
+  EXPECT_EQ(0, query_url_row_.typed_count());
+  ASSERT_EQ(1U, query_url_visits_.size());
+  EXPECT_EQ(PageTransition::LINK,
+            PageTransition::StripQualifier(query_url_visits_[0].transition));
+
+  // Different transition.
+  const GURL test_url5("http://intranet_host/another_path");
+  history->AddPage(test_url5, NULL, 0, GURL(), PageTransition::AUTO_BOOKMARK,
+                   history::RedirectList(), history::SOURCE_BROWSED, false);
+  EXPECT_TRUE(QueryURL(history, test_url5));
+  EXPECT_EQ(1, query_url_row_.visit_count());
+  EXPECT_EQ(0, query_url_row_.typed_count());
+  ASSERT_EQ(1U, query_url_visits_.size());
+  EXPECT_EQ(PageTransition::AUTO_BOOKMARK,
+            PageTransition::StripQualifier(query_url_visits_[0].transition));
+
+  // Original URL.
+  history->AddPage(test_url, NULL, 0, GURL(), PageTransition::LINK,
+                   history::RedirectList(), history::SOURCE_BROWSED, false);
+  EXPECT_TRUE(QueryURL(history, test_url));
+  EXPECT_EQ(2, query_url_row_.visit_count());
+  EXPECT_EQ(1, query_url_row_.typed_count());
+  ASSERT_EQ(2U, query_url_visits_.size());
+  EXPECT_EQ(PageTransition::LINK,
+            PageTransition::StripQualifier(query_url_visits_[1].transition));
+}
+
 TEST_F(HistoryTest, Typed) {
   scoped_refptr<HistoryService> history(new HistoryService);
   history_service_ = history;
