@@ -209,7 +209,8 @@ void Toolbar5Importer::GetAuthenticationFromServer() {
   GURL url(url_string);
 
   token_fetcher_ = new  URLFetcher(url, URLFetcher::GET, this);
-  token_fetcher_->set_request_context(Profile::GetDefaultRequestContext());
+  token_fetcher_->set_request_context(
+      Profile::Deprecated::GetDefaultRequestContext());
   token_fetcher_->Start();
 }
 
@@ -242,7 +243,8 @@ void Toolbar5Importer::GetBookmarkDataFromServer(const std::string& response) {
   GURL url(conn_string);
 
   data_fetcher_ = new URLFetcher(url, URLFetcher::GET, this);
-  data_fetcher_->set_request_context(Profile::GetDefaultRequestContext());
+  data_fetcher_->set_request_context(
+      Profile::Deprecated::GetDefaultRequestContext());
   data_fetcher_->Start();
 }
 
