@@ -10,13 +10,15 @@
 #include "base/debug/leak_tracker.h"
 #include "base/logging.h"
 #include "base/metrics/field_trial.h"
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
 #include "base/string_number_conversions.h"
 #include "base/string_split.h"
 #include "base/string_util.h"
 #include "base/threading/thread_restrictions.h"
+#include "build/build_config.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/extensions/extension_event_router_forwarder.h"
+#include "chrome/browser/media/media_internals.h"
 #include "chrome/browser/net/chrome_network_delegate.h"
 #include "chrome/browser/net/chrome_net_log.h"
 #include "chrome/browser/net/chrome_url_request_context.h"
@@ -34,12 +36,15 @@
 #include "content/common/url_fetcher.h"
 #include "net/base/cert_verifier.h"
 #include "net/base/cookie_monster.h"
+#include "net/base/default_origin_bound_cert_store.h"
 #include "net/base/dnsrr_resolver.h"
 #include "net/base/host_cache.h"
 #include "net/base/host_resolver.h"
 #include "net/base/host_resolver_impl.h"
 #include "net/base/mapped_host_resolver.h"
 #include "net/base/net_util.h"
+#include "net/base/origin_bound_cert_service.h"
+#include "net/dns/async_host_resolver.h"
 #include "net/ftp/ftp_network_layer.h"
 #include "net/http/http_auth_filter.h"
 #include "net/http/http_auth_handler_factory.h"
@@ -158,8 +163,24 @@ net::HostResolver* CreateGlobalHostResolver(net::NetLog* net_log) {
     }
   }
 
-  net::HostResolver* global_host_resolver =
+  net::HostResolver* global_host_resolver = NULL;
+  if (command_line.HasSwitch(switches::kDnsServer)) {
+    std::string dns_ip_string =
+      command_line.GetSwitchValueASCII(switches::kDnsServer);
+    net::IPAddressNumber dns_ip_number;
+    if (net::ParseIPLiteralToNumber(dns_ip_string, &dns_ip_number)) {
+      global_host_resolver =
+        net::CreateAsyncHostResolver(parallelism, dns_ip_number, net_log);
+    } else {
+      LOG(ERROR) << "Invalid IP address specified for --dns-server: "
+                 << dns_ip_string;
+    }
+  }
+
+  if (!global_host_resolver) {
+    global_host_resolver =
       net::CreateSystemHostResolver(parallelism, retry_attempts, net_log);
+  }
 
   // Determine if we should disable IPv6 support.
   if (!command_line.HasSwitch(switches::kEnableIPv6)) {
@@ -234,8 +255,9 @@ ConstructProxyScriptFetcherContext(IOThread::Globals* globals,
       globals->proxy_script_fetcher_http_transaction_factory.get());
   context->set_ftp_transaction_factory(
       globals->proxy_script_fetcher_ftp_transaction_factory.get());
-  // In-memory cookie store.
-  context->set_cookie_store(new net::CookieMonster(NULL, NULL));
+  context->set_cookie_store(globals->system_cookie_store.get());
+  context->set_origin_bound_cert_service(
+      globals->system_origin_bound_cert_service.get());
   context->set_network_delegate(globals->system_network_delegate.get());
   return context;
 }
@@ -256,8 +278,9 @@ ConstructSystemRequestContext(IOThread::Globals* globals,
       globals->system_http_transaction_factory.get());
   context->set_ftp_transaction_factory(
       globals->system_ftp_transaction_factory.get());
-  // In-memory cookie store.
-  context->set_cookie_store(new net::CookieMonster(NULL, NULL));
+  context->set_cookie_store(globals->system_cookie_store.get());
+  context->set_origin_bound_cert_service(
+      globals->system_origin_bound_cert_service.get());
   return context;
 }
 
@@ -289,9 +312,7 @@ SystemURLRequestContextGetter::~SystemURLRequestContextGetter() {}
 
 net::URLRequestContext* SystemURLRequestContextGetter::GetURLRequestContext() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
-
-  if (!io_thread_->globals()->system_request_context)
-    io_thread_->InitSystemRequestContext();
+  DCHECK(io_thread_->globals()->system_request_context);
 
   return io_thread_->globals()->system_request_context;
 }
@@ -309,6 +330,10 @@ IOThread::Globals::Globals() {}
 
 IOThread::Globals::~Globals() {}
 
+IOThread::Globals::MediaGlobals::MediaGlobals() {}
+
+IOThread::Globals::MediaGlobals::~MediaGlobals() {}
+
 // |local_state| is passed in explicitly in order to (1) reduce implicit
 // dependencies and (2) make IOThread more flexible for testing.
 IOThread::IOThread(
@@ -320,7 +345,8 @@ IOThread::IOThread(
       extension_event_router_forwarder_(extension_event_router_forwarder),
       globals_(NULL),
       speculative_interceptor_(NULL),
-      predictor_(NULL) {
+      predictor_(NULL),
+      ALLOW_THIS_IN_INITIALIZER_LIST(method_factory_(this)) {
   // We call RegisterPrefs() here (instead of inside browser_prefs.cc) to make
   // sure that everything is initialized in the right order.
   RegisterPrefs(local_state);
@@ -338,6 +364,9 @@ IOThread::IOThread(
                                                     local_state);
   ssl_config_service_manager_.reset(
       SSLConfigServiceManager::CreateDefaultManager(local_state));
+  MessageLoop::current()->PostTask(FROM_HERE,
+                                   method_factory_.NewRunnableMethod(
+                                       &IOThread::InitSystemRequestContext));
 }
 
 IOThread::~IOThread() {
@@ -376,30 +405,6 @@ void IOThread::InitNetworkPredictor(
           startup_urls, referral_list, preconnect_enabled));
 }
 
-void IOThread::RegisterURLRequestContextGetter(
-    ChromeURLRequestContextGetter* url_request_context_getter) {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
-  std::list<ChromeURLRequestContextGetter*>::const_iterator it =
-      std::find(url_request_context_getters_.begin(),
-                url_request_context_getters_.end(),
-                url_request_context_getter);
-  DCHECK(it == url_request_context_getters_.end());
-  url_request_context_getters_.push_back(url_request_context_getter);
-}
-
-void IOThread::UnregisterURLRequestContextGetter(
-    ChromeURLRequestContextGetter* url_request_context_getter) {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
-  std::list<ChromeURLRequestContextGetter*>::iterator it =
-      std::find(url_request_context_getters_.begin(),
-                url_request_context_getters_.end(),
-                url_request_context_getter);
-  DCHECK(it != url_request_context_getters_.end());
-  // This does not scale, but we shouldn't have many URLRequestContextGetters in
-  // the first place, so this should be fine.
-  url_request_context_getters_.erase(it);
-}
-
 void IOThread::ChangedToOnTheRecord() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
   message_loop()->PostTask(
@@ -412,11 +417,7 @@ void IOThread::ChangedToOnTheRecord() {
 net::URLRequestContextGetter* IOThread::system_url_request_context_getter() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
   if (!system_url_request_context_getter_) {
-    system_proxy_config_service_.reset(
-        ProxyServiceFactory::CreateProxyConfigService(
-            pref_proxy_config_tracker_));
-    system_url_request_context_getter_ =
-        new SystemURLRequestContextGetter(this);
+    InitSystemRequestContext();
   }
   return system_url_request_context_getter_;
 }
@@ -446,6 +447,8 @@ void IOThread::Init() {
   DCHECK(!globals_);
   globals_ = new Globals;
 
+  globals_->media.media_internals.reset(new MediaInternals());
+
   // Add an observer that will emit network change events to the ChromeNetLog.
   // Assuming NetworkChangeNotifier dispatches in FIFO order, we should be
   // logging the network change before other IO thread consumers respond to it.
@@ -456,7 +459,9 @@ void IOThread::Init() {
       extension_event_router_forwarder_;
   globals_->system_network_delegate.reset(new ChromeNetworkDelegate(
       extension_event_router_forwarder_,
-      Profile::kInvalidProfileId,
+      NULL,
+      NULL,
+      NULL,
       &system_enable_referrers_));
   globals_->host_resolver.reset(
       CreateGlobalHostResolver(net_log_));
@@ -468,9 +473,17 @@ void IOThread::Init() {
   // For the ProxyScriptFetcher, we use a direct ProxyService.
   globals_->proxy_script_fetcher_proxy_service.reset(
       net::ProxyService::CreateDirectWithNetLog(net_log_));
+  // In-memory cookie store.
+  globals_->system_cookie_store = new net::CookieMonster(NULL, NULL);
+  // In-memory origin-bound cert store.
+  globals_->system_origin_bound_cert_service.reset(
+      new net::OriginBoundCertService(
+          new net::DefaultOriginBoundCertStore(NULL)));
   net::HttpNetworkSession::Params session_params;
   session_params.host_resolver = globals_->host_resolver.get();
   session_params.cert_verifier = globals_->cert_verifier.get();
+  session_params.origin_bound_cert_service =
+      globals_->system_origin_bound_cert_service.get();
   session_params.proxy_service =
       globals_->proxy_script_fetcher_proxy_service.get();
   session_params.http_auth_handler_factory =
@@ -485,9 +498,8 @@ void IOThread::Init() {
   globals_->proxy_script_fetcher_ftp_transaction_factory.reset(
       new net::FtpNetworkLayer(globals_->host_resolver.get()));
 
-  scoped_refptr<net::URLRequestContext> proxy_script_fetcher_context =
+  globals_->proxy_script_fetcher_context =
       ConstructProxyScriptFetcherContext(globals_, net_log_);
-  globals_->proxy_script_fetcher_context = proxy_script_fetcher_context;
 }
 
 void IOThread::CleanUp() {
@@ -508,20 +520,6 @@ void IOThread::CleanUp() {
   // IO thread only resources they are referencing.
   BrowserChildProcessHost::TerminateAll();
 
-  std::list<ChromeURLRequestContextGetter*> url_request_context_getters;
-  url_request_context_getters.swap(url_request_context_getters_);
-  for (std::list<ChromeURLRequestContextGetter*>::iterator it =
-       url_request_context_getters.begin();
-       it != url_request_context_getters.end(); ++it) {
-    ChromeURLRequestContextGetter* getter = *it;
-    // Stop all pending certificate provenance check uploads
-    net::DnsCertProvenanceChecker* checker =
-        getter->GetURLRequestContext()->dns_cert_checker();
-    if (checker)
-      checker->Shutdown();
-    getter->ReleaseURLRequestContext();
-  }
-
   system_url_request_context_getter_ = NULL;
 
   // Step 2: Release objects that the net::URLRequestContext could have been
@@ -545,11 +543,6 @@ void IOThread::CleanUp() {
   delete speculative_interceptor_;
   speculative_interceptor_ = NULL;
 
-  // TODO(eroman): hack for http://crbug.com/15513
-  if (globals_->host_resolver->GetAsHostResolverImpl()) {
-    globals_->host_resolver.get()->GetAsHostResolverImpl()->Shutdown();
-  }
-
   system_proxy_config_service_.reset();
 
   delete globals_;
@@ -667,6 +660,24 @@ net::SSLConfigService* IOThread::GetSSLConfigService() {
 }
 
 void IOThread::InitSystemRequestContext() {
+  if (system_url_request_context_getter_)
+    return;
+  // If we're in unit_tests, IOThread may not be run.
+  if (!message_loop())
+    return;
+  system_proxy_config_service_.reset(
+      ProxyServiceFactory::CreateProxyConfigService(
+          pref_proxy_config_tracker_));
+  system_url_request_context_getter_ =
+      new SystemURLRequestContextGetter(this);
+  message_loop()->PostTask(
+      FROM_HERE,
+      NewRunnableMethod(
+          this,
+          &IOThread::InitSystemRequestContextOnIOThread));
+}
+
+void IOThread::InitSystemRequestContextOnIOThread() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
   DCHECK(!globals_->system_proxy_service.get());
   DCHECK(system_proxy_config_service_.get());
@@ -681,6 +692,8 @@ void IOThread::InitSystemRequestContext() {
   net::HttpNetworkSession::Params system_params;
   system_params.host_resolver = globals_->host_resolver.get();
   system_params.cert_verifier = globals_->cert_verifier.get();
+  system_params.origin_bound_cert_service =
+      globals_->system_origin_bound_cert_service.get();
   system_params.dnsrr_resolver = globals_->dnsrr_resolver.get();
   system_params.dns_cert_checker = NULL;
   system_params.ssl_host_info_factory = NULL;
