@@ -23,7 +23,6 @@
 #include "net/url_request/url_request_context.h"
 #include "net/url_request/url_request_context_getter.h"
 #include "third_party/tcmalloc/chromium/src/google/malloc_extension.h"
-#include "v8/include/v8.h"
 
 // PurgeMemoryHelper -----------------------------------------------------------
 
@@ -152,5 +151,5 @@ void MemoryPurger::PurgeRenderers() {
 // static
 void MemoryPurger::PurgeRendererForHost(RenderProcessHost* host) {
   // Direct the renderer to free everything it can.
-  host->Send(new ViewMsg_PurgeMemory());
+  host->Send(new ChromeViewMsg_PurgeMemory());
 }
