@@ -174,12 +174,14 @@
 #include "chrome/browser/metrics/histogram_synchronizer.h"
 #include "chrome/browser/metrics/metrics_log.h"
 #include "chrome/browser/metrics/metrics_reporting_scheduler.h"
+#include "chrome/browser/net/network_stats.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/prefs/scoped_user_pref_update.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/search_engines/template_url_model.h"
+#include "chrome/browser/search_engines/template_url_service.h"
 #include "chrome/browser/ui/browser_list.h"
 #include "chrome/common/child_process_logging.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/guid.h"
 #include "chrome/common/pref_names.h"
@@ -189,7 +191,7 @@
 #include "content/common/child_process_info.h"
 #include "content/common/notification_service.h"
 #include "webkit/plugins/npapi/plugin_list.h"
-#include "webkit/plugins/npapi/webplugininfo.h"
+#include "webkit/plugins/webplugininfo.h"
 
 // TODO(port): port browser_distribution.h.
 #if !defined(OS_POSIX)
@@ -199,7 +201,7 @@
 #if defined(OS_CHROMEOS)
 #include "chrome/browser/chromeos/cros/cros_library.h"
 #include "chrome/browser/chromeos/external_metrics.h"
-#include "chrome/browser/chromeos/system_access.h"
+#include "chrome/browser/chromeos/system/statistics_provider.h"
 #endif
 
 namespace {
@@ -264,6 +266,10 @@ static const size_t kMaxOngoingLogsPersisted = 8;
 // checksum of the elements.
 static const size_t kChecksumEntryCount = 2;
 
+// static
+MetricsService::ShutdownCleanliness MetricsService::clean_shutdown_status_ =
+    MetricsService::CLEANLY_SHUTDOWN;
+
 // This is used to quickly log stats from child process related notifications in
 // MetricsService::child_stats_buffer_.  The buffer's contents are transferred
 // out when Local State is periodically saved.  The information is then
@@ -280,9 +286,9 @@ struct MetricsService::ChildProcessStats {
   // an index for which no value has been assigned.
   ChildProcessStats()
       : process_launches(0),
-      process_crashes(0),
-      instances(0),
-      process_type(ChildProcessInfo::UNKNOWN_PROCESS) {}
+        process_crashes(0),
+        instances(0),
+        process_type(ChildProcessInfo::UNKNOWN_PROCESS) {}
 
   // The number of times that the given child process has been launched
   int process_launches;
@@ -319,7 +325,7 @@ class MetricsService::InitTaskComplete : public Task {
  public:
   explicit InitTaskComplete(
       const std::string& hardware_class,
-      const std::vector<webkit::npapi::WebPluginInfo>& plugins)
+      const std::vector<webkit::WebPluginInfo>& plugins)
       : hardware_class_(hardware_class), plugins_(plugins) {}
 
   virtual void Run() {
@@ -329,7 +335,7 @@ class MetricsService::InitTaskComplete : public Task {
 
  private:
   std::string hardware_class_;
-  std::vector<webkit::npapi::WebPluginInfo> plugins_;
+  std::vector<webkit::WebPluginInfo> plugins_;
 };
 
 class MetricsService::InitTask : public Task {
@@ -338,11 +344,11 @@ class MetricsService::InitTask : public Task {
       : callback_loop_(callback_loop) {}
 
   virtual void Run() {
-    std::vector<webkit::npapi::WebPluginInfo> plugins;
-    webkit::npapi::PluginList::Singleton()->GetPlugins(false, &plugins);
+    std::vector<webkit::WebPluginInfo> plugins;
+    webkit::npapi::PluginList::Singleton()->GetPlugins(&plugins);
     std::string hardware_class;  // Empty string by default.
 #if defined(OS_CHROMEOS)
-    chromeos::SystemAccess::GetInstance()->GetMachineStatistic(
+    chromeos::system::StatisticsProvider::GetInstance()->GetMachineStatistic(
         "hardware_class", &hardware_class);
 #endif  // OS_CHROMEOS
     callback_loop_->PostTask(FROM_HERE, new InitTaskComplete(
@@ -438,6 +444,7 @@ MetricsService::MetricsService()
       reporting_active_(false),
       state_(INITIALIZED),
       current_fetch_(NULL),
+      io_thread_(NULL),
       idle_since_last_transmission_(false),
       next_window_id_(0),
       ALLOW_THIS_IN_INITIALIZER_LIST(log_sender_factory_(this)),
@@ -531,39 +538,39 @@ bool MetricsService::reporting_active() const {
 // static
 void MetricsService::SetUpNotifications(NotificationRegistrar* registrar,
                                         NotificationObserver* observer) {
-    registrar->Add(observer, NotificationType::BROWSER_OPENED,
+    registrar->Add(observer, chrome::NOTIFICATION_BROWSER_OPENED,
+                   NotificationService::AllBrowserContextsAndSources());
+    registrar->Add(observer, chrome::NOTIFICATION_BROWSER_CLOSED,
                    NotificationService::AllSources());
-    registrar->Add(observer, NotificationType::BROWSER_CLOSED,
+    registrar->Add(observer, content::NOTIFICATION_USER_ACTION,
                    NotificationService::AllSources());
-    registrar->Add(observer, NotificationType::USER_ACTION,
+    registrar->Add(observer, content::NOTIFICATION_TAB_PARENTED,
                    NotificationService::AllSources());
-    registrar->Add(observer, NotificationType::TAB_PARENTED,
+    registrar->Add(observer, content::NOTIFICATION_TAB_CLOSING,
                    NotificationService::AllSources());
-    registrar->Add(observer, NotificationType::TAB_CLOSING,
+    registrar->Add(observer, content::NOTIFICATION_LOAD_START,
                    NotificationService::AllSources());
-    registrar->Add(observer, NotificationType::LOAD_START,
+    registrar->Add(observer, content::NOTIFICATION_LOAD_STOP,
                    NotificationService::AllSources());
-    registrar->Add(observer, NotificationType::LOAD_STOP,
+    registrar->Add(observer, content::NOTIFICATION_RENDERER_PROCESS_CLOSED,
                    NotificationService::AllSources());
-    registrar->Add(observer, NotificationType::RENDERER_PROCESS_CLOSED,
+    registrar->Add(observer, content::NOTIFICATION_RENDERER_PROCESS_HANG,
                    NotificationService::AllSources());
-    registrar->Add(observer, NotificationType::RENDERER_PROCESS_HANG,
+    registrar->Add(observer, content::NOTIFICATION_CHILD_PROCESS_HOST_CONNECTED,
                    NotificationService::AllSources());
-    registrar->Add(observer, NotificationType::CHILD_PROCESS_HOST_CONNECTED,
+    registrar->Add(observer, content::NOTIFICATION_CHILD_INSTANCE_CREATED,
                    NotificationService::AllSources());
-    registrar->Add(observer, NotificationType::CHILD_INSTANCE_CREATED,
+    registrar->Add(observer, content::NOTIFICATION_CHILD_PROCESS_CRASHED,
                    NotificationService::AllSources());
-    registrar->Add(observer, NotificationType::CHILD_PROCESS_CRASHED,
+    registrar->Add(observer, chrome::NOTIFICATION_TEMPLATE_URL_SERVICE_LOADED,
                    NotificationService::AllSources());
-    registrar->Add(observer, NotificationType::TEMPLATE_URL_MODEL_LOADED,
-                   NotificationService::AllSources());
-    registrar->Add(observer, NotificationType::OMNIBOX_OPENED_URL,
-                   NotificationService::AllSources());
-    registrar->Add(observer, NotificationType::BOOKMARK_MODEL_LOADED,
+    registrar->Add(observer, chrome::NOTIFICATION_OMNIBOX_OPENED_URL,
                    NotificationService::AllSources());
+    registrar->Add(observer, chrome::NOTIFICATION_BOOKMARK_MODEL_LOADED,
+                   NotificationService::AllBrowserContextsAndSources());
 }
 
-void MetricsService::Observe(NotificationType type,
+void MetricsService::Observe(int type,
                              const NotificationSource& source,
                              const NotificationDetails& details) {
   DCHECK(current_log_);
@@ -572,30 +579,30 @@ void MetricsService::Observe(NotificationType type,
   if (!CanLogNotification(type, source, details))
     return;
 
-  switch (type.value) {
-    case NotificationType::USER_ACTION:
+  switch (type) {
+    case content::NOTIFICATION_USER_ACTION:
         current_log_->RecordUserAction(*Details<const char*>(details).ptr());
       break;
 
-    case NotificationType::BROWSER_OPENED:
-    case NotificationType::BROWSER_CLOSED:
+    case chrome::NOTIFICATION_BROWSER_OPENED:
+    case chrome::NOTIFICATION_BROWSER_CLOSED:
       LogWindowChange(type, source, details);
       break;
 
-    case NotificationType::TAB_PARENTED:
-    case NotificationType::TAB_CLOSING:
+    case content::NOTIFICATION_TAB_PARENTED:
+    case content::NOTIFICATION_TAB_CLOSING:
       LogWindowChange(type, source, details);
       break;
 
-    case NotificationType::LOAD_STOP:
+    case content::NOTIFICATION_LOAD_STOP:
       LogLoadComplete(type, source, details);
       break;
 
-    case NotificationType::LOAD_START:
+    case content::NOTIFICATION_LOAD_START:
       LogLoadStarted();
       break;
 
-    case NotificationType::RENDERER_PROCESS_CLOSED: {
+    case content::NOTIFICATION_RENDERER_PROCESS_CLOSED: {
         RenderProcessHost::RendererClosedDetails* process_details =
             Details<RenderProcessHost::RendererClosedDetails>(details).ptr();
         if (process_details->status ==
@@ -611,21 +618,21 @@ void MetricsService::Observe(NotificationType type,
       }
       break;
 
-    case NotificationType::RENDERER_PROCESS_HANG:
+    case content::NOTIFICATION_RENDERER_PROCESS_HANG:
       LogRendererHang();
       break;
 
-    case NotificationType::CHILD_PROCESS_HOST_CONNECTED:
-    case NotificationType::CHILD_PROCESS_CRASHED:
-    case NotificationType::CHILD_INSTANCE_CREATED:
+    case content::NOTIFICATION_CHILD_PROCESS_HOST_CONNECTED:
+    case content::NOTIFICATION_CHILD_PROCESS_CRASHED:
+    case content::NOTIFICATION_CHILD_INSTANCE_CREATED:
       LogChildProcessChange(type, source, details);
       break;
 
-    case NotificationType::TEMPLATE_URL_MODEL_LOADED:
-      LogKeywords(Source<TemplateURLModel>(source).ptr());
+    case chrome::NOTIFICATION_TEMPLATE_URL_SERVICE_LOADED:
+      LogKeywords(Source<TemplateURLService>(source).ptr());
       break;
 
-    case NotificationType::OMNIBOX_OPENED_URL: {
+    case chrome::NOTIFICATION_OMNIBOX_OPENED_URL: {
       MetricsLog* current_log = current_log_->AsMetricsLog();
       DCHECK(current_log);
       current_log->RecordOmniboxOpenedURL(
@@ -633,7 +640,7 @@ void MetricsService::Observe(NotificationType type,
       break;
     }
 
-    case NotificationType::BOOKMARK_MODEL_LOADED: {
+    case chrome::NOTIFICATION_BOOKMARK_MODEL_LOADED: {
       Profile* p = Source<Profile>(source).ptr();
       if (p)
         LogBookmarks(p->GetBookmarkModel());
@@ -694,9 +701,12 @@ void MetricsService::RecordBreakpadHasDebugger(bool has_debugger) {
 void MetricsService::InitializeMetricsState() {
 #if defined(OS_POSIX)
   server_url_ = L"https://clients4.google.com/firefox/metrics/collect";
+  // TODO(rtenneti): Return the network stats server name.
+  network_stats_server_ = "";
 #else
   BrowserDistribution* dist = BrowserDistribution::GetDistribution();
   server_url_ = dist->GetStatsServerURL();
+  network_stats_server_ = dist->GetNetworkStatsServer();
 #endif
 
   PrefService* pref = g_browser_process->local_state();
@@ -725,11 +735,11 @@ void MetricsService::InitializeMetricsState() {
 
   if (!pref->GetBoolean(prefs::kStabilityExitedCleanly)) {
     IncrementPrefValue(prefs::kStabilityCrashCount);
+    // Reset flag, and wait until we call LogNeedForCleanShutdown() before
+    // monitoring.
+    pref->SetBoolean(prefs::kStabilityExitedCleanly, true);
   }
 
-  // This will be set to 'true' if we exit cleanly.
-  pref->SetBoolean(prefs::kStabilityExitedCleanly, false);
-
   if (!pref->GetBoolean(prefs::kStabilitySessionEndCompleted)) {
     IncrementPrefValue(prefs::kStabilityIncompleteSessionEndCount);
     // This is marked false when we get a WM_ENDSESSION.
@@ -776,10 +786,10 @@ void MetricsService::InitializeMetricsState() {
     UMA_HISTOGRAM_COUNTS_100("Chrome.CommandLineAppModeCount", 1);
   }
 
-  UMA_HISTOGRAM_COUNTS_100("Chrome.CommandLineFlagCount",
-                           command_line->GetSwitchCount());
+  size_t switch_count = command_line->GetSwitches().size();
+  UMA_HISTOGRAM_COUNTS_100("Chrome.CommandLineFlagCount", switch_count);
   UMA_HISTOGRAM_COUNTS_100("Chrome.CommandLineUncommonFlagCount",
-                           command_line->GetSwitchCount() - common_commands);
+                           switch_count - common_commands);
 
   // Kick off the process of saving the state (so the uptime numbers keep
   // getting updated) every n minutes.
@@ -788,10 +798,11 @@ void MetricsService::InitializeMetricsState() {
 
 void MetricsService::OnInitTaskComplete(
     const std::string& hardware_class,
-    const std::vector<webkit::npapi::WebPluginInfo>& plugins) {
+    const std::vector<webkit::WebPluginInfo>& plugins) {
   DCHECK(state_ == INIT_TASK_SCHEDULED);
   hardware_class_ = hardware_class;
   plugins_ = plugins;
+  io_thread_ = g_browser_process->io_thread();
   if (state_ == INIT_TASK_SCHEDULED)
     state_ = INIT_TASK_DONE;
 }
@@ -852,11 +863,9 @@ void MetricsService::StopRecording() {
   if (!current_log_)
     return;
 
-  current_log_->set_hardware_class(hardware_class_);  // Adds to ongoing logs.
-
   // TODO(jar): Integrate bounds on log recording more consistently, so that we
   // can stop recording logs that are too big much sooner.
-  if (current_log_->num_events() > kEventLimit) {
+  if (current_log_->num_events() >= kEventLimit) {
     UMA_HISTOGRAM_COUNTS("UMA.Discarded Log Events",
                          current_log_->num_events());
     current_log_->CloseLog();
@@ -865,6 +874,8 @@ void MetricsService::StopRecording() {
     StartRecording();  // Start trivial log to hold our histograms.
   }
 
+  current_log_->set_hardware_class(hardware_class_);  // Adds to ongoing logs.
+
   // Put incremental data (histogram deltas, and realtime stats deltas) at the
   // end of all log transmissions (initial log handles this separately).
   // RecordIncrementalStabilityElements only exists on the derived
@@ -942,7 +953,7 @@ void MetricsService::StartScheduledUpload() {
   // Collect WebCore cache information to put into a histogram.
   for (RenderProcessHost::iterator i(RenderProcessHost::AllHostsIterator());
        !i.IsAtEnd(); i.Advance())
-    i.GetCurrentValue()->Send(new ViewMsg_GetCacheResourceStats());
+    i.GetCurrentValue()->Send(new ChromeViewMsg_GetCacheResourceStats());
 }
 
 void MetricsService::OnMemoryDetailCollectionDone() {
@@ -1048,6 +1059,7 @@ void MetricsService::MakePendingLog() {
     case SEND_OLD_INITIAL_LOGS:
       if (!unsent_initial_logs_.empty()) {
         compressed_log_ = unsent_initial_logs_.back();
+        unsent_initial_logs_.pop_back();
         break;
       }
       state_ = SENDING_OLD_LOGS;
@@ -1056,6 +1068,7 @@ void MetricsService::MakePendingLog() {
     case SENDING_OLD_LOGS:
       if (!unsent_ongoing_logs_.empty()) {
         compressed_log_ = unsent_ongoing_logs_.back();
+        unsent_ongoing_logs_.pop_back();
         break;
       }
       state_ = SENDING_CURRENT_LOGS;
@@ -1113,8 +1126,8 @@ MetricsService::LogRecallStatus MetricsService::RecallUnsentLogsHelper(
       list.GetSize() - kChecksumEntryCount)
     return MakeRecallStatusHistogram(LIST_SIZE_CORRUPTION);
 
-  MD5Context ctx;
-  MD5Init(&ctx);
+  base::MD5Context ctx;
+  base::MD5Init(&ctx);
   std::string encoded_log;
   std::string decoded_log;
   for (ListValue::const_iterator it = list.begin() + 1;
@@ -1125,7 +1138,7 @@ MetricsService::LogRecallStatus MetricsService::RecallUnsentLogsHelper(
       return MakeRecallStatusHistogram(LOG_STRING_CORRUPTION);
     }
 
-    MD5Update(&ctx, encoded_log.data(), encoded_log.length());
+    base::MD5Update(&ctx, encoded_log);
 
     if (!base::Base64Decode(encoded_log, &decoded_log)) {
       local_list->clear();
@@ -1135,8 +1148,8 @@ MetricsService::LogRecallStatus MetricsService::RecallUnsentLogsHelper(
   }
 
   // Verify checksum.
-  MD5Digest digest;
-  MD5Final(&digest, &ctx);
+  base::MD5Digest digest;
+  base::MD5Final(&digest, &ctx);
   std::string recovered_md5;
   // We store the hash at the end of the list.
   valid = (*(list.end() - 1))->GetAsString(&recovered_md5);
@@ -1144,7 +1157,7 @@ MetricsService::LogRecallStatus MetricsService::RecallUnsentLogsHelper(
     local_list->clear();
     return MakeRecallStatusHistogram(CHECKSUM_STRING_CORRUPTION);
   }
-  if (recovered_md5 != MD5DigestToBase16(digest)) {
+  if (recovered_md5 != base::MD5DigestToBase16(digest)) {
     local_list->clear();
     return MakeRecallStatusHistogram(CHECKSUM_CORRUPTION);
   }
@@ -1179,8 +1192,8 @@ void MetricsService::StoreUnsentLogsHelper(
   // Store size at the beginning of the list.
   list->Append(Value::CreateIntegerValue(local_list.size() - start));
 
-  MD5Context ctx;
-  MD5Init(&ctx);
+  base::MD5Context ctx;
+  base::MD5Init(&ctx);
   std::string encoded_log;
   for (std::vector<std::string>::const_iterator it = local_list.begin() + start;
        it != local_list.end(); ++it) {
@@ -1191,14 +1204,14 @@ void MetricsService::StoreUnsentLogsHelper(
       list->Clear();
       return;
     }
-    MD5Update(&ctx, encoded_log.data(), encoded_log.length());
+    base::MD5Update(&ctx, encoded_log);
     list->Append(Value::CreateStringValue(encoded_log));
   }
 
   // Append hash to the end of the list.
-  MD5Digest digest;
-  MD5Final(&digest, &ctx);
-  list->Append(Value::CreateStringValue(MD5DigestToBase16(digest)));
+  base::MD5Digest digest;
+  base::MD5Final(&digest, &ctx);
+  list->Append(Value::CreateStringValue(base::MD5DigestToBase16(digest)));
   DCHECK(list->GetSize() >= 3);  // Minimum of 3 elements (size, data, hash).
   MakeStoreStatusHistogram(STORE_SUCCESS);
 }
@@ -1259,7 +1272,8 @@ void MetricsService::PrepareFetchWithPendingLog() {
   current_fetch_.reset(new URLFetcher(GURL(WideToUTF16(server_url_)),
                                       URLFetcher::POST,
                                       this));
-  current_fetch_->set_request_context(Profile::GetDefaultRequestContext());
+  current_fetch_->set_request_context(
+      g_browser_process->system_request_context());
   current_fetch_->set_upload_data(kMetricsType, compressed_log_);
 }
 
@@ -1330,14 +1344,8 @@ void MetricsService::OnURLFetchComplete(const URLFetcher* source,
         break;
 
       case SEND_OLD_INITIAL_LOGS:
-        DCHECK(!unsent_initial_logs_.empty());
-        unsent_initial_logs_.pop_back();
-        StoreUnsentLogs();
-        break;
-
       case SENDING_OLD_LOGS:
-        DCHECK(!unsent_ongoing_logs_.empty());
-        unsent_ongoing_logs_.pop_back();
+        // Store the updated list to disk now that the removed log is uploaded.
         StoreUnsentLogs();
         break;
 
@@ -1365,6 +1373,10 @@ void MetricsService::OnURLFetchComplete(const URLFetcher* source,
   bool server_is_healthy = upload_succeeded || response_code == 400;
 
   scheduler_->UploadFinished(server_is_healthy, unsent_logs());
+
+  // Collect network stats if UMA upload succeeded.
+  if (server_is_healthy && io_thread_)
+    chrome_browser_net::CollectNetworkStats(network_stats_server_, io_thread_);
 }
 
 void MetricsService::LogBadResponseCode() {
@@ -1378,7 +1390,7 @@ void MetricsService::LogBadResponseCode() {
   }
 }
 
-void MetricsService::LogWindowChange(NotificationType type,
+void MetricsService::LogWindowChange(int type,
                                      const NotificationSource& source,
                                      const NotificationDetails& details) {
   int controller_id = -1;
@@ -1396,14 +1408,14 @@ void MetricsService::LogWindowChange(NotificationType type,
   }
   DCHECK_NE(controller_id, -1);
 
-  switch (type.value) {
-    case NotificationType::TAB_PARENTED:
-    case NotificationType::BROWSER_OPENED:
+  switch (type) {
+    case content::NOTIFICATION_TAB_PARENTED:
+    case chrome::NOTIFICATION_BROWSER_OPENED:
       window_type = MetricsLog::WINDOW_CREATE;
       break;
 
-    case NotificationType::TAB_CLOSING:
-    case NotificationType::BROWSER_CLOSED:
+    case content::NOTIFICATION_TAB_CLOSING:
+    case chrome::NOTIFICATION_BROWSER_CLOSED:
       window_map_.erase(window_map_.find(window_or_tab));
       window_type = MetricsLog::WINDOW_DESTROY;
       break;
@@ -1417,7 +1429,7 @@ void MetricsService::LogWindowChange(NotificationType type,
   current_log_->RecordWindowEvent(window_type, controller_id, 0);
 }
 
-void MetricsService::LogLoadComplete(NotificationType type,
+void MetricsService::LogLoadComplete(int type,
                                      const NotificationSource& source,
                                      const NotificationDetails& details) {
   if (details == NotificationService::NoDetails())
@@ -1452,6 +1464,7 @@ void MetricsService::IncrementLongPrefsValue(const char* path) {
 }
 
 void MetricsService::LogLoadStarted() {
+  HISTOGRAM_ENUMERATION("Chrome.UmaPageloadCounter", 1, 2);
   IncrementPrefValue(prefs::kStabilityPageLoadCount);
   IncrementLongPrefsValue(prefs::kUninstallMetricsPageLoadCount);
   // We need to save the prefs, as page load count is a critical stat, and it
@@ -1470,7 +1483,42 @@ void MetricsService::LogRendererHang() {
   IncrementPrefValue(prefs::kStabilityRendererHangCount);
 }
 
+void MetricsService::LogNeedForCleanShutdown() {
+  PrefService* pref = g_browser_process->local_state();
+  pref->SetBoolean(prefs::kStabilityExitedCleanly, false);
+  // Redundant setting to be sure we call for a clean shutdown.
+  clean_shutdown_status_ = NEED_TO_SHUTDOWN;
+}
+
+bool MetricsService::UmaMetricsProperlyShutdown() {
+  CHECK(clean_shutdown_status_ == CLEANLY_SHUTDOWN ||
+        clean_shutdown_status_ == NEED_TO_SHUTDOWN);
+  return clean_shutdown_status_ == CLEANLY_SHUTDOWN;
+}
+
+// For use in hack in LogCleanShutdown.
+static void Signal(base::WaitableEvent* event) {
+  event->Signal();
+}
+
 void MetricsService::LogCleanShutdown() {
+  // Redundant hack to write pref ASAP.
+  PrefService* pref = g_browser_process->local_state();
+  pref->SetBoolean(prefs::kStabilityExitedCleanly, true);
+  pref->SavePersistentPrefs();
+  // Hack: TBD: Remove this wait.
+  // We are so concerned that the pref gets written, we are now willing to stall
+  // the UI thread until we get assurance that a pref-writing task has
+  // completed.
+  base::WaitableEvent done_writing(false, false);
+  BrowserThread::PostTask(BrowserThread::FILE, FROM_HERE,
+      NewRunnableFunction(Signal, &done_writing));
+  done_writing.TimedWait(base::TimeDelta::FromHours(1));
+
+  // Redundant setting to assure that we always reset this value at shutdown
+  // (and that we don't use some alternate path, and not call LogCleanShutdown).
+  clean_shutdown_status_ = CLEANLY_SHUTDOWN;
+
   RecordBooleanPrefValue(prefs::kStabilityExitedCleanly, true);
 }
 
@@ -1491,11 +1539,11 @@ void MetricsService::LogChromeOSCrash(const std::string &crash_type) {
 #endif  // OS_CHROMEOS
 
 void MetricsService::LogChildProcessChange(
-    NotificationType type,
+    int type,
     const NotificationSource& source,
     const NotificationDetails& details) {
   Details<ChildProcessInfo> child_details(details);
-  const std::wstring& child_name = child_details->name();
+  const string16& child_name = child_details->name();
 
   if (child_process_stats_buffer_.find(child_name) ==
       child_process_stats_buffer_.end()) {
@@ -1504,16 +1552,16 @@ void MetricsService::LogChildProcessChange(
   }
 
   ChildProcessStats& stats = child_process_stats_buffer_[child_name];
-  switch (type.value) {
-    case NotificationType::CHILD_PROCESS_HOST_CONNECTED:
+  switch (type) {
+    case content::NOTIFICATION_CHILD_PROCESS_HOST_CONNECTED:
       stats.process_launches++;
       break;
 
-    case NotificationType::CHILD_INSTANCE_CREATED:
+    case content::NOTIFICATION_CHILD_INSTANCE_CREATED:
       stats.instances++;
       break;
 
-    case NotificationType::CHILD_PROCESS_CRASHED:
+    case content::NOTIFICATION_CHILD_PROCESS_CRASHED:
       stats.process_crashes++;
       // Exclude plugin crashes from the count below because we report them via
       // a separate UMA metric.
@@ -1523,7 +1571,7 @@ void MetricsService::LogChildProcessChange(
       break;
 
     default:
-      NOTREACHED() << "Unexpected notification type " << type.value;
+      NOTREACHED() << "Unexpected notification type " << type;
       return;
   }
 }
@@ -1532,7 +1580,7 @@ void MetricsService::LogChildProcessChange(
 static void CountBookmarks(const BookmarkNode* node,
                            int* bookmarks,
                            int* folders) {
-  if (node->type() == BookmarkNode::URL)
+  if (node->is_url())
     (*bookmarks)++;
   else
     (*folders)++;
@@ -1557,7 +1605,7 @@ void MetricsService::LogBookmarks(const BookmarkNode* node,
 
 void MetricsService::LogBookmarks(BookmarkModel* model) {
   DCHECK(model);
-  LogBookmarks(model->GetBookmarkBarNode(),
+  LogBookmarks(model->bookmark_bar_node(),
                prefs::kNumBookmarksOnBookmarkBar,
                prefs::kNumFoldersOnBookmarkBar);
   LogBookmarks(model->other_node(),
@@ -1566,7 +1614,7 @@ void MetricsService::LogBookmarks(BookmarkModel* model) {
   ScheduleNextStateSave();
 }
 
-void MetricsService::LogKeywords(const TemplateURLModel* url_model) {
+void MetricsService::LogKeywords(const TemplateURLService* url_model) {
   DCHECK(url_model);
 
   PrefService* pref = g_browser_process->local_state();
@@ -1597,12 +1645,13 @@ void MetricsService::RecordPluginChanges(PrefService* pref) {
     }
 
     // TODO(viettrungluu): remove conversions
-    if (child_process_stats_buffer_.find(UTF8ToWide(plugin_name)) ==
-        child_process_stats_buffer_.end())
+    string16 name16 = UTF8ToUTF16(plugin_name);
+    if (child_process_stats_buffer_.find(name16) ==
+        child_process_stats_buffer_.end()) {
       continue;
+    }
 
-    ChildProcessStats stats =
-        child_process_stats_buffer_[UTF8ToWide(plugin_name)];
+    ChildProcessStats stats = child_process_stats_buffer_[name16];
     if (stats.process_launches) {
       int launches = 0;
       plugin_dict->GetInteger(prefs::kStabilityPluginLaunches, &launches);
@@ -1622,12 +1671,12 @@ void MetricsService::RecordPluginChanges(PrefService* pref) {
       plugin_dict->SetInteger(prefs::kStabilityPluginInstances, instances);
     }
 
-    child_process_stats_buffer_.erase(UTF8ToWide(plugin_name));
+    child_process_stats_buffer_.erase(name16);
   }
 
   // Now go through and add dictionaries for plugins that didn't already have
   // reports in Local State.
-  for (std::map<std::wstring, ChildProcessStats>::iterator cache_iter =
+  for (std::map<string16, ChildProcessStats>::iterator cache_iter =
            child_process_stats_buffer_.begin();
        cache_iter != child_process_stats_buffer_.end(); ++cache_iter) {
     ChildProcessStats stats = cache_iter->second;
@@ -1637,7 +1686,7 @@ void MetricsService::RecordPluginChanges(PrefService* pref) {
       continue;
 
     // TODO(viettrungluu): remove conversion
-    std::string plugin_name = WideToUTF8(cache_iter->first);
+    std::string plugin_name = UTF16ToUTF8(cache_iter->first);
 
     DictionaryValue* plugin_dict = new DictionaryValue;
 
@@ -1653,7 +1702,7 @@ void MetricsService::RecordPluginChanges(PrefService* pref) {
   child_process_stats_buffer_.clear();
 }
 
-bool MetricsService::CanLogNotification(NotificationType type,
+bool MetricsService::CanLogNotification(int type,
                                         const NotificationSource& source,
                                         const NotificationDetails& details) {
   // We simply don't log anything to UMA if there is a single incognito
@@ -1691,3 +1740,18 @@ void MetricsService::StartExternalMetrics() {
   external_metrics_->Start();
 }
 #endif
+
+// static
+bool MetricsServiceHelper::IsMetricsReportingEnabled() {
+  bool result = false;
+  const PrefService* local_state = g_browser_process->local_state();
+  if (local_state) {
+    const PrefService::Preference* uma_pref =
+        local_state->FindPreference(prefs::kMetricsReportingEnabled);
+    if (uma_pref) {
+      bool success = uma_pref->GetValue()->GetAsBoolean(&result);
+      DCHECK(success);
+    }
+  }
+  return result;
+}
