@@ -22,16 +22,16 @@
 #include "net/base/host_resolver.h"
 #include "net/base/net_errors.h"
 #include "net/base/net_log.h"
+#include "net/base/single_request_host_resolver.h"
 
 using base::TimeDelta;
 
 namespace chrome_browser_net {
 
 // static
+const int Predictor::kPredictorReferrerVersion = 2;
 const double Predictor::kPreconnectWorthyExpectedValue = 0.8;
-// static
 const double Predictor::kDNSPreresolutionWorthyExpectedValue = 0.1;
-// static
 const double Predictor::kDiscardableExpectedValue = 0.05;
 // The goal is of trimming is to to reduce the importance (number of expected
 // subresources needed) by a factor of 2 after about 24 hours of uptime. We will
@@ -44,13 +44,9 @@ const double Predictor::kDiscardableExpectedValue = 0.05;
 // system that uses a higher trim ratio when the list is large.
 // static
 const double Predictor::kReferrerTrimRatio = 0.97153;
-
-// static
 const TimeDelta Predictor::kDurationBetweenTrimmings = TimeDelta::FromHours(1);
-// static
 const TimeDelta Predictor::kDurationBetweenTrimmingIncrements =
     TimeDelta::FromSeconds(15);
-// static
 const size_t Predictor::kUrlsTrimmedPerIncrement = 5u;
 
 class Predictor::LookupRequest {
@@ -150,13 +146,14 @@ void Predictor::Resolve(const GURL& url,
 void Predictor::LearnFromNavigation(const GURL& referring_url,
                                     const GURL& target_url) {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
-  DCHECK(referring_url == referring_url.GetWithEmptyPath());
-  DCHECK(target_url == target_url.GetWithEmptyPath());
-  if (referring_url.has_host()) {
-    referrers_[referring_url].SuggestHost(target_url);
-    // Possibly do some referrer trimming.
-    TrimReferrers();
-  }
+  DCHECK_EQ(referring_url, Predictor::CanonicalizeUrl(referring_url));
+  DCHECK_NE(referring_url, GURL::EmptyGURL());
+  DCHECK_EQ(target_url, Predictor::CanonicalizeUrl(target_url));
+  DCHECK_NE(target_url, GURL::EmptyGURL());
+
+  referrers_[referring_url].SuggestHost(target_url);
+  // Possibly do some referrer trimming.
+  TrimReferrers();
 }
 
 enum SubresourceValue {
@@ -194,11 +191,12 @@ void Predictor::AnticipateOmniboxUrl(const GURL& url, bool preconnectable) {
       // TODO(jar): Use an A/B test to optimize this.
       const int kMinConsecutiveRequests = 8;
       if (consecutive_omnibox_preconnect_count_ >= kMinConsecutiveRequests) {
-        // TODO(jar): The wild guess of 30 seconds could be tuned/tested, but it
-        // currently is just a guess that most sockets will remain open for at
-        // least 30 seconds.  This avoids a lot of cross-thread posting, and
-        // exercise of the network stack in this common case.
-        const int kMaxSearchKeepaliveSeconds(30);
+        // TODO(jar): Perhaps we should do a GET to leave the socket open in the
+        // pool.  Currently, we just do a connect, which MAY be reset if we
+        // don't use it in 10 secondes!!!  As a result, we may do more
+        // connections, and actually cost the server more than if we did a real
+        // get with a fake request (/gen_204 might be the good path on Google).
+        const int kMaxSearchKeepaliveSeconds(10);
         if ((now - last_omnibox_preconnect_).InSeconds() <
              kMaxSearchKeepaliveSeconds)
           return;  // We've done a preconnect recently.
@@ -245,7 +243,7 @@ void Predictor::PreconnectUrlAndSubresources(const GURL& url) {
 }
 
 void Predictor::PredictFrameSubresources(const GURL& url) {
-  DCHECK(url.GetWithEmptyPath() == url);
+  DCHECK_EQ(url.GetWithEmptyPath(), url);
   // Add one pass through the message loop to allow current navigation to
   // proceed.
   BrowserThread::PostTask(
@@ -256,7 +254,7 @@ void Predictor::PredictFrameSubresources(const GURL& url) {
 
 void Predictor::PrepareFrameSubresources(const GURL& url) {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
-  DCHECK(url.GetWithEmptyPath() == url);
+  DCHECK_EQ(url.GetWithEmptyPath(), url);
   Referrers::iterator it = referrers_.find(url);
   if (referrers_.end() == it) {
     // Only when we don't know anything about this url, make 2 connections
@@ -336,23 +334,18 @@ struct RightToLeftStringSorter {
       if (!left_already_matched) return true;
       if (!right_already_matched) return false;
 
-      size_t left_length, right_length;
       size_t left_start = left_host.find_last_of('.', left_already_matched - 1);
       if (std::string::npos == left_start) {
-        left_length = left_already_matched;
         left_already_matched = left_start = 0;
       } else {
-        left_length = left_already_matched - left_start;
         left_already_matched = left_start;
         ++left_start;  // Don't compare the dot.
       }
       size_t right_start = right_host.find_last_of('.',
                                                    right_already_matched - 1);
       if (std::string::npos == right_start) {
-        right_length = right_already_matched;
         right_already_matched = right_start = 0;
       } else {
-        right_length = right_already_matched - right_start;
         right_already_matched = right_start;
         ++right_start;  // Don't compare the dot.
       }
@@ -608,7 +601,7 @@ void Predictor::TrimReferrersNow() {
 void Predictor::SerializeReferrers(ListValue* referral_list) {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
   referral_list->Clear();
-  referral_list->Append(new FundamentalValue(PREDICTOR_REFERRER_VERSION));
+  referral_list->Append(new base::FundamentalValue(kPredictorReferrerVersion));
   for (Referrers::const_iterator it = referrers_.begin();
        it != referrers_.end(); ++it) {
     // Serialize the list of subresource names.
@@ -628,7 +621,7 @@ void Predictor::DeserializeReferrers(const ListValue& referral_list) {
   int format_version = -1;
   if (referral_list.GetSize() > 0 &&
       referral_list.GetInteger(0, &format_version) &&
-      format_version == PREDICTOR_REFERRER_VERSION) {
+      format_version == kPredictorReferrerVersion) {
     for (size_t i = 1; i < referral_list.GetSize(); ++i) {
       ListValue* motivator;
       if (!referral_list.GetList(i, &motivator)) {
@@ -737,8 +730,10 @@ GURL Predictor::HostNameQueue::Pop() {
 }
 
 void Predictor::DeserializeReferrersThenDelete(ListValue* referral_list) {
-    DeserializeReferrers(*referral_list);
-    delete referral_list;
+  // DeserializeReferrers(*referral_list);
+  // TODO(rlp): Temporarily not deserializing since we are not saving dns
+  // info at shutdown. See http://crbug.com/90114.
+  delete referral_list;
 }
 
 
