@@ -8,7 +8,7 @@
 #define CHROME_BROWSER_NET_URL_REQUEST_MOCK_LINK_DOCTOR_JOB_H_
 #pragma once
 
-#include "chrome/browser/net/url_request_mock_http_job.h"
+#include "content/browser/net/url_request_mock_http_job.h"
 
 class URLRequestMockLinkDoctorJob : public URLRequestMockHTTPJob {
  public:
