@@ -7,20 +7,23 @@
 #include "base/metrics/histogram.h"
 #include "base/threading/thread.h"
 #include "base/utf_string_conversions.h"
+#include "chrome/browser/content_settings/content_settings_details.h"
+#include "chrome/browser/content_settings/content_settings_pattern.h"
 #include "chrome/browser/content_settings/content_settings_provider.h"
 #include "chrome/browser/content_settings/host_content_settings_map.h"
 #include "chrome/browser/extensions/extension_service.h"
+#include "chrome/browser/infobars/infobar_tab_helper.h"
 #include "chrome/browser/notifications/desktop_notification_service_factory.h"
 #include "chrome/browser/notifications/notification.h"
 #include "chrome/browser/notifications/notification_object_proxy.h"
 #include "chrome/browser/notifications/notification_ui_manager.h"
-#include "chrome/browser/notifications/notifications_prefs_cache.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/prefs/scoped_user_pref_update.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/tab_contents/confirm_infobar_delegate.h"
 #include "chrome/browser/ui/browser_list.h"
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/url_constants.h"
 #include "content/browser/browser_child_process_host.h"
@@ -31,13 +34,11 @@
 #include "content/browser/worker_host/worker_process_host.h"
 #include "content/common/desktop_notification_messages.h"
 #include "content/common/notification_service.h"
-#include "content/common/notification_type.h"
 #include "grit/browser_resources.h"
 #include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
 #include "net/base/escape.h"
-#include "third_party/WebKit/Source/WebKit/chromium/public/WebNotificationPresenter.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
 
@@ -46,45 +47,6 @@ using WebKit::WebTextDirection;
 
 const ContentSetting kDefaultSetting = CONTENT_SETTING_ASK;
 
-namespace {
-
-typedef content_settings::ProviderInterface::Rules Rules;
-
-void GetOriginsWithSettingFromContentSettingsRules(
-    const Rules& content_setting_rules,
-    ContentSetting setting,
-    std::vector<GURL>* origins) {
-  origins->clear();
-
-  for (Rules::const_iterator rule = content_setting_rules.begin();
-       rule != content_setting_rules.end();
-       ++rule) {
-    if (setting == rule->content_setting) {
-      std::string url_str = rule->requesting_url_pattern.ToString();
-      if (!rule->requesting_url_pattern.IsValid()) {
-        // TODO(markusheintz): This will be removed in one of the next
-        // refactoring steps as this entire function will disapear.
-        LOG(DFATAL) << "Ignoring invalid content settings pattern: "
-                    << url_str;
-      } else if (url_str.find(ContentSettingsPattern::kDomainWildcard) == 0) {
-        // TODO(markusheintz): This must be changed once the UI code is
-        // refactored and content_settings patterns are fully supported for
-        // notifications settings.
-        LOG(DFATAL) << "Ignoring unsupported content settings pattern: "
-                    << url_str << ". Content settings patterns other than "
-                    << "hostnames (e.g. wildcard patterns) are not supported "
-                    << "for notification content settings yet.";
-      } else {
-        origins->push_back(
-            content_settings::NotificationProvider::ToGURL(
-                rule->requesting_url_pattern));
-      }
-    }
-  }
-}
-
-}  // namespace
-
 // NotificationPermissionInfoBarDelegate --------------------------------------
 
 // The delegate for the infobar shown when an origin requests notification
@@ -141,7 +103,7 @@ NotificationPermissionInfoBarDelegate::NotificationPermissionInfoBarDelegate(
     : ConfirmInfoBarDelegate(contents),
       origin_(origin),
       display_name_(display_name),
-      profile_(contents->profile()),
+      profile_(Profile::FromBrowserContext(contents->browser_context())),
       process_id_(process_id),
       route_id_(route_id),
       callback_context_(callback_context),
@@ -154,15 +116,13 @@ NotificationPermissionInfoBarDelegate::
     UMA_HISTOGRAM_COUNTS("NotificationPermissionRequest.Ignored", 1);
 
   RenderViewHost* host = RenderViewHost::FromID(process_id_, route_id_);
-  if (host) {
-    host->Send(new DesktopNotificationMsg_PermissionRequestDone(
-        route_id_, callback_context_));
-  }
+  if (host)
+    host->DesktopNotificationPermissionRequestDone(callback_context_);
 }
 
 gfx::Image* NotificationPermissionInfoBarDelegate::GetIcon() const {
   return &ResourceBundle::GetSharedInstance().GetNativeImageNamed(
-     IDR_PRODUCT_ICON_32);
+     IDR_PRODUCT_LOGO_32);
 }
 
 InfoBarDelegate::Type
@@ -255,8 +215,6 @@ DesktopNotificationService::DesktopNotificationService(Profile* profile,
     NotificationUIManager* ui_manager)
     : profile_(profile),
       ui_manager_(ui_manager) {
-  prefs_registrar_.Init(profile_->GetPrefs());
-  InitPrefs();
   StartObserving();
 }
 
@@ -264,150 +222,56 @@ DesktopNotificationService::~DesktopNotificationService() {
   StopObserving();
 }
 
-void DesktopNotificationService::RegisterUserPrefs(PrefService* user_prefs) {
-  content_settings::NotificationProvider::RegisterUserPrefs(user_prefs);
-}
-
-// Initialize the cache with the allowed and denied origins, or
-// create the preferences if they don't exist yet.
-void DesktopNotificationService::InitPrefs() {
-  provider_.reset(new content_settings::NotificationProvider(profile_));
-
-  std::vector<GURL> allowed_origins;
-  std::vector<GURL> denied_origins;
-  ContentSetting default_content_setting = CONTENT_SETTING_DEFAULT;
-
-  if (!profile_->IsOffTheRecord()) {
-    default_content_setting =
-        profile_->GetHostContentSettingsMap()->GetDefaultContentSetting(
-            CONTENT_SETTINGS_TYPE_NOTIFICATIONS);
-    allowed_origins = GetAllowedOrigins();
-    denied_origins = GetBlockedOrigins();
-  }
-
-  prefs_cache_ = new NotificationsPrefsCache();
-  prefs_cache_->SetCacheDefaultContentSetting(default_content_setting);
-  prefs_cache_->SetCacheAllowedOrigins(allowed_origins);
-  prefs_cache_->SetCacheDeniedOrigins(denied_origins);
-  prefs_cache_->set_is_initialized(true);
-}
-
 void DesktopNotificationService::StartObserving() {
   if (!profile_->IsOffTheRecord()) {
-    prefs_registrar_.Add(prefs::kDesktopNotificationAllowedOrigins, this);
-    prefs_registrar_.Add(prefs::kDesktopNotificationDeniedOrigins, this);
-    notification_registrar_.Add(this, NotificationType::EXTENSION_UNLOADED,
+    notification_registrar_.Add(this, chrome::NOTIFICATION_EXTENSION_UNLOADED,
                                 NotificationService::AllSources());
-    notification_registrar_.Add(
-        this,
-        NotificationType::CONTENT_SETTINGS_CHANGED,
-        // TODO(markusheintz): Remember to change to HostContentSettingsMap.
-        NotificationService::AllSources());
   }
-  notification_registrar_.Add(this, NotificationType::PROFILE_DESTROYED,
+  notification_registrar_.Add(this, chrome::NOTIFICATION_PROFILE_DESTROYED,
                               Source<Profile>(profile_));
 }
 
 void DesktopNotificationService::StopObserving() {
-  if (!profile_->IsOffTheRecord()) {
-    prefs_registrar_.RemoveAll();
-  }
   notification_registrar_.RemoveAll();
 }
 
 void DesktopNotificationService::GrantPermission(const GURL& origin) {
-  ContentSettingsPattern pattern =
-      content_settings::NotificationProvider::ToContentSettingsPattern(origin);
-  provider_->SetContentSetting(
-      pattern,
-      pattern,
+  ContentSettingsPattern primary_pattern =
+      ContentSettingsPattern::FromURLNoWildcard(origin);
+  profile_->GetHostContentSettingsMap()->SetContentSetting(
+      primary_pattern,
+      ContentSettingsPattern::Wildcard(),
       CONTENT_SETTINGS_TYPE_NOTIFICATIONS,
       NO_RESOURCE_IDENTIFIER,
       CONTENT_SETTING_ALLOW);
-
-  // Schedule a cache update on the IO thread.
-  BrowserThread::PostTask(
-      BrowserThread::IO, FROM_HERE,
-      NewRunnableMethod(
-          prefs_cache_.get(),
-          &NotificationsPrefsCache::CacheAllowedOrigin,
-          origin));
 }
 
 void DesktopNotificationService::DenyPermission(const GURL& origin) {
-  // Update content settings
-  ContentSettingsPattern pattern =
-      content_settings::NotificationProvider::ToContentSettingsPattern(origin);
-  provider_->SetContentSetting(
-      pattern,
-      pattern,
+  ContentSettingsPattern primary_pattern =
+      ContentSettingsPattern::FromURLNoWildcard(origin);
+  profile_->GetHostContentSettingsMap()->SetContentSetting(
+      primary_pattern,
+      ContentSettingsPattern::Wildcard(),
       CONTENT_SETTINGS_TYPE_NOTIFICATIONS,
       NO_RESOURCE_IDENTIFIER,
       CONTENT_SETTING_BLOCK);
-
-  // Schedule a cache update on the IO thread.
-  BrowserThread::PostTask(
-      BrowserThread::IO, FROM_HERE,
-      NewRunnableMethod(
-          prefs_cache_.get(),
-          &NotificationsPrefsCache::CacheDeniedOrigin,
-          origin));
 }
 
-void DesktopNotificationService::Observe(NotificationType type,
+void DesktopNotificationService::Observe(int type,
                                          const NotificationSource& source,
                                          const NotificationDetails& details) {
-  if (NotificationType::PREF_CHANGED == type) {
-    const std::string& name = *Details<std::string>(details).ptr();
-    OnPrefsChanged(name);
-  } else if (NotificationType::CONTENT_SETTINGS_CHANGED == type) {
-    // TODO(markusheintz): Check if content settings type default was changed;
-    const ContentSetting default_content_setting =
-        profile_->GetHostContentSettingsMap()->GetDefaultContentSetting(
-            CONTENT_SETTINGS_TYPE_NOTIFICATIONS);
-    // Schedule a cache update on the IO thread.
-    BrowserThread::PostTask(
-        BrowserThread::IO, FROM_HERE,
-        NewRunnableMethod(
-            prefs_cache_.get(),
-            &NotificationsPrefsCache::SetCacheDefaultContentSetting,
-            default_content_setting));
-  } else if (NotificationType::EXTENSION_UNLOADED == type) {
+  if (chrome::NOTIFICATION_EXTENSION_UNLOADED == type) {
     // Remove all notifications currently shown or queued by the extension
     // which was unloaded.
     const Extension* extension =
         Details<UnloadedExtensionInfo>(details)->extension;
     if (extension)
       ui_manager_->CancelAllBySourceOrigin(extension->url());
-  } else if (NotificationType::PROFILE_DESTROYED == type) {
+  } else if (chrome::NOTIFICATION_PROFILE_DESTROYED == type) {
     StopObserving();
   }
 }
 
-void DesktopNotificationService::OnPrefsChanged(const std::string& pref_name) {
-  if (pref_name == prefs::kDesktopNotificationAllowedOrigins) {
-    // Schedule a cache update on the IO thread.
-    std::vector<GURL> allowed_origins(GetAllowedOrigins());
-    BrowserThread::PostTask(
-        BrowserThread::IO, FROM_HERE,
-        NewRunnableMethod(
-            prefs_cache_.get(),
-            &NotificationsPrefsCache::SetCacheAllowedOrigins,
-            allowed_origins));
-  } else if (pref_name == prefs::kDesktopNotificationDeniedOrigins) {
-    // Schedule a cache update on the IO thread.
-    std::vector<GURL> denied_origins(GetBlockedOrigins());
-    BrowserThread::PostTask(
-        BrowserThread::IO, FROM_HERE,
-        NewRunnableMethod(
-            prefs_cache_.get(),
-            &NotificationsPrefsCache::SetCacheDeniedOrigins,
-            denied_origins));
-  } else {
-    NOTREACHED();
-  }
-}
-
 ContentSetting DesktopNotificationService::GetDefaultContentSetting() {
   return profile_->GetHostContentSettingsMap()->GetDefaultContentSetting(
       CONTENT_SETTINGS_TYPE_NOTIFICATIONS);
@@ -429,70 +293,36 @@ void DesktopNotificationService::ResetToDefaultContentSetting() {
       CONTENT_SETTINGS_TYPE_NOTIFICATIONS, CONTENT_SETTING_DEFAULT);
 }
 
-std::vector<GURL> DesktopNotificationService::GetAllowedOrigins() {
-  content_settings::ProviderInterface::Rules content_setting_rules;
-  provider_->GetAllContentSettingsRules(
+void DesktopNotificationService::GetNotificationsSettings(
+    HostContentSettingsMap::SettingsForOneType* settings) {
+  profile_->GetHostContentSettingsMap()->GetSettingsForOneType(
       CONTENT_SETTINGS_TYPE_NOTIFICATIONS,
       NO_RESOURCE_IDENTIFIER,
-      &content_setting_rules);
-  std::vector<GURL> allowed_origins;
-
-  GetOriginsWithSettingFromContentSettingsRules(
-      content_setting_rules, CONTENT_SETTING_ALLOW, &allowed_origins);
-
-  return allowed_origins;
+      settings);
 }
 
-std::vector<GURL> DesktopNotificationService::GetBlockedOrigins() {
-  content_settings::ProviderInterface::Rules content_settings_rules;
-  provider_->GetAllContentSettingsRules(
-      CONTENT_SETTINGS_TYPE_NOTIFICATIONS,
-      NO_RESOURCE_IDENTIFIER,
-      &content_settings_rules);
-  std::vector<GURL> denied_origins;
-
-  GetOriginsWithSettingFromContentSettingsRules(
-      content_settings_rules, CONTENT_SETTING_BLOCK, &denied_origins);
-
-  return denied_origins;
-}
-
-void DesktopNotificationService::ResetAllowedOrigin(const GURL& origin) {
-  ContentSettingsPattern pattern =
-      ContentSettingsPattern::FromURLNoWildcard(origin);
-  provider_->SetContentSetting(
-      pattern,
-      pattern,
-      CONTENT_SETTINGS_TYPE_NOTIFICATIONS,
-      NO_RESOURCE_IDENTIFIER,
-      CONTENT_SETTING_DEFAULT);
-}
-
-void DesktopNotificationService::ResetBlockedOrigin(const GURL& origin) {
-  ContentSettingsPattern pattern =
-      ContentSettingsPattern::FromURLNoWildcard(origin);
-  provider_->SetContentSetting(
-      pattern,
+void DesktopNotificationService::ClearSetting(
+    const ContentSettingsPattern& pattern) {
+  profile_->GetHostContentSettingsMap()->SetContentSetting(
       pattern,
+      ContentSettingsPattern::Wildcard(),
       CONTENT_SETTINGS_TYPE_NOTIFICATIONS,
       NO_RESOURCE_IDENTIFIER,
       CONTENT_SETTING_DEFAULT);
 }
 
 void DesktopNotificationService::ResetAllOrigins() {
-  provider_->ClearAllContentSettingsRules(CONTENT_SETTINGS_TYPE_NOTIFICATIONS);
+  profile_->GetHostContentSettingsMap()->ClearSettingsForOneType(
+      CONTENT_SETTINGS_TYPE_NOTIFICATIONS);
 }
 
 ContentSetting DesktopNotificationService::GetContentSetting(
     const GURL& origin) {
-  ContentSetting provided_setting = provider_->GetContentSetting(
+  return profile_->GetHostContentSettingsMap()->GetContentSetting(
       origin,
       origin,
       CONTENT_SETTINGS_TYPE_NOTIFICATIONS,
       NO_RESOURCE_IDENTIFIER);
-  if (CONTENT_SETTING_DEFAULT == provided_setting)
-    return GetDefaultContentSetting();
-  return provided_setting;
 }
 
 void DesktopNotificationService::RequestPermission(
@@ -508,9 +338,6 @@ void DesktopNotificationService::RequestPermission(
   if (!tab)
     return;
 
-  TabContentsWrapper* wrapper =
-      TabContentsWrapper::GetCurrentWrapperForContents(tab);
-
   // If |origin| hasn't been seen before and the default content setting for
   // notifications is "ask", show an infobar.
   // The cache can only answer queries on the IO thread once it's initialized,
@@ -518,17 +345,17 @@ void DesktopNotificationService::RequestPermission(
   ContentSetting setting = GetContentSetting(origin);
   if (setting == CONTENT_SETTING_ASK) {
     // Show an info bar requesting permission.
-    wrapper->AddInfoBar(
+    TabContentsWrapper* wrapper =
+        TabContentsWrapper::GetCurrentWrapperForContents(tab);
+    wrapper->infobar_tab_helper()->AddInfoBar(
         new NotificationPermissionInfoBarDelegate(
             tab, origin, DisplayNameForOrigin(origin), process_id,
             route_id, callback_context));
   } else {
     // Notify renderer immediately.
     RenderViewHost* host = RenderViewHost::FromID(process_id, route_id);
-    if (host) {
-      host->Send(new DesktopNotificationMsg_PermissionRequestDone(
-          route_id, callback_context));
-    }
+    if (host)
+      host->DesktopNotificationPermissionRequestDone(callback_context);
   }
 }
 
@@ -545,7 +372,6 @@ bool DesktopNotificationService::CancelDesktopNotification(
   return ui_manager_->CancelById(proxy->id());
 }
 
-
 bool DesktopNotificationService::ShowDesktopNotification(
     const DesktopNotificationHostMsg_Show_Params& params,
     int process_id, int route_id, DesktopNotificationSource source) {
@@ -587,7 +413,28 @@ string16 DesktopNotificationService::DisplayNameForOrigin(
 
 void DesktopNotificationService::NotifySettingsChange() {
   NotificationService::current()->Notify(
-      NotificationType::DESKTOP_NOTIFICATION_SETTINGS_CHANGED,
+      chrome::NOTIFICATION_DESKTOP_NOTIFICATION_SETTINGS_CHANGED,
       Source<DesktopNotificationService>(this),
       NotificationService::NoDetails());
 }
+
+WebKit::WebNotificationPresenter::Permission
+    DesktopNotificationService::HasPermission(const GURL& origin) {
+  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
+  HostContentSettingsMap* host_content_settings_map =
+      profile_->GetHostContentSettingsMap();
+  ContentSetting setting = host_content_settings_map->GetContentSetting(
+      origin,
+      origin,
+      CONTENT_SETTINGS_TYPE_NOTIFICATIONS,
+      NO_RESOURCE_IDENTIFIER);
+
+  if (setting == CONTENT_SETTING_ALLOW)
+    return WebKit::WebNotificationPresenter::PermissionAllowed;
+  if (setting == CONTENT_SETTING_BLOCK)
+    return WebKit::WebNotificationPresenter::PermissionDenied;
+  if (setting == CONTENT_SETTING_ASK)
+    return WebKit::WebNotificationPresenter::PermissionNotAllowed;
+  NOTREACHED() << "Invalid notifications settings value: " << setting;
+  return WebKit::WebNotificationPresenter::PermissionNotAllowed;
+}
