@@ -10,6 +10,7 @@
 #include "base/i18n/time_formatting.h"
 #include "base/string_number_conversions.h"
 #include "base/utf_string_conversions.h"
+#include "chrome/browser/page_info_model_observer.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ssl/ssl_error_info.h"
 #include "content/browser/cert_store.h"
@@ -19,8 +20,8 @@
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
 #include "net/base/cert_status_flags.h"
-#include "net/base/ssl_connection_status_flags.h"
 #include "net/base/ssl_cipher_suite_names.h"
+#include "net/base/ssl_connection_status_flags.h"
 #include "net/base/x509_certificate.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
@@ -339,7 +340,7 @@ void PageInfoModel::OnGotVisitCountToHost(HistoryService::Handle handle,
             base::TimeFormatShortDate(first_visit)),
         SECTION_INFO_FIRST_VISIT));
   }
-  observer_->ModelChanged();
+  observer_->OnPageInfoModelChanged();
 }
 
 PageInfoModel::PageInfoModel() : observer_(NULL) {
