@@ -7,12 +7,13 @@
 #include "base/basictypes.h"
 #include "base/stl_util.h"
 #include "base/string_util.h"
+#include "base/stringprintf.h"
 #include "base/time.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/password_manager/native_backend_gnome_x.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/common/pref_names.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/testing_profile.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 using webkit_glue::PasswordForm;
@@ -46,7 +47,7 @@ struct MockKeyringItem {
                               : value_uint32 == x.value_uint32;
     }
 
-    enum { UINT32, STRING } type;
+    enum Type { UINT32, STRING } type;
     uint32_t value_uint32;
     std::string value_string;
   };
@@ -101,8 +102,9 @@ gpointer mock_gnome_keyring_store_password(
   mock_keyring_items.push_back(
       MockKeyringItem(keyring, display_name, password));
   MockKeyringItem* item = &mock_keyring_items.back();
-  const std::string keyring_desc = keyring ? StringPrintf("keyring %s", keyring)
-                                           : std::string("default keyring");
+  const std::string keyring_desc =
+      keyring ? base::StringPrintf("keyring %s", keyring)
+              : std::string("default keyring");
   VLOG(1) << "Adding item with origin " << display_name
           << " to " << keyring_desc;
   va_list ap;
