@@ -7,7 +7,7 @@
 
 #include "base/memory/scoped_ptr.h"
 #include "base/message_loop.h"
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
 #include "base/string_number_conversions.h"
 #include "base/string_piece.h"
 #include "base/utf_string_conversions.h"
@@ -462,9 +462,9 @@ INSTANTIATE_TEST_CASE_P(
         PolicyTestParams::ForBooleanPolicy(
             kPolicyInstantEnabled,
             key::kInstantEnabled),
-        PolicyTestParams::ForBooleanPolicy(
-            kPolicyIncognitoEnabled,
-            key::kIncognitoEnabled),
+        PolicyTestParams::ForIntegerPolicy(
+            kPolicyIncognitoModeAvailability,
+            key::kIncognitoModeAvailability),
         PolicyTestParams::ForBooleanPolicy(
             kPolicyDisablePluginFinder,
             key::kDisablePluginFinder),
@@ -503,6 +503,22 @@ INSTANTIATE_TEST_CASE_P(
             key::kDisabledSchemes),
         PolicyTestParams::ForStringPolicy(
             kPolicyDiskCacheDir,
-            key::kDiskCacheDir)));
+            key::kDiskCacheDir),
+        PolicyTestParams::ForIntegerPolicy(
+            kPolicyMaxConnectionsPerProxy,
+            key::kMaxConnectionsPerProxy)));
+
+// testing::Values has a limit of 50 test templates, which is reached by the
+// instantiations above. Add tests for new policies here:
+INSTANTIATE_TEST_CASE_P(
+    ConfigurationPolicyProviderWinTestInstance2,
+    ConfigurationPolicyProviderWinTest,
+    testing::Values(
+        PolicyTestParams::ForListPolicy(
+            kPolicyURLBlacklist,
+            key::kURLBlacklist),
+        PolicyTestParams::ForListPolicy(
+            kPolicyURLWhitelist,
+            key::kURLWhitelist)));
 
 }  // namespace policy
