@@ -10,17 +10,18 @@
 #include "chrome/browser/policy/device_management_backend_mock.h"
 #include "chrome/browser/policy/device_management_service.h"
 #include "chrome/browser/policy/proto/device_management_constants.h"
+#include "chrome/test/base/testing_browser_process.h"
 #include "content/browser/browser_thread.h"
-#include "content/common/test_url_fetcher_factory.h"
+#include "content/test/test_url_fetcher_factory.h"
 #include "net/base/escape.h"
 #include "net/url_request/url_request_status.h"
 #include "net/url_request/url_request_test_util.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
-using testing::_;
 using testing::IgnoreResult;
 using testing::InvokeWithoutArgs;
+using testing::_;
 
 namespace policy {
 
@@ -32,7 +33,8 @@ const char kResponseEmpty[] = "\x08\x00";
 #define PROTO_STRING(name) (std::string(name, arraysize(name) - 1))
 
 // Some helper constants.
-const char kAuthToken[] = "auth-token";
+const char kGaiaAuthToken[] = "gaia-auth-token";
+const char kOAuthToken[] = "oauth-token";
 const char kDMToken[] = "device-management-token";
 const char kDeviceId[] = "device-id";
 
@@ -49,15 +51,11 @@ class DeviceManagementServiceTestBase : public TESTBASE {
     InitializeService();
   }
 
-  virtual void SetUp() {
-    URLFetcher::set_factory(&factory_);
-  }
-
   virtual void TearDown() {
-    URLFetcher::set_factory(NULL);
     backend_.reset();
     service_.reset();
     loop_.RunAllPending();
+    TESTBASE::TearDown();
   }
 
   void ResetService() {
@@ -108,7 +106,8 @@ TEST_P(DeviceManagementServiceFailedRequestTest, RegisterRequest) {
   DeviceRegisterResponseDelegateMock mock;
   EXPECT_CALL(mock, OnError(GetParam().expected_error_));
   em::DeviceRegisterRequest request;
-  backend_->ProcessRegisterRequest(kAuthToken, kDeviceId, request, &mock);
+  backend_->ProcessRegisterRequest(kGaiaAuthToken, kOAuthToken,
+                                   kDeviceId, request, &mock);
   TestURLFetcher* fetcher = factory_.GetFetcherByID(0);
   ASSERT_TRUE(fetcher);
 
@@ -144,7 +143,9 @@ TEST_P(DeviceManagementServiceFailedRequestTest, PolicyRequest) {
   em::DevicePolicySettingRequest* setting_request =
       request.add_setting_request();
   setting_request->set_key(kChromeDevicePolicySettingKey);
-  backend_->ProcessPolicyRequest(kDMToken, kDeviceId, request, &mock);
+  backend_->ProcessPolicyRequest(kDMToken, kDeviceId,
+                                 CloudPolicyDataStore::USER_AFFILIATION_NONE,
+                                 request, &mock);
   TestURLFetcher* fetcher = factory_.GetFetcherByID(0);
   ASSERT_TRUE(fetcher);
 
@@ -180,6 +181,16 @@ INSTANTIATE_TEST_CASE_P(
             net::URLRequestStatus::SUCCESS,
             403,
             PROTO_STRING(kResponseEmpty)),
+        FailedRequestParams(
+            DeviceManagementBackend::kErrorServiceInvalidSerialNumber,
+            net::URLRequestStatus::SUCCESS,
+            405,
+            PROTO_STRING(kResponseEmpty)),
+        FailedRequestParams(
+            DeviceManagementBackend::kErrorServiceDeviceIdConflict,
+            net::URLRequestStatus::SUCCESS,
+            409,
+            PROTO_STRING(kResponseEmpty)),
         FailedRequestParams(
             DeviceManagementBackend::kErrorServiceDeviceNotFound,
             net::URLRequestStatus::SUCCESS,
@@ -306,7 +317,8 @@ TEST_F(DeviceManagementServiceTest, RegisterRequest) {
   expected_response.set_device_management_token(kDMToken);
   EXPECT_CALL(mock, HandleRegisterResponse(MessageEquals(expected_response)));
   em::DeviceRegisterRequest request;
-  backend_->ProcessRegisterRequest(kDMToken, kDeviceId, request, &mock);
+  backend_->ProcessRegisterRequest(kGaiaAuthToken, kOAuthToken,
+                                   kDeviceId, request, &mock);
   TestURLFetcher* fetcher = factory_.GetFetcherByID(0);
   ASSERT_TRUE(fetcher);
 
@@ -381,7 +393,8 @@ TEST_F(DeviceManagementServiceTest, CancelRegisterRequest) {
   DeviceRegisterResponseDelegateMock mock;
   EXPECT_CALL(mock, HandleRegisterResponse(_)).Times(0);
   em::DeviceRegisterRequest request;
-  backend_->ProcessRegisterRequest(kAuthToken, kDeviceId, request, &mock);
+  backend_->ProcessRegisterRequest(kGaiaAuthToken, kOAuthToken,
+                                   kDeviceId, request, &mock);
   TestURLFetcher* fetcher = factory_.GetFetcherByID(0);
   ASSERT_TRUE(fetcher);
 
@@ -410,7 +423,9 @@ TEST_F(DeviceManagementServiceTest, CancelPolicyRequest) {
       request.add_setting_request();
   setting_request->set_key(kChromeDevicePolicySettingKey);
   setting_request->set_watermark("stale");
-  backend_->ProcessPolicyRequest(kDMToken, kDeviceId, request, &mock);
+  backend_->ProcessPolicyRequest(kDMToken, kDeviceId,
+                                 CloudPolicyDataStore::USER_AFFILIATION_NONE,
+                                 request, &mock);
   TestURLFetcher* fetcher = factory_.GetFetcherByID(0);
   ASSERT_TRUE(fetcher);
 
@@ -428,7 +443,8 @@ TEST_F(DeviceManagementServiceTest, JobQueueing) {
   expected_response.set_device_management_token(kDMToken);
   EXPECT_CALL(mock, HandleRegisterResponse(MessageEquals(expected_response)));
   em::DeviceRegisterRequest request;
-  backend_->ProcessRegisterRequest(kAuthToken, kDeviceId, request, &mock);
+  backend_->ProcessRegisterRequest(kGaiaAuthToken, kOAuthToken,
+                                   kDeviceId, request, &mock);
   TestURLFetcher* fetcher = factory_.GetFetcherByID(0);
   ASSERT_FALSE(fetcher);
 
@@ -461,7 +477,9 @@ TEST_F(DeviceManagementServiceTest, CancelRequestAfterShutdown) {
       request.add_setting_request();
   setting_request->set_key(kChromeDevicePolicySettingKey);
   setting_request->set_watermark("stale");
-  backend_->ProcessPolicyRequest(kDMToken, kDeviceId, request, &mock);
+  backend_->ProcessPolicyRequest(kDMToken, kDeviceId,
+                                 CloudPolicyDataStore::USER_AFFILIATION_NONE,
+                                 request, &mock);
   TestURLFetcher* fetcher = factory_.GetFetcherByID(0);
   ASSERT_TRUE(fetcher);
 
@@ -478,7 +496,8 @@ TEST_F(DeviceManagementServiceTest, CancelDuringCallback) {
                                   &DeviceManagementServiceTest::ResetBackend))
       .RetiresOnSaturation();
   em::DeviceRegisterRequest request;
-  backend_->ProcessRegisterRequest(kAuthToken, kDeviceId, request, &mock);
+  backend_->ProcessRegisterRequest(kGaiaAuthToken, kOAuthToken,
+                                   kDeviceId, request, &mock);
   TestURLFetcher* fetcher = factory_.GetFetcherByID(0);
   ASSERT_TRUE(fetcher);
 
