@@ -23,7 +23,7 @@ class DefaultPrefStore : public ValueMapPrefStore {
 
   // Returns the registered type for |key| or Value::TYPE_NULL if the |key|
   // has not been registered.
-  Value::ValueType GetType(const std::string& key) const;
+  base::Value::Type GetType(const std::string& key) const;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(DefaultPrefStore);
