@@ -3,6 +3,7 @@
 // found in the LICENSE file.
 
 #include "chrome/browser/prefs/pref_observer_mock.h"
+#include "chrome/common/chrome_notification_types.h"
 
 PrefObserverMock::PrefObserverMock() {}
 
@@ -11,7 +12,7 @@ PrefObserverMock::~PrefObserverMock() {}
 void PrefObserverMock::Expect(const PrefService* prefs,
                               const std::string& pref_name,
                               const Value* value) {
-  EXPECT_CALL(*this, Observe(NotificationType(NotificationType::PREF_CHANGED),
+  EXPECT_CALL(*this, Observe(int(chrome::NOTIFICATION_PREF_CHANGED),
                              Source<PrefService>(prefs),
                              Property(&Details<std::string>::ptr,
                                       Pointee(pref_name))))
