@@ -13,7 +13,7 @@
 #include "base/logging.h"
 #include "base/message_loop.h"
 #include "base/metrics/histogram.h"
-#include "base/stl_util-inl.h"
+#include "base/stl_util.h"
 #include "base/string_number_conversions.h"
 #include "base/string_util.h"
 #include "base/value_conversions.h"
@@ -23,7 +23,7 @@
 #include "chrome/browser/policy/configuration_policy_pref_store.h"
 #include "chrome/browser/prefs/command_line_pref_store.h"
 #include "chrome/browser/prefs/default_pref_store.h"
-#include "chrome/browser/prefs/overlay_persistent_pref_store.h"
+#include "chrome/browser/prefs/incognito_user_pref_store.h"
 #include "chrome/browser/prefs/pref_model_associator.h"
 #include "chrome/browser/prefs/pref_notifier_impl.h"
 #include "chrome/browser/prefs/pref_value_store.h"
@@ -40,7 +40,7 @@ namespace {
 // A helper function for RegisterLocalized*Pref that creates a Value* based on
 // the string value in the locale dll.  Because we control the values in a
 // locale dll, this should always return a Value of the appropriate type.
-Value* CreateLocaleDefaultValue(Value::ValueType type, int message_id) {
+Value* CreateLocaleDefaultValue(base::Value::Type type, int message_id) {
   std::string resource_string = l10n_util::GetStringUTF8(message_id);
   DCHECK(!resource_string.empty());
   switch (type) {
@@ -112,7 +112,6 @@ class ReadErrorHandler : public PersistentPrefStore::ReadErrorDelegate {
 // static
 PrefService* PrefService::CreatePrefService(const FilePath& pref_filename,
                                             PrefStore* extension_prefs,
-                                            Profile* profile,
                                             bool async) {
   using policy::ConfigurationPolicyPrefStore;
 
@@ -129,20 +128,27 @@ PrefService* PrefService::CreatePrefService(const FilePath& pref_filename,
   }
 #endif
 
+#if defined(ENABLE_CONFIGURATION_POLICY)
   ConfigurationPolicyPrefStore* managed_platform =
       ConfigurationPolicyPrefStore::CreateManagedPlatformPolicyPrefStore();
   ConfigurationPolicyPrefStore* managed_cloud =
-      ConfigurationPolicyPrefStore::CreateManagedCloudPolicyPrefStore(profile);
+      ConfigurationPolicyPrefStore::CreateManagedCloudPolicyPrefStore();
+  ConfigurationPolicyPrefStore* recommended_platform =
+      ConfigurationPolicyPrefStore::CreateRecommendedPlatformPolicyPrefStore();
+  ConfigurationPolicyPrefStore* recommended_cloud =
+      ConfigurationPolicyPrefStore::CreateRecommendedCloudPolicyPrefStore();
+#else
+  ConfigurationPolicyPrefStore* managed_platform = NULL;
+  ConfigurationPolicyPrefStore* managed_cloud = NULL;
+  ConfigurationPolicyPrefStore* recommended_platform = NULL;
+  ConfigurationPolicyPrefStore* recommended_cloud = NULL;
+#endif  // ENABLE_CONFIGURATION_POLICY
+
   CommandLinePrefStore* command_line =
       new CommandLinePrefStore(CommandLine::ForCurrentProcess());
   JsonPrefStore* user = new JsonPrefStore(
       pref_filename,
       BrowserThread::GetMessageLoopProxyForThread(BrowserThread::FILE));
-  ConfigurationPolicyPrefStore* recommended_platform =
-      ConfigurationPolicyPrefStore::CreateRecommendedPlatformPolicyPrefStore();
-  ConfigurationPolicyPrefStore* recommended_cloud =
-      ConfigurationPolicyPrefStore::CreateRecommendedCloudPolicyPrefStore(
-          profile);
   DefaultPrefStore* default_pref_store = new DefaultPrefStore();
 
   return new PrefService(
@@ -186,7 +192,7 @@ PrefService::PrefService(PrefStore* managed_platform_prefs,
 PrefService::PrefService(const PrefService& original,
                          PrefStore* incognito_extension_prefs)
       : user_pref_store_(
-            new OverlayPersistentPrefStore(original.user_pref_store_.get())),
+            new IncognitoUserPrefStore(original.user_pref_store_.get())),
         default_store_(original.default_store_.get()) {
   // Incognito mode doesn't sync, so no need to create PrefModelAssociator.
   pref_notifier_.reset(new PrefNotifierImpl(this));
@@ -632,7 +638,7 @@ const PrefService::Preference* PrefService::FindPreference(
   PreferenceSet::const_iterator it = prefs_.find(&p);
   if (it != prefs_.end())
     return *it;
-  const Value::ValueType type = default_store_->GetType(pref_name);
+  const base::Value::Type type = default_store_->GetType(pref_name);
   if (type == Value::TYPE_NULL)
     return NULL;
   Preference* new_pref = new Preference(this, pref_name, type);
@@ -704,7 +710,7 @@ void PrefService::RegisterPreference(const char* path,
     return;
   }
 
-  Value::ValueType orig_type = default_value->GetType();
+  base::Value::Type orig_type = default_value->GetType();
   DCHECK(orig_type != Value::TYPE_NULL && orig_type != Value::TYPE_BINARY) <<
          "invalid preference type: " << orig_type;
 
@@ -773,11 +779,9 @@ int64 PrefService::GetInt64(const char* path) const {
 }
 
 Value* PrefService::GetMutableUserPref(const char* path,
-                                       Value::ValueType type) {
+                                       base::Value::Type type) {
   CHECK(type == Value::TYPE_DICTIONARY || type == Value::TYPE_LIST);
   DCHECK(CalledOnValidThread());
-  DLOG_IF(WARNING, IsManagedPreference(path)) <<
-      "Attempt to change managed preference " << path;
 
   const Preference* pref = FindPreference(path);
   if (!pref) {
@@ -814,8 +818,6 @@ void PrefService::ReportUserPrefChanged(const std::string& key) {
 void PrefService::SetUserPrefValue(const char* path, Value* new_value) {
   scoped_ptr<Value> owned_value(new_value);
   DCHECK(CalledOnValidThread());
-  DLOG_IF(WARNING, IsManagedPreference(path)) <<
-      "Attempt to change managed preference " << path;
 
   const Preference* pref = FindPreference(path);
   if (!pref) {
@@ -841,7 +843,7 @@ SyncableService* PrefService::GetSyncableService() {
 
 PrefService::Preference::Preference(const PrefService* service,
                                     const char* name,
-                                    Value::ValueType type)
+                                    base::Value::Type type)
       : name_(name),
         type_(type),
         pref_service_(service) {
@@ -849,7 +851,7 @@ PrefService::Preference::Preference(const PrefService* service,
   DCHECK(service);
 }
 
-Value::ValueType PrefService::Preference::GetType() const {
+base::Value::Type PrefService::Preference::GetType() const {
   return type_;
 }
 
