@@ -16,6 +16,7 @@
 #include "chrome/browser/prerender/prerender_manager.h"
 #include "chrome/browser/prerender/prerender_render_view_host_observer.h"
 #include "chrome/browser/prerender/prerender_tracker.h"
+#include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_list.h"
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
@@ -257,12 +258,6 @@ void PrerenderContents::StartPrerendering(
       this, content::NOTIFICATION_RENDER_VIEW_HOST_CREATED_FOR_TAB,
       Source<TabContents>(new_contents));
 
-  // Register to be told when the RenderView is ready, so we can hide it.
-  // It will automatically be set to visible when we resize it, otherwise.
-  notification_registrar_.Add(this,
-                              content::NOTIFICATION_TAB_CONTENTS_CONNECTED,
-                              Source<TabContents>(new_contents));
-
   // Register for redirect notifications sourced from |this|.
   notification_registrar_.Add(
       this, content::NOTIFICATION_RESOURCE_RECEIVED_REDIRECT,
@@ -377,22 +372,20 @@ void PrerenderContents::Observe(int type,
         // first navigation, so there's no need to send the message just after
         // the TabContents is created.
         new_render_view_host->Send(
-            new ViewMsg_SetIsPrerendering(new_render_view_host->routing_id(),
-                                          true));
-      }
-      break;
-    }
+            new ChromeViewMsg_SetIsPrerendering(
+                new_render_view_host->routing_id(),
+                true));
 
-    case content::NOTIFICATION_TAB_CONTENTS_CONNECTED: {
-      if (prerender_contents_.get()) {
-        DCHECK_EQ(Source<TabContents>(source).ptr(),
-                  prerender_contents_->tab_contents());
-        // Set the new TabContents and its RenderViewHost as hidden, to reduce
-        // resource usage.  This can only be done after the size has been sent
-        // to the RenderView, which is why it's done here.
+        // Make sure the size of the RenderViewHost has been passed to the new
+        // RenderView.  Otherwise, the size may not be sent until the
+        // RenderViewReady event makes it from the render process to the UI
+        // thread of the browser process.  When the RenderView receives its
+        // size, is also sets itself to be visible, which would then break the
+        // visibility API.
+        new_render_view_host->WasResized();
         prerender_contents_->tab_contents()->HideContents();
       }
-      return;
+      break;
     }
 
     case content::NOTIFICATION_CREATING_NEW_WINDOW_CANCELLED: {
@@ -512,9 +505,6 @@ void PrerenderContents::Destroy(FinalStatus final_status) {
   if (prerendering_has_been_cancelled_)
     return;
 
-  prerendering_has_been_cancelled_ = true;
-  prerender_manager_->MoveEntryToPendingDelete(this);
-
   if (child_id_ != -1 && route_id_ != -1) {
     // Cancel the prerender in the PrerenderTracker.  This is needed
     // because destroy may be called directly from the UI thread without calling
@@ -533,6 +523,11 @@ void PrerenderContents::Destroy(FinalStatus final_status) {
   }
   set_final_status(final_status);
 
+  prerendering_has_been_cancelled_ = true;
+  // This has to be done after setting the final status, as it adds the
+  // prerender to the history.
+  prerender_manager_->MoveEntryToPendingDelete(this);
+
   // We may destroy the PrerenderContents before we have initialized the
   // RenderViewHost. Otherwise set the Observer's PrerenderContents to NULL to
   // avoid any more messages being sent.
