@@ -24,6 +24,9 @@ bool MaybeGetQueryStringBasedAliasURL(const GURL& url, GURL* alias_url);
 // is not an integer in the range 1 to 9.
 uint8 GetQueryStringBasedExperiment(const GURL& url);
 
+// Indicates whether the URL provided could be a Google search result page.
+bool IsGoogleSearchResultURL(const GURL& url);
+
 }  // namespace prerender
 
 #endif  // CHROME_BROWSER_PRERENDER_PRERENDER_UTIL_H_
