@@ -42,6 +42,10 @@ class CloudPrintDataSenderHelper {
   virtual void CallJavascriptFunction(const std::wstring& function_name,
                                       const base::Value& arg1,
                                       const base::Value& arg2);
+  virtual void CallJavascriptFunction(const std::wstring& function_name,
+                                      const base::Value& arg1,
+                                      const base::Value& arg2,
+                                      const base::Value& arg3);
 
  private:
   WebUI* web_ui_;
@@ -59,6 +63,7 @@ class CloudPrintDataSender
   // lifetime of the helper.
   CloudPrintDataSender(CloudPrintDataSenderHelper* helper,
                        const string16& print_job_title,
+                       const string16& print_ticket,
                        const std::string& file_type);
 
   // Calls to read in the PDF file (on the FILE thread) then send that
@@ -80,6 +85,7 @@ class CloudPrintDataSender
   CloudPrintDataSenderHelper* volatile helper_;
   scoped_ptr<base::StringValue> print_data_;
   string16 print_job_title_;
+  string16 print_ticket_;
   std::string file_type_;
 
   DISALLOW_COPY_AND_ASSIGN(CloudPrintDataSender);
@@ -97,9 +103,10 @@ class CloudPrintHtmlDialogDelegate;
 class CloudPrintFlowHandler : public WebUIMessageHandler,
                               public NotificationObserver {
  public:
-  explicit CloudPrintFlowHandler(const FilePath& path_to_file,
-                                 const string16& print_job_title,
-                                 const std::string& file_type);
+  CloudPrintFlowHandler(const FilePath& path_to_file,
+                        const string16& print_job_title,
+                        const string16& print_ticket,
+                        const std::string& file_type);
   virtual ~CloudPrintFlowHandler();
 
   // WebUIMessageHandler implementation.
@@ -132,6 +139,7 @@ class CloudPrintFlowHandler : public WebUIMessageHandler,
   NotificationRegistrar registrar_;
   FilePath path_to_file_;
   string16 print_job_title_;
+  string16 print_ticket_;
   std::string file_type_;
   scoped_refptr<CloudPrintDataSender> print_data_sender_;
   scoped_ptr<CloudPrintDataSenderHelper> print_data_helper_;
@@ -148,22 +156,24 @@ class CloudPrintHtmlDialogDelegate : public HtmlDialogUIDelegate {
                                int width, int height,
                                const std::string& json_arguments,
                                const string16& print_job_title,
+                               const string16& print_ticket,
                                const std::string& file_type,
                                bool modal);
   virtual ~CloudPrintHtmlDialogDelegate();
 
   // HTMLDialogUIDelegate implementation:
-  virtual bool IsDialogModal() const;
-  virtual std::wstring GetDialogTitle() const;
-  virtual GURL GetDialogContentURL() const;
+  virtual bool IsDialogModal() const OVERRIDE;
+  virtual string16 GetDialogTitle() const OVERRIDE;
+  virtual GURL GetDialogContentURL() const OVERRIDE;
   virtual void GetWebUIMessageHandlers(
-      std::vector<WebUIMessageHandler*>* handlers) const;
-  virtual void GetDialogSize(gfx::Size* size) const;
-  virtual std::string GetDialogArgs() const;
-  virtual void OnDialogClosed(const std::string& json_retval);
-  virtual void OnCloseContents(TabContents* source, bool* out_close_dialog);
-  virtual bool ShouldShowDialogTitle() const;
-  virtual bool HandleContextMenu(const ContextMenuParams& params);
+      std::vector<WebUIMessageHandler*>* handlers) const OVERRIDE;
+  virtual void GetDialogSize(gfx::Size* size) const OVERRIDE;
+  virtual std::string GetDialogArgs() const OVERRIDE;
+  virtual void OnDialogClosed(const std::string& json_retval) OVERRIDE;
+  virtual void OnCloseContents(TabContents* source, bool* out_close_dialog)
+      OVERRIDE;
+  virtual bool ShouldShowDialogTitle() const OVERRIDE;
+  virtual bool HandleContextMenu(const ContextMenuParams& params) OVERRIDE;
 
  private:
   friend class ::CloudPrintHtmlDialogDelegateTest;
@@ -188,6 +198,7 @@ class CloudPrintHtmlDialogDelegate : public HtmlDialogUIDelegate {
 
 void CreateDialogImpl(const FilePath& path_to_file,
                       const string16& print_job_title,
+                      const string16& print_ticket,
                       const std::string& file_type,
                       bool modal);
 
