@@ -9,6 +9,9 @@
 #include "base/command_line.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/browser_process.h"
+#include "chrome/browser/chrome_plugin_service_filter.h"
+#include "chrome/browser/printing/print_view_manager.h"
+#include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/sessions/restore_tab_helper.h"
 #include "chrome/browser/tabs/tab_strip_model.h"
 #include "chrome/browser/ui/browser.h"
@@ -19,19 +22,19 @@
 #include "chrome/common/chrome_content_client.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/url_constants.h"
-#include "content/browser/plugin_service.h"
 #include "content/browser/renderer_host/render_view_host.h"
 #include "content/browser/tab_contents/navigation_details.h"
 #include "content/browser/tab_contents/tab_contents.h"
+#include "content/common/content_notification_types.h"
 #include "content/common/notification_details.h"
 #include "content/common/notification_source.h"
 #include "webkit/plugins/npapi/plugin_group.h"
 #include "webkit/plugins/npapi/plugin_list.h"
-#include "webkit/plugins/npapi/webplugininfo.h"
+#include "webkit/plugins/webplugininfo.h"
 
 using webkit::npapi::PluginGroup;
 using webkit::npapi::PluginList;
-using webkit::npapi::WebPluginInfo;
+using webkit::WebPluginInfo;
 
 namespace {
 
@@ -52,16 +55,20 @@ void EnableInternalPDFPluginForTab(TabContentsWrapper* preview_tab) {
     std::vector<WebPluginInfo> plugins = internal_pdf_group->web_plugin_infos();
     DCHECK_EQ(plugins.size(), 1U);
 
-    PluginService::OverriddenPlugin plugin;
-    plugin.render_process_id = preview_tab->render_view_host()->process()->id();
-    plugin.render_view_id = preview_tab->render_view_host()->routing_id();
-    plugin.plugin = plugins[0];
-    plugin.plugin.enabled = WebPluginInfo::USER_ENABLED;
-
-    PluginService::GetInstance()->OverridePluginForTab(plugin);
+    webkit::WebPluginInfo plugin = plugins[0];
+    plugin.enabled = WebPluginInfo::USER_ENABLED;
+    ChromePluginServiceFilter::GetInstance()->OverridePluginForTab(
+        preview_tab->render_view_host()->process()->id(),
+        preview_tab->render_view_host()->routing_id(),
+        GURL(),
+        plugin);
   }
 }
 
+void ResetPreviewTabOverrideTitle(TabContentsWrapper* preview_tab) {
+  preview_tab->print_view_manager()->ResetTitleOverride();
+}
+
 }  // namespace
 
 namespace printing {
@@ -80,33 +87,35 @@ PrintPreviewTabController* PrintPreviewTabController::GetInstance() {
 }
 
 // static
-void PrintPreviewTabController::PrintPreview(TabContents* tab) {
-  if (tab->showing_interstitial_page())
+void PrintPreviewTabController::PrintPreview(TabContentsWrapper* tab) {
+  if (tab->tab_contents()->showing_interstitial_page())
     return;
 
-  printing::PrintPreviewTabController* tab_controller =
-      printing::PrintPreviewTabController::GetInstance();
+  PrintPreviewTabController* tab_controller =
+      PrintPreviewTabController::GetInstance();
   if (!tab_controller)
     return;
   tab_controller->GetOrCreatePreviewTab(tab);
 }
 
-TabContents* PrintPreviewTabController::GetOrCreatePreviewTab(
-    TabContents* initiator_tab) {
+TabContentsWrapper* PrintPreviewTabController::GetOrCreatePreviewTab(
+    TabContentsWrapper* initiator_tab) {
   DCHECK(initiator_tab);
 
   // Get the print preview tab for |initiator_tab|.
-  TabContents* preview_tab = GetPrintPreviewForTab(initiator_tab);
-  if (preview_tab) {
-    // Show current preview tab.
-    static_cast<RenderViewHostDelegate*>(preview_tab)->Activate();
-    return preview_tab;
-  }
-  return CreatePrintPreviewTab(initiator_tab);
+  TabContentsWrapper* preview_tab = GetPrintPreviewForTab(initiator_tab);
+  if (!preview_tab)
+    return CreatePrintPreviewTab(initiator_tab);
+
+  // Show current preview tab.
+  static_cast<RenderViewHostDelegate*>(preview_tab->tab_contents())->Activate();
+  return preview_tab;
 }
 
-TabContents* PrintPreviewTabController::GetPrintPreviewForTab(
-    TabContents* tab) const {
+TabContentsWrapper* PrintPreviewTabController::GetPrintPreviewForTab(
+    TabContentsWrapper* tab) const {
+  // |preview_tab_map_| is keyed by the preview tab, so if find() succeeds, then
+  // |tab| is the preview tab.
   PrintPreviewTabMap::const_iterator it = preview_tab_map_.find(tab);
   if (it != preview_tab_map_.end())
     return tab;
@@ -124,19 +133,27 @@ TabContents* PrintPreviewTabController::GetPrintPreviewForTab(
 void PrintPreviewTabController::Observe(int type,
                                         const NotificationSource& source,
                                         const NotificationDetails& details) {
-  TabContents* source_tab = NULL;
-  content::LoadCommittedDetails* detail_info = NULL;
-
   switch (type) {
+    case content::NOTIFICATION_RENDERER_PROCESS_CLOSED: {
+      OnRendererProcessClosed(Source<RenderProcessHost>(source).ptr());
+      break;
+    }
     case content::NOTIFICATION_TAB_CONTENTS_DESTROYED: {
-      source_tab = Source<TabContents>(source).ptr();
+      TabContents* tab = Source<TabContents>(source).ptr();
+      TabContentsWrapper* wrapper =
+          TabContentsWrapper::GetCurrentWrapperForContents(tab);
+      OnTabContentsDestroyed(wrapper);
       break;
     }
     case content::NOTIFICATION_NAV_ENTRY_COMMITTED: {
       NavigationController* controller =
           Source<NavigationController>(source).ptr();
-      source_tab = controller->tab_contents();
-      detail_info = Details<content::LoadCommittedDetails>(details).ptr();
+      TabContentsWrapper* wrapper =
+          TabContentsWrapper::GetCurrentWrapperForContents(
+              controller->tab_contents());
+      content::LoadCommittedDetails* load_details =
+          Details<content::LoadCommittedDetails>(details).ptr();
+      OnNavEntryCommitted(wrapper, load_details);
       break;
     }
     default: {
@@ -144,20 +161,73 @@ void PrintPreviewTabController::Observe(int type,
       break;
     }
   }
+}
 
-  DCHECK(source_tab);
+void PrintPreviewTabController::OnRendererProcessClosed(
+    RenderProcessHost* rph) {
+  for (PrintPreviewTabMap::iterator iter = preview_tab_map_.begin();
+       iter != preview_tab_map_.end(); ++iter) {
+    if (iter->second != NULL &&
+        iter->second->render_view_host()->process() == rph) {
+      TabContentsWrapper* preview_tab = GetPrintPreviewForTab(iter->second);
+      PrintPreviewUI* print_preview_ui =
+          static_cast<PrintPreviewUI*>(preview_tab->web_ui());
+      print_preview_ui->OnInitiatorTabCrashed();
+    }
+  }
+}
 
-  TabContents* preview_tab = GetPrintPreviewForTab(source_tab);
-  bool source_tab_is_preview_tab = (source_tab == preview_tab);
+void PrintPreviewTabController::OnTabContentsDestroyed(
+    TabContentsWrapper* tab) {
+  TabContentsWrapper* preview_tab = GetPrintPreviewForTab(tab);
+  if (!preview_tab)
+    return;
 
-  if (detail_info) {
-    PageTransition::Type transition_type =
-        detail_info->entry->transition_type();
-    NavigationType::Type nav_type = detail_info->type;
+  if (tab == preview_tab) {
+    // Remove the initiator tab's observers before erasing the mapping.
+    TabContentsWrapper* initiator_tab = GetInitiatorTab(tab);
+    if (initiator_tab)
+      RemoveObservers(initiator_tab);
+
+    // Print preview tab contents are destroyed. Notify |PrintPreviewUI| to
+    // abort the initiator tab preview request.
+    if (IsPrintPreviewTab(tab) && tab->web_ui()) {
+      PrintPreviewUI* print_preview_ui =
+          static_cast<PrintPreviewUI*>(tab->web_ui());
+      print_preview_ui->OnTabDestroyed();
+    }
+
+    // Erase the map entry.
+    preview_tab_map_.erase(tab);
+  } else {
+    // Initiator tab is closed. Disable the controls in preview tab.
+    if (preview_tab->web_ui()) {
+      PrintPreviewUI* print_preview_ui =
+          static_cast<PrintPreviewUI*>(preview_tab->web_ui());
+      print_preview_ui->OnInitiatorTabClosed();
+    }
+
+    // |tab| is an initiator tab, update the map entry and remove observers.
+    preview_tab_map_[preview_tab] = NULL;
+  }
+
+  ResetPreviewTabOverrideTitle(preview_tab);
+  RemoveObservers(tab);
+}
+
+void PrintPreviewTabController::OnNavEntryCommitted(
+    TabContentsWrapper* tab, content::LoadCommittedDetails* details) {
+  TabContentsWrapper* preview_tab = GetPrintPreviewForTab(tab);
+  bool source_tab_is_preview_tab = (tab == preview_tab);
+  if (details) {
+    PageTransition::Type transition_type = details->entry->transition_type();
+    NavigationType::Type nav_type = details->type;
 
     // Don't update/erase the map entry if the page has not changed.
     if (transition_type == PageTransition::RELOAD ||
         nav_type == NavigationType::SAME_PAGE) {
+      if (source_tab_is_preview_tab)
+        SetInitiatorTabURLAndTitle(preview_tab);
       return;
     }
 
@@ -167,6 +237,7 @@ void PrintPreviewTabController::Observe(int type,
         nav_type == NavigationType::NEW_PAGE &&
         source_tab_is_preview_tab) {
       waiting_for_new_preview_page_ = false;
+      SetInitiatorTabURLAndTitle(preview_tab);
       return;
     }
 
@@ -178,73 +249,61 @@ void PrintPreviewTabController::Observe(int type,
     }
   }
 
+  RemoveObservers(tab);
+  ResetPreviewTabOverrideTitle(preview_tab);
   if (source_tab_is_preview_tab) {
     // Remove the initiator tab's observers before erasing the mapping.
-    TabContents* initiator_tab = GetInitiatorTab(source_tab);
+    TabContentsWrapper* initiator_tab = GetInitiatorTab(tab);
     if (initiator_tab)
       RemoveObservers(initiator_tab);
-
-    // |source_tab_is_preview_tab| is misleading in the case where the user
-    // chooses to re-open the initiator tab after closing it, as |source_tab|
-    // has navigated to the URL of the initiator tab at this point. Make sure to
-    // verify that |source_tab| really is a print preview tab.
-    if (IsPrintPreviewTab(source_tab) && source_tab->web_ui()) {
-      PrintPreviewUI* print_preview_ui =
-          static_cast<PrintPreviewUI*>(source_tab->web_ui());
-      print_preview_ui->OnNavigation();
-    }
-
-    // Erase the map entry.
-    preview_tab_map_.erase(source_tab);
+    preview_tab_map_.erase(tab);
   } else {
-    // Initiator tab is closed. Disable the controls in preview tab.
-    PrintPreviewUI* print_preview_ui =
-        static_cast<PrintPreviewUI*>(preview_tab->web_ui());
-    print_preview_ui->OnInitiatorTabClosed(source_tab->GetURL().spec());
-
-    // |source_tab| is an initiator tab, update the map entry.
     preview_tab_map_[preview_tab] = NULL;
+
+    // Initiator tab is closed. Disable the controls in preview tab.
+    if (preview_tab->web_ui()) {
+      PrintPreviewUI* print_preview_ui =
+          static_cast<PrintPreviewUI*>(preview_tab->web_ui());
+      print_preview_ui->OnInitiatorTabClosed();
+    }
   }
-  RemoveObservers(source_tab);
 }
 
 // static
-bool PrintPreviewTabController::IsPrintPreviewTab(TabContents* tab) {
-  const GURL& url = tab->GetURL();
+bool PrintPreviewTabController::IsPrintPreviewTab(TabContentsWrapper* tab) {
+  const GURL& url = tab->tab_contents()->GetURL();
   return (url.SchemeIs(chrome::kChromeUIScheme) &&
           url.host() == chrome::kChromeUIPrintHost);
 }
 
 void PrintPreviewTabController::EraseInitiatorTabInfo(
-    TabContents* preview_tab) {
+    TabContentsWrapper* preview_tab) {
   PrintPreviewTabMap::iterator it = preview_tab_map_.find(preview_tab);
   if (it == preview_tab_map_.end())
     return;
 
   RemoveObservers(it->second);
   preview_tab_map_[preview_tab] = NULL;
+  ResetPreviewTabOverrideTitle(preview_tab);
 }
 
-TabContents* PrintPreviewTabController::GetInitiatorTab(
-    TabContents* preview_tab) {
+TabContentsWrapper* PrintPreviewTabController::GetInitiatorTab(
+    TabContentsWrapper* preview_tab) {
   PrintPreviewTabMap::iterator it = preview_tab_map_.find(preview_tab);
   if (it != preview_tab_map_.end())
     return preview_tab_map_[preview_tab];
   return NULL;
 }
 
-TabContents* PrintPreviewTabController::CreatePrintPreviewTab(
-    TabContents* initiator_tab) {
-  // TODO: this should be converted to TabContentsWrapper.
-  TabContentsWrapper* tab =
-      TabContentsWrapper::GetCurrentWrapperForContents(initiator_tab);
-  DCHECK(tab);
+TabContentsWrapper* PrintPreviewTabController::CreatePrintPreviewTab(
+    TabContentsWrapper* initiator_tab) {
   Browser* current_browser = BrowserList::FindBrowserWithID(
-      tab->restore_tab_helper()->window_id().id());
+      initiator_tab->restore_tab_helper()->window_id().id());
   if (!current_browser) {
     if (CommandLine::ForCurrentProcess()->HasSwitch(switches::kChromeFrame)) {
-      current_browser = Browser::CreateForType(Browser::TYPE_POPUP,
-                                               initiator_tab->profile());
+      Profile* profile = Profile::FromBrowserContext(
+          initiator_tab->tab_contents()->browser_context());
+      current_browser = Browser::CreateForType(Browser::TYPE_POPUP, profile);
       if (!current_browser) {
         NOTREACHED() << "Failed to create popup browser window";
         return NULL;
@@ -262,34 +321,68 @@ TabContents* PrintPreviewTabController::CreatePrintPreviewTab(
   if (CommandLine::ForCurrentProcess()->HasSwitch(switches::kChromeFrame))
     params.disposition = NEW_POPUP;
   params.tabstrip_index = current_browser->tabstrip_model()->
-      GetWrapperIndex(initiator_tab) + 1;
+      GetIndexOfTabContents(initiator_tab) + 1;
   browser::Navigate(&params);
   TabContentsWrapper* preview_tab = params.target_contents;
   EnableInternalPDFPluginForTab(preview_tab);
   static_cast<RenderViewHostDelegate*>(preview_tab->tab_contents())->Activate();
 
   // Add an entry to the map.
-  preview_tab_map_[preview_tab->tab_contents()] = initiator_tab;
+  preview_tab_map_[preview_tab] = initiator_tab;
   waiting_for_new_preview_page_ = true;
 
   AddObservers(initiator_tab);
-  AddObservers(preview_tab->tab_contents());
+  AddObservers(preview_tab);
 
-  return preview_tab->tab_contents();
+  return preview_tab;
 }
 
-void PrintPreviewTabController::AddObservers(TabContents* tab) {
+void PrintPreviewTabController::SetInitiatorTabURLAndTitle(
+    TabContentsWrapper* preview_tab) {
+  TabContentsWrapper* initiator_tab = GetInitiatorTab(preview_tab);
+  if (initiator_tab && preview_tab->web_ui()) {
+    PrintPreviewUI* print_preview_ui =
+        static_cast<PrintPreviewUI*>(preview_tab->web_ui());
+    print_preview_ui->SetInitiatorTabURLAndTitle(
+        initiator_tab->tab_contents()->GetURL().spec(),
+        initiator_tab->print_view_manager()->RenderSourceName());
+  }
+}
+
+void PrintPreviewTabController::AddObservers(TabContentsWrapper* tab) {
+  TabContents* contents = tab->tab_contents();
   registrar_.Add(this, content::NOTIFICATION_TAB_CONTENTS_DESTROYED,
-                 Source<TabContents>(tab));
+                 Source<TabContents>(contents));
   registrar_.Add(this, content::NOTIFICATION_NAV_ENTRY_COMMITTED,
-                 Source<NavigationController>(&tab->controller()));
+                 Source<NavigationController>(&contents->controller()));
+
+  // Multiple sites may share the same RenderProcessHost, so check if this
+  // notification has already been added.
+  RenderProcessHost* rph = tab->render_view_host()->process();
+  if (!registrar_.IsRegistered(this,
+                               content::NOTIFICATION_RENDERER_PROCESS_CLOSED,
+                               Source<RenderProcessHost>(rph))) {
+    registrar_.Add(this, content::NOTIFICATION_RENDERER_PROCESS_CLOSED,
+                   Source<RenderProcessHost>(rph));
+  }
 }
 
-void PrintPreviewTabController::RemoveObservers(TabContents* tab) {
+void PrintPreviewTabController::RemoveObservers(TabContentsWrapper* tab) {
+  TabContents* contents = tab->tab_contents();
   registrar_.Remove(this, content::NOTIFICATION_TAB_CONTENTS_DESTROYED,
-                    Source<TabContents>(tab));
+                    Source<TabContents>(contents));
   registrar_.Remove(this, content::NOTIFICATION_NAV_ENTRY_COMMITTED,
-                    Source<NavigationController>(&tab->controller()));
+                    Source<NavigationController>(&contents->controller()));
+
+  // Multiple sites may share the same RenderProcessHost, so check if this
+  // notification has already been added.
+  RenderProcessHost* rph = tab->render_view_host()->process();
+  if (registrar_.IsRegistered(this,
+                              content::NOTIFICATION_RENDERER_PROCESS_CLOSED,
+                              Source<RenderProcessHost>(rph))) {
+    registrar_.Remove(this, content::NOTIFICATION_RENDERER_PROCESS_CLOSED,
+                      Source<RenderProcessHost>(rph));
+  }
 }
 
 }  // namespace printing
