@@ -9,6 +9,8 @@
 #include <iterator>
 
 #include "chrome/browser/background/background_contents_service_factory.h"
+#include "chrome/browser/plugin_prefs.h"
+#include "chrome/browser/printing/cloud_print/cloud_print_proxy_service_factory.h"
 #include "chrome/browser/profiles/profile_keyed_service.h"
 #include "chrome/browser/profiles/profile_keyed_service_factory.h"
 #include "chrome/browser/search_engines/template_url_service_factory.h"
@@ -34,6 +36,8 @@ bool g_initialized = false;
 void AssertFactoriesBuilt() {
   if (!g_initialized) {
     BackgroundContentsServiceFactory::GetInstance();
+    CloudPrintProxyServiceFactory::GetInstance();
+    PluginPrefs::Initialize();
     SessionServiceFactory::GetInstance();
     TabRestoreServiceFactory::GetInstance();
     TemplateURLServiceFactory::GetInstance();
