@@ -8,6 +8,7 @@
 
 #include <set>
 #include "base/basictypes.h"
+#include "base/callback.h"
 #include "base/file_path.h"
 #include "base/memory/ref_counted.h"
 #include "base/memory/scoped_ptr.h"
@@ -38,12 +39,17 @@ class CookieStore;
 class DnsCertProvenanceChecker;
 class HttpTransactionFactory;
 class NetLog;
+class OriginBoundCertService;
 class ProxyConfigService;
 class ProxyService;
 class SSLConfigService;
 class TransportSecurityState;
 }  // namespace net
 
+namespace policy {
+class URLBlacklistManager;
+}  // namespace policy
+
 namespace prerender {
 class PrerenderManager;
 };  // namespace prerender
@@ -58,19 +64,13 @@ class DatabaseTracker;
 
 // Conceptually speaking, the ProfileIOData represents data that lives on the IO
 // thread that is owned by a Profile, such as, but not limited to, network
-// objects like CookieMonster, HttpTransactionFactory, etc. The Profile
-// implementation will maintain a reference to the ProfileIOData. The
-// ProfileIOData will originally own a reference to the ChromeURLRequestContexts
-// that reference its members. When an accessor for a ChromeURLRequestContext is
-// invoked, then ProfileIOData will release its reference to the
-// ChromeURLRequestContext and the ChromeURLRequestContext will acquire a
-// reference to the ProfileIOData, so they exchange ownership. This is done
-// because it's possible for a context's accessor never to be invoked, so this
-// ownership reversal prevents shutdown leaks. ProfileIOData will lazily
-// initialize its members on the first invocation of a ChromeURLRequestContext
-// accessor.
-class ProfileIOData : public base::RefCountedThreadSafe<ProfileIOData> {
+// objects like CookieMonster, HttpTransactionFactory, etc.  Profile owns
+// ProfileIOData, but will make sure to delete it on the IO thread (except
+// possibly in unit tests where there is no IO thread).
+class ProfileIOData {
  public:
+  virtual ~ProfileIOData();
+
   // Returns true if |scheme| is handled in Chrome, or by default handlers in
   // net::URLRequest.
   static bool IsHandledProtocol(const std::string& scheme);
@@ -105,7 +105,7 @@ class ProfileIOData : public base::RefCountedThreadSafe<ProfileIOData> {
   }
 
   ChromeURLRequestContext* extensions_request_context() const {
-    return weak_extensions_request_context_.get();
+    return extensions_request_context_.get();
   }
 
   BooleanPrefMember* safe_browsing_enabled() const {
@@ -113,35 +113,15 @@ class ProfileIOData : public base::RefCountedThreadSafe<ProfileIOData> {
   }
 
  protected:
-  friend class base::RefCountedThreadSafe<ProfileIOData>;
-
-  class RequestContext : public ChromeURLRequestContext {
+  class AppRequestContext : public ChromeURLRequestContext {
    public:
-    RequestContext();
-    virtual ~RequestContext();
-
-    // Setter is used to transfer ownership of the ProfileIOData to the context.
-    void set_profile_io_data(ProfileIOData* profile_io_data) {
-      profile_io_data_ = profile_io_data;
-    }
-
-   protected:
-    ProfileIOData* profile_io_data() { return profile_io_data_; }
-
-   private:
-    scoped_refptr<ProfileIOData> profile_io_data_;
-  };
-
-  class AppRequestContext : public RequestContext {
-   public:
-    explicit AppRequestContext(const std::string& app_id);
+    AppRequestContext();
     virtual ~AppRequestContext();
 
     void SetCookieStore(net::CookieStore* cookie_store);
     void SetHttpTransactionFactory(net::HttpTransactionFactory* http_factory);
 
    private:
-    const std::string app_id_;
     scoped_refptr<net::CookieStore> cookie_store_;
     scoped_ptr<net::HttpTransactionFactory> http_factory_;
   };
@@ -170,22 +150,22 @@ class ProfileIOData : public base::RefCountedThreadSafe<ProfileIOData> {
     scoped_refptr<quota::QuotaManager> quota_manager;
     scoped_refptr<ExtensionInfoMap> extension_info_map;
     DesktopNotificationService* notification_service;
-    base::WeakPtr<prerender::PrerenderManager> prerender_manager;
+    base::Callback<prerender::PrerenderManager*(void)> prerender_manager_getter;
     scoped_refptr<ProtocolHandlerRegistry> protocol_handler_registry;
     // We need to initialize the ProxyConfigService from the UI thread
     // because on linux it relies on initializing things through gconf,
     // and needs to be on the main thread.
     scoped_ptr<net::ProxyConfigService> proxy_config_service;
     // The profile this struct was populated from. It's passed as a void* to
-    // ensure it's not accidently used on the IO thread. Before using it  on the
+    // ensure it's not accidently used on the IO thread. Before using it on the
     // UI thread, call ProfileManager::IsValidProfile to ensure it's alive.
     void* profile;
+
   };
 
   explicit ProfileIOData(bool is_incognito);
-  virtual ~ProfileIOData();
 
-  void InitializeProfileParams(Profile* profile);
+  void InitializeOnUIThread(Profile* profile);
   void ApplyProfileParamsToContext(ChromeURLRequestContext* context) const;
 
   // Lazy initializes the ProfileIOData object the first time a request context
@@ -205,6 +185,13 @@ class ProfileIOData : public base::RefCountedThreadSafe<ProfileIOData> {
     return chrome_url_data_manager_backend_.get();
   }
 
+  // An OriginBoundCertService object is created by a derived class of
+  // ProfileIOData, and the derived class calls this method to set the
+  // origin_bound_cert_service_ member and transfers ownership to the base
+  // class.
+  void set_origin_bound_cert_service(
+      net::OriginBoundCertService* origin_bound_cert_service) const;
+
   net::NetworkDelegate* network_delegate() const {
     return network_delegate_.get();
   }
@@ -237,7 +224,7 @@ class ProfileIOData : public base::RefCountedThreadSafe<ProfileIOData> {
     const ProfileIOData* const io_data_;
   };
 
-  typedef base::hash_map<std::string, ChromeURLRequestContext*>
+  typedef base::hash_map<std::string, scoped_refptr<ChromeURLRequestContext> >
       AppRequestContextMap;
 
   // --------------------------------------------
@@ -250,7 +237,7 @@ class ProfileIOData : public base::RefCountedThreadSafe<ProfileIOData> {
 
   // Does an on-demand initialization of a RequestContext for the given
   // isolated app.
-  virtual scoped_refptr<RequestContext> InitializeAppRequestContext(
+  virtual scoped_refptr<ChromeURLRequestContext> InitializeAppRequestContext(
       scoped_refptr<ChromeURLRequestContext> main_context,
       const std::string& app_id) const = 0;
 
@@ -258,7 +245,7 @@ class ProfileIOData : public base::RefCountedThreadSafe<ProfileIOData> {
   // context from ProfileIOData to the URLRequestContextGetter.
   virtual scoped_refptr<ChromeURLRequestContext>
       AcquireMediaRequestContext() const = 0;
-  virtual scoped_refptr<RequestContext>
+  virtual scoped_refptr<ChromeURLRequestContext>
       AcquireIsolatedAppRequestContext(
           scoped_refptr<ChromeURLRequestContext> main_context,
           const std::string& app_id) const = 0;
@@ -275,9 +262,13 @@ class ProfileIOData : public base::RefCountedThreadSafe<ProfileIOData> {
   mutable BooleanPrefMember clear_local_state_on_exit_;
   mutable BooleanPrefMember safe_browsing_enabled_;
 
+  // Pointed to by NetworkDelegate.
+  mutable scoped_ptr<policy::URLBlacklistManager> url_blacklist_manager_;
+
   // Pointed to by URLRequestContext.
   mutable scoped_ptr<ChromeURLDataManagerBackend>
       chrome_url_data_manager_backend_;
+  mutable scoped_ptr<net::OriginBoundCertService> origin_bound_cert_service_;
   mutable scoped_ptr<net::NetworkDelegate> network_delegate_;
   mutable scoped_ptr<net::DnsCertProvenanceChecker> dns_cert_checker_;
   mutable scoped_ptr<net::ProxyService> proxy_service_;
@@ -295,23 +286,18 @@ class ProfileIOData : public base::RefCountedThreadSafe<ProfileIOData> {
   mutable scoped_refptr<ExtensionInfoMap> extension_info_map_;
   mutable scoped_refptr<HostContentSettingsMap> host_content_settings_map_;
   mutable DesktopNotificationService* notification_service_;
-  mutable base::WeakPtr<prerender::PrerenderManager> prerender_manager_;
+  mutable base::Callback<prerender::PrerenderManager*(void)>
+      prerender_manager_getter_;
 
   mutable ResourceContext resource_context_;
 
   // These are only valid in between LazyInitialize() and their accessor being
   // called.
-  mutable scoped_refptr<RequestContext> main_request_context_;
-  mutable scoped_refptr<RequestContext> extensions_request_context_;
+  mutable scoped_refptr<ChromeURLRequestContext> main_request_context_;
+  mutable scoped_refptr<ChromeURLRequestContext> extensions_request_context_;
   // One AppRequestContext per isolated app.
   mutable AppRequestContextMap app_request_context_map_;
 
-  // Weak pointers to the request contexts. Only valid after LazyInitialize.
-  // These are weak so that they don't hold a reference to the RequestContext,
-  // because that holds a reference back to ProfileIOData.
-  mutable base::WeakPtr<ChromeURLRequestContext>
-      weak_extensions_request_context_;
-
   DISALLOW_COPY_AND_ASSIGN(ProfileIOData);
 };
 
