@@ -18,6 +18,7 @@
 #include "views/controls/native/native_view_host.h"
 #include "views/events/event.h"
 #include "views/ime/text_input_client.h"
+#include "views/touchui/touch_selection_controller.h"
 #include "views/view.h"
 #include "webkit/glue/webcursor.h"
 
@@ -35,7 +36,7 @@ struct NativeWebKeyboardEvent;
 // See comments in render_widget_host_view.h about this class and its members.
 // -----------------------------------------------------------------------------
 class RenderWidgetHostViewViews : public RenderWidgetHostView,
-                                  public views::View,
+                                  public views::TouchSelectionClientView,
                                   public views::TextInputClient {
  public:
   // Internal class name.
@@ -57,7 +58,8 @@ class RenderWidgetHostViewViews : public RenderWidgetHostView,
   virtual void WasHidden() OVERRIDE;
   virtual void SetSize(const gfx::Size& size) OVERRIDE;
   virtual void SetBounds(const gfx::Rect& rect) OVERRIDE;
-  virtual gfx::NativeView GetNativeView() OVERRIDE;
+  virtual gfx::NativeView GetNativeView() const OVERRIDE;
+  virtual gfx::NativeViewId GetNativeViewId() const OVERRIDE;
   virtual void MovePluginWindows(
       const std::vector<webkit::npapi::WebPluginGeometry>& moves) OVERRIDE;
   virtual bool HasFocus() OVERRIDE;
@@ -79,16 +81,28 @@ class RenderWidgetHostViewViews : public RenderWidgetHostView,
   virtual void Destroy() OVERRIDE;
   virtual void SetTooltipText(const std::wstring& tooltip_text) OVERRIDE;
   virtual void SelectionChanged(const std::string& text,
-                                const ui::Range& range) OVERRIDE;
+                                const ui::Range& range,
+                                const gfx::Point& start,
+                                const gfx::Point& end) OVERRIDE;
   virtual void ShowingContextMenu(bool showing) OVERRIDE;
   virtual BackingStore* AllocBackingStore(const gfx::Size& size) OVERRIDE;
   virtual void SetBackground(const SkBitmap& background) OVERRIDE;
+#if defined(OS_POSIX)
+  virtual void GetScreenInfo(WebKit::WebScreenInfo* results) OVERRIDE;
+  virtual gfx::Rect GetRootWindowBounds() OVERRIDE;
+#endif
 #if defined(TOOLKIT_USES_GTK)
   virtual void CreatePluginContainer(gfx::PluginWindowHandle id) OVERRIDE;
   virtual void DestroyPluginContainer(gfx::PluginWindowHandle id) OVERRIDE;
 #endif
   virtual void SetVisuallyDeemphasized(const SkColor* color,
                                        bool animate) OVERRIDE;
+  virtual void UnhandledWheelEvent(
+      const WebKit::WebMouseWheelEvent& event) OVERRIDE;
+  virtual void SetHasHorizontalScrollbar(
+      bool has_horizontal_scrollbar) OVERRIDE;
+  virtual void SetScrollOffsetPinning(
+      bool is_pinned_to_left, bool is_pinned_to_right) OVERRIDE;
 #if defined(TOOLKIT_USES_GTK)
   virtual void AcceleratedCompositingActivated(bool activated) OVERRIDE;
 #endif
@@ -98,6 +112,18 @@ class RenderWidgetHostViewViews : public RenderWidgetHostView,
 #endif
   virtual gfx::PluginWindowHandle GetCompositingSurface() OVERRIDE;
 
+  // Overridden from views::TouchSelectionClientView.
+  virtual void SelectRect(const gfx::Point& start,
+                          const gfx::Point& end) OVERRIDE;
+
+  // Overridden from ui::SimpleMenuModel::Delegate.
+  virtual bool IsCommandIdChecked(int command_id) const OVERRIDE;
+  virtual bool IsCommandIdEnabled(int command_id) const OVERRIDE;
+  virtual bool GetAcceleratorForCommandId(
+      int command_id,
+      ui::Accelerator* accelerator) OVERRIDE;
+  virtual void ExecuteCommand(int command_id) OVERRIDE;
+
   // Overridden from views::View.
   virtual std::string GetClassName() const OVERRIDE;
   virtual gfx::NativeCursor GetCursor(const views::MouseEvent& event) OVERRIDE;
@@ -148,13 +174,15 @@ class RenderWidgetHostViewViews : public RenderWidgetHostView,
 #endif
 
  protected:
-  // Overridden from RenderWidgetHostView / views::View.
+  // Overridden views::View.
   virtual void OnPaint(gfx::Canvas* canvas) OVERRIDE;
-  virtual void Focus() OVERRIDE;
-  virtual void Blur() OVERRIDE;
   virtual void OnFocus() OVERRIDE;
   virtual void OnBlur() OVERRIDE;
 
+  // Overridden from RenderWidgetHostView
+  virtual void Focus() OVERRIDE;
+  virtual void Blur() OVERRIDE;
+
  private:
   friend class RenderWidgetHostViewViewsWidget;
 
@@ -240,6 +268,10 @@ class RenderWidgetHostViewViews : public RenderWidgetHostView,
 
   string16 tooltip_text_;
 
+  scoped_ptr<views::TouchSelectionController> touch_selection_controller_;
+  gfx::Point selection_start_;
+  gfx::Point selection_end_;
+
 #if defined(TOUCH_UI)
   std::map<uint64, scoped_refptr<AcceleratedSurfaceContainerTouch> >
       accelerated_surface_containers_;
