@@ -9,7 +9,7 @@
 #include "base/threading/thread.h"
 #include "chrome/browser/renderer_host/text_input_client_message_filter.h"
 #include "chrome/common/text_input_client_messages.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/testing_profile.h"
 #include "content/browser/renderer_host/mock_render_process_host.h"
 #include "content/browser/renderer_host/render_process_host.h"
 #include "content/browser/renderer_host/render_widget_host.h"
