@@ -137,6 +137,7 @@ cr.define('options.browser_options', function() {
       this.addEventListener('dragover', this.handleDragOver_.bind(this));
       this.addEventListener('drop', this.handleDrop_.bind(this));
       this.addEventListener('dragleave', this.handleDragLeave_.bind(this));
+      this.addEventListener('dragend', this.handleDragEnd_.bind(this));
     },
 
     /** @inheritDoc */
@@ -253,10 +254,20 @@ cr.define('options.browser_options', function() {
      * @param {Event} e The dragleave event
      * @private
      */
-    handleDragLeave_ : function(e) {
+    handleDragLeave_: function(e) {
       this.hideDropMarker_();
     },
 
+    /**
+     * Handles the dragend event.
+     * @param {Event} e The dragend event
+     * @private
+     */
+    handleDragEnd_: function(e) {
+      this.draggedItem.editable = true;
+      this.draggedItem.updateEditState();
+    },
+
     /*
      * Shows and positions the marker to indicate the drop target.
      * @param {HTMLElement} target The current target list item of drop
