@@ -45,14 +45,20 @@ cr.define('options', function() {
         };
       }
 
-      var manageHandlersButton =
-          this.pageDiv.querySelector('#manage-handlers-button');
+      var manageHandlersButton = $('manage-handlers-button');
       if (manageHandlersButton) {
         manageHandlersButton.onclick = function(event) {
           OptionsPage.navigateToPage('handlers');
         };
       }
 
+      var manageIntentsButton = $('manage-intents-button');
+      if (manageIntentsButton) {
+        manageIntentsButton.onclick = function(event) {
+          OptionsPage.navigateToPage('intents');
+        };
+      }
+
       // Cookies filter page ---------------------------------------------------
       $('show-cookies-button').onclick = function(event) {
         chrome.send('coreOptionsUserMetricsAction', ['Options_ShowCookies']);
@@ -60,13 +66,16 @@ cr.define('options', function() {
       };
 
       if (!templateData.enable_click_to_play)
-        $('click_to_play').style.display = 'none';
+        $('click_to_play').hidden = true;
+
+      if (!templateData.enable_web_intents && $('intent-section'))
+        $('intent-section').hidden = true;
     },
   };
 
   ContentSettings.updateHandlersEnabledRadios = function(enabled) {
-    var selector = '#handlers-section input[type=radio][value=' +
-        (enabled ? 'allow' : 'block') + ']';
+    var selector = '#content-settings-page input[type=radio][value=' +
+        (enabled ? 'allow' : 'block') + '].handler-radio';
     document.querySelector(selector).checked = true;
   };
 
