@@ -19,6 +19,10 @@ cr.define('options', function() {
     OptionsPage.call(this, 'personal',
                      templateData.personalPageTabTitle,
                      'personal-page');
+    if (cr.isChromeOS) {
+      // Email of the currently logged in user (or |kGuestUser|).
+      this.userEmail_ = localStrings.getString('userEmail');
+    }
   }
 
   cr.addSingletonGetter(PersonalOptions);
@@ -112,7 +116,7 @@ cr.define('options', function() {
         $('change-picture-button').onclick = function(event) {
           OptionsPage.navigateToPage('changePicture');
         };
-        chrome.send('loadAccountPicture');
+        this.updateAccountPicture_();
 
         if (cr.commandLine.options['--bwsi']) {
           // Disable the screen lock checkbox and change-picture-button in
@@ -155,10 +159,6 @@ cr.define('options', function() {
       $('customize-sync').hidden = !completed;
     },
 
-    setAccountPicture_: function(image) {
-      $('account-picture').src = image;
-    },
-
     setSyncStatus_: function(status) {
       var statusSet = status != '';
       $('sync-overview').hidden = statusSet;
@@ -264,6 +264,15 @@ cr.define('options', function() {
     hideSyncSection_: function() {
       $('sync-section').hidden = true;
     },
+
+    /**
+     * (Re)loads IMG element with current user account picture.
+     */
+    updateAccountPicture_: function() {
+      $('account-picture').src =
+          'chrome://userimage/' + this.userEmail_ +
+          '?id=' + (new Date()).getTime();
+    },
   };
 
   /**
@@ -283,24 +292,34 @@ cr.define('options', function() {
     return cr.commandLine.options['--bwsi'];
   };
 
+  if (cr.isChromeOS) {
+    /**
+     * Returns email of the user logged in (ChromeOS only).
+     * @return {string} user email.
+     */
+    PersonalOptions.getLoggedInUserEmail = function() {
+      return PersonalOptions.getInstance().userEmail_;
+    };
+  }
+
   // Forward public APIs to private implementations.
   [
-    'setSyncEnabled',
+    'hideSyncSection',
     'setAutoLoginVisible',
-    'setSyncSetupCompleted',
-    'setAccountPicture',
-    'setSyncStatus',
-    'setSyncStatusErrorVisible',
-    'setSyncActionLinkEnabled',
-    'setSyncActionLinkLabel',
+    'setGtkThemeButtonEnabled',
     'setProfilesInfo',
     'setProfilesSectionVisible',
-    'setStartStopButtonVisible',
     'setStartStopButtonEnabled',
     'setStartStopButtonLabel',
-    'setGtkThemeButtonEnabled',
+    'setStartStopButtonVisible',
+    'setSyncActionLinkEnabled',
+    'setSyncActionLinkLabel',
+    'setSyncEnabled',
+    'setSyncSetupCompleted',
+    'setSyncStatus',
+    'setSyncStatusErrorVisible',
     'setThemesResetButtonEnabled',
-    'hideSyncSection',
+    'updateAccountPicture',
   ].forEach(function(name) {
     PersonalOptions[name] = function(value) {
       PersonalOptions.getInstance()[name + '_'](value);
