@@ -22,6 +22,12 @@ const cr = (function() {
    */
   const isChromeOS = /CrOS/.test(navigator.userAgent);
 
+  /**
+   * Whether this is on touchui build or not.
+   * @type {boolean}
+   */
+  const isTouch = /Touch/.test(navigator.userAgent);
+
   /**
    * Whether this is on vanilla Linux (not chromeOS).
    * @type {boolean}
@@ -346,6 +352,7 @@ const cr = (function() {
     isWindows: isWindows,
     isLinux: isLinux,
     isViews: isViews,
+    isTouch: isTouch,
     enablePlatformSpecificCSSRules: enablePlatformSpecificCSSRules,
     define: define,
     defineProperty: defineProperty,
