@@ -31,6 +31,7 @@ struct SafeBrowsingHostMsg_MalwareDOMDetails_Node;
 class MalwareDetailsCacheCollector;
 class MalwareDetailsRedirectsCollector;
 class MalwareDetailsFactory;
+class Profile;
 
 namespace safe_browsing {
 // Maps a URL to its Resource.
@@ -81,6 +82,8 @@ class MalwareDetails : public base::RefCountedThreadSafe<MalwareDetails>,
   virtual void AddDOMDetails(
       const std::vector<SafeBrowsingHostMsg_MalwareDOMDetails_Node>& params);
 
+  Profile* profile_;
+
   // The report protocol buffer.
   scoped_ptr<safe_browsing::ClientMalwareReportRequest> report_;
 
