@@ -8,12 +8,14 @@
 #include "base/i18n/icu_string_conversions.h"
 #include "base/i18n/rtl.h"
 #include "base/logging.h"
+#include "base/metrics/field_trial.h"
 #include "base/stringprintf.h"
 #include "base/string_number_conversions.h"
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/search_engines/search_engine_type.h"
 #include "chrome/browser/search_engines/search_terms_data.h"
-#include "chrome/browser/search_engines/template_url_model.h"
+#include "chrome/browser/search_engines/template_url_service.h"
+#include "chrome/common/guid.h"
 #include "chrome/common/url_constants.h"
 #include "chrome/installer/util/google_update_settings.h"
 #include "content/browser/user_metrics.h"
@@ -51,10 +53,14 @@ static const char kGoogleBaseSuggestURLParameter[] =
     "google:baseSuggestURL";
 static const char kGoogleBaseSuggestURLParameterFull[] =
     "{google:baseSuggestURL}";
+static const char kGoogleInstantFieldTrialGroupParameter[] =
+    "google:instantFieldTrialGroupParameter";
 static const char kGoogleOriginalQueryForSuggestionParameter[] =
     "google:originalQueryForSuggestion";
 static const char kGoogleRLZParameter[] = "google:RLZ";
 // Same as kSearchTermsParameter, with no escaping.
+static const char kGoogleSearchFieldtrialParameter[] =
+    "google:searchFieldtrialParameter";
 static const char kGoogleUnescapedSearchTermsParameter[] =
     "google:unescapedSearchTerms";
 static const char kGoogleUnescapedSearchTermsParameterFull[] =
@@ -85,7 +91,8 @@ TemplateURLRef::TemplateURLRef(const std::string& url,
       page_offset_(page_offset),
       parsed_(false),
       valid_(false),
-      supports_replacements_(false) {
+      supports_replacements_(false),
+      prepopulated_(false) {
 }
 
 void TemplateURLRef::Set(const std::string& url,
@@ -142,16 +149,24 @@ bool TemplateURLRef::ParseParameter(size_t start,
     replacements->push_back(Replacement(GOOGLE_BASE_URL, start));
   } else if (parameter == kGoogleBaseSuggestURLParameter) {
     replacements->push_back(Replacement(GOOGLE_BASE_SUGGEST_URL, start));
+  } else if (parameter == kGoogleInstantFieldTrialGroupParameter) {
+    replacements->push_back(Replacement(GOOGLE_INSTANT_FIELD_TRIAL_GROUP,
+                                        start));
   } else if (parameter == kGoogleOriginalQueryForSuggestionParameter) {
     replacements->push_back(Replacement(GOOGLE_ORIGINAL_QUERY_FOR_SUGGESTION,
                                         start));
   } else if (parameter == kGoogleRLZParameter) {
     replacements->push_back(Replacement(GOOGLE_RLZ, start));
+  } else if (parameter == kGoogleSearchFieldtrialParameter) {
+    replacements->push_back(Replacement(GOOGLE_SEARCH_FIELDTRIAL_GROUP, start));
   } else if (parameter == kGoogleUnescapedSearchTermsParameter) {
     replacements->push_back(Replacement(GOOGLE_UNESCAPED_SEARCH_TERMS, start));
   } else {
-    // It can be some garbage but can also be a javascript block. Put it back.
-    url->insert(start, full_parameter);
+    // If it's a prepopulated URL, we know that it's safe to remove unknown
+    // parameters. Otherwise it could be some garbage but can also be a
+    // javascript block. Put it back.
+    if (!prepopulated_)
+      url->insert(start, full_parameter);
     return false;
   }
   return true;
@@ -271,7 +286,21 @@ std::string TemplateURLRef::ReplaceSearchTerms(
     const string16& terms,
     int accepted_suggestion,
     const string16& original_query_for_suggestion) const {
+  return ReplaceSearchTermsUsingProfile(NULL,
+                                        host,
+                                        terms,
+                                        accepted_suggestion,
+                                        original_query_for_suggestion);
+}
+
+std::string TemplateURLRef::ReplaceSearchTermsUsingProfile(
+    Profile* profile,
+    const TemplateURL& host,
+    const string16& terms,
+    int accepted_suggestion,
+    const string16& original_query_for_suggestion) const {
   UIThreadSearchTermsData search_terms_data;
+  search_terms_data.set_profile(profile);
   return ReplaceSearchTermsUsingTermsData(host,
                                           terms,
                                           accepted_suggestion,
@@ -366,6 +395,10 @@ std::string TemplateURLRef::ReplaceSearchTermsUsingTermsData(
         url.insert(i->index, search_terms_data.GoogleBaseSuggestURLValue());
         break;
 
+      case GOOGLE_INSTANT_FIELD_TRIAL_GROUP:
+        url.insert(i->index, search_terms_data.InstantFieldTrialUrlParam());
+        break;
+
       case GOOGLE_ORIGINAL_QUERY_FOR_SUGGESTION:
         if (accepted_suggestion >= 0)
           url.insert(i->index, "oq=" + UTF16ToUTF8(encoded_original_query) +
@@ -387,6 +420,17 @@ std::string TemplateURLRef::ReplaceSearchTermsUsingTermsData(
         break;
       }
 
+      case GOOGLE_SEARCH_FIELDTRIAL_GROUP: {
+        static bool use_suggest_prefix =
+            base::FieldTrialList::TrialExists("SuggestHostPrefix");
+        if (use_suggest_prefix) {
+          static bool used_www = (base::FieldTrialList::FindFullName(
+              "SuggestHostPrefix") == "Www_Prefix");
+          url.insert(i->index, used_www ? "gcx=w&" : "gcx=c&");
+        }
+        break;
+      }
+
       case GOOGLE_UNESCAPED_SEARCH_TERMS: {
         std::string unescaped_terms;
         base::UTF16ToCodepage(terms, input_encoding.c_str(),
@@ -521,7 +565,7 @@ bool TemplateURLRef::HasGoogleBaseURLs() const {
 
 // static
 bool TemplateURLRef::SameUrlRefs(const TemplateURLRef* ref1,
- const TemplateURLRef* ref2) {
+                                 const TemplateURLRef* ref2) {
   return ref1 == ref2 || (ref1 && ref2 && ref1->url() == ref2->url());
 }
 
@@ -594,11 +638,13 @@ TemplateURL::TemplateURL()
       safe_for_autoreplace_(false),
       id_(0),
       date_created_(base::Time::Now()),
+      last_modified_(base::Time::Now()),
       created_by_policy_(false),
       usage_count_(0),
       search_engine_type_(SEARCH_ENGINE_OTHER),
       logo_id_(kNoSearchEngineLogo),
-      prepopulate_id_(0) {
+      prepopulate_id_(0),
+      sync_guid_(guid::GenerateGUID()) {
 }
 
 TemplateURL::~TemplateURL() {
@@ -643,8 +689,8 @@ string16 TemplateURL::keyword() const {
 void TemplateURL::EnsureKeyword() const {
   if (autogenerate_keyword_ && !keyword_generated_) {
     // Generate a keyword and cache it.
-    keyword_ = TemplateURLModel::GenerateKeyword(
-        TemplateURLModel::GenerateSearchURL(this).GetWithEmptyPath(), true);
+    keyword_ = TemplateURLService::GenerateKeyword(
+        TemplateURLService::GenerateSearchURL(this).GetWithEmptyPath(), true);
     keyword_generated_ = true;
   }
 }
@@ -684,6 +730,11 @@ GURL TemplateURL::GetFaviconURL() const {
   return GURL();
 }
 
+void TemplateURL::SetPrepopulateId(int id) {
+  prepopulate_id_ = id;
+  SetTemplateURLRefsPrepopulated(id > 0);
+}
+
 void TemplateURL::InvalidateCachedValues() const {
   url_.InvalidateCachedValues();
   suggestions_url_.InvalidateCachedValues();
@@ -693,6 +744,12 @@ void TemplateURL::InvalidateCachedValues() const {
   }
 }
 
+void TemplateURL::SetTemplateURLRefsPrepopulated(bool prepopulated) {
+  suggestions_url_.set_prepopulated(prepopulated);
+  url_.set_prepopulated(prepopulated);
+  instant_url_.set_prepopulated(prepopulated);
+}
+
 std::string TemplateURL::GetExtensionId() const {
   DCHECK(IsExtensionKeyword());
   return GURL(url_.url()).host();
