@@ -19,13 +19,14 @@
 #include "chrome/browser/ui/browser_list.h"
 #include "content/common/notification_observer.h"
 #include "content/common/notification_registrar.h"
+#include "ui/base/ui_base_types.h"
 
-class NavigationController;
 class NavigationEntry;
 class Profile;
 class SessionCommand;
 struct SessionTab;
 struct SessionWindow;
+class TabContentsWrapper;
 
 // SessionService ------------------------------------------------------------
 
@@ -58,6 +59,12 @@ class SessionService : public BaseSessionService,
 
   virtual ~SessionService();
 
+  // Returns true if a new window opening should really be treated like the
+  // start of a session (with potential session restore, startup URLs, etc.).
+  // In particular, this is true if there are no tabbed browsers running
+  // currently (eg. because only background or other app pages are running).
+  bool ShouldNewWindowStartSession();
+
   // Invoke at a point when you think session restore might occur. For example,
   // during startup and window creation this is invoked to see if a session
   // needs to be restored. If a session needs to be restored it is done so
@@ -81,7 +88,7 @@ class SessionService : public BaseSessionService,
   // Sets the bounds of a window.
   void SetWindowBounds(const SessionID& window_id,
                        const gfx::Rect& bounds,
-                       bool is_maximized);
+                       ui::WindowShowState show_state);
 
   // Sets the visual index of the tab in its parent window.
   void SetTabIndexInWindow(const SessionID& window_id,
@@ -133,8 +140,7 @@ class SessionService : public BaseSessionService,
 
   // Notification that a tab has restored its entries or a closed tab is being
   // reused.
-  void TabRestored(NavigationController* controller,
-                   bool pinned);
+  void TabRestored(TabContentsWrapper* tab, bool pinned);
 
   // Sets the index of the selected entry in the navigation controller for the
   // specified tab.
@@ -200,7 +206,7 @@ class SessionService : public BaseSessionService,
   bool RestoreIfNecessary(const std::vector<GURL>& urls_to_open,
                           Browser* browser);
 
-  virtual void Observe(NotificationType type,
+  virtual void Observe(int type,
                        const NotificationSource& source,
                        const NotificationDetails& details);
 
@@ -219,7 +225,7 @@ class SessionService : public BaseSessionService,
 
   SessionCommand* CreateSetWindowBoundsCommand(const SessionID& window_id,
                                                const gfx::Rect& bounds,
-                                               bool is_maximized);
+                                               ui::WindowShowState show_state);
 
   SessionCommand* CreateSetTabIndexInWindowCommand(const SessionID& tab_id,
                                                    int new_index);
@@ -303,13 +309,13 @@ class SessionService : public BaseSessionService,
                             std::map<int, SessionWindow*>* windows);
 
   // Adds commands to commands that will recreate the state of the specified
-  // NavigationController. This adds at most kMaxNavigationCountToPersist
-  // navigations (in each direction from the current navigation index).
+  // tab. This adds at most kMaxNavigationCountToPersist navigations (in each
+  // direction from the current navigation index).
   // A pair is added to tab_to_available_range indicating the range of
   // indices that were written.
   void BuildCommandsForTab(
       const SessionID& window_id,
-      NavigationController* controller,
+      TabContentsWrapper* tab,
       int index_in_window,
       bool is_pinned,
       std::vector<SessionCommand*>* commands,
@@ -382,7 +388,7 @@ class SessionService : public BaseSessionService,
   // currently called when Save() is called to compare how often the
   // session data is currently saved verses when we may want to save it.
   // It records the data in UMA stats.
-  void RecordSessionUpdateHistogramData(NotificationType type,
+  void RecordSessionUpdateHistogramData(int type,
     base::TimeTicks* last_updated_time);
 
   // Helper methods to record the histogram data
