@@ -9,13 +9,15 @@
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_navigator.h"
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/test/in_process_browser_test.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "chrome/test/base/ui_test_utils.h"
 #include "content/browser/renderer_host/render_view_host.h"
 #include "content/browser/tab_contents/interstitial_page.h"
 #include "content/browser/tab_contents/navigation_entry.h"
 #include "content/browser/tab_contents/tab_contents.h"
+#include "content/common/notification_service.h"
 #include "net/base/cert_status_flags.h"
 #include "net/test/test_server.h"
 
@@ -38,6 +40,7 @@ class SSLUITest : public InProcessBrowserTest {
 
   // Browser will both run and display insecure content.
   virtual void SetUpCommandLine(CommandLine* command_line) {
+    command_line->AppendSwitch(switches::kAllowRunningInsecureContent);
   }
 
   void CheckAuthenticatedState(TabContents* tab,
@@ -115,9 +118,11 @@ class SSLUITest : public InProcessBrowserTest {
   void ProceedThroughInterstitial(TabContents* tab) {
     InterstitialPage* interstitial_page = tab->interstitial_page();
     ASSERT_TRUE(interstitial_page);
+    ui_test_utils::WindowedNotificationObserver observer(
+        content::NOTIFICATION_LOAD_STOP,
+        Source<NavigationController>(&tab->controller()));
     interstitial_page->Proceed();
-    // Wait for the navigation to be done.
-    ui_test_utils::WaitForNavigation(&(tab->controller()));
+    observer.Wait();
   }
 
   static bool GetFilePathWithHostAndPortReplacement(
@@ -244,8 +249,15 @@ IN_PROC_BROWSER_TEST_F(SSLUITest, TestHTTPWithBrokenHTTPSResource) {
   CheckUnauthenticatedState(browser()->GetSelectedTabContents());
 }
 
+// http://crbug.com/91745
+#if defined(OS_CHROMEOS)
+#define MAYBE_TestOKHTTPS FLAKY_TestOKHTTPS
+#else
+#define MAYBE_TestOKHTTPS TestOKHTTPS
+#endif
+
 // Visits a page over OK https:
-IN_PROC_BROWSER_TEST_F(SSLUITest, TestOKHTTPS) {
+IN_PROC_BROWSER_TEST_F(SSLUITest, MAYBE_TestOKHTTPS) {
   ASSERT_TRUE(https_server_.Start());
 
   ui_test_utils::NavigateToURL(browser(),
@@ -404,8 +416,13 @@ IN_PROC_BROWSER_TEST_F(SSLUITest, FLAKY_TestHTTPSExpiredCertAndGoForward) {
   ASSERT_TRUE(entry2);
 
   // Now go back so that a page is in the forward history.
-  tab->controller().GoBack();
-  ui_test_utils::WaitForNavigation(&(tab->controller()));
+  {
+    ui_test_utils::WindowedNotificationObserver observer(
+        content::NOTIFICATION_LOAD_STOP,
+        Source<NavigationController>(&tab->controller()));
+    tab->controller().GoBack();
+    observer.Wait();
+  }
   ASSERT_TRUE(tab->controller().CanGoForward());
   NavigationEntry* entry3 = tab->controller().GetActiveEntry();
   ASSERT_TRUE(entry1 == entry3);
@@ -417,8 +434,13 @@ IN_PROC_BROWSER_TEST_F(SSLUITest, FLAKY_TestHTTPSExpiredCertAndGoForward) {
                                  true);  // Interstitial showing
 
   // Simulate user clicking and holding on forward button.
-  tab->controller().GoToOffset(1);
-  ui_test_utils::WaitForNavigation(&(tab->controller()));
+  {
+    ui_test_utils::WindowedNotificationObserver observer(
+        content::NOTIFICATION_LOAD_STOP,
+        Source<NavigationController>(&tab->controller()));
+    tab->controller().GoToOffset(1);
+    observer.Wait();
+  }
 
   // We should be showing the second good page.
   EXPECT_FALSE(browser()->GetSelectedTabContents()->interstitial_page());
@@ -428,11 +450,19 @@ IN_PROC_BROWSER_TEST_F(SSLUITest, FLAKY_TestHTTPSExpiredCertAndGoForward) {
   EXPECT_TRUE(entry2 == entry4);
 }
 
+// Flaky on CrOS http://crbug.com/92292
+#if defined(OS_CHROMEOS)
+#define MAYBE_TestHTTPSErrorWithNoNavEntry \
+    DISABLED_TestHTTPSErrorWithNoNavEntry
+#else
+#define MAYBE_TestHTTPSErrorWithNoNavEntry TestHTTPSErrorWithNoNavEntry
+#endif  // defined(OS_CHROMEOS)
+
 // Open a page with a HTTPS error in a tab with no prior navigation (through a
 // link with a blank target).  This is to test that the lack of navigation entry
 // does not cause any problems (it was causing a crasher, see
 // http://crbug.com/19941).
-IN_PROC_BROWSER_TEST_F(SSLUITest, TestHTTPSErrorWithNoNavEntry) {
+IN_PROC_BROWSER_TEST_F(SSLUITest, MAYBE_TestHTTPSErrorWithNoNavEntry) {
   ASSERT_TRUE(https_server_expired_.Start());
 
   GURL url = https_server_expired_.GetURL("files/ssl/google.htm");
@@ -447,6 +477,50 @@ IN_PROC_BROWSER_TEST_F(SSLUITest, TestHTTPSErrorWithNoNavEntry) {
   ASSERT_TRUE(tab2->tab_contents()->interstitial_page());
 }
 
+// Disabled due to crash in downloads code that this triggers.
+// http://crbug.com/95331
+IN_PROC_BROWSER_TEST_F(SSLUITest, DISABLED_TestBadHTTPSDownload) {
+  ASSERT_TRUE(test_server()->Start());
+  ASSERT_TRUE(https_server_expired_.Start());
+  GURL url_non_dangerous = test_server()->GetURL("");
+  GURL url_dangerous = https_server_expired_.GetURL(
+      "files/downloads/dangerous/dangerous.exe");
+
+  // Visit a non-dangerous page.
+  ui_test_utils::NavigateToURL(browser(), url_non_dangerous);
+
+  // Now, start a transition to dangerous download.
+  {
+    ui_test_utils::WindowedNotificationObserver observer(
+        content::NOTIFICATION_LOAD_STOP, NotificationService::AllSources());
+    browser::NavigateParams navigate_params(browser(), url_dangerous,
+                                            PageTransition::TYPED);
+    browser::Navigate(&navigate_params);
+    observer.Wait();
+  }
+
+  // Proceed through the SSL interstitial. This doesn't use
+  // |ProceedThroughInterstitial| since no page load will commit.
+  TabContents* tab = browser()->GetSelectedTabContents();
+  ASSERT_TRUE(tab != NULL);
+  ASSERT_TRUE(tab->interstitial_page() != NULL);
+  {
+    ui_test_utils::WindowedNotificationObserver observer(
+        chrome::NOTIFICATION_DOWNLOAD_INITIATED,
+        NotificationService::AllSources());
+    tab->interstitial_page()->Proceed();
+    observer.Wait();
+  }
+
+  // There should still be an interstitial at this point. Press the
+  // back button on the browser. Note that this doesn't wait for a
+  // NAV_ENTRY_COMMITTED notification because going back with an
+  // active interstitial simply hides the interstitial.
+  ASSERT_TRUE(tab->interstitial_page() != NULL);
+  EXPECT_TRUE(browser()->CanGoBack());
+  browser()->GoBack(CURRENT_TAB);
+}
+
 //
 // Insecure content
 //
@@ -582,9 +656,11 @@ IN_PROC_BROWSER_TEST_F(SSLUITest, TestDisplaysInsecureContentTwoTabs) {
   params.disposition = NEW_FOREGROUND_TAB;
   params.tabstrip_index = 0;
   params.source_contents = tab1;
+  ui_test_utils::WindowedNotificationObserver observer(
+      content::NOTIFICATION_LOAD_STOP, NotificationService::AllSources());
   browser::Navigate(&params);
   TabContentsWrapper* tab2 = params.target_contents;
-  ui_test_utils::WaitForNavigation(&(tab2->controller()));
+  observer.Wait();
 
   // The new tab has insecure content.
   CheckAuthenticatedState(tab2->tab_contents(), true);
@@ -619,9 +695,11 @@ IN_PROC_BROWSER_TEST_F(SSLUITest, TestRunsInsecureContentTwoTabs) {
   browser::NavigateParams params(browser(), url, PageTransition::TYPED);
   params.disposition = NEW_FOREGROUND_TAB;
   params.source_contents = tab1;
+  ui_test_utils::WindowedNotificationObserver observer(
+      content::NOTIFICATION_LOAD_STOP, NotificationService::AllSources());
   browser::Navigate(&params);
   TabContentsWrapper* tab2 = params.target_contents;
-  ui_test_utils::WaitForNavigation(&(tab2->controller()));
+  observer.Wait();
 
   // The new tab has insecure content.
   CheckAuthenticationBrokenState(tab2->tab_contents(), 0, true, false);
@@ -728,9 +806,9 @@ IN_PROC_BROWSER_TEST_F(SSLUITest, DISABLED_TestCNInvalidStickiness) {
 }
 
 #if defined(OS_CHROMEOS)
-// This test seems to be flaky on chromiumos.
+// This test seems to be flaky and hang on chromiumos.
 // http://crbug.com/84419
-#define MAYBE_TestRefNavigation FLAKY_TestRefNavigation
+#define MAYBE_TestRefNavigation DISABLED_TestRefNavigation
 #else
 #define MAYBE_TestRefNavigation TestRefNavigation
 #endif
@@ -792,9 +870,10 @@ IN_PROC_BROWSER_TEST_F(SSLUITest, DISABLED_TestCloseTabWithUnsafePopup) {
   // Let's add another tab to make sure the browser does not exit when we close
   // the first tab.
   GURL url = test_server()->GetURL("files/ssl/google.html");
-  TabContentsWrapper* tab2 =
-      browser()->AddSelectedTabWithURL(url, PageTransition::TYPED);
-  ui_test_utils::WaitForNavigation(&(tab2->controller()));
+  ui_test_utils::WindowedNotificationObserver observer(
+      content::NOTIFICATION_LOAD_STOP, NotificationService::AllSources());
+  browser()->AddSelectedTabWithURL(url, PageTransition::TYPED);
+  observer.Wait();
 
   // Close the first tab.
   browser()->CloseTabContents(tab1);
@@ -937,23 +1016,33 @@ IN_PROC_BROWSER_TEST_F(SSLUITest, DISABLED_TestGoodFrameNavigation) {
 
   bool success = false;
   // Now navigate inside the frame.
-  EXPECT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
-      tab->render_view_host(), std::wstring(),
-      L"window.domAutomationController.send(clickLink('goodHTTPSLink'));",
-      &success));
-  EXPECT_TRUE(success);
-  ui_test_utils::WaitForNavigation(&tab->controller());
+  {
+    ui_test_utils::WindowedNotificationObserver observer(
+        content::NOTIFICATION_LOAD_STOP,
+        Source<NavigationController>(&tab->controller()));
+    EXPECT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
+        tab->render_view_host(), std::wstring(),
+        L"window.domAutomationController.send(clickLink('goodHTTPSLink'));",
+        &success));
+    ASSERT_TRUE(success);
+    observer.Wait();
+  }
 
   // We should still be fine.
   CheckAuthenticatedState(tab, false);
 
   // Now let's hit a bad page.
-  EXPECT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
-      tab->render_view_host(), std::wstring(),
-      L"window.domAutomationController.send(clickLink('badHTTPSLink'));",
-      &success));
-  EXPECT_TRUE(success);
-  ui_test_utils::WaitForNavigation(&tab->controller());
+  {
+    ui_test_utils::WindowedNotificationObserver observer(
+        content::NOTIFICATION_LOAD_STOP,
+        Source<NavigationController>(&tab->controller()));
+    EXPECT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
+        tab->render_view_host(), std::wstring(),
+        L"window.domAutomationController.send(clickLink('badHTTPSLink'));",
+        &success));
+    ASSERT_TRUE(success);
+    observer.Wait();
+  }
 
   // The security style should still be secure.
   CheckAuthenticatedState(tab, false);
@@ -969,24 +1058,39 @@ IN_PROC_BROWSER_TEST_F(SSLUITest, DISABLED_TestGoodFrameNavigation) {
   EXPECT_FALSE(is_content_evil);
 
   // Now go back, our state should still be OK.
-  tab->controller().GoBack();
-  ui_test_utils::WaitForNavigation(&tab->controller());
+  {
+    ui_test_utils::WindowedNotificationObserver observer(
+        content::NOTIFICATION_LOAD_STOP,
+        Source<NavigationController>(&tab->controller()));
+    tab->controller().GoBack();
+    observer.Wait();
+  }
   CheckAuthenticatedState(tab, false);
 
   // Navigate to a page served over HTTP.
-  EXPECT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
-      tab->render_view_host(), std::wstring(),
-      L"window.domAutomationController.send(clickLink('HTTPLink'));",
-      &success));
-  EXPECT_TRUE(success);
-  ui_test_utils::WaitForNavigation(&tab->controller());
+  {
+    ui_test_utils::WindowedNotificationObserver observer(
+        content::NOTIFICATION_LOAD_STOP,
+        Source<NavigationController>(&tab->controller()));
+    EXPECT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
+        tab->render_view_host(), std::wstring(),
+        L"window.domAutomationController.send(clickLink('HTTPLink'));",
+        &success));
+    ASSERT_TRUE(success);
+    observer.Wait();
+  }
 
   // Our state should be insecure.
   CheckAuthenticatedState(tab, true);
 
   // Go back, our state should be unchanged.
-  tab->controller().GoBack();
-  ui_test_utils::WaitForNavigation(&tab->controller());
+  {
+    ui_test_utils::WindowedNotificationObserver observer(
+        content::NOTIFICATION_LOAD_STOP,
+        Source<NavigationController>(&tab->controller()));
+    tab->controller().GoBack();
+    observer.Wait();
+  }
   CheckAuthenticatedState(tab, true);
 }
 
@@ -1013,12 +1117,15 @@ IN_PROC_BROWSER_TEST_F(SSLUITest, FLAKY_TestBadFrameNavigation) {
 
   // Navigate to a good frame.
   bool success = false;
+  ui_test_utils::WindowedNotificationObserver observer(
+        content::NOTIFICATION_LOAD_STOP,
+        Source<NavigationController>(&tab->controller()));
   EXPECT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
       tab->render_view_host(), std::wstring(),
       L"window.domAutomationController.send(clickLink('goodHTTPSLink'));",
       &success));
-  EXPECT_TRUE(success);
-  ui_test_utils::WaitForNavigation(&tab->controller());
+  ASSERT_TRUE(success);
+  observer.Wait();
 
   // We should still be authentication broken.
   CheckAuthenticationBrokenState(tab, net::CERT_STATUS_DATE_INVALID, false,
@@ -1045,24 +1152,35 @@ IN_PROC_BROWSER_TEST_F(SSLUITest, DISABLED_TestUnauthenticatedFrameNavigation) {
   CheckUnauthenticatedState(tab);
 
   // Now navigate inside the frame to a secure HTTPS frame.
-  bool success = false;
-  EXPECT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
-      tab->render_view_host(), std::wstring(),
-      L"window.domAutomationController.send(clickLink('goodHTTPSLink'));",
-      &success));
-  EXPECT_TRUE(success);
-  ui_test_utils::WaitForNavigation(&tab->controller());
+  {
+    bool success = false;
+    ui_test_utils::WindowedNotificationObserver observer(
+        content::NOTIFICATION_LOAD_STOP,
+        Source<NavigationController>(&tab->controller()));
+    EXPECT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
+        tab->render_view_host(), std::wstring(),
+        L"window.domAutomationController.send(clickLink('goodHTTPSLink'));",
+        &success));
+    ASSERT_TRUE(success);
+    observer.Wait();
+  }
 
   // We should still be unauthenticated.
   CheckUnauthenticatedState(tab);
 
   // Now navigate to a bad HTTPS frame.
-  EXPECT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
-      tab->render_view_host(), std::wstring(),
-      L"window.domAutomationController.send(clickLink('badHTTPSLink'));",
-      &success));
-  EXPECT_TRUE(success);
-  ui_test_utils::WaitForNavigation(&tab->controller());
+  {
+    bool success = false;
+    ui_test_utils::WindowedNotificationObserver observer(
+        content::NOTIFICATION_LOAD_STOP,
+        Source<NavigationController>(&tab->controller()));
+    EXPECT_TRUE(ui_test_utils::ExecuteJavaScriptAndExtractBool(
+        tab->render_view_host(), std::wstring(),
+        L"window.domAutomationController.send(clickLink('badHTTPSLink'));",
+        &success));
+    ASSERT_TRUE(success);
+    observer.Wait();
+  }
 
   // State should not have changed.
   CheckUnauthenticatedState(tab);
