@@ -66,8 +66,7 @@ void ProcessUpdatesCommand::ModelChangingExecuteImpl(SyncSession* session) {
 
   status->set_num_consecutive_errors(0);
 
-  // TODO(nick): The following line makes no sense to me.
-  status->set_syncing(true);
+  status->mutable_update_progress()->ClearVerifiedUpdates();
   return;
 }
 
