@@ -17,7 +17,7 @@
 #include "chrome/browser/sync/profile_sync_service_mock.h"
 #include "chrome/browser/sync/profile_sync_test_util.h"
 #include "chrome/browser/webdata/web_data_service.h"
-#include "chrome/test/profile_mock.h"
+#include "chrome/test/base/profile_mock.h"
 #include "content/browser/browser_thread.h"
 #include "content/common/content_notification_types.h"
 #include "content/common/notification_source.h"
@@ -124,15 +124,15 @@ class AutofillDataTypeControllerTest : public testing::Test {
         WillRepeatedly(Return(true));
     EXPECT_CALL(*model_associator_, SyncModelHasUserCreatedNodes(_)).
         WillRepeatedly(DoAll(SetArgumentPointee<0>(true), Return(true)));
-    EXPECT_CALL(*model_associator_, AssociateModels()).
+    EXPECT_CALL(*model_associator_, AssociateModels(_)).
         WillRepeatedly(Return(true));
-    EXPECT_CALL(service_, ActivateDataType(_, _));
+    EXPECT_CALL(service_, ActivateDataType(_, _, _));
     EXPECT_CALL(*change_processor_, IsRunning()).WillRepeatedly(Return(true));
   }
 
   void SetStopExpectations() {
-    EXPECT_CALL(service_, DeactivateDataType(_, _));
-    EXPECT_CALL(*model_associator_, DisassociateModels());
+    EXPECT_CALL(service_, DeactivateDataType(_));
+    EXPECT_CALL(*model_associator_, DisassociateModels(_));
   }
 
   void WaitForEmptyDBMessageLoop() {
@@ -181,7 +181,7 @@ TEST_F(AutofillDataTypeControllerTest, AbortWhilePDMStarting) {
   autofill_dtc_->Start(NewCallback(&start_callback_, &StartCallback::Run));
   EXPECT_EQ(DataTypeController::MODEL_STARTING, autofill_dtc_->state());
 
-  EXPECT_CALL(service_, DeactivateDataType(_, _)).Times(0);
+  EXPECT_CALL(service_, DeactivateDataType(_)).Times(0);
   EXPECT_CALL(start_callback_, Run(DataTypeController::ABORTED, _));
   autofill_dtc_->Stop();
   EXPECT_EQ(DataTypeController::NOT_RUNNING, autofill_dtc_->state());
@@ -199,7 +199,7 @@ TEST_F(AutofillDataTypeControllerTest, AbortWhileWDSStarting) {
   autofill_dtc_->Start(NewCallback(&start_callback_, &StartCallback::Run));
   EXPECT_EQ(DataTypeController::MODEL_STARTING, autofill_dtc_->state());
 
-  EXPECT_CALL(service_, DeactivateDataType(_, _)).Times(0);
+  EXPECT_CALL(service_, DeactivateDataType(_)).Times(0);
   EXPECT_CALL(start_callback_, Run(DataTypeController::ABORTED, _));
   autofill_dtc_->Stop();
   EXPECT_EQ(DataTypeController::NOT_RUNNING, autofill_dtc_->state());
@@ -239,7 +239,7 @@ TEST_F(AutofillDataTypeControllerTest, AbortWhileAssociatingNotActivated) {
   wait_for_db_thread_pause.Wait();
   EXPECT_EQ(DataTypeController::ASSOCIATING, autofill_dtc_->state());
 
-  EXPECT_CALL(service_, DeactivateDataType(_, _)).Times(0);
+  EXPECT_CALL(service_, DeactivateDataType(_));
   EXPECT_CALL(start_callback_, Run(DataTypeController::ABORTED, _));
   autofill_dtc_->Stop();
   EXPECT_EQ(DataTypeController::NOT_RUNNING, autofill_dtc_->state());
@@ -259,7 +259,7 @@ TEST_F(AutofillDataTypeControllerTest, AbortWhileAssociatingActivated) {
       WillRepeatedly(Return(true));
   EXPECT_CALL(*model_associator_, SyncModelHasUserCreatedNodes(_)).
       WillRepeatedly(DoAll(SetArgumentPointee<0>(true), Return(true)));
-  EXPECT_CALL(*model_associator_, AssociateModels()).
+  EXPECT_CALL(*model_associator_, AssociateModels(_)).
       WillRepeatedly(Return(true));
   EXPECT_CALL(*change_processor_, IsRunning()).WillRepeatedly(Return(true));
 
@@ -268,7 +268,7 @@ TEST_F(AutofillDataTypeControllerTest, AbortWhileAssociatingActivated) {
   // ActivateDataType() before  allowing it to continue.
   WaitableEvent pause_db_thread(false, false);
   WaitableEvent wait_for_db_thread_pause(false, false);
-  EXPECT_CALL(service_, ActivateDataType(_, _)).
+  EXPECT_CALL(service_, ActivateDataType(_, _, _)).
       WillOnce(DoAll(
           SignalEvent(&wait_for_db_thread_pause),
           WaitOnEvent(&pause_db_thread)));
