@@ -9,10 +9,15 @@
 #include <vector>
 
 #include "chrome/browser/bookmarks/bookmark_model_observer.h"
-#include "chrome/browser/sync/engine/syncapi.h"
 #include "chrome/browser/sync/glue/bookmark_model_associator.h"
 #include "chrome/browser/sync/glue/change_processor.h"
 #include "chrome/browser/sync/glue/sync_backend_host.h"
+#include "chrome/browser/sync/internal_api/sync_manager.h"
+
+namespace sync_api {
+class WriteNode;
+class WriteTransaction;
+}  // namespace sync_api
 
 namespace browser_sync {
 
