@@ -14,9 +14,12 @@
 #include "base/utf_string_conversions.h"
 #include "chrome/browser/bookmarks/bookmark_model.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/sync/engine/syncapi.h"
+#include "chrome/browser/sync/api/sync_error.h"
 #include "chrome/browser/sync/glue/bookmark_change_processor.h"
-#include "chrome/browser/sync/syncable/nigori_util.h"
+#include "chrome/browser/sync/internal_api/read_node.h"
+#include "chrome/browser/sync/internal_api/read_transaction.h"
+#include "chrome/browser/sync/internal_api/write_node.h"
+#include "chrome/browser/sync/internal_api/write_transaction.h"
 #include "chrome/browser/sync/util/cryptographer.h"
 #include "chrome/common/chrome_switches.h"
 #include "content/browser/browser_thread.h"
@@ -44,6 +47,9 @@ namespace browser_sync {
 static const char kBookmarkBarTag[] = "bookmark_bar";
 static const char kSyncedBookmarksTag[] = "synced_bookmarks";
 static const char kOtherBookmarksTag[] = "other_bookmarks";
+static const char kServerError[] =
+    "Server did not create top-level nodes.  Possibly we are running against "
+    "an out-of-date server?";
 
 // Bookmark comparer for map of bookmark nodes.
 class BookmarkComparer {
@@ -99,7 +105,7 @@ const BookmarkNode* BookmarkNodeFinder::FindBookmarkNode(
     const sync_api::BaseNode& sync_node) {
   // Create a bookmark node from the given sync node.
   BookmarkNode temp_node(sync_node.GetURL());
-  temp_node.set_title(WideToUTF16Hack(sync_node.GetTitle()));
+  temp_node.set_title(UTF8ToUTF16(sync_node.GetTitle()));
   if (sync_node.GetIsFolder())
     temp_node.set_type(BookmarkNode::FOLDER);
   else
@@ -178,7 +184,7 @@ BookmarkModelAssociator::~BookmarkModelAssociator() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
 }
 
-bool BookmarkModelAssociator::DisassociateModels() {
+bool BookmarkModelAssociator::DisassociateModels(SyncError* error) {
   id_map_.clear();
   id_map_inverse_.clear();
   dirty_associations_sync_ids_.clear();
@@ -276,9 +282,10 @@ bool BookmarkModelAssociator::SyncModelHasUserCreatedNodes(bool* has_nodes) {
   return true;
 }
 
-bool BookmarkModelAssociator::NodesMatch(const BookmarkNode* bookmark,
+bool BookmarkModelAssociator::NodesMatch(
+    const BookmarkNode* bookmark,
     const sync_api::BaseNode* sync_node) const {
-  if (bookmark->GetTitle() != WideToUTF16Hack(sync_node->GetTitle()))
+  if (bookmark->GetTitle() != UTF8ToUTF16(sync_node->GetTitle()))
     return false;
   if (bookmark->is_folder() != sync_node->GetIsFolder())
     return false;
@@ -315,20 +322,20 @@ bool BookmarkModelAssociator::GetSyncIdForTaggedNode(const std::string& tag,
   return true;
 }
 
-bool BookmarkModelAssociator::AssociateModels() {
+bool BookmarkModelAssociator::AssociateModels(SyncError* error) {
   // Try to load model associations from persisted associations first. If that
   // succeeds, we don't need to run the complex model matching algorithm.
   if (LoadAssociations())
     return true;
 
-  DisassociateModels();
+  DisassociateModels(error);
 
   // We couldn't load model associations from persisted associations. So build
   // them.
-  return BuildAssociations();
+  return BuildAssociations(error);
 }
 
-bool BookmarkModelAssociator::BuildAssociations() {
+bool BookmarkModelAssociator::BuildAssociations(SyncError* error) {
   // Algorithm description:
   // Match up the roots and recursively do the following:
   // * For each sync node for the current sync parent node, find the best
@@ -352,14 +359,12 @@ bool BookmarkModelAssociator::BuildAssociations() {
   // and Other Bookmarks.
   if (!AssociateTaggedPermanentNode(bookmark_model_->other_node(),
                                     kOtherBookmarksTag)) {
-    LOG(ERROR) << "Server did not create top-level nodes.  Possibly we "
-               << "are running against an out-of-date server?";
+    error->Reset(FROM_HERE, kServerError, model_type());
     return false;
   }
   if (!AssociateTaggedPermanentNode(bookmark_model_->bookmark_bar_node(),
                                     kBookmarkBarTag)) {
-    LOG(ERROR) << "Server did not create top-level nodes.  Possibly we "
-               << "are running against an out-of-date server?";
+    error->Reset(FROM_HERE, kServerError, model_type());
     return false;
   }
   if (!AssociateTaggedPermanentNode(bookmark_model_->synced_node(),
@@ -368,8 +373,7 @@ bool BookmarkModelAssociator::BuildAssociations() {
       // server if the command line flag is set.
       CommandLine::ForCurrentProcess()->HasSwitch(
       switches::kEnableSyncedBookmarksFolder)) {
-    LOG(ERROR) << "Server did not create top-level synced nodes.  Possibly "
-               << "we are running against an out-of-date server?";
+    error->Reset(FROM_HERE, kServerError, model_type());
     return false;
   }
   int64 bookmark_bar_sync_id = GetSyncIdFromChromeId(
@@ -399,6 +403,7 @@ bool BookmarkModelAssociator::BuildAssociations() {
 
     sync_api::ReadNode sync_parent(&trans);
     if (!sync_parent.InitByIdLookup(sync_parent_id)) {
+      error->Reset(FROM_HERE, "Failed to lookup node.", model_type());
       return false;
     }
     // Only folder nodes are pushed on to the stack.
@@ -414,6 +419,7 @@ bool BookmarkModelAssociator::BuildAssociations() {
     while (sync_child_id != sync_api::kInvalidId) {
       sync_api::WriteNode sync_child_node(&trans);
       if (!sync_child_node.InitByIdLookup(sync_child_id)) {
+        error->Reset(FROM_HERE, "Failed to lookup node.", model_type());
         return false;
       }
 
