@@ -6,8 +6,8 @@
 #define CHROME_BROWSER_SYNC_GLUE_CHANGE_PROCESSOR_H_
 #pragma once
 
-#include "chrome/browser/sync/engine/syncapi.h"
 #include "chrome/browser/sync/glue/sync_backend_host.h"
+#include "chrome/browser/sync/internal_api/sync_manager.h"
 
 class Profile;
 
