@@ -5,27 +5,35 @@
 #include "chrome/browser/sync/glue/session_model_associator.h"
 
 #include <algorithm>
+#include <set>
 #include <utility>
 
 #include "base/logging.h"
+#include "base/sys_info.h"
 #include "base/tracked.h"
-#include "chrome/browser/extensions/extension_tab_helper.h"
+#include "chrome/browser/browser_process.h"
 #include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/sessions/restore_tab_helper.h"
 #include "chrome/browser/sessions/session_service_factory.h"
+#include "chrome/browser/sync/api/sync_error.h"
+#include "chrome/browser/sync/glue/synced_session.h"
+#include "chrome/browser/sync/glue/synced_tab_delegate.h"
+#include "chrome/browser/sync/glue/synced_window_delegate.h"
+#include "chrome/browser/sync/internal_api/read_node.h"
+#include "chrome/browser/sync/internal_api/read_transaction.h"
+#include "chrome/browser/sync/internal_api/write_node.h"
+#include "chrome/browser/sync/internal_api/write_transaction.h"
 #include "chrome/browser/sync/profile_sync_service.h"
 #include "chrome/browser/sync/syncable/syncable.h"
-#include "chrome/browser/tabs/tab_strip_model.h"
-#include "chrome/browser/ui/browser_list.h"
-#include "chrome/browser/ui/browser_window.h"
-#include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
 #include "chrome/common/chrome_notification_types.h"
-#include "chrome/common/extensions/extension.h"
 #include "chrome/common/url_constants.h"
-#include "content/browser/tab_contents/navigation_controller.h"
 #include "content/browser/tab_contents/navigation_entry.h"
 #include "content/common/notification_details.h"
 #include "content/common/notification_service.h"
+#if defined(OS_LINUX)
+#include "base/linux_util.h"
+#elif defined(OS_WIN)
+#include <windows.h>
+#endif
 
 namespace browser_sync {
 
@@ -102,7 +110,7 @@ int64 SessionModelAssociator::GetSyncIdFromSessionTag(const std::string& tag) {
   return node.GetId();
 }
 
-const TabContentsWrapper*
+const SyncedTabDelegate*
 SessionModelAssociator::GetChromeNodeFromSyncId(int64 sync_id) {
   NOTREACHED();
   return NULL;
@@ -123,26 +131,40 @@ void SessionModelAssociator::ReassociateWindows(bool reload_tabs) {
   sync_pb::SessionHeader* header_s = specifics.mutable_header();
   SyncedSession* current_session =
       synced_session_tracker_.GetSession(local_tag);
-  current_session->windows.reserve(BrowserList::size());
+  header_s->set_client_name(current_session_name_);
+#if defined(OS_LINUX)
+  header_s->set_device_type(sync_pb::SessionHeader_DeviceType_TYPE_LINUX);
+#elif defined(OS_MACOSX)
+  header_s->set_device_type(sync_pb::SessionHeader_DeviceType_TYPE_MAC);
+#elif defined(OS_WIN)
+  header_s->set_device_type(sync_pb::SessionHeader_DeviceType_TYPE_WIN);
+#elif defined(OS_CHROMEOS)
+  header_s->set_device_type(sync_pb::SessionHeader_DeviceType_TYPE_CROS);
+#else
+  header_s->set_device_type(sync_pb::SessionHeader_DeviceType_TYPE_OTHER);
+#endif
 
   size_t window_num = 0;
-  for (BrowserList::const_iterator i = BrowserList::begin();
-       i != BrowserList::end(); ++i) {
-    // Make sure the browser has tabs and a window. Browsers destructor
-    // removes itself from the BrowserList. When a browser is closed the
+  std::set<SyncedWindowDelegate*> windows =
+      SyncedWindowDelegate::GetSyncedWindowDelegates();
+  current_session->windows.reserve(windows.size());
+  for (std::set<SyncedWindowDelegate*>::const_iterator i =
+       windows.begin(); i != windows.end(); ++i) {
+    // Make sure the window has tabs and a viewable window. The viewable window
+    // check is necessary because, for example, when a browser is closed the
     // destructor is not necessarily run immediately. This means its possible
     // for us to get a handle to a browser that is about to be removed. If
     // the tab count is 0 or the window is NULL, the browser is about to be
     // deleted, so we ignore it.
-    if (ShouldSyncWindow(*i) && (*i)->tab_count() &&
-        (*i)->window()) {
+    if (ShouldSyncWindow(*i) && (*i)->GetTabCount() &&
+        (*i)->HasWindow()) {
       sync_pb::SessionWindow window_s;
-      SessionID::id_type window_id = (*i)->session_id().id();
+      SessionID::id_type window_id = (*i)->GetSessionId();
       VLOG(1) << "Reassociating window " << window_id << " with " <<
-          (*i)->tab_count() << " tabs.";
+          (*i)->GetTabCount() << " tabs.";
       window_s.set_window_id(window_id);
-      window_s.set_selected_tab_index((*i)->active_index());
-      if ((*i)->is_type_tabbed()) {
+      window_s.set_selected_tab_index((*i)->GetActiveIndex());
+      if ((*i)->IsTypeTabbed()) {
         window_s.set_browser_type(
             sync_pb::SessionWindow_BrowserType_TYPE_TABBED);
       } else {
@@ -152,12 +174,12 @@ void SessionModelAssociator::ReassociateWindows(bool reload_tabs) {
 
       // Store the order of tabs.
       bool found_tabs = false;
-      for (int j = 0; j < (*i)->tab_count(); ++j) {
-        TabContentsWrapper* tab = (*i)->GetTabContentsWrapperAt(j);
+      for (int j = 0; j < (*i)->GetTabCount(); ++j) {
+        SyncedTabDelegate* tab = (*i)->GetTabAt(j);
         DCHECK(tab);
         if (IsValidTab(*tab)) {
           found_tabs = true;
-          window_s.add_tab(tab->restore_tab_helper()->session_id().id());
+          window_s.add_tab(tab->GetSessionId());
           if (reload_tabs) {
             ReassociateTab(*tab);
           }
@@ -194,16 +216,17 @@ void SessionModelAssociator::ReassociateWindows(bool reload_tabs) {
 }
 
 // Static.
-bool SessionModelAssociator::ShouldSyncWindow(const Browser* browser) {
-  if (browser->is_app())
+bool SessionModelAssociator::ShouldSyncWindow(
+    const SyncedWindowDelegate* window) {
+  if (window->IsApp())
     return false;
-  return browser->is_type_tabbed() || browser->is_type_popup();
+  return window->IsTypeTabbed() || window->IsTypePopup();
 }
 
 void SessionModelAssociator::ReassociateTabs(
-    const std::vector<TabContentsWrapper*>& tabs) {
+    const std::vector<SyncedTabDelegate*>& tabs) {
   DCHECK(CalledOnValidThread());
-  for (std::vector<TabContentsWrapper*>::const_iterator i = tabs.begin();
+  for (std::vector<SyncedTabDelegate*>::const_iterator i = tabs.begin();
        i != tabs.end();
        ++i) {
     ReassociateTab(**i);
@@ -211,14 +234,11 @@ void SessionModelAssociator::ReassociateTabs(
   if (waiting_for_change_) QuitLoopForTest();
 }
 
-void SessionModelAssociator::ReassociateTab(const TabContentsWrapper& tab) {
+void SessionModelAssociator::ReassociateTab(const SyncedTabDelegate& tab) {
   DCHECK(CalledOnValidThread());
-  if (!IsValidTab(tab))
-    return;
-
   int64 sync_id;
-  SessionID::id_type id = tab.restore_tab_helper()->session_id().id();
-  if (tab.tab_contents()->is_being_destroyed()) {
+  SessionID::id_type id = tab.GetSessionId();
+  if (tab.IsBeingDestroyed()) {
     // This tab is closing.
     TabLinksMap::iterator tab_iter = tab_map_.find(id);
     if (tab_iter == tab_map_.end()) {
@@ -230,10 +250,15 @@ void SessionModelAssociator::ReassociateTab(const TabContentsWrapper& tab) {
     return;
   }
 
+  if (!IsValidTab(tab))
+    return;
+
   TabLinksMap::const_iterator tablink = tab_map_.find(id);
   if (tablink == tab_map_.end()) {
     // This is a new tab, get a sync node for it.
     sync_id = tab_pool_.GetFreeTabNode();
+    if (sync_id == sync_api::kInvalidId)
+      return;
   } else {
     // This tab is already associated with a sync node, reuse it.
     sync_id = tablink->second.sync_id();
@@ -241,29 +266,28 @@ void SessionModelAssociator::ReassociateTab(const TabContentsWrapper& tab) {
   Associate(&tab, sync_id);
 }
 
-void SessionModelAssociator::Associate(const TabContentsWrapper* tab,
+void SessionModelAssociator::Associate(const SyncedTabDelegate* tab,
                                        int64 sync_id) {
   DCHECK(CalledOnValidThread());
-  SessionID::id_type session_id = tab->restore_tab_helper()->session_id().id();
-  Browser* browser = BrowserList::FindBrowserWithID(
-      tab->restore_tab_helper()->window_id().id());
-  if (!browser) {  // Can happen for weird things like developer console.
-    tab_pool_.FreeTabNode(sync_id);
-    return;
-  }
+  SessionID::id_type session_id = tab->GetSessionId();
+  const SyncedWindowDelegate* window =
+      SyncedWindowDelegate::FindSyncedWindowDelegateWithId(
+          tab->GetWindowId());
+  DCHECK(window);
 
   TabLinks t(sync_id, tab);
   tab_map_[session_id] = t;
 
   sync_api::WriteTransaction trans(FROM_HERE, sync_service_->GetUserShare());
-  WriteTabContentsToSyncModel(*browser, *tab, sync_id, &trans);
+  WriteTabContentsToSyncModel(*window, *tab, sync_id, &trans);
 }
 
 bool SessionModelAssociator::WriteTabContentsToSyncModel(
-    const Browser& browser,
-    const TabContentsWrapper& tab,
+    const SyncedWindowDelegate& window,
+    const SyncedTabDelegate& tab,
     int64 sync_id,
     sync_api::WriteTransaction* trans) {
+
   DCHECK(CalledOnValidThread());
   sync_api::WriteNode tab_node(trans);
   if (!tab_node.InitByIdLookup(sync_id)) {
@@ -275,25 +299,22 @@ bool SessionModelAssociator::WriteTabContentsToSyncModel(
   session_s.set_session_tag(GetCurrentMachineTag());
   sync_pb::SessionTab* tab_s = session_s.mutable_tab();
 
-  SessionID::id_type tab_id = tab.restore_tab_helper()->session_id().id();
+  SessionID::id_type tab_id = tab.GetSessionId();
   tab_s->set_tab_id(tab_id);
-  tab_s->set_window_id(tab.restore_tab_helper()->window_id().id());
-  const int current_index = tab.controller().GetCurrentEntryIndex();
+  tab_s->set_window_id(tab.GetWindowId());
+  const int current_index = tab.GetCurrentEntryIndex();
   const int min_index = std::max(0,
                                  current_index - max_sync_navigation_count);
   const int max_index = std::min(current_index + max_sync_navigation_count,
-                                 tab.controller().entry_count());
-  const int pending_index = tab.controller().pending_entry_index();
-  int index_in_window = browser.tabstrip_model()->GetIndexOfTabContents(&tab);
-  DCHECK(index_in_window != TabStripModel::kNoTab);
-  tab_s->set_pinned(browser.tabstrip_model()->IsTabPinned(index_in_window));
-  if (tab.extension_tab_helper()->extension_app()) {
-    tab_s->set_extension_app_id(
-        tab.extension_tab_helper()->extension_app()->id());
+                                 tab.GetEntryCount());
+  const int pending_index = tab.GetPendingEntryIndex();
+  tab_s->set_pinned(window.IsTabPinned(&tab));
+  if (tab.HasExtensionAppId()) {
+    tab_s->set_extension_app_id(tab.GetExtensionAppId());
   }
   for (int i = min_index; i < max_index; ++i) {
     const NavigationEntry* entry = (i == pending_index) ?
-       tab.controller().pending_entry() : tab.controller().GetEntryAtIndex(i);
+       tab.GetPendingEntry() : tab.GetEntryAtIndex(i);
     DCHECK(entry);
     if (entry->virtual_url().is_valid()) {
       if (i == max_index - 1) {
@@ -405,7 +426,7 @@ void SessionModelAssociator::Disassociate(int64 sync_id) {
   // TODO(zea): we will need this once we support deleting foreign sessions.
 }
 
-bool SessionModelAssociator::AssociateModels() {
+bool SessionModelAssociator::AssociateModels(SyncError* error) {
   DCHECK(CalledOnValidThread());
 
   // Ensure that we disassociated properly, otherwise memory might leak.
@@ -421,22 +442,34 @@ bool SessionModelAssociator::AssociateModels() {
 
     sync_api::ReadNode root(&trans);
     if (!root.InitByTagLookup(kSessionsTag)) {
-      LOG(ERROR) << kNoSessionsFolderError;
+      error->Reset(FROM_HERE, kNoSessionsFolderError, model_type());
       return false;
     }
 
     // Make sure we have a machine tag.
-    if (current_machine_tag_.empty())
+    if (current_machine_tag_.empty()) {
       InitializeCurrentMachineTag(&trans);
+      // The session name is retrieved asynchronously so it might not come back
+      // for the writing of the session. However, we write to the session often
+      // enough (on every navigation) that we'll pick it up quickly.
+      InitializeCurrentSessionName();
+    }
     synced_session_tracker_.SetLocalSessionTag(current_machine_tag_);
-    UpdateAssociationsFromSyncModel(root, &trans);
+    if (!UpdateAssociationsFromSyncModel(root, &trans)) {
+      error->Reset(FROM_HERE,
+                   "Failed to update associations from sync",
+                   model_type());
+      return false;
+    }
 
     if (local_session_syncid_ == sync_api::kInvalidId) {
       // The sync db didn't have a header node for us, we need to create one.
       sync_api::WriteNode write_node(&trans);
       if (!write_node.InitUniqueByCreation(syncable::SESSIONS, root,
           current_machine_tag_)) {
-        LOG(ERROR) << "Failed to create sessions header sync node.";
+        error->Reset(FROM_HERE,
+                     "Failed to create sessions header sync node.",
+                     model_type());
         return false;
       }
       write_node.SetTitle(UTF8ToWide(current_machine_tag_));
@@ -452,12 +485,14 @@ bool SessionModelAssociator::AssociateModels() {
   return true;
 }
 
-bool SessionModelAssociator::DisassociateModels() {
+bool SessionModelAssociator::DisassociateModels(SyncError* error) {
   DCHECK(CalledOnValidThread());
   synced_session_tracker_.clear();
   tab_map_.clear();
   tab_pool_.clear();
   local_session_syncid_ = sync_api::kInvalidId;
+  current_machine_tag_ = "";
+  current_session_name_ = "";
 
   // There is no local model stored with which to disassociate, just notify
   // foreign session handlers.
@@ -472,19 +507,64 @@ void SessionModelAssociator::InitializeCurrentMachineTag(
     sync_api::WriteTransaction* trans) {
   DCHECK(CalledOnValidThread());
   syncable::Directory* dir = trans->GetWrappedWriteTrans()->directory();
-
-  // TODO(zea): We need a better way of creating a machine tag. The directory
-  // kernel's cache_guid changes every time syncing is turned on and off. This
-  // will result in session's associated with stale machine tags persisting on
-  // the server since that tag will not be reused. Eventually this should
-  // become some string identifiable to the user. (Home, Work, Laptop, etc.)
-  // See issue at http://crbug.com/59672
   current_machine_tag_ = "session_sync";
   current_machine_tag_.append(dir->cache_guid());
   VLOG(1) << "Creating machine tag: " << current_machine_tag_;
   tab_pool_.set_machine_tag(current_machine_tag_);
 }
 
+void SessionModelAssociator::OnSessionNameInitialized(const std::string name) {
+  DCHECK(CalledOnValidThread());
+  // Only use the default machine name if it hasn't already been set.
+  if (current_session_name_.empty()) {
+    current_session_name_ = name;
+  }
+}
+
+// Task which runs on the file thread because it runs system calls which can
+// block while retrieving sytem information.
+class GetSessionNameTask : public Task {
+ public:
+  explicit GetSessionNameTask(
+      const WeakHandle<SessionModelAssociator> associator) :
+    associator_(associator) {}
+
+  virtual void Run() {
+#if defined(OS_LINUX)
+    std::string session_name = base::GetLinuxDistro();
+#elif defined(OS_MACOSX)
+    std::string session_name = SessionModelAssociator::GetHardwareModelName();
+#elif defined(OS_WIN)
+    std::string session_name = SessionModelAssociator::GetComputerName();
+#else
+    std::string session_name;
+#endif
+    if (session_name == "Unknown" || session_name.empty()) {
+      session_name = base::SysInfo::OperatingSystemName();
+    }
+    associator_.Call(FROM_HERE,
+                     &SessionModelAssociator::OnSessionNameInitialized,
+                     session_name);
+  }
+  const WeakHandle<SessionModelAssociator> associator_;
+
+  DISALLOW_COPY_AND_ASSIGN(GetSessionNameTask);
+};
+
+void SessionModelAssociator::InitializeCurrentSessionName() {
+  DCHECK(CalledOnValidThread());
+  if (setup_for_test_) {
+    OnSessionNameInitialized("TestSessionName");
+  } else {
+#if defined(OS_CHROMEOS)
+    OnSessionNameInitialized("Chromebook");
+#else
+    BrowserThread::PostTask(BrowserThread::FILE, FROM_HERE,
+        new GetSessionNameTask(MakeWeakHandle(AsWeakPtr())));
+#endif
+  }
+}
+
 bool SessionModelAssociator::UpdateAssociationsFromSyncModel(
     const sync_api::ReadNode& root,
     const sync_api::BaseTransaction* trans) {
@@ -509,12 +589,17 @@ bool SessionModelAssociator::UpdateAssociationsFromSyncModel(
     } else if (id != local_session_syncid_) {
       // This is previously stored local session information.
       if (specifics.has_header()) {
-        DCHECK_EQ(sync_api::kInvalidId, local_session_syncid_);
+        if (sync_api::kInvalidId != local_session_syncid_)
+          return false;
 
         // This is our previous header node, reuse it.
         local_session_syncid_ = id;
+        if (specifics.header().has_client_name()) {
+          current_session_name_ = specifics.header().client_name();
+        }
       } else {
-        DCHECK(specifics.has_tab());
+        if (!specifics.has_tab())
+          return false;
 
         // This is a tab node. We want to track these to reuse them in our free
         // tab node pool. They will be overwritten eventually, so need to do
@@ -527,7 +612,8 @@ bool SessionModelAssociator::UpdateAssociationsFromSyncModel(
   }
 
   // After updating from sync model all tabid's should be free.
-  DCHECK(tab_pool_.full());
+  if (!tab_pool_.full())
+    return false;
 
   return true;
 }
@@ -537,7 +623,8 @@ bool SessionModelAssociator::AssociateForeignSpecifics(
     const int64 modification_time) {
   DCHECK(CalledOnValidThread());
   std::string foreign_session_tag = specifics.session_tag();
-  DCHECK(foreign_session_tag != GetCurrentMachineTag() || setup_for_test_);
+  if (foreign_session_tag == GetCurrentMachineTag() && !setup_for_test_)
+    return false;
 
   if (specifics.has_header()) {
     // Read in the header data for this foreign session.
@@ -549,6 +636,7 @@ bool SessionModelAssociator::AssociateForeignSpecifics(
         synced_session_tracker_.GetSession(foreign_session_tag);
 
     const sync_pb::SessionHeader& header = specifics.header();
+    PopulateSessionHeaderFromSpecifics(header, foreign_session);
     foreign_session->windows.reserve(header.window_size());
     VLOG(1) << "Associating " << foreign_session_tag << " with " <<
         header.window_size() << " windows.";
@@ -592,6 +680,36 @@ void SessionModelAssociator::DisassociateForeignSession(
   synced_session_tracker_.DeleteSession(foreign_session_tag);
 }
 
+// Static
+void SessionModelAssociator::PopulateSessionHeaderFromSpecifics(
+    const sync_pb::SessionHeader& header_specifics,
+    SyncedSession* session_header) {
+  if (header_specifics.has_client_name()) {
+    session_header->session_name = header_specifics.client_name();
+  }
+  if (header_specifics.has_device_type()) {
+    switch (header_specifics.device_type()) {
+      case sync_pb::SessionHeader_DeviceType_TYPE_WIN:
+        session_header->device_type = SyncedSession::TYPE_WIN;
+        break;
+      case sync_pb::SessionHeader_DeviceType_TYPE_MAC:
+        session_header->device_type = SyncedSession::TYPE_MACOSX;
+        break;
+      case sync_pb::SessionHeader_DeviceType_TYPE_LINUX:
+        session_header->device_type = SyncedSession::TYPE_LINUX;
+        break;
+      case sync_pb::SessionHeader_DeviceType_TYPE_CROS:
+        session_header->device_type = SyncedSession::TYPE_CHROMEOS;
+        break;
+      case sync_pb::SessionHeader_DeviceType_TYPE_OTHER:
+        // Intentionally fall-through
+      default:
+        session_header->device_type = SyncedSession::TYPE_OTHER;
+        break;
+    }
+  }
+}
+
 // Static
 void SessionModelAssociator::PopulateSessionWindowFromSpecifics(
     const std::string& session_tag,
@@ -764,15 +882,16 @@ int64 SessionModelAssociator::TabNodePool::GetFreeTabNode() {
     sync_api::ReadNode root(&trans);
     if (!root.InitByTagLookup(kSessionsTag)) {
       LOG(ERROR) << kNoSessionsFolderError;
-      return 0;
+      return sync_api::kInvalidId;
     }
     size_t tab_node_id = tab_syncid_pool_.size();
     std::string tab_node_tag = TabIdToTag(machine_tag_, tab_node_id);
     sync_api::WriteNode tab_node(&trans);
     if (!tab_node.InitUniqueByCreation(syncable::SESSIONS, root,
                                        tab_node_tag)) {
-      LOG(ERROR) << "Could not create new node!";
-      return -1;
+      LOG(ERROR) << "Could not create new node with tag "
+                 << tab_node_tag << "!";
+      return sync_api::kInvalidId;
     }
     tab_node.SetTitle(UTF8ToWide(tab_node_tag));
 
@@ -841,16 +960,21 @@ bool SessionModelAssociator::SessionWindowHasNoTabsToSync(
 }
 
 // Valid local tab?
-bool SessionModelAssociator::IsValidTab(const TabContentsWrapper& tab) {
+bool SessionModelAssociator::IsValidTab(const SyncedTabDelegate& tab) {
   DCHECK(CalledOnValidThread());
   if ((tab.profile() == sync_service_->profile() ||
-       sync_service_->profile() == NULL)) {
-    const NavigationEntry* entry = tab.controller().GetActiveEntry();
+       sync_service_->profile() == NULL)) {          // For tests.
+    const SyncedWindowDelegate* window =
+        SyncedWindowDelegate::FindSyncedWindowDelegateWithId(
+            tab.GetWindowId());
+    if (!window)
+      return false;
+    const NavigationEntry* entry = tab.GetActiveEntry();
     if (!entry)
       return false;
     if (entry->virtual_url().is_valid() &&
         (entry->virtual_url() != GURL(chrome::kChromeUINewTabURL) ||
-         tab.controller().entry_count() > 1)) {
+         tab.GetEntryCount() > 1)) {
       return true;
     }
   }
@@ -993,7 +1117,7 @@ bool SessionModelAssociator::SyncLocalWindowToSyncModel(
   for (size_t i = 0; i < window.tabs.size(); ++i) {
     SessionTab* tab = window.tabs[i];
     int64 id = tab_pool_.GetFreeTabNode();
-    if (id == -1) {
+    if (id == sync_api::kInvalidId) {
       LOG(ERROR) << "Failed to find/generate free sync node for tab.";
       return false;
     }
@@ -1058,4 +1182,18 @@ bool SessionModelAssociator::CryptoReadyIfNecessary() {
          sync_service_->IsCryptographerReady(&trans);
 }
 
+#if defined(OS_WIN)
+// Static
+// TODO(nzea): This isn't safe to call on the UI-thread. Move it out to a util
+// or object that lives on the FILE thread.
+std::string SessionModelAssociator::GetComputerName() {
+  char computer_name[MAX_COMPUTERNAME_LENGTH + 1];
+  DWORD size = sizeof(computer_name);
+  if (GetComputerNameA(computer_name, &size)) {
+    return computer_name;
+  }
+  return std::string();
+}
+#endif
+
 }  // namespace browser_sync
