@@ -8,9 +8,13 @@
 #include "base/logging.h"
 #include "base/tracked.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/sync/engine/syncapi.h"
+#include "chrome/browser/sync/api/sync_error.h"
 #include "chrome/browser/sync/glue/sync_backend_host.h"
 #include "chrome/browser/sync/glue/theme_util.h"
+#include "chrome/browser/sync/internal_api/read_node.h"
+#include "chrome/browser/sync/internal_api/read_transaction.h"
+#include "chrome/browser/sync/internal_api/write_node.h"
+#include "chrome/browser/sync/internal_api/write_transaction.h"
 #include "chrome/browser/sync/profile_sync_service.h"
 #include "chrome/browser/sync/protocol/theme_specifics.pb.h"
 
@@ -35,11 +39,11 @@ ThemeModelAssociator::ThemeModelAssociator(
 
 ThemeModelAssociator::~ThemeModelAssociator() {}
 
-bool ThemeModelAssociator::AssociateModels() {
+bool ThemeModelAssociator::AssociateModels(SyncError* error) {
   sync_api::WriteTransaction trans(FROM_HERE, sync_service_->GetUserShare());
   sync_api::ReadNode root(&trans);
   if (!root.InitByTagLookup(kThemesTag)) {
-    LOG(ERROR) << kNoThemesFolderError;
+    error->Reset(FROM_HERE, kNoThemesFolderError, model_type());
     return false;
   }
 
@@ -62,7 +66,9 @@ bool ThemeModelAssociator::AssociateModels() {
     sync_api::WriteNode node(&trans);
     if (!node.InitUniqueByCreation(syncable::THEMES, root,
                                    kCurrentThemeClientTag)) {
-      LOG(ERROR) << "Could not create current theme node.";
+      error->Reset(FROM_HERE,
+                   "Could not create current theme node.",
+                   model_type());
       return false;
     }
     node.SetIsFolder(false);
@@ -74,7 +80,7 @@ bool ThemeModelAssociator::AssociateModels() {
   return true;
 }
 
-bool ThemeModelAssociator::DisassociateModels() {
+bool ThemeModelAssociator::DisassociateModels(SyncError* error) {
   // We don't maintain any association state, so nothing to do.
   return true;
 }
