@@ -27,8 +27,8 @@ class ThemeModelAssociator : public AssociatorInterface {
   static syncable::ModelType model_type() { return syncable::THEMES; }
 
   // AssociatorInterface implementation.
-  virtual bool AssociateModels();
-  virtual bool DisassociateModels();
+  virtual bool AssociateModels(SyncError* error);
+  virtual bool DisassociateModels(SyncError* error);
   virtual bool SyncModelHasUserCreatedNodes(bool* has_nodes);
   virtual void AbortAssociation() {
     // No implementation needed, this associator runs on the main
