@@ -7,10 +7,12 @@
 #include "base/metrics/histogram.h"
 #include "base/task.h"
 #include "chrome/browser/history/history.h"
+#include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/sync/profile_sync_factory.h"
 #include "chrome/browser/sync/profile_sync_service.h"
 #include "chrome/common/chrome_notification_types.h"
+#include "chrome/common/pref_names.h"
 #include "content/browser/browser_thread.h"
 #include "content/common/notification_service.h"
 
@@ -44,6 +46,8 @@ TypedUrlDataTypeController::TypedUrlDataTypeController(
     : NonFrontendDataTypeController(profile_sync_factory,
                                  profile),
       backend_(NULL) {
+  pref_registrar_.Init(profile->GetPrefs());
+  pref_registrar_.Add(prefs::kSavingBrowserHistoryDisabled, this);
 }
 
 TypedUrlDataTypeController::~TypedUrlDataTypeController() {
@@ -64,27 +68,21 @@ void TypedUrlDataTypeController::RunOnHistoryThread(bool start,
   backend_ = NULL;
 }
 
-bool TypedUrlDataTypeController::StartModels() {
-  DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  DCHECK_EQ(state(), MODEL_STARTING);
-  HistoryService* history = profile()->GetHistoryServiceWithoutCreating();
-  if (history) {
-    history_service_ = history;
-    return true;
-  } else {
-    notification_registrar_.Add(this, chrome::NOTIFICATION_HISTORY_LOADED,
-                                NotificationService::AllSources());
-    return false;
-  }
-}
-
 bool TypedUrlDataTypeController::StartAssociationAsync() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
   DCHECK_EQ(state(), ASSOCIATING);
-  DCHECK(history_service_.get());
-  history_service_->ScheduleDBTask(new ControlTask(this, true),
-                                   &cancelable_consumer_);
-  return true;
+  HistoryService* history = profile()->GetHistoryService(
+      Profile::IMPLICIT_ACCESS);
+  if (history) {
+    history_service_ = history;
+    history_service_->ScheduleDBTask(new ControlTask(this, true),
+                                     &cancelable_consumer_);
+    return true;
+  } else {
+    // History must be disabled - don't start.
+    LOG(WARNING) << "Cannot access history service - disabling typed url sync";
+    return false;
+  }
 }
 
 void TypedUrlDataTypeController::CreateSyncComponents() {
@@ -104,14 +102,25 @@ void TypedUrlDataTypeController::Observe(int type,
                                          const NotificationSource& source,
                                          const NotificationDetails& details) {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  DCHECK_EQ(state(), MODEL_STARTING);
-  notification_registrar_.Remove(this,
-                                 chrome::NOTIFICATION_HISTORY_LOADED,
-                                 NotificationService::AllSources());
-  history_service_ = profile()->GetHistoryServiceWithoutCreating();
-  DCHECK(history_service_.get());
-  set_state(ASSOCIATING);
-  StopAssociationAsync();
+  switch (type) {
+    case chrome::NOTIFICATION_PREF_CHANGED:
+      DCHECK(*Details<std::string>(details).ptr() ==
+             prefs::kSavingBrowserHistoryDisabled);
+      if (profile()->GetPrefs()->GetBoolean(
+              prefs::kSavingBrowserHistoryDisabled)) {
+        // We've turned off history persistence, so if we are running,
+        // generate an unrecoverable error. This can be fixed by restarting
+        // Chrome (on restart, typed urls will not be a registered type).
+        if (state() != NOT_RUNNING && state() != STOPPING) {
+          profile_sync_service()->OnUnrecoverableError(
+              FROM_HERE, "History saving is now disabled by policy.");
+        }
+      }
+      break;
+    default:
+      NOTREACHED();
+      break;
+  }
 }
 
 void TypedUrlDataTypeController::StopModels() {
