@@ -83,13 +83,15 @@ void InvalidationNotifier::UpdateCredentials(
 }
 
 void InvalidationNotifier::UpdateEnabledTypes(
-    const syncable::ModelTypeSet& types) {
+    const syncable::ModelTypeSet& enabled_types) {
   DCHECK(non_thread_safe_.CalledOnValidThread());
-  invalidation_client_.RegisterTypes(types);
+  invalidation_client_.RegisterTypes(enabled_types);
 }
 
-void InvalidationNotifier::SendNotification() {
+void InvalidationNotifier::SendNotification(
+    const syncable::ModelTypeSet& changed_types) {
   DCHECK(non_thread_safe_.CalledOnValidThread());
+  // Do nothing.
 }
 
 void InvalidationNotifier::OnConnect(
