@@ -95,20 +95,21 @@ SyncSessionSnapshot SyncSession::TakeSnapshot() const {
   bool is_share_useable = true;
   syncable::ModelTypeBitSet initial_sync_ended;
   std::string download_progress_markers[syncable::MODEL_TYPE_COUNT];
-  for (int i = 0; i < syncable::MODEL_TYPE_COUNT; ++i) {
+  for (int i = syncable::FIRST_REAL_MODEL_TYPE;
+       i < syncable::MODEL_TYPE_COUNT; ++i) {
     syncable::ModelType type(syncable::ModelTypeFromInt(i));
     if (routing_info_.count(type) != 0) {
       if (dir->initial_sync_ended_for_type(type))
         initial_sync_ended.set(type);
       else
         is_share_useable = false;
-      dir->GetDownloadProgressAsString(type, &download_progress_markers[i]);
     }
+    dir->GetDownloadProgressAsString(type, &download_progress_markers[i]);
   }
 
   return SyncSessionSnapshot(
       status_controller_->syncer_status(),
-      status_controller_->error_counters(),
+      status_controller_->error(),
       status_controller_->num_server_changes_remaining(),
       is_share_useable,
       initial_sync_ended,
@@ -120,7 +121,8 @@ SyncSessionSnapshot SyncSession::TakeSnapshot() const {
       status_controller_->TotalNumConflictingItems(),
       status_controller_->did_commit_items(),
       source_,
-      dir->GetEntriesCount());
+      dir->GetEntriesCount(),
+      status_controller_->sync_start_time());
 }
 
 SyncSourceInfo SyncSession::TestAndSetSource() {
