@@ -31,7 +31,6 @@
 #include "chrome/browser/sync/syncable/syncable_id.h"
 #include "chrome/browser/sync/syncable/model_type.h"
 #include "chrome/browser/sync/util/dbgq.h"
-#include "chrome/common/deprecated/event_sys.h"
 
 struct PurgeInfo;
 
@@ -706,8 +705,6 @@ class Directory {
                            TakeSnapshotGetsMetahandlesToPurge);
 
  public:
-  class EventListenerHookup;
-
   // Various data that the Directory::Kernel we are backing (persisting data
   // for) needs saved across runs of the application.
   struct PersistedKernelInfo {
@@ -853,14 +850,7 @@ class Directory {
 
   template <class T> void TestAndSet(T* kernel_data, const T* data_to_set);
 
-  struct DirectoryEventTraits {
-    typedef DirectoryEvent EventType;
-    static inline bool IsChannelShutdownEvent(const DirectoryEvent& event) {
-      return DIRECTORY_DESTROYED == event;
-    }
-  };
  public:
-  typedef EventChannel<DirectoryEventTraits, base::Lock> Channel;
   typedef std::vector<int64> ChildHandles;
 
   // Returns the child meta handles for given parent id.  Clears
@@ -915,11 +905,6 @@ class Directory {
   void GetUnappliedUpdateMetaHandles(BaseTransaction* trans,
                                      UnappliedUpdateMetaHandles* result);
 
-  // Get the channel for post save notification, used by the syncer.
-  inline Channel* channel() const {
-    return kernel_->channel;
-  }
-
   // Checks tree metadata consistency.
   // If full_scan is false, the function will avoid pulling any entries from the
   // db and scan entries currently in ram.
@@ -1049,9 +1034,6 @@ class Directory {
     // them in here so that SaveChanges can persist their permanent deletion.
     MetahandleSet* const metahandles_to_purge;
 
-    // TODO(ncarter): Figure out what the hell this is, and comment it.
-    Channel* const channel;
-
     KernelShareInfoStatus info_status;
 
     // These 3 members are backed in the share_info table, and
