@@ -29,7 +29,7 @@ class ChromeInterstitialPage : public InterstitialPage {
 
  protected:
   // NotificationObserver method:
-  virtual void Observe(NotificationType type,
+  virtual void Observe(int type,
                        const NotificationSource& source,
                        const NotificationDetails& details);
 
