@@ -61,10 +61,11 @@ COMPILE_ASSERT_MATCHING_ENUM(DragOperationEvery);
 - (void)viewDidBecomeFirstResponder:(NSNotification*)notification;
 @end
 
-// static
-TabContentsView* TabContentsView::Create(TabContents* tab_contents) {
+namespace tab_contents_view_mac {
+TabContentsView* CreateTabContentsView(TabContents* tab_contents) {
   return new TabContentsViewMac(tab_contents);
 }
+}
 
 TabContentsViewMac::TabContentsViewMac(TabContents* tab_contents)
     : tab_contents_(tab_contents),
@@ -233,10 +234,6 @@ void TabContentsViewMac::RestoreFocus() {
   focus_tracker_.reset(nil);
 }
 
-void TabContentsViewMac::UpdatePreferredSize(const gfx::Size& pref_size) {
-  preferred_width_ = pref_size.width();
-}
-
 bool TabContentsViewMac::IsDoingDrag() const {
   return false;
 }
