@@ -241,9 +241,8 @@ class TabStripModel : public NotificationObserver {
   // 1)
   void MoveSelectedTabsTo(int index);
 
-  // Returns the currently selected TabContents, or NULL if there is none.
-  // TODO(sky): rename to GetActiveTabContents.
-  TabContentsWrapper* GetSelectedTabContents() const;
+  // Returns the currently active TabContents, or NULL if there is none.
+  TabContentsWrapper* GetActiveTabContents() const;
 
   // Returns the TabContentsWrapper at the specified index, or NULL if there is
   // none.
@@ -462,7 +461,7 @@ class TabStripModel : public NotificationObserver {
   bool WillContextMenuPin(int index);
 
   // Overridden from notificationObserver:
-  virtual void Observe(NotificationType type,
+  virtual void Observe(int type,
                        const NotificationSource& source,
                        const NotificationDetails& details);
 
@@ -515,15 +514,22 @@ class TabStripModel : public NotificationObserver {
 
   TabContentsWrapper* GetContentsAt(int index) const;
 
-  // If the TabContentsWrapper at |to_index| differs from |old_contents|
-  // notifies observers.
-  void NotifyTabSelectedIfChanged(TabContentsWrapper* old_contents,
-                                  int to_index,
-                                  bool user_gesture);
+  // Notifies the observers if the active tab has changed. If |old_contents| is
+  // non-null a TabDeactivated notification is sent right before sending
+  // ActiveTabChanged notification.
+  void NotifyIfActiveTabChanged(TabContentsWrapper* old_contents,
+                                bool user_gesture);
 
-  // Notifies the observers the active tab changed. |old_active_index| gives
-  // the old active index.
-  void NotifyActiveTabChanged(int old_active_index);
+  // Notifies the observers if the active tab or the tab selection has changed.
+  // If |old_contents| is non-null a TabDeactivated notification is sent right
+  // before sending ActiveTabChanged notification. |old_model| is a snapshot of
+  // |selection_model_| before the change.
+  // Note: This function might end up sending 0 to 3 notifications in the
+  // following order: TabDeactivated, ActiveTabChanged, TabSelectionChanged.
+  void NotifyIfActiveOrSelectionChanged(
+      TabContentsWrapper* old_contents,
+      bool user_gesture,
+      const TabStripSelectionModel& old_model);
 
   // Returns the number of New Tab tabs in the TabStripModel.
   int GetNewTabCount() const;
@@ -616,7 +622,25 @@ class TabStripModel : public NotificationObserver {
 
   // The TabContents data currently hosted within this TabStripModel.
   typedef std::vector<TabContentsData*> TabContentsDataVector;
+
+  // TODO(eroman): Temporary for investigating 93747.
+  class Padding {
+   public:
+    Padding();
+    void CheckValid() const;
+
+   private:
+    char bytes_[32];
+  };
+
+  // Surround contents_data_ with some extra bytes to try and catch external
+  // mutation of this memory. This is temporary for debugging 93747.
+  Padding padding1_;
   TabContentsDataVector contents_data_;
+  Padding padding2_;
+
+  // This is temporary for debugging 93747.
+  void CheckIsAliveAndWell() const;
 
   // A profile associated with this TabStripModel, used when creating new Tabs.
   Profile* profile_;
@@ -637,6 +661,8 @@ class TabStripModel : public NotificationObserver {
 
   TabStripSelectionModel selection_model_;
 
+  uint32 magic_id_;
+
   DISALLOW_IMPLICIT_CONSTRUCTORS(TabStripModel);
 };
 
