@@ -27,9 +27,9 @@ BookmarkTabHelper::BookmarkTabHelper(TabContentsWrapper* tab_contents)
       bookmark_drag_(NULL) {
   // Register for notifications about URL starredness changing on any profile.
   registrar_.Add(this, chrome::NOTIFICATION_URLS_STARRED,
-                 NotificationService::AllSources());
+                 NotificationService::AllBrowserContextsAndSources());
   registrar_.Add(this, chrome::NOTIFICATION_BOOKMARK_MODEL_LOADED,
-                 NotificationService::AllSources());
+                 NotificationService::AllBrowserContextsAndSources());
 }
 
 BookmarkTabHelper::~BookmarkTabHelper() {
@@ -96,7 +96,9 @@ BookmarkTabHelper::BookmarkDrag*
 }
 
 void BookmarkTabHelper::UpdateStarredStateForCurrentURL() {
-  BookmarkModel* model = tab_contents()->profile()->GetBookmarkModel();
+  Profile* profile =
+      Profile::FromBrowserContext(tab_contents()->browser_context());
+  BookmarkModel* model = profile->GetBookmarkModel();
   const bool old_state = is_starred_;
   is_starred_ = (model && model->IsBookmarked(tab_contents()->GetURL()));
 
