@@ -83,7 +83,7 @@ void BookmarkMenuBridge::UpdateMenuInternal(NSMenu* bookmark_menu,
   // fill it in.
   if (!model->other_node()->empty()) {
     NSString* other_items_title =
-        l10n_util::GetNSString(IDS_BOOMARK_BAR_OTHER_FOLDER_NAME);
+        l10n_util::GetNSString(IDS_BOOKMARK_BAR_OTHER_FOLDER_NAME);
     [bookmark_menu addItem:[NSMenuItem separatorItem]];
     AddNodeAsSubmenu(bookmark_menu,
                      model->other_node(),
@@ -242,13 +242,13 @@ void BookmarkMenuBridge::AddNodeToMenu(const BookmarkNode* node, NSMenu* menu,
     [menu addItem:[NSMenuItem separatorItem]];
     bool enabled = child_count != 0;
     AddItemToMenu(IDC_BOOKMARK_BAR_OPEN_ALL,
-                  IDS_BOOMARK_BAR_OPEN_ALL,
+                  IDS_BOOKMARK_BAR_OPEN_ALL,
                   node, menu, enabled);
     AddItemToMenu(IDC_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW,
-                  IDS_BOOMARK_BAR_OPEN_ALL_NEW_WINDOW,
+                  IDS_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW,
                   node, menu, enabled);
     AddItemToMenu(IDC_BOOKMARK_BAR_OPEN_ALL_INCOGNITO,
-                  IDS_BOOMARK_BAR_OPEN_INCOGNITO,
+                  IDS_BOOKMARK_BAR_OPEN_ALL_INCOGNITO,
                   node, menu, enabled);
   }
 }
@@ -258,7 +258,7 @@ void BookmarkMenuBridge::AddItemToMenu(int command_id,
                                        const BookmarkNode* node,
                                        NSMenu* menu,
                                        bool enabled) {
-  NSString* title = l10n_util::GetNSString(message_id);
+  NSString* title = l10n_util::GetNSStringWithFixup(message_id);
   SEL action;
   if (!enabled) {
     // A nil action makes a menu item appear disabled. NSMenuItem setEnabled
