@@ -11,7 +11,6 @@
 #include "chrome/app/chrome_command_ids.h"
 #include "chrome/browser/bookmarks/bookmark_utils.h"
 #include "chrome/browser/download/download_shelf.h"
-#include "chrome/browser/global_keyboard_shortcuts_mac.h"
 #include "chrome/browser/page_info_window.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
@@ -21,10 +20,11 @@
 #include "chrome/browser/ui/browser_list.h"
 #import "chrome/browser/ui/cocoa/browser/edit_search_engine_cocoa_controller.h"
 #import "chrome/browser/ui/cocoa/browser_window_controller.h"
-#import "chrome/browser/ui/cocoa/bug_report_window_controller.h"
+#import "chrome/browser/ui/cocoa/browser_window_utils.h"
 #import "chrome/browser/ui/cocoa/chrome_event_processing_window.h"
 #import "chrome/browser/ui/cocoa/content_settings/collected_cookies_mac.h"
 #import "chrome/browser/ui/cocoa/download/download_shelf_controller.h"
+#include "chrome/browser/ui/cocoa/find_bar/find_bar_bridge.h"
 #import "chrome/browser/ui/cocoa/html_dialog_window_controller.h"
 #import "chrome/browser/ui/cocoa/location_bar/location_bar_view_mac.h"
 #import "chrome/browser/ui/cocoa/nsmenuitem_additions.h"
@@ -45,18 +45,40 @@
 #include "ui/base/l10n/l10n_util_mac.h"
 #include "ui/gfx/rect.h"
 
+// Replicate specific 10.7 SDK declarations for building with prior SDKs.
+#if !defined(MAC_OS_X_VERSION_10_7) || \
+    MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_7
+
+enum {
+  NSWindowAnimationBehaviorDefault = 0,
+  NSWindowAnimationBehaviorNone = 2,
+  NSWindowAnimationBehaviorDocumentWindow = 3,
+  NSWindowAnimationBehaviorUtilityWindow = 4,
+  NSWindowAnimationBehaviorAlertPanel = 5
+};
+typedef NSInteger NSWindowAnimationBehavior;
+
+@interface NSWindow (LionSDKDeclarations)
+- (NSWindowAnimationBehavior)animationBehavior;
+- (void)setAnimationBehavior:(NSWindowAnimationBehavior)newAnimationBehavior;
+@end
+
+#endif  // MAC_OS_X_VERSION_10_7
+
 BrowserWindowCocoa::BrowserWindowCocoa(Browser* browser,
-                                       BrowserWindowController* controller,
-                                       NSWindow* window)
+                                       BrowserWindowController* controller)
   : browser_(browser),
     controller_(controller),
     confirm_close_factory_(browser) {
-  // This pref applies to all windows, so all must watch for it.
+  // Listen for bookmark bar visibility changes and set the initial state; we
+  // need to listen to all profiles because of normal profile/incognito issues.
   registrar_.Add(this,
                  chrome::NOTIFICATION_BOOKMARK_BAR_VISIBILITY_PREF_CHANGED,
-                 NotificationService::AllSources());
+                 NotificationService::AllBrowserContextsAndSources());
   registrar_.Add(this, chrome::NOTIFICATION_SIDEBAR_CHANGED,
-                 NotificationService::AllSources());
+                 Source<SidebarManager>(SidebarManager::GetInstance()));
+
+  initial_show_state_ = browser_->GetSavedWindowShowState();
 }
 
 BrowserWindowCocoa::~BrowserWindowCocoa() {
@@ -71,11 +93,37 @@ void BrowserWindowCocoa::Show() {
   // the previous browser instead if we don't explicitly set it here.
   BrowserList::SetLastActive(browser_);
 
+  bool is_session_restore = browser_->is_session_restore();
+  NSWindowAnimationBehavior saved_animation_behavior =
+      NSWindowAnimationBehaviorDefault;
+  bool did_save_animation_behavior = false;
+  // Turn off swishing when restoring windows.
+  if (is_session_restore &&
+      [window() respondsToSelector:@selector(animationBehavior)] &&
+      [window() respondsToSelector:@selector(setAnimationBehavior:)]) {
+    did_save_animation_behavior = true;
+    saved_animation_behavior = [window() animationBehavior];
+    [window() setAnimationBehavior:NSWindowAnimationBehaviorNone];
+  }
+
   [window() makeKeyAndOrderFront:controller_];
+
+  // When creating windows from nibs it is necessary to |makeKeyAndOrderFront:|
+  // prior to |orderOut:| then |miniaturize:| when restoring windows in the
+  // minimized state.
+  if (initial_show_state_ == ui::SHOW_STATE_MINIMIZED) {
+    [window() orderOut:controller_];
+    [window() miniaturize:controller_];
+  }
+  initial_show_state_ = ui::SHOW_STATE_DEFAULT;
+
+  // Restore window animation behavior.
+  if (did_save_animation_behavior)
+    [window() setAnimationBehavior:saved_animation_behavior];
 }
 
 void BrowserWindowCocoa::ShowInactive() {
-    [window() orderFront:controller_];
+  [window() orderFront:controller_];
 }
 
 void BrowserWindowCocoa::SetBounds(const gfx::Rect& bounds) {
@@ -157,24 +205,10 @@ void BrowserWindowCocoa::UpdateTitleBar() {
   NSString* newTitle =
       base::SysUTF16ToNSString(browser_->GetWindowTitleForCurrentTab());
 
-  // Work around Cocoa bug: if a window changes title during the tracking of the
-  // Window menu it doesn't display well and the constant re-sorting of the list
-  // makes it difficult for the user to pick the desired window. Delay window
-  // title updates until the default run-loop mode.
-
-  if (pending_window_title_.get())
-    [[NSRunLoop currentRunLoop]
-        cancelPerformSelector:@selector(setTitle:)
-                       target:window()
-                     argument:pending_window_title_.get()];
-
-  pending_window_title_.reset([newTitle copy]);
-  [[NSRunLoop currentRunLoop]
-      performSelector:@selector(setTitle:)
-               target:window()
-             argument:newTitle
-                order:0
-                modes:[NSArray arrayWithObject:NSDefaultRunLoopMode]];
+  pending_window_title_.reset(
+      [BrowserWindowUtils scheduleReplaceOldTitle:pending_window_title_.get()
+                                     withNewTitle:newTitle
+                                        forWindow:window()]);
 }
 
 void BrowserWindowCocoa::BookmarkBarStateChanged(
@@ -212,6 +246,10 @@ bool BrowserWindowCocoa::IsMaximized() const {
   return [window() isZoomed];
 }
 
+bool BrowserWindowCocoa::IsMinimized() const {
+  return [window() isMiniaturized];
+}
+
 void BrowserWindowCocoa::SetFullscreen(bool fullscreen) {
   [controller_ setFullscreen:fullscreen];
 }
@@ -278,10 +316,7 @@ void BrowserWindowCocoa::FocusChromeOSStatus() {
 }
 
 bool BrowserWindowCocoa::IsBookmarkBarVisible() const {
-  return (browser_->profile()->GetPrefs()->GetBoolean(
-              prefs::kShowBookmarkBar) &&
-          browser_->profile()->GetPrefs()->GetBoolean(
-              prefs::kEnableBookmarkBar));
+  return browser_->profile()->GetPrefs()->GetBoolean(prefs::kShowBookmarkBar);
 }
 
 bool BrowserWindowCocoa::IsBookmarkBarAnimating() const {
@@ -307,7 +342,7 @@ void BrowserWindowCocoa::ToggleBookmarkBar() {
 
 void BrowserWindowCocoa::AddFindBar(
     FindBarCocoaController* find_bar_cocoa_controller) {
-  return [controller_ addFindBar:find_bar_cocoa_controller];
+  [controller_ addFindBar:find_bar_cocoa_controller];
 }
 
 void BrowserWindowCocoa::ShowAboutChromeDialog() {
@@ -347,9 +382,10 @@ void BrowserWindowCocoa::ShowRepostFormWarningDialog(
   RepostFormWarningMac::Create(GetNativeHandle(), tab_contents);
 }
 
-void BrowserWindowCocoa::ShowCollectedCookiesDialog(TabContents* tab_contents) {
+void BrowserWindowCocoa::ShowCollectedCookiesDialog(
+    TabContentsWrapper* wrapper) {
   // Deletes itself on close.
-  new CollectedCookiesMac(GetNativeHandle(), tab_contents);
+  new CollectedCookiesMac(GetNativeHandle(), wrapper);
 }
 
 void BrowserWindowCocoa::ShowThemeInstallBubble() {
@@ -368,10 +404,12 @@ void BrowserWindowCocoa::ConfirmBrowserCloseWithPendingDownloads() {
           true));
 }
 
-void BrowserWindowCocoa::ShowHTMLDialog(HtmlDialogUIDelegate* delegate,
-                                        gfx::NativeWindow parent_window) {
+gfx::NativeWindow BrowserWindowCocoa::ShowHTMLDialog(
+    HtmlDialogUIDelegate* delegate,
+    gfx::NativeWindow parent_window) {
   [HtmlDialogWindowController showHtmlDialog:delegate
                                      profile:browser_->profile()];
+  return NULL;
 }
 
 void BrowserWindowCocoa::UserChangedTheme() {
@@ -400,130 +438,27 @@ void BrowserWindowCocoa::ShowAppMenu() {
 
 bool BrowserWindowCocoa::PreHandleKeyboardEvent(
     const NativeWebKeyboardEvent& event, bool* is_keyboard_shortcut) {
-  if (event.skip_in_browser || event.type == NativeWebKeyboardEvent::Char)
+  if (![BrowserWindowUtils shouldHandleKeyboardEvent:event])
     return false;
 
-  DCHECK(event.os_event != NULL);
-  int id = GetCommandId(event);
+  int id = [BrowserWindowUtils getCommandId:event];
   if (id == -1)
     return false;
 
-  if (browser_->IsReservedCommandOrKey(id, event))
-    return HandleKeyboardEventInternal(event.os_event);
+  if (browser_->IsReservedCommandOrKey(id, event)) {
+      return [BrowserWindowUtils handleKeyboardEvent:event.os_event
+                                            inWindow:window()];
+  }
 
-  DCHECK(is_keyboard_shortcut != NULL);
+  DCHECK(is_keyboard_shortcut);
   *is_keyboard_shortcut = true;
-
   return false;
 }
 
 void BrowserWindowCocoa::HandleKeyboardEvent(
     const NativeWebKeyboardEvent& event) {
-  if (event.skip_in_browser || event.type == NativeWebKeyboardEvent::Char)
-    return;
-
-  DCHECK(event.os_event != NULL);
-  HandleKeyboardEventInternal(event.os_event);
-}
-
-@interface MenuWalker : NSObject
-+ (NSMenuItem*)itemForKeyEquivalent:(NSEvent*)key
-                               menu:(NSMenu*)menu;
-@end
-
-@implementation MenuWalker
-+ (NSMenuItem*)itemForKeyEquivalent:(NSEvent*)key
-                               menu:(NSMenu*)menu {
-  NSMenuItem* result = nil;
-
-  for (NSMenuItem *item in [menu itemArray]) {
-    NSMenu* submenu = [item submenu];
-    if (submenu) {
-      if (submenu != [NSApp servicesMenu])
-        result = [self itemForKeyEquivalent:key
-                                       menu:submenu];
-    } else if ([item cr_firesForKeyEventIfEnabled:key]) {
-      result = item;
-    }
-
-    if (result)
-      break;
-  }
-
-  return result;
-}
-@end
-
-int BrowserWindowCocoa::GetCommandId(const NativeWebKeyboardEvent& event) {
-  if ([event.os_event type] != NSKeyDown)
-    return -1;
-
-  // Look in menu.
-  NSMenuItem* item = [MenuWalker itemForKeyEquivalent:event.os_event
-                                                 menu:[NSApp mainMenu]];
-
-  if (item && [item action] == @selector(commandDispatch:) && [item tag] > 0)
-    return [item tag];
-
-  // "Close window" doesn't use the |commandDispatch:| mechanism. Menu items
-  // that do not correspond to IDC_ constants need no special treatment however,
-  // as they can't be blacklisted in |Browser::IsReservedCommandOrKey()| anyhow.
-  if (item && [item action] == @selector(performClose:))
-    return IDC_CLOSE_WINDOW;
-
-  // "Exit" doesn't use the |commandDispatch:| mechanism either.
-  if (item && [item action] == @selector(terminate:))
-    return IDC_EXIT;
-
-  // Look in secondary keyboard shortcuts.
-  NSUInteger modifiers = [event.os_event modifierFlags];
-  const bool cmdKey = (modifiers & NSCommandKeyMask) != 0;
-  const bool shiftKey = (modifiers & NSShiftKeyMask) != 0;
-  const bool cntrlKey = (modifiers & NSControlKeyMask) != 0;
-  const bool optKey = (modifiers & NSAlternateKeyMask) != 0;
-  const int keyCode = [event.os_event keyCode];
-  const unichar keyChar = KeyCharacterForEvent(event.os_event);
-
-  int cmdNum = CommandForWindowKeyboardShortcut(
-      cmdKey, shiftKey, cntrlKey, optKey, keyCode, keyChar);
-  if (cmdNum != -1)
-    return cmdNum;
-
-  cmdNum = CommandForBrowserKeyboardShortcut(
-      cmdKey, shiftKey, cntrlKey, optKey, keyCode, keyChar);
-  if (cmdNum != -1)
-    return cmdNum;
-
-  return -1;
-}
-
-bool BrowserWindowCocoa::HandleKeyboardEventInternal(NSEvent* event) {
-  ChromeEventProcessingWindow* event_window =
-      static_cast<ChromeEventProcessingWindow*>(window());
-  DCHECK([event_window isKindOfClass:[ChromeEventProcessingWindow class]]);
-
-  // Do not fire shortcuts on key up.
-  if ([event type] == NSKeyDown) {
-    // Send the event to the menu before sending it to the browser/window
-    // shortcut handling, so that if a user configures cmd-left to mean
-    // "previous tab", it takes precedence over the built-in "history back"
-    // binding. Other than that, the |-redispatchKeyEvent:| call would take care
-    // of invoking the original menu item shortcut as well.
-
-    if ([[NSApp mainMenu] performKeyEquivalent:event])
-      return true;
-
-    if ([event_window handleExtraBrowserKeyboardShortcut:event])
-      return true;
-
-    if ([event_window handleExtraWindowKeyboardShortcut:event])
-      return true;
-
-    if ([event_window handleDelayedWindowKeyboardShortcut:event])
-      return true;
-  }
-
-  return [event_window redispatchKeyEvent:event];
+  if ([BrowserWindowUtils shouldHandleKeyboardEvent:event])
+    [BrowserWindowUtils handleKeyboardEvent:event.os_event inWindow:window()];
 }
 
 void BrowserWindowCocoa::ShowCreateWebAppShortcutsDialog(
@@ -593,6 +528,16 @@ WindowOpenDisposition BrowserWindowCocoa::GetDispositionForPopupBounds(
   return NEW_POPUP;
 }
 
+FindBar* BrowserWindowCocoa::CreateFindBar() {
+  // We could push the AddFindBar() call into the FindBarBridge
+  // constructor or the FindBarCocoaController init, but that makes
+  // unit testing difficult, since we would also require a
+  // BrowserWindow object.
+  FindBarBridge* bridge = new FindBarBridge();
+  AddFindBar(bridge->find_bar_cocoa_controller());
+  return bridge;
+}
+
 void BrowserWindowCocoa::Observe(int type,
                                  const NotificationSource& source,
                                  const NotificationDetails& details) {
@@ -600,7 +545,8 @@ void BrowserWindowCocoa::Observe(int type,
     // Only the key window gets a direct toggle from the menu.
     // Other windows hear about it from the notification.
     case chrome::NOTIFICATION_BOOKMARK_BAR_VISIBILITY_PREF_CHANGED:
-      [controller_ updateBookmarkBarVisibilityWithAnimation:YES];
+      if (browser_->profile()->IsSameProfile(Source<Profile>(source).ptr()))
+        [controller_ updateBookmarkBarVisibilityWithAnimation:YES];
       break;
     case chrome::NOTIFICATION_SIDEBAR_CHANGED:
       UpdateSidebarForContents(
