@@ -28,9 +28,6 @@
 // Saves the window's position in the local state preferences.
 - (void)saveWindowPositionIfNeeded;
 
-// Saves the window's position to the given pref service.
-- (void)saveWindowPositionToPrefs:(PrefService*)prefs;
-
 // We need to adjust where sheets come out of the window, as by default they
 // erupt from the omnibox, which is rather weird.
 - (NSRect)window:(NSWindow*)window
@@ -46,6 +43,10 @@
 // call even when not in presentation mode.
 - (CGFloat)floatingBarHeight;
 
+// Shows the informational "how to exit fullscreen" bubble.
+- (void)showFullscreenExitBubbleIfNecessary;
+- (void)destroyFullscreenExitBubbleIfNecessary;
+
 // Lays out the presentation mode toggle button at the top right corner of the
 // overlay.  Creates the button if needed, and removes it if it is not needed.
 // This method is safe to call on all OS versions.
