@@ -2,24 +2,26 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "base/mac/mac_util.h"
 #include "base/memory/scoped_nsobject.h"
 #include "base/memory/scoped_ptr.h"
 #include "chrome/app/chrome_command_ids.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/sync/sync_ui_util.h"
+#include "chrome/browser/ui/browser_list.h"
 #include "chrome/browser/ui/browser_window.h"
 #include "chrome/browser/ui/cocoa/browser_test_helper.h"
 #include "chrome/browser/ui/cocoa/browser_window_controller.h"
 #include "chrome/browser/ui/cocoa/cocoa_test_helper.h"
 #include "chrome/browser/ui/cocoa/find_bar/find_bar_bridge.h"
 #include "chrome/common/pref_names.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/testing_profile.h"
 #include "grit/generated_resources.h"
 #include "ui/base/l10n/l10n_util_mac.h"
 
 @interface BrowserWindowController (JustForTesting)
 // Already defined in BWC.
-- (void)saveWindowPositionToPrefs:(PrefService*)prefs;
+- (void)saveWindowPositionIfNeeded;
 - (void)layoutSubviews;
 @end
 
@@ -92,7 +94,8 @@ TEST_F(BrowserWindowControllerTest, TestSaveWindowPosition) {
 
   // Ask the window to save its position, then check that a preference
   // exists.
-  [controller_ saveWindowPositionToPrefs:prefs];
+  BrowserList::SetLastActive(browser_helper_.browser());
+  [controller_ saveWindowPositionIfNeeded];
   browser_window_placement =
       prefs->GetDictionary(prefs::kBrowserWindowPlacement);
   ASSERT_TRUE(browser_window_placement);
@@ -652,6 +655,13 @@ class BrowserWindowFullScreenControllerTest : public CocoaTest {
  public:
   virtual void SetUp() {
     CocoaTest::SetUp();
+
+    // This test case crashes when run on Lion. Fail early.
+    if (base::mac::IsOSLionOrLater()) {
+      controller_ = nil;  // Need to make sure this isn't uninitialized memory.
+      FAIL() << "This test crashes on Lion; http://crbug.com/93925";
+    }
+
     Browser* browser = browser_helper_.browser();
     controller_ =
         [[BrowserWindowControllerFakeFullscreen alloc] initWithBrowser:browser
@@ -680,7 +690,7 @@ static bool IsFrontWindow(NSWindow *window) {
          [[frontmostWindow parentWindow] isEqual:window];
 }
 
-TEST_F(BrowserWindowFullScreenControllerTest, TestFullscreen) {
+TEST_F(BrowserWindowFullScreenControllerTest, TestFullscreenNotLion) {
   EXPECT_FALSE([controller_ isFullscreen]);
   [controller_ setFullscreen:YES];
   EXPECT_TRUE([controller_ isFullscreen]);
@@ -692,7 +702,7 @@ TEST_F(BrowserWindowFullScreenControllerTest, TestFullscreen) {
 // problem (such as a modal dialog up).  This tests is a very useful canary, so
 // please do not mark it as flaky without first verifying that there are no bot
 // problems.
-TEST_F(BrowserWindowFullScreenControllerTest, TestActivate) {
+TEST_F(BrowserWindowFullScreenControllerTest, TestActivateNotLion) {
   EXPECT_FALSE([controller_ isFullscreen]);
 
   [controller_ activate];
