@@ -13,7 +13,7 @@
 #include "chrome/browser/ui/content_settings/content_setting_bubble_model.h"
 #include "chrome/browser/ui/tab_contents/test_tab_contents_wrapper.h"
 #include "chrome/common/content_settings_types.h"
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/testing_profile.h"
 #include "content/browser/browser_thread.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -55,8 +55,12 @@ ContentSettingBubbleControllerTest::~ContentSettingBubbleControllerTest() {
 // Check that the bubble doesn't crash or leak for any settings type
 TEST_F(ContentSettingBubbleControllerTest, Init) {
   for (int i = 0; i < CONTENT_SETTINGS_NUM_TYPES; ++i) {
-    if (i == CONTENT_SETTINGS_TYPE_NOTIFICATIONS)
-      continue;  // Notifications have no bubble.
+    if (i == CONTENT_SETTINGS_TYPE_NOTIFICATIONS ||
+        i == CONTENT_SETTINGS_TYPE_INTENTS ||
+        i == CONTENT_SETTINGS_TYPE_AUTO_SELECT_CERTIFICATE) {
+      // Notifications, web intents and auto select certificate have no bubble.
+      continue;
+    }
 
     ContentSettingsType settingsType = static_cast<ContentSettingsType>(i);
 
