@@ -6,10 +6,10 @@
 
 #include "base/callback.h"
 #include "chrome/browser/browser_process.h"
-#include "chrome/browser/download/download_item.h"
 #include "chrome/browser/download/download_item_model.h"
 #import "chrome/browser/ui/cocoa/download/download_item_controller.h"
 #include "chrome/browser/ui/cocoa/download/download_util_mac.h"
+#include "content/browser/download/download_item.h"
 #include "ui/gfx/image/image.h"
 
 // DownloadItemMac -------------------------------------------------------------
