@@ -22,8 +22,7 @@ class DownloadItemMac;
 
 class DownloadShelfMac : public DownloadShelf {
  public:
-  explicit DownloadShelfMac(Browser* browser,
-                            DownloadShelfController* controller);
+  DownloadShelfMac(Browser* browser, DownloadShelfController* controller);
 
   // DownloadShelf implementation.
   virtual void AddDownload(BaseDownloadItemModel* download_model);
