@@ -7,6 +7,8 @@
 #include "base/basictypes.h"
 #include "base/logging.h"
 #include "base/sys_string_conversions.h"
+#include "chrome/browser/profiles/profile.h"
+#include "content/browser/plugin_service.h"
 #include "content/common/url_constants.h"
 #include "googleurl/src/gurl.h"
 #include "net/base/mime_util.h"
@@ -65,7 +67,7 @@ GURL GetFileURLFromDropData(id<NSDraggingInfo> info) {
   return GURL();
 }
 
-static BOOL IsSupportedFileURL(const GURL& url) {
+static BOOL IsSupportedFileURL(Profile* profile, const GURL& url) {
   FilePath full_path;
   net::FileURLToFilePath(url, &full_path);
 
@@ -79,22 +81,25 @@ static BOOL IsSupportedFileURL(const GURL& url) {
     return YES;
 
   // Check whether there is a plugin that supports the mime type. (e.g. PDF)
-  webkit::npapi::PluginList* list = webkit::npapi::PluginList::Singleton();
-  webkit::npapi::WebPluginInfo info;
-  if (!list->stale() &&
-      list->GetPluginInfo(GURL(), mime_type, false, &info, NULL)) {
-    return webkit::npapi::IsPluginEnabled(info);
-  }
-
-  return NO;
+  // TODO(bauerb): This possibly uses stale information, but it's guaranteed not
+  // to do disk access.
+  bool allow_wildcard = false;
+  bool stale = false;
+  webkit::WebPluginInfo plugin;
+  return PluginService::GetInstance()->GetPluginInfo(
+      -1,                // process ID
+      MSG_ROUTING_NONE,  // routing ID
+      profile->GetResourceContext(),
+      url, GURL(), mime_type, allow_wildcard,
+      &stale, &plugin, NULL);
 }
 
-BOOL IsUnsupportedDropData(id<NSDraggingInfo> info) {
+BOOL IsUnsupportedDropData(Profile* profile, id<NSDraggingInfo> info) {
   GURL url = GetFileURLFromDropData(info);
   if (!url.is_empty()) {
     // If dragging a file, only allow dropping supported file types (that the
     // web view can display).
-    return !IsSupportedFileURL(url);
+    return !IsSupportedFileURL(profile, url);
   }
   return NO;
 }
