@@ -12,6 +12,7 @@
 #include "base/message_loop.h"
 #include "base/task.h"
 #include "chrome/browser/first_run/first_run.h"
+#include "chrome/browser/infobars/infobar_tab_helper.h"
 #import "chrome/browser/mac/keystone_glue.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
@@ -69,7 +70,7 @@ class KeystonePromotionInfoBarDelegate : public ConfirmInfoBarDelegate {
 KeystonePromotionInfoBarDelegate::KeystonePromotionInfoBarDelegate(
     TabContents* tab_contents)
     : ConfirmInfoBarDelegate(tab_contents),
-      profile_(tab_contents->profile()),
+      profile_(Profile::FromBrowserContext(tab_contents->browser_context())),
       can_expire_(false),
       ALLOW_THIS_IN_INITIALIZER_LIST(method_factory_(this)) {
   const int kCanExpireOnNavigationAfterMilliseconds = 8 * 1000;
@@ -190,9 +191,9 @@ bool KeystonePromotionInfoBarDelegate::Cancel() {
 
       // Only show if no other info bars are showing, because that's how the
       // default browser info bar works.
-      if (wrapper && wrapper->infobar_count() == 0) {
-        wrapper->AddInfoBar(new KeystonePromotionInfoBarDelegate(
-            wrapper->tab_contents()));
+      if (wrapper && wrapper->infobar_tab_helper()->infobar_count() == 0) {
+        wrapper->infobar_tab_helper()->AddInfoBar(
+            new KeystonePromotionInfoBarDelegate(wrapper->tab_contents()));
       }
     }
   }
