@@ -4,11 +4,14 @@
 
 #import "chrome/browser/ui/cocoa/page_info_bubble_controller.h"
 
+#include "base/compiler_specific.h"
 #include "base/message_loop.h"
 #include "base/sys_string_conversions.h"
 #include "base/task.h"
 #include "chrome/browser/certificate_viewer.h"
 #include "chrome/browser/google/google_util.h"
+#include "chrome/browser/page_info_model.h"
+#include "chrome/browser/page_info_model_observer.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/browser_list.h"
 #import "chrome/browser/ui/cocoa/browser_window_controller.h"
@@ -93,7 +96,7 @@ const CGFloat kTextWidth = kWindowWidth - (kImageSize + kImageSpacing +
     kFramePadding * 2);
 
 // Bridge that listens for change notifications from the model.
-class PageInfoModelBubbleBridge : public PageInfoModel::PageInfoModelObserver {
+class PageInfoModelBubbleBridge : public PageInfoModelObserver {
  public:
   PageInfoModelBubbleBridge()
       : controller_(nil),
@@ -101,7 +104,7 @@ class PageInfoModelBubbleBridge : public PageInfoModel::PageInfoModelObserver {
   }
 
   // PageInfoModelObserver implementation.
-  virtual void ModelChanged() {
+  virtual void OnPageInfoModelChanged() OVERRIDE {
     // Check to see if a layout has already been scheduled.
     if (!task_factory_.empty())
       return;
@@ -171,7 +174,7 @@ void ShowPageInfoBubble(gfx::NativeWindow parent,
 @synthesize certID = certID_;
 
 - (id)initWithPageInfoModel:(PageInfoModel*)model
-              modelObserver:(PageInfoModel::PageInfoModelObserver*)bridge
+              modelObserver:(PageInfoModelObserver*)bridge
                parentWindow:(NSWindow*)parentWindow {
   DCHECK(parentWindow);
 
@@ -289,6 +292,7 @@ void ShowPageInfoBubble(gfx::NativeWindow parent,
   scoped_nsobject<PageInfoContentView> contentView(
       [[PageInfoContentView alloc] initWithFrame:contentFrame]);
   [contentView setSubviews:subviews];
+  [contentView setAutoresizingMask:NSViewMinYMargin];
 
   NSRect windowFrame = NSMakeRect(0, 0, kWindowWidth, offset);
   windowFrame.size = [[[self window] contentView] convertSize:windowFrame.size
