@@ -8,26 +8,18 @@
 
 #import <Cocoa/Cocoa.h>
 
-#import "base/memory/scoped_nsobject.h"
+#import "chrome/browser/ui/cocoa/image_button_cell.h"
 #import "chrome/browser/ui/cocoa/toolbar/toolbar_button.h"
 
 // ToolbarButton subclass which defers certain state changes when the mouse
 // is hovering over it.
 
-@interface ReloadButton : ToolbarButton {
+@interface ReloadButton : ToolbarButton<ImageButton> {
  @private
-  // Tracks whether the mouse is hovering for purposes of not making
-  // unexpected state changes.
-  BOOL isMouseInside_;
-  scoped_nsobject<NSTrackingArea> trackingArea_;
-
   // Timer used when setting reload mode while the mouse is hovered.
-  scoped_nsobject<NSTimer> pendingReloadTimer_;
+  NSTimer* pendingReloadTimer_;
 }
 
-// Returns YES if the mouse is currently inside the bounds.
-- (BOOL)isMouseInside;
-
 // Update the tag, and the image and tooltip to match.  If |anInt|
 // matches the current tag, no action is taken.  |anInt| must be
 // either |IDC_STOP| or |IDC_RELOAD|.
@@ -43,9 +35,4 @@
 
 @end
 
-@interface ReloadButton (PrivateTestingMethods)
-+ (void)setPendingReloadTimeout:(NSTimeInterval)seconds;
-- (NSTrackingArea*)trackingArea;
-@end
-
 #endif  // CHROME_BROWSER_UI_COCOA_TOOLBAR_RELOAD_BUTTON_H_
