@@ -16,8 +16,8 @@
 #include "chrome/browser/ui/find_bar/find_tab_helper.h"
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
 #include "chrome/common/url_constants.h"
-#include "chrome/test/in_process_browser_test.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "chrome/test/base/ui_test_utils.h"
 #include "content/browser/renderer_host/render_view_host.h"
 #include "content/browser/tab_contents/tab_contents.h"
 #include "content/browser/tab_contents/tab_contents_view.h"
@@ -27,6 +27,7 @@
 #if defined(TOOLKIT_VIEWS)
 #include "chrome/browser/ui/views/find_bar_host.h"
 #include "views/focus/focus_manager.h"
+#include "views/widget/widget.h"
 #elif defined(TOOLKIT_GTK)
 #include "chrome/browser/ui/gtk/slide_animator_gtk.h"
 #elif defined(OS_MACOSX)
@@ -580,8 +581,12 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FindDisappearOnNavigate) {
   EXPECT_TRUE(fully_visible);
 
   // Reload the tab and make sure Find window doesn't go away.
+  ui_test_utils::WindowedNotificationObserver observer(
+      content::NOTIFICATION_LOAD_STOP,
+      Source<NavigationController>(
+          &browser()->GetSelectedTabContentsWrapper()->controller()));
   browser()->Reload(CURRENT_TAB);
-  ui_test_utils::WaitForNavigationInCurrentTab(browser());
+  observer.Wait();
 
   EXPECT_TRUE(GetFindBarWindowInfo(&position, &fully_visible));
   EXPECT_TRUE(fully_visible);
@@ -751,9 +756,9 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, AcceleratorRestoring) {
   GURL url = test_server()->GetURL(kSimplePage);
   ui_test_utils::NavigateToURL(browser(), url);
 
-  views::FocusManager* focus_manager =
-      views::FocusManager::GetFocusManagerForNativeWindow(
-          browser()->window()->GetNativeHandle());
+  gfx::NativeWindow window = browser()->window()->GetNativeHandle();
+  views::Widget* widget = views::Widget::GetWidgetForNativeWindow(window);
+  views::FocusManager* focus_manager = widget->GetFocusManager();
 
   // See where Escape is registered.
   views::Accelerator escape(ui::VKEY_ESCAPE, false, false, false);
@@ -777,6 +782,13 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, AcceleratorRestoring) {
   // The accelerator for Escape should be back to what it was before.
   EXPECT_EQ(old_target,
             focus_manager->GetCurrentTargetForAccelerator(escape));
+
+  // Show find bar again with animation on, and the target should be
+  // on find bar.
+  DropdownBarHost::disable_animations_during_testing_ = false;
+  browser()->ShowFindBar();
+  EXPECT_EQ(new_target,
+            focus_manager->GetCurrentTargetForAccelerator(escape));
 }
 #endif  // TOOLKIT_VIEWS
 
@@ -1047,9 +1059,10 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, MAYBE_NoIncognitoPrepopulate) {
   // Open a new incognito window and navigate to the same page.
   Profile* incognito_profile = browser()->profile()->GetOffTheRecordProfile();
   Browser* incognito_browser = Browser::Create(incognito_profile);
+  ui_test_utils::WindowedNotificationObserver observer(
+      content::NOTIFICATION_LOAD_STOP, NotificationService::AllSources());
   incognito_browser->AddSelectedTabWithURL(url, PageTransition::START_PAGE);
-  ui_test_utils::WaitForNavigation(
-      &incognito_browser->GetSelectedTabContents()->controller());
+  observer.Wait();
   incognito_browser->window()->Show();
 
   // Open the find box and make sure that it is prepopulated with "page".
@@ -1092,8 +1105,11 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, ActivateLinkNavigatesPage) {
   EXPECT_EQ(ordinal, 1);
 
   // End the find session, click on the link.
+  ui_test_utils::WindowedNotificationObserver observer(
+      content::NOTIFICATION_LOAD_STOP,
+      Source<NavigationController>(&tab->controller()));
   tab->find_tab_helper()->StopFinding(FindBarController::kActivateSelection);
-  EXPECT_TRUE(ui_test_utils::WaitForNavigationInCurrentTab(browser()));
+  observer.Wait();
 }
 
 // Tests that FindBar fits within a narrow browser window.
@@ -1101,12 +1117,12 @@ IN_PROC_BROWSER_TEST_F(FindInPageControllerTest, FitWindow) {
   Browser::CreateParams params(Browser::TYPE_POPUP, browser()->profile());
   params.initial_bounds = gfx::Rect(0, 0, 250, 500);
   Browser* popup = Browser::CreateWithParams(params);
+  ui_test_utils::WindowedNotificationObserver observer(
+      content::NOTIFICATION_LOAD_STOP, NotificationService::AllSources());
   popup->AddSelectedTabWithURL(GURL(chrome::kAboutBlankURL),
                                PageTransition::LINK);
-
   // Wait for the page to finish loading.
-  ui_test_utils::WaitForNavigation(
-      &popup->GetSelectedTabContents()->controller());
+  observer.Wait();
   popup->window()->Show();
 
   // On GTK, bounds change is asynchronous.
