@@ -19,19 +19,20 @@
 #include "ui/base/gtk/gtk_signal.h"
 
 class CookiesTreeModel;
+class TabContentsWrapper;
 
 // CollectedCookiesGtk is a dialog that displays the allowed and blocked
 // cookies of the current tab contents.  To display the dialog, invoke
 // ShowCollectedCookiesDialog() on the delegate of the tab contents wrapper's
 // content settings tab helper.
 
-class CollectedCookiesGtk : public ConstrainedDialogDelegate,
-                                   gtk_tree::TreeAdapter::Delegate,
-                                   NotificationObserver {
+class CollectedCookiesGtk : public ConstrainedWindowGtkDelegate,
+                            public gtk_tree::TreeAdapter::Delegate,
+                            public NotificationObserver {
  public:
-  CollectedCookiesGtk(GtkWindow* parent, TabContents* tab_contents);
+  CollectedCookiesGtk(GtkWindow* parent, TabContentsWrapper* wrapper);
 
-  // ConstrainedDialogDelegate methods.
+  // ConstrainedWindowGtkDelegate methods.
   virtual GtkWidget* GetWidgetRoot();
   virtual GtkWidget* GetFocusWidget();
   virtual void DeleteDelegate();
@@ -112,8 +113,8 @@ class CollectedCookiesGtk : public ConstrainedDialogDelegate,
   // Displays information about selected cookie.
   GtkWidget* cookie_info_view_;
 
-  // The tab contents.
-  TabContents* tab_contents_;
+  // The tab contents wrapper.
+  TabContentsWrapper* wrapper_;
 
   bool status_changed_;
 
