@@ -9,13 +9,13 @@
 #include <gtk/gtk.h>
 
 #include "base/memory/scoped_ptr.h"
-#include "chrome/browser/ui/gtk/owned_widget_gtk.h"
 #include "content/common/notification_observer.h"
 #include "content/common/notification_registrar.h"
 #include "third_party/skia/include/core/SkColor.h"
 #include "ui/base/animation/animation_delegate.h"
 #include "ui/base/animation/slide_animation.h"
 #include "ui/base/gtk/gtk_signal.h"
+#include "ui/base/gtk/owned_widget_gtk.h"
 #include "ui/gfx/rect.h"
 
 class CairoCachedSurface;
@@ -158,6 +158,10 @@ class CustomDrawButton : public NotificationObserver {
 
   void Init();
 
+  // Make this CustomDrawButton always use the chrome style rendering; it will
+  // never render gtk-like.
+  void ForceChromeTheme();
+
   // Flip the image horizontally. Not to be used for RTL/LTR reasons. (In RTL
   // mode, this will unflip the image.)
   void set_flipped(bool flipped) { button_base_.set_flipped(flipped); }
@@ -210,18 +214,21 @@ class CustomDrawButton : public NotificationObserver {
                        GdkEventExpose*);
 
   // The actual button widget.
-  OwnedWidgetGtk widget_;
+  ui::OwnedWidgetGtk widget_;
 
   CustomDrawButtonBase button_base_;
 
   CustomDrawHoverController hover_controller_;
 
   // The widget to use when we are displaying in GTK+ theme mode.
-  OwnedWidgetGtk native_widget_;
+  ui::OwnedWidgetGtk native_widget_;
 
   // Our theme provider.
   GtkThemeService* theme_service_;
 
+  // True if we should never do gtk rendering.
+  bool forcing_chrome_theme_;
+
   // Used to listen for theme change notifications.
   NotificationRegistrar registrar_;
 
