@@ -20,6 +20,7 @@
 #include "grit/generated_resources.h"
 #include "grit/theme_resources.h"
 #include "grit/ui_resources.h"
+#include "ui/base/gtk/gtk_hig_constants.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
 #include "ui/gfx/image/image.h"
@@ -207,7 +208,7 @@ void EditSearchEngineDialog::Init(GtkWindow* parent_window, Profile* profile) {
                      FALSE, FALSE, 0);
 
   gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(dialog_)->vbox),
-                      gtk_util::kContentAreaSpacing);
+                      ui::kContentAreaSpacing);
 
   EnableControls();
 
