@@ -24,34 +24,6 @@
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/gfx/gtk_util.h"
 
-namespace {
-
-// We need to know whether we're using a newer GTK at run time because we need
-// to prevent.
-//
-// TODO(erg): Once we've dropped Hardy support, remove this hack.
-typedef void (*gtk_menu_item_set_label_func)(GtkMenuItem*, const gchar*);
-gtk_menu_item_set_label_func gtk_menu_item_set_label_sym =
-#if GTK_CHECK_VERSION(2, 16, 1)
-    gtk_menu_item_set_label;
-#else
-    NULL;
-#endif
-
-void EnsureMenuItemFunctions() {
-#if !GTK_CHECK_VERSION(2, 16, 1)
-  static bool methods_looked_up = false;
-  if (!methods_looked_up) {
-    methods_looked_up = true;
-    gtk_menu_item_set_label_sym =
-        reinterpret_cast<gtk_menu_item_set_label_func>(
-            dlsym(NULL, "gtk_menu_item_set_label"));
-  }
-#endif
-}
-
-}  // namespace
-
 GlobalBookmarkMenu::GlobalBookmarkMenu(Browser* browser)
     : browser_(browser),
       profile_(browser->profile()),
@@ -74,13 +46,10 @@ void GlobalBookmarkMenu::Init(GtkWidget* bookmark_menu,
                               GtkWidget* bookmark_menu_item) {
   bookmark_menu_.Own(bookmark_menu);
 
-  EnsureMenuItemFunctions();
-  if (gtk_menu_item_set_label_sym) {
-    BookmarkModel* model = profile_->GetBookmarkModel();
-    model->AddObserver(this);
-    if (model->IsLoaded())
-      Loaded(model, false);
-  }
+  BookmarkModel* model = profile_->GetBookmarkModel();
+  model->AddObserver(this);
+  if (model->IsLoaded())
+    Loaded(model, false);
 }
 
 void GlobalBookmarkMenu::RebuildMenuInFuture() {
@@ -112,7 +81,7 @@ void GlobalBookmarkMenu::RebuildMenu() {
     AddBookmarkMenuItem(bookmark_menu_.get(), gtk_separator_menu_item_new());
 
     GtkWidget* menu_item = gtk_image_menu_item_new_with_label(
-        l10n_util::GetStringUTF8(IDS_BOOMARK_BAR_OTHER_FOLDER_NAME).c_str());
+        l10n_util::GetStringUTF8(IDS_BOOKMARK_BAR_OTHER_FOLDER_NAME).c_str());
     gtk_menu_item_set_submenu(GTK_MENU_ITEM(menu_item), submenu);
     gtk_image_menu_item_set_image(
         GTK_IMAGE_MENU_ITEM(menu_item),
@@ -167,14 +136,8 @@ void GlobalBookmarkMenu::ConfigureMenuItem(const BookmarkNode* node,
   CHECK(node);
   CHECK(menu_item);
 
-  // This check is only to make things compile on Hardy; this code won't
-  // display any visible widgets in older systems that don't have a global menu
-  // bar.
-  if (gtk_menu_item_set_label_sym) {
-    gtk_menu_item_set_label_sym(
-        GTK_MENU_ITEM(menu_item),
-        bookmark_utils::BuildMenuLabelFor(node).c_str());
-  }
+  gtk_menu_item_set_label(GTK_MENU_ITEM(menu_item),
+                          bookmark_utils::BuildMenuLabelFor(node).c_str());
 
   if (node->is_url()) {
     gtk_widget_set_tooltip_markup(
@@ -298,7 +261,7 @@ void GlobalBookmarkMenu::OnBookmarkItemActivated(GtkWidget* menu_item) {
   const BookmarkNode* node = static_cast<const BookmarkNode*>(
       g_object_get_data(G_OBJECT(menu_item), "bookmark-node"));
 
-  browser_->OpenURL(node->url(), GURL(), NEW_FOREGROUND_TAB,
-                    PageTransition::AUTO_BOOKMARK);
+  browser_->OpenURL(OpenURLParams(node->url(), GURL(), NEW_FOREGROUND_TAB,
+                    PageTransition::AUTO_BOOKMARK));
 }
 
