@@ -10,10 +10,10 @@
 #include "chrome/browser/command_updater.h"
 #include "chrome/browser/ui/gtk/global_bookmark_menu.h"
 #include "chrome/browser/ui/gtk/global_history_menu.h"
-#include "chrome/browser/ui/gtk/owned_widget_gtk.h"
 #include "content/common/notification_observer.h"
 #include "content/common/notification_registrar.h"
 #include "ui/base/gtk/gtk_signal.h"
+#include "ui/base/gtk/owned_widget_gtk.h"
 
 class Browser;
 struct GlobalMenuBarCommand;
@@ -43,6 +43,10 @@ class GlobalMenuBar : public CommandUpdater::CommandObserver,
   explicit GlobalMenuBar(Browser* browser);
   virtual ~GlobalMenuBar();
 
+  // Use this method to remove the GlobalMenuBar from any further notifications
+  // and command updates but not destroy the widgets.
+  virtual void Disable();
+
   GtkWidget* widget() { return menu_bar_.get(); }
 
  private:
@@ -72,12 +76,11 @@ class GlobalMenuBar : public CommandUpdater::CommandObserver,
   CHROMEGTK_CALLBACK_0(GlobalMenuBar, void, OnItemActivated);
 
   Browser* browser_;
-  Profile* profile_;
 
   NotificationRegistrar registrar_;
 
   // Our menu bar widget.
-  OwnedWidgetGtk menu_bar_;
+  ui::OwnedWidgetGtk menu_bar_;
 
   // Listens to the TabRestoreService and the HistoryService and keeps the
   // history menu fresh.
