@@ -6,8 +6,8 @@
 
 #include "base/i18n/time_formatting.h"
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/ui/gtk/gtk_util.h"
 #include "grit/generated_resources.h"
+#include "ui/base/gtk/gtk_hig_constants.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/text/bytes_formatting.h"
 
@@ -295,7 +295,7 @@ void BuildWidgets(GtkChromeCookieView *self, gboolean editable_expiration) {
   gtk_container_add(GTK_CONTAINER(self->table_box_),
                     self->cookie_details_table_);
   gtk_table_set_col_spacing(GTK_TABLE(self->cookie_details_table_), 0,
-                            gtk_util::kLabelSpacing);
+                            ui::kLabelSpacing);
 
   int row = 0;
   self->first_label_ = InitDetailRow(row++, IDS_COOKIES_COOKIE_NAME_LABEL,
@@ -325,7 +325,7 @@ void BuildWidgets(GtkChromeCookieView *self, gboolean editable_expiration) {
   gtk_container_add(GTK_CONTAINER(self->table_box_),
                     self->database_details_table_);
   gtk_table_set_col_spacing(GTK_TABLE(self->database_details_table_), 0,
-                            gtk_util::kLabelSpacing);
+                            ui::kLabelSpacing);
 
   InitDetailRow(row++, IDS_COOKIES_COOKIE_NAME_LABEL,
                 self->database_details_table_, &self->database_name_entry_);
@@ -343,7 +343,7 @@ void BuildWidgets(GtkChromeCookieView *self, gboolean editable_expiration) {
   gtk_container_add(GTK_CONTAINER(self->table_box_),
                     self->local_storage_details_table_);
   gtk_table_set_col_spacing(GTK_TABLE(self->local_storage_details_table_), 0,
-                            gtk_util::kLabelSpacing);
+                            ui::kLabelSpacing);
 
   row = 0;
   InitDetailRow(row++, IDS_COOKIES_LOCAL_STORAGE_ORIGIN_LABEL,
@@ -361,7 +361,7 @@ void BuildWidgets(GtkChromeCookieView *self, gboolean editable_expiration) {
   gtk_container_add(GTK_CONTAINER(self->table_box_),
                     self->appcache_details_table_);
   gtk_table_set_col_spacing(GTK_TABLE(self->appcache_details_table_), 0,
-                            gtk_util::kLabelSpacing);
+                            ui::kLabelSpacing);
 
   row = 0;
   InitDetailRow(row++, IDS_COOKIES_APPLICATION_CACHE_MANIFEST_LABEL,
@@ -380,7 +380,7 @@ void BuildWidgets(GtkChromeCookieView *self, gboolean editable_expiration) {
   gtk_container_add(GTK_CONTAINER(self->table_box_),
                     self->indexed_db_details_table_);
   gtk_table_set_col_spacing(GTK_TABLE(self->indexed_db_details_table_), 0,
-                            gtk_util::kLabelSpacing);
+                            ui::kLabelSpacing);
 
   row = 0;
   InitDetailRow(row++, IDS_COOKIES_LOCAL_STORAGE_ORIGIN_LABEL,
@@ -397,7 +397,7 @@ void BuildWidgets(GtkChromeCookieView *self, gboolean editable_expiration) {
   gtk_container_add(GTK_CONTAINER(self->table_box_),
                     self->local_storage_item_table_);
   gtk_table_set_col_spacing(GTK_TABLE(self->local_storage_item_table_), 0,
-                            gtk_util::kLabelSpacing);
+                            ui::kLabelSpacing);
 
   row = 0;
   InitDetailRow(row++, IDS_COOKIES_COOKIE_DOMAIN_LABEL,
@@ -415,7 +415,7 @@ void BuildWidgets(GtkChromeCookieView *self, gboolean editable_expiration) {
   gtk_container_add(GTK_CONTAINER(self->table_box_),
                     self->database_accessed_table_);
   gtk_table_set_col_spacing(GTK_TABLE(self->local_storage_item_table_), 0,
-                            gtk_util::kLabelSpacing);
+                            ui::kLabelSpacing);
 
   row = 0;
   InitDetailRow(row++, IDS_COOKIES_COOKIE_DOMAIN_LABEL,
@@ -436,7 +436,7 @@ void BuildWidgets(GtkChromeCookieView *self, gboolean editable_expiration) {
   gtk_container_add(GTK_CONTAINER(self->table_box_),
                     self->appcache_created_table_);
   gtk_table_set_col_spacing(GTK_TABLE(self->appcache_created_table_), 0,
-                            gtk_util::kLabelSpacing);
+                            ui::kLabelSpacing);
   row = 0;
   InitDetailRow(row++, IDS_COOKIES_APPLICATION_CACHE_MANIFEST_LABEL,
                 self->appcache_created_table_,
@@ -592,7 +592,7 @@ void gtk_chrome_cookie_view_display_indexed_db(
   UpdateVisibleDetailedInfo(self, self->indexed_db_details_table_);
 
   gtk_entry_set_text(GTK_ENTRY(self->indexed_db_origin_entry_),
-                     indexed_db_info.origin.c_str());
+                     indexed_db_info.origin.spec().c_str());
   gtk_entry_set_text(GTK_ENTRY(self->indexed_db_size_entry_),
                      UTF16ToUTF8(ui::FormatBytes(
                          indexed_db_info.size)).c_str());
