@@ -13,8 +13,8 @@ class TranslateInfoBarDelegate;
 
 class AfterTranslateInfoBar : public TranslateInfoBarBase {
  public:
-  explicit AfterTranslateInfoBar(TabContentsWrapper* owner,
-                                 TranslateInfoBarDelegate* delegate);
+  AfterTranslateInfoBar(TabContentsWrapper* owner,
+                        TranslateInfoBarDelegate* delegate);
   virtual ~AfterTranslateInfoBar();
 
   // Overridden from TranslateInfoBarBase:
