@@ -19,8 +19,8 @@ typedef struct _GtkWidget GtkWidget;
 // "Would you like to do X? [Yes] [No]                         _Learn More_ [x]"
 class ConfirmInfoBarGtk : public InfoBarGtk {
  public:
-  explicit ConfirmInfoBarGtk(TabContentsWrapper* owner,
-                             ConfirmInfoBarDelegate* delegate);
+  ConfirmInfoBarGtk(TabContentsWrapper* owner,
+                    ConfirmInfoBarDelegate* delegate);
 
  private:
   virtual ~ConfirmInfoBarGtk();
