@@ -119,7 +119,7 @@ void ReloadButtonGtk::ChangeMode(Mode mode, bool force) {
     // Go ahead and change to reload after a bit, which allows repeated reloads
     // without moving the mouse.
     if (!stop_to_reload_timer_.IsRunning()) {
-      stop_to_reload_timer_.Start(stop_to_reload_timer_delay_, this,
+      stop_to_reload_timer_.Start(FROM_HERE, stop_to_reload_timer_delay_, this,
                                   &ReloadButtonGtk::OnStopToReloadTimer);
     }
   }
@@ -172,7 +172,7 @@ void ReloadButtonGtk::OnClicked(GtkWidget* /* sender */) {
     }
 
     WindowOpenDisposition disposition =
-        event_utils::DispositionFromEventFlags(modifier_state_uint);
+        event_utils::DispositionFromGdkState(modifier_state_uint);
     if ((disposition == CURRENT_TAB) && location_bar_) {
       // Forcibly reset the location bar, since otherwise it won't discard any
       // ongoing user edits, since it doesn't realize this is a user-initiated
@@ -185,7 +185,7 @@ void ReloadButtonGtk::OnClicked(GtkWidget* /* sender */) {
     // here as the browser will do that when it actually starts loading (which
     // may happen synchronously, thus the need to do this before telling the
     // browser to execute the reload command).
-    double_click_timer_.Start(double_click_timer_delay_, this,
+    double_click_timer_.Start(FROM_HERE, double_click_timer_delay_, this,
                               &ReloadButtonGtk::OnDoubleClickTimer);
 
     if (browser_)
