@@ -11,10 +11,10 @@
 #include "base/basictypes.h"
 #include "base/timer.h"
 #include "chrome/browser/ui/gtk/custom_button.h"
-#include "chrome/browser/ui/gtk/owned_widget_gtk.h"
 #include "content/common/notification_observer.h"
 #include "content/common/notification_registrar.h"
 #include "ui/base/gtk/gtk_signal.h"
+#include "ui/base/gtk/owned_widget_gtk.h"
 
 class Browser;
 class GtkThemeService;
@@ -83,7 +83,7 @@ class ReloadButtonGtk : public NotificationObserver {
   CustomDrawButtonBase stop_;
   CustomDrawHoverController hover_controller_;
 
-  OwnedWidgetGtk widget_;
+  ui::OwnedWidgetGtk widget_;
 
   // The delay times for the timers.  These are members so that tests can modify
   // them.
