@@ -9,10 +9,13 @@
 #include "chrome/browser/ui/browser_window.h"
 
 #include "base/memory/scoped_ptr.h"
+#include "content/common/notification_observer.h"
+#include "content/common/notification_registrar.h"
 #include "ui/gfx/rect.h"
 
 class NativePanel;
 class PanelManager;
+class RenderViewHost;
 
 // A platform independent implementation of BrowserWindow for Panels.  This
 // class would get the first crack at all the BrowserWindow calls for Panels and
@@ -24,7 +27,7 @@ class PanelManager;
 //   Panel size is restricted to certain limits.
 // - Invoke an appropriate PanelManager function to do stuff that might affect
 //   other Panels.  For example deleting a panel would rearrange other panels.
-class Panel : public BrowserWindow {
+class Panel : public BrowserWindow, public NotificationObserver {
  public:
   enum ExpansionState {
    // The panel is fully expanded with both title-bar and the client-area.
@@ -42,10 +45,15 @@ class Panel : public BrowserWindow {
 
   void SetExpansionState(ExpansionState new_expansion_state);
 
-  bool ShouldBringUpTitleBar(int mouse_x, int mouse_y) const;
+  bool ShouldBringUpTitlebar(int mouse_x, int mouse_y) const;
 
   bool IsDrawingAttention() const;
 
+  // Gets or sets the restored height, which is the full height of the panel
+  // when it is expanded.
+  int GetRestoredHeight() const;
+  void SetRestoredHeight(int height);
+
   // BrowserWindow overrides.
   virtual void Show() OVERRIDE;
   virtual void ShowInactive() OVERRIDE;
@@ -68,6 +76,7 @@ class Panel : public BrowserWindow {
   virtual gfx::Rect GetRestoredBounds() const OVERRIDE;
   virtual gfx::Rect GetBounds() const OVERRIDE;
   virtual bool IsMaximized() const OVERRIDE;
+  virtual bool IsMinimized() const OVERRIDE;
   virtual void SetFullscreen(bool fullscreen) OVERRIDE;
   virtual bool IsFullscreen() const OVERRIDE;
   virtual bool IsFullscreenBubbleVisible() const OVERRIDE;
@@ -102,11 +111,12 @@ class Panel : public BrowserWindow {
   virtual bool IsDownloadShelfVisible() const OVERRIDE;
   virtual DownloadShelf* GetDownloadShelf() OVERRIDE;
   virtual void ShowRepostFormWarningDialog(TabContents* tab_contents) OVERRIDE;
-  virtual void ShowCollectedCookiesDialog(TabContents* tab_contents) OVERRIDE;
+  virtual void ShowCollectedCookiesDialog(TabContentsWrapper* wrapper) OVERRIDE;
   virtual void ShowThemeInstallBubble() OVERRIDE;
   virtual void ConfirmBrowserCloseWithPendingDownloads() OVERRIDE;
-  virtual void ShowHTMLDialog(HtmlDialogUIDelegate* delegate,
-                              gfx::NativeWindow parent_window) OVERRIDE;
+  virtual gfx::NativeWindow ShowHTMLDialog(
+      HtmlDialogUIDelegate* delegate,
+      gfx::NativeWindow parent_window) OVERRIDE;
   virtual void UserChangedTheme() OVERRIDE;
   virtual int GetExtraRenderViewHeight() const OVERRIDE;
   virtual void TabContentsFocused(TabContents* tab_contents) OVERRIDE;
@@ -140,9 +150,17 @@ class Panel : public BrowserWindow {
   virtual gfx::Rect GetInstantBounds() OVERRIDE;
   virtual WindowOpenDisposition GetDispositionForPopupBounds(
       const gfx::Rect& bounds) OVERRIDE;
+  virtual FindBar* CreateFindBar() OVERRIDE;
 #if defined(OS_CHROMEOS)
   virtual void ShowKeyboardOverlay(gfx::NativeWindow owning_window) OVERRIDE;
 #endif
+  virtual void UpdatePreferredSize(TabContents* tab_contents,
+                                   const gfx::Size& pref_size) OVERRIDE;
+
+  // NotificationObserver overrides.
+  virtual void Observe(int type,
+                       const NotificationSource& source,
+                       const NotificationDetails& details) OVERRIDE;
 
   // Construct a native panel BrowserWindow implementation for the specified
   // |browser|.
@@ -154,11 +172,11 @@ class Panel : public BrowserWindow {
   // Returns NULL if it cannot be found.
   static const Extension* GetExtension(Browser* browser);
 
-#ifdef UNIT_TEST
   NativePanel* native_panel() { return native_panel_; }
-#endif
-
+  Browser* browser() const;
   ExpansionState expansion_state() const { return expansion_state_; }
+  const gfx::Size& min_size() const { return min_size_; }
+  const gfx::Size& max_size() const { return max_size_; }
 
  protected:
   virtual void DestroyBrowser() OVERRIDE;
@@ -175,7 +193,23 @@ class Panel : public BrowserWindow {
   //   not allowed for Panel.
   void SetPanelBounds(const gfx::Rect& bounds);
 
-  Browser* browser_;  // Weak, owned by native_panel.
+  // Updates the maximum size.
+  void SetMaxSize(const gfx::Size& max_size);
+
+  // NULL might be returned if the tab has not been added.
+  RenderViewHost* GetRenderViewHost() const;
+
+  // Requests RenderViewHost not to show the scrollbars till |max_size_| since
+  // the panel can grow to |max_size_|.
+  void RequestRenderViewHostToDisableScrollbars(
+      RenderViewHost* render_view_host);
+
+  // This is the minimum size that the panel can shrink to.
+  gfx::Size min_size_;
+
+  // This is the size beyond which the panel is not going to grow to accomodate
+  // the growing content and WebKit would add the scrollbars in such case.
+  gfx::Size max_size_;
 
   // Platform specifc implementation for panels.  It'd be one of
   // PanelBrowserWindowGtk/PanelBrowserView/PanelBrowserWindowCocoa.
@@ -183,6 +217,8 @@ class Panel : public BrowserWindow {
 
   ExpansionState expansion_state_;
 
+  NotificationRegistrar registrar_;
+
   DISALLOW_COPY_AND_ASSIGN(Panel);
 };
 
