@@ -8,15 +8,25 @@
 
 #include "ui/base/models/simple_menu_model.h"
 
-// ProfileMenuModel
-//
-// Menu for the multi-profile button displayed on the browser frame when the
-// user is in a multi-profile-enabled account. Stub for now. TODO(mirandac):
-// enable and fill in as part of multi-profile work.
+class Browser;
+
+// Menu for the multi-profile button displayed on the browser frame.
 class ProfileMenuModel : public ui::SimpleMenuModel,
                          public ui::SimpleMenuModel::Delegate {
  public:
-  explicit ProfileMenuModel();
+  enum {
+    COMMAND_PROFILE_NAME,
+    COMMAND_CHOOSE_AVATAR_ICON,
+    COMMAND_CUSTOMIZE_PROFILE,
+    COMMAND_CREATE_NEW_PROFILE,
+    COMMAND_SWITCH_PROFILE_MENU,
+    // The profiles submenu contains a menu item for each profile. For the i'th
+    // profile the command ID is COMMAND_SWITCH_TO_PROFILE + i. Since there can
+    // be any number of profiles this must be the last command id.
+    COMMAND_SWITCH_TO_PROFILE,
+  };
+
+  explicit ProfileMenuModel(Browser* browser);
   virtual ~ProfileMenuModel();
 
   // ui::SimpleMenuModel::Delegate implementation
@@ -27,13 +37,10 @@ class ProfileMenuModel : public ui::SimpleMenuModel,
   virtual void ExecuteCommand(int command_id) OVERRIDE;
 
  private:
-  enum {
-    COMMAND_CREATE_NEW_PROFILE,
-  };
+  Browser* browser_;
+  scoped_ptr<ui::SimpleMenuModel> switch_profiles_sub_menu_model_;
 
   DISALLOW_COPY_AND_ASSIGN(ProfileMenuModel);
 };
 
 #endif  // CHROME_BROWSER_UI_PROFILE_MENU_MODEL_H_
-
-
