@@ -4,60 +4,56 @@
 
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
 
-#include "base/command_line.h"
+#include "base/utf_string_conversions.h"
+
 #include "base/lazy_instance.h"
+#include "base/stringprintf.h"
 #include "chrome/browser/autocomplete_history_manager.h"
 #include "chrome/browser/autofill/autofill_manager.h"
 #include "chrome/browser/automation/automation_tab_helper.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/browser_shutdown.h"
 #include "chrome/browser/content_settings/tab_specific_content_settings.h"
-#include "chrome/browser/custom_handlers/protocol_handler_registry.h"
-#include "chrome/browser/custom_handlers/register_protocol_handler_infobar_delegate.h"
+#include "chrome/browser/download/download_request_limiter_observer.h"
 #include "chrome/browser/extensions/extension_tab_helper.h"
 #include "chrome/browser/extensions/extension_webnavigation_api.h"
 #include "chrome/browser/external_protocol/external_protocol_observer.h"
 #include "chrome/browser/favicon/favicon_tab_helper.h"
-#include "chrome/browser/file_select_helper.h"
 #include "chrome/browser/google/google_util.h"
 #include "chrome/browser/history/history_tab_helper.h"
+#include "chrome/browser/infobars/infobar_tab_helper.h"
 #include "chrome/browser/omnibox_search_hint.h"
 #include "chrome/browser/password_manager/password_manager.h"
 #include "chrome/browser/password_manager_delegate_impl.h"
 #include "chrome/browser/pdf_unsupported_feature.h"
 #include "chrome/browser/plugin_observer.h"
 #include "chrome/browser/prefs/pref_service.h"
-#include "chrome/browser/prerender/prerender_observer.h"
+#include "chrome/browser/prerender/prerender_tab_helper.h"
 #include "chrome/browser/printing/print_preview_message_handler.h"
-#include "chrome/browser/profiles/profile.h"
-#include "chrome/browser/remoting/firewall_traversal_tab_helper.h"
+#include "chrome/browser/printing/print_view_manager.h"
+#include "chrome/browser/remoting/firewall_traversal_observer.h"
 #include "chrome/browser/renderer_host/web_cache_manager.h"
 #include "chrome/browser/renderer_preferences_util.h"
 #include "chrome/browser/sessions/restore_tab_helper.h"
 #include "chrome/browser/safe_browsing/client_side_detection_host.h"
-#include "chrome/browser/tab_contents/infobar.h"
-#include "chrome/browser/tab_contents/infobar_delegate.h"
-#include "chrome/browser/tab_contents/insecure_content_infobar_delegate.h"
-#include "chrome/browser/tab_contents/simple_alert_infobar_delegate.h"
 #include "chrome/browser/tab_contents/tab_contents_ssl_helper.h"
 #include "chrome/browser/tab_contents/thumbnail_generator.h"
+#include "chrome/browser/themes/theme_service.h"
+#include "chrome/browser/themes/theme_service_factory.h"
 #include "chrome/browser/translate/translate_tab_helper.h"
 #include "chrome/browser/ui/blocked_content/blocked_content_tab_helper.h"
 #include "chrome/browser/ui/bookmarks/bookmark_tab_helper.h"
 #include "chrome/browser/ui/find_bar/find_tab_helper.h"
+#include "chrome/browser/ui/intents/web_intent_constrained_dialog_factory.h"
+#include "chrome/browser/ui/intents/web_intent_picker_controller.h"
 #include "chrome/browser/ui/search_engines/search_engine_tab_helper.h"
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper_delegate.h"
 #include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/chrome_switches.h"
-#include "chrome/common/custom_handlers/protocol_handler.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/render_messages.h"
-#include "content/browser/child_process_security_policy.h"
 #include "content/browser/renderer_host/render_view_host.h"
-#include "content/browser/tab_contents/navigation_details.h"
-#include "content/browser/tab_contents/tab_contents.h"
 #include "content/browser/tab_contents/tab_contents_view.h"
-#include "content/browser/user_metrics.h"
 #include "content/common/notification_service.h"
 #include "content/common/view_messages.h"
 #include "grit/generated_resources.h"
@@ -100,15 +96,121 @@ const char* kPrefsToObserve[] = {
 
 const int kPrefsToObserveLength = arraysize(kPrefsToObserve);
 
+// Registers a preference under the path |map_name| for each script used for
+// per-script font prefs.  For example, if |map_name| is "fonts.serif", then
+// "fonts.serif.Arab", "fonts.serif.Hang", etc. are registered.
+void RegisterFontFamilyMap(PrefService* prefs, const char* map_name) {
+  for (size_t i = 0; i < prefs::kWebKitScriptsForFontFamilyMapsLength; ++i) {
+    const char* script = prefs::kWebKitScriptsForFontFamilyMaps[i];
+    std::string pref_name_str = base::StringPrintf("%s.%s", map_name, script);
+    const char* pref_name = pref_name_str.c_str();
+    if (!prefs->FindPreference(pref_name))
+      prefs->RegisterStringPref(pref_name, "", PrefService::UNSYNCABLE_PREF);
+  }
 }
 
+struct PerScriptFontDefault {
+  const char* pref_name;
+  int resource_id;
+  const char* native_locale;
+};
+
+// Per-script font pref defaults.  The prefs that have defaults vary by
+// platform, since not all platforms have fonts for all scripts for all generic
+// families.
+// TODO(falken): add proper defaults when possible for all
+// platforms/scripts/generic families.
+const PerScriptFontDefault kPerScriptFontDefaults[] = {
+#if defined(OS_CHROMEOS)
+  { prefs::kWebKitStandardFontFamilyArabic, IDS_STANDARD_FONT_FAMILY_ARABIC,
+    "ar" },
+  { prefs::kWebKitSerifFontFamilyArabic, IDS_SERIF_FONT_FAMILY_ARABIC, "ar" },
+  { prefs::kWebKitSansSerifFontFamilyArabic,
+    IDS_SANS_SERIF_FONT_FAMILY_ARABIC, "ar" },
+  { prefs::kWebKitStandardFontFamilyJapanese,
+    IDS_STANDARD_FONT_FAMILY_JAPANESE, "ja" },
+  { prefs::kWebKitFixedFontFamilyJapanese, IDS_FIXED_FONT_FAMILY_JAPANESE,
+    "ja" },
+  { prefs::kWebKitSerifFontFamilyJapanese, IDS_SERIF_FONT_FAMILY_JAPANESE,
+    "ja" },
+  { prefs::kWebKitSansSerifFontFamilyJapanese,
+    IDS_SANS_SERIF_FONT_FAMILY_JAPANESE, "ja" },
+  { prefs::kWebKitStandardFontFamilyKorean, IDS_STANDARD_FONT_FAMILY_KOREAN,
+    "ko" },
+  { prefs::kWebKitFixedFontFamilyKorean, IDS_FIXED_FONT_FAMILY_KOREAN, "ko" },
+  { prefs::kWebKitSerifFontFamilyKorean, IDS_SERIF_FONT_FAMILY_KOREAN, "ko" },
+  { prefs::kWebKitSansSerifFontFamilyKorean,
+    IDS_SANS_SERIF_FONT_FAMILY_KOREAN, "ko" },
+  { prefs::kWebKitStandardFontFamilySimplifiedHan,
+    IDS_STANDARD_FONT_FAMILY_SIMPLIFIED_HAN, "zh-CN" },
+  { prefs::kWebKitFixedFontFamilySimplifiedHan,
+    IDS_FIXED_FONT_FAMILY_SIMPLIFIED_HAN, "zh-CN" },
+  { prefs::kWebKitSerifFontFamilySimplifiedHan,
+    IDS_SERIF_FONT_FAMILY_SIMPLIFIED_HAN, "zh-CN" },
+  { prefs::kWebKitSansSerifFontFamilySimplifiedHan,
+    IDS_SANS_SERIF_FONT_FAMILY_SIMPLIFIED_HAN, "zh-CN" },
+  { prefs::kWebKitStandardFontFamilyTraditionalHan,
+    IDS_STANDARD_FONT_FAMILY_TRADITIONAL_HAN, "zh-TW" },
+  { prefs::kWebKitFixedFontFamilyTraditionalHan,
+    IDS_FIXED_FONT_FAMILY_TRADITIONAL_HAN, "zh-TW" },
+  { prefs::kWebKitSerifFontFamilyTraditionalHan,
+    IDS_SERIF_FONT_FAMILY_TRADITIONAL_HAN, "zh-TW" },
+  { prefs::kWebKitSansSerifFontFamilyTraditionalHan,
+    IDS_SANS_SERIF_FONT_FAMILY_TRADITIONAL_HAN, "zh-TW" }
+#elif defined(OS_WIN)
+  { prefs::kWebKitStandardFontFamilyJapanese,
+    IDS_STANDARD_FONT_FAMILY_JAPANESE, "ja" },
+  { prefs::kWebKitFixedFontFamilyJapanese, IDS_FIXED_FONT_FAMILY_JAPANESE,
+    "ja" },
+  { prefs::kWebKitSerifFontFamilyJapanese, IDS_SERIF_FONT_FAMILY_JAPANESE,
+    "ja" },
+  { prefs::kWebKitSansSerifFontFamilyJapanese,
+    IDS_SANS_SERIF_FONT_FAMILY_JAPANESE, "ja" },
+  { prefs::kWebKitStandardFontFamilyKorean, IDS_STANDARD_FONT_FAMILY_KOREAN,
+    "ko" },
+  { prefs::kWebKitFixedFontFamilyKorean, IDS_FIXED_FONT_FAMILY_KOREAN, "ko" },
+  { prefs::kWebKitSerifFontFamilyKorean, IDS_SERIF_FONT_FAMILY_KOREAN, "ko" },
+  { prefs::kWebKitSansSerifFontFamilyKorean,
+    IDS_SANS_SERIF_FONT_FAMILY_KOREAN, "ko" },
+  { prefs::kWebKitCursiveFontFamilyKorean, IDS_CURSIVE_FONT_FAMILY_KOREAN,
+    "ko" },
+  { prefs::kWebKitStandardFontFamilySimplifiedHan,
+    IDS_STANDARD_FONT_FAMILY_SIMPLIFIED_HAN, "zh-CN" },
+  { prefs::kWebKitFixedFontFamilySimplifiedHan,
+    IDS_FIXED_FONT_FAMILY_SIMPLIFIED_HAN, "zh-CN" },
+  { prefs::kWebKitSerifFontFamilySimplifiedHan,
+    IDS_SERIF_FONT_FAMILY_SIMPLIFIED_HAN, "zh-CN" },
+  { prefs::kWebKitSansSerifFontFamilySimplifiedHan,
+    IDS_SANS_SERIF_FONT_FAMILY_SIMPLIFIED_HAN, "zh-CN" },
+  { prefs::kWebKitStandardFontFamilyTraditionalHan,
+    IDS_STANDARD_FONT_FAMILY_TRADITIONAL_HAN, "zh-TW" },
+  { prefs::kWebKitFixedFontFamilyTraditionalHan,
+    IDS_FIXED_FONT_FAMILY_TRADITIONAL_HAN, "zh-TW" },
+  { prefs::kWebKitSerifFontFamilyTraditionalHan,
+    IDS_SERIF_FONT_FAMILY_TRADITIONAL_HAN, "zh-TW" },
+  { prefs::kWebKitSansSerifFontFamilyTraditionalHan,
+    IDS_SANS_SERIF_FONT_FAMILY_TRADITIONAL_HAN, "zh-TW" }
+#endif
+};
+
+#if defined(OS_CHROMEOS) || defined(OS_WIN)
+// To avoid Clang warning, only define kPerScriptFontDefaultsLength when it is
+// non-zero.  When it is zero, code like
+//  for (size_t i = 0; i < kPerScriptFontDefaultsLength; ++i)
+// causes a warning due to comparison of unsigned expression < 0.
+const size_t kPerScriptFontDefaultsLength = arraysize(kPerScriptFontDefaults);
+#endif
+
+}  // namespace
+
 ////////////////////////////////////////////////////////////////////////////////
 // TabContentsWrapper, public:
 
 TabContentsWrapper::TabContentsWrapper(TabContents* contents)
     : TabContentsObserver(contents),
       delegate_(NULL),
-      infobars_enabled_(true),
+      ALLOW_THIS_IN_INITIALIZER_LIST(
+          synced_tab_delegate_(new TabContentsWrapperSyncedTabDelegate(this))),
       in_destructor_(false),
       tab_contents_(contents) {
   DCHECK(contents);
@@ -126,13 +228,14 @@ TabContentsWrapper::TabContentsWrapper(TabContents* contents)
   extension_tab_helper_.reset(new ExtensionTabHelper(this));
   favicon_tab_helper_.reset(new FaviconTabHelper(contents));
   find_tab_helper_.reset(new FindTabHelper(contents));
-  firewall_traversal_tab_helper_.reset(
-      new FirewallTraversalTabHelper(contents));
   history_tab_helper_.reset(new HistoryTabHelper(contents));
-  restore_tab_helper_.reset(new RestoreTabHelper(this));
+  infobar_tab_helper_.reset(new InfoBarTabHelper(this));
   password_manager_delegate_.reset(new PasswordManagerDelegateImpl(this));
   password_manager_.reset(
       new PasswordManager(contents, password_manager_delegate_.get()));
+  prerender_tab_helper_.reset(new prerender::PrerenderTabHelper(this));
+  print_view_manager_.reset(new printing::PrintViewManager(this));
+  restore_tab_helper_.reset(new RestoreTabHelper(this));
 #if defined(ENABLE_SAFE_BROWSING)
   if (profile()->GetPrefs()->GetBoolean(prefs::kSafeBrowsingEnabled) &&
       g_browser_process->safe_browsing_detection_service()) {
@@ -144,16 +247,19 @@ TabContentsWrapper::TabContentsWrapper(TabContents* contents)
   ssl_helper_.reset(new TabContentsSSLHelper(this));
   content_settings_.reset(new TabSpecificContentSettings(contents));
   translate_tab_helper_.reset(new TranslateTabHelper(contents));
-  print_view_manager_.reset(new printing::PrintViewManager(this));
+  web_intent_picker_controller_.reset(new WebIntentPickerController(
+        contents,
+        new WebIntentConstrainedDialogFactory()));
 
   // Create the per-tab observers.
-  external_protocol_observer_.reset(new ExternalProtocolObserver(contents));
-  file_select_observer_.reset(new FileSelectObserver(contents));
-  plugin_observer_.reset(new PluginObserver(this));
-  prerender_observer_.reset(new prerender::PrerenderObserver(this));
-  print_preview_.reset(new printing::PrintPreviewMessageHandler(contents));
+  download_request_limiter_observer_.reset(
+      new DownloadRequestLimiterObserver(contents));
   webnavigation_observer_.reset(
       new ExtensionWebNavigationTabObserver(contents));
+  external_protocol_observer_.reset(new ExternalProtocolObserver(contents));
+  firewall_traversal_observer_.reset(new FirewallTraversalObserver(contents));
+  plugin_observer_.reset(new PluginObserver(this));
+  print_preview_.reset(new printing::PrintPreviewMessageHandler(contents));
   // Start the in-browser thumbnailing if the feature is enabled.
   if (switches::IsInBrowserThumbnailingEnabled()) {
     thumbnail_generation_observer_.reset(new ThumbnailGenerator);
@@ -161,7 +267,7 @@ TabContentsWrapper::TabContentsWrapper(TabContents* contents)
   }
 
   // Set-up the showing of the omnibox search infobar if applicable.
-  if (OmniboxSearchHint::IsEnabled(contents->profile()))
+  if (OmniboxSearchHint::IsEnabled(profile()))
     omnibox_search_hint_.reset(new OmniboxSearchHint(this));
 
   registrar_.Add(this, chrome::NOTIFICATION_GOOGLE_URL_UPDATED,
@@ -170,7 +276,8 @@ TabContentsWrapper::TabContentsWrapper(TabContents* contents)
                  NotificationService::AllSources());
 #if defined(OS_POSIX) && !defined(OS_MACOSX)
   registrar_.Add(this, chrome::NOTIFICATION_BROWSER_THEME_CHANGED,
-                 NotificationService::AllSources());
+                 Source<ThemeService>(
+                     ThemeServiceFactory::GetForProfile(profile())));
 #endif
 
   // Register for notifications about all interested prefs change.
@@ -188,13 +295,8 @@ TabContentsWrapper::TabContentsWrapper(TabContents* contents)
 TabContentsWrapper::~TabContentsWrapper() {
   in_destructor_ = true;
 
-  // Destroy all remaining InfoBars.  It's important to not animate here so that
-  // we guarantee that we'll delete all delegates before we do anything else.
-  //
-  // TODO(pkasting): If there is no InfoBarContainer, this leaks all the
-  // InfoBarDelegates.  This will be fixed once we call CloseSoon() directly on
-  // Infobars.
-  RemoveAllInfoBars(false);
+  // Need to tear down infobars before the TabContents goes away.
+  infobar_tab_helper_.reset();
 }
 
 PropertyAccessor<TabContentsWrapper*>* TabContentsWrapper::property_accessor() {
@@ -240,6 +342,12 @@ void TabContentsWrapper::RegisterUserPrefs(PrefService* prefs) {
   prefs->RegisterBooleanPref(prefs::kWebkitTabsToLinks,
                              pref_defaults.tabs_to_links,
                              PrefService::UNSYNCABLE_PREF);
+  prefs->RegisterBooleanPref(prefs::kWebKitAllowRunningInsecureContent,
+                             false,
+                             PrefService::UNSYNCABLE_PREF);
+  prefs->RegisterBooleanPref(prefs::kWebKitAllowDisplayingInsecureContent,
+                             true,
+                             PrefService::UNSYNCABLE_PREF);
 
 #if !defined(OS_MACOSX)
   prefs->RegisterLocalizedStringPref(prefs::kAcceptLanguages,
@@ -272,6 +380,34 @@ void TabContentsWrapper::RegisterUserPrefs(PrefService* prefs) {
   prefs->RegisterLocalizedStringPref(prefs::kWebKitFantasyFontFamily,
                                      IDS_FANTASY_FONT_FAMILY,
                                      PrefService::UNSYNCABLE_PREF);
+
+  // Register per-script font prefs that have defaults.
+#if defined(OS_CHROMEOS) || defined(OS_WIN)
+  // As explained by its definition, kPerScriptFontDefaultsLength is only
+  // defined for platforms where it would be non-zero.
+  std::string locale = g_browser_process->GetApplicationLocale();
+  for (size_t i = 0; i < kPerScriptFontDefaultsLength; ++i) {
+    const PerScriptFontDefault& pref = kPerScriptFontDefaults[i];
+    // Suppress default per-script font when the script matches the browser's
+    // locale.  Otherwise, the default would override the user's preferences
+    // when viewing pages in their native language.  This can be removed when
+    // per-script fonts are added to Preferences UI.
+    if (!StartsWithASCII(locale, pref.native_locale, false)) {
+      prefs->RegisterLocalizedStringPref(pref.pref_name,
+                                         pref.resource_id,
+                                         PrefService::UNSYNCABLE_PREF);
+    }
+  }
+#endif
+
+  // Register the rest of the per-script font prefs.
+  RegisterFontFamilyMap(prefs, prefs::kWebKitStandardFontFamilyMap);
+  RegisterFontFamilyMap(prefs, prefs::kWebKitFixedFontFamilyMap);
+  RegisterFontFamilyMap(prefs, prefs::kWebKitSerifFontFamilyMap);
+  RegisterFontFamilyMap(prefs, prefs::kWebKitSansSerifFontFamilyMap);
+  RegisterFontFamilyMap(prefs, prefs::kWebKitCursiveFontFamilyMap);
+  RegisterFontFamilyMap(prefs, prefs::kWebKitFantasyFontFamilyMap);
+
   prefs->RegisterLocalizedIntegerPref(prefs::kWebKitDefaultFontSize,
                                       IDS_DEFAULT_FONT_SIZE,
                                       PrefService::UNSYNCABLE_PREF);
@@ -301,11 +437,14 @@ string16 TabContentsWrapper::GetDefaultTitle() {
 
 string16 TabContentsWrapper::GetStatusText() const {
   if (!tab_contents()->IsLoading() ||
-      tab_contents()->load_state() == net::LOAD_STATE_IDLE) {
+      tab_contents()->load_state().state == net::LOAD_STATE_IDLE) {
     return string16();
   }
 
-  switch (tab_contents()->load_state()) {
+  switch (tab_contents()->load_state().state) {
+    case net::LOAD_STATE_WAITING_FOR_DELEGATE:
+      return l10n_util::GetStringFUTF16(IDS_LOAD_STATE_WAITING_FOR_DELEGATE,
+                                        tab_contents()->load_state().param);
     case net::LOAD_STATE_WAITING_FOR_CACHE:
       return l10n_util::GetStringUTF16(IDS_LOAD_STATE_WAITING_FOR_CACHE);
     case net::LOAD_STATE_ESTABLISHING_PROXY_TUNNEL:
@@ -349,7 +488,7 @@ TabContentsWrapper* TabContentsWrapper::Clone() {
 }
 
 void TabContentsWrapper::CaptureSnapshot() {
-  Send(new ViewMsg_CaptureSnapshot(routing_id()));
+  Send(new ChromeViewMsg_CaptureSnapshot(routing_id()));
 }
 
 // static
@@ -370,6 +509,10 @@ const TabContentsWrapper* TabContentsWrapper::GetCurrentWrapperForContents(
   return wrapper ? *wrapper : NULL;
 }
 
+Profile* TabContentsWrapper::profile() const {
+  return Profile::FromBrowserContext(tab_contents()->browser_context());
+}
+
 ////////////////////////////////////////////////////////////////////////////////
 // TabContentsWrapper implementation:
 
@@ -378,8 +521,6 @@ void TabContentsWrapper::RenderViewCreated(RenderViewHost* render_view_host) {
 }
 
 void TabContentsWrapper::RenderViewGone() {
-  RemoveAllInfoBars(true);
-
   // Tell the view that we've crashed so it can prepare the sad tab page.
   // Only do this if we're not in browser shutdown, so that TabContents
   // objects that are not in a browser (e.g., HTML dialogs) and thus are
@@ -398,16 +539,9 @@ void TabContentsWrapper::DidBecomeSelected() {
 bool TabContentsWrapper::OnMessageReceived(const IPC::Message& message) {
   bool handled = true;
   IPC_BEGIN_MESSAGE_MAP(TabContentsWrapper, message)
-    IPC_MESSAGE_HANDLER(ViewHostMsg_JSOutOfMemory, OnJSOutOfMemory)
-    IPC_MESSAGE_HANDLER(ViewHostMsg_RegisterProtocolHandler,
-                        OnRegisterProtocolHandler)
-    IPC_MESSAGE_HANDLER(ViewHostMsg_Snapshot, OnSnapshot)
-    IPC_MESSAGE_HANDLER(ViewHostMsg_PDFHasUnsupportedFeature,
+    IPC_MESSAGE_HANDLER(ChromeViewHostMsg_Snapshot, OnSnapshot)
+    IPC_MESSAGE_HANDLER(ChromeViewHostMsg_PDFHasUnsupportedFeature,
                         OnPDFHasUnsupportedFeature)
-    IPC_MESSAGE_HANDLER(ViewHostMsg_DidBlockDisplayingInsecureContent,
-                        OnDidBlockDisplayingInsecureContent)
-    IPC_MESSAGE_HANDLER(ViewHostMsg_DidBlockRunningInsecureContent,
-                        OnDidBlockRunningInsecureContent)
     IPC_MESSAGE_UNHANDLED(handled = false)
   IPC_END_MESSAGE_MAP()
   return handled;
@@ -424,24 +558,6 @@ void TabContentsWrapper::Observe(int type,
                                  const NotificationSource& source,
                                  const NotificationDetails& details) {
   switch (type) {
-    case content::NOTIFICATION_NAV_ENTRY_COMMITTED: {
-      DCHECK(&tab_contents_->controller() ==
-             Source<NavigationController>(source).ptr());
-
-      content::LoadCommittedDetails& committed_details =
-          *(Details<content::LoadCommittedDetails>(details).ptr());
-
-      // NOTE: It is not safe to change the following code to count upwards or
-      // use iterators, as the RemoveInfoBar() call synchronously modifies our
-      // delegate list.
-      for (size_t i = infobars_.size(); i > 0; --i) {
-        InfoBarDelegate* delegate = GetInfoBarDelegateAt(i - 1);
-        if (delegate->ShouldExpire(committed_details))
-          RemoveInfoBar(delegate);
-      }
-
-      break;
-    }
     case chrome::NOTIFICATION_GOOGLE_URL_UPDATED:
       UpdateAlternateErrorPageURL(render_view_host());
       break;
@@ -479,104 +595,9 @@ void TabContentsWrapper::Observe(int type,
   }
 }
 
-void TabContentsWrapper::AddInfoBar(InfoBarDelegate* delegate) {
-  if (!infobars_enabled_) {
-    delegate->InfoBarClosed();
-    return;
-  }
-
-  for (size_t i = 0; i < infobars_.size(); ++i) {
-    if (GetInfoBarDelegateAt(i)->EqualsDelegate(delegate)) {
-      delegate->InfoBarClosed();
-      return;
-    }
-  }
-
-  infobars_.push_back(delegate);
-  NotificationService::current()->Notify(
-      chrome::NOTIFICATION_TAB_CONTENTS_INFOBAR_ADDED,
-      Source<TabContentsWrapper>(this), Details<InfoBarAddedDetails>(delegate));
-
-  // Add ourselves as an observer for navigations the first time a delegate is
-  // added. We use this notification to expire InfoBars that need to expire on
-  // page transitions.
-  if (infobars_.size() == 1) {
-    registrar_.Add(this, content::NOTIFICATION_NAV_ENTRY_COMMITTED,
-                   Source<NavigationController>(&tab_contents_->controller()));
-  }
-}
-
-void TabContentsWrapper::RemoveInfoBar(InfoBarDelegate* delegate) {
-  RemoveInfoBarInternal(delegate, true);
-}
-
-void TabContentsWrapper::ReplaceInfoBar(InfoBarDelegate* old_delegate,
-                                        InfoBarDelegate* new_delegate) {
-  if (!infobars_enabled_) {
-    AddInfoBar(new_delegate);  // Deletes the delegate.
-    return;
-  }
-
-  size_t i;
-  for (i = 0; i < infobars_.size(); ++i) {
-    if (GetInfoBarDelegateAt(i) == old_delegate)
-      break;
-  }
-  DCHECK_LT(i, infobars_.size());
-
-  infobars_.insert(infobars_.begin() + i, new_delegate);
-
-  old_delegate->clear_owner();
-  InfoBarReplacedDetails replaced_details(old_delegate, new_delegate);
-  NotificationService::current()->Notify(
-      chrome::NOTIFICATION_TAB_CONTENTS_INFOBAR_REPLACED,
-      Source<TabContentsWrapper>(this),
-      Details<InfoBarReplacedDetails>(&replaced_details));
-
-  infobars_.erase(infobars_.begin() + i + 1);
-}
-
-InfoBarDelegate* TabContentsWrapper::GetInfoBarDelegateAt(size_t index) {
-  return infobars_[index];
-}
-
 ////////////////////////////////////////////////////////////////////////////////
 // Internal helpers
 
-void TabContentsWrapper::OnJSOutOfMemory() {
-  AddInfoBar(new SimpleAlertInfoBarDelegate(tab_contents(),
-      NULL, l10n_util::GetStringUTF16(IDS_JS_OUT_OF_MEMORY_PROMPT), true));
-}
-
-void TabContentsWrapper::OnRegisterProtocolHandler(const std::string& protocol,
-                                                   const GURL& url,
-                                                   const string16& title) {
-  if (profile()->IsOffTheRecord())
-    return;
-
-  ChildProcessSecurityPolicy* policy =
-      ChildProcessSecurityPolicy::GetInstance();
-  if (policy->IsPseudoScheme(protocol) || policy->IsDisabledScheme(protocol))
-    return;
-
-  ProtocolHandler handler =
-      ProtocolHandler::CreateProtocolHandler(protocol, url, title);
-
-  ProtocolHandlerRegistry* registry = profile()->GetProtocolHandlerRegistry();
-  if (!registry->enabled() || registry->IsRegistered(handler) ||
-      registry->IsIgnored(handler))
-    return;
-
-  if (!handler.IsEmpty() &&
-      registry->CanSchemeBeOverridden(handler.protocol())) {
-    UserMetrics::RecordAction(
-        UserMetricsAction("RegisterProtocolHandler.InfoBar_Shown"));
-    AddInfoBar(new RegisterProtocolHandlerInfoBarDelegate(tab_contents(),
-                                                          registry,
-                                                          handler));
-  }
-}
-
 void TabContentsWrapper::OnSnapshot(const SkBitmap& bitmap) {
   NotificationService::current()->Notify(
       chrome::NOTIFICATION_TAB_SNAPSHOT_TAKEN,
@@ -588,33 +609,6 @@ void TabContentsWrapper::OnPDFHasUnsupportedFeature() {
   PDFHasUnsupportedFeature(this);
 }
 
-void TabContentsWrapper::OnDidBlockDisplayingInsecureContent() {
-  // At most one infobar and do not supersede the stronger running content bar.
-  for (size_t i = 0; i < infobars_.size(); ++i) {
-    if (GetInfoBarDelegateAt(i)->AsInsecureContentInfoBarDelegate())
-      return;
-  }
-  AddInfoBar(new InsecureContentInfoBarDelegate(this,
-      InsecureContentInfoBarDelegate::DISPLAY));
-}
-
-void TabContentsWrapper::OnDidBlockRunningInsecureContent() {
-  // At most one infobar superseding any weaker displaying content bar.
-  for (size_t i = 0; i < infobars_.size(); ++i) {
-    InsecureContentInfoBarDelegate* delegate =
-        GetInfoBarDelegateAt(i)->AsInsecureContentInfoBarDelegate();
-    if (delegate) {
-      if (delegate->type() != InsecureContentInfoBarDelegate::RUN) {
-        ReplaceInfoBar(delegate, new InsecureContentInfoBarDelegate(this,
-            InsecureContentInfoBarDelegate::RUN));
-      }
-      return;
-    }
-  }
-  AddInfoBar(new InsecureContentInfoBarDelegate(this,
-      InsecureContentInfoBarDelegate::RUN));
-}
-
 GURL TabContentsWrapper::GetAlternateErrorPageURL() const {
   GURL url;
   // Disable alternate error pages when in Incognito mode.
@@ -660,41 +654,11 @@ void TabContentsWrapper::UpdateSafebrowsingDetectionHost() {
     safebrowsing_detection_host_.reset();
   }
   render_view_host()->Send(
-      new ViewMsg_SetClientSidePhishingDetection(routing_id(), safe_browsing));
+      new ChromeViewMsg_SetClientSidePhishingDetection(routing_id(),
+                                                       safe_browsing));
 #endif
 }
 
-void TabContentsWrapper::RemoveInfoBarInternal(InfoBarDelegate* delegate,
-                                               bool animate) {
-  if (!infobars_enabled_) {
-    DCHECK(infobars_.empty());
-    return;
-  }
-
-  size_t i;
-  for (i = 0; i < infobars_.size(); ++i) {
-    if (GetInfoBarDelegateAt(i) == delegate)
-      break;
-  }
-  DCHECK_LT(i, infobars_.size());
-  InfoBarDelegate* infobar = infobars_[i];
-
-  infobar->clear_owner();
-  InfoBarRemovedDetails removed_details(infobar, animate);
-  NotificationService::current()->Notify(
-      chrome::NOTIFICATION_TAB_CONTENTS_INFOBAR_REMOVED,
-      Source<TabContentsWrapper>(this),
-      Details<InfoBarRemovedDetails>(&removed_details));
-
-  infobars_.erase(infobars_.begin() + i);
-  // Remove ourselves as an observer if we are tracking no more InfoBars.
-  if (infobars_.empty()) {
-    registrar_.Remove(this, content::NOTIFICATION_NAV_ENTRY_COMMITTED,
-        Source<NavigationController>(&tab_contents_->controller()));
-  }
-}
-
-void TabContentsWrapper::RemoveAllInfoBars(bool animate) {
-  while (!infobars_.empty())
-    RemoveInfoBarInternal(GetInfoBarDelegateAt(infobar_count() - 1), animate);
+void TabContentsWrapper::ExitFullscreenMode() {
+  Send(new ViewMsg_ExitFullscreen(routing_id()));
 }
