@@ -10,23 +10,25 @@
 #include "chrome/browser/bookmarks/bookmark_folder_editor_controller.h"
 #include "chrome/browser/bookmarks/bookmark_model.h"
 #include "chrome/browser/bookmarks/bookmark_utils.h"
+#include "chrome/browser/prefs/incognito_mode_prefs.h"
 #include "chrome/browser/prefs/pref_service.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_list.h"
-#include "content/browser/user_metrics.h"
 #include "chrome/common/pref_names.h"
 #include "content/browser/tab_contents/page_navigator.h"
+#include "content/browser/user_metrics.h"
 #include "grit/generated_resources.h"
+#include "views/widget/widget.h"
 
 BookmarkContextMenuControllerViews::BookmarkContextMenuControllerViews(
-    gfx::NativeWindow parent_window,
+    views::Widget* parent_widget,
     BookmarkContextMenuControllerViewsDelegate* delegate,
     Profile* profile,
     PageNavigator* navigator,
     const BookmarkNode* parent,
     const std::vector<const BookmarkNode*>& selection)
-    : parent_window_(parent_window),
+    : parent_widget_(parent_widget),
       delegate_(delegate),
       profile_(profile),
       navigator_(navigator),
@@ -46,18 +48,18 @@ BookmarkContextMenuControllerViews::~BookmarkContextMenuControllerViews() {
 void BookmarkContextMenuControllerViews::BuildMenu() {
   if (selection_.size() == 1 && selection_[0]->is_url()) {
     delegate_->AddItemWithStringId(IDC_BOOKMARK_BAR_OPEN_ALL,
-                                   IDS_BOOMARK_BAR_OPEN_IN_NEW_TAB);
+                                   IDS_BOOKMARK_BAR_OPEN_IN_NEW_TAB);
     delegate_->AddItemWithStringId(IDC_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW,
-                                   IDS_BOOMARK_BAR_OPEN_IN_NEW_WINDOW);
+                                   IDS_BOOKMARK_BAR_OPEN_IN_NEW_WINDOW);
     delegate_->AddItemWithStringId(IDC_BOOKMARK_BAR_OPEN_ALL_INCOGNITO,
-                                   IDS_BOOMARK_BAR_OPEN_INCOGNITO);
+                                   IDS_BOOKMARK_BAR_OPEN_INCOGNITO);
   } else {
     delegate_->AddItemWithStringId(IDC_BOOKMARK_BAR_OPEN_ALL,
-                                   IDS_BOOMARK_BAR_OPEN_ALL);
+                                   IDS_BOOKMARK_BAR_OPEN_ALL);
     delegate_->AddItemWithStringId(IDC_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW,
-                                   IDS_BOOMARK_BAR_OPEN_ALL_NEW_WINDOW);
+                                   IDS_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW);
     delegate_->AddItemWithStringId(IDC_BOOKMARK_BAR_OPEN_ALL_INCOGNITO,
-                                   IDS_BOOMARK_BAR_OPEN_ALL_INCOGNITO);
+                                   IDS_BOOKMARK_BAR_OPEN_ALL_INCOGNITO);
   }
 
   delegate_->AddSeparator();
@@ -80,14 +82,14 @@ void BookmarkContextMenuControllerViews::BuildMenu() {
 
   delegate_->AddSeparator();
   delegate_->AddItemWithStringId(IDC_BOOKMARK_BAR_ADD_NEW_BOOKMARK,
-                                 IDS_BOOMARK_BAR_ADD_NEW_BOOKMARK);
+                                 IDS_BOOKMARK_BAR_ADD_NEW_BOOKMARK);
   delegate_->AddItemWithStringId(IDC_BOOKMARK_BAR_NEW_FOLDER,
-                                 IDS_BOOMARK_BAR_NEW_FOLDER);
+                                 IDS_BOOKMARK_BAR_NEW_FOLDER);
 
   delegate_->AddSeparator();
   delegate_->AddItemWithStringId(IDC_BOOKMARK_MANAGER, IDS_BOOKMARK_MANAGER);
   delegate_->AddCheckboxItem(IDC_BOOKMARK_BAR_ALWAYS_SHOW,
-                             IDS_BOOMARK_BAR_ALWAYS_SHOW);
+                             IDS_SHOW_BOOKMARK_BAR);
 }
 
 void BookmarkContextMenuControllerViews::ExecuteCommand(int id) {
@@ -111,7 +113,8 @@ void BookmarkContextMenuControllerViews::ExecuteCommand(int id) {
         UserMetrics::RecordAction(
             UserMetricsAction("BookmarkBar_ContextMenu_OpenAllIncognito"));
       }
-      bookmark_utils::OpenAll(parent_window_, profile_, navigator_, selection_,
+      bookmark_utils::OpenAll(parent_widget_->GetNativeWindow(),
+                              profile_, navigator_, selection_,
                               initial_disposition);
       break;
     }
@@ -127,12 +130,18 @@ void BookmarkContextMenuControllerViews::ExecuteCommand(int id) {
       }
 
       if (selection_[0]->is_url()) {
-        BookmarkEditor::Show(parent_window_, profile_, parent_,
-                             BookmarkEditor::EditDetails(selection_[0]),
-                             BookmarkEditor::SHOW_TREE);
+#if defined(WEBUI_DIALOGS)
+        Browser* browser = BrowserList::GetLastActiveWithProfile(profile_);
+        DCHECK(browser);
+        browser->OpenBookmarkManagerEditNode(selection_[0]->id());
+#else
+        BookmarkEditor::Show(parent_widget_->GetNativeWindow(), profile_,
+            parent_, BookmarkEditor::EditDetails(selection_[0]),
+            BookmarkEditor::SHOW_TREE);
+#endif
       } else {
-        BookmarkFolderEditorController::Show(profile_, parent_window_,
-            selection_[0], -1,
+        BookmarkFolderEditorController::Show(profile_,
+            parent_widget_->GetNativeWindow(), selection_[0], -1,
             BookmarkFolderEditorController::EXISTING_BOOKMARK);
       }
       break;
@@ -155,11 +164,17 @@ void BookmarkContextMenuControllerViews::ExecuteCommand(int id) {
       UserMetrics::RecordAction(
           UserMetricsAction("BookmarkBar_ContextMenu_Add"));
 
+#if defined(WEBUI_DIALOGS)
+      Browser* browser = BrowserList::GetLastActiveWithProfile(profile_);
+      DCHECK(browser);
+      browser->OpenBookmarkManagerAddNodeIn(selection_[0]->id());
+#else
       // TODO: this should honor the index from GetParentForNewNodes.
       BookmarkEditor::Show(
-          parent_window_, profile_,
+          parent_widget_->GetNativeWindow(), profile_,
           bookmark_utils::GetParentForNewNodes(parent_, selection_, NULL),
           BookmarkEditor::EditDetails(), BookmarkEditor::SHOW_TREE);
+#endif
       break;
     }
 
@@ -169,8 +184,9 @@ void BookmarkContextMenuControllerViews::ExecuteCommand(int id) {
       int index;
       const BookmarkNode* parent =
           bookmark_utils::GetParentForNewNodes(parent_, selection_, &index);
-      BookmarkFolderEditorController::Show(profile_, parent_window_, parent,
-          index, BookmarkFolderEditorController::NEW_BOOKMARK);
+      BookmarkFolderEditorController::Show(profile_,
+           parent_widget_->GetNativeWindow(), parent, index,
+           BookmarkFolderEditorController::NEW_BOOKMARK);
       break;
     }
 
@@ -231,18 +247,23 @@ bool BookmarkContextMenuControllerViews::IsCommandEnabled(int id) const {
        selection_[0]->parent() == model_->root_node());
   bool can_edit =
       profile_->GetPrefs()->GetBoolean(prefs::kEditBookmarksEnabled);
+  IncognitoModePrefs::Availability incognito_avail =
+      IncognitoModePrefs::GetAvailability(profile_->GetPrefs());
   switch (id) {
     case IDC_BOOKMARK_BAR_OPEN_INCOGNITO:
       return !profile_->IsOffTheRecord() &&
-             profile_->GetPrefs()->GetBoolean(prefs::kIncognitoEnabled);
+             incognito_avail != IncognitoModePrefs::DISABLED;
 
     case IDC_BOOKMARK_BAR_OPEN_ALL_INCOGNITO:
-      return HasURLs() && !profile_->IsOffTheRecord() &&
-             profile_->GetPrefs()->GetBoolean(prefs::kIncognitoEnabled);
+      return HasURLs() &&
+             !profile_->IsOffTheRecord() &&
+             incognito_avail != IncognitoModePrefs::DISABLED;
 
     case IDC_BOOKMARK_BAR_OPEN_ALL:
-    case IDC_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW:
       return HasURLs();
+    case IDC_BOOKMARK_BAR_OPEN_ALL_NEW_WINDOW:
+      return HasURLs() &&
+             incognito_avail != IncognitoModePrefs::FORCED;
 
     case IDC_BOOKMARK_BAR_RENAME_FOLDER:
     case IDC_BOOKMARK_BAR_EDIT:
