@@ -10,7 +10,6 @@
 
 #include "chrome/browser/bookmarks/base_bookmark_model_observer.h"
 #include "chrome/browser/bookmarks/bookmark_node_data.h"
-#include "ui/gfx/native_widget_types.h"
 #include "views/controls/menu/menu_delegate.h"
 #include "views/controls/menu/menu_item_view.h"
 
@@ -30,6 +29,8 @@ class OSExchangeData;
 
 namespace views {
 class MenuButton;
+class MenuRunner;
+class Widget;
 }  // namespace views
 
 // BookmarkMenuController is responsible for showing a menu of bookmarks,
@@ -52,17 +53,12 @@ class BookmarkMenuController : public BaseBookmarkModelObserver,
   // at |start_child_index|.
   BookmarkMenuController(Profile* profile,
                          PageNavigator* page_navigator,
-                         gfx::NativeWindow parent,
+                         views::Widget* parent,
                          const BookmarkNode* node,
                          int start_child_index);
 
   void RunMenuAt(BookmarkBarView* bookmark_bar, bool for_drop);
 
-  // Shows the menu.
-  void RunMenuAt(views::MenuButton* button,
-                 views::MenuItemView::AnchorPosition position,
-                 bool for_drop);
-
   // Hides the menu.
   void Cancel();
 
@@ -122,6 +118,8 @@ class BookmarkMenuController : public BaseBookmarkModelObserver,
   // BookmarkMenuController deletes itself as necessary.
   virtual ~BookmarkMenuController();
 
+  scoped_ptr<views::MenuRunner> menu_runner_;
+
   scoped_ptr<BookmarkMenuDelegate> menu_delegate_;
 
   // The node we're showing the contents of.
