@@ -7,9 +7,11 @@
 #include "chrome/browser/content_settings/host_content_settings_map.h"
 #include "chrome/browser/content_settings/tab_specific_content_settings.h"
 #include "chrome/browser/cookies_tree_model.h"
+#include "chrome/browser/infobars/infobar_tab_helper.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/collected_cookies_infobar_delegate.h"
 #include "chrome/browser/ui/tab_contents/tab_contents_wrapper.h"
+#include "chrome/browser/ui/views/constrained_window_views.h"
 #include "chrome/browser/ui/views/cookie_info_view.h"
 #include "chrome/common/chrome_notification_types.h"
 #include "content/common/notification_details.h"
@@ -34,9 +36,9 @@ namespace browser {
 
 // Declared in browser_dialogs.h so others don't have to depend on our header.
 void ShowCollectedCookiesDialog(gfx::NativeWindow parent_window,
-                                TabContents* tab_contents) {
+                                TabContentsWrapper* wrapper) {
   // Deletes itself on close.
-  new CollectedCookiesWin(parent_window, tab_contents);
+  new CollectedCookiesWin(parent_window, wrapper);
 }
 
 }  // namespace browser
@@ -162,8 +164,8 @@ class InfobarView : public views::View {
 // CollectedCookiesWin, constructor and destructor:
 
 CollectedCookiesWin::CollectedCookiesWin(gfx::NativeWindow parent_window,
-                                         TabContents* tab_contents)
-    : tab_contents_(tab_contents),
+                                         TabContentsWrapper* wrapper)
+    : wrapper_(wrapper),
       allowed_label_(NULL),
       blocked_label_(NULL),
       allowed_cookies_tree_(NULL),
@@ -173,15 +175,13 @@ CollectedCookiesWin::CollectedCookiesWin(gfx::NativeWindow parent_window,
       for_session_blocked_button_(NULL),
       infobar_(NULL),
       status_changed_(false) {
-  TabSpecificContentSettings* content_settings =
-      TabContentsWrapper::GetCurrentWrapperForContents(tab_contents)->
-          content_settings();
+  TabSpecificContentSettings* content_settings = wrapper->content_settings();
   registrar_.Add(this, chrome::NOTIFICATION_COLLECTED_COOKIES_SHOWN,
                  Source<TabSpecificContentSettings>(content_settings));
 
   Init();
 
-  window_ = tab_contents_->CreateConstrainedDialog(this);
+  window_ = new ConstrainedWindowViews(wrapper->tab_contents(), this);
 }
 
 CollectedCookiesWin::~CollectedCookiesWin() {
@@ -236,9 +236,7 @@ void CollectedCookiesWin::Init() {
 }
 
 views::View* CollectedCookiesWin::CreateAllowedPane() {
-  TabSpecificContentSettings* content_settings =
-      TabContentsWrapper::GetCurrentWrapperForContents(tab_contents_)->
-          content_settings();
+  TabSpecificContentSettings* content_settings = wrapper_->content_settings();
 
   // Create the controls that go into the pane.
   allowed_label_ = new views::Label(UTF16ToWide(l10n_util::GetStringUTF16(
@@ -286,12 +284,10 @@ views::View* CollectedCookiesWin::CreateAllowedPane() {
 }
 
 views::View* CollectedCookiesWin::CreateBlockedPane() {
-  TabSpecificContentSettings* content_settings =
-      TabContentsWrapper::GetCurrentWrapperForContents(tab_contents_)->
-          content_settings();
+  TabSpecificContentSettings* content_settings = wrapper_->content_settings();
 
   HostContentSettingsMap* host_content_settings_map =
-      tab_contents_->profile()->GetHostContentSettingsMap();
+      wrapper_->profile()->GetHostContentSettingsMap();
 
   // Create the controls that go into the pane.
   blocked_label_ = new views::Label(
@@ -355,7 +351,7 @@ views::View* CollectedCookiesWin::CreateBlockedPane() {
 }
 
 ///////////////////////////////////////////////////////////////////////////////
-// ConstrainedDialogDelegate implementation.
+// views::DialogDelegate implementation.
 
 std::wstring CollectedCookiesWin::GetWindowTitle() const {
   return UTF16ToWide(
@@ -377,8 +373,8 @@ void CollectedCookiesWin::DeleteDelegate() {
 
 bool CollectedCookiesWin::Cancel() {
   if (status_changed_) {
-    TabContentsWrapper::GetCurrentWrapperForContents(tab_contents_)->AddInfoBar(
-        new CollectedCookiesInfoBarDelegate(tab_contents_));
+    wrapper_->infobar_tab_helper()->AddInfoBar(
+        new CollectedCookiesInfoBarDelegate(wrapper_->tab_contents()));
   }
 
   return true;
@@ -483,18 +479,25 @@ void CollectedCookiesWin::AddContentException(views::TreeView* tree_view,
                                               ContentSetting setting) {
   CookieTreeOriginNode* origin_node =
       static_cast<CookieTreeOriginNode*>(tree_view->GetSelectedNode());
-  origin_node->CreateContentException(
-      tab_contents_->profile()->GetHostContentSettingsMap(), setting);
+  Profile* profile = wrapper_->profile();
+  origin_node->CreateContentException(profile->GetHostContentSettingsMap(),
+                                      setting);
   infobar_->UpdateVisibility(true, setting, origin_node->GetTitle());
   gfx::Rect bounds = GetWidget()->GetClientAreaScreenBounds();
+#if defined(USE_AURA)
+  // TODO(beng): convert the conversion to use views conversion methods.
+  NOTIMPLEMENTED();
+#else
   // NativeWidgetWin::GetBounds returns the bounds relative to the parent
   // window, while NativeWidgetWin::SetBounds wants screen coordinates. Do the
   // translation here until http://crbug.com/52851 is fixed.
   POINT topleft = {bounds.x(), bounds.y()};
-  MapWindowPoints(HWND_DESKTOP, tab_contents_->GetNativeView(), &topleft, 1);
+  MapWindowPoints(HWND_DESKTOP, wrapper_->tab_contents()->GetNativeView(),
+                  &topleft, 1);
   gfx::Size size = GetWidget()->GetRootView()->GetPreferredSize();
   bounds.SetRect(topleft.x, topleft.y, size.width(), size.height());
   GetWidget()->SetBounds(bounds);
+#endif
   status_changed_ = true;
 }
 
