@@ -4,8 +4,8 @@
 
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/views/dom_view.h"
-#include "chrome/test/in_process_browser_test.h"
-#include "chrome/test/ui_test_utils.h"
+#include "chrome/test/base/in_process_browser_test.h"
+#include "chrome/test/base/ui_test_utils.h"
 #include "views/widget/widget.h"
 
 using views::Widget;
