@@ -24,10 +24,10 @@
 #include "base/string_util.h"
 #include "base/time.h"
 #include "base/timer.h"
-#include "chrome/browser/download/download_item.h"
-#include "chrome/browser/download/download_manager.h"
 #include "chrome/browser/icon_manager.h"
 #include "content/browser/cancelable_request.h"
+#include "content/browser/download/download_item.h"
+#include "content/browser/download/download_manager.h"
 #include "ui/base/animation/animation_delegate.h"
 #include "ui/gfx/font.h"
 #include "views/controls/button/button.h"
