@@ -22,7 +22,6 @@
 #include "chrome/browser/bookmarks/bookmark_utils.h"
 #include "chrome/browser/browser_process.h"
 #include "chrome/browser/debugger/devtools_window.h"
-#include "chrome/browser/download/download_manager.h"
 #include "chrome/browser/extensions/extension_tab_helper.h"
 #include "chrome/browser/extensions/extension_tts_api.h"
 #include "chrome/browser/instant/instant_controller.h"
@@ -56,7 +55,7 @@
 #include "chrome/browser/ui/views/download/download_in_progress_dialog_view.h"
 #include "chrome/browser/ui/views/frame/browser_view_layout.h"
 #include "chrome/browser/ui/views/frame/contents_container.h"
-#include "chrome/browser/ui/views/fullscreen_exit_bubble.h"
+#include "chrome/browser/ui/views/fullscreen_exit_bubble_views.h"
 #include "chrome/browser/ui/views/infobars/infobar_container_view.h"
 #include "chrome/browser/ui/views/location_bar/location_icon_view.h"
 #include "chrome/browser/ui/views/status_bubble_views.h"
@@ -76,6 +75,7 @@
 #include "chrome/common/native_window_notification_source.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/common/url_constants.h"
+#include "content/browser/download/download_manager.h"
 #include "content/browser/renderer_host/render_view_host.h"
 #include "content/browser/renderer_host/render_widget_host_view.h"
 #include "content/browser/tab_contents/tab_contents.h"
@@ -105,7 +105,8 @@
 #include "views/widget/widget.h"
 #include "views/window/dialog_delegate.h"
 
-#if defined(OS_WIN)
+#if defined(USE_AURA)
+#elif defined(OS_WIN)
 #include "chrome/browser/aeropeek_manager.h"
 #include "chrome/browser/jumplist_win.h"
 #include "ui/base/message_box_win.h"
@@ -181,9 +182,9 @@ static gfx::NativeWindow GetNormalBrowserWindowForBrowser(Browser* browser,
 // and paint the bookmark bar.
 class BookmarkExtensionBackground : public views::Background {
  public:
-  explicit BookmarkExtensionBackground(BrowserView* browser_view,
-                                       DetachableToolbarView* host_view,
-                                       Browser* browser);
+  BookmarkExtensionBackground(BrowserView* browser_view,
+                              DetachableToolbarView* host_view,
+                              Browser* browser);
 
   // View methods overridden from views:Background.
   virtual void Paint(gfx::Canvas* canvas, views::View* view) const;
@@ -330,7 +331,7 @@ BrowserView::BrowserView(Browser* browser)
       contents_split_(NULL),
       initialized_(false),
       ignore_layout_(true)
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
       , hung_window_detector_(&hung_plugin_action_),
       ticker_(0)
 #endif
@@ -345,7 +346,7 @@ BrowserView::BrowserView(Browser* browser)
 BrowserView::~BrowserView() {
   browser_->tabstrip_model()->RemoveObserver(this);
 
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   // Remove this observer.
   if (aeropeek_manager_.get())
     browser_->tabstrip_model()->RemoveObserver(aeropeek_manager_.get());
@@ -353,6 +354,12 @@ BrowserView::~BrowserView() {
   // Stop hung plugin monitoring.
   ticker_.Stop();
   ticker_.UnregisterTickHandler(&hung_window_detector_);
+
+  // Terminate the jumplist (must be called before browser_->profile() is
+  // destroyed.
+  if (jumplist_) {
+    jumplist_->Terminate();
+  }
 #endif
 
 #if !defined(OS_CHROMEOS)
@@ -380,7 +387,10 @@ BrowserView::~BrowserView() {
 // static
 BrowserView* BrowserView::GetBrowserViewForNativeWindow(
     gfx::NativeWindow window) {
-#if defined(OS_WIN)
+#if defined(USE_AURA)
+  // TODO(beng):
+  NOTIMPLEMENTED();
+#elif defined(OS_WIN)
   if (IsWindow(window)) {
     return reinterpret_cast<BrowserView*>(
         ui::ViewProp::GetValue(window, kBrowserViewKey));
@@ -478,9 +488,10 @@ bool BrowserView::ShouldShowAvatar() const {
     return true;
 
   if (ProfileManager::IsMultipleProfilesEnabled()) {
-    // TODO(sail): Once the multi-profile options UI is done we only want to
-    // show the avatar if the user has more than one profile.
-    return true;
+    // Show the profile avatar after the user has created more than one profile.
+    ProfileInfoCache& cache =
+        g_browser_process->profile_manager()->GetProfileInfoCache();
+    return cache.GetNumberOfProfiles() > 1;
   }
 
   return false;
@@ -567,7 +578,7 @@ SkBitmap BrowserView::GetOTRAvatarIcon() {
   return *otr_avatar_;
 }
 
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
 void BrowserView::PrepareToRunSystemMenu(HMENU menu) {
   system_menu_->UpdateStates();
 }
@@ -648,7 +659,7 @@ bool BrowserView::IsActive() const {
 }
 
 void BrowserView::FlashFrame() {
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   FLASHWINFO fwi;
   fwi.cbSize = sizeof(fwi);
   fwi.hwnd = frame_->GetNativeWindow();
@@ -752,7 +763,7 @@ void BrowserView::UpdateLoadingAnimations(bool should_animate) {
     if (!loading_animation_timer_.IsRunning()) {
       // Loads are happening, and the timer isn't running, so start it.
       last_animation_time_ = base::TimeTicks::Now();
-      loading_animation_timer_.Start(
+      loading_animation_timer_.Start(FROM_HERE,
           TimeDelta::FromMilliseconds(kLoadingAnimationFrameTimeMs), this,
           &BrowserView::LoadingAnimationCallback);
     }
@@ -782,11 +793,15 @@ bool BrowserView::IsMaximized() const {
   return frame_->IsMaximized();
 }
 
+bool BrowserView::IsMinimized() const {
+  return frame_->IsMinimized();
+}
+
 void BrowserView::SetFullscreen(bool fullscreen) {
   if (IsFullscreen() == fullscreen)
     return;  // Nothing to do.
 
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   ProcessFullscreen(fullscreen);
 #else
   // On Linux changing fullscreen is async. Ask the window to change it's
@@ -1003,7 +1018,7 @@ bool BrowserView::IsToolbarVisible() const {
 }
 
 void BrowserView::DisableInactiveFrame() {
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   frame_->DisableInactiveRendering();
 #endif  // No tricks are needed to get the right behavior on Linux.
 }
@@ -1012,7 +1027,7 @@ void BrowserView::ConfirmSetDefaultSearchProvider(
     TabContents* tab_contents,
     TemplateURL* template_url,
     TemplateURLService* template_url_service) {
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   DefaultSearchView::Show(tab_contents, template_url, template_url_service);
 #else
   // TODO(levin): Implement for other platforms. Right now this is behind
@@ -1127,8 +1142,8 @@ void BrowserView::ShowRepostFormWarningDialog(TabContents* tab_contents) {
   browser::ShowRepostFormWarningDialog(GetNativeHandle(), tab_contents);
 }
 
-void BrowserView::ShowCollectedCookiesDialog(TabContents* tab_contents) {
-  browser::ShowCollectedCookiesDialog(GetNativeHandle(), tab_contents);
+void BrowserView::ShowCollectedCookiesDialog(TabContentsWrapper* wrapper) {
+  browser::ShowCollectedCookiesDialog(GetNativeHandle(), wrapper);
 }
 
 void BrowserView::ShowThemeInstallBubble() {
@@ -1141,11 +1156,11 @@ void BrowserView::ShowThemeInstallBubble() {
 void BrowserView::ConfirmBrowserCloseWithPendingDownloads() {
   DownloadInProgressDialogView* view =
       new DownloadInProgressDialogView(browser_.get());
-  browser::CreateViewsWindow(GetNativeHandle(), gfx::Rect(), view)->Show();
+  browser::CreateViewsWindow(GetNativeHandle(), view)->Show();
 }
 
-void BrowserView::ShowHTMLDialog(HtmlDialogUIDelegate* delegate,
-                                 gfx::NativeWindow parent_window) {
+gfx::NativeWindow BrowserView::ShowHTMLDialog(HtmlDialogUIDelegate* delegate,
+                                              gfx::NativeWindow parent_window) {
   // Default to using our window as the parent if the argument is not specified.
   gfx::NativeWindow parent = parent_window ? parent_window
                                            : GetNativeHandle();
@@ -1153,7 +1168,7 @@ void BrowserView::ShowHTMLDialog(HtmlDialogUIDelegate* delegate,
   parent = GetNormalBrowserWindowForBrowser(browser(), NULL);
 #endif  // defined(OS_CHROMEOS)
 
-  browser::ShowHtmlDialog(parent, browser_.get()->profile(), delegate);
+  return browser::ShowHtmlDialog(parent, browser_.get()->profile(), delegate);
 }
 
 void BrowserView::ShowCreateWebAppShortcutsDialog(
@@ -1198,7 +1213,7 @@ bool BrowserView::PreHandleKeyboardEvent(const NativeWebKeyboardEvent& event,
   if (event.type != WebKit::WebInputEvent::RawKeyDown)
     return false;
 
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   // As Alt+F4 is the close-app keyboard shortcut, it needs processing
   // immediately.
   if (event.windowsKeyCode == ui::VKEY_F4 &&
@@ -1336,7 +1351,7 @@ void BrowserView::ShowInstant(TabContentsWrapper* preview) {
   contents_->SetPreview(preview_container_, preview->tab_contents());
   preview_container_->ChangeTabContents(preview->tab_contents());
 
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   // Removing the fade is instant (on windows). We need to force the preview to
   // draw, otherwise the current page flickers before the new page appears.
   RedrawWindow(preview->tab_contents()->view()->GetContentNativeView(), NULL,
@@ -1371,6 +1386,10 @@ WindowOpenDisposition BrowserView::GetDispositionForPopupBounds(
   return NEW_POPUP;
 }
 
+FindBar* BrowserView::CreateFindBar() {
+  return browser::CreateFindBar(this);
+}
+
 #if defined(OS_CHROMEOS)
 void BrowserView::ShowKeyboardOverlay(gfx::NativeWindow owning_window) {
   KeyboardOverlayDialogView::ShowDialog(owning_window, this);
@@ -1599,7 +1618,7 @@ bool BrowserView::ShouldShowWindowIcon() const {
 
 bool BrowserView::ExecuteWindowsCommand(int command_id) {
   // This function handles WM_SYSCOMMAND, WM_APPCOMMAND, and WM_COMMAND.
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   if (command_id == IDC_DEBUG_FRAME_TOGGLE)
     GetWidget()->DebugToggleFrameType();
 #endif
@@ -1617,19 +1636,24 @@ std::wstring BrowserView::GetWindowName() const {
 }
 
 void BrowserView::SaveWindowPlacement(const gfx::Rect& bounds,
-                                      bool maximized) {
+                                      ui::WindowShowState show_state) {
   // If IsFullscreen() is true, we've just changed into fullscreen mode, and
   // we're catching the going-into-fullscreen sizing and positioning calls,
   // which we want to ignore.
   if (!IsFullscreen() && browser_->ShouldSaveWindowPlacement()) {
-    WidgetDelegate::SaveWindowPlacement(bounds, maximized);
-    browser_->SaveWindowPlacement(bounds, maximized);
+    WidgetDelegate::SaveWindowPlacement(bounds, show_state);
+    browser_->SaveWindowPlacement(bounds, show_state);
   }
 }
 
-bool BrowserView::GetSavedWindowBounds(gfx::Rect* bounds) const {
+bool BrowserView::GetSavedWindowPlacement(
+    gfx::Rect* bounds,
+    ui::WindowShowState* show_state) const {
   *bounds = browser_->GetSavedWindowBounds();
-  if (browser_->is_type_popup() || browser_->is_type_panel()) {
+  *show_state = browser_->GetSavedWindowShowState();
+
+  if ((browser_->is_type_popup() || browser_->is_type_panel())
+      && !browser_->is_devtools()) {
     // We are a popup window. The value passed in |bounds| represents two
     // pieces of information:
     // - the position of the window, in screen coordinates (outer position).
@@ -1666,11 +1690,6 @@ bool BrowserView::GetSavedWindowBounds(gfx::Rect* bounds) const {
   return true;
 }
 
-bool BrowserView::GetSavedMaximizedState(bool* maximized) const {
-  *maximized = browser_->GetSavedMaximizedState();
-  return true;
-}
-
 views::View* BrowserView::GetContentsView() {
   return contents_container_;
 }
@@ -1696,6 +1715,12 @@ void BrowserView::OnWindowBeginUserBoundsChange() {
 }
 
 void BrowserView::OnWidgetMove() {
+  if (!initialized_) {
+    // Creating the widget can trigger a move. Ignore it until we've initialized
+    // things.
+    return;
+  }
+
   // Cancel any tabstrip animations, some of them may be invalidated by the
   // window being repositioned.
   // Comment out for one cycle to see if this fixes dist tests.
@@ -1755,7 +1780,7 @@ bool BrowserView::CanClose() {
 }
 
 int BrowserView::NonClientHitTest(const gfx::Point& point) {
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   // The following code is not in the LayoutManager because it's
   // independent of layout and also depends on the ResizeCorner which
   // is private.
@@ -1818,7 +1843,7 @@ void BrowserView::Layout() {
   // The status bubble position requires that all other layout finish first.
   LayoutStatusBubble();
 
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   // Send the margins of the "user-perceived content area" of this
   // browser window so AeroPeekManager can render a background-tab image in
   // the area.
@@ -1985,13 +2010,13 @@ void BrowserView::Init() {
 
   status_bubble_.reset(new StatusBubbleViews(contents_));
 
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   InitSystemMenu();
 
   // Create a custom JumpList and add it to an observer of TabRestoreService
   // so we can update the custom JumpList when a tab is added or removed.
   if (JumpList::Enabled()) {
-    jumplist_.reset(new JumpList);
+    jumplist_ = new JumpList();
     jumplist_->AddObserver(browser_->profile());
   }
 
@@ -2001,11 +2026,15 @@ void BrowserView::Init() {
     browser_->tabstrip_model()->AddObserver(aeropeek_manager_.get());
   }
 #endif
+  // TODO(stevet): Remove all of this once we clean up compact navigation. Note
+  // that we revert back to checking the pref as we've guaranteed that it will
+  // always return false, regardless of flags.
   // Only create our compact navigation classes if the switch is enabled. Note
   // that we directly check the switch and not the pref as the switch may be on
   // and the pref could be off (currently not selected in the context menu).
-  if (CommandLine::ForCurrentProcess()->HasSwitch(
-      switches::kEnableCompactNavigation)) {
+  // if (CommandLine::ForCurrentProcess()->HasSwitch(
+  //     switches::kEnableCompactNavigation)) {
+  if (UseCompactNavigationBar()) {
     compact_location_bar_view_host_.reset(new CompactLocationBarViewHost(this));
     compact_navigation_bar_ = new CompactNavigationBar(this);
     compact_navigation_bar_->set_id(VIEW_ID_COMPACT_NAV_BAR);
@@ -2054,7 +2083,7 @@ void BrowserView::LoadingAnimationCallback() {
 
 // BrowserView, private --------------------------------------------------------
 
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
 void BrowserView::InitSystemMenu() {
   system_menu_contents_.reset(new views::SystemMenuModel(this));
   // We add the menu items in reverse order so that insertion_index never needs
@@ -2093,8 +2122,7 @@ bool BrowserView::MaybeShowBookmarkBar(TabContentsWrapper* contents) {
   if (browser_->SupportsWindowFeature(Browser::FEATURE_BOOKMARKBAR) &&
       contents) {
     if (!bookmark_bar_view_.get()) {
-      bookmark_bar_view_.reset(new BookmarkBarView(contents->profile(),
-                                                   browser_.get()));
+      bookmark_bar_view_.reset(new BookmarkBarView(browser_.get()));
       bookmark_bar_view_->set_parent_owned(false);
       bookmark_bar_view_->set_background(
           new BookmarkExtensionBackground(this, bookmark_bar_view_.get(),
@@ -2102,8 +2130,6 @@ bool BrowserView::MaybeShowBookmarkBar(TabContentsWrapper* contents) {
       bookmark_bar_view_->SetBookmarkBarState(
           browser_->bookmark_bar_state(),
           BookmarkBar::DONT_ANIMATE_STATE_CHANGE);
-    } else {
-      bookmark_bar_view_->SetProfile(contents->profile());
     }
     bookmark_bar_view_->SetPageNavigator(contents->tab_contents());
     new_bookmark_bar_view = bookmark_bar_view_.get();
@@ -2295,7 +2321,7 @@ void BrowserView::ProcessFullscreen(bool fullscreen) {
   //     thus are slow and look ugly
   ignore_layout_ = true;
   LocationBarView* location_bar = GetLocationBarView();
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   OmniboxViewWin* omnibox_view =
       static_cast<OmniboxViewWin*>(location_bar->location_entry());
 #endif
@@ -2310,7 +2336,7 @@ void BrowserView::ProcessFullscreen(bool fullscreen) {
     if (focus_manager->GetFocusedView() == location_bar)
       focus_manager->ClearFocus();
 
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
     // If we don't hide the edit and force it to not show until we come out of
     // fullscreen, then if the user was on the New Tab Page, the edit contents
     // will appear atop the web contents once we go into fullscreen mode.  This
@@ -2320,13 +2346,13 @@ void BrowserView::ProcessFullscreen(bool fullscreen) {
     ShowWindow(omnibox_view->m_hWnd, SW_HIDE);
 #endif
   }
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   static_cast<views::NativeWidgetWin*>(frame_->native_widget())->
       PushForceHidden();
 #endif
 
   // Toggle fullscreen mode.
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   frame_->SetFullscreen(fullscreen);
 #endif  // No need to invoke SetFullscreen for linux as this code is executed
         // once we're already fullscreen on linux.
@@ -2337,11 +2363,11 @@ void BrowserView::ProcessFullscreen(bool fullscreen) {
     bool is_kiosk =
         CommandLine::ForCurrentProcess()->HasSwitch(switches::kKioskMode);
     if (!is_kiosk) {
-      fullscreen_bubble_.reset(new FullscreenExitBubble(GetWidget(),
-                                                        browser_.get()));
+      fullscreen_bubble_.reset(new FullscreenExitBubbleViews(GetWidget(),
+                                                             browser_.get()));
     }
   } else {
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
     // Show the edit again since we're no longer in fullscreen mode.
     omnibox_view->set_force_hidden(false);
     ShowWindow(omnibox_view->m_hWnd, SW_SHOW);
@@ -2352,7 +2378,7 @@ void BrowserView::ProcessFullscreen(bool fullscreen) {
   // it's in its final position.
   ignore_layout_ = false;
   Layout();
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   static_cast<views::NativeWidgetWin*>(frame_->native_widget())->
       PopForceHidden();
 #endif
@@ -2360,7 +2386,10 @@ void BrowserView::ProcessFullscreen(bool fullscreen) {
 
 
 void BrowserView::LoadAccelerators() {
-#if defined(OS_WIN)
+#if defined(USE_AURA)
+  // TODO(beng):
+  NOTIMPLEMENTED();
+#elif defined(OS_WIN)
   HACCEL accelerator_table = AtlLoadAccelerators(IDR_MAINFRAME);
   DCHECK(accelerator_table);
 
@@ -2410,7 +2439,7 @@ void BrowserView::LoadAccelerators() {
 #endif
 }
 
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
 void BrowserView::BuildSystemMenuForBrowserWindow() {
   system_menu_contents_->AddSeparator();
   system_menu_contents_->AddItemWithStringId(IDC_TASK_MANAGER,
@@ -2473,7 +2502,7 @@ void BrowserView::AddFrameToggleItems() {
 #endif
 
 int BrowserView::GetCommandIDForAppCommandID(int app_command_id) const {
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   switch (app_command_id) {
     // NOTE: The order here matches the APPCOMMAND declaration order in the
     // Windows headers.
@@ -2506,7 +2535,7 @@ int BrowserView::GetCommandIDForAppCommandID(int app_command_id) const {
 }
 
 void BrowserView::InitHangMonitor() {
-#if defined(OS_WIN)
+#if defined(OS_WIN) && !defined(USE_AURA)
   PrefService* pref_service = g_browser_process->local_state();
   if (!pref_service)
     return;
@@ -2643,7 +2672,7 @@ void BrowserView::SetToolbar(ToolbarView* toolbar) {
   toolbar_ = toolbar;
   if (toolbar) {
     AddChildView(toolbar_);
-    toolbar_->Init(browser_->profile());
+    toolbar_->Init();
   }
 }
 
@@ -2659,8 +2688,3 @@ BrowserWindow* BrowserWindow::CreateBrowserWindow(Browser* browser) {
   return view;
 }
 #endif
-
-// static
-FindBar* BrowserWindow::CreateFindBar(Browser* browser) {
-  return browser::CreateFindBar(static_cast<BrowserView*>(browser->window()));
-}
