@@ -85,15 +85,10 @@ GlassBrowserFrameView::GlassBrowserFrameView(BrowserFrame* frame,
   if (browser_view_->ShouldShowWindowIcon())
     InitThrobberIcons();
 
-  if (browser_view_->ShouldShowAvatar()) {
-    avatar_button_.reset(new AvatarMenuButton(
-        browser_view_->browser(), !browser_view_->IsOffTheRecord()));
-    AddChildView(avatar_button_.get());
-    UpdateAvatarInfo();
-    if (!browser_view_->IsOffTheRecord()) {
-      registrar_.Add(this, chrome::NOTIFICATION_PROFILE_CACHED_INFO_CHANGED,
-                     NotificationService::AllSources());
-    }
+  UpdateAvatarInfo();
+  if (!browser_view_->IsOffTheRecord()) {
+    registrar_.Add(this, chrome::NOTIFICATION_PROFILE_CACHED_INFO_CHANGED,
+                   NotificationService::AllSources());
   }
 }
 
@@ -551,7 +546,6 @@ void GlassBrowserFrameView::Observe(int type,
   switch (type) {
     case chrome::NOTIFICATION_PROFILE_CACHED_INFO_CHANGED:
       UpdateAvatarInfo();
-      LayoutAvatar();
       break;
     default:
       NOTREACHED() << "Got a notification we didn't register for!";
@@ -573,6 +567,22 @@ void GlassBrowserFrameView::InitThrobberIcons() {
 }
 
 void GlassBrowserFrameView::UpdateAvatarInfo() {
+  if (browser_view_->ShouldShowAvatar()) {
+    if (!avatar_button_.get()) {
+      avatar_button_.reset(new AvatarMenuButton(
+          browser_view_->browser(), !browser_view_->IsOffTheRecord()));
+      AddChildView(avatar_button_.get());
+      frame_->GetRootView()->Layout();
+    }
+  } else if (avatar_button_.get()) {
+    RemoveChildView(avatar_button_.get());
+    avatar_button_.reset();
+    frame_->GetRootView()->Layout();
+  }
+
+  if (!avatar_button_.get())
+    return;
+
   if (browser_view_->IsOffTheRecord()) {
     avatar_button_->SetIcon(browser_view_->GetOTRAvatarIcon());
   } else {
