@@ -28,6 +28,7 @@
 #include "chrome/browser/command_updater.h"
 #include "chrome/browser/net/url_fixer_upper.h"
 #include "chrome/browser/profiles/profile.h"
+#include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/views/location_bar/location_bar_view.h"
 #include "chrome/common/chrome_notification_types.h"
 #include "content/browser/tab_contents/tab_contents.h"
@@ -47,12 +48,12 @@
 #include "ui/base/keycodes/keyboard_codes.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/l10n/l10n_util_win.h"
+#include "ui/base/win/mouse_wheel_util.h"
 #include "ui/gfx/canvas.h"
 #include "ui/gfx/canvas_skia.h"
 #include "views/controls/textfield/native_textfield_win.h"
 #include "views/drag_utils.h"
 #include "views/events/event_utils_win.h"
-#include "views/focus/focus_util_win.h"
 #include "views/widget/widget.h"
 
 #pragma comment(lib, "oleacc.lib")  // Needed for accessibility support.
@@ -395,13 +396,13 @@ OmniboxViewWin::OmniboxViewWin(const gfx::Font& font,
                                ToolbarModel* toolbar_model,
                                LocationBarView* parent_view,
                                HWND hwnd,
-                               Profile* profile,
                                CommandUpdater* command_updater,
                                bool popup_window_mode,
-                               const views::View* location_bar)
-    : model_(new AutocompleteEditModel(this, controller, profile)),
+                               views::View* location_bar)
+    : model_(new AutocompleteEditModel(this, controller,
+                                       parent_view->browser()->profile())),
       popup_view_(new AutocompletePopupContentsView(font, this, model_.get(),
-                                                    profile, location_bar)),
+                                                    location_bar)),
       controller_(controller),
       parent_view_(parent_view),
       toolbar_model_(toolbar_model),
@@ -483,10 +484,6 @@ OmniboxViewWin::OmniboxViewWin(const gfx::Font& font,
 }
 
 OmniboxViewWin::~OmniboxViewWin() {
-  NotificationService::current()->Notify(
-      chrome::NOTIFICATION_OMNIBOX_DESTROYED, Source<OmniboxViewWin>(this),
-      NotificationService::NoDetails());
-
   // Explicitly release the text object model now that we're done with it, and
   // before we free the library. If the library gets unloaded before this
   // released, it becomes garbage.
@@ -1816,8 +1813,8 @@ void OmniboxViewWin::OnWindowPosChanging(WINDOWPOS* window_pos) {
 
 BOOL OmniboxViewWin::OnMouseWheel(UINT flags, short delta, CPoint point) {
   // Forward the mouse-wheel message to the window under the mouse.
-  if (!views::RerouteMouseWheel(m_hWnd, MAKEWPARAM(flags, delta),
-                                MAKELPARAM(point.x, point.y)))
+  if (!ui::RerouteMouseWheel(m_hWnd, MAKEWPARAM(flags, delta),
+                             MAKELPARAM(point.x, point.y)))
     SetMsgHandled(false);
   return 0;
 }
