@@ -5,16 +5,21 @@
 #include "chrome/browser/ui/views/tab_contents/native_tab_contents_view_gtk.h"
 
 #include "base/utf_string_conversions.h"
-#include "chrome/browser/renderer_host/render_widget_host_view_gtk.h"
 #include "chrome/browser/tab_contents/web_drag_dest_gtk.h"
 #include "chrome/browser/ui/gtk/constrained_window_gtk.h"
 #include "chrome/browser/ui/gtk/tab_contents_drag_source.h"
 #include "chrome/browser/ui/views/tab_contents/native_tab_contents_view_delegate.h"
 #include "chrome/browser/ui/views/tab_contents/native_tab_contents_view_views.h"
+#include "content/browser/renderer_host/render_widget_host_view_gtk.h"
 #include "content/browser/tab_contents/tab_contents.h"
 #include "content/browser/tab_contents/tab_contents_view.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebDragData.h"
 #include "third_party/WebKit/Source/WebKit/chromium/public/WebInputEvent.h"
+#include "views/views_delegate.h"
+
+#if defined(OS_CHROMEOS)
+#include "chrome/browser/chromeos/login/user_manager.h"
+#endif
 
 using WebKit::WebDragOperation;
 using WebKit::WebDragOperationsMask;
@@ -251,7 +256,15 @@ void NativeTabContentsViewGtk::PositionConstrainedWindows(
     gtk_widget_size_request(widget, &requisition);
 
     int child_x = std::max(half_view_width - (requisition.width / 2), 0);
-    PositionChild(widget, child_x, 0, 0, 0);
+    int child_y = 0;
+
+#if defined(OS_CHROMEOS)
+    // TODO(ivankr): this is a hack to display proxy authentication dialog
+    // centered during the sign-in phase. See http://crosbug/20819.
+    if (!chromeos::UserManager::Get()->user_is_logged_in())
+      child_y = std::max(view_size.height() / 2 - (requisition.height / 2), 0);
+#endif
+    PositionChild(widget, child_x, child_y, 0, 0);
   }
 }
 
@@ -261,8 +274,8 @@ void NativeTabContentsViewGtk::PositionConstrainedWindows(
 // static
 NativeTabContentsView* NativeTabContentsView::CreateNativeTabContentsView(
     internal::NativeTabContentsViewDelegate* delegate) {
-  if (views::Widget::IsPureViews())
+  if (views::Widget::IsPureViews() &&
+      views::ViewsDelegate::views_delegate->GetDefaultParentView())
     return new NativeTabContentsViewViews(delegate);
   return new NativeTabContentsViewGtk(delegate);
 }
-
