@@ -32,11 +32,6 @@ using WebKit::WebDragOperationNone;
 using WebKit::WebDragOperationsMask;
 using WebKit::WebInputEvent;
 
-// static
-TabContentsView* TabContentsView::Create(TabContents* tab_contents) {
-  return new TabContentsViewViews(tab_contents);
-}
-
 TabContentsViewViews::TabContentsViewViews(TabContents* tab_contents)
     : tab_contents_(tab_contents),
       native_tab_contents_view_(NULL),
@@ -103,7 +98,8 @@ gfx::NativeView TabContentsViewViews::GetContentNativeView() const {
 }
 
 gfx::NativeWindow TabContentsViewViews::GetTopLevelNativeWindow() const {
-  return GetTopLevelWidget()->GetNativeWindow();
+  const Widget* toplevel = GetTopLevelWidget();
+  return toplevel ? toplevel->GetNativeWindow() : NULL;
 }
 
 void TabContentsViewViews::GetContainerBounds(gfx::Rect* out) const {
@@ -169,8 +165,10 @@ void TabContentsViewViews::Focus() {
   }
 
   RenderWidgetHostView* rwhv = tab_contents_->GetRenderWidgetHostView();
-  GetFocusManager()->FocusNativeView(rwhv ? rwhv->GetNativeView()
-                                          : GetNativeView());
+  views::FocusManager* focus_manager = GetFocusManager();
+  if (focus_manager)
+    focus_manager->FocusNativeView(rwhv ? rwhv->GetNativeView()
+                                   : GetNativeView());
 }
 
 void TabContentsViewViews::SetInitialFocus() {
@@ -186,8 +184,7 @@ void TabContentsViewViews::StoreFocus() {
   if (view_storage->RetrieveView(last_focused_view_storage_id_) != NULL)
     view_storage->RemoveView(last_focused_view_storage_id_);
 
-  views::FocusManager* focus_manager =
-      views::FocusManager::GetFocusManagerForNativeView(GetNativeView());
+  views::FocusManager* focus_manager = GetFocusManager();
   if (focus_manager) {
     // |focus_manager| can be NULL if the tab has been detached but still
     // exists.
@@ -205,9 +202,7 @@ void TabContentsViewViews::RestoreFocus() {
   if (!last_focused_view) {
     SetInitialFocus();
   } else {
-    views::FocusManager* focus_manager =
-        views::FocusManager::GetFocusManagerForNativeView(GetNativeView());
-
+    views::FocusManager* focus_manager = GetFocusManager();
     // If you hit this DCHECK, please report it to Jay (jcampan).
     DCHECK(focus_manager != NULL) << "No focus manager when restoring focus.";
 
@@ -226,9 +221,6 @@ void TabContentsViewViews::RestoreFocus() {
   }
 }
 
-void TabContentsViewViews::UpdatePreferredSize(const gfx::Size& pref_size) {
-}
-
 bool TabContentsViewViews::IsDoingDrag() const {
   return native_tab_contents_view_->IsDoingDrag();
 }
@@ -264,8 +256,7 @@ void TabContentsViewViews::GotFocus() {
 
 void TabContentsViewViews::TakeFocus(bool reverse) {
   if (!tab_contents_->delegate()->TakeFocus(reverse)) {
-    views::FocusManager* focus_manager =
-        views::FocusManager::GetFocusManagerForNativeView(GetNativeView());
+    views::FocusManager* focus_manager = GetFocusManager();
 
     // We may not have a focus manager if the tab has been switched before this
     // message arrived.
@@ -392,8 +383,8 @@ void TabContentsViewViews::OnNativeTabContentsViewMouseMove(bool motion) {
 
 void TabContentsViewViews::OnNativeTabContentsViewDraggingEnded() {
   if (close_tab_after_drag_ends_) {
-    close_tab_timer_.Start(base::TimeDelta::FromMilliseconds(0), this,
-                           &TabContentsViewViews::CloseTab);
+    close_tab_timer_.Start(FROM_HERE, base::TimeDelta::FromMilliseconds(0),
+                           this, &TabContentsViewViews::CloseTab);
   }
   tab_contents_->SystemDragEnded();
 }
