@@ -458,6 +458,7 @@ void TaskManagerView::Init() {
   kill_button_->AddAccelerator(views::Accelerator(ui::VKEY_E,
                                                   false, false, false));
   kill_button_->SetAccessibleKeyboardShortcut(L"E");
+  kill_button_->set_prefix_type(views::TextButtonBase::PREFIX_SHOW);
   about_memory_link_ = new views::Link(UTF16ToWide(
       l10n_util::GetStringUTF16(IDS_TASK_MANAGER_ABOUT_MEMORY_LINK)));
   about_memory_link_->set_listener(this);
@@ -610,6 +611,7 @@ bool TaskManagerView::CanMaximize() const {
 }
 
 bool TaskManagerView::ExecuteWindowsCommand(int command_id) {
+#if defined(OS_WIN) && !defined(USE_AURA)
   if (command_id == IDC_ALWAYS_ON_TOP) {
     is_always_on_top_ = !is_always_on_top_;
 
@@ -638,6 +640,7 @@ bool TaskManagerView::ExecuteWindowsCommand(int command_id) {
     }
     return true;
   }
+#endif
   return false;
 }
 
@@ -734,6 +737,7 @@ void TaskManagerView::ActivateFocusedTab() {
 }
 
 void TaskManagerView::AddAlwaysOnTopSystemMenuItem() {
+#if defined(OS_WIN) && !defined(USE_AURA)
   // The Win32 API requires that we own the text.
   always_on_top_menu_text_ =
       UTF16ToWide(l10n_util::GetStringUTF16(IDS_ALWAYS_ON_TOP));
@@ -763,6 +767,7 @@ void TaskManagerView::AddAlwaysOnTopSystemMenuItem() {
   menu_info.wID = IDC_ALWAYS_ON_TOP;
   menu_info.dwTypeData = const_cast<wchar_t*>(always_on_top_menu_text_.c_str());
   ::InsertMenuItem(system_menu, index, TRUE, &menu_info);
+#endif
 }
 
 bool TaskManagerView::GetSavedAlwaysOnTopState(bool* always_on_top) const {
