@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,10 +8,6 @@
 
 #include "ui/gfx/native_widget_types.h"
 
-namespace gfx {
-class Rect;
-}
-
 namespace views {
 class Widget;
 class WidgetDelegate;
@@ -21,7 +17,6 @@ namespace browser {
 
 // Create a window for given |delegate| using default frame view.
 views::Widget* CreateViewsWindow(gfx::NativeWindow parent,
-                                 const gfx::Rect& bounds,
                                  views::WidgetDelegate* delegate);
 
 }  // namespace browser
