@@ -23,8 +23,8 @@ class BookmarksUIHTMLSource : public ChromeURLDataManager::DataSource {
   // the path we registered.
   virtual void StartDataRequest(const std::string& path,
                                 bool is_incognito,
-                                int request_id);
-  virtual std::string GetMimeType(const std::string& path) const;
+                                int request_id) OVERRIDE;
+  virtual std::string GetMimeType(const std::string& path) const OVERRIDE;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(BookmarksUIHTMLSource);
