@@ -21,6 +21,7 @@
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_list.h"
 #include "chrome/browser/ui/browser_window.h"
+#include "chrome/browser/ui/webui/chrome_web_ui_data_source.h"
 #include "chrome/browser/ui/webui/screenshot_source.h"
 #include "chrome/browser/ui/window_snapshot/window_snapshot.h"
 #include "chrome/common/chrome_paths.h"
@@ -52,9 +53,7 @@ const char kScreenshotBaseUrl[] = "chrome://screenshots/";
 const char kCurrentScreenshotUrl[] = "chrome://screenshots/current";
 #if defined(OS_CHROMEOS)
 const char kSavedScreenshotsUrl[] = "chrome://screenshots/saved/";
-
-const char kScreenshotPattern[] = "*.png";
-const char kScreenshotsRelativePath[] = "/Screenshots";
+const char kScreenshotPattern[] = "screenshot-*.png";
 
 const size_t kMaxSavedScreenshots = 2;
 #endif
@@ -72,11 +71,7 @@ void GetSavedScreenshots(std::vector<std::string>* saved_screenshots,
     return;
   }
 
-  // TODO(rkc): Change this to use FilePath.Append() once the cros
-  // issue with it is fixed
-  FilePath screenshots_path(fileshelf_path.value() +
-                            std::string(kScreenshotsRelativePath));
-  file_util::FileEnumerator screenshots(screenshots_path, false,
+  file_util::FileEnumerator screenshots(fileshelf_path, false,
                                         file_util::FileEnumerator::FILES,
                                         std::string(kScreenshotPattern));
   FilePath screenshot = screenshots.Next();
@@ -127,20 +122,6 @@ int GetIndexOfFeedbackTab(Browser* browser) {
 
 namespace browser {
 
-// TODO(rkc): Eventually find a better way to do this
-std::vector<unsigned char>* last_screenshot_png = 0;
-gfx::Rect screen_size;
-
-void RefreshLastScreenshot(Browser* browser) {
-  if (last_screenshot_png)
-    last_screenshot_png->clear();
-  else
-    last_screenshot_png = new std::vector<unsigned char>;
-
-  gfx::NativeWindow native_window = browser->window()->GetNativeHandle();
-  screen_size = browser::GrabWindowSnapshot(native_window, last_screenshot_png);
-}
-
 void ShowHtmlBugReportView(Browser* browser,
                            const std::string& description_template,
                            size_t issue_type) {
@@ -155,7 +136,14 @@ void ShowHtmlBugReportView(Browser* browser,
     return;
   }
 
-  RefreshLastScreenshot(browser);
+  std::vector<unsigned char>* last_screenshot_png =
+      BugReportUtil::GetScreenshotPng();
+  last_screenshot_png->clear();
+
+  gfx::NativeWindow native_window = browser->window()->GetNativeHandle();
+  BugReportUtil::SetScreenshotSize(
+      browser::GrabWindowSnapshot(native_window, last_screenshot_png));
+
   std::string bug_report_url = std::string(chrome::kChromeUIBugReportURL) +
       "#" + base::IntToString(browser->active_index()) +
       "?description=" + EscapeUrlEncodedData(description_template, false) +
@@ -165,27 +153,6 @@ void ShowHtmlBugReportView(Browser* browser,
 
 }  // namespace browser
 
-
-class BugReportUIHTMLSource : public ChromeURLDataManager::DataSource {
- public:
-  explicit BugReportUIHTMLSource(base::StringPiece html);
-
-  // Called when the network layer has requested a resource underneath
-  // the path we registered.
-  virtual void StartDataRequest(const std::string& path,
-                                bool is_incognito,
-                                int request_id);
-  virtual std::string GetMimeType(const std::string&) const {
-    return "text/html";
-  }
-
- private:
-  base::StringPiece bug_report_html_;
-  ~BugReportUIHTMLSource() {}
-
-  DISALLOW_COPY_AND_ASSIGN(BugReportUIHTMLSource);
-};
-
 // The handler for Javascript messages related to the "bug report" dialog
 class BugReportHandler : public WebUIMessageHandler,
                          public base::SupportsWeakPtr<BugReportHandler> {
@@ -193,12 +160,12 @@ class BugReportHandler : public WebUIMessageHandler,
   explicit BugReportHandler(TabContents* tab);
   virtual ~BugReportHandler();
 
-  // Init work after Attach.
-  base::StringPiece Init();
+  // Init work after Attach.  Returns true on success.
+  bool Init();
 
   // WebUIMessageHandler implementation.
-  virtual WebUIMessageHandler* Attach(WebUI* web_ui);
-  virtual void RegisterMessages();
+  virtual WebUIMessageHandler* Attach(WebUI* web_ui) OVERRIDE;
+  virtual void RegisterMessages() OVERRIDE;
 
  private:
   void HandleGetDialogDefaults(const ListValue* args);
@@ -219,7 +186,7 @@ class BugReportHandler : public WebUIMessageHandler,
   TabContents* tab_;
   ScreenshotSource* screenshot_source_;
 
-  BugReportData* bug_report_;
+  BugReportData* bug_report_data_;
   std::string target_tab_url_;
 #if defined(OS_CHROMEOS)
   // Variables to track SyslogsProvider::RequestSyslogs callback.
@@ -230,80 +197,49 @@ class BugReportHandler : public WebUIMessageHandler,
   DISALLOW_COPY_AND_ASSIGN(BugReportHandler);
 };
 
-////////////////////////////////////////////////////////////////////////////////
-//
-// BugReportHTMLSource
-//
-////////////////////////////////////////////////////////////////////////////////
+ChromeWebUIDataSource* CreateBugReportUIHTMLSource(bool successful_init) {
+  ChromeWebUIDataSource* source =
+      new ChromeWebUIDataSource(chrome::kChromeUIBugReportHost);
 
-BugReportUIHTMLSource::BugReportUIHTMLSource(base::StringPiece html)
-    : DataSource(chrome::kChromeUIBugReportHost, MessageLoop::current()) {
-  bug_report_html_ = html;
-}
-
-void BugReportUIHTMLSource::StartDataRequest(const std::string& path,
-                                             bool is_incognito,
-                                             int request_id) {
-  DictionaryValue localized_strings;
-  localized_strings.SetString(std::string("title"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_TITLE));
-  localized_strings.SetString(std::string("page-title"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_REPORT_PAGE_TITLE));
-  localized_strings.SetString(std::string("issue-with"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_ISSUE_WITH));
-  localized_strings.SetString(std::string("page-url"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_REPORT_URL_LABEL));
-  localized_strings.SetString(std::string("description"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_DESCRIPTION_LABEL));
-  localized_strings.SetString(std::string("current-screenshot"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_SCREENSHOT_LABEL));
-  localized_strings.SetString(std::string("saved-screenshot"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_SAVED_SCREENSHOT_LABEL));
+  source->AddLocalizedString("title", IDS_BUGREPORT_TITLE);
+  source->AddLocalizedString("page-title", IDS_BUGREPORT_REPORT_PAGE_TITLE);
+  source->AddLocalizedString("issue-with", IDS_BUGREPORT_ISSUE_WITH);
+  source->AddLocalizedString("page-url", IDS_BUGREPORT_REPORT_URL_LABEL);
+  source->AddLocalizedString("description", IDS_BUGREPORT_DESCRIPTION_LABEL);
+  source->AddLocalizedString("current-screenshot",
+                             IDS_BUGREPORT_SCREENSHOT_LABEL);
+  source->AddLocalizedString("saved-screenshot",
+                             IDS_BUGREPORT_SAVED_SCREENSHOT_LABEL);
 #if defined(OS_CHROMEOS)
-  localized_strings.SetString(std::string("user-email"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_USER_EMAIL_LABEL));
-  localized_strings.SetString(std::string("sysinfo"),
-      l10n_util::GetStringUTF8(
-          IDS_BUGREPORT_INCLUDE_SYSTEM_INFORMATION_CHKBOX));
-
-  localized_strings.SetString(std::string("currentscreenshots"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_CURRENT_SCREENSHOTS));
-  localized_strings.SetString(std::string("savedscreenshots"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_SAVED_SCREENSHOTS));
-
-  localized_strings.SetString(std::string("choose-different-screenshot"),
-      l10n_util::GetStringUTF8(
-          IDS_BUGREPORT_CHOOSE_DIFFERENT_SCREENSHOT));
-  localized_strings.SetString(std::string("choose-original-screenshot"),
-      l10n_util::GetStringUTF8(
-          IDS_BUGREPORT_CHOOSE_ORIGINAL_SCREENSHOT));
+  source->AddLocalizedString("user-email", IDS_BUGREPORT_USER_EMAIL_LABEL);
+  source->AddLocalizedString("sysinfo",
+                             IDS_BUGREPORT_INCLUDE_SYSTEM_INFORMATION_CHKBOX);
+  source->AddLocalizedString("currentscreenshots",
+                             IDS_BUGREPORT_CURRENT_SCREENSHOTS);
+  source->AddLocalizedString("savedscreenshots",
+                             IDS_BUGREPORT_SAVED_SCREENSHOTS);
+  source->AddLocalizedString("choose-different-screenshot",
+                             IDS_BUGREPORT_CHOOSE_DIFFERENT_SCREENSHOT);
+  source->AddLocalizedString("choose-original-screenshot",
+                             IDS_BUGREPORT_CHOOSE_ORIGINAL_SCREENSHOT);
 #else
-  localized_strings.SetString(std::string("currentscreenshots"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_INCLUDE_NEW_SCREEN_IMAGE));
+  source->AddLocalizedString("currentscreenshots",
+                             IDS_BUGREPORT_INCLUDE_NEW_SCREEN_IMAGE);
 #endif
-  localized_strings.SetString(std::string("noscreenshot"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_INCLUDE_NO_SCREENSHOT));
+  source->AddLocalizedString("noscreenshot",
+                             IDS_BUGREPORT_INCLUDE_NO_SCREENSHOT);
 
-  localized_strings.SetString(std::string("send-report"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_SEND_REPORT));
-  localized_strings.SetString(std::string("cancel"),
-      l10n_util::GetStringUTF8(IDS_CANCEL));
+  source->AddLocalizedString("send-report", IDS_BUGREPORT_SEND_REPORT);
+  source->AddLocalizedString("cancel", IDS_CANCEL);
 
   // Option strings for the "issue with" drop-down.
-  localized_strings.SetString(std::string("issue-choose"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_CHOOSE_ISSUE));
-
-  localized_strings.SetString(std::string("no-issue-selected"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_NO_ISSUE_SELECTED));
-
-  localized_strings.SetString(std::string("no-description"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_NO_DESCRIPTION));
-
-  localized_strings.SetString(std::string("no-saved-screenshots"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_NO_SAVED_SCREENSHOTS_HELP));
-
-  localized_strings.SetString(std::string("privacy-note"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_PRIVACY_NOTE));
+  source->AddLocalizedString("issue-choose", IDS_BUGREPORT_CHOOSE_ISSUE);
+  source->AddLocalizedString("no-issue-selected",
+                             IDS_BUGREPORT_NO_ISSUE_SELECTED);
+  source->AddLocalizedString("no-description", IDS_BUGREPORT_NO_DESCRIPTION);
+  source->AddLocalizedString("no-saved-screenshots",
+                             IDS_BUGREPORT_NO_SAVED_SCREENSHOTS_HELP);
+  source->AddLocalizedString("privacy-note", IDS_BUGREPORT_PRIVACY_NOTE);
 
   // TODO(rkc): Find some way to ensure this order of dropdowns is in sync
   // with the order in the userfeedback ChromeData proto buffer
@@ -320,24 +256,16 @@ void BugReportUIHTMLSource::StartDataRequest(const std::string& path,
   // General Feedback/Other
   // Autofill (hidden by default)
 
-  localized_strings.SetString(std::string("issue-connectivity"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_CONNECTIVITY));
-  localized_strings.SetString(std::string("issue-sync"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_SYNC));
-  localized_strings.SetString(std::string("issue-crashes"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_CRASHES));
-  localized_strings.SetString(std::string("issue-page-formatting"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_PAGE_FORMATTING));
-  localized_strings.SetString(std::string("issue-extensions"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_EXTENSIONS));
-  localized_strings.SetString(std::string("issue-standby"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_STANDBY_RESUME));
-  localized_strings.SetString(std::string("issue-phishing"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_PHISHING_PAGE));
-  localized_strings.SetString(std::string("issue-other"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_GENERAL));
-  localized_strings.SetString(std::string("issue-autofill"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_AUTOFILL));
+  source->AddLocalizedString("issue-connectivity", IDS_BUGREPORT_CONNECTIVITY);
+  source->AddLocalizedString("issue-sync", IDS_BUGREPORT_SYNC);
+  source->AddLocalizedString("issue-crashes", IDS_BUGREPORT_CRASHES);
+  source->AddLocalizedString("issue-page-formatting",
+                             IDS_BUGREPORT_PAGE_FORMATTING);
+  source->AddLocalizedString("issue-extensions", IDS_BUGREPORT_EXTENSIONS);
+  source->AddLocalizedString("issue-standby", IDS_BUGREPORT_STANDBY_RESUME);
+  source->AddLocalizedString("issue-phishing", IDS_BUGREPORT_PHISHING_PAGE);
+  source->AddLocalizedString("issue-other", IDS_BUGREPORT_GENERAL);
+  source->AddLocalizedString("issue-autofill", IDS_BUGREPORT_AUTOFILL);
 #else
   // Dropdown for Chrome:
   //
@@ -352,87 +280,27 @@ void BugReportUIHTMLSource::StartDataRequest(const std::string& path,
   // Other
   // Autofill (hidden by default)
 
-  localized_strings.SetString(std::string("issue-page-formatting"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_PAGE_FORMATTING));
-  localized_strings.SetString(std::string("issue-page-load"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_PAGE_LOAD));
-  localized_strings.SetString(std::string("issue-plugins"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_PLUGINS));
-  localized_strings.SetString(std::string("issue-tabs"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_TABS));
-  localized_strings.SetString(std::string("issue-sync"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_SYNC));
-  localized_strings.SetString(std::string("issue-crashes"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_CRASHES));
-  localized_strings.SetString(std::string("issue-extensions"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_EXTENSIONS));
-  localized_strings.SetString(std::string("issue-phishing"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_PHISHING_PAGE));
-  localized_strings.SetString(std::string("issue-other"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_OTHER));
-  localized_strings.SetString(std::string("issue-autofill"),
-      l10n_util::GetStringUTF8(IDS_BUGREPORT_AUTOFILL));
-
+  source->AddLocalizedString("issue-page-formatting",
+                             IDS_BUGREPORT_PAGE_FORMATTING);
+  source->AddLocalizedString("issue-page-load", IDS_BUGREPORT_PAGE_LOAD);
+  source->AddLocalizedString("issue-plugins", IDS_BUGREPORT_PLUGINS);
+  source->AddLocalizedString("issue-tabs", IDS_BUGREPORT_TABS);
+  source->AddLocalizedString("issue-sync", IDS_BUGREPORT_SYNC);
+  source->AddLocalizedString("issue-crashes", IDS_BUGREPORT_CRASHES);
+  source->AddLocalizedString("issue-extensions", IDS_BUGREPORT_EXTENSIONS);
+  source->AddLocalizedString("issue-phishing", IDS_BUGREPORT_PHISHING_PAGE);
+  source->AddLocalizedString("issue-other", IDS_BUGREPORT_OTHER);
+  source->AddLocalizedString("issue-autofill", IDS_BUGREPORT_AUTOFILL);
 #endif
 
-  SetFontAndTextDirection(&localized_strings);
+  source->set_json_path("strings.js");
+  source->add_resource_path("bug_report.js", IDR_BUGREPORT_JS);
+  source->set_default_resource(
+      successful_init ? IDR_BUGREPORT_HTML : IDR_BUGREPORT_HTML_INVALID);
 
-  std::string full_html = jstemplate_builder::GetI18nTemplateHtml(
-      bug_report_html_, &localized_strings);
-
-  SendResponse(request_id, base::RefCountedString::TakeString(&full_html));
+  return source;
 }
 
-
-////////////////////////////////////////////////////////////////////////////////
-//
-// BugReportData
-//
-////////////////////////////////////////////////////////////////////////////////
-void BugReportData::SendReport() {
-#if defined(OS_CHROMEOS)
-  // In case we already got the syslogs and sent the report, leave
-  if (sent_report_) return;
-  // Set send_report_ so that no one else processes SendReport
-  sent_report_ = true;
-#endif
-
-  int image_data_size = image_.size();
-  char* image_data = image_data_size ?
-      reinterpret_cast<char*>(&(image_.front())) : NULL;
-  BugReportUtil::SendReport(profile_
-                            , problem_type_
-                            , page_url_
-                            , description_
-                            , image_data
-                            , image_data_size
-                            , browser::screen_size.width()
-                            , browser::screen_size.height()
-#if defined(OS_CHROMEOS)
-                            , user_email_
-                            , zip_content_ ? zip_content_->c_str() : NULL
-                            , zip_content_ ? zip_content_->length() : 0
-                            , send_sys_info_ ? sys_info_ : NULL
-#endif
-                            );
-
-#if defined(OS_CHROMEOS)
-  if (sys_info_) {
-    delete sys_info_;
-    sys_info_ = NULL;
-  }
-  if (zip_content_) {
-    delete zip_content_;
-    zip_content_ = NULL;
-  }
-#endif
-
-  // Once the report has been sent, this object has no purpose in life, delete
-  // ourselves.
-  delete this;
-}
-
-
 ////////////////////////////////////////////////////////////////////////////////
 //
 // BugReportHandler
@@ -441,7 +309,7 @@ void BugReportData::SendReport() {
 BugReportHandler::BugReportHandler(TabContents* tab)
     : tab_(tab),
       screenshot_source_(NULL),
-      bug_report_(NULL)
+      bug_report_data_(NULL)
 #if defined(OS_CHROMEOS)
     , syslogs_handle_(0)
 #endif
@@ -449,34 +317,35 @@ BugReportHandler::BugReportHandler(TabContents* tab)
 }
 
 BugReportHandler::~BugReportHandler() {
-  // Just in case we didn't send off bug_report_ to SendReport
-  if (bug_report_) {
+  // Just in case we didn't send off bug_report_data_ to SendReport
+  if (bug_report_data_) {
     // If we're deleting the report object, cancel feedback collection first
     CancelFeedbackCollection();
-    delete bug_report_;
+    delete bug_report_data_;
   }
+  // Make sure we don't leave any screenshot data around.
+  BugReportUtil::ClearScreenshotPng();
 }
 
 void BugReportHandler::ClobberScreenshotsSource() {
   // Re-create our screenshots data source (this clobbers the last source)
   // setting the screenshot to NULL, effectively disabling the source
   // TODO(rkc): Once there is a method to 'remove' a source, change this code
-  tab_->profile()->GetChromeURLDataManager()->AddDataSource(
-      new ScreenshotSource(NULL));
+  Profile* profile = Profile::FromBrowserContext(tab_->browser_context());
+  profile->GetChromeURLDataManager()->AddDataSource(new ScreenshotSource(NULL));
 
-  // clobber last screenshot
-  if (browser::last_screenshot_png)
-    browser::last_screenshot_png->clear();
+  BugReportUtil::ClearScreenshotPng();
 }
 
 void BugReportHandler::SetupScreenshotsSource() {
   // If we don't already have a screenshot source object created, create one.
-  if (!screenshot_source_)
-    screenshot_source_ = new ScreenshotSource(
-        browser::last_screenshot_png);
-
+  if (!screenshot_source_) {
+    screenshot_source_ =
+        new ScreenshotSource(BugReportUtil::GetScreenshotPng());
+  }
   // Add the source to the data manager.
-  tab_->profile()->GetChromeURLDataManager()->AddDataSource(screenshot_source_);
+  Profile* profile = Profile::FromBrowserContext(tab_->browser_context());
+  profile->GetChromeURLDataManager()->AddDataSource(screenshot_source_);
 }
 
 WebUIMessageHandler* BugReportHandler::Attach(WebUI* web_ui) {
@@ -484,7 +353,7 @@ WebUIMessageHandler* BugReportHandler::Attach(WebUI* web_ui) {
   return WebUIMessageHandler::Attach(web_ui);
 }
 
-base::StringPiece BugReportHandler::Init() {
+bool BugReportHandler::Init() {
   std::string page_url;
   if (tab_->controller().GetActiveEntry()) {
      page_url = tab_->controller().GetActiveEntry()->url().spec();
@@ -500,19 +369,14 @@ base::StringPiece BugReportHandler::Init() {
     params.erase(params.begin() + additional_params_pos, params.end());
 
   int index = 0;
-  if (!base::StringToInt(params, &index)) {
-    return base::StringPiece(
-        ResourceBundle::GetSharedInstance().GetRawDataResource(
-            IDR_BUGREPORT_HTML_INVALID));
-  }
+  if (!base::StringToInt(params, &index))
+    return false;
 
   Browser* browser = BrowserList::GetLastActive();
   // Sanity checks.
   if (((index == 0) && (params != "0")) || !browser ||
       index >= browser->tab_count()) {
-    return base::StringPiece(
-        ResourceBundle::GetSharedInstance().GetRawDataResource(
-            IDR_BUGREPORT_HTML_INVALID));
+    return false;
   }
 
   TabContents* target_tab = browser->GetTabContentsAt(index);
@@ -523,9 +387,7 @@ base::StringPiece BugReportHandler::Init() {
   // Setup the screenshot source after we've verified input is legit.
   SetupScreenshotsSource();
 
-  return base::StringPiece(
-      ResourceBundle::GetSharedInstance().GetRawDataResource(
-          IDR_BUGREPORT_HTML));
+  return true;
 }
 
 void BugReportHandler::RegisterMessages() {
@@ -546,7 +408,8 @@ void BugReportHandler::RegisterMessages() {
 }
 
 void BugReportHandler::HandleGetDialogDefaults(const ListValue*) {
-  bug_report_ = new BugReportData();
+  // Will delete itself when bug_report_data_->SendReport() is called.
+  bug_report_data_ = new BugReportData();
 
   // send back values which the dialog js needs initially
   ListValue dialog_defaults;
@@ -568,7 +431,7 @@ void BugReportHandler::HandleGetDialogDefaults(const ListValue*) {
         true,  // don't compress.
         chromeos::system::SyslogsProvider::SYSLOGS_FEEDBACK,
         &syslogs_consumer_,
-        NewCallback(bug_report_, &BugReportData::SyslogsComplete));
+        NewCallback(bug_report_data_, &BugReportData::SyslogsComplete));
   }
   // 2: user e-mail
   dialog_defaults.Append(new StringValue(GetUserEmail()));
@@ -598,7 +461,7 @@ void BugReportHandler::HandleRefreshSavedScreenshots(const ListValue*) {
 
 
 void BugReportHandler::HandleSendReport(const ListValue* list_value) {
-  if (!bug_report_) {
+  if (!bug_report_data_) {
     LOG(ERROR) << "Bug report hasn't been intialized yet.";
     return;
   }
@@ -644,9 +507,9 @@ void BugReportHandler::HandleSendReport(const ListValue* list_value) {
   screenshot_path.erase(0, strlen(kScreenshotBaseUrl));
 
   // Get the image to send in the report.
-  std::vector<unsigned char> image;
+  ScreenshotDataPtr image_ptr;
   if (!screenshot_path.empty())
-    image = screenshot_source_->GetScreenshot(screenshot_path);
+    image_ptr = screenshot_source_->GetCachedScreenshot(screenshot_path);
 
 #if defined(OS_CHROMEOS)
   if (++i == list_value->end()) {
@@ -672,35 +535,35 @@ void BugReportHandler::HandleSendReport(const ListValue* list_value) {
     CancelFeedbackCollection();
 #endif
 
-  // Update the data in bug_report_ so it can be sent
-  bug_report_->UpdateData(web_ui_->GetProfile()
-                          , target_tab_url_
-                          , problem_type
-                          , page_url
-                          , description
-                          , image
+  // Update the data in bug_report_data_ so it can be sent
+  bug_report_data_->UpdateData(Profile::FromWebUI(web_ui_)
+                               , target_tab_url_
+                               , problem_type
+                               , page_url
+                               , description
+                               , image_ptr
 #if defined(OS_CHROMEOS)
-                          , user_email
-                          , send_sys_info
-                          , false // sent_report
+                               , user_email
+                               , send_sys_info
+                               , false // sent_report
 #endif
-                          );
+                               );
 
 #if defined(OS_CHROMEOS)
   // If we don't require sys_info, or we have it, or we never requested it
   // (because libcros failed to load), then send the report now.
   // Otherwise, the report will get sent when we receive sys_info.
-  if (!send_sys_info || bug_report_->sys_info() != NULL ||
+  if (!send_sys_info || bug_report_data_->sys_info() != NULL ||
       syslogs_handle_ == 0) {
-    bug_report_->SendReport();
+    bug_report_data_->SendReport();
   }
 #else
-  bug_report_->SendReport();
+  bug_report_data_->SendReport();
 #endif
   // Lose the pointer to the BugReportData object; the object will delete itself
   // from SendReport, whether we called it, or will be called by the log
   // completion routine.
-  bug_report_ = NULL;
+  bug_report_data_ = NULL;
 
   // Whether we sent the report, or if it will be sent by the Syslogs complete
   // function, close our feedback tab anyway, we have no more use for it.
@@ -748,10 +611,11 @@ BugReportUI::BugReportUI(TabContents* tab) : HtmlDialogUI(tab) {
   BugReportHandler* handler = new BugReportHandler(tab);
   AddMessageHandler((handler)->Attach(this));
 
-  // The handler's init will specify which html
-  // resource we'll display to the user
-  BugReportUIHTMLSource* html_source =
-      new BugReportUIHTMLSource(handler->Init());
+  // The handler's init will determine whether we show the error html page.
+  ChromeWebUIDataSource* html_source =
+      CreateBugReportUIHTMLSource(handler->Init());
+
   // Set up the chrome://bugreport/ source.
-  tab->profile()->GetChromeURLDataManager()->AddDataSource(html_source);
+  Profile* profile = Profile::FromBrowserContext(tab->browser_context());
+  profile->GetChromeURLDataManager()->AddDataSource(html_source);
 }
