@@ -14,7 +14,14 @@
 
 ChromeWebUIDataSource::ChromeWebUIDataSource(const std::string& source_name)
     : DataSource(source_name, MessageLoop::current()),
-      default_resource_(-1) {}
+      default_resource_(-1) {
+}
+
+ChromeWebUIDataSource::ChromeWebUIDataSource(const std::string& source_name,
+                                             MessageLoop* loop)
+    : DataSource(source_name, loop),
+      default_resource_(-1) {
+}
 
 ChromeWebUIDataSource::~ChromeWebUIDataSource() {
 }
@@ -33,6 +40,9 @@ std::string ChromeWebUIDataSource::GetMimeType(const std::string& path) const {
   if (EndsWith(path, ".js", false))
     return "application/javascript";
 
+  if (EndsWith(path, ".json", false))
+    return "application/json";
+
   if (EndsWith(path, ".pdf", false))
     return "application/pdf";
 
@@ -42,7 +52,7 @@ std::string ChromeWebUIDataSource::GetMimeType(const std::string& path) const {
 void ChromeWebUIDataSource::StartDataRequest(const std::string& path,
                                              bool is_incognito,
                                              int request_id) {
-  if (path == json_path_) {
+  if (!json_path_.empty() && path == json_path_) {
     SendLocalizedStringsAsJSON(request_id);
   } else {
     int resource_id = default_resource_;
