@@ -5,20 +5,26 @@
 #include "chrome/browser/ui/webui/chromeos/login/signin_screen_handler.h"
 
 #include "base/command_line.h"
+#include "base/hash_tables.h"
+#include "base/logging.h"
 #include "base/stringprintf.h"
 #include "base/task.h"
 #include "base/values.h"
 #include "chrome/browser/browser_process.h"
+#include "chrome/browser/browser_shutdown.h"
 #include "chrome/browser/io_thread.h"
 #include "chrome/browser/chromeos/cros/cros_library.h"
+#include "chrome/browser/chromeos/cros/network_library.h"
 #include "chrome/browser/chromeos/cros/power_library.h"
 #include "chrome/browser/chromeos/login/user_manager.h"
 #include "chrome/browser/chromeos/login/webui_login_display.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/chromeos/user_cros_settings_provider.h"
+#include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/url_constants.h"
 #include "content/browser/tab_contents/tab_contents.h"
+#include "content/common/notification_service.h"
 #include "grit/generated_resources.h"
 #include "net/base/dnsrr_resolver.h"
 #include "ui/base/l10n/l10n_util.h"
@@ -62,82 +68,152 @@ std::string SanitizeEmail(const std::string& email) {
 
 namespace chromeos {
 
-// TODO(zelidrag): Make this notification task cancelable and controlled by
-// SigninScreenHandler.
-class ReportDnsCacheClearedOnUIThread : public Task {
+// Class which observes network state changes and calls registerd callbacks.
+// State is considered changed if connection or the active network has been
+// changed. Also, it answers to the requests about current network state.
+class NetworkStateInformer
+    : public chromeos::NetworkLibrary::NetworkManagerObserver {
  public:
-  explicit ReportDnsCacheClearedOnUIThread(SigninScreenHandler* handler)
-      : handler_(handler)  {
-  }
-  virtual ~ReportDnsCacheClearedOnUIThread() {}
+  explicit NetworkStateInformer(WebUI* web_ui);
+  virtual ~NetworkStateInformer();
 
-  // CancelableTask overrides.
-  virtual void Run() OVERRIDE {
-    DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-    if (handler_)
-      handler_->OnDnsCleared();
-  }
+  // Adds observer's callback to be called when network state has been changed.
+  void AddObserver(const std::string& callback);
+
+  // Removes observer's callback.
+  void RemoveObserver(const std::string& callback);
+
+  // Sends current network state using the callback.
+  void SendState(const std::string& callback);
+
+  // NetworkLibrary::NetworkManagerObserver implementation:
+  virtual void OnNetworkManagerChanged(chromeos::NetworkLibrary* cros) OVERRIDE;
 
  private:
-  SigninScreenHandler* handler_;
-  DISALLOW_COPY_AND_ASSIGN(ReportDnsCacheClearedOnUIThread);
+  enum State {OFFLINE, ONLINE, CAPTIVE_PORTAL};
+
+  bool UpdateState(chromeos::NetworkLibrary* cros);
+
+  base::hash_set<std::string> observers_;
+  std::string active_network_;
+  State state_;
+  WebUI* web_ui_;
 };
 
 // Clears DNS cache on IO thread.
-class ClearDnsCacheTaskOnIOThread : public CancelableTask {
+class ClearDnsCacheTaskOnIOThread : public Task {
  public:
-  ClearDnsCacheTaskOnIOThread(SigninScreenHandler* handler,
-                              IOThread* io_thread)
-      : handler_(handler), io_thread_(io_thread), should_run_(true)  {
+  ClearDnsCacheTaskOnIOThread(Task* callback, IOThread* io_thread)
+      : callback_(callback), io_thread_(io_thread) {
   }
   virtual ~ClearDnsCacheTaskOnIOThread() {}
 
-  // CancelableTask overrides.
+  // Task overrides.
   virtual void Run() OVERRIDE {
     DCHECK(BrowserThread::CurrentlyOn(BrowserThread::IO));
-    if (!should_run_)
-      return;
+    if (browser_shutdown::IsTryingToQuit())
+     return;
 
     io_thread_->globals()->dnsrr_resolver.get()->OnIPAddressChanged();
-    BrowserThread::PostTask(
-        BrowserThread::UI, FROM_HERE,
-        new ReportDnsCacheClearedOnUIThread(handler_));
-  }
-
-  virtual void Cancel() OVERRIDE {
-    should_run_ = false;
+    BrowserThread::PostTask(BrowserThread::UI, FROM_HERE, callback_);
   }
 
  private:
-  SigninScreenHandler* handler_;
+  Task* callback_;
   IOThread* io_thread_;
-  bool should_run_;
   DISALLOW_COPY_AND_ASSIGN(ClearDnsCacheTaskOnIOThread);
 };
 
+// NetworkStateInformer implementation -----------------------------------------
+
+NetworkStateInformer::NetworkStateInformer(WebUI* web_ui) : web_ui_(web_ui) {
+  NetworkLibrary* cros = CrosLibrary::Get()->GetNetworkLibrary();
+  UpdateState(cros);
+  cros->AddNetworkManagerObserver(this);
+}
+
+NetworkStateInformer::~NetworkStateInformer() {
+  CrosLibrary::Get()->GetNetworkLibrary()->
+      RemoveNetworkManagerObserver(this);
+}
+
+void NetworkStateInformer::AddObserver(const std::string& callback) {
+  observers_.insert(callback);
+}
+
+void NetworkStateInformer::RemoveObserver(const std::string& callback) {
+  observers_.erase(callback);
+}
+
+void NetworkStateInformer::SendState(const std::string& callback) {
+  base::FundamentalValue state_value(state_);
+  web_ui_->CallJavascriptFunction(callback, state_value);
+}
+
+void NetworkStateInformer::OnNetworkManagerChanged(NetworkLibrary* cros) {
+  if (UpdateState(cros)) {
+    for (base::hash_set<std::string>::iterator it = observers_.begin();
+         it != observers_.end(); ++it) {
+      SendState(*it);
+    }
+  }
+}
+
+bool NetworkStateInformer::UpdateState(NetworkLibrary* cros) {
+  State new_state;
+  std::string new_active_network;
+  if (!cros->Connected()) {
+    new_state = OFFLINE;
+  } else {
+    const Network* active_network = cros->active_network();
+    if (active_network) {
+      new_active_network = active_network->unique_id();
+      if (active_network->restricted_pool()) {
+        new_state = CAPTIVE_PORTAL;
+      } else {
+        new_state = ONLINE;
+      }
+    } else {
+      // Bogus network situation:
+      // Connected() returns true but no active network.
+      new_state = OFFLINE;
+      NOTREACHED();
+    }
+  }
+  bool updated = (new_state != state_) ||
+      (active_network_ != new_active_network);
+  state_ = new_state;
+  active_network_ = new_active_network;
+  return updated;
+}
+
+// SigninScreenHandler implementation ------------------------------------------
 
 SigninScreenHandler::SigninScreenHandler()
     : delegate_(WebUILoginDisplay::GetInstance()),
       show_on_init_(false),
       oobe_ui_(false),
       dns_cleared_(false),
+      dns_clear_task_running_(false),
       cookies_cleared_(false),
       extension_driven_(
           CommandLine::ForCurrentProcess()->HasSwitch(
               switches::kWebUILogin)),
-      clear_dns_task_(NULL) {
+      cookie_remover_(NULL),
+      ALLOW_THIS_IN_INITIALIZER_LIST(method_factory_(this)) {
   delegate_->SetWebUIHandler(this);
 }
 
 SigninScreenHandler::~SigninScreenHandler() {
-  if (clear_dns_task_)
-    clear_dns_task_->Cancel();
+  method_factory_.RevokeAll();
+  if (cookie_remover_)
+    cookie_remover_->RemoveObserver(this);
 }
 
 void SigninScreenHandler::GetLocalizedStrings(
     DictionaryValue* localized_strings) {
   localized_strings->SetString("signinScreenTitle",
-      l10n_util::GetStringUTF16(IDS_OOBE_SIGNIN));
+      l10n_util::GetStringUTF16(IDS_SIGNIN_SCREEN_TITLE));
   localized_strings->SetString("emailHint",
       l10n_util::GetStringUTF16(IDS_LOGIN_USERNAME));
   localized_strings->SetString("passwordHint",
@@ -156,12 +232,17 @@ void SigninScreenHandler::GetLocalizedStrings(
       l10n_util::GetStringUTF16(IDS_LOGIN_ERROR_ADD_USER_OFFLINE));
   localized_strings->SetString("offlineMessageTitle",
       l10n_util::GetStringUTF16(IDS_LOGIN_OFFLINE_TITLE));
+  // TODO(altimofeev): we don't want to change IDS, so we use the closest one.
   localized_strings->SetString("offlineMessageBody",
-      l10n_util::GetStringUTF16(IDS_LOGIN_OFFLINE_MESSAGE));
+      l10n_util::GetStringUTF16(IDS_IMAGEBURN_NO_CONNECTION_WARNING));
+  localized_strings->SetString("captivePortalMessage",
+        l10n_util::GetStringUTF16(IDS_LOGIN_MAYBE_CAPTIVE_PORTAL));
+  localized_strings->SetString("captivePortalStartGuestSession",
+      l10n_util::GetStringUTF16(IDS_LOGIN_FIX_CAPTIVE_PORTAL));
   localized_strings->SetString("createAccount",
-      l10n_util::GetStringUTF16(IDS_CREATE_ACCOUNT_BUTTON));
+      l10n_util::GetStringUTF16(IDS_CREATE_ACCOUNT_HTML));
   localized_strings->SetString("guestSignin",
-      l10n_util::GetStringUTF16(IDS_BROWSE_WITHOUT_SIGNING_IN_BUTTON));
+      l10n_util::GetStringUTF16(IDS_BROWSE_WITHOUT_SIGNING_IN_HTML));
   localized_strings->SetString("removeUser",
       l10n_util::GetStringUTF16(IDS_LOGIN_REMOVE));
 
@@ -186,11 +267,7 @@ void SigninScreenHandler::Show(bool oobe_ui) {
     // figure out how to make it fast enough.
     SendUserList(false);
 
-    // Show sign-in UI if there is no visible users.
-    if (WebUILoginDisplay::GetInstance()->users().empty())
-      HandleShowAddUser(NULL);
-    else
-      ShowScreen(kAccountPickerScreen, NULL);
+    ShowScreen(kAccountPickerScreen, NULL);
   }
 }
 
@@ -204,6 +281,8 @@ void SigninScreenHandler::Initialize() {
 }
 
 void SigninScreenHandler::RegisterMessages() {
+  network_state_informer_.reset(new NetworkStateInformer(web_ui_));
+
   web_ui_->RegisterMessageCallback("authenticateUser",
       NewCallback(this, &SigninScreenHandler::HandleAuthenticateUser));
   web_ui_->RegisterMessageCallback("completeLogin",
@@ -212,6 +291,8 @@ void SigninScreenHandler::RegisterMessages() {
       NewCallback(this, &SigninScreenHandler::HandleGetUsers));
   web_ui_->RegisterMessageCallback("launchIncognito",
       NewCallback(this, &SigninScreenHandler::HandleLaunchIncognito));
+  web_ui_->RegisterMessageCallback("fixCaptivePortal",
+      NewCallback(this, &SigninScreenHandler::HandleFixCaptivePortal));
   web_ui_->RegisterMessageCallback("showAddUser",
       NewCallback(this, &SigninScreenHandler::HandleShowAddUser));
   web_ui_->RegisterMessageCallback("shutdownSystem",
@@ -224,6 +305,16 @@ void SigninScreenHandler::RegisterMessages() {
       NewCallback(this, &SigninScreenHandler::HandleLaunchHelpApp));
   web_ui_->RegisterMessageCallback("createAccount",
       NewCallback(this, &SigninScreenHandler::HandleCreateAccount));
+  web_ui_->RegisterMessageCallback("loginWebuiReady",
+      NewCallback(this, &SigninScreenHandler::HandleLoginWebuiReady));
+  web_ui_->RegisterMessageCallback("loginRequestNetworkState",
+      NewCallback(this, &SigninScreenHandler::HandleLoginRequestNetworkState));
+  web_ui_->RegisterMessageCallback("loginAddNetworkStateObserver",
+      NewCallback(this,
+                  &SigninScreenHandler::HandleLoginAddNetworkStateObserver));
+  web_ui_->RegisterMessageCallback("loginRemoveNetworkStateObserver",
+      NewCallback(this,
+                  &SigninScreenHandler::HandleLoginRemoveNetworkStateObserver));
 }
 
 void SigninScreenHandler::HandleGetUsers(const base::ListValue* args) {
@@ -260,13 +351,14 @@ void SigninScreenHandler::ShowError(int login_attempts,
 
 void SigninScreenHandler::OnBrowsingDataRemoverDone() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
+  cookie_remover_ = NULL;
   cookies_cleared_ = true;
   ShowSigninScreenIfReady();
 }
 
 void SigninScreenHandler::OnDnsCleared() {
   DCHECK(BrowserThread::CurrentlyOn(BrowserThread::UI));
-  clear_dns_task_ = NULL;
+  dns_clear_task_running_ = false;
   dns_cleared_ = true;
   ShowSigninScreenIfReady();
 }
@@ -277,24 +369,44 @@ void SigninScreenHandler::ShowSigninScreenIfReady() {
     return;
 
   DictionaryValue params;
+  params.SetString("startUrl", kGaiaExtStartPage);
   params.SetString("email", email_);
+  email_.clear();
 
-  std::string url = kGaiaExtStartPage;
   const std::string app_locale = g_browser_process->GetApplicationLocale();
-  if (!app_locale.empty()) {
-    url.append("?hl=");
-    url.append(app_locale);
-  }
-  params.SetString("startUrl", url);
+  if (!app_locale.empty())
+    params.SetString("hl", app_locale);
 
   params.SetBoolean("createAccount",
       UserCrosSettingsProvider::cached_allow_new_user());
   params.SetBoolean("guestSignin",
       UserCrosSettingsProvider::cached_allow_guest());
-  email_.clear();
+
+  // Test automation data:
+  const CommandLine* command_line = CommandLine::ForCurrentProcess();
+  if (command_line->HasSwitch(switches::kAuthExtensionPath)) {
+    if (!test_user_.empty()) {
+      params.SetString("test_email", test_user_);
+      test_user_.clear();
+    }
+    if (!test_pass_.empty()) {
+      params.SetString("test_password", test_pass_);
+      test_pass_.clear();
+    }
+  }
   ShowScreen(kGaiaSigninScreen, &params);
 }
 
+void SigninScreenHandler::ShowSigninScreenForCreds(
+    const std::string& username,
+    const std::string& password) {
+  VLOG(2) << "ShowSigninScreenForCreds " << username << " " << password;
+
+  test_user_ = username;
+  test_pass_ = password;
+  HandleShowAddUser(NULL);
+}
+
 void SigninScreenHandler::HandleCompleteLogin(const base::ListValue* args) {
   std::string username;
   std::string password;
@@ -325,6 +437,10 @@ void SigninScreenHandler::HandleLaunchIncognito(const base::ListValue* args) {
   delegate_->LoginAsGuest();
 }
 
+void SigninScreenHandler::HandleFixCaptivePortal(const base::ListValue* args) {
+  delegate_->FixCaptivePortal();
+}
+
 void SigninScreenHandler::HandleShutdownSystem(const base::ListValue* args) {
   DCHECK(CrosLibrary::Get()->EnsureLoaded());
   CrosLibrary::Get()->GetPowerLibrary()->RequestShutdown();
@@ -442,30 +558,70 @@ void SigninScreenHandler::SendUserList(bool animated) {
                                   users_list, animated_value);
 }
 
+void SigninScreenHandler::HandleLoginWebuiReady(const base::ListValue* args) {
+  NotificationService::current()->Notify(
+      chrome::NOTIFICATION_LOGIN_WEBUI_READY,
+      NotificationService::AllSources(),
+      NotificationService::NoDetails());
+}
+
+void SigninScreenHandler::HandleLoginRequestNetworkState(
+    const base::ListValue* args) {
+  std::string callback;
+  if (!args->GetString(0, &callback)) {
+    NOTREACHED();
+    return;
+  }
+  network_state_informer_->SendState(callback);
+}
+
+void SigninScreenHandler::HandleLoginAddNetworkStateObserver(
+    const base::ListValue* args) {
+  std::string callback;
+  if (!args->GetString(0, &callback)) {
+    NOTREACHED();
+    return;
+  }
+  network_state_informer_->AddObserver(callback);
+}
+
+void SigninScreenHandler::HandleLoginRemoveNetworkStateObserver(
+    const base::ListValue* args) {
+  std::string callback;
+  if (!args->GetString(0, &callback)) {
+    NOTREACHED();
+    return;
+  }
+  network_state_informer_->RemoveObserver(callback);
+}
+
 void SigninScreenHandler::HandleCreateAccount(const base::ListValue* args) {
   delegate_->CreateAccount();
 }
 
 void SigninScreenHandler::StartClearingDnsCache() {
-  if (!g_browser_process->io_thread())
+  if (dns_clear_task_running_ || !g_browser_process->io_thread())
     return;
+
   dns_cleared_ = false;
-  if (clear_dns_task_)
-    clear_dns_task_->Cancel();
-  clear_dns_task_ = new ClearDnsCacheTaskOnIOThread(
-      this, g_browser_process->io_thread());
-  BrowserThread::PostTask(BrowserThread::IO, FROM_HERE, clear_dns_task_);
+  ClearDnsCacheTaskOnIOThread* clear_dns_task = new ClearDnsCacheTaskOnIOThread(
+      method_factory_.NewRunnableMethod(&SigninScreenHandler::OnDnsCleared),
+      g_browser_process->io_thread());
+  BrowserThread::PostTask(BrowserThread::IO, FROM_HERE, clear_dns_task);
+  dns_clear_task_running_ = true;
 }
 
 void SigninScreenHandler::StartClearingCookies() {
   cookies_cleared_ = false;
-  BrowsingDataRemover* remover = new BrowsingDataRemover(
-      web_ui_->tab_contents()->profile(),
+  if (cookie_remover_)
+    cookie_remover_->RemoveObserver(this);
+
+  cookie_remover_ = new BrowsingDataRemover(
+      Profile::FromBrowserContext(web_ui_->tab_contents()->browser_context()),
       BrowsingDataRemover::EVERYTHING,
       base::Time());
-  remover->AddObserver(this);
-  remover->Remove(BrowsingDataRemover::REMOVE_COOKIES |
-                  BrowsingDataRemover::REMOVE_LSO_DATA);
+  cookie_remover_->AddObserver(this);
+  cookie_remover_->Remove(BrowsingDataRemover::REMOVE_SITE_DATA);
 }
 
 }  // namespace chromeos
