@@ -130,23 +130,22 @@ void CollectedCookiesSource::StartDataRequest(const std::string& path,
 }  // namespace
 
 // static
-void CollectedCookiesUIDelegate::Show(TabContents* tab_contents) {
+void CollectedCookiesUIDelegate::Show(TabContentsWrapper* wrapper) {
   CollectedCookiesUIDelegate* delegate =
-      new CollectedCookiesUIDelegate(tab_contents);
-  ConstrainedHtmlUI::CreateConstrainedHtmlDialog(tab_contents->profile(),
+      new CollectedCookiesUIDelegate(wrapper);
+  Profile* profile = wrapper->profile();
+  ConstrainedHtmlUI::CreateConstrainedHtmlDialog(profile,
                                                  delegate,
-                                                 tab_contents);
+                                                 wrapper->tab_contents());
 }
 
 CollectedCookiesUIDelegate::CollectedCookiesUIDelegate(
-    TabContents* tab_contents)
-    : tab_contents_(tab_contents),
+    TabContentsWrapper* wrapper)
+    : wrapper_(wrapper),
       closed_(false) {
-  TabSpecificContentSettings* content_settings =
-      TabContentsWrapper::GetCurrentWrapperForContents(tab_contents)->
-          content_settings();
+  TabSpecificContentSettings* content_settings = wrapper->content_settings();
   HostContentSettingsMap* host_content_settings_map =
-      tab_contents_->profile()->GetHostContentSettingsMap();
+      wrapper->profile()->GetHostContentSettingsMap();
 
   registrar_.Add(this, chrome::NOTIFICATION_COLLECTED_COOKIES_SHOWN,
                  Source<TabSpecificContentSettings>(content_settings));
@@ -158,7 +157,7 @@ CollectedCookiesUIDelegate::CollectedCookiesUIDelegate(
 
   CollectedCookiesSource* source = new CollectedCookiesSource(
       host_content_settings_map->BlockThirdPartyCookies());
-  tab_contents->profile()->GetChromeURLDataManager()->AddDataSource(source);
+  wrapper->profile()->GetChromeURLDataManager()->AddDataSource(source);
 }
 
 CollectedCookiesUIDelegate::~CollectedCookiesUIDelegate() {
@@ -168,8 +167,8 @@ bool CollectedCookiesUIDelegate::IsDialogModal() const {
   return false;
 }
 
-std::wstring CollectedCookiesUIDelegate::GetDialogTitle() const {
-  return std::wstring();
+string16 CollectedCookiesUIDelegate::GetDialogTitle() const {
+  return string16();
 }
 
 GURL CollectedCookiesUIDelegate::GetDialogContentURL() const {
@@ -227,8 +226,9 @@ void CollectedCookiesUIDelegate::SetInfobarLabel(const std::string& text) {
 void CollectedCookiesUIDelegate::AddContentException(
     CookieTreeOriginNode* origin_node, ContentSetting setting) {
   if (origin_node->CanCreateContentException()) {
-    origin_node->CreateContentException(
-        tab_contents_->profile()->GetHostContentSettingsMap(), setting);
+    Profile* profile = wrapper_->profile();
+    origin_node->CreateContentException(profile->GetHostContentSettingsMap(),
+                                        setting);
 
     SetInfobarLabel(GetInfobarLabel(setting, origin_node->GetTitle()));
   }
