@@ -6,14 +6,14 @@
 #define CHROME_BROWSER_UI_WEBUI_CONFLICTS_UI_H_
 #pragma once
 
-#include "content/browser/webui/web_ui.h"
+#include "chrome/browser/ui/webui/chrome_web_ui.h"
 
 #if defined(OS_WIN)
 
 class RefCountedMemory;
 
 // The Web UI handler for about:conflicts.
-class ConflictsUI : public WebUI {
+class ConflictsUI : public ChromeWebUI {
  public:
   explicit ConflictsUI(TabContents* contents);
 
