@@ -18,10 +18,12 @@
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
 #include "chrome/browser/bookmarks/bookmark_model.h"
+#include "chrome/browser/history/history_notifications.h"
 #include "chrome/browser/history/history_types.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/browser.h"
 #include "chrome/browser/ui/browser_list.h"
+#include "chrome/browser/ui/webui/chrome_web_ui_data_source.h"
 #include "chrome/browser/ui/webui/favicon_source.h"
 #include "chrome/common/chrome_notification_types.h"
 #include "chrome/common/jstemplate_builder.h"
@@ -32,6 +34,7 @@
 #include "content/browser/tab_contents/tab_contents_delegate.h"
 #include "content/browser/user_metrics.h"
 #include "content/common/notification_source.h"
+#include "content/common/notification_details.h"
 #include "grit/browser_resources.h"
 #include "grit/chromium_strings.h"
 #include "grit/generated_resources.h"
@@ -46,69 +49,42 @@
 // eventually remove this.
 static const int kMaxSearchResults = 100;
 
-////////////////////////////////////////////////////////////////////////////////
-//
-// HistoryHTMLSource
-//
-////////////////////////////////////////////////////////////////////////////////
+namespace {
 
-HistoryUIHTMLSource2::HistoryUIHTMLSource2()
-    : DataSource(chrome::kChromeUIHistory2Host, MessageLoop::current()) {
-}
+ChromeWebUIDataSource* CreateHistory2UIHTMLSource() {
+  ChromeWebUIDataSource* source =
+      new ChromeWebUIDataSource(chrome::kChromeUIHistory2Host);
 
-void HistoryUIHTMLSource2::StartDataRequest(const std::string& path,
-                                            bool is_incognito,
-                                            int request_id) {
-  DictionaryValue localized_strings;
-  localized_strings.SetString("loading",
-      l10n_util::GetStringUTF16(IDS_HISTORY_LOADING));
-  localized_strings.SetString("title",
-      l10n_util::GetStringUTF16(IDS_HISTORY_TITLE));
-  localized_strings.SetString("loading",
-      l10n_util::GetStringUTF16(IDS_HISTORY_LOADING));
-  localized_strings.SetString("newest",
-      l10n_util::GetStringUTF16(IDS_HISTORY_NEWEST));
-  localized_strings.SetString("newer",
-      l10n_util::GetStringUTF16(IDS_HISTORY_NEWER));
-  localized_strings.SetString("older",
-      l10n_util::GetStringUTF16(IDS_HISTORY_OLDER));
-  localized_strings.SetString("searchresultsfor",
-      l10n_util::GetStringUTF16(IDS_HISTORY_SEARCHRESULTSFOR));
-  localized_strings.SetString("history",
-      l10n_util::GetStringUTF16(IDS_HISTORY_BROWSERESULTS));
-  localized_strings.SetString("cont",
-      l10n_util::GetStringUTF16(IDS_HISTORY_CONTINUED));
-  localized_strings.SetString("searchbutton",
-      l10n_util::GetStringUTF16(IDS_HISTORY_SEARCH_BUTTON));
-  localized_strings.SetString("noresults",
-      l10n_util::GetStringUTF16(IDS_HISTORY_NO_RESULTS));
-  localized_strings.SetString("noitems",
-      l10n_util::GetStringUTF16(IDS_HISTORY_NO_ITEMS));
-  localized_strings.SetString("edithistory",
-      l10n_util::GetStringUTF16(IDS_HISTORY_START_EDITING_HISTORY));
-  localized_strings.SetString("doneediting",
-      l10n_util::GetStringUTF16(IDS_HISTORY_STOP_EDITING_HISTORY));
-  localized_strings.SetString("removeselected",
-      l10n_util::GetStringUTF16(IDS_HISTORY_REMOVE_SELECTED_ITEMS));
-  localized_strings.SetString("clearallhistory",
-      l10n_util::GetStringUTF16(IDS_HISTORY_OPEN_CLEAR_BROWSING_DATA_DIALOG));
-  localized_strings.SetString("deletewarning",
-      l10n_util::GetStringUTF16(IDS_HISTORY_DELETE_PRIOR_VISITS_WARNING));
+  source->AddLocalizedString("loading", IDS_HISTORY_LOADING);
+  source->AddLocalizedString("title", IDS_HISTORY_TITLE);
+  source->AddLocalizedString("newest", IDS_HISTORY_NEWEST);
+  source->AddLocalizedString("newer", IDS_HISTORY_NEWER);
+  source->AddLocalizedString("older", IDS_HISTORY_OLDER);
+  source->AddLocalizedString("searchresultsfor", IDS_HISTORY_SEARCHRESULTSFOR);
+  source->AddLocalizedString("history", IDS_HISTORY_BROWSERESULTS);
+  source->AddLocalizedString("cont", IDS_HISTORY_CONTINUED);
+  source->AddLocalizedString("searchbutton", IDS_HISTORY_SEARCH_BUTTON);
+  source->AddLocalizedString("noresults", IDS_HISTORY_NO_RESULTS);
+  source->AddLocalizedString("noitems", IDS_HISTORY_NO_ITEMS);
+  source->AddLocalizedString("edithistory", IDS_HISTORY_START_EDITING_HISTORY);
+  source->AddLocalizedString("doneediting", IDS_HISTORY_STOP_EDITING_HISTORY);
+  source->AddLocalizedString("removeselected",
+                             IDS_HISTORY_REMOVE_SELECTED_ITEMS);
+  source->AddLocalizedString("clearallhistory",
+                             IDS_HISTORY_OPEN_CLEAR_BROWSING_DATA_DIALOG);
+  source->AddLocalizedString("deletewarning",
+                             IDS_HISTORY_DELETE_PRIOR_VISITS_WARNING);
+  source->AddLocalizedString("actionMenuDescription",
+                             IDS_HISTORY_ACTION_MENU_DESCRIPTION);
+  source->AddLocalizedString("removeFromHistory", IDS_HISTORY_REMOVE_PAGE);
+  source->AddLocalizedString("moreFromSite", IDS_HISTORY_MORE_FROM_SITE);
+  source->set_json_path("strings.js");
+  source->add_resource_path("history2.js", IDR_HISTORY2_JS);
+  source->set_default_resource(IDR_HISTORY2_HTML);
+  return source;
+};
 
-  SetFontAndTextDirection(&localized_strings);
-
-  static const base::StringPiece history_html(
-      ResourceBundle::GetSharedInstance().GetRawDataResource(
-          IDR_HISTORY2_HTML));
-  std::string full_html = jstemplate_builder::GetI18nTemplateHtml(
-      history_html, &localized_strings);
-
-  SendResponse(request_id, base::RefCountedString::TakeString(&full_html));
-}
-
-std::string HistoryUIHTMLSource2::GetMimeType(const std::string&) const {
-  return "text/html";
-}
+}  // namespace
 
 ////////////////////////////////////////////////////////////////////////////////
 //
@@ -126,13 +102,13 @@ BrowsingHistoryHandler2::~BrowsingHistoryHandler2() {
 
 WebUIMessageHandler* BrowsingHistoryHandler2::Attach(WebUI* web_ui) {
   // Create our favicon data source.
-  Profile* profile = web_ui->GetProfile();
+  Profile* profile = Profile::FromWebUI(web_ui);
   profile->GetChromeURLDataManager()->AddDataSource(
       new FaviconSource(profile, FaviconSource::FAVICON));
 
   // Get notifications when history is cleared.
   registrar_.Add(this, chrome::NOTIFICATION_HISTORY_URLS_DELETED,
-      Source<Profile>(web_ui->GetProfile()->GetOriginalProfile()));
+      Source<Profile>(profile->GetOriginalProfile()));
   return WebUIMessageHandler::Attach(web_ui);
 }
 
@@ -166,7 +142,7 @@ void BrowsingHistoryHandler2::HandleGetHistory(const ListValue* args) {
   search_text_ = string16();
 
   HistoryService* hs =
-      web_ui_->GetProfile()->GetHistoryService(Profile::EXPLICIT_ACCESS);
+      Profile::FromWebUI(web_ui_)->GetHistoryService(Profile::EXPLICIT_ACCESS);
   hs->QueryHistory(search_text_,
       options,
       &cancelable_search_consumer_,
@@ -191,7 +167,7 @@ void BrowsingHistoryHandler2::HandleSearchHistory(const ListValue* args) {
   // Need to remember the query string for our results.
   search_text_ = query;
   HistoryService* hs =
-      web_ui_->GetProfile()->GetHistoryService(Profile::EXPLICIT_ACCESS);
+      Profile::FromWebUI(web_ui_)->GetHistoryService(Profile::EXPLICIT_ACCESS);
   hs->QueryHistory(search_text_,
       options,
       &cancelable_search_consumer_,
@@ -206,7 +182,11 @@ void BrowsingHistoryHandler2::HandleRemoveURLsOnOneDay(const ListValue* args) {
 
   // Get day to delete data from.
   int visit_time = 0;
-  ExtractIntegerValue(args, &visit_time);
+  if (!ExtractIntegerValue(args, &visit_time)) {
+    LOG(ERROR) << "Unable to extract integer argument.";
+    web_ui_->CallJavascriptFunction("deleteFailed");
+    return;
+  }
   base::Time::Exploded exploded;
   base::Time::FromTimeT(
       static_cast<time_t>(visit_time)).LocalExplode(&exploded);
@@ -215,7 +195,7 @@ void BrowsingHistoryHandler2::HandleRemoveURLsOnOneDay(const ListValue* args) {
   base::Time end_time = begin_time + base::TimeDelta::FromDays(1);
 
   // Get URLs.
-  std::set<GURL> urls;
+  DCHECK(urls_to_be_deleted_.empty());
   for (ListValue::const_iterator v = args->begin() + 1;
        v != args->end(); ++v) {
     if ((*v)->GetType() != Value::TYPE_STRING)
@@ -224,20 +204,22 @@ void BrowsingHistoryHandler2::HandleRemoveURLsOnOneDay(const ListValue* args) {
     string16 string16_value;
     if (!string_value->GetAsString(&string16_value))
       continue;
-    urls.insert(GURL(string16_value));
+
+    urls_to_be_deleted_.insert(GURL(string16_value));
   }
 
   HistoryService* hs =
-      web_ui_->GetProfile()->GetHistoryService(Profile::EXPLICIT_ACCESS);
+      Profile::FromWebUI(web_ui_)->GetHistoryService(Profile::EXPLICIT_ACCESS);
   hs->ExpireHistoryBetween(
-      urls, begin_time, end_time, &cancelable_delete_consumer_,
+      urls_to_be_deleted_, begin_time, end_time, &cancelable_delete_consumer_,
       NewCallback(this, &BrowsingHistoryHandler2::RemoveComplete));
 }
 
 void BrowsingHistoryHandler2::HandleClearBrowsingData(const ListValue* args) {
   // TODO(beng): This is an improper direct dependency on Browser. Route this
   // through some sort of delegate.
-  Browser* browser = BrowserList::FindBrowserWithProfile(web_ui_->GetProfile());
+  Profile* profile = Profile::FromWebUI(web_ui_);
+  Browser* browser = BrowserList::FindBrowserWithProfile(profile);
   if (browser)
     browser->OpenClearBrowsingDataDialog();
 }
@@ -284,8 +266,9 @@ void BrowsingHistoryHandler2::QueryComplete(
           base::TimeFormatShortDate(page.visit_time()));
       page_value->SetString("snippet", page.snippet().text());
     }
+    Profile* profile = Profile::FromWebUI(web_ui_);
     page_value->SetBoolean("starred",
-        web_ui_->GetProfile()->GetBookmarkModel()->IsBookmarked(page.url()));
+        profile->GetBookmarkModel()->IsBookmarked(page.url()));
     results_value.Append(page_value);
   }
 
@@ -297,7 +280,9 @@ void BrowsingHistoryHandler2::QueryComplete(
 }
 
 void BrowsingHistoryHandler2::RemoveComplete() {
-  // Some Visits were deleted from history. Reload the list.
+  urls_to_be_deleted_.clear();
+
+  // Notify the page that the deletion request succeeded.
   web_ui_->CallJavascriptFunction("deleteComplete");
 }
 
@@ -377,9 +362,12 @@ void BrowsingHistoryHandler2::Observe(int type,
     NOTREACHED();
     return;
   }
-
-  // Some URLs were deleted from history. Reload the list.
-  web_ui_->CallJavascriptFunction("historyDeleted");
+  history::URLsDeletedDetails* deletedDetails =
+      Details<history::URLsDeletedDetails>(details).ptr();
+  if (deletedDetails->urls != urls_to_be_deleted_) {
+    // Notify the page that someone else deleted from the history.
+    web_ui_->CallJavascriptFunction("historyDeleted");
+  }
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -391,10 +379,10 @@ void BrowsingHistoryHandler2::Observe(int type,
 HistoryUI2::HistoryUI2(TabContents* contents) : ChromeWebUI(contents) {
   AddMessageHandler((new BrowsingHistoryHandler2())->Attach(this));
 
-  HistoryUIHTMLSource2* html_source = new HistoryUIHTMLSource2();
-
   // Set up the chrome://history2/ source.
-  contents->profile()->GetChromeURLDataManager()->AddDataSource(html_source);
+  Profile* profile = Profile::FromBrowserContext(contents->browser_context());
+  profile->GetChromeURLDataManager()->AddDataSource(
+      CreateHistory2UIHTMLSource());
 }
 
 // static
