@@ -63,9 +63,9 @@ class HistoryUIHTMLSource : public ChromeWebUIDataSource {
   // the path we registered.
   virtual void StartDataRequest(const std::string& path,
                                 bool is_incognito,
-                                int request_id);
+                                int request_id) OVERRIDE;
 
-  virtual std::string GetMimeType(const std::string& path) const;
+  virtual std::string GetMimeType(const std::string& path) const OVERRIDE;
 
  private:
   ~HistoryUIHTMLSource();
@@ -133,7 +133,7 @@ BrowsingHistoryHandler::~BrowsingHistoryHandler() {
 
 WebUIMessageHandler* BrowsingHistoryHandler::Attach(WebUI* web_ui) {
   // Create our favicon data source.
-  Profile* profile = web_ui->GetProfile();
+  Profile* profile = Profile::FromWebUI(web_ui);
   profile->GetChromeURLDataManager()->AddDataSource(
       new FaviconSource(profile, FaviconSource::FAVICON));
 
@@ -173,7 +173,7 @@ void BrowsingHistoryHandler::HandleGetHistory(const ListValue* args) {
   search_text_ = string16();
 
   HistoryService* hs =
-      web_ui_->GetProfile()->GetHistoryService(Profile::EXPLICIT_ACCESS);
+      Profile::FromWebUI(web_ui_)->GetHistoryService(Profile::EXPLICIT_ACCESS);
   hs->QueryHistory(search_text_,
       options,
       &cancelable_search_consumer_,
@@ -198,7 +198,7 @@ void BrowsingHistoryHandler::HandleSearchHistory(const ListValue* args) {
   // Need to remember the query string for our results.
   search_text_ = query;
   HistoryService* hs =
-      web_ui_->GetProfile()->GetHistoryService(Profile::EXPLICIT_ACCESS);
+      Profile::FromWebUI(web_ui_)->GetHistoryService(Profile::EXPLICIT_ACCESS);
   hs->QueryHistory(search_text_,
       options,
       &cancelable_search_consumer_,
@@ -235,7 +235,7 @@ void BrowsingHistoryHandler::HandleRemoveURLsOnOneDay(const ListValue* args) {
   }
 
   HistoryService* hs =
-      web_ui_->GetProfile()->GetHistoryService(Profile::EXPLICIT_ACCESS);
+      Profile::FromWebUI(web_ui_)->GetHistoryService(Profile::EXPLICIT_ACCESS);
   hs->ExpireHistoryBetween(
       urls, begin_time, end_time, &cancelable_delete_consumer_,
       NewCallback(this, &BrowsingHistoryHandler::RemoveComplete));
@@ -244,7 +244,8 @@ void BrowsingHistoryHandler::HandleRemoveURLsOnOneDay(const ListValue* args) {
 void BrowsingHistoryHandler::HandleClearBrowsingData(const ListValue* args) {
   // TODO(beng): This is an improper direct dependency on Browser. Route this
   // through some sort of delegate.
-  Browser* browser = BrowserList::FindBrowserWithProfile(web_ui_->GetProfile());
+  Profile* profile = Profile::FromWebUI(web_ui_);
+  Browser* browser = BrowserList::FindBrowserWithProfile(profile);
   if (browser)
     browser->OpenClearBrowsingDataDialog();
 }
@@ -291,8 +292,9 @@ void BrowsingHistoryHandler::QueryComplete(
           base::TimeFormatShortDate(page.visit_time()));
       page_value->SetString("snippet", page.snippet().text());
     }
+    Profile* profile = Profile::FromWebUI(web_ui_);
     page_value->SetBoolean("starred",
-        web_ui_->GetProfile()->GetBookmarkModel()->IsBookmarked(page.url()));
+        profile->GetBookmarkModel()->IsBookmarked(page.url()));
     results_value.Append(page_value);
   }
 
@@ -389,7 +391,8 @@ HistoryUI::HistoryUI(TabContents* contents) : ChromeWebUI(contents) {
   HistoryUIHTMLSource* html_source = new HistoryUIHTMLSource();
 
   // Set up the chrome://history/ source.
-  contents->profile()->GetChromeURLDataManager()->AddDataSource(html_source);
+  Profile* profile = Profile::FromBrowserContext(contents->browser_context());
+  profile->GetChromeURLDataManager()->AddDataSource(html_source);
 }
 
 // static
