@@ -25,8 +25,8 @@ class BrowsingHistoryHandler : public WebUIMessageHandler,
   virtual ~BrowsingHistoryHandler();
 
   // WebUIMessageHandler implementation.
-  virtual WebUIMessageHandler* Attach(WebUI* web_ui);
-  virtual void RegisterMessages();
+  virtual WebUIMessageHandler* Attach(WebUI* web_ui) OVERRIDE;
+  virtual void RegisterMessages() OVERRIDE;
 
   // Callback for the "getHistory" message.
   void HandleGetHistory(const base::ListValue* args);
@@ -43,7 +43,7 @@ class BrowsingHistoryHandler : public WebUIMessageHandler,
   // NotificationObserver implementation.
   virtual void Observe(int type,
                        const NotificationSource& source,
-                       const NotificationDetails& details);
+                       const NotificationDetails& details) OVERRIDE;
 
  private:
   // Callback from the history system when the history list is available.
