@@ -9,6 +9,7 @@
 #include "chrome/browser/ui/webui/media/media_internals_proxy.h"
 #include "content/browser/browser_thread.h"
 #include "content/browser/renderer_host/render_view_host.h"
+#include "content/browser/tab_contents/tab_contents.h"
 
 MediaInternalsMessageHandler::MediaInternalsMessageHandler()
     : proxy_(new MediaInternalsProxy()) {}
@@ -37,5 +38,8 @@ void MediaInternalsMessageHandler::OnGetEverything(const ListValue* list) {
 }
 
 void MediaInternalsMessageHandler::OnUpdate(const string16& update) {
-  web_ui_->GetRenderViewHost()->ExecuteJavascriptInWebFrame(string16(), update);
+  // Don't try to execute JavaScript in a RenderView that no longer exists.
+  RenderViewHost* host = web_ui_->tab_contents()->render_view_host();
+  if (host)
+    host->ExecuteJavascriptInWebFrame(string16(), update);
 }
