@@ -30,17 +30,13 @@ class NewTabUI : public ChromeWebUI,
 
   // Override WebUI methods so we can hook up the paint timer to the render
   // view host.
-  virtual void RenderViewCreated(RenderViewHost* render_view_host);
-  virtual void RenderViewReused(RenderViewHost* render_view_host);
+  virtual void RenderViewCreated(RenderViewHost* render_view_host) OVERRIDE;
+  virtual void RenderViewReused(RenderViewHost* render_view_host) OVERRIDE;
 
   static void RegisterUserPrefs(PrefService* prefs);
   static void MigrateUserPrefs(PrefService* prefs, int old_pref_version,
                                int new_pref_version);
 
-  // Whether we should disable the first run notification based on the command
-  // line switch.
-  static bool FirstRunDisabled();
-
   // Adds "url", "title", and "direction" keys on incoming dictionary, setting
   // title as the url as a fallback on empty title.
   static void SetURLTitleAndDirection(base::DictionaryValue* dictionary,
@@ -50,6 +46,9 @@ class NewTabUI : public ChromeWebUI,
   // The current preference version.
   static int current_pref_version() { return current_pref_version_; }
 
+  // Returns whether NTP4 is enabled.
+  static bool NTP4Enabled();
+
   class NewTabHTMLSource : public ChromeURLDataManager::DataSource {
    public:
     explicit NewTabHTMLSource(Profile* profile);
@@ -58,22 +57,15 @@ class NewTabUI : public ChromeWebUI,
     // the path we registered.
     virtual void StartDataRequest(const std::string& path,
                                   bool is_incognito,
-                                  int request_id);
+                                  int request_id) OVERRIDE;
 
-    virtual std::string GetMimeType(const std::string&) const;
+    virtual std::string GetMimeType(const std::string&) const OVERRIDE;
 
     virtual bool ShouldReplaceExistingSource() const;
 
-    // Setters and getters for first_run.
-    static void set_first_run(bool first_run) { first_run_ = first_run; }
-    static bool first_run() { return first_run_; }
-
    private:
     virtual ~NewTabHTMLSource() {}
 
-    // Whether this is the first run.
-    static bool first_run_;
-
     // Pointer back to the original profile.
     Profile* profile_;
 
