@@ -136,7 +136,7 @@ void BrowserOptionsHandler::RegisterMessages() {
 }
 
 void BrowserOptionsHandler::Initialize() {
-  Profile* profile = web_ui_->GetProfile();
+  Profile* profile = Profile::FromWebUI(web_ui_);
 
   // Create our favicon data source.
   profile->GetChromeURLDataManager()->AddDataSource(
@@ -207,7 +207,7 @@ void BrowserOptionsHandler::BecomeDefaultBrowser(const ListValue* args) {
   if (default_browser_policy_.IsManaged())
     return;
 
-  UserMetricsRecordAction(UserMetricsAction("Options_SetAsDefaultBrowser"));
+  UserMetrics::RecordAction(UserMetricsAction("Options_SetAsDefaultBrowser"));
 #if defined(OS_MACOSX)
   if (ShellIntegration::SetAsDefaultBrowser())
     UpdateDefaultBrowserState();
@@ -218,7 +218,7 @@ void BrowserOptionsHandler::BecomeDefaultBrowser(const ListValue* args) {
 
   // If the user attempted to make Chrome the default browser, then he/she
   // arguably wants to be notified when that changes.
-  PrefService* prefs = web_ui_->GetProfile()->GetPrefs();
+  PrefService* prefs = Profile::FromWebUI(web_ui_)->GetPrefs();
   prefs->SetBoolean(prefs::kCheckDefaultBrowser, true);
 }
 
@@ -308,12 +308,12 @@ void BrowserOptionsHandler::SetDefaultSearchEngine(const ListValue* args) {
       selected_index < static_cast<int>(model_urls.size()))
     template_url_service_->SetDefaultSearchProvider(model_urls[selected_index]);
 
-  UserMetricsRecordAction(UserMetricsAction("Options_SearchEngineChanged"));
+  UserMetrics::RecordAction(UserMetricsAction("Options_SearchEngineChanged"));
 }
 
 void BrowserOptionsHandler::UpdateSearchEngines() {
-  template_url_service_ = TemplateURLServiceFactory::GetForProfile(
-      web_ui_->GetProfile());
+  template_url_service_ =
+      TemplateURLServiceFactory::GetForProfile(Profile::FromWebUI(web_ui_));
   if (template_url_service_) {
     template_url_service_->Load();
     template_url_service_->AddObserver(this);
@@ -322,7 +322,7 @@ void BrowserOptionsHandler::UpdateSearchEngines() {
 }
 
 void BrowserOptionsHandler::UpdateStartupPages() {
-  Profile* profile = web_ui_->GetProfile();
+  Profile* profile = Profile::FromWebUI(web_ui_);
   const SessionStartupPref startup_pref =
       SessionStartupPref::GetStartupPref(profile->GetPrefs());
   startup_custom_pages_table_model_->SetURLs(startup_pref.urls);
@@ -454,7 +454,7 @@ void BrowserOptionsHandler::DragDropStartupPage(const ListValue* args) {
 }
 
 void BrowserOptionsHandler::SaveStartupPagesPref() {
-  PrefService* prefs = web_ui_->GetProfile()->GetPrefs();
+  PrefService* prefs = Profile::FromWebUI(web_ui_)->GetPrefs();
 
   SessionStartupPref pref = SessionStartupPref::GetStartupPref(prefs);
   pref.urls = startup_custom_pages_table_model_->GetURLs();
@@ -473,7 +473,7 @@ void BrowserOptionsHandler::RequestAutocompleteSuggestions(
 }
 
 void BrowserOptionsHandler::ToggleShowBookmarksBar(const ListValue* args) {
-  Source<Profile> source(web_ui_->GetProfile());
+  Source<Profile> source(Profile::FromWebUI(web_ui_));
   NotificationService::current()->Notify(
       chrome::NOTIFICATION_BOOKMARK_BAR_VISIBILITY_PREF_CHANGED,
       source,
@@ -481,16 +481,16 @@ void BrowserOptionsHandler::ToggleShowBookmarksBar(const ListValue* args) {
 }
 
 void BrowserOptionsHandler::EnableInstant(const ListValue* args) {
-  InstantController::Enable(web_ui_->GetProfile());
+  InstantController::Enable(Profile::FromWebUI(web_ui_));
 }
 
 void BrowserOptionsHandler::DisableInstant(const ListValue* args) {
-  InstantController::Disable(web_ui_->GetProfile());
+  InstantController::Disable(Profile::FromWebUI(web_ui_));
 }
 
 void BrowserOptionsHandler::GetInstantFieldTrialStatus(const ListValue* args) {
-  FundamentalValue enabled(
-      InstantFieldTrial::IsExperimentGroup(web_ui_->GetProfile()));
+  base::FundamentalValue enabled(
+      InstantFieldTrial::IsExperimentGroup(Profile::FromWebUI(web_ui_)));
   web_ui_->CallJavascriptFunction("BrowserOptions.setInstantFieldTrialStatus",
                                   enabled);
 }
