@@ -8,8 +8,6 @@
 
 #include "chrome/browser/ui/webui/options/chromeos/cros_options_page_ui_handler.h"
 
-class OptionsManagedBannerHandler;
-
 namespace chromeos {
 
 class UserCrosSettingsProvider;
@@ -24,20 +22,17 @@ class AccountsOptionsHandler : public CrosOptionsPageUIHandler {
   virtual void RegisterMessages();
 
   // OptionsPageUIHandler implementation.
-  virtual void GetLocalizedValues(DictionaryValue* localized_strings);
-  virtual void Initialize();
+  virtual void GetLocalizedValues(base::DictionaryValue* localized_strings);
 
  private:
   UserCrosSettingsProvider* users_settings() const;
 
   // Javascript callbacks to whitelist/unwhitelist user.
-  void WhitelistUser(const ListValue* args);
-  void UnwhitelistUser(const ListValue* args);
+  void WhitelistUser(const base::ListValue* args);
+  void UnwhitelistUser(const base::ListValue* args);
 
   // Javascript callback to auto add existing users to white list.
-  void WhitelistExistingUsers(const ListValue* args);
-
-  scoped_ptr<OptionsManagedBannerHandler> banner_handler_;
+  void WhitelistExistingUsers(const base::ListValue* args);
 
   DISALLOW_COPY_AND_ASSIGN(AccountsOptionsHandler);
 };
