@@ -8,6 +8,7 @@
 #include <set>
 #include <vector>
 
+#include "base/stringprintf.h"
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
 #include "chrome/app/chrome_command_ids.h"
@@ -55,6 +56,9 @@ void CrosLanguageOptionsHandler::GetLocalizedValues(
   localized_strings->SetString("restart_button",
       l10n_util::GetStringUTF16(
           IDS_OPTIONS_SETTINGS_LANGUAGES_SIGN_OUT_BUTTON));
+  localized_strings->SetString("virtual_keyboard_button",
+      l10n_util::GetStringUTF16(
+          IDS_OPTIONS_SETTINGS_LANGUAGES_VIRTUAL_KEYBOARD_BUTTON));
 
   // GetSupportedInputMethods() never return NULL.
   scoped_ptr<input_method::InputMethodDescriptors> descriptors(
@@ -182,7 +186,7 @@ string16 CrosLanguageOptionsHandler::GetProductName() {
 
 void CrosLanguageOptionsHandler::SetApplicationLocale(
     const std::string& language_code) {
-  web_ui_->GetProfile()->ChangeAppLocale(
+  Profile::FromWebUI(web_ui_)->ChangeAppLocale(
       language_code, Profile::APP_LOCALE_CHANGED_VIA_SETTINGS);
 }
 
@@ -198,7 +202,7 @@ void CrosLanguageOptionsHandler::RestartCallback(const ListValue* args) {
 void CrosLanguageOptionsHandler::InputMethodDisableCallback(
     const ListValue* args) {
   const std::string input_method_id = UTF16ToASCII(ExtractStringValue(args));
-  const std::string action = StringPrintf(
+  const std::string action = base::StringPrintf(
       "LanguageOptions_DisableInputMethod_%s", input_method_id.c_str());
   UserMetrics::RecordComputedAction(action);
 }
@@ -206,7 +210,7 @@ void CrosLanguageOptionsHandler::InputMethodDisableCallback(
 void CrosLanguageOptionsHandler::InputMethodEnableCallback(
     const ListValue* args) {
   const std::string input_method_id = UTF16ToASCII(ExtractStringValue(args));
-  const std::string action = StringPrintf(
+  const std::string action = base::StringPrintf(
       "LanguageOptions_EnableInputMethod_%s", input_method_id.c_str());
   UserMetrics::RecordComputedAction(action);
 }
@@ -214,7 +218,7 @@ void CrosLanguageOptionsHandler::InputMethodEnableCallback(
 void CrosLanguageOptionsHandler::InputMethodOptionsOpenCallback(
     const ListValue* args) {
   const std::string input_method_id = UTF16ToASCII(ExtractStringValue(args));
-  const std::string action = StringPrintf(
+  const std::string action = base::StringPrintf(
       "InputMethodOptions_Open_%s", input_method_id.c_str());
   UserMetrics::RecordComputedAction(action);
 }
