@@ -7,14 +7,15 @@
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
 #include "chrome/browser/browsing_data_appcache_helper.h"
+#include "chrome/browser/browsing_data_cookie_helper.h"
 #include "chrome/browser/browsing_data_database_helper.h"
 #include "chrome/browser/browsing_data_file_system_helper.h"
 #include "chrome/browser/browsing_data_indexed_db_helper.h"
+#include "chrome/browser/browsing_data_quota_helper.h"
 #include "chrome/browser/browsing_data_local_storage_helper.h"
 #include "chrome/browser/profiles/profile.h"
 #include "chrome/browser/ui/webui/cookies_tree_model_util.h"
 #include "grit/generated_resources.h"
-#include "net/url_request/url_request_context_getter.h"
 #include "ui/base/l10n/l10n_util.h"
 
 CookiesViewHandler::CookiesViewHandler() : batch_update_(false) {
@@ -66,7 +67,7 @@ void CookiesViewHandler::GetLocalizedValues(
     { "label_file_system_temporary_usage",
       IDS_COOKIES_FILE_SYSTEM_TEMPORARY_USAGE_LABEL },
     { "label_file_system_persistent_usage",
-      IDS_COOKIES_FILE_SYSTEM_PERSISTENT_USAGE_LABEL }
+      IDS_COOKIES_FILE_SYSTEM_PERSISTENT_USAGE_LABEL },
   };
 
   RegisterStrings(localized_strings, resources, arraysize(resources));
@@ -141,16 +142,16 @@ void CookiesViewHandler::TreeModelEndBatch(CookiesTreeModel* model) {
 
 void CookiesViewHandler::EnsureCookiesTreeModelCreated() {
   if (!cookies_tree_model_.get()) {
-    Profile* profile = web_ui_->GetProfile();
+    Profile* profile = Profile::FromWebUI(web_ui_);
     cookies_tree_model_.reset(new CookiesTreeModel(
-        profile->GetRequestContext()->DONTUSEME_GetCookieStore()->
-            GetCookieMonster(),
+        new BrowsingDataCookieHelper(profile),
         new BrowsingDataDatabaseHelper(profile),
         new BrowsingDataLocalStorageHelper(profile),
         NULL,
         new BrowsingDataAppCacheHelper(profile),
         BrowsingDataIndexedDBHelper::Create(profile),
         BrowsingDataFileSystemHelper::Create(profile),
+        BrowsingDataQuotaHelper::Create(profile),
         false));
     cookies_tree_model_->AddCookiesTreeObserver(this);
   }
