@@ -23,6 +23,19 @@
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
 
+namespace{
+
+#if defined(OS_MACOSX)
+// U+0028 U+21E7 U+2318 U+0050 U+0029 in UTF8
+const char kAdvancedPrintShortcut[] = "\x28\xE2\x8c\xA5\xE2\x8C\x98\x50\x29";
+#elif defined(OS_WIN)
+const char kAdvancedPrintShortcut[] = "(Ctrl+Shift+P)";
+#else
+const char kAdvancedPrintShortcut[] = "(Shift+Ctrl+P)";
+#endif
+
+};  // namespace
+
 PrintPreviewDataSource::PrintPreviewDataSource()
     : ChromeWebUIDataSource(chrome::kChromeUIPrintHost) {
 
@@ -36,6 +49,10 @@ PrintPreviewDataSource::PrintPreviewDataSource()
 #endif
   AddLocalizedString("launchNativeDialog", IDS_PRINT_PREVIEW_NATIVE_DIALOG);
   AddLocalizedString("previewFailed", IDS_PRINT_PREVIEW_FAILED);
+  AddLocalizedString("invalidPrinterSettings",
+                     IDS_PRINT_PREVIEW_INVALID_PRINTER_SETTINGS);
+  AddLocalizedString("initiatorTabCrashed",
+                     IDS_PRINT_PREVIEW_INITIATOR_TAB_CRASHED);
   AddLocalizedString("initiatorTabClosed",
                      IDS_PRINT_PREVIEW_INITIATOR_TAB_CLOSED);
   AddLocalizedString("reopenPage", IDS_PRINT_PREVIEW_REOPEN_PAGE);
@@ -48,8 +65,6 @@ PrintPreviewDataSource::PrintPreviewDataSource()
   AddLocalizedString("copiesLabel", IDS_PRINT_PREVIEW_COPIES_LABEL);
   AddLocalizedString("examplePageRangeText",
                      IDS_PRINT_PREVIEW_EXAMPLE_PAGE_RANGE_TEXT);
-  AddLocalizedString("invalidNumberOfCopies",
-                     IDS_PRINT_PREVIEW_INVALID_NUMBER_OF_COPIES);
   AddLocalizedString("layoutLabel", IDS_PRINT_PREVIEW_LAYOUT_LABEL);
   AddLocalizedString("optionAllPages", IDS_PRINT_PREVIEW_OPTION_ALL_PAGES);
   AddLocalizedString("optionBw", IDS_PRINT_PREVIEW_OPTION_BW);
@@ -75,8 +90,11 @@ PrintPreviewDataSource::PrintPreviewDataSource()
                      IDS_PRINT_PREVIEW_PAGE_LABEL_SINGULAR);
   AddLocalizedString("printPreviewPageLabelPlural",
                      IDS_PRINT_PREVIEW_PAGE_LABEL_PLURAL);
-  AddLocalizedString("systemDialogOption",
-                     IDS_PRINT_PREVIEW_SYSTEM_DIALOG_OPTION);
+  const string16 shortcut_text(UTF8ToUTF16(kAdvancedPrintShortcut));
+  AddString("systemDialogOption", l10n_util::GetStringFUTF16(
+      IDS_PRINT_PREVIEW_SYSTEM_DIALOG_OPTION,
+      shortcut_text));
+
   AddLocalizedString("pageRangeInstruction",
                      IDS_PRINT_PREVIEW_PAGE_RANGE_INSTRUCTION);
   AddLocalizedString("copiesInstruction", IDS_PRINT_PREVIEW_COPIES_INSTRUCTION);
@@ -93,6 +111,13 @@ PrintPreviewDataSource::PrintPreviewDataSource()
   AddLocalizedString("incrementTitle", IDS_PRINT_PREVIEW_INCREMENT_TITLE);
   AddLocalizedString("decrementTitle", IDS_PRINT_PREVIEW_DECREMENT_TITLE);
   AddLocalizedString("printPagesLabel", IDS_PRINT_PREVIEW_PRINT_PAGES_LABEL);
+  AddLocalizedString("optionsLabel", IDS_PRINT_PREVIEW_OPTIONS_LABEL);
+  AddLocalizedString("optionHeaderFooter",
+                     IDS_PRINT_PREVIEW_OPTION_HEADER_FOOTER);
+  AddLocalizedString("marginsLabel", IDS_PRINT_PREVIEW_MARGINS_LABEL);
+  AddLocalizedString("defaultMargins", IDS_PRINT_PREVIEW_DEFAULT_MARGINS);
+  AddLocalizedString("noMargins", IDS_PRINT_PREVIEW_NO_MARGINS);
+  AddLocalizedString("customMargins", IDS_PRINT_PREVIEW_CUSTOM_MARGINS);
 
   set_json_path("strings.js");
   add_resource_path("print_preview.js", IDR_PRINT_PREVIEW_JS);
