@@ -8,18 +8,23 @@
 
 #include <string>
 
-#include "base/compiler_specific.h"
 #include "base/basictypes.h"
-#include "chrome/browser/sync/js_event_handler.h"
+#include "base/compiler_specific.h"
+#include "base/memory/weak_ptr.h"
+#include "chrome/browser/sync/js/js_event_handler.h"
+#include "chrome/browser/sync/js/js_reply_handler.h"
 #include "chrome/browser/ui/webui/chrome_web_ui.h"
 
+class ProfileSyncService;
+
 namespace browser_sync {
-class JsFrontend;
+class JsController;
 }  // namespace browser_sync
 
 // The implementation for the chrome://sync-internals page.
 class SyncInternalsUI : public ChromeWebUI,
-                        public browser_sync::JsEventHandler {
+                        public browser_sync::JsEventHandler,
+                        public browser_sync::JsReplyHandler {
  public:
   explicit SyncInternalsUI(TabContents* contents);
   virtual ~SyncInternalsUI();
@@ -45,14 +50,15 @@ class SyncInternalsUI : public ChromeWebUI,
   virtual void HandleJsEvent(
       const std::string& name,
       const browser_sync::JsEventDetails& details) OVERRIDE;
-  virtual void HandleJsMessageReply(
+
+  // browser_sync::JsReplyHandler implementation.
+  virtual void HandleJsReply(
       const std::string& name,
       const browser_sync::JsArgList& args) OVERRIDE;
 
  private:
-  // Returns the sync service's JsFrontend object, or NULL if the sync
-  // service does not exist.
-  browser_sync::JsFrontend* GetJsFrontend();
+  base::WeakPtrFactory<SyncInternalsUI> weak_ptr_factory_;
+  base::WeakPtr<browser_sync::JsController> js_controller_;
 
   DISALLOW_COPY_AND_ASSIGN(SyncInternalsUI);
 };
