@@ -12,21 +12,26 @@ class GpuBlacklistUpdater : public WebResourceService {
  public:
   GpuBlacklistUpdater();
 
+  // Initialize GpuDataManager, and post a SetupOnUIThread task.
+  static void SetupOnFileThread();
+
   // URL of the up-to-date gpu_blacklist.json file.
   static const char* kDefaultGpuBlacklistURL;
 
  private:
   virtual ~GpuBlacklistUpdater();
 
-  virtual void Unpack(const DictionaryValue& parsed_json);
+  virtual void Unpack(const base::DictionaryValue& parsed_json);
+
+  // Initialize GpuBlacklistUpdater and schedule an auto update.
+  static void SetupOnUIThread();
 
   void InitializeGpuBlacklist();
 
   void UpdateGpuBlacklist(
-      const DictionaryValue& gpu_blacklist_cache, bool preliminary);
+      const base::DictionaryValue& gpu_blacklist_cache, bool preliminary);
 
   DISALLOW_COPY_AND_ASSIGN(GpuBlacklistUpdater);
 };
 
 #endif  // CHROME_BROWSER_WEB_RESOURCE_GPU_BLACKLIST_UPDATER_H_
-
