@@ -10,12 +10,12 @@
 #include <string>
 #include <vector>
 
-#include "app/sql/init_status.h"
 #include "base/file_path.h"
 #include "base/memory/ref_counted.h"
 #include "base/synchronization/lock.h"
 #include "chrome/browser/search_engines/template_url_id.h"
 #include "content/browser/browser_thread.h"
+#include "sql/init_status.h"
 
 class AutofillChange;
 class AutofillProfile;
@@ -29,6 +29,7 @@ class SkBitmap;
 class Task;
 class TemplateURL;
 class WebDatabase;
+struct WebIntentData;
 
 namespace base {
 class Thread;
@@ -73,7 +74,8 @@ typedef enum {
   AUTOFILL_PROFILE_RESULT,     // WDResult<AutofillProfile>
   AUTOFILL_PROFILES_RESULT,    // WDResult<std::vector<AutofillProfile*>>
   AUTOFILL_CREDITCARD_RESULT,  // WDResult<CreditCard>
-  AUTOFILL_CREDITCARDS_RESULT  // WDResult<std::vector<CreditCard*>>
+  AUTOFILL_CREDITCARDS_RESULT, // WDResult<std::vector<CreditCard*>>
+  WEB_INTENTS_RESULT           // WDResult<std::vector<string16>>
 } WDResultType;
 
 typedef std::vector<AutofillChange> AutofillChangeList;
@@ -229,9 +231,7 @@ class WebDataService
     virtual ~GenericRequest() {
     }
 
-    T GetArgument() {
-      return arg_;
-    }
+    const T& arg() const { return arg_; }
 
    private:
     T arg_;
@@ -252,13 +252,9 @@ class WebDataService
 
     virtual ~GenericRequest2() { }
 
-    T GetArgument1() {
-      return arg1_;
-    }
+    const T& arg1() const { return arg1_; }
 
-    U GetArgument2() {
-      return arg2_;
-    }
+    const U& arg2() const { return arg2_; }
 
    private:
     T arg1_;
@@ -299,7 +295,7 @@ class WebDataService
   // done on the background thread.
   //
   // Many of the keyword related methods do not return a handle. This is because
-  // the caller (TemplateURLModel) does not need to know when the request is
+  // the caller (TemplateURLService) does not need to know when the request is
   // done.
   void AddKeyword(const TemplateURL& url);
 
@@ -338,6 +334,26 @@ class WebDataService
   // specified web app.
   Handle GetWebAppImages(const GURL& app_url, WebDataServiceConsumer* consumer);
 
+  //////////////////////////////////////////////////////////////////////////////
+  //
+  // Web Intents
+  //
+  //////////////////////////////////////////////////////////////////////////////
+
+  // Adds a web intent provider registration.
+  void AddWebIntent(const WebIntentData& intent);
+
+  // Removes a web intent provider registration.
+  void RemoveWebIntent(const WebIntentData& intent);
+
+  // Get all web intent providers registered for the specified |action|.
+  // |consumer| must not be NULL.
+  Handle GetWebIntents(const string16& action,
+                       WebDataServiceConsumer* consumer);
+
+  // Get all web intent providers registered. |consumer| must not be NULL.
+  Handle GetAllWebIntents(WebDataServiceConsumer* consumer);
+
   //////////////////////////////////////////////////////////////////////////////
   //
   // Token Service
@@ -485,8 +501,8 @@ class WebDataService
 #endif
 
  protected:
-  friend class TemplateURLModelTest;
-  friend class TemplateURLModelTestingProfile;
+  friend class TemplateURLServiceTest;
+  friend class TemplateURLServiceTestingProfile;
   friend class WebDataServiceTest;
   friend class WebDataRequest;
 
@@ -560,6 +576,16 @@ class WebDataService
   void RemoveWebAppImpl(GenericRequest<GURL>* request);
   void GetWebAppImagesImpl(GenericRequest<GURL>* request);
 
+  //////////////////////////////////////////////////////////////////////////////
+  //
+  // Web Intents.
+  //
+  //////////////////////////////////////////////////////////////////////////////
+  void AddWebIntentImpl(GenericRequest<WebIntentData>* request);
+  void RemoveWebIntentImpl(GenericRequest<WebIntentData>* request);
+  void GetWebIntentsImpl(GenericRequest<string16>* request);
+  void GetAllWebIntentsImpl(GenericRequest<std::string>* request);
+
   //////////////////////////////////////////////////////////////////////////////
   //
   // Token Service.
