@@ -14,10 +14,6 @@ namespace switches {
 // base/base_switches.cc instead.
 // -----------------------------------------------------------------------------
 
-// Activate (make foreground) myself on launch.  Helpful when Chrome
-// is launched on the command line (e.g. by Selenium).  Only needed on Mac.
-const char kActivateOnLaunch[]              = "activate-on-launch";
-
 // Allow third party content included on a page to prompt for a HTTP
 // basic auth username/password pair.
 const char kAllowCrossOriginAuthPrompt[]    = "allow-cross-origin-auth-prompt";
@@ -32,6 +28,10 @@ const char kAllowHTTPBackgroundPage[]       = "allow-http-background-page";
 // Don't block outdated plugins.
 const char kAllowOutdatedPlugins[]          = "allow-outdated-plugins";
 
+// By default, an https page cannot run JavaScript, CSS or plug-ins from http
+// URLs. This provides an override to get the old insecure behavior.
+const char kAllowRunningInsecureContent[]   = "allow-running-insecure-content";
+
 // Allows injecting extensions and user scripts on the extensions
 // gallery site. Normally prevented for security reasons, but can be
 // useful for automation testing of the gallery.
@@ -41,14 +41,13 @@ const char kAllowScriptingGallery[]         = "allow-scripting-gallery";
 // websocket proxy.
 const char kAllowWebSocketProxy[]           = "allow-websocket-proxy";
 
+// Allow compositing on chrome:// pages.
+const char kAllowWebUICompositing[]         = "allow-webui-compositing";
+
 // This prevents Chrome from requiring authorization to run certain widely
 // installed but less commonly used plug-ins.
 const char kAlwaysAuthorizePlugins[]        = "always-authorize-plugins";
 
-// Enable web inspector for all windows, even if they're part of the browser.
-// Allows us to use our dev tools to debug browser windows itself.
-const char kAlwaysEnableDevTools[]          = "always-enable-dev-tools";
-
 // Specifies that the associated value should be launched in "application" mode.
 const char kApp[]                           = "app";
 
@@ -71,6 +70,9 @@ const char kAppsGalleryURL[]                = "apps-gallery-url";
 // The update url used by gallery/webstore extensions.
 const char kAppsGalleryUpdateURL[]          = "apps-gallery-update-url";
 
+// Whether to always use the new app install bubble when installing an app.
+const char kAppsNewInstallBubble[]          = "apps-new-install-bubble";
+
 // Disable throbber for extension apps.
 const char kAppsNoThrob[]                   = "apps-no-throb";
 
@@ -93,17 +95,16 @@ const char kAuthServerWhitelist[]           = "auth-server-whitelist";
 // automation-related messages on IPC channel with the given ID.
 const char kAutomationClientChannelID[]     = "automation-channel";
 
+// Causes the automation provider to reinitialize its IPC channel instead of
+// shutting down when a client disconnects.
+const char kAutomationReinitializeOnChannelError[] =
+    "automation-reinitialize-on-channel-error";
+
 // When the option to block third-party cookies from being set is enabled,
 // also block third-party cookies from being read.
 const char kBlockReadingThirdPartyCookies[] =
     "block-reading-third-party-cookies";
 
-// Causes the browser process to throw an assertion on startup.
-const char kBrowserAssertTest[]             = "assert-test";
-
-// Causes the browser process to crash on startup.
-const char kBrowserCrashTest[]              = "crash-test";
-
 // How often (in seconds) to check for updates. Should only be used for
 // testing purposes.
 const char kCheckForUpdateIntervalSec[]     = "check-for-update-interval";
@@ -112,6 +113,9 @@ const char kCheckForUpdateIntervalSec[]     = "check-for-update-interval";
 // this version cannot be loaded, Chrome will exit.
 const char kChromeVersion[]                 = "chrome-version";
 
+// Comma-separated list of SSL cipher suites to disable.
+const char kCipherSuiteBlacklist[]          = "cipher-suite-blacklist";
+
 // Used with kCloudPrintFile.  Tells Chrome to delete the file when
 // finished displaying the print dialog.
 const char kCloudPrintDeleteFile[]          = "cloud-print-delete-file";
@@ -125,6 +129,11 @@ const char kCloudPrintFile[]                = "cloud-print-file";
 // Defaults to "application/pdf" if unspecified.
 const char kCloudPrintFileType[]            = "cloud-print-file-type";
 
+// Used with kCloudPrintFile to specify a JSON print ticket for the resulting
+// print job.
+// Defaults to null if unspecified.
+const char kCloudPrintPrintTicket[]            = "cloud-print-print-ticket";
+
 // Used with kCloudPrintFile to specify a title for the resulting print
 // job.
 const char kCloudPrintJobTitle[]            = "cloud-print-job-title";
@@ -137,6 +146,10 @@ const char kCloudPrintProxyId[]             = "cloud-print-proxy-id";
 // print service has been enabled (see enable-cloud-print).
 const char kCloudPrintServiceURL[]          = "cloud-print-service";
 
+// Comma-separated options to troubleshoot the component updater.
+// Only valid for the browser process.
+const char kComponentUpdaterDebug[]         = "component-updater-debug";
+
 // Causes the browser process to inspect loaded and registered DLLs for
 // known conflicts and warn the user.
 const char kConflictingModulesCheck[]       = "conflicting-modules-check";
@@ -146,6 +159,27 @@ const char kConflictingModulesCheck[]       = "conflicting-modules-check";
 // string value, the 2 letter code from ISO 3166-1.
 const char kCountry[]                       = "country";
 
+// Causes the browser process to crash if browser threads are not responding for
+// the given number of seconds.
+const char kCrashOnHangSeconds[]            = "crash-on-hang-seconds";
+
+// Comma separated list of BrowserThreads that cause browser process to crash if
+// the given browser thread is not responsive. UI,IO,DB,FILE,CACHE are the list
+// of BrowserThreads that are supported.
+//
+// For example:
+//    --crash-on-hang-threads=UI,IO --> Crash the browser if UI or IO thread is
+//                                      not responsive.
+const char kCrashOnHangThreads[]            = "crash-on-hang-threads";
+
+// Causes the browser process to crash if the number of browser threads that are
+// responding is equal to the given number.
+//
+// For example:
+//    --crash-on-live=1 --> Crash if only one thread is responsive and all other
+//                          threads are not responsive.
+const char kCrashOnLive[]                   = "crash-on-live";
+
 // Path to the inspector files on disk (allows reloading of devtool files
 // without having to restart the browser).
 const char kDebugDevToolsFrontend[]         = "debug-devtools-frontend";
@@ -165,12 +199,6 @@ const char kDeviceManagementUrl[]           = "device-management-url";
 // Triggers a pletora of diagnostic modes.
 const char kDiagnostics[]                   = "diagnostics";
 
-// Disables the hardware acceleration of 3D CSS and animation.
-const char kDisableAcceleratedLayers[]      = "disable-accelerated-layers";
-
-// Disables GPU accelerated video display.
-const char kDisableAcceleratedVideo[]       = "disable-accelerated-video";
-
 // Replaces the audio IPC layer for <audio> and <video> with a mock audio
 // device, useful when using remote desktop or machines without sound cards.
 // This is temporary until we fix the underlying problem.
@@ -204,8 +232,13 @@ const char kDisableConnectBackupJobs[]      = "disable-connect-backup-jobs";
 // Disables the custom JumpList on Windows 7.
 const char kDisableCustomJumpList[]         = "disable-custom-jumplist";
 
-// Browser flag to disable the web inspector for all renderers.
-const char kDisableDevTools[]               = "disable-dev-tools";
+// Disables checking whether custom protocol handlers are registered with the
+// OS and removing those that are not. This is used during automated testing.
+const char kDisableCustomProtocolOSCheck[]  =
+    "disable-custom-protocol-os-check";
+
+// Disables retrieval of PAC URLs from DHCP as per the WPAD standard.
+const char kDisableDhcpWpad[]               = "disable-dhcp-wpad";
 
 // Disable extensions.
 const char kDisableExtensions[]             = "disable-extensions";
@@ -218,11 +251,6 @@ const char kDisableExtensionsFileAccessCheck[] =
 // Disables the sandbox for the built-in flash player.
 const char kDisableFlashSandbox[]           = "disable-flash-sandbox";
 
-// Suppresses hang monitor dialogs in renderer processes.  This may allow slow
-// unload handlers on a page to prevent the tab from closing, but the Task
-// Manager can be used to terminate the offending process in this case.
-const char kDisableHangMonitor[]            = "disable-hang-monitor";
-
 // Disable the use of the HistoryQuickProvider for autocomplete results.
 const char kDisableHistoryQuickProvider[]   = "disable-history-quick-provider";
 
@@ -250,10 +278,6 @@ const char kDisableIPPooling[]              = "disable-ip-pooling";
 // Disable speculative TCP/IP preconnection.
 const char kDisablePreconnect[]             = "disable-preconnect";
 
-// Whether we should prevent the new tab page from showing the first run
-// notification.
-const char kDisableNewTabFirstRun[]         = "disable-new-tab-first-run";
-
 // Normally when the user attempts to navigate to a page that was the result of
 // a post we prompt to make sure they want to. This switch may be used to
 // disable that check. This switch is used during automated testing.
@@ -263,19 +287,22 @@ const char kDisablePromptOnRepost[]         = "disable-prompt-on-repost";
 // this option is specified or not.
 const char kDisableRemoteFonts[]            = "disable-remote-fonts";
 
-// Turns off the accessibility in the renderer.
-const char kDisableRendererAccessibility[]  = "disable-renderer-accessibility";
-
 // Prevents the URLs of BackgroundContents from being remembered and re-launched
 // when the browser restarts.
 const char kDisableRestoreBackgroundContents[] =
     "disable-restore-background-contents";
 
+// Disable the sanitized version of client-side phishing detection, for use by
+// non-UMA users.  Any features containing portions of the URL or page content
+// are not sent as part of the pingback in this mode.
+const char kDisableSanitizedClientSidePhishingDetection[] =
+    "disable-sanitized-client-side-phishing-detection";
+
 // Disable site-specific tailoring to compatibility issues in WebKit.
 const char kDisableSiteSpecificQuirks[]     = "disable-site-specific-quirks";
 
-// Disable False Start in SSL and TLS connections.
-const char kDisableSSLFalseStart[]          = "disable-ssl-false-start";
+// Disable SSL v3 (usually for testing purposes).
+const char kDisableSSL3[]                   = "disable-ssl3";
 
 // Disable syncing browser data to a Google Account.
 const char kDisableSync[]                   = "disable-sync";
@@ -292,6 +319,9 @@ const char kDisableSyncAutofillProfile[]     = "disable-sync-autofill-profile";
 // Disable syncing of bookmarks.
 const char kDisableSyncBookmarks[]          = "disable-sync-bookmarks";
 
+// Disable sync encryption options.
+const char kDisableSyncEncryption[]          = "disable-sync-encryption";
+
 // Disable syncing of extensions.
 const char kDisableSyncExtensions[]         = "disable-sync-extensions";
 
@@ -310,6 +340,9 @@ const char kDisableSyncThemes[]             = "disable-sync-themes";
 const char kDisableTabCloseableStateWatcher[] =
     "disable-tab-closeable-state-watcher";
 
+// Disable TLS v1.0 (usually for testing purposes).
+const char kDisableTLS1[]                   = "disable-tls1";
+
 // Allow disabling of translate from the command line to assist with
 // automated browser testing (e.g. Selenium/WebDriver).  Normal
 // browser users should disable translate with the preference.
@@ -349,17 +382,14 @@ const char kDnsServer[]                     = "dns-server";
 // |DOMAutomationController| to the |AutomationRenderViewHelper|.
 const char kDomAutomationController[]       = "dom-automation";
 
+// Replaces the download shelf with a new experimental UI.
+const char kDownloadsNewUI[]                = "downloads-new-ui";
+
 // Dump any accumualted histograms to the log when browser terminates (requires
 // logging to be enabled to really do anything).  Used by developers and test
 // scripts.
 const char kDumpHistogramsOnExit[]          = "dump-histograms-on-exit";
 
-// Enable gpu-accelerated 2d canvas.
-const char kEnableAccelerated2dCanvas[]     = "enable-accelerated-2d-canvas";
-
-// Enables the hardware acceleration of plugins.
-const char kEnableAcceleratedPlugins[]      = "enable-accelerated-plugins";
-
 // Enables AeroPeek for each tab. (This switch only works on Windows 7).
 const char kEnableAeroPeekTabs[]            = "enable-aero-peek-tabs";
 
@@ -368,12 +398,16 @@ const char kEnableAeroPeekTabs[]            = "enable-aero-peek-tabs";
 // for more background.
 const char kEnableAuthNegotiatePort[]       = "enable-auth-negotiate-port";
 
-// At this point, even if client-side phishing detection is enabled we will not,
-// by default, display an interstitial if we detected a phishing site.  Once
-// we are confident that the false-positive rate is as low as expected we can
-// remove this flag.
-const char kEnableClientSidePhishingInterstitial[] =
-    "enable-client-side-phishing-interstitial";
+// With this flag set, Chrome will occasionally prompt the user to volunteer
+// Autofill usage data beyond what is collected by default.  This is data that
+// we expect to be helpful for debugging, but that we do not want to send up
+// automatically due to privacy concerns.
+const char kEnableAutofillFeedback[]        = "enable-autofill-feedback";
+
+// Enables the pre- and auto-login features.  When a user signs in to sync,
+// the browser's cookie jar is pre-filled with GAIA cookies.  When the user
+// visits a GAIA login page, an info bar can help the user login.
+const char kEnableAutologin[]               = "enable-autologin";
 
 // This flag enables UI for clearing server data.  Temporarily in place
 // until there's a server endpoint deployed.
@@ -389,10 +423,6 @@ const char kEnableCloudPrintProxy[]         = "enable-cloud-print-proxy";
 // Enables the Cloud Print dialog hosting code.
 const char kEnableCloudPrint[]              = "enable-cloud-print";
 
-// Enables compact navigation mode, which removes the toolbar and moves most of
-// UI to the tab strip.
-const char kEnableCompactNavigation[]       = "enable-compact-navigation";
-
 // Enables compositing to texture instead of display.
 const char kEnableCompositeToTexture[]      = "enable-composite-to-texture";
 
@@ -403,22 +433,6 @@ const char kEnableConnectBackupJobs[]       = "enable-connect-backup-jobs";
 // Enables web developers to create apps for Chrome without using crx packages.
 const char kEnableCrxlessWebApps[]          = "enable-crxless-web-apps";
 
-// Enables retrieval of PAC URLs from DHCP as per the WPAD standard. Note
-// that this feature is not supported on all platforms, and using the flag
-// is a no-op on such platforms.
-const char kEnableDhcpWpad[]                = "enable-dhcp-wpad";
-
-// Enable DNS side checking of certificates. Still experimental, should only
-// be used by developers at the current time.
-const char kEnableDNSCertProvenanceChecking[] =
-     "enable-dns-cert-provenance-checking";
-
-const char kEnableDNSSECCerts[]             = "enable-dnssec-certs";
-
-// Enables app manifest features that are in development.
-const char kEnableExperimentalAppManifests[] =
-    "enable-experimental-app-manifests";
-
 // Enables extension APIs that are in development.
 const char kEnableExperimentalExtensionApis[] =
     "enable-experimental-extension-apis";
@@ -433,9 +447,6 @@ const char kEnableFastback[]                = "enable-fastback";
 // testing, for example page cycler and layout tests.  See bug 1157243.
 const char kEnableFileCookies[]             = "enable-file-cookies";
 
-// Enable the JavaScript Full Screen API.
-const char kEnableFullScreen[]              = "enable-fullscreen";
-
 // Enable the in-browser thumbnailing, which is more efficient than the
 // in-renderer thumbnailing, as we can use more information to determine
 // if we need to update thumbnails.
@@ -455,6 +466,10 @@ const char kEnableIPCFuzzing[]               = "enable-ipc-fuzzing";
 // attempt to use the existing connection.
 const char kEnableIPPooling[]               = "enable-ip-pooling";
 
+// Enables some extension background pages to be loaded when they are
+// needed rather than when the extensions are first loaded.
+const char kEnableLazyBackgroundPages[]     = "enable-lazy-background-pages";
+
 // Enables MAC cookies in the network stack.  These cookies use HMAC to
 // protect session state from passive network attackers.
 // http://tools.ietf.org/html/draft-hammer-oauth-v2-mac-token
@@ -470,10 +485,6 @@ const char kEnableNaCl[]                    = "enable-nacl";
 // Enables debugging via RSP over a socket.
 const char kEnableNaClDebug[]               = "enable-nacl-debug";
 
-// This applies only when the process type is "service". Enables the
-// Chromoting Host Process within the service process.
-const char kEnableRemoting[]                = "enable-remoting";
-
 // Enable content settings based on host *and* plug-in.
 const char kEnableResourceContentSettings[] =
     "enable-resource-content-settings";
@@ -488,33 +499,32 @@ const char kEnablePreconnect[]              = "enable-preconnect";
 // parameter to indicate if the provider should be the default.
 const char kEnableSearchProviderApiV2[]     = "enable-search-provider-api-v2";
 
-// Enables 0-RTT HTTPS handshakes.
-const char kEnableSnapStart[]               = "enable-snap-start";
+// Enable the use of the ShortcutsProvider for autocomplete results.
+const char kEnableShortcutsProvider[]       = "enable-shortcuts-provider";
 
-// Enable syncing browser data to a Google Account.
-const char kEnableSync[]                    = "enable-sync";
+// On platforms that support it, enable smooth scroll animation.
+const char kEnableSmoothScrolling[]         = "enable-smooth-scrolling";
+// Enable OAuth sign-in for sync.
+const char kEnableSyncOAuth[]               = "enable-sync-oauth";
 
-// Enable syncing browser autofill.
-const char kEnableSyncAutofill[]            = "enable-sync-autofill";
-
-// Enable sync encryption options.
-const char kEnableSyncEncryption[]          = "enable-sync-encryption";
+// Enable syncing custom search engines.
+const char kEnableSyncSearchEngines[]       = "enable-sync-search-engines";
 
 // Enable syncing browser sessions.
 const char kEnableSyncSessions[]            = "enable-sync-sessions";
 
-// Enables context menu for selecting groups of tabs.
-const char kEnableTabGroupsContextMenu[]     = "enable-tab-groups-context-menu";
+// Enable syncing browser sessions for other synced clients.
+const char kEnableSyncSessionsForOtherClients[] =
+    "enable-sync-sessions-for-other-clients";
 
 // Enable syncing browser typed urls.
 const char kEnableSyncTypedUrls[]           = "enable-sync-typed-urls";
 
-// Enable syncing browser typed urls.
-const char kEnableSyncedBookmarksFolder[]   = "enable-synced-bookmarks-folder";
+// Enables context menu for selecting groups of tabs.
+const char kEnableTabGroupsContextMenu[]     = "enable-tab-groups-context-menu";
 
-// Enable use of experimental TCP sockets API for sending data in the
-// SYN packet.
-const char kEnableTcpFastOpen[]             = "enable-tcp-fastopen";
+// Enable the "synced bookmarks" folder.
+const char kEnableSyncedBookmarksFolder[]   = "enable-synced-bookmarks-folder";
 
 // Enables the option to show tabs as a vertical stack down the side of the
 // browser window.
@@ -529,6 +539,9 @@ const char kEnableWatchdog[]                = "enable-watchdog";
 // Use WebSocket over SPDY.
 const char kEnableWebSocketOverSpdy[]       = "enable-websocket-over-spdy";
 
+// Enable WebUI TaskManager.
+const char kEnableWebUITaskManager[]        = "enable-webui-taskmanager";
+
 // Enables experimental features for Spellchecker. Right now, the first
 // experimental feature is auto spell correct, which corrects words which are
 // misppelled by typing the word with two consecutive letters swapped. The
@@ -575,10 +588,10 @@ const char kForceAppsPromoVisible[]         = "force-apps-promo-visible";
 // the base layer even when compositing is not strictly required.
 const char kForceCompositingMode[]          = "force-compositing-mode";
 
-// Force renderer accessibility to be on instead of enabling it on demand when
-// a screen reader is detected. The disable-renderer-accessibility switch
-// overrides this if present.
-const char kForceRendererAccessibility[]    = "force-renderer-accessibility";
+// Specify the backend server used for gaia authentications, like sync or
+// policies for example. The https:// prefix and the trailing slash should be
+// omitted. The default value is "www.google.com".
+const char kGaiaHost[]                       = "gaia-host";
 
 // Specifies a custom name for the GSSAPI library to load.
 const char kGSSAPILibraryName[]             = "gssapi-library-name";
@@ -635,9 +648,6 @@ const char kHostResolverRules[]             = "host-resolver-rules";
 // <profile_dir>/Default/TransportSecurity
 const char kHstsHosts[]                     = "hsts-hosts";
 
-// Ignores GPU blacklist.
-const char kIgnoreGpuBlacklist[]            = "ignore-gpu-blacklist";
-
 // Perform importing from another browser. The value associated with this
 // setting encodes the target browser and what items to import.
 const char kImport[]                        = "import";
@@ -718,13 +728,19 @@ const char kNaClStartupDialog[]             = "nacl-startup-dialog";
 // Intended primarily for use with --log-net-log.
 const char kNetLogLevel[]                   = "net-log-level";
 
-// Use the latest incarnation of the new tab page.
-const char kNewTabPage4[]                   = "new-tab-page-4";
+// Disable the latest incarnation of the new tab page. Only intended for testing
+// use. For TOUCH_UI, the meaning is inverted (i.e. default is to disable NTP4,
+// and passing the flag will enable it).
+const char kNewTabPage[]                    = "new-tab-page";
 
 // Disables the default browser check. Useful for UI/browser tests where we
 // want to avoid having the default browser info-bar displayed.
 const char kNoDefaultBrowserCheck[]         = "no-default-browser-check";
 
+// By default, an https page can load images, fonts or frames from an http page.
+// This switch overrides this to block this lesser mixed-content problem.
+const char kNoDisplayingInsecureContent[]   = "no-displaying-insecure-content";
+
 // Don't record/playback events when using record & playback.
 const char kNoEvents[]                      = "no-events";
 
@@ -745,6 +761,11 @@ const char kNoPings[]                       = "no-pings";
 // other proxy server flags that are passed.
 const char kNoProxyServer[]                 = "no-proxy-server";
 
+// Stronger version of insecure content blocking, for the case where the
+// blocking would only be applied to a whitelist of domains.  Switch is
+// expected to become obsolete once the whitelist goes away.
+const char kNoRunningInsecureContent[]      = "no-running-insecure-content";
+
 // Disables the service process from adding itself as an autorun process. This
 // does not delete existing autorun registrations, it just prevents the service
 // from registering a new one.
@@ -789,6 +810,10 @@ const char kPreloadInstantSearch[]          = "preload-instant-search";
 // Controls speculative prerendering of pages, and content prefetching.  Both
 // are dispatched from <link rel=prefetch href=...> elements.
 const char kPrerender[]                     = "prerender";
+// Trigger prerendering of pages from suggestions in the omnibox. Only has an
+// effect when Instant is either disabled or restricted to search, and when
+// prerender is enabled.
+const char kPrerenderFromOmnibox[]          = "prerender-from-omnibox";
 // These are the values the switch may have, as in "--prerender=auto".
 //   auto: Allow field trial selection in both prerender and prefetch.
 const char kPrerenderSwitchValueAuto[]      = "auto";
@@ -806,6 +831,9 @@ const char kPrint[]                         = "print";
 // detect the installed version of Chrome on Linux.
 const char kProductVersion[]                = "product-version";
 
+// Selects directory of profile to associate with the first browser launched.
+const char kProfileDirectory[]              = "profile-directory";
+
 // Starts the sampling based profiler for the browser process at
 // startup. This will only work if chrome has been built with
 // the gyp variable profiling=1. The output will go to the value
@@ -826,6 +854,9 @@ const char kProfilingFile[]                 = "profiling-file";
 // A time in seconds can be specified.
 const char kProfilingFlush[]                = "profiling-flush";
 
+// Specifies a custom URL for fetching NTP promo data.
+const char kPromoServerURL[]                = "promo-server-url";
+
 // Force proxy auto-detection.
 const char kProxyAutoDetect[]               = "proxy-auto-detect";
 
@@ -856,9 +887,6 @@ const char kReloadKilledTabs[]              = "reload-killed-tabs";
 // Enable remote debug over HTTP on the specified port.
 const char kRemoteDebuggingPort[]           = "remote-debugging-port";
 
-// Enable remote debug / automation shell on the specified port.
-const char kRemoteShellPort[]               = "remote-shell-port";
-
 // Indicates the last session should be restored on startup. This overrides
 // the preferences value and is primarily intended for testing. The value of
 // this switch is the number of tabs to wait until loaded before
@@ -904,6 +932,9 @@ const char kSearchInOmniboxHint[]           = "search-in-omnibox-hint";
 // The LSID of the account to use for the service process.
 const char kServiceAccountLsid[]            = "service-account-lsid";
 
+// Annotate forms with Autofill field type predictions.
+const char kShowAutofillTypePredictions[]   = "show-autofill-type-predictions";
+
 // See kHideIcons.
 const char kShowIcons[]                     = "show-icons";
 
@@ -926,6 +957,10 @@ const char kSilentDumpOnDCHECK[]            = "silent-dump-on-dcheck";
 // Replaces the buffered data source for <audio> and <video> with a simplified
 // resource loader that downloads the entire resource into memory.
 
+// Choose the socket reuse policy specified. The value should be of type
+// enum ClientSocketReusePolicy.
+const char kSocketReusePolicy[]             = "socket-reuse-policy";
+
 // Start the browser maximized, regardless of any previous settings.
 const char kStartMaximized[]                = "start-maximized";
 
@@ -952,6 +987,9 @@ const char kSyncNotificationHost[]          = "sync-notification-host";
 // Override the default server used for profile sync.
 const char kSyncServiceURL[]                = "sync-url";
 
+// Show the sync promo on the new tab page.
+const char kSyncShowPromo[]                 = "sync-show-promo";
+
 // Try to connect to XMPP using SSLTCP first (for testing).
 const char kSyncTrySsltcpFirstForXmpp[]     = "sync-try-ssltcp-first-for-xmpp";
 
@@ -996,14 +1034,6 @@ const char kMaxSpdyConcurrentStreams[]      = "max-spdy-concurrent-streams";
 // for all of its state.
 const char kUserDataDir[]                   = "user-data-dir";
 
-// directory to locate user scripts in as an over-ride of the default
-const char kUserScriptsDir[]                = "user-scripts-dir";
-
-// On POSIX only: the contents of this flag are prepended to the utility
-// process command line. Useful values might be "valgrind" or "xterm -e gdb
-// --args".
-const char kUtilityCmdPrefix[]              = "utility-cmd-prefix";
-
 // Print version information and quit.
 const char kVersion[]                       = "version";
 
@@ -1012,8 +1042,11 @@ const char kVersion[]                       = "version";
 const char kWinHttpProxyResolver[]          = "winhttp-proxy-resolver";
 
 #if defined(OS_CHROMEOS)
-// Enable DOM based login screens.
-const char kDOMLogin[]                      = "dom-login";
+// Enable WebUI based OOBE and login.
+const char kWebUILogin[]                    = "webui-login";
+
+// Skips OAuth part of ChromeOS login process.
+const char kSkipOAuthLogin[]                = "skip-oauth-login";
 
 // Enables device policy support on ChromeOS.
 const char kEnableDevicePolicy[]            = "enable-device-policy";
@@ -1042,6 +1075,9 @@ const char kLoginScreen[]                   = "login-screen";
 // Allows control over the initial login screen size. Pass width,height.
 const char kLoginScreenSize[]               = "login-screen-size";
 
+// Shows a memory consumption status area widget for OOM debugging.
+const char kMemoryWidget[]                  = "memory-widget";
+
 // Attempts to load libcros and validate it, then exits. A nonzero return code
 // means the library could not be loaded correctly.
 const char kTestLoadLibcros[]               = "test-load-libcros";
@@ -1080,23 +1116,23 @@ const char kScreenSaverUrl[]                = "screen-saver-url";
 // Flag to trigger ChromeOS system log compression during feedback submit.
 const char kCompressSystemFeedback[]        = "compress-sys-feedback";
 
-// Flag to skip loading ChromeOS specific component extensions. This one is
-// needed to prevent these component interfering with the some of the tests.
-// TODO(zelidrag): http://crosbug.com/14463 - we should remove this switch once
-// get rid of ChromeOS component extensions with background pages.
-const char kSkipChromeOSComponents[]        = "skip-chromeos-components";
+// Enables overriding the path for the default authentication extension.
+const char kAuthExtensionPath[]             = "auth-ext-path";
 
-// Forces usage of libcros stub implementation. For testing purposes, this
-// switch separates chrome code from the rest of ChromeOS.
-const char kForceStubLibcros[]              = "force-stub-libcros";
+// Enables overriding user credential for GAIA login.
+const char kTestAuthEmail[]                 = "test-auth-email";
+const char kTestAuthPassword[]              = "test-auth-password";
 
-// Enables Advanced File System.
-const char kEnableAdvancedFileSystem[]      = "enable-advanced-fs";
+// Enables archive file (zip) handling in file manager.
+const char kEnableArchives[]                = "enable-archives";
+
+#ifndef NDEBUG
+// Skips all other OOBE pages after user login.
+const char kOobeSkipPostLogin[]             = "oobe-skip-postlogin";
+
+// Enables overriding the path off file manager extension.
+const char kFileManagerExtensionPath[]      = "filemgr-ext-path";
 #endif
-
-#if defined(OS_POSIX) && !defined(OS_MACOSX)
-// Specify the amount the trackpad should scroll by.
-const char kScrollPixels[]                  = "scroll-pixels";
 #endif
 
 #if defined(OS_MACOSX) || defined(OS_WIN)
@@ -1125,6 +1161,14 @@ const char kPasswordStore[]                 = "password-store";
 #if defined(OS_MACOSX)
 // Enables the tabs expose feature ( http://crbug.com/50307 ).
 const char kEnableExposeForTabs[]           = "enable-expose-for-tabs";
+
+// A process type (switches::kProcessType) that relaunches the browser. See
+// chrome/browser/mac/relauncher.h.
+const char kRelauncherProcess[]             = "relauncher";
+
+// Use mock keychain for testing purposes, which prevents blocking dialogs from
+// causing timeouts.
+const char kUseMockKeychain[]               = "use-mock-keychain";
 #endif
 
 #if !defined(OS_MACOSX)
@@ -1132,19 +1176,38 @@ const char kEnableExposeForTabs[]           = "enable-expose-for-tabs";
 const char kKioskMode[]                     = "kiosk";
 #endif
 
-// Enables debug paint in views framework. Enabling this causes the damaged
-// region being painted to flash in red.
-#if defined(TOOLKIT_VIEWS)
-const char kDebugViewsPaint[]               = "debug-views-paint";
+#if defined(TOOLKIT_GTK)
+// Enables the global bookmark menu in our global menu bar on Ubuntu
+// Natty. This is disabled because of issues where dbus will hang for over a
+// minute because it doesn't like it when we thrown hundreds of kilobytes (or
+// even megabytes) of favicon data at it.
+const char kEnableGlobalBookmarkMenu[]     = "enable-global-bookmark-menu";
 #endif
 
+#if defined(TOOLKIT_VIEWS)
+// Enables debug paint in views framework. Enabling this causes the damaged
+// region being painted to flash in red.
+const char kDebugViewsPaint[]               = "debug-views-paint";
+
+// Runs Chrome inside the views desktop shell.
+const char kViewsDesktop[]                  = "views-desktop";
+#endif
+
+#if defined(TOUCH_UI)
 // Debug only switch to prevent the mouse cursor from disappearing when
 // touch is enabled
-#if defined(TOUCH_UI)
 const char kKeepMouseCursor[]               = "keep-mouse-cursor";
+
+// Tells chrome to interpret events from these devices as touch events. Only
+// available with XInput 2 (i.e. X server 1.8 or above). The id's of the devices
+// can be retrieved from 'xinput list'.
+const char kTouchDevices[]                  = "touch-devices";
 #endif
 
 #ifndef NDEBUG
+// URL of the OAuth server host
+const char kOAuthHostUrl[]                  = "oauth-host-url";
+
 // Clear the token service before using it.  This allows simulating
 // the expiration of credentials during testing.
 const char kClearTokenService[]             = "clear-token-service";
@@ -1162,28 +1225,18 @@ const char kWebSocketLiveExperimentHost[]   = "websocket-live-experiment-host";
 const char kExposePrivateExtensionApi[]   = "expose-private-extension-api";
 #endif
 
-#if defined(HAVE_XINPUT2)
-// Tells chrome to interpret events from these devices as touch events. Only
-// available with XInput 2 (i.e. X server 1.8 or above). The id's of the devices
-// can be retrieved from 'xinput list'.
-const char kTouchDevices[]                  = "touch-devices";
-#endif
-
-#if defined(GOOGLE_CHROME_BUILD) && !defined(OS_CHROMEOS) && !defined(OS_MACOSX)
 // Disable print preview (Not exposed via about:flags. Only used for testing.)
 const char kDisablePrintPreview[]           = "disable-print-preview";
 
-bool IsPrintPreviewEnabled() {
-  return !CommandLine::ForCurrentProcess()->HasSwitch(kDisablePrintPreview);
-}
-#else
 // Enable print preview (no PDF viewer, thus not supported with Chromium).
+// kDisablePrintPreview overrides this.
 const char kEnablePrintPreview[]            = "enable-print-preview";
 
 bool IsPrintPreviewEnabled() {
+  if (CommandLine::ForCurrentProcess()->HasSwitch(kDisablePrintPreview))
+    return false;
   return CommandLine::ForCurrentProcess()->HasSwitch(kEnablePrintPreview);
 }
-#endif
 
 bool IsInBrowserThumbnailingEnabled() {
   return CommandLine::ForCurrentProcess()->HasSwitch(
