@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 //
@@ -18,9 +18,18 @@ extern const char kGaiaService[];  // uber token
 extern const char kContactsService[];
 extern const char kTalkService[];
 extern const char kSyncService[];
+extern const char kSyncServiceOAuth[];
 extern const char kRemotingService[];
 extern const char kCloudPrintService[];
 extern const char kDeviceManagementService[];
+extern const char kDeviceManagementServiceOAuth[];
+
+// Used with uber auth tokens when needed.
+extern const char kGaiaSid[];
+extern const char kGaiaLsid[];
+extern const char kGaiaOAuthToken[];
+extern const char kGaiaOAuthSecret[];
+extern const char kGaiaOAuthDuration[];
 
 }  // namespace GaiaConstants
 
