@@ -8,6 +8,8 @@
 #define CHROME_COMMON_PREF_NAMES_H_
 #pragma once
 
+#include <stddef.h>
+
 #include "build/build_config.h"
 
 namespace prefs {
@@ -17,9 +19,11 @@ extern const char kAppsPromoCounter[];
 extern const char kDefaultAppsInstalled[];
 extern const char kHomePageIsNewTabPage[];
 extern const char kHomePage[];
+extern const char kHomePageChanged[];
 extern const char kSessionExitedCleanly[];
 extern const char kRestoreOnStartup[];
 extern const char kURLsToRestoreOnStartup[];
+extern const char kSyncPromoExpanded[];
 
 // For OS_CHROMEOS we maintain kApplicationLocale property in both local state
 // and user's profile.  Global property determines locale of login screen,
@@ -36,12 +40,49 @@ extern const char kAcceptLanguages[];
 extern const char kStaticEncodings[];
 extern const char kPopupWhitelistedHosts[];
 extern const char kShowBookmarkBar[];
+extern const char kBookmarkEditorExpandedNodes[];
 extern const char kWebKitStandardFontFamily[];
 extern const char kWebKitFixedFontFamily[];
 extern const char kWebKitSerifFontFamily[];
 extern const char kWebKitSansSerifFontFamily[];
 extern const char kWebKitCursiveFontFamily[];
 extern const char kWebKitFantasyFontFamily[];
+extern const char kWebKitStandardFontFamilyMap[];
+extern const char kWebKitFixedFontFamilyMap[];
+extern const char kWebKitSerifFontFamilyMap[];
+extern const char kWebKitSansSerifFontFamilyMap[];
+extern const char kWebKitCursiveFontFamilyMap[];
+extern const char kWebKitFantasyFontFamilyMap[];
+
+// ISO 15924 four-letter script codes that per-script font prefs are supported
+// for.
+extern const char* kWebKitScriptsForFontFamilyMaps[];
+extern const size_t kWebKitScriptsForFontFamilyMapsLength;
+
+// Per-script font prefs that have defaults, for easy reference when registering
+// the defaults.
+extern const char kWebKitStandardFontFamilyArabic[];
+extern const char kWebKitFixedFontFamilyArabic[];
+extern const char kWebKitSerifFontFamilyArabic[];
+extern const char kWebKitSansSerifFontFamilyArabic[];
+extern const char kWebKitStandardFontFamilyJapanese[];
+extern const char kWebKitFixedFontFamilyJapanese[];
+extern const char kWebKitSerifFontFamilyJapanese[];
+extern const char kWebKitSansSerifFontFamilyJapanese[];
+extern const char kWebKitStandardFontFamilyKorean[];
+extern const char kWebKitFixedFontFamilyKorean[];
+extern const char kWebKitSerifFontFamilyKorean[];
+extern const char kWebKitSansSerifFontFamilyKorean[];
+extern const char kWebKitCursiveFontFamilyKorean[];
+extern const char kWebKitStandardFontFamilySimplifiedHan[];
+extern const char kWebKitFixedFontFamilySimplifiedHan[];
+extern const char kWebKitSerifFontFamilySimplifiedHan[];
+extern const char kWebKitSansSerifFontFamilySimplifiedHan[];
+extern const char kWebKitStandardFontFamilyTraditionalHan[];
+extern const char kWebKitFixedFontFamilyTraditionalHan[];
+extern const char kWebKitSerifFontFamilyTraditionalHan[];
+extern const char kWebKitSansSerifFontFamilyTraditionalHan[];
+
 extern const char kWebKitDefaultFontSize[];
 extern const char kWebKitDefaultFixedFontSize[];
 extern const char kWebKitMinimumFontSize[];
@@ -62,9 +103,10 @@ extern const char kWebKitAllowDisplayingInsecureContent[];
 extern const char kWebKitAllowRunningInsecureContent[];
 extern const char kPasswordManagerEnabled[];
 extern const char kPasswordManagerAllowShowPasswords[];
+extern const char kAutologinEnabled[];
 extern const char kSafeBrowsingEnabled[];
 extern const char kSafeBrowsingReportingEnabled[];
-extern const char kIncognitoEnabled[];
+extern const char kIncognitoModeAvailability[];
 extern const char kSearchSuggestEnabled[];
 extern const char kConfirmToQuitEnabled[];
 extern const char kCookieBehavior[];  // OBSOLETE
@@ -88,6 +130,8 @@ extern const char kDnsHostReferralList[];  // OBSOLETE
 extern const char kDnsPrefetchingHostReferralList[];
 extern const char kDisableSpdy[];
 extern const char kDisabledSchemes[];
+extern const char kUrlBlacklist[];
+extern const char kUrlWhitelist[];
 extern const char kInstantConfirmDialogShown[];
 extern const char kInstantEnabled[];
 extern const char kInstantEnabledOnce[];
@@ -121,7 +165,7 @@ extern const char kLanguageChewingKeyboardType[];
 extern const char kLanguageChewingSelKeys[];
 extern const char kLanguageChewingHsuSelKeyType[];
 extern const char kLanguageHangulKeyboard[];
-extern const char kLanguageHangulHanjaKeys[];
+extern const char kLanguageHangulHanjaBindingKeys[];
 extern const char kLanguagePinyinCorrectPinyin[];
 extern const char kLanguagePinyinFuzzyPinyin[];
 extern const char kLanguagePinyinLookupTablePageSize[];
@@ -147,10 +191,6 @@ extern const char kLanguageMozcShiftKeyModeSwitch[];
 extern const char kLanguageMozcNumpadCharacterForm[];
 extern const char kLanguageMozcIncognitoMode[];
 extern const char kLanguageMozcUseAutoImeTurnOff[];
-extern const char kLanguageMozcUseDateConversion[];
-extern const char kLanguageMozcUseSingleKanjiConversion[];
-extern const char kLanguageMozcUseSymbolConversion[];
-extern const char kLanguageMozcUseNumberConversion[];
 extern const char kLanguageMozcUseHistorySuggest[];
 extern const char kLanguageMozcUseDictionarySuggest[];
 extern const char kLanguageMozcSuggestionsSize[];
@@ -160,6 +200,7 @@ extern const char kLanguageXkbRemapAltKeyTo[];
 extern const char kLanguageXkbAutoRepeatEnabled[];
 extern const char kLanguageXkbAutoRepeatDelay[];
 extern const char kLanguageXkbAutoRepeatInterval[];
+extern const char kLanguagePreferredVirtualKeyboard[];
 extern const char kAccessibilityEnabled[];
 extern const char kLabsAdvancedFilesystemEnabled[];
 extern const char kLabsMediaplayerEnabled[];
@@ -167,6 +208,9 @@ extern const char kEnableScreenLock[];
 extern const char kShowPlanNotifications[];
 extern const char kShow3gPromoNotification[];
 extern const char kLastUsedFileBrowserHandlers[];
+extern const char kUseSharedProxies[];
+extern const char kOAuth1Token[];
+extern const char kOAuth1Secret[];
 #endif
 extern const char kIpcDisabledMessages[];
 extern const char kShowHomeButton[];
@@ -178,17 +222,15 @@ extern const char kDeleteCookies[];
 extern const char kDeletePasswords[];
 extern const char kDeleteFormData[];
 extern const char kEnableSpellCheck[];
+extern const char kSpeechInputCensorResults[];
 extern const char kEnabledLabsExperiments[];
 extern const char kEnableAutoSpellCorrect[];
 extern const char kSavingBrowserHistoryDisabled[];
 extern const char kDeleteTimePeriod[];
 extern const char kPrintingEnabled[];
-extern const char kPrintingPageHeaderLeft[];
-extern const char kPrintingPageHeaderCenter[];
-extern const char kPrintingPageHeaderRight[];
-extern const char kPrintingPageFooterLeft[];
-extern const char kPrintingPageFooterCenter[];
-extern const char kPrintingPageFooterRight[];
+#if defined(GOOGLE_CHROME_BUILD) && !defined(OS_CHROMEOS)
+extern const char kPrintingPrintPreviewEnabledOnce[];
+#endif
 #if defined(TOOLKIT_USES_GTK)
 extern const char kUsesSystemTheme[];
 #endif
@@ -206,6 +248,7 @@ extern const char kPluginsDisabledPlugins[];
 extern const char kPluginsDisabledPluginsExceptions[];
 extern const char kPluginsEnabledPlugins[];
 extern const char kPluginsEnabledInternalPDF[];
+extern const char kPluginsEnabledNaCl[];
 extern const char kPluginsShowSetReaderDefaultInfobar[];
 extern const char kPluginsShowDetails[];
 extern const char kPluginsAllowOutdated[];
@@ -217,21 +260,21 @@ extern const char kShowUpdatePromotionInfoBar[];
 #endif
 extern const char kUseCustomChromeFrame[];
 extern const char kShowOmniboxSearchHint[];
-extern const char kDesktopNotificationDefaultContentSetting[];
-extern const char kDesktopNotificationAllowedOrigins[];
-extern const char kDesktopNotificationDeniedOrigins[];
+extern const char kDesktopNotificationDefaultContentSetting[];  // OBSOLETE
+extern const char kDesktopNotificationAllowedOrigins[];  // OBSOLETE
+extern const char kDesktopNotificationDeniedOrigins[];  // OBSOLETE
 extern const char kDesktopNotificationPosition[];
 extern const char kDefaultContentSettings[];
 extern const char kPerHostContentSettings[];  // OBSOLETE
 extern const char kContentSettingsVersion[];
-extern const char kContentSettingsPatterns[];
+extern const char kContentSettingsPatterns[];  // OBSOLETE
+extern const char kContentSettingsPatternPairs[];
 extern const char kBlockThirdPartyCookies[];
 extern const char kClearSiteDataOnExit[];
 extern const char kDefaultZoomLevel[];
 extern const char kPerHostZoomLevels[];
 extern const char kAutofillEnabled[];
 extern const char kAutofillAuxiliaryProfilesEnabled[];
-extern const char kAutofillDialogPlacement[];
 extern const char kAutofillPositiveUploadRate[];
 extern const char kAutofillNegativeUploadRate[];
 extern const char kAutofillPersonalDataManagerFirstRun[];
@@ -243,17 +286,30 @@ extern const char kEnableTranslate[];
 extern const char kEnableBookmarkBar[];
 extern const char kPinnedTabs[];
 extern const char kHttpThrottlingEnabled[];
-
-extern const char kPolicyRefreshRate[];
+extern const char kHttpThrottlingMayExperiment[];
 
 extern const char kDisable3DAPIs[];
 extern const char kEnableHyperlinkAuditing[];
 extern const char kEnableReferrers[];
 
+#if defined(OS_MACOSX)
+extern const char kPresentationModeEnabled[];
+#endif
+
+extern const char kImportBookmarks[];
+extern const char kImportHistory[];
+extern const char kImportHomepage[];
+extern const char kImportSearchEngine[];
+extern const char kImportSavedPasswords[];
+
+#if !defined(OS_MACOSX) && !defined(OS_CHROMEOS) && defined(OS_POSIX)
+extern const char kLocalProfileId[];
+extern const char kPasswordsUseLocalProfileId[];
+#endif
+
 // Local state prefs. Please add Profile prefs above instead.
 extern const char kCertRevocationCheckingEnabled[];
-extern const char kSSL3Enabled[];
-extern const char kTLS1Enabled[];
+extern const char kCipherSuiteBlacklist[];
 
 extern const char kMetricsClientID[];
 extern const char kMetricsSessionID[];
@@ -265,6 +321,7 @@ extern const char kMetricsOngoingLogs[];
 extern const char kProfileLastUsed[];
 extern const char kProfileDirectoryMap[];
 extern const char kProfilesNumCreated[];
+extern const char kProfileInfoCache[];
 
 extern const char kProfileMetrics[];
 extern const char kProfilePrefix[];
@@ -327,6 +384,7 @@ extern const char kHungPluginDetectFrequency[];
 extern const char kPluginMessageResponseTimeout[];
 
 extern const char kSpellCheckDictionary[];
+extern const char kSpellCheckUseSpellingService[];
 
 extern const char kExcludedSchemes[];
 
@@ -381,12 +439,15 @@ extern const char kExtensionSidebarWidth[];
 
 extern const char kNTPTipsResourceServer[];
 
+extern const char kNTP4IntroDisplayCount[];
 extern const char kNTPMostVisitedURLsBlacklist[];
 extern const char kNTPMostVisitedPinnedURLs[];
 extern const char kNTPPromoResourceCache[];
 extern const char kNTPPromoResourceCacheUpdate[];
 extern const char kNTPPromoResourceServer[];
 extern const char kNTPDateResourceServer[];
+extern const char kNTPShownBookmarksFolder[];
+extern const char kNTPShownPage[];
 extern const char kNTPShownSections[];
 extern const char kNTPPrefVersion[];
 extern const char kNTPCustomLogoStart[];
@@ -399,6 +460,7 @@ extern const char kNTPPromoLine[];
 extern const char kNTPPromoClosed[];
 extern const char kNTPPromoGroup[];
 extern const char kNTPPromoGroupTimeSlice[];
+extern const char kNTPPromoGroupMax[];
 extern const char kNTPPromoBuild[];
 extern const char kNTPWebStoreEnabled[];
 extern const char kNTPWebStorePromoLastId[];
@@ -409,6 +471,8 @@ extern const char kNTPWebStorePromoLink[];
 extern const char kNTPWebStorePromoLogo[];
 extern const char kNTPWebStorePromoExpire[];
 extern const char kNTPWebStorePromoUserGroup[];
+extern const char kNTPAppPageNames[];
+extern const char kNTPHideWebStorePromo[];
 
 extern const char kGpuBlacklist[];
 extern const char kGpuBlacklistUpdate[];
@@ -416,7 +480,6 @@ extern const char kGpuBlacklistUpdate[];
 extern const char kDevToolsDisabled[];
 extern const char kDevToolsOpenDocked[];
 extern const char kDevToolsSplitLocation[];
-extern const char kSyncSessions[];
 
 extern const char kSyncLastSyncedTime[];
 extern const char kSyncHasSetupCompleted[];
@@ -431,11 +494,15 @@ extern const char kSyncThemes[];
 extern const char kSyncTypedUrls[];
 extern const char kSyncExtensions[];
 extern const char kSyncManaged[];
+extern const char kSyncSearchEngines[];
+extern const char kSyncSessions[];
 extern const char kSyncSuppressStart[];
 extern const char kGoogleServicesUsername[];
+extern const char kSyncUsingOAuth[];
 extern const char kSyncUsingSecondaryPassphrase[];
 extern const char kEncryptionBootstrapToken[];
 extern const char kAutofillProfileMigrated[];
+extern const char kAcknowledgedSyncTypes[];
 
 extern const char kWebAppCreateOnDesktop[];
 extern const char kWebAppCreateInAppsMenu[];
@@ -447,9 +514,15 @@ extern const char kGeolocationContentSettings[];
 
 extern const char kLoginDatabaseMigrated[];
 
+extern const char kRemoteAccessClientFirewallTraversal[];
+extern const char kRemoteAccessHostFirewallTraversal[];
+
 extern const char kCloudPrintServiceURL[];
+extern const char kCloudPrintSigninURL[];
 extern const char kCloudPrintDialogWidth[];
 extern const char kCloudPrintDialogHeight[];
+extern const char kCloudPrintSigninDialogWidth[];
+extern const char kCloudPrintSigninDialogHeight[];
 extern const char kCloudPrintProxyEnabled[];
 extern const char kCloudPrintProxyId[];
 extern const char kCloudPrintAuthToken[];
@@ -459,14 +532,18 @@ extern const char kCloudPrintPrintSystemSettings[];
 extern const char kCloudPrintEnableJobPoll[];
 extern const char kCloudPrintRobotRefreshToken[];
 extern const char kCloudPrintRobotEmail[];
+extern const char kVirtualPrinterDriverEnabled[];
 
 extern const char kProxy[];
+extern const char kMaxConnectionsPerProxy[];
 
 extern const char kManagedDefaultCookiesSetting[];
 extern const char kManagedDefaultImagesSetting[];
 extern const char kManagedDefaultJavaScriptSetting[];
 extern const char kManagedDefaultPluginsSetting[];
 extern const char kManagedDefaultPopupsSetting[];
+extern const char kManagedDefaultGeolocationSetting[];
+extern const char kManagedDefaultNotificationsSetting[];
 
 extern const char kManagedCookiesAllowedForUrls[];
 extern const char kManagedCookiesBlockedForUrls[];
@@ -479,6 +556,7 @@ extern const char kManagedPluginsAllowedForUrls[];
 extern const char kManagedPluginsBlockedForUrls[];
 extern const char kManagedPopupsAllowedForUrls[];
 extern const char kManagedPopupsBlockedForUrls[];
+extern const char kManagedAutoSelectCertificateForUrls[];
 
 #if defined(OS_CHROMEOS)
 extern const char kSignedSettingsTempStorage[];
@@ -487,9 +565,10 @@ extern const char kCarrierDealPromoShown[];
 #endif
 
 extern const char kClearPluginLSODataEnabled[];
-
 extern const char kDiskCacheDir[];
 
+extern const char kChromeOsReleaseChannel[];
+
 extern const char kRegisteredBackgroundContents[];
 
 extern const char kAuthSchemes[];
@@ -507,6 +586,9 @@ extern const char kCustomHandlersEnabled[];
 extern const char kUserCreatedLoginItem[];
 extern const char kBackgroundModeEnabled[];
 
+extern const char kDevicePolicyRefreshRate[];
+extern const char kUserPolicyRefreshRate[];
+
 }  // namespace prefs
 
 #endif  // CHROME_COMMON_PREF_NAMES_H_
