@@ -9,13 +9,16 @@
 #include "base/file_util.h"
 #include "base/path_service.h"
 #include "base/string_util.h"
+#include "chrome/common/chrome_plugin_messages.h"
 #include "chrome/default_plugin/plugin_main.h"
+#include "content/common/child_thread.h"
 #include "googleurl/src/gurl.h"
 #include "grit/default_plugin_resources.h"
 #include "grit/webkit_strings.h"
+#include "ui/base/l10n/l10n_util.h"
 #include "ui/base/l10n/l10n_util_mac.h"
 #include "ui/base/resource/resource_bundle.h"
-#include "ui/gfx/image.h"
+#include "ui/gfx/image/image.h"
 #include "ui/gfx/scoped_ns_graphics_context_save_gstate_mac.h"
 #include "unicode/locid.h"
 #include "webkit/glue/webkit_glue.h"
@@ -48,13 +51,14 @@ bool PluginInstallerImpl::Initialize(void* module_handle, NPP instance,
   instance_ = instance;
   mime_type_ = mime_type;
 
-  command_ = [l10n_util::FixUpWindowsStyleLabel(webkit_glue::GetLocalizedString(
+  command_ = [l10n_util::FixUpWindowsStyleLabel(l10n_util::GetStringUTF16(
       IDS_DEFAULT_PLUGIN_NO_PLUGIN_AVAILABLE_MSG)) retain];
 
   ResourceBundle& rb = ResourceBundle::GetSharedInstance();
   image_ = rb.GetNativeImageNamed(IDR_PLUGIN_ICON);
 
-  return true;
+  return PluginInstallerBase::Initialize(module_handle, instance, mime_type,
+                                         argc, argn, argv);
 }
 
 bool PluginInstallerImpl::NPP_SetWindow(NPWindow* window_info) {
@@ -191,9 +195,10 @@ void PluginInstallerImpl::ShowInstallDialog() {
 }
 
 void PluginInstallerImpl::NotifyPluginStatus(int status) {
-  default_plugin::g_browser->getvalue(
-      instance_,
-      static_cast<NPNVariable>(
-          webkit::npapi::default_plugin::kMissingPluginStatusStart + status),
-      NULL);
+  ChildThread::current()->Send(
+      new ChromePluginProcessHostMsg_MissingPluginStatus(
+          status,
+          renderer_process_id(),
+          render_view_id(),
+          0));
 }
