@@ -8,8 +8,6 @@
 
 #include <string>
 
-#include "v8/include/v8.h"
-
 class ExtensionDispatcher;
 class GURL;
 class RenderThreadBase;
@@ -19,6 +17,12 @@ namespace base {
 class ListValue;
 }
 
+namespace v8 {
+class Context;
+class Extension;
+template <class T> class Handle;
+}
+
 namespace WebKit {
 class WebFrame;
 }
@@ -37,8 +41,9 @@ class EventBindings {
 
   // Handle a script context coming / going away.
   static void HandleContextCreated(WebKit::WebFrame* frame,
-                                   bool content_script,
-                                   ExtensionDispatcher* extension_dispatcher);
+                                   v8::Handle<v8::Context> context,
+                                   ExtensionDispatcher* extension_dispatcher,
+                                   int isolated_world_id);
   static void HandleContextDestroyed(WebKit::WebFrame* frame);
 
   // Calls the given function in each registered context which is listening for
