@@ -6,14 +6,15 @@
 #define CHROME_RENDERER_EXTENSIONS_RENDERER_EXTENSION_BINDINGS_H_
 #pragma once
 
-#include "v8/include/v8.h"
-
 #include <string>
 
 class ExtensionDispatcher;
-class GURL;
 class RenderView;
 
+namespace v8 {
+class Extension;
+}
+
 // This class adds extension-related javascript bindings to a renderer.  It is
 // used by both web renderers and extension processes.
 class RendererExtensionBindings {
@@ -23,6 +24,12 @@ class RendererExtensionBindings {
 
   // Creates an instance of the extension.
   static v8::Extension* Get(ExtensionDispatcher* dispatcher);
+
+  // Delivers a message sent using content script messaging. If
+  // restrict_to_render_view is specified, only contexts in that render view
+  // will receive the message.
+  static void DeliverMessage(int target_port_id, const std::string& message,
+                             RenderView* restrict_to_render_view);
 };
 
 #endif  // CHROME_RENDERER_EXTENSIONS_RENDERER_EXTENSION_BINDINGS_H_
