@@ -1034,8 +1034,10 @@ void PrintWebViewHelper::UpdatePrintableSizeInPrintParameters(
     WebNode* node,
     PrepareFrameAndViewForPrint* prepare,
     PrintMsg_Print_Params* params) {
-  if (PrintingNodeOrPdfFrame(frame, node))
+  if (PrintingNodeOrPdfFrame(frame, node)) {
+    prepare->UpdatePrintParams(*params);
     return;
+  }
   PageSizeMargins page_layout_in_points;
   PrintWebViewHelper::GetPageSizeAndMarginsInPoints(frame, 0, *params,
                                                     &page_layout_in_points);
@@ -1394,8 +1396,6 @@ bool PrintWebViewHelper::PrintPreviewContext::CreatePreviewDocument(
   DCHECK(IsReadyToRender());
   state_ = RENDERING;
 
-  print_params_.reset(new PrintMsg_Print_Params(*print_params));
-
   metafile_.reset(new printing::PreviewMetafile);
   if (!metafile_->Init()) {
     LOG(ERROR) << "PreviewMetafile Init failed";
@@ -1408,6 +1408,8 @@ bool PrintWebViewHelper::PrintPreviewContext::CreatePreviewDocument(
   UpdatePrintableSizeInPrintParameters(frame_, node_.get(),
                                        prep_frame_view_.get(), print_params);
 
+  print_params_.reset(new PrintMsg_Print_Params(*print_params));
+
   total_page_count_ = prep_frame_view_->GetExpectedPageCount();
   if (total_page_count_ == 0) {
     LOG(ERROR) << "CreatePreviewDocument got 0 page count";
