@@ -31,6 +31,8 @@
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/gfx/rect.h"
 
+#pragma comment(lib, "rpcrt4.lib")  // for UuidToString & Co.
+
 namespace {
 
 class DevMode {
@@ -529,7 +531,7 @@ class PrintSystemWin : public PrintSystem {
                 render_area,
                 printer_dpi,
                 page_ranges,
-                base::MessageLoopProxy::CreateForCurrentThread()));
+                base::MessageLoopProxy::current()));
       }
       // Called on the service process IO thread.
       void RenderPDFPagesInSandbox(
@@ -647,7 +649,7 @@ class PrintSystemWin : public PrintSystem {
           NewRunnableMethod(
               this,
               &PrinterCapsHandler::GetPrinterCapsAndDefaultsImpl,
-              base::MessageLoopProxy::CreateForCurrentThread()));
+              base::MessageLoopProxy::current()));
     }
 
     virtual void OnChildDied() {
@@ -862,7 +864,7 @@ std::string PrintSystem::GenerateProxyId() {
 }
 
 scoped_refptr<PrintSystem> PrintSystem::CreateInstance(
-    const DictionaryValue* print_system_settings) {
+    const base::DictionaryValue* print_system_settings) {
   return new PrintSystemWin;
 }
 
