@@ -47,7 +47,7 @@ PrinterJobHandler::PrinterJobHandler(
       server_error_count_(0),
       print_thread_("Chrome_CloudPrintJobPrintThread"),
       job_handler_message_loop_proxy_(
-          base::MessageLoopProxy::CreateForCurrentThread()),
+          base::MessageLoopProxy::current()),
       shutting_down_(false),
       job_check_pending_(false),
       printer_update_pending_(true),
@@ -163,6 +163,13 @@ void PrinterJobHandler::CheckForJobs(const std::string& reason) {
 }
 
 bool PrinterJobHandler::UpdatePrinterInfo() {
+  if (!printer_watcher_) {
+    LOG(ERROR) << "CP_PROXY: Printer watcher is missing."
+               << "Check printer server url for printer id: "
+               << printer_info_cloud_.printer_id;
+    return false;
+  }
+
   VLOG(1) << "CP_PROXY: Update printer info, id: "
           << printer_info_cloud_.printer_id;
   // We need to update the parts of the printer info that have changed
@@ -187,14 +194,16 @@ void PrinterJobHandler::OnReceivePrinterCaps(
     const std::string& printer_name,
     const printing::PrinterCapsAndDefaults& caps_and_defaults) {
   printing::PrinterBasicInfo printer_info;
-  printer_watcher_->GetCurrentPrinterInfo(&printer_info);
+  if (printer_watcher_)
+    printer_watcher_->GetCurrentPrinterInfo(&printer_info);
 
   std::string post_data;
   std::string mime_boundary;
   CloudPrintHelpers::CreateMimeBoundaryForUpload(&mime_boundary);
 
   if (succeeded) {
-    std::string caps_hash = MD5String(caps_and_defaults.printer_capabilities);
+    std::string caps_hash =
+        base::MD5String(caps_and_defaults.printer_capabilities);
     if (caps_hash != printer_info_cloud_.caps_hash) {
       // Hashes don't match, we need to upload new capabilities (the defaults
       // go for free along with the capabilities)
@@ -414,8 +423,7 @@ PrinterJobHandler::HandleJobMetadataResponse(
   bool job_available = false;
   if (succeeded) {
     ListValue* job_list = NULL;
-    json_data->GetList(kJobListValue, &job_list);
-    if (job_list) {
+    if (json_data->GetList(kJobListValue, &job_list) && job_list) {
       // Even though it is a job list, for now we are only interested in the
       // first job
       DictionaryValue* job_data = NULL;
