@@ -1,3 +1,6 @@
+from mod_pywebsocket import handshake
+
+
 def web_socket_do_extra_handshake(request):
     pass
 
@@ -8,4 +11,4 @@ def web_socket_transfer_data(request):
     msg += "\x01\xff"
     msg += "\0should be skipped\xff"
     request.connection.write(msg)
-    raise Exception("Abort the connection") # Prevents pywebsocket from starting closing handshake.
+    raise handshake.AbortedByUserException("Abort the connection") # Prevents pywebsocket from starting closing handshake.
