@@ -1,3 +1,6 @@
+from mod_pywebsocket import handshake
+
+
 def web_socket_do_extra_handshake(request):
     msg = 'HTTP/1.1 101 WebSocket Protocol Handshake\n' # Does not end with "\r\n".
     msg += 'Upgrade: WebSocket\r\n'
@@ -8,7 +11,7 @@ def web_socket_do_extra_handshake(request):
     msg += request.ws_challenge_md5
     request.connection.write(msg)
     print msg
-    raise Exception('Abort the connection') # Prevents pywebsocket from sending its own handshake message.
+    raise handshake.AbortedByUserException('Abort the connection') # Prevents pywebsocket from sending its own handshake message.
 
 
 def web_socket_transfer_data(request):
