@@ -30,7 +30,7 @@ class ThemesTest(pyauto.PyUITest):
     self.assertFalse(self.GetThemeInfo())  # Verify there's no theme at startup
     crx_file = os.path.abspath(
         os.path.join(self.DataDir(), 'extensions', 'theme.crx'))
-    self.assertTrue(self.SetTheme(pyauto.FilePath(crx_file)))
+    self.assertTrue(self.SetTheme(crx_file))
     # Verify "theme installed" infobar shows up
     self.assertTrue(self.WaitForInfobarCount(1))
     theme = self.GetThemeInfo()
@@ -43,7 +43,7 @@ class ThemesTest(pyauto.PyUITest):
     self.assertFalse(self.GetThemeInfo())  # Verify there's no theme at startup
     crx_file = os.path.abspath(
         os.path.join(self.DataDir(), 'extensions', 'theme.crx'))
-    self.assertTrue(self.SetTheme(pyauto.FilePath(crx_file)))
+    self.assertTrue(self.SetTheme(crx_file))
     # Verify "theme installed" infobar shows up
     self.assertTrue(self.WaitForInfobarCount(1))
     theme = self.GetThemeInfo()
@@ -53,7 +53,7 @@ class ThemesTest(pyauto.PyUITest):
     """Verify theme reset."""
     crx_file = os.path.abspath(
         os.path.join(self.DataDir(), 'extensions', 'theme.crx'))
-    self.assertTrue(self.SetTheme(pyauto.FilePath(crx_file)))
+    self.assertTrue(self.SetTheme(crx_file))
     self.assertTrue(self.ResetToDefaultTheme())
     self.assertFalse(self.GetThemeInfo())
 
@@ -80,7 +80,7 @@ class ThemesTest(pyauto.PyUITest):
       # Apply each theme in this group.
       for theme in this_group:
         logging.debug('Applying theme: %s' % theme)
-        self.assertTrue(self.SetTheme(pyauto.FilePath(theme)),
+        self.assertTrue(self.SetTheme(theme),
                         'Theme %s not installed.' % theme)
 
       for url in urls:
@@ -138,4 +138,3 @@ class ThemesTest(pyauto.PyUITest):
 
 if __name__ == '__main__':
   pyauto_functional.Main()
-
