@@ -6,6 +6,7 @@
 #include "base/path_service.h"
 #include "base/test/test_timeouts.h"
 #include "build/build_config.h"
+#include "content/common/content_switches.h"
 #include "chrome/common/chrome_switches.h"
 #include "chrome/test/automation/tab_proxy.h"
 #include "chrome/test/ui/ui_test.h"
@@ -55,6 +56,9 @@ class PPAPITest : public UITest {
     // TODO(dumi): remove this switch once we have a quota management
     // system in place.
     launch_arguments_.AppendSwitch(switches::kUnlimitedQuotaForFiles);
+
+    // Smooth scrolling confuses the scrollbar test.
+    launch_arguments_.AppendSwitch(switches::kDisableSmoothScrolling);
   }
 
   void RunTest(const std::string& test_case) {
@@ -90,26 +94,33 @@ class PPAPITest : public UITest {
     ASSERT_TRUE(tab.get());
     ASSERT_TRUE(tab->NavigateToURL(test_url));
 
-    // First wait for the "starting" signal. This cookie is set at the start
-    // of every test. Waiting for this separately allows us to avoid a single
-    // long timeout. Instead, we can have two timeouts which allow startup +
-    // test execution time to take a while on a loaded computer, while also
-    // making sure we're making forward progress.
-    std::string startup_cookie =
-        WaitUntilCookieNonEmpty(tab.get(), test_url,
-            "STARTUP_COOKIE", TestTimeouts::action_max_timeout_ms());
+    // See comment above TestingInstance in ppapi/test/testing_instance.h.
+    // Basically it sets a series of numbered cookies. The value of "..." means
+    // it's still working and we should continue to wait, any other value
+    // indicates completion (in this case it will start with "PASS" or "FAIL").
+    // This keeps us from timing out on cookie waits for long tests.
+    int progress_number = 0;
+    std::string progress;
+    while (true) {
+      std::string cookie_name = StringPrintf("PPAPI_PROGRESS_%d",
+                                             progress_number);
+      progress = WaitUntilCookieNonEmpty(tab.get(), test_url,
+            cookie_name.c_str(), TestTimeouts::large_test_timeout_ms());
+      if (progress != "...")
+        break;
+      progress_number++;
+    }
 
-    // If this fails, the plugin couldn't be loaded in the given amount of
-    // time. This may mean the plugin was not found or possibly the system
-    // can't load it due to missing symbols, etc.
-    ASSERT_STREQ("STARTED", startup_cookie.c_str())
-        << "Plugin couldn't be loaded. Make sure the PPAPI test plugin is "
-        << "built, in the right place, and doesn't have any missing symbols.";
+    if (progress_number == 0) {
+      // Failing the first time probably means the plugin wasn't loaded.
+      ASSERT_FALSE(progress.empty())
+          << "Plugin couldn't be loaded. Make sure the PPAPI test plugin is "
+          << "built, in the right place, and doesn't have any missing symbols.";
+    } else {
+      ASSERT_FALSE(progress.empty()) << "Test timed out.";
+    }
 
-    std::string escaped_value =
-        WaitUntilCookieNonEmpty(tab.get(), test_url,
-            "COMPLETION_COOKIE", TestTimeouts::large_test_timeout_ms());
-    EXPECT_STREQ("PASS", escaped_value.c_str());
+    EXPECT_STREQ("PASS", progress.c_str());
   }
 };
 
@@ -159,7 +170,8 @@ TEST_PPAPI_IN_PROCESS(CursorControl)
 TEST_PPAPI_OUT_OF_PROCESS(CursorControl)
 
 TEST_PPAPI_IN_PROCESS(Instance)
-TEST_PPAPI_OUT_OF_PROCESS(Instance)
+// http://crbug.com/91729
+TEST_PPAPI_OUT_OF_PROCESS(DISABLED_Instance)
 
 TEST_PPAPI_IN_PROCESS(Graphics2D)
 // Disabled because it times out: http://crbug.com/89961
@@ -172,8 +184,16 @@ TEST_PPAPI_IN_PROCESS(Buffer)
 TEST_PPAPI_OUT_OF_PROCESS(Buffer)
 
 TEST_PPAPI_IN_PROCESS_VIA_HTTP(URLLoader)
+
 // http://crbug.com/89961
-TEST_F(OutOfProcessPPAPITest, FAILS_URLLoader) {
+#if defined(OS_WIN)
+// It often takes too long time (and fails otherwise) on Windows.
+#define MAYBE_URLLoader DISABLED_URLLoader
+#else
+#define MAYBE_URLLoader FAILS_URLLoader
+#endif
+
+TEST_F(OutOfProcessPPAPITest, MAYBE_URLLoader) {
   RunTestViaHTTP("URLLoader");
 }
 
@@ -202,9 +222,15 @@ TEST_PPAPI_IN_PROCESS(VarDeprecated)
 // Disabled because it times out: http://crbug.com/89961
 //TEST_PPAPI_OUT_OF_PROCESS(VarDeprecated)
 
+// Windows defines 'PostMessage', so we have to undef it.
+#ifdef PostMessage
+#undef PostMessage
+#endif
 TEST_PPAPI_IN_PROCESS(PostMessage)
-// Disabled because it times out: http://crbug.com/89961
-//TEST_PPAPI_OUT_OF_PROCESS(PostMessage)
+#if !defined(OS_WIN)
+// Times out on Windows XP: http://crbug.com/95557
+TEST_PPAPI_OUT_OF_PROCESS(PostMessage)
+#endif
 
 TEST_PPAPI_IN_PROCESS(Memory)
 TEST_PPAPI_OUT_OF_PROCESS(Memory)
@@ -212,6 +238,9 @@ TEST_PPAPI_OUT_OF_PROCESS(Memory)
 TEST_PPAPI_IN_PROCESS(QueryPolicy)
 //TEST_PPAPI_OUT_OF_PROCESS(QueryPolicy)
 
+TEST_PPAPI_IN_PROCESS(VideoDecoder)
+TEST_PPAPI_OUT_OF_PROCESS(VideoDecoder)
+
 // http://crbug.com/90039 and http://crbug.com/83443 (Mac)
 TEST_F(PPAPITest, FAILS_FileIO) {
   RunTestViaHTTP("FileIO");
@@ -246,7 +275,7 @@ TEST_F(OutOfProcessPPAPITest, FAILS_DirectoryReader) {
 }
 
 #if defined(ENABLE_P2P_APIS)
-// Flaky. http://crbug.com/84295
+// Flaky. http://crbug.com/84294
 TEST_F(PPAPITest, FLAKY_Transport) {
   RunTest("Transport");
 }
@@ -261,9 +290,3 @@ TEST_PPAPI_IN_PROCESS(UMA)
 TEST_F(OutOfProcessPPAPITest, FAILS_UMA) {
   RunTest("UMA");
 }
-
-TEST_PPAPI_IN_PROCESS(VideoDecoder)
-// There is no proxy yet (vrk is adding it).
-TEST_F(OutOfProcessPPAPITest, FAILS_VideoDecoder) {
-  RunTest("VideoDecoder");
-}
