@@ -7,8 +7,8 @@
 #include <string>
 
 #include "chrome/test/webdriver/commands/response.h"
-#include "chrome/test/webdriver/session.h"
 #include "chrome/test/webdriver/webdriver_error.h"
+#include "chrome/test/webdriver/webdriver_session.h"
 
 namespace webdriver {
 
