@@ -16,6 +16,7 @@ class DictionaryValue;
 
 namespace webdriver {
 
+class Error;
 class Response;
 class Session;
 
@@ -34,6 +35,8 @@ class WebDriverCommand : public Command {
   // Initializes this webdriver command by fetching the command session.
   virtual bool Init(Response* const response);
 
+  virtual void Finish();
+
  protected:
   Session* session_;
 
