@@ -19,17 +19,10 @@
 
 #include "base/command_line.h"
 #include "base/debug/trace_event.h"
-#include "base/file_util.h"
 #include "base/logging.h"
-#include "base/path_service.h"
-#include "base/process_util.h"
-#include "base/string_tokenizer.h"
 #include "base/string_util.h"
-#include "base/threading/thread.h"
-#include "base/threading/thread_local.h"
 #include "base/utf_string_conversions.h"
 #include "base/win/scoped_variant.h"
-#include "base/win/win_util.h"
 #include "chrome/app/chrome_command_ids.h"
 #include "chrome/app/chrome_dll_resource.h"
 #include "chrome/common/automation_messages.h"
@@ -49,8 +42,6 @@
 DEFINE_GUID(CGID_DocHostCmdPriv, 0x000214D4L, 0, 0, 0xC0, 0, 0, 0, 0, 0, 0,
             0x46);
 
-base::ThreadLocalPointer<ChromeActiveDocument> g_active_doc_cache;
-
 bool g_first_launch_by_process_ = true;
 
 const DWORD kIEEncodingIdArray[] = {
@@ -73,29 +64,14 @@ ChromeActiveDocument::ChromeActiveDocument()
 }
 
 HRESULT ChromeActiveDocument::FinalConstruct() {
-  // If we have a cached ChromeActiveDocument instance in TLS, then grab
-  // ownership of the cached document's automation client. This is an
-  // optimization to get Chrome active documents to load faster.
-  ChromeActiveDocument* cached_document = g_active_doc_cache.Get();
-  if (cached_document && cached_document->IsValid()) {
-    SetResourceModule();
-    DCHECK(automation_client_.get() == NULL);
-    automation_client_.swap(cached_document->automation_client_);
-    DVLOG(1) << "Reusing automation client instance from " << cached_document;
-    DCHECK(automation_client_.get() != NULL);
-    automation_client_->Reinitialize(this, url_fetcher_.get());
-    is_automation_client_reused_ = true;
-    OnAutomationServerReady();
-  } else {
-    // The FinalConstruct implementation in the ChromeFrameActivexBase class
-    // i.e. Base creates an instance of the ChromeFrameAutomationClient class
-    // and initializes it, which would spawn a new Chrome process, etc.
-    // We don't want to be doing this if we have a cached document, whose
-    // automation client instance can be reused.
-    HRESULT hr = BaseActiveX::FinalConstruct();
-    if (FAILED(hr))
-      return hr;
-  }
+  // The FinalConstruct implementation in the ChromeFrameActivexBase class
+  // i.e. Base creates an instance of the ChromeFrameAutomationClient class
+  // and initializes it, which would spawn a new Chrome process, etc.
+  // We don't want to be doing this if we have a cached document, whose
+  // automation client instance can be reused.
+  HRESULT hr = BaseActiveX::FinalConstruct();
+  if (FAILED(hr))
+    return hr;
 
   InitializeAutomationSettings();
 
@@ -543,13 +519,6 @@ HRESULT ChromeActiveDocument::GetInPlaceFrame(
 HRESULT ChromeActiveDocument::IOleObject_SetClientSite(
     IOleClientSite* client_site) {
   if (client_site == NULL) {
-    ChromeActiveDocument* cached_document = g_active_doc_cache.Get();
-    if (cached_document) {
-      DCHECK(this == cached_document);
-      g_active_doc_cache.Set(NULL);
-      cached_document->Release();
-    }
-
     base::win::ScopedComPtr<IDocHostUIHandler> doc_host_handler;
     if (doc_site_)
       doc_host_handler.QueryFrom(doc_site_);
@@ -930,6 +899,43 @@ void ChromeActiveDocument::OnUnload(const GUID* cmd_group_guid,
                                     VARIANT* in_args,
                                     VARIANT* out_args) {
   if (IsValid() && out_args) {
+    // If the navigation is attempted to the url loaded in CF, with the only
+    // difference being the addition of an anchor, IE will attempt to unload
+    // the currently loaded document which basically would end up running
+    // unload handlers on the currently loaded document thus rendering it non
+    // functional. We handle this as below:-
+    // The BHO receives the new url in the BeforeNavigate event.
+    // We compare the non anchor portions of the url in the BHO with the loaded
+    // url and if they match, we initiate a Chrome navigation to the url with
+    // the anchor which works nicely.
+    // We don't want to continue processing the unload in this case.
+    // Note:-
+    // IE handles these navigations by querying the loaded document for
+    // IHTMLDocument which then handles the new navigation. That won't work for
+    // us as we don't implement IHTMLDocument.
+    NavigationManager* mgr = NavigationManager::GetThreadInstance();
+    DLOG_IF(ERROR, !mgr) << "Couldn't get instance of NavigationManager";
+    if (mgr) {
+      ChromeFrameUrl url;
+      url.Parse(mgr->url());
+      if (url.gurl().has_ref()) {
+        url_canon::Replacements<char> replacements;
+        replacements.ClearRef();
+
+        if (url.gurl().ReplaceComponents(replacements) ==
+            GURL(static_cast<BSTR>(url_))) {
+          // We want to reuse the existing automation client and channel for
+          // initiating the new navigation. Setting the
+          // is_automation_client_reused_ flag to true before calling the
+          // LaunchUrl function achieves that.
+          is_automation_client_reused_ = true;
+          LaunchUrl(url, mgr->referrer());
+          out_args->vt = VT_BOOL;
+          out_args->boolVal = VARIANT_FALSE;
+          return;
+        }
+      }
+    }
     bool should_unload = true;
     automation_client_->OnUnload(&should_unload);
     out_args->vt = VT_BOOL;
@@ -937,23 +943,6 @@ void ChromeActiveDocument::OnUnload(const GUID* cmd_group_guid,
   }
 }
 
-void ChromeActiveDocument::OnOpenURL(const GURL& url_to_open,
-                                     const GURL& referrer,
-                                     int open_disposition) {
-  // If the disposition indicates that we should be opening the URL in the
-  // current tab, then we can reuse the ChromeFrameAutomationClient instance
-  // maintained by the current ChromeActiveDocument instance. We cache this
-  // instance so that it can be used by the new ChromeActiveDocument instance
-  // which may be instantiated for handling the new URL.
-  if (open_disposition == CURRENT_TAB) {
-    // Grab a reference to ensure that the document remains valid.
-    AddRef();
-    g_active_doc_cache.Set(this);
-  }
-
-  BaseActiveX::OnOpenURL(url_to_open, referrer, open_disposition);
-}
-
 void ChromeActiveDocument::OnAttachExternalTab(
     const AttachExternalTabParams& params) {
   if (!automation_client_.get()) {
@@ -1054,6 +1043,8 @@ bool ChromeActiveDocument::LaunchUrl(const ChromeFrameUrl& cf_url,
     }
   }
 
+  document_url_ = cf_url.gurl().spec();
+
   url_.Allocate(UTF8ToWide(cf_url.gurl().spec()).c_str());
   if (cf_url.attach_to_external_tab()) {
     automation_client_->AttachExternalTab(cf_url.cookie());
