@@ -8,20 +8,21 @@
 #include <atlcom.h>
 #include <exdisp.h>
 
-#include "app/app_paths.h"
-#include "app/win/scoped_com_initializer.h"
 #include "base/command_line.h"
 #include "base/debug/debugger.h"
 #include "base/file_util.h"
 #include "base/file_version_info.h"
 #include "base/i18n/icu_util.h"
+#include "base/lazy_instance.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/path_service.h"
+#include "base/scoped_temp_dir.h"
 #include "base/string_util.h"
 #include "base/stringprintf.h"
 #include "base/system_monitor/system_monitor.h"
 #include "base/test/test_timeouts.h"
 #include "base/threading/platform_thread.h"
+#include "base/win/scoped_com_initializer.h"
 #include "base/win/scoped_comptr.h"
 #include "base/win/scoped_handle.h"
 #include "chrome/browser/automation/automation_provider_list.h"
@@ -39,6 +40,7 @@
 #include "chrome/common/chrome_switches.h"
 #include "chrome/common/pref_names.h"
 #include "chrome/renderer/chrome_content_renderer_client.h"
+#include "chrome_frame/crash_server_init.h"
 #include "chrome_frame/test/chrome_frame_test_utils.h"
 #include "chrome_frame/test/net/test_automation_resource_message_filter.h"
 #include "chrome_frame/test/simulate_input.h"
@@ -109,18 +111,38 @@ bool SetFocusToAccessibleWindow(HWND hwnd) {
 class FakeBrowserProcessImpl : public BrowserProcessImpl {
  public:
   explicit FakeBrowserProcessImpl(const CommandLine& command_line)
-      : BrowserProcessImpl(command_line) {}
+      : BrowserProcessImpl(command_line) {
+    profiles_dir_.CreateUniqueTempDir();
+  }
 
   virtual ProfileManager* profile_manager() {
-    if (!profile_manager_.get())
-      profile_manager_.reset(new ProfileManagerWithoutInit);
+    if (!profile_manager_.get()) {
+      profile_manager_.reset(
+          new ProfileManagerWithoutInit(profiles_dir_.path()));
+    }
     return profile_manager_.get();
   }
 
+  virtual MetricsService* metrics_service() {
+    return NULL;
+  }
+
  private:
+  ScopedTempDir profiles_dir_;
   scoped_ptr<ProfileManager> profile_manager_;
 };
 
+base::LazyInstance<chrome::ChromeContentClient>
+    g_chrome_content_client(base::LINKER_INITIALIZED);
+
+// Override the default ContentBrowserClient to let Chrome participate in
+// content logic.  Must be done before any tabs are created.
+base::LazyInstance<chrome::ChromeContentBrowserClient>
+    g_browser_client(base::LINKER_INITIALIZED);
+
+base::LazyInstance<chrome::ChromeContentRendererClient>
+    g_renderer_client(base::LINKER_INITIALIZED);
+
 }  // namespace
 
 
@@ -212,8 +234,11 @@ void FakeExternalTab::Initialize() {
   base::SystemMonitor system_monitor;
 
   icu_util::Initialize();
+  TestTimeouts::Initialize();
 
-  app::RegisterPathProvider();
+  // Do not call chrome::RegisterPathProvider() since it is also called by our
+  // test runner, CFUrlRequestUnittestRunner, and calling it twice unfortunately
+  // causes a DCHECK().
   content::RegisterPathProvider();
   ui::RegisterPathProvider();
 
@@ -238,12 +263,24 @@ void FakeExternalTab::Initialize() {
   g_browser_process->SetApplicationLocale("en-US");
 
   RenderProcessHost::set_run_renderer_in_process(true);
-  browser::RegisterLocalState(browser_process_->local_state());
+
+  browser_process_->local_state()->RegisterBooleanPref(
+      prefs::kMetricsReportingEnabled, false);
 
   FilePath profile_path(ProfileManager::GetDefaultProfileDir(user_data()));
 
   Profile* profile =
       g_browser_process->profile_manager()->GetProfile(profile_path);
+
+  // Initialize the content client which that code uses to talk to Chrome.
+  content::SetContentClient(&g_chrome_content_client.Get());
+
+  // Override the default ContentBrowserClient to let Chrome participate in
+  // content logic.  Must be done before any tabs are created.
+  content::GetContentClient()->set_browser(&g_browser_client.Get());
+
+  content::GetContentClient()->set_renderer(&g_renderer_client.Get());
+
   // Create the child threads.
   g_browser_process->db_thread();
   g_browser_process->file_thread();
@@ -281,7 +318,7 @@ void CFUrlRequestUnittestRunner::StartChromeFrameInHostBrowser() {
   if (!ShouldLaunchBrowser())
     return;
 
-  app::win::ScopedCOMInitializer com;
+  base::win::ScopedCOMInitializer com;
   chrome_frame_test::CloseAllIEWindows();
 
   test_http_server_.reset(new test_server::SimpleWebServer(kTestServerPort));
@@ -303,7 +340,7 @@ void CFUrlRequestUnittestRunner::StartChromeFrameInHostBrowser() {
 
 void CFUrlRequestUnittestRunner::ShutDownHostBrowser() {
   if (ShouldLaunchBrowser()) {
-    app::win::ScopedCOMInitializer com;
+    base::win::ScopedCOMInitializer com;
     chrome_frame_test::CloseAllIEWindows();
   }
 }
@@ -321,9 +358,7 @@ void CFUrlRequestUnittestRunner::Initialize() {
 
   SuppressErrorDialogs();
   base::debug::SetSuppressDebugUI(true);
-#if !defined(PURIFY)
   logging::SetLogAssertHandler(UnitTestAssertHandler);
-#endif  // !defined(PURIFY)
 
   // Next, do some initialization for NetTestSuite.
   NetTestSuite::InitializeTestThread();
@@ -449,6 +484,7 @@ void FilterDisabledTests() {
     // later by using the new INTERNET_OPTION_SUPPRESS_BEHAVIOR flags
     // See http://msdn.microsoft.com/en-us/library/aa385328(VS.85).aspx
     "URLRequestTest.DoNotSaveCookies",
+    "URLRequestTest.DelayedCookieCallback",
 
     // TODO(ananta): This test has been consistently failing. Disabling it for
     // now.
@@ -476,6 +512,12 @@ void FilterDisabledTests() {
     // This test modifies the UploadData object after it has been marshaled to
     // ChromeFrame. We don't support this.
     "URLRequestTestHTTP.TestPostChunkedDataAfterStart",
+
+    // Do not work in CF, it may well be that IE is unconditionally
+    // adding Accept-Encoding header by default to outgoing requests.
+    "URLRequestTestHTTP.DefaultAcceptEncoding",
+    "URLRequestTestHTTP.OverrideAcceptEncoding",
+
     // Not supported in ChromeFrame as we use IE's network stack.
     "URLRequestTest.NetworkDelegateProxyError",
   };
@@ -516,17 +558,8 @@ int main(int argc, char** argv) {
     return 0;
   }
 
-  // Initialize the content client which that code uses to talk to Chrome.
-  chrome::ChromeContentClient chrome_content_client;
-  content::SetContentClient(&chrome_content_client);
-
-  // Override the default ContentBrowserClient to let Chrome participate in
-  // content logic.  Must be done before any tabs are created.
-  chrome::ChromeContentBrowserClient browser_client;
-  content::GetContentClient()->set_browser(&browser_client);
-
-  chrome::ChromeContentRendererClient renderer_client;
-  content::GetContentClient()->set_renderer(&renderer_client);
+  google_breakpad::scoped_ptr<google_breakpad::ExceptionHandler> breakpad(
+      InitializeCrashReporting(HEADLESS));
 
   // TODO(tommi): Stuff be broke. Needs a fixin'.
   // This is awkward: the TestSuite derived CFUrlRequestUnittestRunner contains
@@ -534,6 +567,8 @@ int main(int argc, char** argv) {
   // below require. So we have to instantiate this first.
   CFUrlRequestUnittestRunner test_suite(argc, argv);
 
+  base::ProcessHandle crash_service = chrome_frame_test::StartCrashService();
+
   WindowWatchdog watchdog;
   // See url_request_unittest.cc for these credentials.
   SupplyProxyCredentials credentials("user", "secret");
@@ -541,5 +576,17 @@ int main(int argc, char** argv) {
   testing::InitGoogleTest(&argc, argv);
   FilterDisabledTests();
   test_suite.RunMainUIThread();
+
+  if (crash_service)
+    base::KillProcess(crash_service, 0, false);
+
+  base::KillProcesses(chrome_frame_test::kIEImageName, 0, NULL);
+  base::KillProcesses(chrome_frame_test::kIEBrokerImageName, 0, NULL);
+  // Avoid CRT cleanup in debug test runs to ensure that webkit ASSERTs which
+  // check if globals are created and destroyed on the same thread don't fire.
+  // Webkit global objects are created on the inproc renderer thread.
+#if !defined(NDEBUG)
+  ExitProcess(test_suite.test_result());
+#endif  // NDEBUG
   return test_suite.test_result();
 }
