@@ -20,7 +20,7 @@
 // --noclearprofile: do not clear profile dir before firing up each time.
 // --savedebuglog: save Chrome, V8, and test debug log for each page loaded.
 #include <fstream>
-#include <iostream>
+#include <ostream>
 
 #include "base/command_line.h"
 #include "base/file_path.h"
