@@ -133,16 +133,15 @@ TEST_P(FullTabUITest, CtrlN) {
   // Watch for new window. It appears that the window close message cannot be
   // reliably delivered immediately upon receipt of the window open event.
   EXPECT_CALL(win_observer_mock, OnWindowOpen(_))
-      .Times(testing::AtMost(2))
-      .WillOnce(DelayDoCloseWindow(500))
-      .WillOnce(testing::Return());
-
-  EXPECT_CALL(win_observer_mock, OnWindowClose(_))
       .Times(testing::AtMost(2))
       .WillOnce(CloseBrowserMock(&ie_mock_))
       .WillOnce(testing::Return());
 
-  LaunchIEAndNavigate(GetSimplePageUrl());
+  EXPECT_CALL(win_observer_mock, OnWindowClose(_))
+      .Times(testing::AtMost(2));
+
+  LaunchIENavigateAndLoop(GetSimplePageUrl(),
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
 // Test that Ctrl+F opens the Find dialog.
@@ -168,20 +167,15 @@ TEST_P(FullTabUITest, CtrlF) {
           SetFocusToRenderer(&ie_mock_),
           DelaySendChar(&loop_, 1500, 'f', simulate_input::CONTROL)));
 
-  // Watch for find dialog. It appears that the window close message cannot be
-  // reliably delivered immediately upon receipt of the window open event.
   EXPECT_CALL(win_observer_mock, OnWindowOpen(_))
-      .WillOnce(DelayDoCloseWindow(500));
-
-  EXPECT_CALL(win_observer_mock, OnWindowClose(_))
       .WillOnce(CloseBrowserMock(&ie_mock_));
 
-  LaunchIEAndNavigate(GetSimplePageUrl());
+  LaunchIENavigateAndLoop(GetSimplePageUrl(),
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
 // Test that ctrl+r does cause a refresh.
-// http://code.google.com/p/chromium/issues/detail?id=84297
-TEST_P(FullTabUITest, FLAKY_CtrlR) {
+TEST_P(FullTabUITest, CtrlR) {
   if (IsWorkstationLocked()) {
     LOG(ERROR) << "This test cannot be run in a locked workstation.";
     return;
@@ -200,7 +194,8 @@ TEST_P(FullTabUITest, FLAKY_CtrlR) {
           DelayCloseBrowserMock(&loop_, 4000, &ie_mock_)))
       .WillRepeatedly(testing::Return());
 
-  LaunchIEAndNavigate(GetSimplePageUrl());
+  LaunchIENavigateAndLoop(GetSimplePageUrl(),
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
 // Test window close with ctrl+w.
@@ -216,12 +211,12 @@ TEST_P(FullTabUITest, CtrlW) {
           SetFocusToRenderer(&ie_mock_),
           DelaySendChar(&loop_, 1000, 'w', simulate_input::CONTROL)));
 
-  LaunchIEAndNavigate(GetSimplePageUrl());
+  LaunchIENavigateAndLoop(GetSimplePageUrl(),
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
 // Test address bar navigation with Alt+d and URL.
-// http://code.google.com/p/chromium/issues/detail?id=84297
-TEST_P(FullTabUITest, FLAKY_AltD) {
+TEST_P(FullTabUITest, AltD) {
   if (IsWorkstationLocked()) {
     LOG(ERROR) << "This test cannot be run in a locked workstation.";
     return;
@@ -237,11 +232,13 @@ TEST_P(FullTabUITest, FLAKY_AltD) {
                                StrEq(GetLinkPageUrl())))
       .WillOnce(CloseBrowserMock(&ie_mock_));
 
-  LaunchIEAndNavigate(GetSimplePageUrl());
+  LaunchIENavigateAndLoop(GetSimplePageUrl(),
+                          kChromeFrameVeryLongNavigationTimeoutInSeconds);
 }
 
 // Tests that the renderer has focus after navigation.
-TEST_P(FullTabUITest, RendererHasFocus) {
+// Flaky, see http://crbug.com/90791 .
+TEST_P(FullTabUITest, FLAKY_RendererHasFocus) {
   EXPECT_CALL(ie_mock_, OnLoad(GetParam().invokes_cf(),
                                StrEq(GetSimplePageUrl())))
       .WillOnce(testing::DoAll(
@@ -698,12 +695,8 @@ TEST_F(ContextMenuTest, IEBackForward) {
 }
 
 // Test CF link context menu - Open link in new window.
-TEST_F(ContextMenuTest, CFOpenLinkInNewWindow) {
-  // See crbug.com/64794.
-  if (GetInstalledIEVersion() == IE_7) {
-    LOG(INFO) << "Not running test with IE7";
-    return;
-  }
+// Failing intermittently on IE6/7. See crbug.com/64794.
+TEST_F(ContextMenuTest, FLAKY_CFOpenLinkInNewWindow) {
   server_mock_.ExpectAndServeAnyRequests(CFInvocation::MetaTag());
   MockIEEventSink new_window_mock;
   new_window_mock.ExpectAnyNavigations();
