@@ -7,8 +7,18 @@
 
 #include <string>
 
+#include "base/file_path.h"
+
+namespace printer_driver_util {
+
 void LaunchPrintDialog(const std::string& output_path,
                        const std::string& job_title,
-                       const std::string& current_user);
+                       const std::string& current_user,
+                       const std::string& print_ticket);
+void WriteToTemp(FILE* input_pdf, FilePath output_path);
+void SetUser(const char* user);
+void GetOptions(const char* options, std::string* print_ticket);
+
+}  // namespace printer_driver_util
 
 #endif  // CLOUD_PRINT_VIRTUAL_DRIVER_POSIX_PRINTER_DRIVER_UTIL_POSIX_H_
