@@ -357,6 +357,16 @@ BOOL WINAPI Monitor2StartDocPort(HANDLE port_handle,
                                  DWORD,
                                  BYTE*) {
   LOG(INFO) << "Monitor2StartDocPort";
+  const wchar_t* kUsageKey = L"dr";
+  // Set appropriate key to 1 to let Omaha record usage.
+  base::win::RegKey key;
+  if (key.Create(HKEY_CURRENT_USER, kKeyLocation,
+                 KEY_SET_VALUE) != ERROR_SUCCESS) {
+    LOG(ERROR) << "Unable to open key to log usage";
+  }
+  if (key.WriteValue(kUsageKey, L"1") != ERROR_SUCCESS) {
+    LOG(ERROR) << "Unable to set usage key";
+  }
   if (port_handle == NULL) {
     LOG(ERROR) << "port_handle should not be NULL.";
     SetLastError(ERROR_INVALID_PARAMETER);
