@@ -4,6 +4,7 @@
 
 #include "content/browser/browser_thread.h"
 
+#include "base/bind.h"
 #include "base/message_loop.h"
 #include "base/message_loop_proxy.h"
 #include "base/threading/thread_restrictions.h"
@@ -53,6 +54,30 @@ class BrowserThreadMessageLoopProxy : public base::MessageLoopProxy {
     return BrowserThread::PostNonNestableDelayedTask(id_, from_here, task,
                                                     delay_ms);
   }
+
+  virtual bool PostTask(const tracked_objects::Location& from_here,
+                        const base::Closure& task) {
+    return BrowserThread::PostTask(id_, from_here, task);
+  }
+
+  virtual bool PostDelayedTask(const tracked_objects::Location& from_here,
+                               const base::Closure& task, int64 delay_ms) {
+    return BrowserThread::PostDelayedTask(id_, from_here, task, delay_ms);
+  }
+
+  virtual bool PostNonNestableTask(const tracked_objects::Location& from_here,
+                                   const base::Closure& task) {
+    return BrowserThread::PostNonNestableTask(id_, from_here, task);
+  }
+
+  virtual bool PostNonNestableDelayedTask(
+      const tracked_objects::Location& from_here,
+      const base::Closure& task,
+      int64 delay_ms) {
+    return BrowserThread::PostNonNestableDelayedTask(id_, from_here, task,
+                                                     delay_ms);
+  }
+
   virtual bool BelongsToCurrentThread() {
     return BrowserThread::CurrentlyOn(id_);
   }
@@ -196,6 +221,17 @@ bool BrowserThread::PostNonNestableDelayedTask(
   return PostTaskHelper(identifier, from_here, task, delay_ms, false);
 }
 
+// static
+bool BrowserThread::PostTaskAndReply(
+    ID identifier,
+    const tracked_objects::Location& from_here,
+    const base::Closure& task,
+    const base::Closure& reply) {
+  return GetMessageLoopProxyForThread(identifier)->PostTaskAndReply(from_here,
+                                                                    task,
+                                                                    reply);
+}
+
 // static
 bool BrowserThread::GetCurrentThreadIdentifier(ID* identifier) {
   // We shouldn't use MessageLoop::current() since it uses LazyInstance which
