@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -7,6 +7,7 @@
 #pragma once
 
 #include "content/browser/geolocation/access_token_store.h"
+#include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 // A fake (non-persisted) access token store instance useful for testing.
@@ -17,10 +18,17 @@ class FakeAccessTokenStore : public AccessTokenStore {
   void NotifyDelegateTokensLoaded();
 
   // AccessTokenStore
-  virtual void DoLoadAccessTokens(
+  MOCK_METHOD1(DoLoadAccessTokens,
+               void(scoped_refptr<CancelableRequest
+                    <LoadAccessTokensCallbackType> > request));
+  MOCK_METHOD2(SaveAccessToken,
+               void(const GURL& server_url, const string16& access_token));
+
+  void DefaultDoLoadAccessTokens(
       scoped_refptr<CancelableRequest<LoadAccessTokensCallbackType> > request);
-  virtual void SaveAccessToken(
-      const GURL& server_url, const string16& access_token);
+
+  void DefaultSaveAccessToken(const GURL& server_url,
+                              const string16& access_token);
 
   AccessTokenSet access_token_set_;
   scoped_refptr<CancelableRequest<LoadAccessTokensCallbackType> > request_;
