@@ -4,12 +4,14 @@
 
 #include "base/json/json_reader.h"
 #include "base/memory/scoped_ptr.h"
+#include "base/string_number_conversions.h"
+#include "base/string_util.h"
 #include "base/stringprintf.h"
 #include "base/utf_string_conversions.h"
 #include "base/values.h"
 #include "content/browser/geolocation/fake_access_token_store.h"
 #include "content/browser/geolocation/network_location_provider.h"
-#include "content/common/test_url_fetcher_factory.h"
+#include "content/test/test_url_fetcher_factory.h"
 #include "net/url_request/url_request_status.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -19,6 +21,9 @@ namespace {
 const char kTestServerUrl[] = "https://www.geolocation.test/service";
 const char kTestHost[] = "myclienthost.test";
 const char kTestHostUrl[] = "http://myclienthost.test/some/path";
+const char kTestJson[] = "?browser=chromium&sensor=true";
+const char kTestBrowser[] = "browser=chromium";
+const char kTestSensor[] = "sensor=true";
 // Using #define so we can easily paste this into various other strings.
 #define REFERENCE_ACCESS_TOKEN "2:k7j3G6LaL6u_lafw:4iXOeOpTh1glSXe"
 
@@ -115,10 +120,7 @@ MockDeviceDataProviderImpl<DataType>::instance_ = NULL;
 class GeolocationNetworkProviderTest : public testing::Test {
  public:
   virtual void SetUp() {
-    URLFetcher::set_factory(&url_fetcher_factory_);
     access_token_store_ = new FakeAccessTokenStore;
-    gateway_data_provider_ =
-        MockDeviceDataProviderImpl<GatewayData>::CreateInstance();
     radio_data_provider_ =
         MockDeviceDataProviderImpl<RadioData>::CreateInstance();
     wifi_data_provider_ =
@@ -128,8 +130,6 @@ class GeolocationNetworkProviderTest : public testing::Test {
   virtual void TearDown() {
     WifiDataProvider::ResetFactory();
     RadioDataProvider::ResetFactory();
-    GatewayDataProvider::ResetFactory();
-    URLFetcher::set_factory(NULL);
   }
 
   LocationProviderBase* CreateProvider(bool set_permission_granted) {
@@ -147,8 +147,6 @@ class GeolocationNetworkProviderTest : public testing::Test {
   GeolocationNetworkProviderTest() : test_server_url_(kTestServerUrl) {
     // TODO(joth): Really these should be in SetUp, not here, but they take no
     // effect on Mac OS Release builds if done there. I kid not. Figure out why.
-    GatewayDataProvider::SetFactory(
-        MockDeviceDataProviderImpl<GatewayData>::GetInstance);
     RadioDataProvider::SetFactory(
         MockDeviceDataProviderImpl<RadioData>::GetInstance);
     WifiDataProvider::SetFactory(
@@ -165,7 +163,7 @@ class GeolocationNetworkProviderTest : public testing::Test {
     return fetcher;
   }
 
-  static int IndexToChannal(int index) { return index + 4; }
+  static int IndexToChannel(int index) { return index + 4; }
 
   // Creates wifi data containing the specified number of access points, with
   // some differentiating charactistics in each.
@@ -175,26 +173,30 @@ class GeolocationNetworkProviderTest : public testing::Test {
       AccessPointData ap;
       ap.mac_address =
           ASCIIToUTF16(base::StringPrintf("%02d-34-56-78-54-32", i));
-      ap.radio_signal_strength = i;
-      ap.channel = IndexToChannal(i);
+      ap.radio_signal_strength = ap_count - i;
+      ap.channel = IndexToChannel(i);
       ap.signal_to_noise = i + 42;
-      ap.ssid = ASCIIToUTF16("Some nice network");
+      ap.ssid = ASCIIToUTF16("Some nice+network|name\\");
       data.access_point_data.insert(ap);
     }
     return data;
   }
 
-  // Creates gateway data containing the specified number of routers, with
-  // some differentiating charactistics in each.
-  static GatewayData CreateReferenceRouterData(int router_count) {
-    GatewayData data;
-    for (int i = 0; i < router_count; ++i) {
-      RouterData router;
-      router.mac_address =
-          ASCIIToUTF16(base::StringPrintf("%02d-34-56-78-54-32", i));
-      data.router_data.insert(router);
+  static std::vector<std::string> CreateReferenceWifiScanDataJson(
+      int ap_count, int start_index) {
+    std::vector<std::string> wifi_data;
+    for (int i = 0; i < ap_count; ++i) {
+      std::string wifi_part;
+      wifi_part += "wifi=";
+      wifi_part += "mac%3A" + base::StringPrintf("%02d-34-56-78-54-32", i);
+      wifi_part += "%7Css%3A" + base::IntToString(start_index + ap_count - i);
+      wifi_part += "%7Cage%3A0";
+      wifi_part += "%7Cchan%3A" + base::IntToString(IndexToChannel(i));
+      wifi_part += "%7Csnr%3A" + base::IntToString(i + 42);
+      wifi_part += "%7Cssid%3ASome%20nice%2Bnetwork%5C%7Cname%5C%5C";
+      wifi_data.push_back(wifi_part);
     }
-    return data;
+    return wifi_data;
   }
 
   static Geoposition CreateReferencePosition(int id) {
@@ -206,141 +208,63 @@ class GeolocationNetworkProviderTest : public testing::Test {
     return pos;
   }
 
-  static void ParseGatewayRequest(const std::string& request_data,
-                                  GatewayData* gateway_data_out) {
-    scoped_ptr<Value> value(base::JSONReader::Read(request_data, false));
-    EXPECT_TRUE(value != NULL);
-    EXPECT_EQ(Value::TYPE_DICTIONARY, value->GetType());
-    DictionaryValue* dictionary = static_cast<DictionaryValue*>(value.get());
-    std::string attr_value;
-    EXPECT_TRUE(dictionary->GetString("version", &attr_value));
-    EXPECT_EQ(attr_value, "1.1.0");
-    EXPECT_TRUE(dictionary->GetString("host", &attr_value));
-    EXPECT_EQ(attr_value, kTestHost);
-    // Everything else is optional.
-    ListValue* gateways;
-    if (dictionary->GetList("gateways", &gateways)) {
-      int i = 0;
-      for (ListValue::const_iterator it = gateways->begin();
-           it < gateways->end(); ++it, ++i) {
-        EXPECT_EQ(Value::TYPE_DICTIONARY, (*it)->GetType());
-        DictionaryValue* gateway = static_cast<DictionaryValue*>(*it);
-        RouterData data;
-        gateway->GetString("mac_address", &data.mac_address);
-        gateway_data_out->router_data.insert(data);
-      }
-    } else {
-      gateway_data_out->router_data.clear();
-    }
-  }
-
-  static void ParseWifiRequest(const std::string& request_data,
-                               WifiData* wifi_data_out,
-                               int* max_age_out,
-                               std::string* access_token_out) {
-    CHECK(wifi_data_out && max_age_out && access_token_out);
-    scoped_ptr<Value> value(base::JSONReader::Read(request_data, false));
-    EXPECT_TRUE(value != NULL);
-    EXPECT_EQ(Value::TYPE_DICTIONARY, value->GetType());
-    DictionaryValue* dictionary = static_cast<DictionaryValue*>(value.get());
-    std::string attr_value;
-    EXPECT_TRUE(dictionary->GetString("version", &attr_value));
-    EXPECT_EQ(attr_value, "1.1.0");
-    EXPECT_TRUE(dictionary->GetString("host", &attr_value));
-    EXPECT_EQ(attr_value, kTestHost);
-    // Everything else is optional.
-    ListValue* wifi_aps;
-    *max_age_out = kint32min;
-    if (dictionary->GetList("wifi_towers", &wifi_aps)) {
-      int i = 0;
-      for (ListValue::const_iterator it = wifi_aps->begin();
-           it < wifi_aps->end(); ++it, ++i) {
-        EXPECT_EQ(Value::TYPE_DICTIONARY, (*it)->GetType());
-        DictionaryValue* ap = static_cast<DictionaryValue*>(*it);
-        AccessPointData data;
-        ap->GetString("mac_address", &data.mac_address);
-        ap->GetInteger("signal_strength", &data.radio_signal_strength);
-        int age = kint32min;
-        ap->GetInteger("age", &age);
-        if (age > *max_age_out)
-          *max_age_out = age;
-        ap->GetInteger("channel", &data.channel);
-        ap->GetInteger("signal_to_noise", &data.signal_to_noise);
-        ap->GetString("ssid", &data.ssid);
-        wifi_data_out->access_point_data.insert(data);
-      }
-    } else {
-      wifi_data_out->access_point_data.clear();
-    }
-    if (!dictionary->GetString("access_token", access_token_out))
-      access_token_out->clear();
-  }
-
-  static void CheckEmptyRequestIsValid(const std::string& request_data) {
-    WifiData wifi_aps;
-    std::string access_token;
-    int max_age;
-    ParseWifiRequest(request_data, &wifi_aps, &max_age, &access_token);
-    EXPECT_EQ(kint32min, max_age);
-    EXPECT_EQ(0, static_cast<int>(wifi_aps.access_point_data.size()));
-    EXPECT_TRUE(access_token.empty());
-  }
-
-  static void CheckRequestIsValid(const std::string& request_data,
+  static void CheckRequestIsValid(const std::string& request_url,
                                   int expected_routers,
                                   int expected_wifi_aps,
+                                  int wifi_start_index,
                                   const std::string& expected_access_token) {
-    WifiData wifi_aps;
-    std::string access_token;
-    int max_age;
-    ParseWifiRequest(request_data, &wifi_aps, &max_age, &access_token);
-    EXPECT_EQ(expected_wifi_aps,
-              static_cast<int>(wifi_aps.access_point_data.size()));
-    if (expected_wifi_aps > 0) {
-      EXPECT_GE(max_age, 0) << "Age must not be negative.";
-      EXPECT_LT(max_age, 10 * 1000) << "This test really shouldn't take 10s.";
-      WifiData expected_data = CreateReferenceWifiScanData(expected_wifi_aps);
-      WifiData::AccessPointDataSet::const_iterator expected =
-          expected_data.access_point_data.begin();
-      WifiData::AccessPointDataSet::const_iterator actual =
-          wifi_aps.access_point_data.begin();
-      for (int i = 0; i < expected_wifi_aps; ++i) {
-        EXPECT_EQ(expected->mac_address, actual->mac_address) << i;
-        EXPECT_EQ(expected->radio_signal_strength,
-            actual->radio_signal_strength) << i;
-        EXPECT_EQ(expected->channel, actual->channel) << i;
-        EXPECT_EQ(expected->signal_to_noise, actual->signal_to_noise) << i;
-        EXPECT_EQ(expected->ssid, actual->ssid) << i;
-        ++expected;
-        ++actual;
-      }
-    } else {
-      EXPECT_EQ(max_age, kint32min);
-    }
-    EXPECT_EQ(expected_access_token, access_token);
+    std::vector<std::string> url_tokens;
+    EXPECT_EQ(size_t(2), Tokenize(request_url, "?", &url_tokens));
+    EXPECT_EQ(kTestServerUrl, url_tokens[0]);
 
-    GatewayData gateway_data;
-    ParseGatewayRequest(request_data, &gateway_data);
-    EXPECT_EQ(expected_routers,
-              static_cast<int>(gateway_data.router_data.size()));
-    GatewayData expected_data = CreateReferenceRouterData(expected_routers);
-    GatewayData::RouterDataSet::const_iterator expected =
-        expected_data.router_data.begin();
-    GatewayData::RouterDataSet::const_iterator actual =
-        gateway_data.router_data.begin();
-    for (int i = 0; i < expected_routers; ++i) {
-      EXPECT_EQ(expected->mac_address, actual->mac_address) << i;
-      ++expected;
-      ++actual;
+    std::vector<std::string> json_tokens;
+    size_t expected_info_tokens = expected_access_token.empty() ? 2 : 3;
+    EXPECT_EQ(expected_info_tokens + expected_wifi_aps,
+        Tokenize(url_tokens[1], "&", &json_tokens));
+    EXPECT_EQ(kTestBrowser, json_tokens[0]);
+    EXPECT_EQ(kTestSensor, json_tokens[1]);
+    if (!expected_access_token.empty())
+      EXPECT_EQ("token=" + expected_access_token, json_tokens[2]);
+
+    std::vector<std::string> expected_json_tokens =
+        CreateReferenceWifiScanDataJson(expected_wifi_aps, wifi_start_index);
+    EXPECT_EQ(size_t(expected_wifi_aps), expected_json_tokens.size());
+    for (size_t i = 0; i < expected_json_tokens.size(); ++i ) {
+      std::vector<std::string> actual_wifi_tokens;
+      std::vector<std::string> expected_wifi_tokens;
+      ReplaceSubstringsAfterOffset(&json_tokens[i + expected_info_tokens],
+                                   0, "%7C", "|");
+      ReplaceSubstringsAfterOffset(&expected_json_tokens[i],
+                                   0, "%7C", "|");
+      Tokenize(json_tokens[i + expected_info_tokens],
+               "|", &actual_wifi_tokens);
+      Tokenize(expected_json_tokens[i], "|", &expected_wifi_tokens);
+
+      // MAC address.
+      EXPECT_EQ(expected_wifi_tokens[0], actual_wifi_tokens[0]);
+      // Signal Strength.
+      EXPECT_EQ(expected_wifi_tokens[1], actual_wifi_tokens[1]);
+      int age;
+      base::StringToInt(actual_wifi_tokens[2].substr(
+                            4, actual_wifi_tokens[2].size() - 4),
+                        &age);
+      // Age.
+      EXPECT_LT(age, 20 * 1000); // Should not take longer than 20 seconds.
+      // Channel.
+      EXPECT_EQ(expected_wifi_tokens[3], actual_wifi_tokens[3]);
+      // Signal to noise ratio.
+      EXPECT_EQ(expected_wifi_tokens[4], actual_wifi_tokens[4]);
+      // SSID.
+      EXPECT_EQ(expected_wifi_tokens[5], actual_wifi_tokens[5]);
+      EXPECT_EQ(expected_wifi_tokens[6], actual_wifi_tokens[6]);
     }
+    EXPECT_TRUE(GURL(request_url).is_valid());
   }
 
   const GURL test_server_url_;
   MessageLoop main_message_loop_;
   scoped_refptr<FakeAccessTokenStore> access_token_store_;
   TestURLFetcherFactory url_fetcher_factory_;
-  scoped_refptr<MockDeviceDataProviderImpl<GatewayData> >
-      gateway_data_provider_;
   scoped_refptr<MockDeviceDataProviderImpl<RadioData> > radio_data_provider_;
   scoped_refptr<MockDeviceDataProviderImpl<WifiData> > wifi_data_provider_;
 };
@@ -361,44 +285,52 @@ TEST_F(GeolocationNetworkProviderTest, StartProvider) {
   TestURLFetcher* fetcher = get_url_fetcher_and_advance_id();
   ASSERT_TRUE(fetcher != NULL);
 
-  EXPECT_EQ(test_server_url_, fetcher->original_url());
+  EXPECT_EQ(test_server_url_.spec() + kTestJson,
+            fetcher->original_url().spec());
 
-  // No wifi data so expect an empty request.
-  CheckEmptyRequestIsValid(fetcher->upload_data());
+  CheckRequestIsValid(fetcher->original_url().spec(), 0, 0, 0, "");
+}
+
+TEST_F(GeolocationNetworkProviderTest, StartProviderLongRequest) {
+  scoped_ptr<LocationProviderBase> provider(CreateProvider(true));
+  EXPECT_TRUE(provider->StartProvider(false));
+  const int kFirstScanAps = 20;
+  wifi_data_provider_->SetData(CreateReferenceWifiScanData(kFirstScanAps));
+  main_message_loop_.RunAllPending();
+  TestURLFetcher* fetcher = get_url_fetcher_and_advance_id();
+  ASSERT_TRUE(fetcher != NULL);
+  // The request url should have been shortened to less than 2048 characters
+  // in length by not including access points with the lowest signal strength
+  // in the request.
+  EXPECT_LT(fetcher->original_url().spec().size(), size_t(2048));
+  CheckRequestIsValid(fetcher->original_url().spec(), 0, 16, 4, "");
 }
 
 TEST_F(GeolocationNetworkProviderTest, MultipleStartProvider) {
   scoped_ptr<LocationProviderBase> provider_1(CreateProvider(true));
   scoped_ptr<LocationProviderBase> provider_2(CreateProvider(true));
-  ASSERT_TRUE(gateway_data_provider_);
   ASSERT_TRUE(radio_data_provider_);
   ASSERT_TRUE(wifi_data_provider_);
-  EXPECT_EQ(0, gateway_data_provider_->start_calls_);
   EXPECT_EQ(0, radio_data_provider_->start_calls_);
   EXPECT_EQ(0, wifi_data_provider_->start_calls_);
-  EXPECT_EQ(0, gateway_data_provider_->stop_calls_);
   EXPECT_EQ(0, radio_data_provider_->stop_calls_);
   EXPECT_EQ(0, wifi_data_provider_->stop_calls_);
 
   // Start first provider.
   EXPECT_TRUE(provider_1->StartProvider(false));
-  EXPECT_EQ(1, gateway_data_provider_->start_calls_);
   EXPECT_EQ(1, radio_data_provider_->start_calls_);
   EXPECT_EQ(1, wifi_data_provider_->start_calls_);
   // Start second provider.
   EXPECT_TRUE(provider_2->StartProvider(false));
-  EXPECT_EQ(1, gateway_data_provider_->start_calls_);
   EXPECT_EQ(1, radio_data_provider_->start_calls_);
   EXPECT_EQ(1, wifi_data_provider_->start_calls_);
 
   // Stop first provider.
   provider_1->StopProvider();
-  EXPECT_EQ(0, gateway_data_provider_->stop_calls_);
   EXPECT_EQ(0, radio_data_provider_->stop_calls_);
   EXPECT_EQ(0, wifi_data_provider_->stop_calls_);
   // Stop second provider.
   provider_2->StopProvider();
-  EXPECT_EQ(1, gateway_data_provider_->stop_calls_);
   EXPECT_EQ(1, radio_data_provider_->stop_calls_);
   EXPECT_EQ(1, wifi_data_provider_->stop_calls_);
 }
@@ -425,21 +357,17 @@ TEST_F(GeolocationNetworkProviderTest, MultipleWifiScansComplete) {
 
   TestURLFetcher* fetcher = get_url_fetcher_and_advance_id();
   ASSERT_TRUE(fetcher != NULL);
-  CheckEmptyRequestIsValid(fetcher->upload_data());
+  EXPECT_EQ(test_server_url_.spec() + kTestJson,
+            fetcher->original_url().spec());
   // Complete the network request with bad position fix.
   const char* kNoFixNetworkResponse =
       "{"
-      "  \"location\": null,"
-      "  \"access_token\": \"" REFERENCE_ACCESS_TOKEN "\""
+      "  \"status\": \"ZERO_RESULTS\""
       "}";
   fetcher->delegate()->OnURLFetchComplete(
       fetcher, test_server_url_, net::URLRequestStatus(), 200,  // OK
       net::ResponseCookies(), kNoFixNetworkResponse);
 
-  // This should have set the access token anyhow
-  EXPECT_EQ(UTF8ToUTF16(REFERENCE_ACCESS_TOKEN),
-            access_token_store_->access_token_set_[test_server_url_]);
-
   Geoposition position;
   provider->GetPosition(&position);
   EXPECT_FALSE(position.IsValidFix());
@@ -450,20 +378,18 @@ TEST_F(GeolocationNetworkProviderTest, MultipleWifiScansComplete) {
   main_message_loop_.RunAllPending();
   fetcher = get_url_fetcher_and_advance_id();
   ASSERT_TRUE(fetcher != NULL);
-  // The request should have access token (set previously) and the wifi data.
-  CheckRequestIsValid(fetcher->upload_data(), 0,
-                      kFirstScanAps,
-                      REFERENCE_ACCESS_TOKEN);
+  // The request should have the wifi data.
+  CheckRequestIsValid(fetcher->original_url().spec(), 0, kFirstScanAps, 0, "");
 
   // Send a reply with good position fix.
   const char* kReferenceNetworkResponse =
       "{"
+      "  \"status\": \"OK\","
+      "  \"access_token\": \"" REFERENCE_ACCESS_TOKEN "\","
+      "  \"accuracy\": 1200.4,"
       "  \"location\": {"
-      "    \"latitude\": 51.0,"
-      "    \"longitude\": -0.1,"
-      "    \"altitude\": 30.1,"
-      "    \"accuracy\": 1200.4,"
-      "    \"altitude_accuracy\": 10.6"
+      "    \"lat\": 51.0,"
+      "    \"lng\": -0.1"
       "  }"
       "}";
   fetcher->delegate()->OnURLFetchComplete(
@@ -473,13 +399,11 @@ TEST_F(GeolocationNetworkProviderTest, MultipleWifiScansComplete) {
   provider->GetPosition(&position);
   EXPECT_EQ(51.0, position.latitude);
   EXPECT_EQ(-0.1, position.longitude);
-  EXPECT_EQ(30.1, position.altitude);
   EXPECT_EQ(1200.4, position.accuracy);
-  EXPECT_EQ(10.6, position.altitude_accuracy);
   EXPECT_TRUE(position.is_valid_timestamp());
   EXPECT_TRUE(position.IsValidFix());
 
-  // Token should still be in the store.
+  // Token should be in the store.
   EXPECT_EQ(UTF8ToUTF16(REFERENCE_ACCESS_TOKEN),
             access_token_store_->access_token_set_[test_server_url_]);
 
@@ -502,6 +426,9 @@ TEST_F(GeolocationNetworkProviderTest, MultipleWifiScansComplete) {
   main_message_loop_.RunAllPending();
   fetcher = get_url_fetcher_and_advance_id();
   EXPECT_TRUE(fetcher);
+  CheckRequestIsValid(fetcher->original_url().spec(), 0,
+                      kThirdScanAps, 0,
+                      REFERENCE_ACCESS_TOKEN);
   // ...reply with a network error.
   fetcher->delegate()->OnURLFetchComplete(
       fetcher, test_server_url_,
@@ -525,157 +452,6 @@ TEST_F(GeolocationNetworkProviderTest, MultipleWifiScansComplete) {
   EXPECT_TRUE(position.IsValidFix());
 }
 
-TEST_F(GeolocationNetworkProviderTest, GatewayAndWifiScans) {
-  scoped_ptr<LocationProviderBase> provider(CreateProvider(true));
-  EXPECT_TRUE(provider->StartProvider(false));
-
-  TestURLFetcher* fetcher = get_url_fetcher_and_advance_id();
-  ASSERT_TRUE(fetcher != NULL);
-  CheckEmptyRequestIsValid(fetcher->upload_data());
-  // Complete the network request with bad position fix (using #define so we
-  // can paste this into various other strings below)
-  #define REFERENCE_ACCESS_TOKEN "2:k7j3G6LaL6u_lafw:4iXOeOpTh1glSXe"
-  const char* kNoFixNetworkResponse =
-      "{"
-      "  \"location\": null,"
-      "  \"access_token\": \"" REFERENCE_ACCESS_TOKEN "\""
-      "}";
-  fetcher->delegate()->OnURLFetchComplete(
-      fetcher, test_server_url_, net::URLRequestStatus(), 200,  // OK
-      net::ResponseCookies(), kNoFixNetworkResponse);
-
-  // This should have set the access token anyhow
-  EXPECT_EQ(UTF8ToUTF16(REFERENCE_ACCESS_TOKEN),
-            access_token_store_->access_token_set_[test_server_url_]);
-
-  Geoposition position;
-  provider->GetPosition(&position);
-  EXPECT_FALSE(position.IsValidFix());
-
-  // Now gateway data arrives -- SetData will notify listeners.
-  const int kFirstScanRouters = 1;
-  gateway_data_provider_->SetData(
-      CreateReferenceRouterData(kFirstScanRouters));
-  main_message_loop_.RunAllPending();
-  fetcher = get_url_fetcher_and_advance_id();
-  ASSERT_TRUE(fetcher != NULL);
-  // The request should have access token (set previously) and the
-  // gateway data.
-  CheckRequestIsValid(fetcher->upload_data(), kFirstScanRouters,
-                      0, REFERENCE_ACCESS_TOKEN);
-
-  // Send a reply with good position fix.
-  const char* kReferenceNetworkResponse_1 =
-      "{"
-      "  \"location\": {"
-      "    \"latitude\": 51.0,"
-      "    \"longitude\": -0.1,"
-      "    \"altitude\": 30.1,"
-      "    \"accuracy\": 1200.4,"
-      "    \"altitude_accuracy\": 10.6"
-      "  }"
-      "}";
-  fetcher->delegate()->OnURLFetchComplete(
-      fetcher, test_server_url_, net::URLRequestStatus(), 200,  // OK
-      net::ResponseCookies(), kReferenceNetworkResponse_1);
-
-  provider->GetPosition(&position);
-  EXPECT_EQ(51.0, position.latitude);
-  EXPECT_EQ(-0.1, position.longitude);
-  EXPECT_EQ(30.1, position.altitude);
-  EXPECT_EQ(1200.4, position.accuracy);
-  EXPECT_EQ(10.6, position.altitude_accuracy);
-  EXPECT_TRUE(position.is_valid_timestamp());
-  EXPECT_TRUE(position.IsValidFix());
-
-  // Token should still be in the store.
-  EXPECT_EQ(UTF8ToUTF16(REFERENCE_ACCESS_TOKEN),
-            access_token_store_->access_token_set_[test_server_url_]);
-
-  // Gateway updated again, with one more router. This is a significant change
-  // so a new request is made.
-  const int kSecondScanRouters = kFirstScanRouters + 1;
-  gateway_data_provider_->SetData(
-      CreateReferenceRouterData(kSecondScanRouters));
-  main_message_loop_.RunAllPending();
-  fetcher = get_url_fetcher_and_advance_id();
-  EXPECT_TRUE(fetcher);
-
-  CheckRequestIsValid(fetcher->upload_data(), kSecondScanRouters,
-                      0, REFERENCE_ACCESS_TOKEN);
-
-  // Send a reply with good position fix.
-  const char* kReferenceNetworkResponse_2 =
-      "{"
-      "  \"location\": {"
-      "    \"latitude\": 51.1,"
-      "    \"longitude\": -0.1,"
-      "    \"altitude\": 30.2,"
-      "    \"accuracy\": 1100.4,"
-      "    \"altitude_accuracy\": 10.6"
-      "  }"
-      "}";
-  fetcher->delegate()->OnURLFetchComplete(
-      fetcher, test_server_url_, net::URLRequestStatus(), 200,  // OK
-      net::ResponseCookies(), kReferenceNetworkResponse_2);
-
-  provider->GetPosition(&position);
-  EXPECT_EQ(51.1, position.latitude);
-  EXPECT_EQ(-0.1, position.longitude);
-  EXPECT_EQ(30.2, position.altitude);
-  EXPECT_EQ(1100.4, position.accuracy);
-  EXPECT_EQ(10.6, position.altitude_accuracy);
-  EXPECT_TRUE(position.is_valid_timestamp());
-  EXPECT_TRUE(position.IsValidFix());
-
-  // Now add new wifi scan data.
-  const int kScanAps = 4;
-  wifi_data_provider_->SetData(CreateReferenceWifiScanData(kScanAps));
-  main_message_loop_.RunAllPending();
-  fetcher = get_url_fetcher_and_advance_id();
-  EXPECT_TRUE(fetcher);
-  CheckRequestIsValid(fetcher->upload_data(), kSecondScanRouters,
-                      kScanAps, REFERENCE_ACCESS_TOKEN);
-
-  // Send a reply with good position fix.
-  const char* kReferenceNetworkResponse_3 =
-      "{"
-      "  \"location\": {"
-      "    \"latitude\": 51.3,"
-      "    \"longitude\": -0.1,"
-      "    \"altitude\": 30.2,"
-      "    \"accuracy\": 50.4,"
-      "    \"altitude_accuracy\": 10.6"
-      "  }"
-      "}";
-  fetcher->delegate()->OnURLFetchComplete(
-      fetcher, test_server_url_, net::URLRequestStatus(), 200,  // OK
-      net::ResponseCookies(), kReferenceNetworkResponse_3);
-
-  provider->GetPosition(&position);
-  EXPECT_EQ(51.3, position.latitude);
-  EXPECT_EQ(-0.1, position.longitude);
-  EXPECT_EQ(30.2, position.altitude);
-  EXPECT_EQ(50.4, position.accuracy);
-  EXPECT_EQ(10.6, position.altitude_accuracy);
-  EXPECT_TRUE(position.is_valid_timestamp());
-  EXPECT_TRUE(position.IsValidFix());
-
-  // Wifi scan returns no access points found: should be serviced from cache.
-  wifi_data_provider_->SetData(CreateReferenceWifiScanData(0));
-  main_message_loop_.RunAllPending();
-  EXPECT_FALSE(get_url_fetcher_and_advance_id());  // No new request created.
-
-  provider->GetPosition(&position);
-  EXPECT_EQ(51.1, position.latitude);
-  EXPECT_EQ(-0.1, position.longitude);
-  EXPECT_EQ(30.2, position.altitude);
-  EXPECT_EQ(1100.4, position.accuracy);
-  EXPECT_EQ(10.6, position.altitude_accuracy);
-  EXPECT_TRUE(position.is_valid_timestamp());
-  EXPECT_TRUE(position.IsValidFix());
-}
-
 TEST_F(GeolocationNetworkProviderTest, NoRequestOnStartupUntilWifiData) {
   MessageLoopQuitListener listener;
   wifi_data_provider_->set_got_data(false);
@@ -717,10 +493,8 @@ TEST_F(GeolocationNetworkProviderTest, NetworkRequestDeferredForPermission) {
   fetcher = get_url_fetcher_and_advance_id();
   ASSERT_TRUE(fetcher != NULL);
 
-  EXPECT_EQ(test_server_url_, fetcher->original_url());
-
-  // No wifi data so expect an empty request.
-  CheckEmptyRequestIsValid(fetcher->upload_data());
+  EXPECT_EQ(test_server_url_.spec() + kTestJson,
+            fetcher->original_url().spec());
 }
 
 TEST_F(GeolocationNetworkProviderTest,
@@ -744,37 +518,30 @@ TEST_F(GeolocationNetworkProviderTest,
   fetcher = get_url_fetcher_and_advance_id();
   ASSERT_TRUE(fetcher != NULL);
 
-  EXPECT_EQ(test_server_url_, fetcher->original_url());
-
-  CheckRequestIsValid(fetcher->upload_data(), 0,
-                      kScanCount, REFERENCE_ACCESS_TOKEN);
+  CheckRequestIsValid(fetcher->original_url().spec(), 0,
+                      kScanCount, 0, REFERENCE_ACCESS_TOKEN);
 }
 
 TEST_F(GeolocationNetworkProviderTest, NetworkPositionCache) {
   NetworkLocationProvider::PositionCache cache;
 
   const int kCacheSize = NetworkLocationProvider::PositionCache::kMaximumSize;
-  for (int i = 0; i < kCacheSize * 2 + 1; ++i) {
+  for (int i = 1; i < kCacheSize * 2 + 1; ++i) {
     Geoposition pos = CreateReferencePosition(i);
-    bool ret = cache.CachePosition(CreateReferenceRouterData(2),
-                                   CreateReferenceWifiScanData(i), pos);
+    bool ret = cache.CachePosition(CreateReferenceWifiScanData(i), pos);
     EXPECT_TRUE(ret)  << i;
-    const Geoposition* item = cache.FindPosition(
-        CreateReferenceRouterData(2),
-        CreateReferenceWifiScanData(i));
+    const Geoposition* item =
+        cache.FindPosition(CreateReferenceWifiScanData(i));
     ASSERT_TRUE(item) << i;
     EXPECT_EQ(pos.latitude, item->latitude)  << i;
     EXPECT_EQ(pos.longitude, item->longitude)  << i;
-    if (i < kCacheSize) {
+    if (i <= kCacheSize) {
       // Nothing should have spilled yet; check oldest item is still there.
-      EXPECT_TRUE(cache.FindPosition(CreateReferenceRouterData(2),
-                                     CreateReferenceWifiScanData(0)));
+      EXPECT_TRUE(cache.FindPosition(CreateReferenceWifiScanData(1)));
     } else {
       const int evicted = i - kCacheSize;
-      EXPECT_FALSE(cache.FindPosition(CreateReferenceRouterData(2),
-                                      CreateReferenceWifiScanData(evicted)));
-      EXPECT_TRUE(cache.FindPosition(CreateReferenceRouterData(2),
-                                     CreateReferenceWifiScanData(evicted + 1)));
+      EXPECT_FALSE(cache.FindPosition(CreateReferenceWifiScanData(evicted)));
+      EXPECT_TRUE(cache.FindPosition(CreateReferenceWifiScanData(evicted + 1)));
     }
   }
 }
