@@ -186,8 +186,8 @@ Value* GpuDataManager::GetFeatureStatus() {
     return gpu_blacklist_->GetFeatureStatus(GpuAccessAllowed(),
         browser_command_line.HasSwitch(
             switches::kDisableAcceleratedCompositing),
-        browser_command_line.HasSwitch(
-            switches::kDisableAccelerated2dCanvas),
+        !browser_command_line.HasSwitch(
+            switches::kEnableAccelerated2dCanvas),
         browser_command_line.HasSwitch(switches::kDisableExperimentalWebGL),
         browser_command_line.HasSwitch(switches::kDisableGLMultisampling));
   return NULL;
@@ -264,9 +264,6 @@ void GpuDataManager::AppendRendererCommandLine(
   if ((flags & GpuFeatureFlags::kGpuFeatureAcceleratedCompositing) &&
       !command_line->HasSwitch(switches::kDisableAcceleratedCompositing))
     command_line->AppendSwitch(switches::kDisableAcceleratedCompositing);
-  if ((flags & GpuFeatureFlags::kGpuFeatureAccelerated2dCanvas) &&
-      !command_line->HasSwitch(switches::kDisableAccelerated2dCanvas))
-    command_line->AppendSwitch(switches::kDisableAccelerated2dCanvas);
 }
 
 void GpuDataManager::SetBuiltInGpuBlacklist(GpuBlacklist* built_in_list) {
