@@ -6,8 +6,8 @@
 #include "base/file_util.h"
 #include "base/test/test_timeouts.h"
 #include "chrome/test/automation/tab_proxy.h"
+#include "chrome/test/base/ui_test_utils.h"
 #include "chrome/test/ui/ui_layout_test.h"
-#include "chrome/test/ui_test_utils.h"
 #include "content/common/content_switches.h"
 #include "net/base/net_util.h"
 
