@@ -66,6 +66,16 @@ void WebKitContext::DeleteDataModifiedSince(const base::Time& cutoff) {
   dom_storage_context_->DeleteDataModifiedSince(cutoff);
 }
 
+void WebKitContext::DeleteSessionOnlyData() {
+  if (!BrowserThread::CurrentlyOn(BrowserThread::WEBKIT)) {
+    BrowserThread::PostTask(
+        BrowserThread::WEBKIT, FROM_HERE,
+        NewRunnableMethod(this, &WebKitContext::DeleteSessionOnlyData));
+    return;
+  }
+
+  dom_storage_context_->DeleteSessionOnlyData();
+}
 
 void WebKitContext::DeleteSessionStorageNamespace(
     int64 session_storage_namespace_id) {
