@@ -24,10 +24,10 @@ class QuotaManagerProxy;
 class SpecialStoragePolicy;
 }
 
-// There's one WebKitContext per profile.  Various DispatcherHost classes
-// have a pointer to the Context to store shared state.  Unfortunately, this
-// class has become a bit of a dumping ground for calls made on the UI thread
-// that need to be proxied over to the WebKit thread.
+// There's one WebKitContext per browser context.  Various DispatcherHost
+// classes have a pointer to the Context to store shared state.  Unfortunately,
+// this class has become a bit of a dumping ground for calls made on the UI
+// thread that need to be proxied over to the WebKit thread.
 //
 // This class is created on the UI thread and accessed on the UI, IO, and WebKit
 // threads.
@@ -69,6 +69,10 @@ class WebKitContext : public base::RefCountedThreadSafe<WebKitContext> {
   // last modified on or after the following time.
   void DeleteDataModifiedSince(const base::Time& cutoff);
 
+  // Tell all children (where applicable) to delete any objects that are allowed
+  // to be stored only until the end of the session.
+  void DeleteSessionOnlyData();
+
   // Delete the session storage namespace associated with this id.  Can be
   // called from any thread.
   void DeleteSessionStorageNamespace(int64 session_storage_namespace_id);
@@ -77,7 +81,7 @@ class WebKitContext : public base::RefCountedThreadSafe<WebKitContext> {
   friend class base::RefCountedThreadSafe<WebKitContext>;
   virtual ~WebKitContext();
 
-  // Copies of profile data that can be accessed on any thread.
+  // Copies of browser context data that can be accessed on any thread.
   const FilePath data_path_;
   const bool is_incognito_;
 
