@@ -9,9 +9,10 @@
 #include "base/logging.h"
 #include "base/metrics/histogram.h"
 #include "base/string_util.h"
-#include "chrome/browser/renderer_host/download_throttling_resource_handler.h"
 #include "content/browser/browser_thread.h"
 #include "content/browser/content_browser_client.h"
+#include "content/browser/download/download_resource_handler.h"
+#include "content/browser/plugin_service.h"
 #include "content/browser/renderer_host/resource_dispatcher_host.h"
 #include "content/browser/renderer_host/resource_dispatcher_host_delegate.h"
 #include "content/browser/renderer_host/resource_dispatcher_host_request_info.h"
@@ -308,15 +309,24 @@ bool BufferedResourceHandler::CompleteResponseStarted(int request_id,
 
     info->set_is_download(true);
 
-    DownloadThrottlingResourceHandler* download_handler =
-        new DownloadThrottlingResourceHandler(host_,
-                                              request_,
-                                              request_->url(),
-                                              info->child_id(),
-                                              info->route_id(),
-                                              request_id,
-                                              in_complete);
-    UseAlternateResourceHandler(request_id, download_handler);
+    scoped_refptr<ResourceHandler> handler(
+      new DownloadResourceHandler(host_,
+                                  info->child_id(),
+                                  info->route_id(),
+                                  info->request_id(),
+                                  request_->url(),
+                                  host_->download_file_manager(),
+                                  request_,
+                                  false,
+                                  DownloadSaveInfo()));
+
+    if (host_->delegate()) {
+      handler = host_->delegate()->DownloadStarting(
+          handler, *info->context(), request_, info->child_id(),
+          info->route_id(), info->request_id(), false, in_complete);
+    }
+
+    UseAlternateResourceHandler(request_id, handler);
   }
   return real_handler_->OnResponseStarted(request_id, response_);
 }
@@ -386,21 +396,27 @@ bool BufferedResourceHandler::ShouldDownload(bool* need_plugin_list) {
   if (net::IsSupportedMimeType(type))
     return false;
 
+  // Finally, check the plugin list.
+  bool allow_wildcard = false;
+  ResourceDispatcherHostRequestInfo* info =
+      ResourceDispatcherHost::InfoForRequest(request_);
+  bool stale = false;
+  webkit::WebPluginInfo plugin;
+  bool found = PluginService::GetInstance()->GetPluginInfo(
+      info->child_id(), info->route_id(), *info->context(),
+      request_->url(), GURL(), type, allow_wildcard,
+      &stale, &plugin, NULL);
+
   if (need_plugin_list) {
-    if (webkit::npapi::PluginList::Singleton()->stale()) {
+    if (stale) {
       *need_plugin_list = true;
       return true;
     }
   } else {
-    DCHECK(!webkit::npapi::PluginList::Singleton()->stale());
+    DCHECK(!stale);
   }
 
-  // Finally, check the plugin list.
-  webkit::npapi::WebPluginInfo info;
-  bool allow_wildcard = false;
-  return !webkit::npapi::PluginList::Singleton()->GetPluginInfo(
-      GURL(), type, allow_wildcard, &info, NULL) ||
-      !webkit::npapi::IsPluginEnabled(info);
+  return !found;
 }
 
 void BufferedResourceHandler::UseAlternateResourceHandler(
@@ -435,8 +451,8 @@ void BufferedResourceHandler::UseAlternateResourceHandler(
 }
 
 void BufferedResourceHandler::LoadPlugins() {
-  std::vector<webkit::npapi::WebPluginInfo> plugins;
-  webkit::npapi::PluginList::Singleton()->GetPlugins(false, &plugins);
+  std::vector<webkit::WebPluginInfo> plugins;
+  webkit::npapi::PluginList::Singleton()->GetPlugins(&plugins);
 
   BrowserThread::PostTask(
       BrowserThread::IO, FROM_HERE,
