@@ -6,78 +6,25 @@
 #include "base/message_loop.h"
 #include "content/browser/browser_thread.h"
 #include "content/browser/mock_resource_context.h"
+#include "content/browser/renderer_host/dummy_resource_handler.h"
 #include "content/browser/renderer_host/global_request_id.h"
 #include "content/browser/renderer_host/resource_dispatcher_host_request_info.h"
-#include "content/browser/renderer_host/resource_handler.h"
 #include "content/browser/renderer_host/resource_queue.h"
 #include "googleurl/src/gurl.h"
 #include "net/url_request/url_request.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
+using content::DummyResourceHandler;
+
 namespace {
 
 const char kTestUrl[] = "data:text/plain,Hello World!";
 
-class DummyResourceHandler : public ResourceHandler {
- public:
-  DummyResourceHandler() {
-  }
-
-  bool OnUploadProgress(int request_id, uint64 position, uint64 size) {
-    NOTREACHED();
-    return true;
-  }
-
-  virtual bool OnRequestRedirected(int request_id, const GURL& url,
-                                   ResourceResponse* response,
-                                   bool* defer) {
-    NOTREACHED();
-    return true;
-  }
-
-  virtual bool OnResponseStarted(int request_id,
-                                 ResourceResponse* response) {
-    NOTREACHED();
-    return true;
-  }
-
-  virtual bool OnWillStart(int request_id, const GURL& url, bool* defer) {
-    NOTREACHED();
-    return true;
-  }
-
-  virtual bool OnWillRead(int request_id,
-                          net::IOBuffer** buf,
-                          int* buf_size,
-                          int min_size) {
-    NOTREACHED();
-    return true;
-  }
-
-  virtual bool OnReadCompleted(int request_id, int* bytes_read) {
-    NOTREACHED();
-    return true;
-  }
-
-  virtual bool OnResponseCompleted(int request_id,
-                                   const net::URLRequestStatus& status,
-                                   const std::string& security_info) {
-    NOTREACHED();
-    return true;
-  }
-
-  virtual void OnRequestClosed() {
-  }
-
- private:
-  DISALLOW_COPY_AND_ASSIGN(DummyResourceHandler);
-};
-
 ResourceDispatcherHostRequestInfo* GetRequestInfo(int request_id) {
   return new ResourceDispatcherHostRequestInfo(
       new DummyResourceHandler(), ChildProcessInfo::RENDER_PROCESS, 0, 0, 0,
-      request_id, false, -1, ResourceType::MAIN_FRAME, 0, false, false, false,
-      content::MockResourceContext::GetInstance());
+      request_id, false, -1, ResourceType::MAIN_FRAME, PageTransition::LINK, 0,
+      false, false, false, content::MockResourceContext::GetInstance());
 }
 
 void InitializeQueue(ResourceQueue* queue, ResourceQueueDelegate* delegate) {
