@@ -2,7 +2,7 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "chrome/test/testing_profile.h"
+#include "chrome/test/base/testing_profile.h"
 #include "content/browser/browser_url_handler.h"
 #include "content/browser/renderer_host/test_backing_store.h"
 #include "content/browser/renderer_host/test_render_view_host.h"
@@ -137,10 +137,14 @@ RenderWidgetHost* TestRenderWidgetHostView::GetRenderWidgetHost() const {
   return NULL;
 }
 
-gfx::NativeView TestRenderWidgetHostView::GetNativeView() {
+gfx::NativeView TestRenderWidgetHostView::GetNativeView() const {
   return NULL;
 }
 
+gfx::NativeViewId TestRenderWidgetHostView::GetNativeViewId() const {
+  return 0;
+}
+
 bool TestRenderWidgetHostView::HasFocus() {
   return true;
 }
@@ -173,23 +177,10 @@ BackingStore* TestRenderWidgetHostView::AllocBackingStore(
 
 #if defined(OS_MACOSX)
 
-void TestRenderWidgetHostView::ShowPopupWithItems(
-    gfx::Rect bounds,
-    int item_height,
-    double item_font_size,
-    int selected_item,
-    const std::vector<WebMenuItem>& items,
-    bool right_aligned) {
-}
-
 gfx::Rect TestRenderWidgetHostView::GetViewCocoaBounds() const {
   return gfx::Rect();
 }
 
-gfx::Rect TestRenderWidgetHostView::GetRootWindowRect() {
-  return gfx::Rect();
-}
-
 void TestRenderWidgetHostView::SetActive(bool active) {
   // <viettrungluu@gmail.com>: Do I need to do anything here?
 }
@@ -242,6 +233,7 @@ void TestRenderWidgetHostView::AcceleratedSurfaceBuffersSwapped(
 
 void TestRenderWidgetHostView::GpuRenderingStateDidChange() {
 }
+
 #elif defined(OS_WIN)
 void TestRenderWidgetHostView::WillWmDestroy() {
 }
@@ -250,6 +242,12 @@ void TestRenderWidgetHostView::ShowCompositorHostWindow(bool show) {
 }
 #endif
 
+#if defined(OS_POSIX)
+gfx::Rect TestRenderWidgetHostView::GetRootWindowBounds() {
+  return gfx::Rect();
+}
+#endif
+
 gfx::PluginWindowHandle TestRenderWidgetHostView::GetCompositingSurface() {
   return gfx::kNullPluginWindow;
 }
