@@ -27,11 +27,6 @@ class TabContentsView : public RenderViewHostDelegate::View {
  public:
   virtual ~TabContentsView();
 
-  // Creates the appropriate type of TabContentsView for the current system.
-  // The return value is a new heap allocated view with ownership passing to
-  // the caller.
-  static TabContentsView* Create(TabContents* tab_contents);
-
   virtual void CreateView(const gfx::Size& initial_size) = 0;
 
   // Sets up the View that holds the rendered web page, receives messages for
@@ -103,9 +98,6 @@ class TabContentsView : public RenderViewHostDelegate::View {
   // invoked, SetInitialFocus is invoked.
   virtual void RestoreFocus() = 0;
 
-  // Notification that the preferred size of the contents has changed.
-  virtual void UpdatePreferredSize(const gfx::Size& pref_size) = 0;
-
   // If we try to close the tab while a drag is in progress, we crash.  These
   // methods allow the tab contents to determine if a drag is in progress and
   // postpone the tab closing.
