@@ -17,13 +17,14 @@ class EmptyWebUIFactory : public content::WebUIFactory {
   static EmptyWebUIFactory* GetInstance();
 
   virtual WebUI* CreateWebUIForURL(TabContents* source,
-                                   const GURL& url) const;
-  virtual WebUI::TypeID GetWebUIType(Profile* profile,
-                                     const GURL& url) const;
-  virtual bool UseWebUIForURL(Profile* profile, const GURL& url) const;
-  virtual bool HasWebUIScheme(const GURL& url) const;
-  virtual bool IsURLAcceptableForWebUI(Profile* profile,
-                                       const GURL& url) const;
+                                   const GURL& url) const OVERRIDE;
+  virtual WebUI::TypeID GetWebUIType(content::BrowserContext* browser_context,
+                                     const GURL& url) const OVERRIDE;
+  virtual bool UseWebUIForURL(content::BrowserContext* browser_context,
+                              const GURL& url) const OVERRIDE;
+  virtual bool HasWebUIScheme(const GURL& url) const OVERRIDE;
+  virtual bool IsURLAcceptableForWebUI(content::BrowserContext* browser_context,
+                                       const GURL& url) const OVERRIDE;
 
  protected:
   EmptyWebUIFactory();
