@@ -17,6 +17,7 @@
 #include "base/command_line.h"
 #include "base/eintr_wrapper.h"
 #include "base/file_path.h"
+#include "base/file_util.h"
 #include "base/global_descriptors_posix.h"
 #include "base/hash_tables.h"
 #include "base/linux_util.h"
@@ -35,12 +36,13 @@
 #include "content/common/process_watcher.h"
 #include "content/common/result_codes.h"
 #include "content/common/sandbox_methods_linux.h"
+#include "content/common/seccomp_sandbox.h"
 #include "content/common/set_process_title.h"
 #include "content/common/unix_domain_socket_posix.h"
 #include "content/common/zygote_fork_delegate_linux.h"
-#include "seccompsandbox/sandbox.h"
 #include "skia/ext/SkFontHost_fontconfig_control.h"
 #include "unicode/timezone.h"
+#include "ipc/ipc_channel.h"
 #include "ipc/ipc_switches.h"
 
 #if defined(OS_LINUX)
@@ -56,21 +58,14 @@
 #include <selinux/context.h>
 #endif
 
-#if defined(ARCH_CPU_X86_FAMILY) && !defined(CHROMIUM_SELINUX) && \
-    !defined(__clang__)
-// The seccomp sandbox is enabled on all ia32 and x86-64 processor as long as
-// we aren't using SELinux or clang.
-#define SECCOMP_SANDBOX
-#endif
-
 // http://code.google.com/p/chromium/wiki/LinuxZygote
 
 static const int kBrowserDescriptor = 3;
 static const int kMagicSandboxIPCDescriptor = 5;
 static const int kZygoteIdDescriptor = 7;
 static bool g_suid_sandbox_active = false;
+
 #if defined(SECCOMP_SANDBOX)
-// |g_proc_fd| is used only by the seccomp sandbox.
 static int g_proc_fd = -1;
 #endif
 
@@ -99,9 +94,8 @@ static void SELinuxTransitionToTypeOrDie(const char* type) {
 // runs it.
 class Zygote {
  public:
-  explicit Zygote(int sandbox_flags, ZygoteForkDelegate* helper)
-    : sandbox_flags_(sandbox_flags),
-      helper_(helper) {
+  Zygote(int sandbox_flags, ZygoteForkDelegate* helper)
+      : sandbox_flags_(sandbox_flags), helper_(helper) {
   }
 
   bool ProcessRequests() {
@@ -119,7 +113,7 @@ class Zygote {
 
     if (g_suid_sandbox_active) {
       // Let the ZygoteHost know we are ready to go.
-      // The receiving code is in chrome/browser/zygote_host_linux.cc.
+      // The receiving code is in content/browser/zygote_host_linux.cc.
       std::vector<int> empty;
       bool r = UnixDomainSocket::SendMsg(kBrowserDescriptor, kZygoteMagic,
                                          sizeof(kZygoteMagic), empty);
@@ -298,15 +292,24 @@ class Zygote {
     } else if (pid == 0) {
       // In the child process.
       close(pipe_fds[1]);
-      char buffer[1];
+      base::ProcessId real_pid;
       // Wait until the parent process has discovered our PID.  We
       // should not fork any child processes (which the seccomp
       // sandbox does) until then, because that can interfere with the
       // parent's discovery of our PID.
-      if (HANDLE_EINTR(read(pipe_fds[0], buffer, 1)) != 1 ||
-          buffer[0] != 'x') {
+      if (!file_util::ReadFromFD(pipe_fds[0],
+                                 reinterpret_cast<char*>(&real_pid),
+                                 sizeof(real_pid))) {
         LOG(FATAL) << "Failed to synchronise with parent zygote process";
       }
+      if (real_pid <= 0) {
+        LOG(FATAL) << "Invalid pid from parent zygote";
+      }
+#if defined(OS_LINUX)
+      // Sandboxed processes need to send the global, non-namespaced PID when
+      // setting up an IPC channel to their parent.
+      IPC::Channel::SetGlobalPid(real_pid);
+#endif
       close(pipe_fds[0]);
       close(dummy_fd);
       return 0;
@@ -349,7 +352,9 @@ class Zygote {
           goto error;
         }
       } else {
-        if (HANDLE_EINTR(write(pipe_fds[1], "x", 1)) != 1) {
+        int written =
+            HANDLE_EINTR(write(pipe_fds[1], &real_pid, sizeof(real_pid)));
+        if (written != sizeof(real_pid)) {
           LOG(ERROR) << "Failed to synchronise with child process";
           goto error;
         }
@@ -372,58 +377,62 @@ class Zygote {
     return -1;
   }
 
-  // Handle a 'fork' request from the browser: this means that the browser
-  // wishes to start a new renderer.
-  bool HandleForkRequest(int fd, const Pickle& pickle,
-                         void* iter, std::vector<int>& fds) {
+  // Unpacks process type and arguments from |pickle| and forks a new process.
+  // Returns -1 on error, otherwise returns twice, returning 0 to the child
+  // process and the child process ID to the parent process, like fork().
+  base::ProcessId ReadArgsAndFork(const Pickle& pickle,
+                                  void* iter,
+                                  std::vector<int>& fds) {
     std::vector<std::string> args;
-    int argc, numfds;
+    int argc = 0;
+    int numfds = 0;
     base::GlobalDescriptors::Mapping mapping;
-    base::ProcessId child;
     std::string process_type;
     std::string channel_id;
     const std::string channel_id_prefix = std::string("--")
         + switches::kProcessChannelID + std::string("=");
 
     if (!pickle.ReadString(&iter, &process_type))
-      goto error;
-
+      return -1;
     if (!pickle.ReadInt(&iter, &argc))
-      goto error;
+      return -1;
 
     for (int i = 0; i < argc; ++i) {
       std::string arg;
       if (!pickle.ReadString(&iter, &arg))
-        goto error;
+        return -1;
       args.push_back(arg);
       if (arg.compare(0, channel_id_prefix.length(), channel_id_prefix) == 0)
         channel_id = arg;
     }
 
     if (!pickle.ReadInt(&iter, &numfds))
-      goto error;
+      return -1;
     if (numfds != static_cast<int>(fds.size()))
-      goto error;
+      return -1;
 
     for (int i = 0; i < numfds; ++i) {
       base::GlobalDescriptors::Key key;
       if (!pickle.ReadUInt32(&iter, &key))
-        goto error;
+        return -1;
       mapping.push_back(std::make_pair(key, fds[i]));
     }
 
     mapping.push_back(std::make_pair(
         static_cast<uint32_t>(kSandboxIPCChannel), kMagicSandboxIPCDescriptor));
 
-    child = ForkWithRealPid(process_type, fds, channel_id);
-
-    if (!child) {
+    // Returns twice, once per process.
+    base::ProcessId child_pid = ForkWithRealPid(process_type, fds, channel_id);
+    if (!child_pid) {
+      // This is the child process.
 #if defined(SECCOMP_SANDBOX)
-      // Try to open /proc/self/maps as the seccomp sandbox needs access to it
-      if (g_proc_fd >= 0) {
+      if (SeccompSandboxEnabled() && g_proc_fd >= 0) {
+        // Try to open /proc/self/maps as the seccomp sandbox needs access to it
         int proc_self_maps = openat(g_proc_fd, "self/maps", O_RDONLY);
         if (proc_self_maps >= 0) {
           SeccompSandboxSetProcSelfMaps(proc_self_maps);
+        } else {
+          PLOG(ERROR) << "openat(/proc/self/maps)";
         }
         close(g_proc_fd);
         g_proc_fd = -1;
@@ -448,28 +457,29 @@ class Zygote {
       // SetProcessTitleFromCommandLine in ChromeMain, so we can pass NULL here
       // (we don't have the original argv at this point).
       SetProcessTitleFromCommandLine(NULL);
-
-      // The fork() request is handled further up the call stack.
-      return true;
-    } else if (child < 0) {
-      LOG(ERROR) << "Zygote could not fork: " << errno;
-      goto error;
+    } else if (child_pid < 0) {
+      LOG(ERROR) << "Zygote could not fork: process_type " << process_type
+          << " numfds " << numfds << " child_pid " << child_pid;
     }
+    return child_pid;
+  }
 
+  // Handle a 'fork' request from the browser: this means that the browser
+  // wishes to start a new renderer.  Returns true if we are in a new process,
+  // otherwise writes the child_pid back to the browser via |fd|.  Writes a
+  // child_pid of -1 on error.
+  bool HandleForkRequest(int fd, const Pickle& pickle,
+                         void* iter, std::vector<int>& fds) {
+    base::ProcessId child_pid = ReadArgsAndFork(pickle, iter, fds);
+    if (child_pid == 0)
+      return true;
     for (std::vector<int>::const_iterator
          i = fds.begin(); i != fds.end(); ++i)
       close(*i);
-
-    if (HANDLE_EINTR(write(fd, &child, sizeof(child))) < 0)
+    // Must always send reply, as ZygoteHost blocks while waiting for it.
+    if (HANDLE_EINTR(write(fd, &child_pid, sizeof(child_pid))) < 0)
       PLOG(ERROR) << "write";
     return false;
-
-   error:
-    LOG(ERROR) << "Error parsing fork request from browser";
-    for (std::vector<int>::const_iterator
-         i = fds.begin(); i != fds.end(); ++i)
-      close(*i);
-    return false;
   }
 
   bool HandleGetSandboxStatus(int fd, const Pickle& pickle, void* iter) {
@@ -726,11 +736,12 @@ static bool EnterSandbox() {
         return false;
       }
     }
-  } else if (CommandLine::ForCurrentProcess()->HasSwitch(
-        switches::kEnableSeccompSandbox)) {
+#if defined(SECCOMP_SANDBOX)
+  } else if (SeccompSandboxEnabled()) {
     PreSandboxInit();
     SkiaFontConfigSetImplementation(
         new FontConfigIPC(kMagicSandboxIPCDescriptor));
+#endif
   } else {
     SkiaFontConfigUseDirectImplementation();
   }
@@ -754,15 +765,14 @@ bool ZygoteMain(const MainFunctionParams& params,
 #endif
 
 #if defined(SECCOMP_SANDBOX)
-  // The seccomp sandbox needs access to files in /proc, which might be denied
-  // after one of the other sandboxes have been started. So, obtain a suitable
-  // file handle in advance.
-  if (CommandLine::ForCurrentProcess()->HasSwitch(
-          switches::kEnableSeccompSandbox)) {
+  if (SeccompSandboxEnabled()) {
+    // The seccomp sandbox needs access to files in /proc, which might be denied
+    // after one of the other sandboxes have been started. So, obtain a suitable
+    // file handle in advance.
     g_proc_fd = open("/proc", O_DIRECTORY | O_RDONLY);
     if (g_proc_fd < 0) {
       LOG(ERROR) << "WARNING! Cannot access \"/proc\". Disabling seccomp "
-                    "sandboxing.";
+          "sandboxing.";
     }
   }
 #endif  // SECCOMP_SANDBOX
@@ -795,8 +805,7 @@ bool ZygoteMain(const MainFunctionParams& params,
   // The seccomp sandbox will be turned on when the renderers start. But we can
   // already check if sufficient support is available so that we only need to
   // print one error message for the entire browser session.
-  if (g_proc_fd >= 0 && CommandLine::ForCurrentProcess()->HasSwitch(
-          switches::kEnableSeccompSandbox)) {
+  if (g_proc_fd >= 0 && SeccompSandboxEnabled()) {
     if (!SupportsSeccompSandbox(g_proc_fd)) {
       // There are a good number of users who cannot use the seccomp sandbox
       // (e.g. because their distribution does not enable seccomp mode by
