@@ -30,6 +30,15 @@ extern const size_t kMaxTitleChars;
 extern const size_t kMaxURLChars;
 extern const size_t kMaxURLDisplayChars;
 
+// The render view and render process id associated with the default plugin
+// instance.
+extern const char kDefaultPluginRenderViewId[];
+extern const char kDefaultPluginRenderProcessId[];
+
+extern const char kStatsFilename[];
+extern const int kStatsMaxThreads;
+extern const int kStatsMaxCounters;
+
 }  // namespace content
 
 #endif  // CONTENT_COMMON_CHROME_CONSTANTS_H_
