@@ -32,9 +32,6 @@ const char kChromeFrame[]                   = "chrome-frame";
 // support for controlling access to these APIs.
 const char kDisable3DAPIs[]                 = "disable-3d-apis";
 
-// Disable gpu-accelerated 2d canvas.
-const char kDisableAccelerated2dCanvas[]    = "disable-accelerated-2d-canvas";
-
 // Disables accelerated compositing.
 const char kDisableAcceleratedCompositing[] = "disable-accelerated-compositing";
 
