@@ -116,64 +116,3 @@ WebKit::WebFileWriter* WebFileSystemImpl::createFileWriter(
     const WebURL& path, WebKit::WebFileWriterClient* client) {
   return new WebFileWriterImpl(GURL(path), client);
 }
-
-// These are temporary shims to link up the old calls to the new implementation.
-// They'll go away as soon as the webkit side gets updated.
-void WebFileSystemImpl::move(const WebString& src_path,
-                             const WebString& dest_path,
-                             WebFileSystemCallbacks* callbacks) {
-  move(GURL(src_path), GURL(dest_path), callbacks);
-}
-
-void WebFileSystemImpl::copy(const WebString& src_path,
-                             const WebString& dest_path,
-                             WebFileSystemCallbacks* callbacks) {
-  copy(GURL(src_path), GURL(dest_path), callbacks);
-}
-
-void WebFileSystemImpl::remove(const WebString& path,
-                               WebFileSystemCallbacks* callbacks) {
-  remove(GURL(path), callbacks);
-}
-
-void WebFileSystemImpl::removeRecursively(const WebString& path,
-                                          WebFileSystemCallbacks* callbacks) {
-  removeRecursively(GURL(path), callbacks);
-}
-
-void WebFileSystemImpl::readMetadata(const WebString& path,
-                                     WebFileSystemCallbacks* callbacks) {
-  readMetadata(GURL(path), callbacks);
-}
-
-void WebFileSystemImpl::createFile(const WebString& path,
-                                   bool exclusive,
-                                   WebFileSystemCallbacks* callbacks) {
-  createFile(GURL(path), exclusive, callbacks);
-}
-
-void WebFileSystemImpl::createDirectory(const WebString& path,
-                                        bool exclusive,
-                                        WebFileSystemCallbacks* callbacks) {
-  createDirectory(GURL(path), exclusive, callbacks);
-}
-
-void WebFileSystemImpl::fileExists(const WebString& path,
-                                   WebFileSystemCallbacks* callbacks) {
-  fileExists(GURL(path), callbacks);
-}
-
-void WebFileSystemImpl::directoryExists(const WebString& path,
-                                        WebFileSystemCallbacks* callbacks) {
-  directoryExists(GURL(path), callbacks);
-}
-
-void WebFileSystemImpl::readDirectory(const WebString& path,
-                                      WebFileSystemCallbacks* callbacks) {
-  readDirectory(GURL(path), callbacks);
-}
-
-WebKit::WebFileWriter* WebFileSystemImpl::createFileWriter(
-    const WebString& path, WebKit::WebFileWriterClient* client) {
-  return createFileWriter(GURL(path), client);
-}
