@@ -16,9 +16,7 @@
 #include "base/memory/scoped_ptr.h"
 #include "base/process.h"
 #include "build/build_config.h"
-#include "gpu/command_buffer/service/surface_manager.h"
 #include "content/common/gpu/gpu_command_buffer_stub.h"
-#include "content/common/gpu/gpu_surface_stub.h"
 #include "content/common/message_router.h"
 #include "ipc/ipc_sync_channel.h"
 #include "ui/gfx/gl/gl_share_group.h"
@@ -43,7 +41,6 @@ class GLSurface;
 // process. On the renderer side there's a corresponding GpuChannelHost.
 class GpuChannel : public IPC::Channel::Listener,
                    public IPC::Message::Sender,
-                   public gpu::SurfaceManager,
                    public base::RefCountedThreadSafe<GpuChannel> {
  public:
   // Takes ownership of the renderer process handle.
@@ -73,9 +70,9 @@ class GpuChannel : public IPC::Channel::Listener,
   }
 
   // IPC::Channel::Listener implementation:
-  virtual bool OnMessageReceived(const IPC::Message& msg);
-  virtual void OnChannelError();
-  virtual void OnChannelConnected(int32 peer_pid);
+  virtual bool OnMessageReceived(const IPC::Message& msg) OVERRIDE;
+  virtual void OnChannelError() OVERRIDE;
+  virtual void OnChannelConnected(int32 peer_pid) OVERRIDE;
 
   // IPC::Message::Sender implementation:
   virtual bool Send(IPC::Message* msg);
@@ -112,9 +109,6 @@ class GpuChannel : public IPC::Channel::Listener,
   // Destroy channel and all contained contexts.
   void DestroySoon();
 
-  // Look up a GLSurface by ID. In this case the ID is the IPC routing ID.
-  virtual gfx::GLSurface* LookupSurface(int surface_id);
-
   // Get the TransportTexture by ID.
   TransportTexture* GetTransportTexture(int32 route_id);
 
@@ -122,6 +116,13 @@ class GpuChannel : public IPC::Channel::Listener,
   // TransportTexture to delete and detach itself.
   void DestroyTransportTexture(int32 route_id);
 
+  // Generate a route ID guaranteed to be unique for this channel.
+  int GenerateRouteID();
+
+  // Called to add/remove a listener for a particular message routing ID.
+  void AddRoute(int32 route_id, IPC::Channel::Listener* listener);
+  void RemoveRoute(int32 route_id);
+
  private:
   void OnDestroy();
 
@@ -129,8 +130,6 @@ class GpuChannel : public IPC::Channel::Listener,
 
   void HandleDeferredMessages();
 
-  int GenerateRouteID();
-
   // Message handlers.
   void OnInitialize(base::ProcessHandle renderer_process);
   void OnCreateOffscreenCommandBuffer(
@@ -139,12 +138,10 @@ class GpuChannel : public IPC::Channel::Listener,
       IPC::Message* reply_message);
   void OnDestroyCommandBuffer(int32 route_id, IPC::Message* reply_message);
 
-  void OnCreateOffscreenSurface(const gfx::Size& size,
-                                IPC::Message* reply_message);
-  void OnDestroySurface(int route_id);
-
   void OnCreateTransportTexture(int32 context_route_id, int32 host_id);
 
+  void OnEcho(const IPC::Message& message);
+
   // The lifetime of objects of this class is managed by a GpuChannelManager.
   // The GpuChannelManager destroy all the GpuChannels that they own when they
   // are destroyed. So a raw pointer is safe.
@@ -173,9 +170,6 @@ class GpuChannel : public IPC::Channel::Listener,
 #if defined(ENABLE_GPU)
   typedef IDMap<GpuCommandBufferStub, IDMapOwnPointer> StubMap;
   StubMap stubs_;
-
-  typedef IDMap<GpuSurfaceStub, IDMapOwnPointer> SurfaceMap;
-  SurfaceMap surfaces_;
 #endif  // defined (ENABLE_GPU)
 
   // A collection of transport textures created.
