@@ -58,8 +58,9 @@ namespace {
 // - |minFPSwithRender| and |minFPSnoRender| are minimum frames/second speeds
 //   expected to be achieved with and without rendering to the screen, resp.
 //   (the latter tests just decode speed).
+// - |profile| is the media::H264Profile set during Initialization.
 // An empty value for a numeric field means "ignore".
-const char* test_video_data = "test-25fps.h264:320:240:250:258:50:175";
+const char* test_video_data = "test-25fps.h264:320:240:250:258:50:175:1";
 
 // Parse |data| into its constituent parts and set the various output fields
 // accordingly.  CHECK-fails on unexpected or missing required data.
@@ -70,14 +71,16 @@ void ParseTestVideoData(std::string data,
                         int* num_frames,
                         int* num_NALUs,
                         int* min_fps_render,
-                        int* min_fps_no_render) {
+                        int* min_fps_no_render,
+                        int* profile) {
   std::vector<std::string> elements;
   base::SplitString(data, ':', &elements);
   CHECK_GE(elements.size(), 1U) << data;
-  CHECK_LE(elements.size(), 7U) << data;
+  CHECK_LE(elements.size(), 8U) << data;
   *file_name = elements[0];
   *width = *height = *num_frames = *num_NALUs = -1;
   *min_fps_render = *min_fps_no_render = -1;
+  *profile = -1;
   if (!elements[1].empty())
     CHECK(base::StringToInt(elements[1], width));
   if (!elements[2].empty())
@@ -90,6 +93,8 @@ void ParseTestVideoData(std::string data,
     CHECK(base::StringToInt(elements[5], min_fps_render));
   if (!elements[6].empty())
     CHECK(base::StringToInt(elements[6], min_fps_no_render));
+  if (!elements[7].empty())
+    CHECK(base::StringToInt(elements[7], profile));
 }
 
 
@@ -367,8 +372,8 @@ void RenderingHelper::RenderTexture(GLuint texture_id) {
   glActiveTexture(GL_TEXTURE0);
   glBindTexture(GL_TEXTURE_2D, texture_id);
   glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);
-  DCHECK_EQ(static_cast<int>(glGetError()), GL_NO_ERROR);
-  DCHECK_EQ(static_cast<int>(eglGetError()), EGL_SUCCESS);
+  CHECK_EQ(static_cast<int>(glGetError()), GL_NO_ERROR);
+  CHECK_EQ(static_cast<int>(eglGetError()), EGL_SUCCESS);
   if (!suppress_swap_to_display_) {
     int window_id = texture_id_to_surface_index_[texture_id];
     CHECK(eglMakeCurrent(egl_display_, egl_surfaces_[window_id],
@@ -376,12 +381,12 @@ void RenderingHelper::RenderTexture(GLuint texture_id) {
         << eglGetError();
     eglSwapBuffers(egl_display_, egl_surfaces_[window_id]);
   }
-  DCHECK_EQ(static_cast<int>(eglGetError()), EGL_SUCCESS);
+  CHECK_EQ(static_cast<int>(eglGetError()), EGL_SUCCESS);
 }
 
 void RenderingHelper::DeleteTexture(GLuint texture_id) {
   glDeleteTextures(1, &texture_id);
-  DCHECK_EQ(static_cast<int>(glGetError()), GL_NO_ERROR);
+  CHECK_EQ(static_cast<int>(glGetError()), GL_NO_ERROR);
 }
 
 // State of the EglRenderingVDAClient below.  Order matters here as the test
@@ -390,8 +395,10 @@ enum ClientState {
   CS_CREATED,
   CS_DECODER_SET,
   CS_INITIALIZED,
+  CS_FLUSHING,
   CS_FLUSHED,
   CS_DONE,
+  CS_RESETTING,
   CS_RESET,
   CS_ERROR,
   CS_DESTROYED,
@@ -436,18 +443,34 @@ ClientState ClientStateNotification::Wait() {
   return ret;
 }
 
+// Magic constants for differentiating the reasons for NotifyResetDone being
+// called.
+enum ResetPoint {
+  MID_STREAM_RESET = -2,
+  END_OF_STREAM_RESET = -1
+};
+
 // Client that can accept callbacks from a VideoDecodeAccelerator and is used by
 // the TESTs below.
 class EglRenderingVDAClient : public VideoDecodeAccelerator::Client {
  public:
-  // Doesn't take ownership of |note| or |encoded_data|, which must outlive
+  // Doesn't take ownership of |rendering_helper| or |note|, which must outlive
   // |*this|.
+  // |reset_after_frame_num| can be a frame number >=0 indicating a mid-stream
+  // Reset() should be done after that frame number is delivered, or
+  // END_OF_STREAM_RESET to indicate no mid-stream Reset().
+  // |delete_decoder_state| indicates when the underlying decoder should be
+  // Destroy()'d and deleted and can take values: N<0: delete after -N Decode()
+  // calls have been made, N>=0 means interpret as ClientState.
   EglRenderingVDAClient(RenderingHelper* rendering_helper,
                         int rendering_window_id,
                         ClientStateNotification* note,
-                        std::string* encoded_data,
+                        const std::string& encoded_data,
                         int num_NALUs_per_decode,
-                        int delete_decoder_state);
+                        int num_in_flight_decodes,
+                        int reset_after_frame_num,
+                        int delete_decoder_state,
+                        int profile);
   virtual ~EglRenderingVDAClient();
   void CreateDecoder();
 
@@ -492,12 +515,16 @@ class EglRenderingVDAClient : public VideoDecodeAccelerator::Client {
 
   RenderingHelper* rendering_helper_;
   int rendering_window_id_;
-  const std::string* encoded_data_;
+  std::string encoded_data_;
   const int num_NALUs_per_decode_;
+  const int num_in_flight_decodes_;
+  int outstanding_decodes_;
   size_t encoded_data_next_pos_to_decode_;
   int next_bitstream_buffer_id_;
   ClientStateNotification* note_;
   scoped_refptr<OmxVideoDecodeAccelerator> decoder_;
+  std::set<int> outstanding_texture_ids_;
+  int reset_after_frame_num_;
   int delete_decoder_state_;
   ClientState state_;
   int num_decoded_frames_;
@@ -505,22 +532,31 @@ class EglRenderingVDAClient : public VideoDecodeAccelerator::Client {
   PictureBufferById picture_buffers_by_id_;
   base::TimeTicks initialize_done_ticks_;
   base::TimeTicks last_frame_delivered_ticks_;
+  int profile_;
 };
 
-EglRenderingVDAClient::EglRenderingVDAClient(RenderingHelper* rendering_helper,
-                                             int rendering_window_id,
-                                             ClientStateNotification* note,
-                                             std::string* encoded_data,
-                                             int num_NALUs_per_decode,
-                                             int delete_decoder_state)
+EglRenderingVDAClient::EglRenderingVDAClient(
+    RenderingHelper* rendering_helper,
+    int rendering_window_id,
+    ClientStateNotification* note,
+    const std::string& encoded_data,
+    int num_NALUs_per_decode,
+    int num_in_flight_decodes,
+    int reset_after_frame_num,
+    int delete_decoder_state,
+    int profile)
     : rendering_helper_(rendering_helper),
       rendering_window_id_(rendering_window_id),
       encoded_data_(encoded_data), num_NALUs_per_decode_(num_NALUs_per_decode),
+      num_in_flight_decodes_(num_in_flight_decodes), outstanding_decodes_(0),
       encoded_data_next_pos_to_decode_(0), next_bitstream_buffer_id_(0),
-      note_(note), delete_decoder_state_(delete_decoder_state),
+      note_(note), reset_after_frame_num_(reset_after_frame_num),
+      delete_decoder_state_(delete_decoder_state),
       state_(CS_CREATED),
-      num_decoded_frames_(0), num_done_bitstream_buffers_(0) {
+      num_decoded_frames_(0), num_done_bitstream_buffers_(0),
+      profile_(profile) {
   CHECK_GT(num_NALUs_per_decode, 0);
+  CHECK_GT(num_in_flight_decodes, 0);
 }
 
 EglRenderingVDAClient::~EglRenderingVDAClient() {
@@ -539,14 +575,11 @@ void EglRenderingVDAClient::CreateDecoder() {
     return;
 
   // Configure the decoder.
-  int32 config_array[] = {
-    media::VIDEOATTRIBUTEKEY_BITSTREAMFORMAT_FOURCC,
-    media::VIDEOCODECFOURCC_H264,
-    media::VIDEOATTRIBUTEKEY_VIDEOCOLORFORMAT, media::VIDEOCOLORFORMAT_RGBA,
-  };
-  std::vector<uint32> config(
-      config_array, config_array + arraysize(config_array));
-  CHECK(decoder_->Initialize(config));
+  media::VideoDecodeAccelerator::Profile profile =
+      media::VideoDecodeAccelerator::H264PROFILE_BASELINE;
+  if (profile_ != -1)
+    profile = static_cast<media::VideoDecodeAccelerator::Profile>(profile_);
+  CHECK(decoder_->Initialize(profile));
 }
 
 void EglRenderingVDAClient::ProvidePictureBuffers(
@@ -562,6 +595,7 @@ void EglRenderingVDAClient::ProvidePictureBuffers(
     base::WaitableEvent done(false, false);
     rendering_helper_->CreateTexture(rendering_window_id_, &texture_id, &done);
     done.Wait();
+    CHECK(outstanding_texture_ids_.insert(texture_id).second);
     media::PictureBuffer* buffer =
         new media::PictureBuffer(id, dimensions, texture_id);
     CHECK(picture_buffers_by_id_.insert(std::make_pair(id, buffer)).second);
@@ -575,7 +609,8 @@ void EglRenderingVDAClient::ProvidePictureBuffers(
 void EglRenderingVDAClient::DismissPictureBuffer(int32 picture_buffer_id) {
   PictureBufferById::iterator it =
       picture_buffers_by_id_.find(picture_buffer_id);
-  DCHECK(it != picture_buffers_by_id_.end());
+  CHECK(it != picture_buffers_by_id_.end());
+  CHECK_EQ(outstanding_texture_ids_.erase(it->second->texture_id()), 1U);
   rendering_helper_->DeleteTexture(it->second->texture_id());
   delete it->second;
   picture_buffers_by_id_.erase(it);
@@ -583,7 +618,7 @@ void EglRenderingVDAClient::DismissPictureBuffer(int32 picture_buffer_id) {
 
 void EglRenderingVDAClient::PictureReady(const media::Picture& picture) {
   // We shouldn't be getting pictures delivered after Reset has completed.
-  DCHECK_LT(state_, CS_RESET);
+  CHECK_LT(state_, CS_RESET);
 
   if (decoder_deleted())
     return;
@@ -597,6 +632,14 @@ void EglRenderingVDAClient::PictureReady(const media::Picture& picture) {
   CHECK_LE(picture.bitstream_buffer_id(), next_bitstream_buffer_id_);
   ++num_decoded_frames_;
 
+  if (reset_after_frame_num_ == num_decoded_frames_) {
+    reset_after_frame_num_ = MID_STREAM_RESET;
+    decoder_->Reset();
+    // Re-start decoding from the beginning of the stream to avoid needing to
+    // know how to find I-frames and so on in this test.
+    encoded_data_next_pos_to_decode_ = 0;
+  }
+
   media::PictureBuffer* picture_buffer =
       picture_buffers_by_id_[picture.picture_buffer_id()];
   CHECK(picture_buffer);
@@ -608,7 +651,8 @@ void EglRenderingVDAClient::PictureReady(const media::Picture& picture) {
 void EglRenderingVDAClient::NotifyInitializeDone() {
   SetState(CS_INITIALIZED);
   initialize_done_ticks_ = base::TimeTicks::Now();
-  DecodeNextNALUs();
+  for (int i = 0; i < num_in_flight_decodes_; ++i)
+    DecodeNextNALUs();
 }
 
 void EglRenderingVDAClient::NotifyEndOfStream() {
@@ -618,6 +662,7 @@ void EglRenderingVDAClient::NotifyEndOfStream() {
 void EglRenderingVDAClient::NotifyEndOfBitstreamBuffer(
     int32 bitstream_buffer_id) {
   ++num_done_bitstream_buffers_;
+  --outstanding_decodes_;
   DecodeNextNALUs();
 }
 
@@ -628,11 +673,16 @@ void EglRenderingVDAClient::NotifyFlushDone() {
   if (decoder_deleted())
     return;
   decoder_->Reset();
+  SetState(CS_RESETTING);
 }
 
 void EglRenderingVDAClient::NotifyResetDone() {
   if (decoder_deleted())
     return;
+  if (reset_after_frame_num_ == MID_STREAM_RESET) {
+    reset_after_frame_num_ = END_OF_STREAM_RESET;
+    return;
+  }
   SetState(CS_RESET);
   if (!decoder_deleted())
     DeleteDecoder();
@@ -662,6 +712,12 @@ void EglRenderingVDAClient::DeleteDecoder() {
     return;
   decoder_->Destroy();
   decoder_ = NULL;
+  STLClearObject(&encoded_data_);
+  for (std::set<int>::iterator it = outstanding_texture_ids_.begin();
+       it != outstanding_texture_ids_.end(); ++it) {
+    rendering_helper_->DeleteTexture(*it);
+  }
+  outstanding_texture_ids_.clear();
   // Cascade through the rest of the states to simplify test code below.
   for (int i = state_ + 1; i < CS_MAX; ++i)
     SetState(static_cast<ClientState>(i));
@@ -670,15 +726,15 @@ void EglRenderingVDAClient::DeleteDecoder() {
 void EglRenderingVDAClient::GetRangeForNextNALUs(
     size_t start_pos, size_t* end_pos) {
   *end_pos = start_pos;
-  CHECK(LookingAtNAL(*encoded_data_, start_pos));
+  CHECK(LookingAtNAL(encoded_data_, start_pos));
   for (int i = 0; i < num_NALUs_per_decode_; ++i) {
     *end_pos += 4;
-    while (*end_pos + 3 < encoded_data_->size() &&
-           !LookingAtNAL(*encoded_data_, *end_pos)) {
+    while (*end_pos + 3 < encoded_data_.size() &&
+           !LookingAtNAL(encoded_data_, *end_pos)) {
       ++*end_pos;
     }
-    if (*end_pos + 3 >= encoded_data_->size()) {
-      *end_pos = encoded_data_->size();
+    if (*end_pos + 3 >= encoded_data_.size()) {
+      *end_pos = encoded_data_.size();
       return;
     }
   }
@@ -687,8 +743,11 @@ void EglRenderingVDAClient::GetRangeForNextNALUs(
 void EglRenderingVDAClient::DecodeNextNALUs() {
   if (decoder_deleted())
     return;
-  if (encoded_data_next_pos_to_decode_ == encoded_data_->size()) {
-    decoder_->Flush();
+  if (encoded_data_next_pos_to_decode_ == encoded_data_.size()) {
+    if (outstanding_decodes_ == 0) {
+      decoder_->Flush();
+      SetState(CS_FLUSHING);
+    }
     return;
   }
   size_t start_pos = encoded_data_next_pos_to_decode_;
@@ -700,12 +759,13 @@ void EglRenderingVDAClient::DecodeNextNALUs() {
   base::SharedMemory shm;
   CHECK(shm.CreateAndMapAnonymous(end_pos - start_pos))
       << start_pos << ", " << end_pos;
-  memcpy(shm.memory(), encoded_data_->data() + start_pos, end_pos - start_pos);
+  memcpy(shm.memory(), encoded_data_.data() + start_pos, end_pos - start_pos);
   base::SharedMemoryHandle dup_handle;
   CHECK(shm.ShareToProcess(base::Process::Current().handle(), &dup_handle));
   media::BitstreamBuffer bitstream_buffer(
       next_bitstream_buffer_id_++, dup_handle, end_pos - start_pos);
   decoder_->Decode(bitstream_buffer);
+  ++outstanding_decodes_;
   encoded_data_next_pos_to_decode_ = end_pos;
 
   if (-delete_decoder_state_ == next_bitstream_buffer_id_)
@@ -722,10 +782,12 @@ double EglRenderingVDAClient::frames_per_second() {
 // Test parameters:
 // - Number of NALUs per Decode() call.
 // - Number of concurrent decoders.
-// - When to delete the decoder.  N<0 means after -N Decode() calls have been
-//   made, N>=0 means interpret as ClientState.
+// - Number of concurrent in-flight Decode() calls per decoder.
+// - reset_after_frame_num: see EglRenderingVDAClient ctor.
+// - delete_decoder_phase: see EglRenderingVDAClient ctor.
 class OmxVideoDecodeAcceleratorTest
-    : public ::testing::TestWithParam<Tuple3<int, int, ClientState> > {
+    : public ::testing::TestWithParam<
+  Tuple5<int, int, int, ResetPoint, ClientState> > {
 };
 
 // Wait for |note| to report a state and if it's not |expected_state| then
@@ -742,7 +804,7 @@ static void AssertWaitForStateOrDeleted(ClientStateNotification* note,
 
 // We assert the exact number of concurrent decoders we expect to succeed and
 // that one more than that fails initialization.
-enum { kMaxSupportedNumConcurrentDecoders = 5 };
+enum { kMaxSupportedNumConcurrentDecoders = 3 };
 
 // Test the most straightforward case possible: data is decoded from a single
 // chunk and rendered to the screen.
@@ -755,17 +817,24 @@ TEST_P(OmxVideoDecodeAcceleratorTest, TestSimpleDecode) {
 
   const int num_NALUs_per_decode = GetParam().a;
   const size_t num_concurrent_decoders = GetParam().b;
-  const int delete_decoder_state = GetParam().c;
+  const size_t num_in_flight_decodes = GetParam().c;
+  const int reset_after_frame_num = GetParam().d;
+  const int delete_decoder_state = GetParam().e;
 
   std::string test_video_file;
   int frame_width, frame_height;
-  int num_frames, num_NALUs, min_fps_render, min_fps_no_render;
+  int num_frames, num_NALUs, min_fps_render, min_fps_no_render, profile;
   ParseTestVideoData(test_video_data, &test_video_file, &frame_width,
                      &frame_height, &num_frames, &num_NALUs,
-                     &min_fps_render, &min_fps_no_render);
+                     &min_fps_render, &min_fps_no_render, &profile);
   min_fps_render /= num_concurrent_decoders;
   min_fps_no_render /= num_concurrent_decoders;
 
+  // If we reset mid-stream and start playback over, account for frames that are
+  // decoded twice in our expectations.
+  if (num_frames > 0 && reset_after_frame_num >= 0)
+    num_frames += reset_after_frame_num;
+
   // Suppress EGL surface swapping in all but a few tests, to cut down overall
   // test runtime.
   const bool suppress_swap_to_display = num_NALUs_per_decode > 1;
@@ -797,8 +866,9 @@ TEST_P(OmxVideoDecodeAcceleratorTest, TestSimpleDecode) {
     notes[index] = note;
     EglRenderingVDAClient* client = new EglRenderingVDAClient(
         &rendering_helper, index,
-        note, &data_str, num_NALUs_per_decode,
-        delete_decoder_state);
+        note, data_str, num_NALUs_per_decode,
+        num_in_flight_decodes,
+        reset_after_frame_num, delete_decoder_state, profile);
     clients[index] = client;
 
     rendering_thread.message_loop()->PostTask(
@@ -824,9 +894,13 @@ TEST_P(OmxVideoDecodeAcceleratorTest, TestSimpleDecode) {
     ASSERT_EQ(state, CS_INITIALIZED);
     // InitializeDone kicks off decoding inside the client, so we just need to
     // wait for Flush.
+    ASSERT_NO_FATAL_FAILURE(
+        AssertWaitForStateOrDeleted(note, clients[i], CS_FLUSHING));
     ASSERT_NO_FATAL_FAILURE(
         AssertWaitForStateOrDeleted(note, clients[i], CS_FLUSHED));
     // FlushDone requests Reset().
+    ASSERT_NO_FATAL_FAILURE(
+        AssertWaitForStateOrDeleted(note, clients[i], CS_RESETTING));
     ASSERT_NO_FATAL_FAILURE(
         AssertWaitForStateOrDeleted(note, clients[i], CS_RESET));
     // ResetDone requests Destroy().
@@ -845,7 +919,7 @@ TEST_P(OmxVideoDecodeAcceleratorTest, TestSimpleDecode) {
     EglRenderingVDAClient* client = clients[i];
     if (num_frames > 0)
       EXPECT_EQ(client->num_decoded_frames(), num_frames);
-    if (num_NALUs > 0) {
+    if (num_NALUs > 0 && reset_after_frame_num < 0) {
       EXPECT_EQ(client->num_done_bitstream_buffers(),
                 ceil(static_cast<double>(num_NALUs) / num_NALUs_per_decode));
     }
@@ -872,27 +946,49 @@ TEST_P(OmxVideoDecodeAcceleratorTest, TestSimpleDecode) {
   rendering_thread.Stop();
 };
 
+// Test that Reset() mid-stream works fine and doesn't affect decoding even when
+// Decode() calls are made during the reset.
+INSTANTIATE_TEST_CASE_P(
+    MidStreamReset, OmxVideoDecodeAcceleratorTest,
+    ::testing::Values(
+        MakeTuple(1, 1, 1, static_cast<ResetPoint>(100), CS_RESET)));
+
+// Test that Destroy() mid-stream works fine (primarily this is testing that no
+// crashes occur).
 INSTANTIATE_TEST_CASE_P(
     TearDownTiming, OmxVideoDecodeAcceleratorTest,
     ::testing::Values(
-        MakeTuple(1, 1, CS_DECODER_SET), MakeTuple(1, 1, CS_INITIALIZED),
-        MakeTuple(1, 1, CS_FLUSHED), MakeTuple(1, 1, CS_RESET),
-        MakeTuple(1, 1, static_cast<ClientState>(-1)),
-        MakeTuple(1, 1, static_cast<ClientState>(-10)),
-        MakeTuple(1, 1, static_cast<ClientState>(-100))));
+        MakeTuple(1, 1, 1, END_OF_STREAM_RESET, CS_DECODER_SET),
+        MakeTuple(1, 1, 1, END_OF_STREAM_RESET, CS_INITIALIZED),
+        MakeTuple(1, 1, 1, END_OF_STREAM_RESET, CS_FLUSHING),
+        MakeTuple(1, 1, 1, END_OF_STREAM_RESET, CS_FLUSHED),
+        MakeTuple(1, 1, 1, END_OF_STREAM_RESET, CS_RESETTING),
+        MakeTuple(1, 1, 1, END_OF_STREAM_RESET, CS_RESET),
+        MakeTuple(1, 1, 1, END_OF_STREAM_RESET, static_cast<ClientState>(-1)),
+        MakeTuple(1, 1, 1, END_OF_STREAM_RESET, static_cast<ClientState>(-10)),
+        MakeTuple(1, 1, 1, END_OF_STREAM_RESET,
+                  static_cast<ClientState>(-100))));
 
-// TODO(fischman): using 1st param of 16 and higher below breaks decode - visual
-// artifacts are introduced as well as spurious frames are delivered (more
-// pictures are returned than NALUs are fed to the decoder).  Increase the "15"
-// below when http://code.google.com/p/chrome-os-partner/issues/detail?id=4378
-// is fixed.
+// Test that decoding various variation works: multiple concurrent decoders and
+// multiple NALUs per Decode() call.
 INSTANTIATE_TEST_CASE_P(
     DecodeVariations, OmxVideoDecodeAcceleratorTest,
     ::testing::Values(
-        MakeTuple(1, 1, CS_RESET), MakeTuple(1, 3, CS_RESET),
-        MakeTuple(2, 1, CS_RESET), MakeTuple(3, 1, CS_RESET),
-        MakeTuple(5, 1, CS_RESET), MakeTuple(8, 1, CS_RESET),
-        MakeTuple(15, 1, CS_RESET)));
+        MakeTuple(1, 1, 1, END_OF_STREAM_RESET, CS_RESET),
+        MakeTuple(1, 1, 10, END_OF_STREAM_RESET, CS_RESET),
+        MakeTuple(1, 1, 15, END_OF_STREAM_RESET, CS_RESET),  // Tests queuing.
+        MakeTuple(1, 3, 1, END_OF_STREAM_RESET, CS_RESET),
+        MakeTuple(2, 1, 1, END_OF_STREAM_RESET, CS_RESET),
+        MakeTuple(3, 1, 1, END_OF_STREAM_RESET, CS_RESET),
+        MakeTuple(5, 1, 1, END_OF_STREAM_RESET, CS_RESET),
+        MakeTuple(8, 1, 1, END_OF_STREAM_RESET, CS_RESET),
+        // TODO(fischman): decoding more than 15 NALUs at once breaks decode -
+        // visual artifacts are introduced as well as spurious frames are
+        // delivered (more pictures are returned than NALUs are fed to the
+        // decoder).  Increase the "15" below when
+        // http://code.google.com/p/chrome-os-partner/issues/detail?id=4378 is
+        // fixed.
+        MakeTuple(15, 1, 1, END_OF_STREAM_RESET, CS_RESET)));
 
 // Find out how many concurrent decoders can go before we exhaust system
 // resources.
@@ -900,8 +996,10 @@ INSTANTIATE_TEST_CASE_P(
     ResourceExhaustion, OmxVideoDecodeAcceleratorTest,
     ::testing::Values(
         // +0 hack below to promote enum to int.
-        MakeTuple(1, kMaxSupportedNumConcurrentDecoders + 0, CS_RESET),
-        MakeTuple(1, kMaxSupportedNumConcurrentDecoders + 1, CS_RESET)));
+        MakeTuple(1, kMaxSupportedNumConcurrentDecoders + 0, 1,
+                  END_OF_STREAM_RESET, CS_RESET),
+        MakeTuple(1, kMaxSupportedNumConcurrentDecoders + 1, 1,
+                  END_OF_STREAM_RESET, CS_RESET)));
 
 // TODO(fischman, vrk): add more tests!  In particular:
 // - Test life-cycle: Seek/Stop/Pause/Play/RePlay for a single decoder.
