@@ -10,7 +10,7 @@ namespace plugin_interpose_strings {
 
 const char kDYLDInsertLibrariesKey[] = "DYLD_INSERT_LIBRARIES";
 const char kInterposeLibraryPath[] =
-    "@executable_path/libplugin_carbon_interpose.dylib";
+    "@executable_path/../../../libplugin_carbon_interpose.dylib";
 
 }  // namespace plugin_interpose_strings
 
