@@ -9,6 +9,7 @@
 #include "base/message_loop.h"
 #include "base/process.h"
 #include "base/process_util.h"
+#include "content/common/request_extra_data.h"
 #include "content/common/resource_dispatcher.h"
 #include "content/common/resource_messages.h"
 #include "content/common/resource_response.h"
@@ -170,8 +171,8 @@ class ResourceDispatcherTest : public testing::Test,
     request_info.request_type = ResourceType::SUB_RESOURCE;
     request_info.appcache_host_id = appcache::kNoHostId;
     request_info.routing_id = 0;
-    request_info.is_main_frame = true;
-    request_info.frame_id = 0;
+    RequestExtraData extra_data(true, 0, PageTransition::LINK);
+    request_info.extra_data = &extra_data;
 
     return dispatcher_->CreateBridge(request_info);
   }
