@@ -23,7 +23,8 @@ const int kPluginReleaseTimeMs = 30 * 1000;  // 30 seconds.
 BrokerProcessDispatcher::BrokerProcessDispatcher(
     base::ProcessHandle remote_process_handle,
     PP_ConnectInstance_Func connect_instance)
-    : pp::proxy::BrokerSideDispatcher(remote_process_handle, connect_instance) {
+    : ppapi::proxy::BrokerSideDispatcher(remote_process_handle,
+                                         connect_instance) {
   ChildProcess::current()->AddRefProcess();
 }
 
