@@ -14,6 +14,8 @@
 #include "base/callback_old.h"
 #include "base/memory/linked_ptr.h"
 #include "base/memory/scoped_ptr.h"
+#include "base/memory/weak_ptr.h"
+#include "base/task.h"
 #include "content/renderer/gpu/gpu_video_decode_accelerator_host.h"
 #include "gpu/command_buffer/common/command_buffer.h"
 #include "ipc/ipc_channel.h"
@@ -27,15 +29,17 @@ namespace gfx {
 class Size;
 }
 
+class GpuChannelHost;
 class PluginChannelHost;
 class Task;
 
 // Client side proxy that forwards messages synchronously to a
 // CommandBufferStub.
 class CommandBufferProxy : public gpu::CommandBuffer,
-                           public IPC::Channel::Listener {
+                           public IPC::Channel::Listener,
+                           public base::SupportsWeakPtr<CommandBufferProxy> {
  public:
-  CommandBufferProxy(IPC::Channel::Sender* channel, int route_id);
+  CommandBufferProxy(GpuChannelHost* channel, int route_id);
   virtual ~CommandBufferProxy();
 
   // IPC::Channel::Listener implementation:
@@ -62,7 +66,10 @@ class CommandBufferProxy : public gpu::CommandBuffer,
   virtual void SetToken(int32 token);
   virtual void SetParseError(gpu::error::Error error);
   virtual void SetContextLostReason(gpu::error::ContextLostReason reason);
-  virtual void OnSwapBuffers();
+
+  // Invoke the task when the channel has been flushed. Takes care of deleting
+  // the task whether the echo succeeds or not.
+  bool Echo(Task* task);
 
   // Reparent a command buffer. TODO(apatrick): going forward, the notion of
   // the parent / child relationship between command buffers is going away in
@@ -71,14 +78,8 @@ class CommandBufferProxy : public gpu::CommandBuffer,
   virtual bool SetParent(CommandBufferProxy* parent_command_buffer,
                          uint32 parent_texture_id);
 
-  // Set a callback that will be invoked when the SwapBuffers call has been
-  // issued.
-  void SetSwapBuffersCallback(Callback0::Type* callback);
   void SetChannelErrorCallback(Callback0::Type* callback);
 
-  // Asynchronously resizes an offscreen frame buffer.
-  void ResizeOffscreenFrameBuffer(const gfx::Size& size);
-
   // Set a task that will be invoked the next time the window becomes invalid
   // and needs to be repainted. Takes ownership of task.
   void SetNotifyRepaintTask(Task* task);
@@ -90,14 +91,9 @@ class CommandBufferProxy : public gpu::CommandBuffer,
   // the GPU process, even if this returns non-NULL. In this case the client is
   // notified of an error later.
   scoped_refptr<GpuVideoDecodeAcceleratorHost> CreateVideoDecoder(
-      const std::vector<uint32>& configs,
-      gpu::CommandBufferHelper* cmd_buffer_helper,
+      media::VideoDecodeAccelerator::Profile profile,
       media::VideoDecodeAccelerator::Client* client);
 
-#if defined(OS_MACOSX)
-  virtual void SetWindowSize(const gfx::Size& size);
-#endif
-
  private:
 
   // Send an IPC message over the GPU channel. This is private to fully
@@ -109,6 +105,7 @@ class CommandBufferProxy : public gpu::CommandBuffer,
   void OnUpdateState(const gpu::CommandBuffer::State& state);
   void OnNotifyRepaint();
   void OnDestroyed(gpu::error::ContextLostReason reason);
+  void OnEchoAck();
 
   // As with the service, the client takes ownership of the ring buffer.
   int32 num_entries_;
@@ -118,20 +115,25 @@ class CommandBufferProxy : public gpu::CommandBuffer,
   typedef std::map<int32, gpu::Buffer> TransferBufferMap;
   TransferBufferMap transfer_buffers_;
 
-  // The video decoder host corresponding to the stub's video decoder in the GPU
-  // process, if one exists.
-  scoped_refptr<GpuVideoDecodeAcceleratorHost> video_decoder_host_;
+  // Zero or more video decoder hosts owned by this proxy, keyed by their
+  // decoder_route_id.
+  typedef std::map<int, scoped_refptr<GpuVideoDecodeAcceleratorHost> > Decoders;
+  Decoders video_decoder_hosts_;
 
   // The last cached state received from the service.
   State last_state_;
 
-  IPC::Channel::Sender* channel_;
+  // |*this| is owned by |*channel_| and so is always outlived by it, so using a
+  // raw pointer is ok.
+  GpuChannelHost* channel_;
   int route_id_;
   unsigned int flush_count_;
 
+  // Tasks to be invoked in echo responses.
+  std::queue<linked_ptr<Task> > echo_tasks_;
+
   scoped_ptr<Task> notify_repaint_task_;
 
-  scoped_ptr<Callback0::Type> swap_buffers_callback_;
   scoped_ptr<Callback0::Type> channel_error_callback_;
 
   DISALLOW_COPY_AND_ASSIGN(CommandBufferProxy);
