@@ -14,14 +14,15 @@
 #include "base/memory/ref_counted.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
+#include "base/threading/non_thread_safe.h"
 #include "build/build_config.h"
 #include "ui/gfx/native_widget_types.h"
 #include "ui/gfx/size.h"
 
 class GpuChannelHost;
-class MessageLoop;
 class CommandBufferProxy;
 class GURL;
+class Task;
 class TransportTextureHost;
 
 namespace gpu {
@@ -31,29 +32,35 @@ class GLES2Implementation;
 }
 }
 
-class RendererGLContext : public base::SupportsWeakPtr<RendererGLContext> {
+class RendererGLContext : public base::SupportsWeakPtr<RendererGLContext>,
+                          public base::NonThreadSafe {
  public:
   // These are the same error codes as used by EGL.
   enum Error {
-    SUCCESS             = 0x3000,
-    NOT_INITIALIZED     = 0x3001,
-    BAD_ATTRIBUTE       = 0x3004,
-    BAD_RendererGLContext         = 0x3006,
-    CONTEXT_LOST        = 0x300E
+    SUCCESS               = 0x3000,
+    NOT_INITIALIZED       = 0x3001,
+    BAD_ATTRIBUTE         = 0x3004,
+    BAD_RendererGLContext = 0x3006,
+    CONTEXT_LOST          = 0x300E
   };
 
-  // RendererGLContext configuration attributes. These are the same as used by
-  // EGL. Attributes are matched using a closest fit algorithm.
+  // RendererGLContext configuration attributes. Those in the 16-bit range are
+  // the same as used by EGL. Those outside the 16-bit range are unique to
+  // Chromium. Attributes are matched using a closest fit algorithm.
   enum Attribute {
-    ALPHA_SIZE     = 0x3021,
-    BLUE_SIZE      = 0x3022,
-    GREEN_SIZE     = 0x3023,
-    RED_SIZE       = 0x3024,
-    DEPTH_SIZE     = 0x3025,
-    STENCIL_SIZE   = 0x3026,
-    SAMPLES        = 0x3031,
-    SAMPLE_BUFFERS = 0x3032,
-    NONE           = 0x3038  // Attrib list = terminator
+    ALPHA_SIZE                = 0x3021,
+    BLUE_SIZE                 = 0x3022,
+    GREEN_SIZE                = 0x3023,
+    RED_SIZE                  = 0x3024,
+    DEPTH_SIZE                = 0x3025,
+    STENCIL_SIZE              = 0x3026,
+    SAMPLES                   = 0x3031,
+    SAMPLE_BUFFERS            = 0x3032,
+    HEIGHT                    = 0x3056,
+    WIDTH                     = 0x3057,
+    NONE                      = 0x3038,  // Attrib list = terminator
+    SHARE_RESOURCES           = 0x10000,
+    BIND_GENERATES_RESOURCES  = 0x10001
   };
 
   // Reasons that a lost context might have been provoked.
@@ -98,19 +105,11 @@ class RendererGLContext : public base::SupportsWeakPtr<RendererGLContext> {
   static RendererGLContext* CreateViewContext(
       GpuChannelHost* channel,
       int render_view_id,
-      bool share_resources,
       RendererGLContext* share_group,
       const char* allowed_extensions,
       const int32* attrib_list,
       const GURL& active_arl);
 
-#if defined(OS_MACOSX)
-  // On Mac OS X only, view RendererGLContexts actually behave like offscreen
-  // RendererGLContexts, and require an explicit resize operation which is
-  // slightly different from that of offscreen RendererGLContexts.
-  void ResizeOnscreen(const gfx::Size& size);
-#endif
-
   // Create a RendererGLContext that renders to an offscreen frame buffer. If
   // parent is not NULL, that RendererGLContext can access a copy of the created
   // RendererGLContext's frame buffer that is updated every time SwapBuffers is
@@ -122,7 +121,6 @@ class RendererGLContext : public base::SupportsWeakPtr<RendererGLContext> {
   static RendererGLContext* CreateOffscreenContext(
       GpuChannelHost* channel,
       const gfx::Size& size,
-      bool share_resources,
       RendererGLContext* share_group,
       const char* allowed_extensions,
       const int32* attrib_list,
@@ -132,13 +130,6 @@ class RendererGLContext : public base::SupportsWeakPtr<RendererGLContext> {
   // another parent, it is important to delete them before changing the parent.
   bool SetParent(RendererGLContext* parent);
 
-  // Resize an offscreen frame buffer. The resize occurs on the next call to
-  // SwapBuffers. This is to avoid waiting until all pending GL calls have been
-  // executed by the GPU process. Everything rendered up to the call to
-  // SwapBuffers will be lost. A lost RendererGLContext will be reported if the
-  // resize fails.
-  void ResizeOffscreen(const gfx::Size& size);
-
   // For an offscreen frame buffer RendererGLContext, return the texture ID with
   // respect to the parent RendererGLContext. Returns zero if RendererGLContext
   // does not have a parent.
@@ -151,10 +142,6 @@ class RendererGLContext : public base::SupportsWeakPtr<RendererGLContext> {
   // Deletes a texture in the parent's RendererGLContext.
   void DeleteParentTexture(uint32 texture);
 
-  // Provides a callback that will be invoked when SwapBuffers has completed
-  // service side.
-  void SetSwapBuffersCallback(Callback0::Type* callback);
-
   void SetContextLostCallback(Callback1<ContextLostReason>::Type* callback);
 
   // Set the current RendererGLContext for the calling thread.
@@ -166,6 +153,10 @@ class RendererGLContext : public base::SupportsWeakPtr<RendererGLContext> {
   // by the parent RendererGLContext.
   bool SwapBuffers();
 
+  // Run the task once the channel has been flushed. Takes care of deleting the
+  // task whether the echo succeeds or not.
+  bool Echo(Task* task);
+
   // Create a TransportTextureHost object associated with the context.
   scoped_refptr<TransportTextureHost> CreateTransportTextureHost();
 
@@ -191,27 +182,22 @@ class RendererGLContext : public base::SupportsWeakPtr<RendererGLContext> {
   bool Initialize(bool onscreen,
                   int render_view_id,
                   const gfx::Size& size,
-                  bool share_resources,
-                  bool bind_generates_resource,
                   RendererGLContext* share_group,
                   const char* allowed_extensions,
                   const int32* attrib_list,
                   const GURL& active_url);
   void Destroy();
 
-  void OnSwapBuffers();
   void OnContextLost();
 
   scoped_refptr<GpuChannelHost> channel_;
   base::WeakPtr<RendererGLContext> parent_;
-  scoped_ptr<Callback0::Type> swap_buffers_callback_;
   scoped_ptr<Callback1<ContextLostReason>::Type> context_lost_callback_;
   uint32 parent_texture_id_;
   CommandBufferProxy* command_buffer_;
   gpu::gles2::GLES2CmdHelper* gles2_helper_;
   int32 transfer_buffer_id_;
   gpu::gles2::GLES2Implementation* gles2_implementation_;
-  gfx::Size size_;
   Error last_error_;
   int frame_number_;
 
