@@ -42,6 +42,12 @@ HMAC::~HMAC() {
 bool HMAC::Sign(const base::StringPiece& data,
                 unsigned char* digest,
                 int digest_length) const {
+  if (plat_->key_.empty()) {
+    // Init has not been called or has failed.
+    NOTREACHED();
+    return false;
+  }
+
   CCHmacAlgorithm algorithm;
   int algorithm_digest_length;
   switch (hash_alg_) {
