@@ -7,13 +7,13 @@
 #pragma once
 
 #include "base/basictypes.h"
-#include "crypto/crypto_api.h"
+#include "crypto/crypto_export.h"
 
 namespace crypto {
 
 // A wrapper to calculate secure hashes incrementally, allowing to
 // be used when the full input is not known in advance.
-class CRYPTO_API SecureHash {
+class CRYPTO_EXPORT SecureHash {
  public:
   enum Algorithm {
     SHA256,
