@@ -13,6 +13,7 @@ import fnmatch
 import logging
 import os
 import re
+import shutil
 import subprocess
 import sys
 import tempfile
@@ -92,7 +93,7 @@ class CheckoutBase(object):
     """
     raise NotImplementedError()
 
-  def apply_patch(self, patches):
+  def apply_patch(self, patches, post_processors=None):
     """Applies a patch and returns the list of modified files.
 
     This function should throw patch.UnsupportedPatchFormat or
@@ -117,8 +118,9 @@ class RawCheckout(CheckoutBase):
     """Stubbed out."""
     pass
 
-  def apply_patch(self, patches):
+  def apply_patch(self, patches, post_processors=None):
     """Ignores svn properties."""
+    post_processors = post_processors or self.post_processors or []
     for p in patches:
       try:
         stdout = ''
@@ -136,16 +138,27 @@ class RawCheckout(CheckoutBase):
             with open(filepath, 'wb') as f:
               f.write(p.get())
           else:
+            if p.source_filename:
+              if not p.is_new:
+                raise PatchApplicationFailed(
+                    p.filename,
+                    'File has a source filename specified but is not new')
+              # Copy the file first.
+              if os.path.isfile(filepath):
+                raise PatchApplicationFailed(
+                    p.filename, 'File exist but was about to be overwriten')
+              shutil.copy2(
+                  os.path.join(self.project_path, p.source_filename), filepath)
             if p.diff_hunks:
               stdout = subprocess2.check_output(
-                  ['patch', '-p%s' % p.patchlevel],
-                  stdin=p.get(),
+                  ['patch', '-u', '--binary', '-p%s' % p.patchlevel],
+                  stdin=p.get(False),
                   stderr=subprocess2.STDOUT,
                   cwd=self.project_path)
             elif p.is_new and not os.path.exists(filepath):
               # There is only a header. Just create the file.
               open(filepath, 'w').close()
-        for post in (self.post_processors or []):
+        for post in post_processors:
           post(self, p)
       except OSError, e:
         raise PatchApplicationFailed(p.filename, '%s%s' % (stdout, e))
@@ -261,7 +274,8 @@ class SvnCheckout(CheckoutBase, SvnMixIn):
           (self.project_name, self.project_path))
     return self._revert(revision)
 
-  def apply_patch(self, patches):
+  def apply_patch(self, patches, post_processors=None):
+    post_processors = post_processors or self.post_processors or []
     for p in patches:
       try:
         # It is important to use credentials=False otherwise credentials could
@@ -291,10 +305,21 @@ class SvnCheckout(CheckoutBase, SvnMixIn):
             with open(filepath, 'wb') as f:
               f.write(p.get())
           else:
+            if p.source_filename:
+              if not p.is_new:
+                raise PatchApplicationFailed(
+                    p.filename,
+                    'File has a source filename specified but is not new')
+              # Copy the file first.
+              if os.path.isfile(filepath):
+                raise PatchApplicationFailed(
+                    p.filename, 'File exist but was about to be overwriten')
+              shutil.copy2(
+                  os.path.join(self.project_path, p.source_filename), filepath)
             if p.diff_hunks:
               cmd = ['patch', '-p%s' % p.patchlevel, '--forward', '--force']
               stdout += subprocess2.check_output(
-                  cmd, stdin=p.get(), cwd=self.project_path)
+                  cmd, stdin=p.get(False), cwd=self.project_path)
             elif p.is_new and not os.path.exists(filepath):
               # There is only a header. Just create the file if it doesn't
               # exist.
@@ -314,7 +339,7 @@ class SvnCheckout(CheckoutBase, SvnMixIn):
                   params = value.split('=', 1)
                 stdout += self._check_output_svn(
                     ['propset'] + params + [p.filename], credentials=False)
-        for post in (self.post_processors or []):
+        for post in post_processors:
           post(self, p)
       except OSError, e:
         raise PatchApplicationFailed(p.filename, '%s%s' % (stdout, e))
@@ -418,24 +443,32 @@ class GitCheckoutBase(CheckoutBase):
       if self.working_branch in branches:
         self._call_git(['branch', '-D', self.working_branch])
 
-  def apply_patch(self, patches):
+  def apply_patch(self, patches, post_processors=None):
     """Applies a patch on 'working_branch' and switch to it.
 
     Also commits the changes on the local branch.
 
     Ignores svn properties and raise an exception on unexpected ones.
     """
+    post_processors = post_processors or self.post_processors or []
     # It this throws, the checkout is corrupted. Maybe worth deleting it and
     # trying again?
     if self.remote_branch:
       self._check_call_git(
           ['checkout', '-b', self.working_branch,
             '%s/%s' % (self.remote, self.remote_branch), '--quiet'])
-    for p in patches:
+    for index, p in enumerate(patches):
       try:
         stdout = ''
         if p.is_delete:
-          stdout += self._check_output_git(['rm', p.filename])
+          if (not os.path.exists(p.filename) and
+              any(p1.source_filename == p.filename for p1 in patches[0:index])):
+            # The file could already be deleted if a prior patch with file
+            # rename was already processed. To be sure, look at all the previous
+            # patches to see if they were a file rename.
+            pass
+          else:
+            stdout += self._check_output_git(['rm', p.filename])
         else:
           dirname = os.path.dirname(p.filename)
           full_dir = os.path.join(self.project_path, dirname)
@@ -449,7 +482,7 @@ class GitCheckoutBase(CheckoutBase):
             # No need to do anything special with p.is_new or if not
             # p.diff_hunks. git apply manages all that already.
             stdout += self._check_output_git(
-                ['apply', '--index', '-p%s' % p.patchlevel], stdin=p.get())
+                ['apply', '--index', '-p%s' % p.patchlevel], stdin=p.get(True))
           for prop in p.svn_properties:
             # Ignore some known auto-props flags through .subversion/config,
             # bails out on the other ones.
@@ -461,7 +494,7 @@ class GitCheckoutBase(CheckoutBase):
                   p.filename,
                   'Cannot apply svn property %s to file %s.' % (
                         prop[0], p.filename))
-        for post in (self.post_processors or []):
+        for post in post_processors:
           post(self, p)
       except OSError, e:
         raise PatchApplicationFailed(p.filename, '%s%s' % (stdout, e))
@@ -693,9 +726,11 @@ class GitSvnCheckout(GitSvnCheckoutBase):
 
 class ReadOnlyCheckout(object):
   """Converts a checkout into a read-only one."""
-  def __init__(self, checkout):
+  def __init__(self, checkout, post_processors=None):
     super(ReadOnlyCheckout, self).__init__()
     self.checkout = checkout
+    self.post_processors = (post_processors or []) + (
+        self.checkout.post_processors or [])
 
   def prepare(self, revision):
     return self.checkout.prepare(revision)
@@ -703,8 +738,9 @@ class ReadOnlyCheckout(object):
   def get_settings(self, key):
     return self.checkout.get_settings(key)
 
-  def apply_patch(self, patches):
-    return self.checkout.apply_patch(patches)
+  def apply_patch(self, patches, post_processors=None):
+    return self.checkout.apply_patch(
+        patches, post_processors or self.post_processors)
 
   def commit(self, message, user):  # pylint: disable=R0201
     logging.info('Would have committed for %s with message: %s' % (
