@@ -49,7 +49,7 @@ Hooks
     ]
 """
 
-__version__ = "0.6.2"
+__version__ = "0.6.3"
 
 import copy
 import logging
@@ -69,6 +69,9 @@ import gclient_scm
 import gclient_utils
 from third_party.repo.progress import Progress
 import subprocess2
+from third_party import colorama
+# Import shortcut.
+from third_party.colorama import Fore
 
 
 def attr(attribute, data):
@@ -258,13 +261,17 @@ class Dependency(gclient_utils.WorkItem, DependencySettings):
     # This dependency had its hook run
     self._hooks_ran = False
 
-    # Setup self.requirements and find any other dependency who would have self
-    # as a requirement.
+    if not self.name and self.parent:
+      raise gclient_utils.Error('Dependency without name')
 
+  @property
+  def requirements(self):
+    """Calculate the list of requirements."""
+    requirements = set()
     # self.parent is implicitly a requirement. This will be recursive by
     # definition.
     if self.parent and self.parent.name:
-      self.add_requirement(self.parent.name)
+      requirements.add(self.parent.name)
 
     # For a tree with at least 2 levels*, the leaf node needs to depend
     # on the level higher up in an orderly way.
@@ -277,30 +284,49 @@ class Dependency(gclient_utils.WorkItem, DependencySettings):
     # Interestingly enough, the following condition only works in the case we
     # want: self is a 2nd level node. 3nd level node wouldn't need this since
     # they already have their parent as a requirement.
-    root_deps = self.root.dependencies
-    if self.parent in root_deps:
-      for i in root_deps:
-        if i is self.parent:
-          break
-        if i.name:
-          self.add_requirement(i.name)
+    if self.parent and self.parent.parent and not self.parent.parent.parent:
+      requirements |= set(i.name for i in self.root.dependencies if i.name)
 
     if isinstance(self.url, self.FromImpl):
-      self.add_requirement(self.url.module_name)
+      requirements.add(self.url.module_name)
 
-    if self.name and self.should_process:
-      for obj in self.root.depth_first_tree():
-        if obj is self or not obj.name:
-          continue
-        # Step 1: Find any requirements self may need.
-        if self.name.startswith(posixpath.join(obj.name, '')):
-          self.add_requirement(obj.name)
-        # Step 2: Find any requirements self may impose.
-        if obj.name.startswith(posixpath.join(self.name, '')):
-          obj.add_requirement(self.name)
+    if self.name:
+      requirements |= set(
+          obj.name for obj in self.root.subtree(False)
+          if (obj is not self
+              and obj.name and
+              self.name.startswith(posixpath.join(obj.name, ''))))
+    requirements = tuple(sorted(requirements))
+    logging.info('Dependency(%s).requirements = %s' % (self.name, requirements))
+    return requirements
 
-    if not self.name and self.parent:
-      raise gclient_utils.Error('Dependency without name')
+  def verify_validity(self):
+    """Verifies that this Dependency is fine to add as a child of another one.
+
+    Returns True if this entry should be added, False if it is a duplicate of
+    another entry.
+    """
+    logging.info('Dependency(%s).verify_validity()' % self.name)
+    if self.name in [s.name for s in self.parent.dependencies]:
+      raise gclient_utils.Error(
+          'The same name "%s" appears multiple times in the deps section' %
+              self.name)
+    if not self.should_process:
+      # Return early, no need to set requirements.
+      return True
+
+    # This require a full tree traversal with locks.
+    siblings = [d for d in self.root.subtree(False) if d.name == self.name]
+    for sibling in siblings:
+      if self.url != sibling.url:
+        raise gclient_utils.Error(
+            'Dependency %s specified more than once:\n  %s\nvs\n  %s' %
+            (self.name, sibling.hierarchy(), self.hierarchy()))
+      # In theory we could keep it as a shadow of the other one. In
+      # practice, simply ignore it.
+      logging.warn('Won\'t process duplicate dependency %s' % sibling)
+      return False
+    return True
 
   def LateOverride(self, url):
     """Resolves the parsed url from url.
@@ -310,10 +336,13 @@ class Dependency(gclient_utils.WorkItem, DependencySettings):
     assert self.parsed_url == None or not self.should_process, self.parsed_url
     parsed_url = self.get_custom_deps(self.name, url)
     if parsed_url != url:
-      logging.info('LateOverride(%s, %s) -> %s' % (self.name, url, parsed_url))
+      logging.info(
+          'Dependency(%s).LateOverride(%s) -> %s' %
+          (self.name, url, parsed_url))
       return parsed_url
 
     if isinstance(url, self.FromImpl):
+      # Requires tree traversal.
       ref = [
           dep for dep in self.root.subtree(True) if url.module_name == dep.name
       ]
@@ -323,24 +352,19 @@ class Dependency(gclient_utils.WorkItem, DependencySettings):
       # It may happen that len(ref) > 1 but it's no big deal.
       ref = ref[0]
       sub_target = url.sub_target_name or self.name
-      # Make sure the referenced dependency DEPS file is loaded and file the
-      # inner referenced dependency.
-      ref.ParseDepsFile()
-      found_dep = None
-      for d in ref.dependencies:
-        if d.name == sub_target:
-          found_dep = d
-          break
-      if not found_dep:
+      found_deps = [d for d in ref.dependencies if d.name == sub_target]
+      if len(found_deps) != 1:
         raise gclient_utils.Error(
             'Couldn\'t find %s in %s, referenced by %s (parent: %s)\n%s' % (
                 sub_target, ref.name, self.name, self.parent.name,
                 str(self.root)))
 
       # Call LateOverride() again.
+      found_dep = found_deps[0]
       parsed_url = found_dep.LateOverride(found_dep.url)
       logging.info(
-          'LateOverride(%s, %s) -> %s (From)' % (self.name, url, parsed_url))
+          'Dependency(%s).LateOverride(%s) -> %s (From)' %
+          (self.name, url, parsed_url))
       return parsed_url
 
     if isinstance(url, basestring):
@@ -359,26 +383,29 @@ class Dependency(gclient_utils.WorkItem, DependencySettings):
         parsed_url = scm.FullUrlForRelativeUrl(url)
       else:
         parsed_url = url
-      logging.info('LateOverride(%s, %s) -> %s' % (self.name, url, parsed_url))
+      logging.info(
+          'Dependency(%s).LateOverride(%s) -> %s' %
+          (self.name, url, parsed_url))
       return parsed_url
 
     if isinstance(url, self.FileImpl):
-      logging.info('LateOverride(%s, %s) -> %s (File)' % (self.name, url, url))
+      logging.info(
+          'Dependency(%s).LateOverride(%s) -> %s (File)' %
+          (self.name, url, url))
       return url
 
     if url is None:
-      logging.info('LateOverride(%s, %s) -> %s' % (self.name, url, url))
+      logging.info(
+          'Dependency(%s).LateOverride(%s) -> %s' % (self.name, url, url))
       return url
-    else:
-      raise gclient_utils.Error('Unknown url type')
+
+    raise gclient_utils.Error('Unknown url type')
 
   def ParseDepsFile(self):
     """Parses the DEPS file for this dependency."""
-    assert self.processed == True
-    if self.deps_parsed:
-      logging.debug('%s was already parsed' % self.name)
-      return
-    # One thing is unintuitive, vars= {} must happen before Var() use.
+    assert not self.deps_parsed
+    assert not self.dependencies
+    # One thing is unintuitive, vars = {} must happen before Var() use.
     local_scope = {}
     var = self.VarImpl(self.custom_vars, local_scope)
     global_scope = {
@@ -416,8 +443,6 @@ class Dependency(gclient_utils.WorkItem, DependencySettings):
         else:
           deps.update(os_deps)
 
-    self._deps_hooks.extend(local_scope.get('hooks', []))
-
     # If a line is in custom_deps, but not in the solution, we want to append
     # this line to the solution.
     for d in self.custom_deps:
@@ -437,71 +462,65 @@ class Dependency(gclient_utils.WorkItem, DependencySettings):
       deps = rel_deps
 
     # Convert the deps into real Dependency.
+    deps_to_add = []
     for name, url in deps.iteritems():
-      if name in [s.name for s in self._dependencies]:
-        raise gclient_utils.Error(
-            'The same name "%s" appears multiple times in the deps section' %
-                name)
       should_process = self.recursion_limit and self.should_process
-      if should_process:
-        tree = dict((d.name, d) for d in self.root.subtree(False))
-        if name in tree:
-          if url == tree[name].url:
-            logging.info('Won\'t process duplicate dependency %s' % tree[name])
-            # In theory we could keep it as a shadow of the other one. In
-            # practice, simply ignore it.
-            #should_process = False
-            continue
-          else:
-            raise gclient_utils.Error(
-                'Dependency %s specified more than once:\n  %s\nvs\n  %s' %
-                (name, tree[name].hierarchy(), self.hierarchy()))
-      self._dependencies.append(
-          Dependency(
-            self, name, url, None, None, None, None,
-            self.deps_file, should_process))
-    self._deps_parsed = True
-    logging.debug('Loaded: %s' % str(self))
+      deps_to_add.append(Dependency(
+          self, name, url, None, None, None, None,
+          self.deps_file, should_process))
+    deps_to_add.sort(key=lambda x: x.name)
+    self.add_dependencies_and_close(deps_to_add, local_scope.get('hooks', []))
+    logging.info('ParseDepsFile(%s) done' % self.name)
+
+  def add_dependencies_and_close(self, deps_to_add, hooks):
+    """Adds the dependencies, hooks and mark the parsing as done."""
+    for dep in deps_to_add:
+      if dep.verify_validity():
+        self.add_dependency(dep)
+    self._mark_as_parsed(hooks)
+
+  @staticmethod
+  def maybeGetParentRevision(
+      command, options, parsed_url, parent_name, revision_overrides):
+    """If we are performing an update and --transitive is set, set the
+    revision to the parent's revision. If we have an explicit revision
+    do nothing."""
+    if command == 'update' and options.transitive and not options.revision:
+      _, revision = gclient_utils.SplitUrlRevision(parsed_url)
+      if not revision:
+        options.revision = revision_overrides.get(parent_name)
+        if options.verbose and options.revision:
+          print("Using parent's revision date: %s" % options.revision)
+        # If the parent has a revision override, then it must have been
+        # converted to date format.
+        assert (not options.revision or
+                gclient_utils.IsDateRevision(options.revision))
+
+  @staticmethod
+  def maybeConvertToDateRevision(
+      command, options, name, scm, revision_overrides):
+    """If we are performing an update and --transitive is set, convert the
+    revision to a date-revision (if necessary). Instead of having
+    -r 101 replace the revision with the time stamp of 101 (e.g.
+    "{2011-18-04}").
+    This way dependencies are upgraded to the revision they had at the
+    check-in of revision 101."""
+    if (command == 'update' and
+        options.transitive and
+        options.revision and
+        not gclient_utils.IsDateRevision(options.revision)):
+      revision_date = scm.GetRevisionDate(options.revision)
+      revision = gclient_utils.MakeDateRevision(revision_date)
+      if options.verbose:
+        print("Updating revision override from %s to %s." %
+              (options.revision, revision))
+      revision_overrides[name] = revision
 
   # Arguments number differs from overridden method
   # pylint: disable=W0221
   def run(self, revision_overrides, command, args, work_queue, options):
-    """Runs 'command' before parsing the DEPS in case it's a initial checkout
-    or a revert."""
-
-    def maybeGetParentRevision(options):
-      """If we are performing an update and --transitive is set, set the
-      revision to the parent's revision. If we have an explicit revision
-      do nothing."""
-      if command == 'update' and options.transitive and not options.revision:
-        _, revision = gclient_utils.SplitUrlRevision(self.parsed_url)
-        if not revision:
-          options.revision = revision_overrides.get(self.parent.name)
-          if options.verbose and options.revision:
-            print("Using parent's revision date: %s" % options.revision)
-          # If the parent has a revision override, then it must have been
-          # converted to date format.
-          assert (not options.revision or
-                  gclient_utils.IsDateRevision(options.revision))
-
-    def maybeConvertToDateRevision(options):
-      """If we are performing an update and --transitive is set, convert the
-      revision to a date-revision (if necessary). Instead of having
-      -r 101 replace the revision with the time stamp of 101 (e.g.
-      "{2011-18-04}").
-      This way dependencies are upgraded to the revision they had at the
-      check-in of revision 101."""
-      if (command == 'update' and
-          options.transitive and
-          options.revision and
-          not gclient_utils.IsDateRevision(options.revision)):
-        revision_date = scm.GetRevisionDate(options.revision)
-        revision = gclient_utils.MakeDateRevision(revision_date)
-        if options.verbose:
-          print("Updating revision override from %s to %s." %
-                (options.revision, revision))
-        revision_overrides[self.name] = revision
-
+    """Runs |command| then parse the DEPS file."""
+    logging.info('Dependency(%s).run()' % self.name)
     assert self._file_list == []
     if not self.should_process:
       return
@@ -509,52 +528,63 @@ class Dependency(gclient_utils.WorkItem, DependencySettings):
     # All known hooks are expected to run unconditionally regardless of working
     # copy state, so skip the SCM status check.
     run_scm = command not in ('runhooks', None)
-    self._parsed_url = self.LateOverride(self.url)
-    if run_scm and self.parsed_url:
-      if isinstance(self.parsed_url, self.FileImpl):
+    parsed_url = self.LateOverride(self.url)
+    file_list = []
+    if run_scm and parsed_url:
+      if isinstance(parsed_url, self.FileImpl):
         # Special support for single-file checkout.
         if not command in (None, 'cleanup', 'diff', 'pack', 'status'):
-          options.revision = self.parsed_url.GetRevision()
-          scm = gclient_scm.SVNWrapper(self.parsed_url.GetPath(),
+          # Sadly, pylint doesn't realize that parsed_url is of FileImpl.
+          # pylint: disable=E1103
+          options.revision = parsed_url.GetRevision()
+          scm = gclient_scm.SVNWrapper(parsed_url.GetPath(),
                                        self.root.root_dir,
                                        self.name)
           scm.RunCommand('updatesingle', options,
-                         args + [self.parsed_url.GetFilename()],
-                         self._file_list)
+                         args + [parsed_url.GetFilename()],
+                         file_list)
       else:
         # Create a shallow copy to mutate revision.
         options = copy.copy(options)
         options.revision = revision_overrides.get(self.name)
-        maybeGetParentRevision(options)
-        scm = gclient_scm.CreateSCM(
-            self.parsed_url, self.root.root_dir, self.name)
-        scm.RunCommand(command, options, args, self._file_list)
-        maybeConvertToDateRevision(options)
-        self._file_list = [os.path.join(self.name, f.strip())
-                           for f in self._file_list]
+        self.maybeGetParentRevision(
+            command, options, parsed_url, self.parent.name, revision_overrides)
+        scm = gclient_scm.CreateSCM(parsed_url, self.root.root_dir, self.name)
+        scm.RunCommand(command, options, args, file_list)
+        self.maybeConvertToDateRevision(
+            command, options, self.name, scm, revision_overrides)
+        file_list = [os.path.join(self.name, f.strip()) for f in file_list]
 
       # TODO(phajdan.jr): We should know exactly when the paths are absolute.
       # Convert all absolute paths to relative.
-      for i in range(len(self._file_list)):
+      for i in range(len(file_list)):
         # It depends on the command being executed (like runhooks vs sync).
-        if not os.path.isabs(self._file_list[i]):
+        if not os.path.isabs(file_list[i]):
           continue
         prefix = os.path.commonprefix(
-            [self.root.root_dir.lower(), self._file_list[i].lower()])
-        self._file_list[i] = self._file_list[i][len(prefix):]
+            [self.root.root_dir.lower(), file_list[i].lower()])
+        file_list[i] = file_list[i][len(prefix):]
         # Strip any leading path separators.
-        while (self._file_list[i].startswith('\\') or
-                self._file_list[i].startswith('/')):
-          self._file_list[i] = self._file_list[i][1:]
-    self._processed = True
-    if self.recursion_limit:
-      # Then we can parse the DEPS file.
-      self.ParseDepsFile()
+        while file_list[i].startswith(('\\', '/')):
+          file_list[i] = file_list[i][1:]
 
+    # Always parse the DEPS file.
+    self.ParseDepsFile()
+
+    self._run_is_done(file_list, parsed_url)
+
+    if self.recursion_limit:
       # Parse the dependencies of this dependency.
       for s in self.dependencies:
         work_queue.enqueue(s)
 
+  @gclient_utils.lockedmethod
+  def _run_is_done(self, file_list, parsed_url):
+    # Both these are kept for hooks that are run as a separate tree traversal.
+    self._file_list = file_list
+    self._parsed_url = parsed_url
+    self._processed = True
+
   def RunHooksRecursively(self, options):
     """Evaluates all hooks, running actions as needed. run()
     must have been called before to load the DEPS."""
@@ -579,7 +609,9 @@ class Dependency(gclient_utils.WorkItem, DependencySettings):
         # match each hook's pattern.
         for hook_dict in self.deps_hooks:
           pattern = re.compile(hook_dict['pattern'])
-          matching_file_list = [f for f in self.file_list if pattern.search(f)]
+          matching_file_list = [
+              f for f in self.file_list_and_children if pattern.search(f)
+          ]
           if matching_file_list:
             self._RunHookAction(hook_dict, matching_file_list)
     for s in self.dependencies:
@@ -632,35 +664,56 @@ class Dependency(gclient_utils.WorkItem, DependencySettings):
         if j.should_process:
           yield j
 
+  @gclient_utils.lockedmethod
+  def add_dependency(self, new_dep):
+    self._dependencies.append(new_dep)
+
+  @gclient_utils.lockedmethod
+  def _mark_as_parsed(self, new_hooks):
+    self._deps_hooks.extend(new_hooks)
+    self._deps_parsed = True
+
   @property
+  @gclient_utils.lockedmethod
   def dependencies(self):
     return tuple(self._dependencies)
 
   @property
+  @gclient_utils.lockedmethod
   def deps_hooks(self):
     return tuple(self._deps_hooks)
 
   @property
+  @gclient_utils.lockedmethod
   def parsed_url(self):
     return self._parsed_url
 
   @property
+  @gclient_utils.lockedmethod
   def deps_parsed(self):
+    """This is purely for debugging purposes. It's not used anywhere."""
     return self._deps_parsed
 
   @property
+  @gclient_utils.lockedmethod
   def processed(self):
     return self._processed
 
   @property
+  @gclient_utils.lockedmethod
   def hooks_ran(self):
     return self._hooks_ran
 
   @property
+  @gclient_utils.lockedmethod
   def file_list(self):
-    result = self._file_list[:]
+    return tuple(self._file_list)
+
+  @property
+  def file_list_and_children(self):
+    result = list(self.file_list)
     for d in self.dependencies:
-      result.extend(d.file_list)
+      result.extend(d.file_list_and_children)
     return tuple(result)
 
   def __str__(self):
@@ -765,13 +818,11 @@ solutions = [
       exec(content, config_dict)
     except SyntaxError, e:
       gclient_utils.SyntaxErrorToError('.gclient', e)
+
+    deps_to_add = []
     for s in config_dict.get('solutions', []):
       try:
-        tree = dict((d.name, d) for d in self.root.subtree(False))
-        if s['name'] in tree:
-          raise gclient_utils.Error(
-              'Dependency %s specified more than once in .gclient' % s['name'])
-        self._dependencies.append(Dependency(
+        deps_to_add.append(Dependency(
             self, s['name'], s['url'],
             s.get('safesync_url', None),
             s.get('managed', True),
@@ -782,9 +833,8 @@ solutions = [
       except KeyError:
         raise gclient_utils.Error('Invalid .gclient file. Solution is '
                                   'incomplete: %s' % s)
-    # .gclient can have hooks.
-    self._deps_hooks = config_dict.get('hooks', [])
-    self._deps_parsed = True
+    self.add_dependencies_and_close(deps_to_add, config_dict.get('hooks', []))
+    logging.info('SetConfig() done')
 
   def SaveConfig(self):
     gclient_utils.FileWrite(os.path.join(self.root_dir,
@@ -801,6 +851,17 @@ solutions = [
     client = GClient(path, options)
     client.SetConfig(gclient_utils.FileRead(
         os.path.join(path, options.config_filename)))
+
+    if (options.revisions and
+        len(client.dependencies) > 1 and
+        any('@' not in r for r in options.revisions)):
+      print >> sys.stderr, (
+          'You must specify the full solution name like --revision %s@%s\n'
+          'when you have multiple solutions setup in your .gclient file.\n'
+          'Other solutions present are: %s.') % (
+              client.dependencies[0].name,
+              options.revisions[0],
+              ', '.join(s.name for s in client.dependencies[1:]))
     return client
 
   def SetDefaultConfig(self, solution_name, deps_file, solution_url,
@@ -1057,6 +1118,7 @@ def CMDrecurse(parser, args):
   parser.disable_interspersed_args()
   parser.add_option('-s', '--scm', action='append', default=[],
                     help='choose scm types to operate upon')
+  parser.remove_option('--jobs')
   options, args = parser.parse_args(args)
   if not args:
     print >> sys.stderr, 'Need to supply a command!'
@@ -1374,7 +1436,9 @@ def GenUsage(parser, command):
 def Parser():
   """Returns the default parser."""
   parser = optparse.OptionParser(version='%prog ' + __version__)
-  parser.add_option('-j', '--jobs', default=1, type='int',
+  # cygwin has issues with parallel sync
+  jobs = 1 if sys.platform == 'cygwin' else 8
+  parser.add_option('-j', '--jobs', default=jobs, type='int',
                     help='Specify how many SCM commands can run in parallel; '
                           'default=%default')
   parser.add_option('-v', '--verbose', action='count', default=0,
@@ -1422,6 +1486,7 @@ def Main(argv):
   if sys.hexversion < 0x02050000:
     print >> sys.stderr, (
         '\nYour python version is unsupported, please upgrade.\n')
+  colorama.init()
   try:
     # Make stdout auto-flush so buildbot doesn't kill us during lengthy
     # operations. Python as a strong tendency to buffer sys.stdout.
@@ -1431,9 +1496,14 @@ def Main(argv):
     # Do it late so all commands are listed.
     # Unused variable 'usage'
     # pylint: disable=W0612
-    CMDhelp.usage = ('\n\nCommands are:\n' + '\n'.join([
-        '  %-10s %s' % (fn[3:], Command(fn[3:]).__doc__.split('\n')[0].strip())
-        for fn in dir(sys.modules[__name__]) if fn.startswith('CMD')]))
+    def to_str(fn):
+      return (
+          '  %s%-10s%s' % (Fore.GREEN, fn[3:], Fore.RESET) +
+          ' %s' % Command(fn[3:]).__doc__.split('\n')[0].strip())
+    cmds = (
+        to_str(fn) for fn in dir(sys.modules[__name__]) if fn.startswith('CMD')
+    )
+    CMDhelp.usage = '\n\nCommands are:\n' + '\n'.join(cmds)
     parser = Parser()
     if argv:
       command = Command(argv[0])
