@@ -11,6 +11,7 @@ import Queue
 import re
 import stat
 import sys
+import tempfile
 import threading
 import time
 
@@ -171,6 +172,26 @@ def rmtree(path):
 RemoveDirectory = rmtree
 
 
+def safe_makedirs(tree):
+  """Creates the directory in a safe manner.
+
+  Because multiple threads can create these directories concurently, trap the
+  exception and pass on.
+  """
+  count = 0
+  while not os.path.exists(tree):
+    count += 1
+    try:
+      os.makedirs(tree)
+    except OSError, e:
+      # 17 POSIX, 183 Windows
+      if e.errno not in (17, 183):
+        raise
+      if count > 40:
+        # Give up.
+        raise
+
+
 def CheckCallAndFilterAndHeader(args, always=False, **kwargs):
   """Adds 'header' support to CheckCallAndFilter.
 
@@ -197,124 +218,120 @@ def CheckCallAndFilterAndHeader(args, always=False, **kwargs):
   return CheckCallAndFilter(args, **kwargs)
 
 
-def SoftClone(obj):
-  """Clones an object. copy.copy() doesn't work on 'file' objects."""
-  if obj.__class__.__name__ == 'SoftCloned':
-    return obj
-  class SoftCloned(object):
-    pass
-  new_obj = SoftCloned()
-  for member in dir(obj):
-    if member.startswith('_'):
-      continue
-    setattr(new_obj, member, getattr(obj, member))
-  return new_obj
+class Wrapper(object):
+  """Wraps an object, acting as a transparent proxy for all properties by
+  default.
+  """
+  def __init__(self, wrapped):
+    self._wrapped = wrapped
+
+  def __getattr__(self, name):
+    return getattr(self._wrapped, name)
 
 
-def MakeFileAutoFlush(fileobj, delay=10):
+class AutoFlush(Wrapper):
   """Creates a file object clone to automatically flush after N seconds."""
-  if hasattr(fileobj, 'last_flushed_at'):
-    # Already patched. Just update delay.
-    fileobj.delay = delay
-    return fileobj
+  def __init__(self, wrapped, delay):
+    super(AutoFlush, self).__init__(wrapped)
+    if not hasattr(self, 'lock'):
+      self.lock = threading.Lock()
+    self.__last_flushed_at = time.time()
+    self.delay = delay
 
-  # Attribute 'XXX' defined outside __init__
-  # pylint: disable=W0201
-  new_fileobj = SoftClone(fileobj)
-  if not hasattr(new_fileobj, 'lock'):
-    new_fileobj.lock = threading.Lock()
-  new_fileobj.last_flushed_at = time.time()
-  new_fileobj.delay = delay
-  new_fileobj.old_auto_flush_write = new_fileobj.write
-  # Silence pylint.
-  new_fileobj.flush = fileobj.flush
+  @property
+  def autoflush(self):
+    return self
 
-  def auto_flush_write(out):
-    new_fileobj.old_auto_flush_write(out)
+  def write(self, out, *args, **kwargs):
+    self._wrapped.write(out, *args, **kwargs)
     should_flush = False
-    new_fileobj.lock.acquire()
+    self.lock.acquire()
     try:
-      if (new_fileobj.delay and
-          (time.time() - new_fileobj.last_flushed_at) > new_fileobj.delay):
+      if self.delay and (time.time() - self.__last_flushed_at) > self.delay:
         should_flush = True
-        new_fileobj.last_flushed_at = time.time()
+        self.__last_flushed_at = time.time()
     finally:
-      new_fileobj.lock.release()
+      self.lock.release()
     if should_flush:
-      new_fileobj.flush()
-
-  new_fileobj.write = auto_flush_write
-  return new_fileobj
+      self.flush()
 
 
-def MakeFileAnnotated(fileobj, include_zero=False):
+class Annotated(Wrapper):
   """Creates a file object clone to automatically prepends every line in worker
-  threads with a NN> prefix."""
-  if hasattr(fileobj, 'output_buffers'):
-    # Already patched.
-    return fileobj
+  threads with a NN> prefix.
+  """
+  def __init__(self, wrapped, include_zero=False):
+    super(Annotated, self).__init__(wrapped)
+    if not hasattr(self, 'lock'):
+      self.lock = threading.Lock()
+    self.__output_buffers = {}
+    self.__include_zero = include_zero
 
-  # Attribute 'XXX' defined outside __init__
-  # pylint: disable=W0201
-  new_fileobj = SoftClone(fileobj)
-  if not hasattr(new_fileobj, 'lock'):
-    new_fileobj.lock = threading.Lock()
-  new_fileobj.output_buffers = {}
-  new_fileobj.old_annotated_write = new_fileobj.write
+  @property
+  def annotated(self):
+    return self
 
-  def annotated_write(out):
-    index = getattr(threading.currentThread(), 'index', None)
-    if index is None:
-      if not include_zero:
-        # Unindexed threads aren't buffered.
-        new_fileobj.old_annotated_write(out)
-        return
-      index = 0
+  def write(self, out):
+    index = getattr(threading.currentThread(), 'index', 0)
+    if not index and not self.__include_zero:
+      # Unindexed threads aren't buffered.
+      return self._wrapped.write(out)
 
-    new_fileobj.lock.acquire()
+    self.lock.acquire()
     try:
       # Use a dummy array to hold the string so the code can be lockless.
       # Strings are immutable, requiring to keep a lock for the whole dictionary
       # otherwise. Using an array is faster than using a dummy object.
-      if not index in new_fileobj.output_buffers:
-        obj = new_fileobj.output_buffers[index] = ['']
+      if not index in self.__output_buffers:
+        obj = self.__output_buffers[index] = ['']
       else:
-        obj = new_fileobj.output_buffers[index]
+        obj = self.__output_buffers[index]
     finally:
-      new_fileobj.lock.release()
+      self.lock.release()
 
     # Continue lockless.
     obj[0] += out
     while '\n' in obj[0]:
       line, remaining = obj[0].split('\n', 1)
       if line:
-        new_fileobj.old_annotated_write('%d>%s\n' % (index, line))
+        self._wrapped.write('%d>%s\n' % (index, line))
       obj[0] = remaining
 
-  def full_flush():
+  def flush(self):
     """Flush buffered output."""
     orphans = []
-    new_fileobj.lock.acquire()
+    self.lock.acquire()
     try:
       # Detect threads no longer existing.
       indexes = (getattr(t, 'index', None) for t in threading.enumerate())
       indexes = filter(None, indexes)
-      for index in new_fileobj.output_buffers:
+      for index in self.__output_buffers:
         if not index in indexes:
-          orphans.append((index, new_fileobj.output_buffers[index][0]))
+          orphans.append((index, self.__output_buffers[index][0]))
       for orphan in orphans:
-        del new_fileobj.output_buffers[orphan[0]]
+        del self.__output_buffers[orphan[0]]
     finally:
-      new_fileobj.lock.release()
+      self.lock.release()
 
     # Don't keep the lock while writting. Will append \n when it shouldn't.
     for orphan in orphans:
       if orphan[1]:
-        new_fileobj.old_annotated_write('%d>%s\n' % (orphan[0], orphan[1]))
+        self._wrapped.write('%d>%s\n' % (orphan[0], orphan[1]))
+    return self._wrapped.flush()
 
-  new_fileobj.write = annotated_write
-  new_fileobj.full_flush = full_flush
-  return new_fileobj
+
+def MakeFileAutoFlush(fileobj, delay=10):
+  autoflush = getattr(fileobj, 'autoflush', None)
+  if autoflush:
+    autoflush.delay = delay
+    return fileobj
+  return AutoFlush(fileobj, delay)
+
+
+def MakeFileAnnotated(fileobj, include_zero=False):
+  if getattr(fileobj, 'annotated', None):
+    return fileobj
+  return Annotated(fileobj)
 
 
 def CheckCallAndFilter(args, stdout=None, filter_fn=None,
@@ -360,6 +377,9 @@ def CheckCallAndFilter(args, stdout=None, filter_fn=None,
           else:
             filter_fn(in_line)
             in_line = ''
+        else:
+          filter_fn(in_line)
+          in_line = ''
         in_byte = kid.stdout.read(1)
       # Flush the rest of buffered output. This is only an issue with
       # stdout/stderr not ending with a \n.
@@ -429,14 +449,17 @@ def PathDifference(root, subpath):
 
 
 def FindFileUpwards(filename, path=None):
-  """Search upwards from the a directory (default: current) to find a file."""
+  """Search upwards from the a directory (default: current) to find a file.
+  
+  Returns nearest upper-level directory with the passed in file.
+  """
   if not path:
     path = os.getcwd()
   path = os.path.realpath(path)
   while True:
     file_path = os.path.join(path, filename)
-    if os.path.isfile(file_path):
-      return file_path
+    if os.path.exists(file_path):
+      return path
     (new_path, _) = os.path.split(path)
     if new_path == path:
       return None
@@ -446,12 +469,11 @@ def FindFileUpwards(filename, path=None):
 def GetGClientRootAndEntries(path=None):
   """Returns the gclient root and the dict of entries."""
   config_file = '.gclient_entries'
-  config_path = FindFileUpwards(config_file, path)
-
-  if not config_path:
+  root = FindFileUpwards(config_file, path)
+  if not root:
     print "Can't find %s" % config_file
     return None
-
+  config_path = os.path.join(root, config_file)
   env = {}
   execfile(config_path, env)
   config_dir = os.path.dirname(config_path)
@@ -475,14 +497,15 @@ def lockedmethod(method):
 
 class WorkItem(object):
   """One work item."""
+  # On cygwin, creating a lock throwing randomly when nearing ~100 locks.
+  # As a workaround, use a single lock. Yep you read it right. Single lock for
+  # all the 100 objects.
+  lock = threading.Lock()
+
   def __init__(self, name):
-    # A list of string, each being a WorkItem name.
-    self._requirements = set()
     # A unique string representing this work item.
     self._name = name
-    self.lock = threading.RLock()
 
-  @lockedmethod
   def run(self, work_queue):
     """work_queue is passed as keyword argument so it should be
     the last parameters of the function when you override it."""
@@ -492,15 +515,6 @@ class WorkItem(object):
   def name(self):
     return self._name
 
-  @property
-  @lockedmethod
-  def requirements(self):
-    return tuple(self._requirements)
-
-  @lockedmethod
-  def add_requirement(self, new):
-    self._requirements.add(new)
-
 
 class ExecutionQueue(object):
   """Runs a set of WorkItem that have interdependencies and were WorkItem are
@@ -621,7 +635,7 @@ class ExecutionQueue(object):
         self.running.append(t)
       else:
         t.join()
-        sys.stdout.full_flush()  # pylint: disable=E1101
+        sys.stdout.flush()
         if self.progress:
           self.progress.update(1, t.item.name)
         if t.item.name in self.ran:
@@ -674,3 +688,47 @@ class ExecutionQueue(object):
         work_queue.ready_cond.notifyAll()
       finally:
         work_queue.ready_cond.release()
+
+
+def GetEditor(git):
+  """Returns the most plausible editor to use."""
+  if git:
+    editor = os.environ.get('GIT_EDITOR')
+  else:
+    editor = os.environ.get('SVN_EDITOR')
+  if not editor:
+    editor = os.environ.get('EDITOR')
+  if not editor:
+    if sys.platform.startswith('win'):
+      editor = 'notepad'
+    else:
+      editor = 'vim'
+  return editor
+
+
+def RunEditor(content, git):
+  """Opens up the default editor in the system to get the CL description."""
+  file_handle, filename = tempfile.mkstemp(text=True)
+  # Make sure CRLF is handled properly by requiring none.
+  if '\r' in content:
+    print >> sys.stderr, (
+        '!! Please remove \\r from your change description !!')
+  fileobj = os.fdopen(file_handle, 'w')
+  # Still remove \r if present.
+  fileobj.write(re.sub('\r?\n', '\n', content))
+  fileobj.close()
+
+  try:
+    cmd = '%s %s' % (GetEditor(git), filename)
+    if sys.platform == 'win32' and os.environ.get('TERM') == 'msys':
+      # Msysgit requires the usage of 'env' to be present.
+      cmd = 'env ' + cmd
+    try:
+      # shell=True to allow the shell to handle all forms of quotes in
+      # $EDITOR.
+      subprocess2.check_call(cmd, shell=True)
+    except subprocess2.CalledProcessError:
+      return None
+    return FileRead(filename)
+  finally:
+    os.remove(filename)
