@@ -12,7 +12,6 @@ import optparse
 import os
 import re
 import sys
-import tempfile
 import textwrap
 import urlparse
 import urllib2
@@ -36,6 +35,7 @@ except ImportError:
 from third_party import upload
 import breakpad  # pylint: disable=W0611
 import fix_encoding
+import gclient_utils
 import presubmit_support
 import rietveld
 import scm
@@ -495,7 +495,16 @@ or verify this branch is set up to track another (via the --track argument to
     # We use the sha1 of HEAD as a name of this change.
     name = RunCommand(['git', 'rev-parse', 'HEAD']).strip()
     # Need to pass a relative path for msysgit.
-    files = scm.GIT.CaptureStatus([root], upstream_branch)
+    try:
+      files = scm.GIT.CaptureStatus([root], upstream_branch)
+    except subprocess2.CalledProcessError:
+      DieWithError(
+          ('\nFailed to diff against upstream branch %s!\n\n'
+           'This branch probably doesn\'t exist anymore. To reset the\n'
+           'tracking branch, please run\n'
+           '    git branch --set-upstream %s trunk\n'
+           'replacing trunk with origin/master or the relevant branch') %
+          (upstream_branch, self.GetBranch()))
 
     issue = ConvertToInteger(self.GetIssue())
     patchset = ConvertToInteger(self.GetPatchset())
@@ -639,7 +648,13 @@ class ChangeDescription(object):
       initial_text += '\nBUG='
     if 'TEST=' not in self.description:
       initial_text += '\nTEST='
-    self._ParseDescription(UserEditedLog(initial_text))
+    content = gclient_utils.RunEditor(initial_text, True)
+    if not content:
+      DieWithError('Running editor failed')
+    content = re.compile(r'^#.*$', re.MULTILINE).sub('', content).strip()
+    if not content:
+      DieWithError('No CL description, aborting')
+    self._ParseDescription(content)
 
   def _ParseDescription(self, description):
     if not description:
@@ -815,39 +830,6 @@ def CreateDescriptionFromLog(args):
   return RunGit(['log', '--pretty=format:%s\n\n%b'] + log_args)
 
 
-def UserEditedLog(starting_text):
-  """Given some starting text, let the user edit it and return the result."""
-  editor = os.getenv('EDITOR', 'vi')
-
-  (file_handle, filename) = tempfile.mkstemp()
-  fileobj = os.fdopen(file_handle, 'w')
-  fileobj.write(starting_text)
-  fileobj.close()
-
-  # Open up the default editor in the system to get the CL description.
-  try:
-    cmd = '%s %s' % (editor, filename)
-    if sys.platform == 'win32' and os.environ.get('TERM') == 'msys':
-      # Msysgit requires the usage of 'env' to be present.
-      cmd = 'env ' + cmd
-    # shell=True to allow the shell to handle all forms of quotes in $EDITOR.
-    try:
-      subprocess2.check_call(cmd, shell=True)
-    except subprocess2.CalledProcessError, e:
-      DieWithError('Editor returned %d' % e.returncode)
-    fileobj = open(filename)
-    text = fileobj.read()
-    fileobj.close()
-  finally:
-    os.remove(filename)
-
-  if not text:
-    return
-
-  stripcomment_re = re.compile(r'^#.*$', re.MULTILINE)
-  return stripcomment_re.sub('', text).strip()
-
-
 def ConvertToInteger(inputval):
   """Convert a string to integer, but returns either an int or None."""
   try:
@@ -1302,8 +1284,7 @@ def CMDrebase(parser, args):
   # git svn dcommit.
   # It's the only command that doesn't use parser at all since we just defer
   # execution to git-svn.
-  subprocess2.check_call(['git', 'svn', 'rebase'] + args)
-  return 0
+  return subprocess2.call(['git', 'svn', 'rebase'] + args)
 
 
 def GetTreeStatus():
