@@ -5,20 +5,22 @@
 
 """Unit tests for presubmit_support.py and presubmit_canned_checks.py."""
 
-# pylint is too confused.
-# pylint: disable=E1101,E1103,R0201,W0212,W0403
+# pylint: disable=E1101,E1103
 
 import logging
+import os
 import StringIO
 import sys
 import time
 
-# Fixes include path.
-from super_mox import mox, SuperMoxTestBase
+sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
+
+from testing_support.super_mox import mox, SuperMoxTestBase
 
 import owners
 import presubmit_support as presubmit
 import rietveld
+
 # Shortcut.
 presubmit_canned_checks = presubmit.presubmit_canned_checks
 
@@ -233,7 +235,7 @@ class PresubmitUnittest(PresubmitTestsBase):
 
   def testTagLineRe(self):
     self.mox.ReplayAll()
-    m = presubmit.Change._TAG_LINE_RE.match(' BUG =1223, 1445  \t')
+    m = presubmit.Change.TAG_LINE_RE.match(' BUG =1223, 1445  \t')
     self.failUnless(m)
     self.failUnlessEqual(m.group('key'), 'BUG')
     self.failUnlessEqual(m.group('value'), '1223, 1445')
@@ -1020,6 +1022,8 @@ class InputApiUnittest(PresubmitTestsBase):
           f('b.c/.git'),
           f('a/.git/bleh.py'),
           f('.git/bleh.py'),
+          f('bleh.diff'),
+          f('foo/bleh.patch'),
         ],
         [
           # Expected.
@@ -1031,7 +1035,7 @@ class InputApiUnittest(PresubmitTestsBase):
     self.mox.ReplayAll()
 
     self.assertEqual(len(input_api.DEFAULT_WHITE_LIST), 21)
-    self.assertEqual(len(input_api.DEFAULT_BLACK_LIST), 9)
+    self.assertEqual(len(input_api.DEFAULT_BLACK_LIST), 11)
     for item in files:
       results = filter(input_api.FilterSourceFile, item[0])
       for i in range(len(results)):
@@ -1344,6 +1348,7 @@ class ChangeUnittest(PresubmitTestsBase):
         'AbsoluteLocalPaths', 'AffectedFiles', 'AffectedTextFiles',
         'DescriptionText', 'FullDescriptionText', 'LocalPaths', 'Name',
         'RepositoryRoot', 'RightHandSideLines', 'ServerPaths',
+        'TAG_LINE_RE',
         'author_email', 'issue', 'patchset', 'scm', 'tags',
     ]
     # If this test fails, you should add the relevant test.
@@ -1370,10 +1375,8 @@ class ChangeUnittest(PresubmitTestsBase):
 class CannedChecksUnittest(PresubmitTestsBase):
   """Tests presubmit_canned_checks.py."""
 
-  def setUp(self):
-    PresubmitTestsBase.setUp(self)
-
   def MockInputApi(self, change, committing):
+    # pylint: disable=R0201
     input_api = self.mox.CreateMock(presubmit.InputApi)
     input_api.cStringIO = presubmit.cStringIO
     input_api.json = presubmit.json
@@ -1682,6 +1685,7 @@ class CannedChecksUnittest(PresubmitTestsBase):
     self.assertEquals(len(results1), 1)
     self.assertEquals(results1[0].__class__,
         presubmit.OutputApi.PresubmitPromptWarning)
+    # pylint: disable=W0212
     self.assertEquals(results1[0]._long_text,
         'makefile.foo, line 46')
 
@@ -1971,6 +1975,7 @@ class CannedChecksUnittest(PresubmitTestsBase):
     self.assertEquals(len(results), 1)
     self.assertEquals(results[0].__class__,
                       presubmit.OutputApi.PresubmitNotifyResult)
+    # pylint: disable=W0212
     self.assertEquals('test_module failed!\nfoo', results[0]._message)
 
   def testRunPythonUnitTestsFailureCommitting(self):
@@ -1985,6 +1990,7 @@ class CannedChecksUnittest(PresubmitTestsBase):
         input_api, presubmit.OutputApi, ['test_module'])
     self.assertEquals(len(results), 1)
     self.assertEquals(results[0].__class__, presubmit.OutputApi.PresubmitError)
+    # pylint: disable=W0212
     self.assertEquals('test_module failed!\nfoo', results[0]._message)
 
   def testRunPythonUnitTestsSuccess(self):
@@ -2225,6 +2231,7 @@ class CannedChecksUnittest(PresubmitTestsBase):
         project_name=None,
         owners_check=True)
     self.assertEqual(1, len(results))
+    # pylint: disable=W0212
     self.assertEqual(
         'Found line ending with white spaces in:', results[0]._message)
     self.checkstdout('')
