@@ -10,15 +10,12 @@ import os
 import sys
 import unittest
 
-ROOT_DIR = os.path.dirname(os.path.abspath(__file__))
-sys.path.insert(0, os.path.join(ROOT_DIR, '..'))
+sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
+
+from testing_support.patches_data import GIT, RAW
 
 import patch
 import rietveld
-from tests.patches_data import GIT, RAW
-
-# Access to a protected member XX of a client class
-# pylint: disable=W0212
 
 
 def _api(files):
@@ -40,12 +37,16 @@ def _file(
 
 class RietveldTest(unittest.TestCase):
   def setUp(self):
+    super(RietveldTest, self).setUp()
+    # Access to a protected member XX of a client class
+    # pylint: disable=W0212
     self.rietveld = rietveld.Rietveld('url', 'email', 'password')
     self.rietveld._send = self._rietveld_send
     self.requests = []
 
   def tearDown(self):
     self.assertEquals([], self.requests)
+    super(RietveldTest, self).tearDown()
 
   def _rietveld_send(self, url, *args, **kwargs):
     self.assertTrue(self.requests, url)
@@ -187,7 +188,23 @@ class RietveldTest(unittest.TestCase):
     ]
     patches = self.rietveld.get_patch(123, 456)
     self.assertEquals(1, len(patches.patches))
-    self._check_patch(patches.patches[0], name, None, is_delete=True)
+    self._check_patch(patches.patches[0], name, RAW.DELETE, is_delete=True)
+
+  def test_delete_empty(self):
+    name = 'tests/__init__.py'
+    self.requests = [
+        ('/api/123/456', _api({name: _file('D')})),
+        ('/download/issue123_456_789.diff', GIT.DELETE_EMPTY),
+    ]
+    patches = self.rietveld.get_patch(123, 456)
+    self.assertEquals(1, len(patches.patches))
+    self._check_patch(
+        patches.patches[0],
+        name,
+        GIT.DELETE_EMPTY,
+        is_delete=True,
+        is_git_diff=True,
+        patchlevel=1)
 
   def test_m_plus(self):
     properties = '\nAdded: svn:eol-style\n   + LF\n'
@@ -347,5 +364,6 @@ class RietveldTest(unittest.TestCase):
 
 
 if __name__ == '__main__':
-  logging.basicConfig(level=logging.ERROR)
+  logging.basicConfig(level=[
+    logging.ERROR, logging.INFO, logging.DEBUG][min(2, sys.argv.count('-v'))])
   unittest.main()
