@@ -79,7 +79,7 @@ class CommandBuffer {
   virtual State GetState() = 0;
 
   // Returns the last state without synchronizing with the service.
-  virtual State GetLastState();
+  virtual State GetLastState() = 0;
 
   // The writer calls this to update its put offset. This ensures the reader
   // sees the latest added commands, and will eventually process them. On the
@@ -127,39 +127,12 @@ class CommandBuffer {
   // Allows the reader to set the current context lost reason.
   // NOTE: if calling this in conjunction with SetParseError,
   // call this first.
-  //
-  // TODO(kbr): this temporarily has a definition (i.e., is not pure
-  // virtual) to work around a difficult interdependency with the NaCl
-  // build. Make this pure virtual and remove the body once this is
-  // defined in CommandBufferNaCl and NaCl has rolled forward. See
-  // http://crbug.com/89127 .
-  virtual void SetContextLostReason(error::ContextLostReason);
+  virtual void SetContextLostReason(error::ContextLostReason) = 0;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(CommandBuffer);
 };
 
-// Synchronizing other mechanisms (such as IPC) with the CommandBuffer requires
-// inserting (writing) a token into the buffer and knowing what the last token
-// read at that point was.  ReadWriteTokens is a convenience struct for passing
-// these pairs around.  Expected usage is to compare a current token to
-// [last_token_read,last_token_written).
-class ReadWriteTokens {
- public:
-  ReadWriteTokens(int32 read, int32 written);
-  // Required to support pickling.  Use by anything else will DCHECK in InRange.
-  ReadWriteTokens();
-
-  // Return true iff |value| is in the range described by |tokens|, accounting
-  // for (up to) one wrap-around.
-  bool InRange(int32 token) const;
-
-  // These want to be private (and const) but can't in order to support
-  // pickling.
-  int32 last_token_read;
-  int32 last_token_written;
-};
-
 }  // namespace gpu
 
 #endif  // GPU_COMMAND_BUFFER_COMMON_COMMAND_BUFFER_H_
