@@ -131,8 +131,7 @@ class MockCommandBufferEngine : public CommandBufferEngine {
 
  private:
   bool IsValidSharedMemoryId(int32 shm_id) {
-    return shm_id == kValidShmId || shm_id == kStartValidShmId ||
-        shm_id == gpu::kLatchSharedMemoryId;
+    return shm_id == kValidShmId || shm_id == kStartValidShmId;
   }
 
   int8 buffer_[kBufferSize];
@@ -556,11 +555,5 @@ TEST_F(CommonDecoderTest, GetBucketData) {
   EXPECT_NE(error::kNoError, ExecuteCmd(cmd));
 }
 
-TEST_F(CommonDecoderTest, YieldScheduler) {
-  cmd::YieldScheduler cmd;
-  cmd.Init();
-  EXPECT_EQ(error::kYield, ExecuteCmd(cmd));
-}
-
 }  // namespace gpu
 
