@@ -57,10 +57,4 @@ void GpuScheduler::Destroy() {
   DestroyCommon();
 }
 
-void GpuScheduler::WillSwapBuffers() {
-  if (wrapped_swap_buffers_callback_.get()) {
-    wrapped_swap_buffers_callback_->Run();
-  }
-}
-
 }  // namespace gpu
