@@ -10,7 +10,6 @@
 #include "gpu/command_buffer/service/context_group.h"
 #include "gpu/command_buffer/service/gles2_cmd_decoder.h"
 #include "gpu/command_buffer/service/gpu_scheduler.h"
-#include "gpu/GLES2/gles2_command_buffer.h"
 #include "gpu/gles2_conform_support/egl/config.h"
 #include "gpu/gles2_conform_support/egl/surface.h"
 
@@ -111,9 +110,7 @@ EGLSurface Display::CreateWindowSurface(EGLConfig config,
   std::vector<int32> attribs;
   gpu::gles2::ContextGroup::Ref group(new gpu::gles2::ContextGroup(true));
   scoped_ptr<GpuScheduler> gpu_scheduler(
-      GpuScheduler::Create(command_buffer_.get(),
-                           NULL,
-                           NULL));
+      GpuScheduler::Create(command_buffer_.get(), group.get()));
   if (!gpu_scheduler->Initialize(
       win, gfx::Size(), false, gpu::gles2::DisallowedExtensions(), NULL,
       attribs, NULL))
@@ -164,10 +161,10 @@ EGLContext Display::CreateContext(EGLConfig config,
       share_resources,
       true));
 
-  context_->CommandBufferEnableCHROMIUM(
-      PEPPER3D_ALLOW_BUFFERS_ON_MULTIPLE_TARGETS);
-  context_->CommandBufferEnableCHROMIUM(
-      PEPPER3D_SUPPORT_FIXED_ATTRIBS);
+  context_->EnableFeatureCHROMIUM(
+      "pepper3d_allow_buffers_on_multiple_targets");
+  context_->EnableFeatureCHROMIUM(
+      "pepper3d_support_fixed_attribs");
 
   return context_.get();
 }
