@@ -1,4 +1,4 @@
-// Copyright (c) 2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -21,12 +21,17 @@ class TaskPump : public talk_base::TaskRunner {
   virtual void WakeTasks();
   virtual int64 CurrentTime();
 
+  // No tasks will be processed after this is called, even if
+  // WakeTasks() is called.
+  void Stop();
+
  private:
   void CheckAndRunTasks();
 
   base::NonThreadSafe non_thread_safe_;
   ScopedRunnableMethodFactory<TaskPump> scoped_runnable_method_factory_;
   bool posted_wake_;
+  bool stopped_;
 
   DISALLOW_COPY_AND_ASSIGN(TaskPump);
 };
