@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -6,11 +6,12 @@
 #define MEDIA_AUDIO_AUDIO_BUFFERS_STATE_H_
 
 #include "base/time.h"
+#include "media/base/media_export.h"
 
 // AudioBuffersState struct stores current state of audio buffers along with
 // the timestamp of the moment this state corresponds to. It is used for audio
 // synchronization.
-struct AudioBuffersState {
+struct MEDIA_EXPORT AudioBuffersState {
   AudioBuffersState();
   AudioBuffersState(int pending_bytes, int hardware_delay_bytes);
 
