@@ -21,8 +21,11 @@ class AudioManagerMac : public AudioManagerBase {
   // Implementation of AudioManager.
   virtual bool HasAudioOutputDevices();
   virtual bool HasAudioInputDevices();
-  virtual AudioOutputStream* MakeAudioOutputStream(AudioParameters params);
-  virtual AudioInputStream* MakeAudioInputStream(AudioParameters params);
+  virtual void GetAudioInputDeviceNames(media::AudioDeviceNames* device_names);
+  virtual AudioOutputStream* MakeAudioOutputStream(
+      const AudioParameters& params);
+  virtual AudioInputStream* MakeAudioInputStream(
+      const AudioParameters& params);
   virtual void MuteAll();
   virtual void UnMuteAll();
 
