@@ -10,13 +10,14 @@
 #include "base/memory/ref_counted.h"
 #include "base/memory/scoped_ptr.h"
 #include "base/message_loop_proxy.h"
+#include "media/base/media_export.h"
 
 class MessageLoop;
 
 namespace media {
 
 // Factory object that manages named MessageLoops.
-class MessageLoopFactory {
+class MEDIA_EXPORT MessageLoopFactory {
  public:
   // Get the message loop associated with |name|. A new MessageLoop
   // is created if the factory doesn't have one associated with |name|.
