@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,10 +8,11 @@
 #include <map>
 
 #include "base/logging.h"
+#include "media/base/media_export.h"
 
 namespace media {
 
-class StateMatrix {
+class MEDIA_EXPORT StateMatrix {
  public:
   StateMatrix();
   ~StateMatrix();
