@@ -8,6 +8,7 @@
 #include <X11/Xlib.h>
 
 #include "base/at_exit.h"
+#include "base/bind.h"
 #include "base/command_line.h"
 #include "base/file_path.h"
 #include "base/memory/scoped_ptr.h"
@@ -16,6 +17,7 @@
 #include "media/base/callback.h"
 #include "media/base/filter_collection.h"
 #include "media/base/media.h"
+#include "media/base/media_log.h"
 #include "media/base/media_switches.h"
 #include "media/base/message_loop_factory_impl.h"
 #include "media/base/pipeline_impl.h"
@@ -108,7 +110,7 @@ bool InitPipeline(MessageLoop* message_loop,
     collection->AddAudioRenderer(new media::NullAudioRenderer());
 
   // Create the pipeline and start it.
-  *pipeline = new media::PipelineImpl(message_loop);
+  *pipeline = new media::PipelineImpl(message_loop, new media::MediaLog());
   media::PipelineStatusNotification note;
   (*pipeline)->Start(collection.release(), filename, note.Callback());
 
@@ -116,7 +118,7 @@ bool InitPipeline(MessageLoop* message_loop,
   note.Wait();
   if (note.status() != media::PIPELINE_OK) {
     std::cout << "InitPipeline: " << note.status() << std::endl;
-    (*pipeline)->Stop(NULL);
+    (*pipeline)->Stop(media::PipelineStatusCB());
     return false;
   }
 
@@ -137,7 +139,8 @@ void PeriodicalUpdate(
     // interrupt signal was received during last time period.
     // Quit message_loop only when pipeline is fully stopped.
     MessageLoopQuitter* quitter = new MessageLoopQuitter(message_loop);
-    pipeline->Stop(NewCallback(quitter, &MessageLoopQuitter::Quit));
+    pipeline->Stop(base::Bind(&MessageLoopQuitter::Quit,
+                              base::Unretained(quitter)));
     return;
   }
 
@@ -161,7 +164,7 @@ void PeriodicalUpdate(
                        &border_width,
                        &depth);
           base::TimeDelta time = pipeline->GetMediaDuration();
-          pipeline->Seek(time*e.xbutton.x/width, NULL);
+          pipeline->Seek(time*e.xbutton.x/width, media::PipelineStatusCB());
         }
         break;
       case KeyPress:
@@ -171,7 +174,8 @@ void PeriodicalUpdate(
             g_running = false;
             // Quit message_loop only when pipeline is fully stopped.
             MessageLoopQuitter* quitter = new MessageLoopQuitter(message_loop);
-            pipeline->Stop(NewCallback(quitter, &MessageLoopQuitter::Quit));
+            pipeline->Stop(base::Bind(&MessageLoopQuitter::Quit,
+                                      base::Unretained(quitter)));
             return;
           } else if (key == XK_space) {
             if (pipeline->GetPlaybackRate() < 0.01f) // paused
