@@ -39,11 +39,12 @@ cd ../../..
 
 if [[ "${BUILDBOT_SLAVE_TYPE:-Trybot}" != "Trybot" ]]; then
   echo @@@BUILD_STEP archive_build@@@
+  gsutil=buildbot/gsutil.sh
   GS_BASE=gs://nativeclient-archive2/toolchain
-  /b/build/scripts/slave/gsutil -h Cache-Control:no-cache cp -a public-read \
+  ${gsutil} -h Cache-Control:no-cache cp -a public-read \
       arm-trusted.tgz \
       ${GS_BASE}/${BUILDBOT_GOT_REVISION}/naclsdk_linux_arm-trusted.tgz
-  /b/build/scripts/slave/gsutil -h Cache-Control:no-cache cp -a public-read \
+  ${gsutil} -h Cache-Control:no-cache cp -a public-read \
       arm-trusted.tgz \
       ${GS_BASE}/latest/naclsdk_linux_arm-trusted.tgz
-fi
\ No newline at end of file
+fi
