@@ -1,6 +1,7 @@
 #!/usr/bin/python2.4
-# Copyright 2009, Google Inc.
-# All rights reserved.
+# Copyright (c) 2011 The Native Client Authors. All rights reserved.
+# Use of this source code is governed by a BSD-style license that can be
+# found in the LICENSE file.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are
@@ -32,7 +33,7 @@
 
 
 import SCons
-
+import library_deps
 
 __component_list = {}
 
@@ -133,7 +134,12 @@ def _ComponentPlatformSetup(env, builder_name, **kwargs):
   # approach allows us to use an unmodified version of SCons.
   # In general, the use of EXTRA_LIBS is discouraged.
   if 'EXTRA_LIBS' in env:
-    env['LIBS'] = env['EXTRA_LIBS'] + env['LIBS']
+    # The SubstList2 method expands and flattens so that scons will
+    # correctly know about the library dependencies in cases like
+    # EXTRA_LIBS=['${FOO_LIBS}', 'bar'].
+    env['LIBS'] = (library_deps.AddLibDeps(env['TARGET_FULLARCH'],
+                                           env.SubstList2('${EXTRA_LIBS}')) +
+                   env.SubstList2('${LIBS}'))
 
   # Call platform-specific component setup function, if any
   if env.get('COMPONENT_PLATFORM_SETUP'):
@@ -462,12 +468,26 @@ def ComponentProgram(self, prog_name, *args, **kwargs):
 
   env['PROGRAM_BASENAME'] = prog_name
 
+  if env['PROGSUFFIX'] and env.subst(prog_name).endswith(env['PROGSUFFIX']):
+    # Temporary hack: If there's already an extension, remove it.
+    # Because PPAPI is revision locked, and expects to be able to use .nexe
+    # TODO: When PPAPI deps is rolled, replace with this:
+    # raise Exception("Program name shouldn't have a suffix")
+    prog_name = env.subst(prog_name)
+    prog_name = prog_name[:-len(env['PROGSUFFIX'])]
+
   # Call env.Program()
   out_nodes = env.Program(prog_name, *args, **kwargs)
 
   # Add dependencies on includes
   env.Depends(out_nodes, env['INCLUDES'])
 
+  # Add dependencies on libraries marked as implicitly included in the link.
+  # These are libraries that are not passed on the command line, but are
+  # always linked in by the toolchain, i.e. startup files and -lc and such.
+  if 'IMPLICIT_LIBS' in env:
+    env.Depends(out_nodes, env['IMPLICIT_LIBS'])
+
   # Publish output
   env.Publish(prog_name, 'run', out_nodes[0])
   env.Publish(prog_name, 'debug', out_nodes[1:])
@@ -627,4 +647,3 @@ def generate(env):
   AddTargetGroup('run_small_tests', 'small tests can be run')
   AddTargetGroup('run_medium_tests', 'medium tests can be run')
   AddTargetGroup('run_large_tests', 'large tests can be run')
-
