@@ -1,4 +1,4 @@
-// Copyright (c) 2006-2009 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,14 +9,14 @@
 
 LaunchNaClProcessFunc launch_nacl_process = NULL;
 
+#if !defined(NACL_STANDALONE)
 namespace nacl {
-  bool SelLdrLauncher::StartFromBrowser(const char* url, int socket_count,
-                                        Handle* result_sockets) {
+  bool SelLdrLauncher::Start(int socket_count, Handle* result_sockets) {
     // send a synchronous message to the browser process
     Handle nacl_proc_handle;
     int nacl_proc_id;
     if (!launch_nacl_process ||
-        !launch_nacl_process(url,
+        !launch_nacl_process("",
                              socket_count,
                              result_sockets,
                              &nacl_proc_handle,
@@ -24,13 +24,16 @@ namespace nacl {
       return false;
     }
 
-#if NACL_WINDOWS && !defined(NACL_STANDALONE)
+#if NACL_WINDOWS
     NaClHandlePassBrowserRememberHandle(nacl_proc_id, nacl_proc_handle);
 #endif
 
     CloseHandlesAfterLaunch();
     // TODO(gregoryd): the handle is currently returned on Windows only.
     child_process_ = nacl_proc_handle;
+    // channel_ conveys the bootstrap channel.
+    channel_ = result_sockets[0];
     return true;
   }
 }
+#endif  // !defined(NACL_STANDALONE)
