@@ -15,8 +15,10 @@
 #include <stdlib.h>
 #include <string.h>
 
+#define NACL_LOG_MODULE_NAME  "elf_util"
+
 #include "native_client/src/include/elf_constants.h"
-#include "native_client/src/include/nacl_elf.h"
+#include "native_client/src/include/elf.h"
 #include "native_client/src/include/nacl_macros.h"
 #include "native_client/src/include/nacl_platform.h"
 
@@ -145,40 +147,6 @@ static void NaClDumpElfProgramHeader(int     loglevel,
 }
 
 
-NaClErrorCode NaClElfImageValidateAbi(struct NaClElfImage *image) {
-  const Elf_Ehdr *hdr = &image->ehdr;
-
-  if (ELFOSABI_NACL != hdr->e_ident[EI_OSABI]) {
-    NaClLog(LOG_ERROR, "Expected OSABI %d, got %d\n",
-            ELFOSABI_NACL,
-            hdr->e_ident[EI_OSABI]);
-    return LOAD_BAD_ABI;
-  }
-
-  /*
-   * TODO(bsy,khim): e_ident[EI_ABIVERSION] for nacl64 should match nacl32.
-   */
-#if NACL_TARGET_SUBARCH == 64
-  if (EF_NACL_ABIVERSION != hdr->e_ident[EI_ABIVERSION]
-      && 0 != hdr->e_ident[EI_ABIVERSION]) {
-    NaClLog(LOG_ERROR, "Expected ABIVERSION %d, got %d\n",
-            EF_NACL_ABIVERSION,
-            hdr->e_ident[EI_ABIVERSION]);
-      return LOAD_BAD_ABI;
-  }
-#else
-  if (EF_NACL_ABIVERSION != hdr->e_ident[EI_ABIVERSION]) {
-    NaClLog(LOG_ERROR, "Expected ABIVERSION %d, got %d\n",
-            EF_NACL_ABIVERSION,
-            hdr->e_ident[EI_ABIVERSION]);
-      return LOAD_BAD_ABI;
-  }
-#endif
-
-  return LOAD_OK;
-}
-
-
 NaClErrorCode NaClElfImageValidateElfHeader(struct NaClElfImage *image) {
   const Elf_Ehdr *hdr = &image->ehdr;
 
@@ -670,15 +638,12 @@ NaClErrorCode NaClElfImageLoadDynamically(struct NaClElfImage *image,
           return LOAD_MPROTECT_FAIL;
         }
 
-        if (!NaClVmmapAdd(&nap->mem_map,
-                          php->p_vaddr >> NACL_PAGESHIFT,
-                          mapping_size >> NACL_PAGESHIFT,
-                          PROT_READ,
-                          NULL)) {
-          NaClLog(1, "NaClElfImageLoadDynamically: "
-                  "failed to update mapping info to read-only\n");
-          return LOAD_MPROTECT_FAIL;
-        }
+        NaClVmmapUpdate(&nap->mem_map,
+                        php->p_vaddr >> NACL_PAGESHIFT,
+                        mapping_size >> NACL_PAGESHIFT,
+                        PROT_READ,
+                        NULL,
+                        0  /* remove: false */);
       }
     }
   }
@@ -694,18 +659,3 @@ void NaClElfImageDelete(struct NaClElfImage *image) {
 uintptr_t NaClElfImageGetEntryPoint(struct NaClElfImage *image) {
   return image->ehdr.e_entry;
 }
-
-
-int NaClElfImageGetBundleSize(struct NaClElfImage *image) {
-  switch (image->ehdr.e_flags & EF_NACL_ALIGN_MASK) {
-   case  EF_NACL_ALIGN_16:
-    return 16;
-   case EF_NACL_ALIGN_32:
-    return 32;
-   default:
-#if 0 == NACL_DANGEROUS_DEBUG_MODE_DISABLE_INNER_SANDBOX
-    NaClLog(LOG_FATAL, "strange alignment");
-#endif
-    return 0;
-  }
-}
