@@ -68,83 +68,18 @@ static INLINE size_t  size_min(size_t a, size_t b) {
 
 static int const kKnownInvalidDescNumber = -1;
 
-/*
- * natp should be thread_self(), called while holding no locks.
- */
-void NaClSysCommonThreadSuicide(struct NaClAppThread  *natp) {
-  struct NaClApp  *nap;
-  size_t          thread_idx;
-
-  /*
-   * mark this thread as dead; doesn't matter if some other thread is
-   * asking us to commit suicide.
-   */
-  NaClLog(3, "NaClSysCommonThreadSuicide(0x%08"NACL_PRIxPTR")\n",
-          (uintptr_t) natp);
-  nap = natp->nap;
-  NaClLog(3, " getting thread table lock\n");
-  NaClXMutexLock(&nap->threads_mu);
-  NaClLog(3, " getting thread lock\n");
-  NaClXMutexLock(&natp->mu);
-  natp->state = NACL_APP_THREAD_DEAD;
-  /*
-   * Remove ourselves from the ldt-indexed global tables.  The ldt
-   * entry is released as part of NaClAppThreadDtor (via
-   * NaClAppThreadDecRef), and if another thread is immediately
-   * created (from some other running thread) we want to be sure that
-   * any ldt-based lookups will not reach this dying thread's data.
-   */
-  thread_idx = NaClGetThreadIdx(natp);
-  nacl_sys[thread_idx] = NULL;
-  nacl_user[thread_idx] = NULL;
-  nacl_thread[thread_idx] = NULL;
-  NaClLog(3, " removing thread from thread table\n");
-  NaClRemoveThreadMu(nap, natp->thread_num);
-  NaClLog(3, " unlocking thread\n");
-  NaClXMutexUnlock(&natp->mu);
-  NaClLog(3, " announcing thread count change\n");
-  NaClXCondVarBroadcast(&nap->threads_cv);
-  NaClLog(3, " unlocking thread table\n");
-  NaClXMutexUnlock(&nap->threads_mu);
-  NaClLog(3, " freeing thread object\n");
-  NaClAppThreadDtor(natp);
-  NaClLog(3, " NaClThreadExit\n");
-
-  /*
-   * There appears to be a race on Windows where the process can sometimes
-   * return a thread exit status instead of the process exit status when
-   * they occur near simultaneously on two seperate threads.  Since this is
-   * non-deterministic, we always exit a thread with the current value of the
-   * process exit status to mitigate the possibility of exiting with an
-   * incorrect value.  See BUG= nacl1715
-   */
-  NaClThreadExit(nap->exit_status);
-  /* should not return */
-  NaClLog(LOG_ERROR, "INCONCEIVABLE!\n");
-  NaClAbort();
-  /* NOTREACHED */
-}
-
 void NaClSysCommonThreadSyscallEnter(struct NaClAppThread *natp) {
-  NaClLog(4, "NaClSysCommonThreadSyscallEnter: locking 0x%08"NACL_PRIxPTR"\n",
-          (uintptr_t) &natp->mu);
-  NaClXMutexLock(&natp->mu);
-  natp->holding_sr_locks = 1;
-  NaClLog(4, "NaClSysCommonThreadSyscallEnter: unlocking 0x%08"NACL_PRIxPTR
-          "\n\n",
-          (uintptr_t) &natp->mu);
-  NaClXMutexUnlock(&natp->mu);
+  UNREFERENCED_PARAMETER(natp);
 }
 
 void NaClSysCommonThreadSyscallLeave(struct NaClAppThread *natp) {
   NaClXMutexLock(&natp->mu);
-  natp->holding_sr_locks = 0;
   switch (natp->state) {
     case NACL_APP_THREAD_ALIVE:
       break;
     case NACL_APP_THREAD_SUICIDE_PENDING:
       NaClXMutexUnlock(&natp->mu);
-      NaClSysCommonThreadSuicide(natp);
+      NaClAppThreadTeardown(natp);
       /* NOTREACHED */
       break;
     case NACL_APP_THREAD_DEAD:
@@ -393,7 +328,6 @@ int32_t NaClCommonSysExit(struct NaClAppThread  *natp,
   NaClSysCommonThreadSyscallEnter(natp);
 
   nap = natp->nap;
-  NaClSyncQueueQuit(&nap->work_queue);
 
   NaClXMutexLock(&nap->mu);
   nap->exit_status = status;
@@ -401,7 +335,7 @@ int32_t NaClCommonSysExit(struct NaClAppThread  *natp,
   NaClXCondVarSignal(&nap->cv);
   NaClXMutexUnlock(&nap->mu);
 
-  NaClSysCommonThreadSuicide(natp);
+  NaClAppThreadTeardown(natp);
   /* NOTREACHED */
   return -NACL_ABI_EINVAL;
 }
@@ -443,7 +377,7 @@ int32_t NaClCommonSysThreadExit(struct NaClAppThread  *natp,
     }
   }
 
-  NaClSysCommonThreadSuicide(natp);
+  NaClAppThreadTeardown(natp);
   /* NOTREACHED */
   return -NACL_ABI_EINVAL;
 }
@@ -722,18 +656,19 @@ int32_t NaClCommonSysGetdents(struct NaClAppThread *natp,
 
   NaClSysCommonThreadSyscallEnter(natp);
 
-  sysaddr = NaClUserToSysAddrRange(natp->nap, (uintptr_t) dirp, count);
-  if (kNaClBadAddress == sysaddr) {
-    NaClLog(4, " illegal address for directory data\n");
-    retval = -NACL_ABI_EFAULT;
-    goto cleanup;
-  }
   ndp = NaClGetDesc(natp->nap, d);
   if (NULL == ndp) {
     retval = -NACL_ABI_EBADF;
     goto cleanup;
   }
 
+  sysaddr = NaClUserToSysAddrRange(natp->nap, (uintptr_t) dirp, count);
+  if (kNaClBadAddress == sysaddr) {
+    NaClLog(4, " illegal address for directory data\n");
+    retval = -NACL_ABI_EFAULT;
+    goto cleanup_unref;
+  }
+
   /*
    * Clamp count to INT32_MAX to avoid the possibility of Getdents returning
    * a value that is outside the range of an int32.
@@ -759,6 +694,8 @@ int32_t NaClCommonSysGetdents(struct NaClAppThread *natp,
   } else {
     NaClLog(4, "getdents returned %d\n", retval);
   }
+
+cleanup_unref:
   NaClDescUnref(ndp);
 
 cleanup:
@@ -785,23 +722,34 @@ int32_t NaClCommonSysRead(struct NaClAppThread  *natp,
 
   NaClSysCommonThreadSyscallEnter(natp);
 
-  sysaddr = NaClUserToSysAddrRange(natp->nap, (uintptr_t) buf, count);
-  if (kNaClBadAddress == sysaddr) {
-    retval = -NACL_ABI_EFAULT;
-    goto cleanup;
-  }
   ndp = NaClGetDesc(natp->nap, d);
   if (NULL == ndp) {
     retval = -NACL_ABI_EBADF;
     goto cleanup;
   }
 
+  sysaddr = NaClUserToSysAddrRange(natp->nap, (uintptr_t) buf, count);
+  if (kNaClBadAddress == sysaddr) {
+    NaClDescUnref(ndp);
+    retval = -NACL_ABI_EFAULT;
+    goto cleanup;
+  }
+
+  /*
+   * The maximum length for read and write is INT32_MAX--anything larger and
+   * the return value would overflow. Passing larger values isn't an error--
+   * we'll just clamp the request size if it's too large.
+   */
+  if (count > INT32_MAX) {
+    count = INT32_MAX;
+  }
+
   read_result = (*((struct NaClDescVtbl const *) ndp->base.vtbl)->
                  Read)(ndp, (void *) sysaddr, count);
   if (read_result > 0) {
     NaClLog(4, "read returned %"NACL_PRIdS" bytes\n", read_result);
     NaClLog(8, "read result: %.*s\n",
-           (read_result < INT_MAX) ? (int) read_result : INT_MAX,
+           (int) read_result,
            (char *) sysaddr);
   } else {
     NaClLog(4, "read returned %"NACL_PRIdS"\n", read_result);
@@ -833,8 +781,15 @@ int32_t NaClCommonSysWrite(struct NaClAppThread *natp,
 
   NaClSysCommonThreadSyscallEnter(natp);
 
+  ndp = NaClGetDesc(natp->nap, d);
+  if (NULL == ndp) {
+    retval = -NACL_ABI_EBADF;
+    goto cleanup;
+  }
+
   sysaddr = NaClUserToSysAddrRange(natp->nap, (uintptr_t) buf, count);
   if (kNaClBadAddress == sysaddr) {
+    NaClDescUnref(ndp);
     retval = -NACL_ABI_EFAULT;
     goto cleanup;
   }
@@ -842,12 +797,6 @@ int32_t NaClCommonSysWrite(struct NaClAppThread *natp,
   NaClLog(4, "In NaClSysWrite(%d, %.*s, %"NACL_PRIdS")\n",
           d, (int) count, (char *) sysaddr, count);
 
-  ndp = NaClGetDesc(natp->nap, d);
-  if (NULL == ndp) {
-    retval = -NACL_ABI_EBADF;
-    goto cleanup;
-  }
-
   /*
    * The maximum length for read and write is INT32_MAX--anything larger and
    * the return value would overflow. Passing larger values isn't an error--
@@ -892,19 +841,20 @@ int32_t NaClCommonSysLseek(struct NaClAppThread *natp,
 
   NaClSysCommonThreadSyscallEnter(natp);
 
-  sysaddr = NaClUserToSysAddrRange(natp->nap, (uintptr_t) offp, sizeof offset);
-  if (kNaClBadAddress == sysaddr) {
-    retval = -NACL_ABI_EFAULT;
-    goto cleanup;
-  }
-  offset = *(nacl_abi_off_t volatile *) sysaddr;
-  NaClLog(4, "offset 0x%08"NACL_PRIxNACL_OFF"\n", offset);
   ndp = NaClGetDesc(natp->nap, d);
   if (NULL == ndp) {
     retval = -NACL_ABI_EBADF;
     goto cleanup;
   }
 
+  sysaddr = NaClUserToSysAddrRange(natp->nap, (uintptr_t) offp, sizeof offset);
+  if (kNaClBadAddress == sysaddr) {
+    retval = -NACL_ABI_EFAULT;
+    goto cleanup_unref;
+  }
+  offset = *(nacl_abi_off_t volatile *) sysaddr;
+  NaClLog(4, "offset 0x%08"NACL_PRIxNACL_OFF"\n", offset);
+
   retval64 = (*((struct NaClDescVtbl const *) ndp->base.vtbl)->
               Seek)(ndp, (nacl_off64_t) offset, whence);
   if (NaClOff64IsNegErrno(&retval64)) {
@@ -913,6 +863,7 @@ int32_t NaClCommonSysLseek(struct NaClAppThread *natp,
     *(nacl_abi_off_t volatile *) sysaddr = retval64;
     retval = 0;
   }
+cleanup_unref:
   NaClDescUnref(ndp);
 cleanup:
   NaClSysCommonThreadSyscallLeave(natp);
@@ -991,21 +942,21 @@ int32_t NaClCommonSysFstat(struct NaClAppThread *natp,
           " sizeof(struct nacl_abi_stat) = %"NACL_PRIdS" (0x%"NACL_PRIxS")\n",
           sizeof *nasp, sizeof *nasp);
 
-  sysaddr = NaClUserToSysAddrRange(natp->nap, (uintptr_t) nasp, sizeof *nasp);
-  if (kNaClBadAddress == sysaddr) {
-    NaClLog(4, "bad addr\n");
-    retval = -NACL_ABI_EFAULT;
-    goto cleanup;
-  }
-
   ndp = NaClGetDesc(natp->nap, d);
   if (NULL == ndp) {
     NaClLog(4, "bad desc\n");
     retval = -NACL_ABI_EBADF;
     goto cleanup;
   }
-  retval = (*((struct NaClDescVtbl const *) ndp->base.vtbl)->
-            Fstat)(ndp, (struct nacl_abi_stat *) sysaddr);
+
+  sysaddr = NaClUserToSysAddrRange(natp->nap, (uintptr_t) nasp, sizeof *nasp);
+  if (kNaClBadAddress == sysaddr) {
+    NaClLog(4, "bad addr\n");
+    retval = -NACL_ABI_EFAULT;
+  } else {
+    retval = (*((struct NaClDescVtbl const *) ndp->base.vtbl)->
+              Fstat)(ndp, (struct nacl_abi_stat *) sysaddr);
+  }
 
   NaClDescUnref(ndp);
 cleanup:
@@ -2249,39 +2200,34 @@ cleanup:
 }
 
 int32_t NaClCommonSysTls_Init(struct NaClAppThread  *natp,
-                              void                  *tdb,
-                              size_t                tdb_size) {
+                              void                  *thread_ptr) {
   int32_t   retval = -NACL_ABI_EINVAL;
-  uintptr_t sys_tdb;
+  uintptr_t sys_tls;
 
   NaClLog(3,
           ("Entered NaClCommonSysTls_Init(0x%08"NACL_PRIxPTR
-           " 0x%08"NACL_PRIxPTR", 0x%08"NACL_PRIxS")\n"),
-          (uintptr_t) natp, (uintptr_t) tdb, tdb_size);
+           ", 0x%08"NACL_PRIxPTR")\n"),
+          (uintptr_t) natp, (uintptr_t) thread_ptr);
 
   NaClSysCommonThreadSyscallEnter(natp);
 
   /* Verify that the address in the app's range and translated from
    * nacl module address to service runtime address - a nop on ARM
    */
-  sys_tdb = NaClUserToSysAddrRange(natp->nap, (uintptr_t) tdb, tdb_size);
-  NaClLog(4, "NaClCommonSysTls_Init: tdb 0x%p, sys_tdb 0x%"NACL_PRIxPTR"\n",
-          tdb, sys_tdb);
-  if (kNaClBadAddress == sys_tdb) {
+  sys_tls = NaClUserToSysAddrRange(natp->nap, (uintptr_t) thread_ptr, 4);
+  NaClLog(4,
+          "NaClCommonSysTls_Init: thread_ptr 0x%p, sys_tls 0x%"NACL_PRIxPTR"\n",
+          thread_ptr, sys_tls);
+  if (kNaClBadAddress == sys_tls) {
     retval = -NACL_ABI_EFAULT;
     goto cleanup;
   }
 
-  if (tdb_size > UINT32_MAX) {
-    retval = -NACL_ABI_EOVERFLOW;
-    goto cleanup;
-  }
-
-  if (0 == NaClTlsChange(natp, (void *) sys_tdb, (int32_t) tdb_size)) {
+  if (0 == NaClTlsChange(natp, (void *) sys_tls)) {
     retval = -NACL_ABI_EINVAL;
     goto cleanup;
   }
-  natp->sys_tdb = sys_tdb;
+  natp->sys_tls = sys_tls;
   retval = 0;
 cleanup:
   NaClSysCommonThreadSyscallLeave(natp);
@@ -2291,18 +2237,18 @@ cleanup:
 int32_t NaClCommonSysThread_Create(struct NaClAppThread *natp,
                                    void                 *prog_ctr,
                                    void                 *stack_ptr,
-                                   void                 *tdb,
-                                   size_t               tdb_size) {
+                                   void                 *thread_ptr,
+                                   void                 *second_thread_ptr) {
   int32_t     retval = -NACL_ABI_EINVAL;
-  uintptr_t   sys_tdb;
+  uintptr_t   sys_tls;
   uintptr_t   sys_stack;
 
   NaClLog(3,
           ("Entered NaClCommonSysThread_Create(0x%08"NACL_PRIxPTR
-           " pc=0x%08"NACL_PRIxPTR", sp=0x%08"NACL_PRIxPTR", tdb=0x%08"
-           NACL_PRIxPTR", tdb_size=0x%08"NACL_PRIxS")\n"),
+           " pc=0x%08"NACL_PRIxPTR", sp=0x%08"NACL_PRIxPTR", thread_ptr=0x%08"
+           NACL_PRIxPTR")\n"),
           (uintptr_t) natp, (uintptr_t) prog_ctr, (uintptr_t) stack_ptr,
-          (uintptr_t) tdb, tdb_size);
+          (uintptr_t) thread_ptr);
 
   NaClSysCommonThreadSyscallEnter(natp);
 
@@ -2329,46 +2275,32 @@ int32_t NaClCommonSysThread_Create(struct NaClAppThread *natp,
     retval = -NACL_ABI_EFAULT;
     goto cleanup;
   }
-  sys_tdb = NaClUserToSysAddrRange(natp->nap, (uintptr_t) tdb, tdb_size);
-  if (kNaClBadAddress == sys_tdb) {
-    NaClLog(LOG_ERROR, "bad tdb\n");
+  sys_tls = NaClUserToSysAddrRange(natp->nap, (uintptr_t) thread_ptr, 4);
+  if (kNaClBadAddress == sys_tls) {
+    NaClLog(LOG_ERROR, "bad TLS pointer\n");
     retval = -NACL_ABI_EFAULT;
     goto cleanup;
   }
 
-  NaClXMutexLock(&natp->nap->mu);
-
-  /* ensure no virtual memory hole may appear */
-  while (natp->nap->vm_hole_may_exist) {
-    NaClXCondVarWait(&natp->nap->cv, &natp->nap->mu);
-  }
-
-  ++natp->nap->threads_launching;
-  NaClXMutexUnlock(&natp->nap->mu);
-  /*
-   * NB: Dropped lock, so many threads launching can starve VM
-   * operations.  If this becomes a problem in practice, we can use a
-   * reader/writer lock so that a waiting writer will block new
-   * readers.
-   */
+  NaClVmHoleWaitToStartThread(natp->nap);
 
   retval = NaClCreateAdditionalThread(natp->nap,
                                       (uintptr_t) prog_ctr,
                                       sys_stack,
-                                      sys_tdb,
-                                      tdb_size);
+                                      sys_tls,
+                                      (uint32_t) (uintptr_t) second_thread_ptr);
 
 cleanup:
   NaClSysCommonThreadSyscallLeave(natp);
   return retval;
 }
 
-int32_t NaClCommonSysTdbGet(struct NaClAppThread *natp) {
-  uint32_t user_tdb;
+int32_t NaClCommonSysTlsGet(struct NaClAppThread *natp) {
+  uint32_t user_tls;
 
   /* too frequently used, and syscall-number level logging suffices */
-  user_tdb = (int32_t) NaClSysToUser(natp->nap, natp->sys_tdb);
-  return user_tdb;
+  user_tls = (int32_t) NaClSysToUser(natp->nap, natp->sys_tls);
+  return user_tls;
 }
 
 int NaClCommonSysThread_Nice(struct NaClAppThread *natp,
