@@ -28,24 +28,32 @@
 #include "native_client/src/trusted/desc/nacl_desc_io.h"
 #include "native_client/src/trusted/desc/nrd_xfer.h"
 
+#include "native_client/src/trusted/fault_injection/fault_injection.h"
+
 #include "native_client/src/trusted/handle_pass/ldr_handle.h"
 
-#include "native_client/src/trusted/service_runtime/arch/sel_ldr_arch.h"
+#include "native_client/src/trusted/reverse_service/reverse_control_rpc.h"
+
 #include "native_client/src/trusted/gio/gio_nacl_desc.h"
 #include "native_client/src/trusted/gio/gio_shm.h"
+#include "native_client/src/trusted/service_runtime/arch/sel_ldr_arch.h"
 #include "native_client/src/trusted/service_runtime/include/sys/fcntl.h"
 #include "native_client/src/trusted/service_runtime/nacl_app.h"
 #include "native_client/src/trusted/service_runtime/nacl_app_thread.h"
 #include "native_client/src/trusted/service_runtime/nacl_desc_effector_ldr.h"
 #include "native_client/src/trusted/service_runtime/nacl_globals.h"
 #include "native_client/src/trusted/service_runtime/nacl_syscall_common.h"
+#include "native_client/src/trusted/service_runtime/nacl_syscall_handlers.h"
 #include "native_client/src/trusted/service_runtime/nacl_valgrind_hooks.h"
 #include "native_client/src/trusted/service_runtime/sel_addrspace.h"
 #include "native_client/src/trusted/service_runtime/sel_ldr.h"
 #include "native_client/src/trusted/service_runtime/sel_memory.h"
 
-#include "native_client/src/trusted/service_runtime/name_service/name_service.h"
 #include "native_client/src/trusted/service_runtime/name_service/default_name_service.h"
+#include "native_client/src/trusted/service_runtime/name_service/name_service.h"
+
+#include "native_client/src/trusted/service_runtime/sel_ldr_thread_interface.h"
+#include "native_client/src/trusted/threading/nacl_thread_interface.h"
 
 #include "native_client/src/trusted/service_runtime/include/sys/stat.h"
 
@@ -62,17 +70,27 @@ static int ShouldEnableDynamicLoading() {
   return !IsEnvironmentVariableSet("NACL_DISABLE_DYNAMIC_LOADING");
 }
 
-int NaClAppCtor(struct NaClApp  *nap) {
+int NaClAppWithSyscallTableCtor(struct NaClApp               *nap,
+                                struct NaClSyscallTableEntry *table) {
   struct NaClDescEffectorLdr  *effp;
 
   nap->addr_bits = NACL_MAX_ADDR_BITS;
 
-  nap->max_data_alloc = NACL_DEFAULT_ALLOC_MAX;
   nap->stack_size = NACL_DEFAULT_STACK_MAX;
 
   nap->aux_info = NULL;
 
   nap->mem_start = 0;
+
+#if (NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 \
+     && NACL_BUILD_SUBARCH == 32 && __PIC__)
+  nap->pcrel_thunk = 0;
+#endif
+#if (NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 \
+     && NACL_BUILD_SUBARCH == 64)
+  nap->dispatch_thunk = 0;
+#endif
+
   nap->static_text_end = 0;
   nap->dynamic_text_start = 0;
   nap->dynamic_text_end = 0;
@@ -121,26 +139,14 @@ int NaClAppCtor(struct NaClApp  *nap) {
   nap->service_port = NULL;
   nap->service_address = NULL;
   nap->secure_service = NULL;
+  nap->manifest_proxy = NULL;
+  nap->kern_service = NULL;
   nap->reverse_client = NULL;
-  nap->reverse_channel_initialized = 0;
-
-  nap->name_service = (struct NaClNameService *) malloc(
-      sizeof *nap->name_service);
-  if (NULL == nap->name_service) {
-    goto cleanup_dynamic_load_mutex;
-  }
-  if (!NaClNameServiceCtor(nap->name_service)) {
-    free(nap->name_service);
-    goto cleanup_dynamic_load_mutex;
-  }
-  nap->name_service_conn_cap = NaClDescRef(nap->name_service->
-                                           base.base.bound_and_cap[1]);
-  if (!NaClDefaultNameServiceInit(nap->name_service)) {
-    goto cleanup_name_service;
-  }
+  nap->reverse_channel_initialization_state =
+      NACL_REVERSE_CHANNEL_UNINITIALIZED;
 
   if (!NaClMutexCtor(&nap->mu)) {
-    goto cleanup_name_service;
+    goto cleanup_dynamic_load_mutex;
   }
   if (!NaClCondVarCtor(&nap->cv)) {
     goto cleanup_mu;
@@ -149,9 +155,28 @@ int NaClAppCtor(struct NaClApp  *nap) {
   nap->vm_hole_may_exist = 0;
   nap->threads_launching = 0;
 
+  nap->syscall_table = table;
+
   nap->module_load_status = LOAD_STATUS_UNKNOWN;
   nap->module_may_start = 0;  /* only when secure_service != NULL */
 
+  nap->name_service = (struct NaClNameService *) malloc(
+      sizeof *nap->name_service);
+  if (NULL == nap->name_service) {
+    goto cleanup_cv;
+  }
+  if (!NaClNameServiceCtor(nap->name_service,
+                           NaClAddrSpSquattingThreadIfFactoryFunction,
+                           (void *) nap)) {
+    free(nap->name_service);
+    goto cleanup_cv;
+  }
+  nap->name_service_conn_cap = NaClDescRef(nap->name_service->
+                                           base.base.bound_and_cap[1]);
+  if (!NaClDefaultNameServiceInit(nap->name_service)) {
+    goto cleanup_name_service;
+  }
+
   nap->ignore_validator_result = 0;
   nap->skip_validator = 0;
   nap->validator_stub_out_mode = 0;
@@ -173,12 +198,8 @@ int NaClAppCtor(struct NaClApp  *nap) {
     NaClLog(LOG_INFO, "DANGER: ENABLED FILE ACCESS\n");
   }
 
-  if (!NaClSyncQueueCtor(&nap->work_queue)) {
-    goto cleanup_cv;
-  }
-
   if (!NaClMutexCtor(&nap->threads_mu)) {
-    goto cleanup_work_queue;
+    goto cleanup_name_service;
   }
   if (!NaClCondVarCtor(&nap->threads_cv)) {
     goto cleanup_threads_mu;
@@ -209,15 +230,13 @@ int NaClAppCtor(struct NaClApp  *nap) {
   NaClCondVarDtor(&nap->threads_cv);
  cleanup_threads_mu:
   NaClMutexDtor(&nap->threads_mu);
- cleanup_work_queue:
-  NaClSyncQueueDtor(&nap->work_queue);
+ cleanup_name_service:
+  NaClDescUnref(nap->name_service_conn_cap);
+  NaClRefCountUnref((struct NaClRefCount *) nap->name_service);
  cleanup_cv:
   NaClCondVarDtor(&nap->cv);
  cleanup_mu:
   NaClMutexDtor(&nap->mu);
- cleanup_name_service:
-  NaClDescUnref(nap->name_service_conn_cap);
-  NaClRefCountUnref((struct NaClRefCount *) nap->name_service);
  cleanup_dynamic_load_mutex:
   NaClMutexDtor(&nap->dynamic_load_mutex);
  cleanup_effp_dtor:
@@ -234,6 +253,10 @@ int NaClAppCtor(struct NaClApp  *nap) {
   return 0;
 }
 
+int NaClAppCtor(struct NaClApp *nap) {
+  return NaClAppWithSyscallTableCtor(nap, nacl_syscall);
+}
+
 size_t  NaClAlignPad(size_t val, size_t align) {
   /* align is always a power of 2, but we do not depend on it */
   if (!align) {
@@ -386,6 +409,11 @@ void  NaClLoadTrampoline(struct NaClApp *nap) {
   if (!NaClMakePcrelThunk(nap)) {
     NaClLog(LOG_FATAL, "NaClMakePcrelThunk failed!\n");
   }
+#endif
+#if NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 && NACL_BUILD_SUBARCH == 64
+  if (!NaClMakeDispatchThunk(nap)) {
+    NaClLog(LOG_FATAL, "NaClMakeDispatchThunk failed!\n");
+  }
 #endif
   NaClFillTrampolineRegion(nap);
 
@@ -402,11 +430,26 @@ void  NaClLoadTrampoline(struct NaClApp *nap) {
 
   NaClLog(2, "num_syscalls = %d (0x%x)\n", num_syscalls, num_syscalls);
 
+#if defined(NACL_TARGET_ARM_THUMB2_MODE)
+  CHECK(0 != ((nap->user_entry_pt | nap->initial_entry_pt) & 0x1));
+  /*
+   * Thumb trampolines start 2 bytes before the aligned syscall address used
+   * by ordinary ARM.  We initialize this by adding 0xe to the start address
+   * of each trampoline.  Because the last start address would actually start
+   * into user code above, this allows one fewer trampolines than in ARM.
+   */
+  for (i = 0, addr = nap->mem_start + NACL_SYSCALL_START_ADDR + 0xe;
+       i < num_syscalls - 1;
+       ++i, addr += NACL_SYSCALL_BLOCK_SIZE) {
+    NaClPatchOneTrampoline(nap, addr);
+  }
+#else
   for (i = 0, addr = nap->mem_start + NACL_SYSCALL_START_ADDR;
        i < num_syscalls;
        ++i, addr += NACL_SYSCALL_BLOCK_SIZE) {
     NaClPatchOneTrampoline(nap, addr);
   }
+#endif
 }
 
 void  NaClMemRegionPrinter(void                   *state,
@@ -434,7 +477,6 @@ void  NaClAppPrintDetails(struct NaClApp  *nap,
           "(struct Gio *) 0x%08"NACL_PRIxPTR")\n", (uintptr_t) nap,
           (uintptr_t) gp);
   gprintf(gp, "addr space size:  2**%"NACL_PRId32"\n", nap->addr_bits);
-  gprintf(gp, "max data alloc:   0x%08"NACL_PRIx32"\n", nap->max_data_alloc);
   gprintf(gp, "stack size:       0x%08"NACL_PRIx32"\n", nap->stack_size);
 
   gprintf(gp, "mem start addr:   0x%08"NACL_PRIxPTR"\n", nap->mem_start);
@@ -908,9 +950,14 @@ static void NaClLoadModuleRpc(struct NaClSrpcRpc      *rpc,
   NaClXMutexLock(&nap->mu);
 
   if (LOAD_STATUS_UNKNOWN != nap->module_load_status) {
-    NaClLog(LOG_ERROR, "Repeated LoadModule RPC?!?\n");
-    suberr = LOAD_DUP_LOAD_MODULE;
-    nap->module_load_status = suberr;
+    NaClLog(LOG_ERROR, "Repeated LoadModule RPC, or platform qual error?!?\n");
+    if (LOAD_OK == nap->module_load_status) {
+      NaClLog(LOG_ERROR, "LoadModule when module_load_status is LOAD_OK?!?\n");
+      suberr = LOAD_DUP_LOAD_MODULE;
+      nap->module_load_status = suberr;
+    } else {
+      suberr = nap->module_load_status;
+    }
     rpc->result = NACL_SRPC_RESULT_OK;
     NaClXCondVarBroadcast(&nap->cv);
     goto cleanup_status_mu;
@@ -919,7 +966,8 @@ static void NaClLoadModuleRpc(struct NaClSrpcRpc      *rpc,
   free(nap->aux_info);
   nap->aux_info = aux;
 
-  suberr = NaClAppLoadFile(load_src, nap, NACL_ABI_CHECK_OPTION_CHECK);
+  suberr = NACL_FI_VAL("load_module", NaClErrorCode,
+                       NaClAppLoadFile(load_src, nap));
   (*NACL_VTBL(Gio, load_src)->Close)(load_src);
   (*NACL_VTBL(Gio, load_src)->Dtor)(load_src);
 
@@ -1023,6 +1071,19 @@ static void NaClSecureChannelStartModuleRpc(struct NaClSrpcRpc     *rpc,
 
   NaClLog(4, "NaClSecureChannelStartModuleRpc: load status %d\n", status);
 
+  out_args[0]->u.ival = status;
+  rpc->result = NACL_SRPC_RESULT_OK;
+
+  NaClLog(4, "NaClSecureChannelStartModuleRpc running closure\n");
+  (*done->Run)(done);
+  /*
+   * The RPC reply is now sent.  This has to occur before we signal
+   * the main thread to possibly start, since in the case of a failure
+   * the main thread may quickly exit.  If the main thread does this
+   * before we sent the RPC reply, then the plugin will be left
+   * without an answer.
+   */
+
   NaClXMutexLock(&nap->mu);
   if (nap->module_may_start) {
     NaClLog(LOG_ERROR, "Duplicate StartModule RPC?!?\n");
@@ -1034,12 +1095,6 @@ static void NaClSecureChannelStartModuleRpc(struct NaClSrpcRpc     *rpc,
   NaClXCondVarBroadcast(&nap->cv);
   NaClXMutexUnlock(&nap->mu);
 
-  out_args[0]->u.ival = status;
-  NaClLog(4, "NaClSecureChannelStartModuleRpc finished\n");
-  rpc->result = NACL_SRPC_RESULT_OK;
-
-  NaClLog(4, "NaClSecureChannelStartModuleRpc running closure\n");
-  (*done->Run)(done);
   NaClLog(4, "NaClSecureChannelStartModuleRpc exiting\n");
 }
 
@@ -1117,6 +1172,9 @@ struct NaClSimpleRevClientVtbl const kNaClSecureReverseClientVtbl;
 int NaClSecureServiceCtor(struct NaClSecureService          *self,
                           struct NaClSrpcHandlerDesc const  *srpc_handlers,
                           struct NaClApp                    *nap) {
+  NaClLog(4,
+          "Entered NaClSecureServiceCtor: self 0x%"NACL_PRIxPTR"\n",
+          (uintptr_t) self);
   if (!NaClSimpleServiceWithSocketCtor(&self->base,
                                        srpc_handlers,
                                        NaClThreadInterfaceThreadFactory,
@@ -1142,14 +1200,6 @@ void NaClSecureServiceDtor(struct NaClRefCount *vself) {
   (*NACL_VTBL(NaClRefCount, self)->Dtor)(vself);
 }
 
-/* fwd */
-int NaClSecureReverseClientCtor(
-    struct NaClSecureReverseClient *self,
-    void                            (*client_callback)(
-        void *, struct NaClThreadInterface *, struct NaClDesc *),
-    void                            *state,
-    struct NaClApp                  *nap);
-
 /*
  * The first connection is performed by this callback handler.  This
  * spawns a client thread that will bootstrap the other connections by
@@ -1164,7 +1214,7 @@ int NaClSecureReverseClientCtor(
 static void NaClSecureReverseClientCallback(
     void                        *state,
     struct NaClThreadInterface  *tif,
-    struct NaClDesc             *conn) {
+    struct NaClDesc             *new_conn) {
   struct NaClSecureReverseClient *self =
       (struct NaClSecureReverseClient *) state;
   struct NaClApp *nap = self->nap;
@@ -1177,13 +1227,21 @@ static void NaClSecureReverseClientCallback(
   NaClLog(4, " nap=0x%"NACL_PRIxPTR"\n", (uintptr_t) nap);
   NaClXMutexLock(&nap->mu);
 
-  if (nap->reverse_channel_initialized) {
+  if (NACL_REVERSE_CHANNEL_INITIALIZATION_STARTED !=
+      nap->reverse_channel_initialization_state) {
+    /*
+     * The reverse channel connection capability is used to make the
+     * RPC that invokes this callback (this callback is invoked on a
+     * reverse channel connect), so the plugin wants to initialize the
+     * reverse channel and in particular the state must be either be
+     * in-progress or finished.
+     */
     NaClLog(LOG_FATAL, "Reverse channel already initialized\n");
   }
-  if (!NaClSrpcClientCtor(&nap->reverse_channel, conn)) {
+  if (!NaClSrpcClientCtor(&nap->reverse_channel, new_conn)) {
     NaClLog(LOG_FATAL, "Reverse channel SRPC Client Ctor failed\n");
   }
-  nap->reverse_channel_initialized = 1;
+  nap->reverse_channel_initialization_state = NACL_REVERSE_CHANNEL_INITIALIZED;
 
   NaClXCondVarBroadcast(&nap->cv);
   NaClXMutexUnlock(&nap->mu);
@@ -1191,6 +1249,14 @@ static void NaClSecureReverseClientCallback(
   NaClLog(4, "Leaving NaClSecureReverseClientCallback\n");
 }
 
+/* fwd */
+int NaClSecureReverseClientCtor(
+    struct NaClSecureReverseClient *self,
+    void                            (*client_callback)(
+        void *, struct NaClThreadInterface *, struct NaClDesc *),
+    void                            *state,
+    struct NaClApp                  *nap);
+
 static void NaClSecureReverseClientSetup(struct NaClSrpcRpc     *rpc,
                                          struct NaClSrpcArg     **in_args,
                                          struct NaClSrpcArg     **out_args,
@@ -1201,9 +1267,19 @@ static void NaClSecureReverseClientSetup(struct NaClSrpcRpc     *rpc,
 
   UNREFERENCED_PARAMETER(in_args);
   NaClLog(4, "Entered NaClSecureReverseClientSetup\n");
+
+  NaClXMutexLock(&nap->mu);
   if (NULL != nap->reverse_client) {
     NaClLog(LOG_FATAL, "Double reverse setup RPC\n");
   }
+  if (NACL_REVERSE_CHANNEL_UNINITIALIZED !=
+      nap->reverse_channel_initialization_state) {
+    NaClLog(LOG_FATAL,
+            "Reverse channel initialization state not uninitialized\n");
+  }
+  nap->reverse_channel_initialization_state =
+      NACL_REVERSE_CHANNEL_INITIALIZATION_STARTED;
+  /* the reverse connection is still coming */
   rev = (struct NaClSecureReverseClient *) malloc(sizeof *rev);
   if (NULL == rev) {
     rpc->result = NACL_SRPC_RESULT_APP_ERROR;
@@ -1220,15 +1296,20 @@ static void NaClSecureReverseClientSetup(struct NaClSrpcRpc     *rpc,
     rpc->result = NACL_SRPC_RESULT_APP_ERROR;
     goto done;
   }
-  nap->reverse_client = rev;
+  nap->reverse_client = (struct NaClSecureReverseClient *) NaClRefCountRef(
+      (struct NaClRefCount *) rev);
   out_args[0]->u.hval = NaClDescRef(rev->base.bound_and_cap[1]);
   rpc->result = NACL_SRPC_RESULT_OK;
 
+  /*
+   * Service thread takes the reference rev.
+   */
   if (!NaClSimpleRevClientStartServiceThread(&rev->base)) {
     NaClLog(LOG_FATAL, "Could not start reverse service thread\n");
   }
 
 done:
+  NaClXMutexUnlock(&nap->mu);
   (*done->Run)(done);
   NaClLog(4, "Leaving NaClSecureReverseClientSetup\n");
 }
@@ -1278,17 +1359,18 @@ int NaClSecureReverseClientInsertHandler_mu(
  * server side spawn threads that asynchronously connect twice, in the
  * "incorrect" order, etc.
  */
-static
 int NaClSecureReverseClientInsertHandler(
     struct NaClSecureReverseClient *self,
-    void                            (*h)(void *,
-                                         struct NaClThreadInterface *,
-                                         struct NaClDesc *),
-    void                            *d) {
+    void                            (*handler)(
+        void                        *handlr_state,
+        struct NaClThreadInterface  *thread_if,
+        struct NaClDesc             *new_conn),
+    void                            *handler_state) {
   int retval;
 
   NaClXMutexLock(&self->mu);
-  retval = NaClSecureReverseClientInsertHandler_mu(self, h, d);
+  retval = NaClSecureReverseClientInsertHandler_mu(self,
+                                                   handler, handler_state);
   NaClXMutexUnlock(&self->mu);
   return retval;
 }
@@ -1433,7 +1515,7 @@ int NaClSecureServiceConnectionFactory(
 
   /* our instance_data is not connection specific */
   return NaClSimpleServiceConnectionFactoryWithInstanceData(
-      vself, conn, (void*) self->nap, out);
+      vself, conn, (void *) self->nap, out);
 }
 
 int NaClSecureServiceAcceptAndSpawnHandler(struct NaClSimpleService *vself) {
@@ -1481,39 +1563,43 @@ struct NaClSimpleRevClientVtbl const kNaClSecureReverseClientVtbl = {
 };
 
 
-static void WINAPI ReverseTestThread(void *thread_state) {
+static void WINAPI ReverseSetupThread(void *thread_state) {
   struct NaClApp *nap = (struct NaClApp *) thread_state;
   NaClSrpcError rpc_result;
 
   /* wait for reverse connection */
-  NaClLog(1, "ReverseTestThread\n");
+  NaClLog(1, "ReverseSetupThread\n");
   NaClXMutexLock(&nap->mu);
-  while (!nap->reverse_channel_initialized) {
-    NaClLog(1, "ReverseTestThread waiting\n");
+  while (NACL_REVERSE_CHANNEL_INITIALIZED !=
+         nap->reverse_channel_initialization_state) {
+    NaClLog(1, "ReverseSetupThread waiting\n");
     NaClXCondVarWait(&nap->cv, &nap->mu);
   }
-  NaClLog(1, "ReverseTestThread initialized, making rpc\n");
+  NaClLog(1, "ReverseSetupThread initialized, making rpc\n");
   rpc_result = NaClSrpcInvokeBySignature(&nap->reverse_channel,
-                                         "test:s:", "Hello world");
+                                         NACL_REVERSE_CONTROL_TEST,
+                                         "Hello world");
   if (NACL_SRPC_RESULT_OK != rpc_result) {
     NaClLog(LOG_ERROR,
-            "ReverseTestThread, test:s: rpc_result %d\n",
+            "ReverseSetupThread, test:s: rpc_result %d\n",
             rpc_result);
   }
   NaClLog(4,
           "Hello world should be logged browser side, rpc_result %d.\n",
           rpc_result);
   rpc_result = NaClSrpcInvokeBySignature(&nap->reverse_channel,
-                                         "revlog:s:",
+                                         NACL_REVERSE_CONTROL_LOG,
                                          ("Log message that should show up"
                                           " on the JavaScript console"));
   NaClLog(4, "revlog returned %d.\n", rpc_result);
   if (NACL_SRPC_RESULT_OK != rpc_result) {
     NaClLog(LOG_ERROR,
-            "ReverseTestThread, revlog:s: rpc_result %d\n",
+            "ReverseSetupThread, revlog:s: rpc_result %d\n",
             rpc_result);
   }
   NaClXMutexUnlock(&nap->mu);
+
+  NaClThreadDtor(&nap->reverse_setup_thread);
 }
 
 
@@ -1552,16 +1638,47 @@ void NaClSecureCommandChannel(struct NaClApp *nap) {
             "Could not start secure command channel service thread\n");
   }
 
-  if (1) {
-    struct NaClThread thr;
-
-    NaClThreadCtor(&thr, ReverseTestThread, nap, NACL_KERN_STACK_SIZE);
+  if (!NaClThreadCtor(&nap->reverse_setup_thread, ReverseSetupThread, nap,
+                      NACL_KERN_STACK_SIZE)) {
+    NaClLog(LOG_FATAL,
+            "Could not start reverse setup thread\n");
   }
 
   NaClLog(4, "Leaving NaClSecureCommandChannel\n");
 }
 
 
+void NaClVmHoleWaitToStartThread(struct NaClApp *nap) {
+  NaClXMutexLock(&nap->mu);
+
+  /* ensure no virtual memory hole may appear */
+  while (nap->vm_hole_may_exist) {
+    NaClXCondVarWait(&nap->cv, &nap->mu);
+  }
+
+  ++nap->threads_launching;
+  NaClXMutexUnlock(&nap->mu);
+  /*
+   * NB: Dropped lock, so many threads launching can starve VM
+   * operations.  If this becomes a problem in practice, we can use a
+   * reader/writer lock so that a waiting writer will block new
+   * readers.
+   */
+}
+
+void NaClVmHoleThreadStackIsSafe(struct NaClApp *nap) {
+  NaClXMutexLock(&nap->mu);
+
+  if (0 == --nap->threads_launching) {
+    /*
+     * Wake up the threads waiting to do VM operations.
+     */
+    NaClXCondVarBroadcast(&nap->cv);
+  }
+
+  NaClXMutexUnlock(&nap->mu);
+}
+
 #ifdef __GNUC__
 
 /*
