@@ -41,42 +41,30 @@
 #include "native_client/src/trusted/service_runtime/dyn_array.h"
 #include "native_client/src/trusted/service_runtime/nacl_config_dangerous.h"
 #include "native_client/src/trusted/service_runtime/nacl_error_code.h"
-#include "native_client/src/trusted/service_runtime/nacl_sync_queue.h"
+#include "native_client/src/trusted/service_runtime/nacl_kern_services.h"
+
 #include "native_client/src/trusted/service_runtime/sel_mem.h"
 #include "native_client/src/trusted/service_runtime/sel_util.h"
 #include "native_client/src/trusted/service_runtime/sel_rt.h"
 
 #include "native_client/src/trusted/service_runtime/name_service/name_service.h"
 
-
 EXTERN_C_BEGIN
 
 #define NACL_SERVICE_PORT_DESCRIPTOR    3
 #define NACL_SERVICE_ADDRESS_DESCRIPTOR 4
 
-#define NACL_DEFAULT_ALLOC_MAX  (32 << 20)  /* total brk and mmap allocs */
 #define NACL_DEFAULT_STACK_MAX  (16 << 20)  /* main thread stack */
 
 #define NACL_SANDBOX_CHROOT_FD  "SBX_D"
 
-#if NACL_WINDOWS
-#define WINDOWS_EXCEPTION_TRY do { __try {
-#define WINDOWS_EXCEPTION_CATCH } __except(EXCEPTION_EXECUTE_HANDLER) { \
-                                  NaClLog(LOG_ERROR, \
-                                      "Unhandled Windows exception\n"); \
-                                  exit(1); \
-                                } \
-                              } while (0)
-#else
-#define WINDOWS_EXCEPTION_TRY do {
-#define WINDOWS_EXCEPTION_CATCH } while (0)
-#endif
-
 struct NaClAppThread;
 struct NaClDesc;  /* see native_client/src/trusted/desc/nacl_desc_base.h */
 struct NaClDynamicRegion;
+struct NaClManifestProxy;
 struct NaClSecureService;
 struct NaClSecureReverseService;
+struct NaClThreadInterface;  /* see sel_ldr_thread_interface.h */
 
 struct NaClDebugCallbacks {
   void (*thread_create_hook)(struct NaClAppThread *natp);
@@ -89,17 +77,8 @@ struct NaClApp {
    * public, user settable prior to app start.
    */
   uint8_t                   addr_bits;
-  uintptr_t                 max_data_alloc;
   uintptr_t                 stack_size;
   /*
-   * max_data_alloc controls how much total data memory can be
-   * allocated to the NaCl process; this is initialized data,
-   * uninitialized data, and heap and affects the brk system call.
-   * the text size and rodata size are not included, even though in
-   * NaCl the text and rodata pages are also backed by the pager
-   * since due to relocation the text pages and rodata contents
-   * cannot simply be memory mapped from the executable.
-   *
    * stack_size is the maximum size of the (main) stack.  The stack
    * memory is eager allocated (mapped in w/o MAP_NORESERVE) so
    * there must be enough swap space; page table entries are not
@@ -125,6 +104,10 @@ struct NaClApp {
      && NACL_BUILD_SUBARCH == 32 && __PIC__)
   uintptr_t                 pcrel_thunk;
 #endif
+#if (NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 \
+     && NACL_BUILD_SUBARCH == 64)
+  uintptr_t                 dispatch_thunk;
+#endif
 
   /* only used for ET_EXEC:  for CS restriction */
   uintptr_t                 static_text_end;  /* relative to mem_start */
@@ -189,9 +172,6 @@ struct NaClApp {
   struct NaClDesc           *service_port;
   struct NaClDesc           *service_address;
 
-  struct NaClNameService    *name_service;  /* default name server */
-  struct NaClDesc           *name_service_conn_cap;
-
   struct NaClMutex          mu;
   struct NaClCondVar        cv;
 
@@ -210,10 +190,33 @@ struct NaClApp {
   int                       vm_hole_may_exist;
   int                       threads_launching;
 
+  /*
+   * An array of NaCl syscall handlers. The length of the array must be
+   * at least NACL_MAX_SYSCALLS.
+   */
+  struct NaClSyscallTableEntry *syscall_table;
+
+  /*
+   * Name service must launch after mu, cv, vm_hole_may_exit,
+   * threads_launching are initialized.
+   */
+  struct NaClNameService    *name_service;  /* default name server */
+  struct NaClDesc           *name_service_conn_cap;
+
   struct NaClSecureService        *secure_service;
+  struct NaClThread               reverse_setup_thread;
+  /* used only during setup, thread dtors before exiting */
+  struct NaClManifestProxy        *manifest_proxy;
+  struct NaClKernService          *kern_service;
+
   struct NaClSecureReverseClient  *reverse_client;
-  int                             reverse_channel_initialized;
-  NaClSrpcChannel                 reverse_channel;
+  enum NaClReverseChannelInitializationState {
+    NACL_REVERSE_CHANNEL_UNINITIALIZED,
+    NACL_REVERSE_CHANNEL_INITIALIZATION_STARTED,
+    NACL_REVERSE_CHANNEL_INITIALIZED
+  }                               reverse_channel_initialization_state;
+  struct NaClSrpcChannel          reverse_channel;
+
 
   NaClErrorCode             module_load_status;
   int                       module_may_start;
@@ -284,8 +287,6 @@ struct NaClApp {
   int                       ignore_validator_result;
   int                       skip_validator;
   int                       validator_stub_out_mode;
-  /* all threads enqueue the "special" syscalls to the work queue */
-  struct NaClSyncQueue      work_queue;
 
 #if NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 && NACL_BUILD_SUBARCH == 32
   uint16_t                  code_seg_sel;
@@ -316,6 +317,21 @@ struct NaClApp {
 
 void  NaClAppIncrVerbosity(void);
 
+/*
+ * Initializes a NaCl application with the default parameters
+ * and the specified syscall table.
+ *
+ * nap is a pointer to the NaCl object that is being filled in.
+ *
+ * table is the NaCl syscall table. The syscall table must contain at least
+ * NACL_MAX_SYSCALLS valid entries.
+ *
+ * Caution! Syscall handlers must be extremely careful with respect to
+ * argument validation, including time-of-check vs time-of-use defense, etc.
+ */
+int NaClAppWithSyscallTableCtor(struct NaClApp               *nap,
+                                struct NaClSyscallTableEntry *table) NACL_WUR;
+
 int   NaClAppCtor(struct NaClApp  *nap) NACL_WUR;
 
 /*
@@ -341,14 +357,8 @@ int   NaClAppCtor(struct NaClApp  *nap) NACL_WUR;
  * self-modifying code / data writes and automatically invalidate the
  * cache lines.
  */
-enum NaClAbiCheckOption {
-  NACL_ABI_CHECK_OPTION_SKIP,
-  NACL_ABI_CHECK_OPTION_CHECK
-};
-
 NaClErrorCode NaClAppLoadFile(struct Gio      *gp,
-                              struct NaClApp  *nap,
-                              enum NaClAbiCheckOption check_abi) NACL_WUR;
+                              struct NaClApp  *nap) NACL_WUR;
 
 NaClErrorCode NaClAppLoadFileDynamically(struct NaClApp *nap,
                                          struct Gio     *gio_file) NACL_WUR;
@@ -405,6 +415,17 @@ void NaClAddImcHandle(struct NaClApp  *nap,
                       NaClHandle      h,
                       int             nacl_desc);
 
+/*
+ * Launch system-level service threads.  After this, access to the
+ * NaClApp object must be done in a thread-safe manner, using nap->mu
+ * etc, or access only read-only data.
+ *
+ * NB: the "secure command channel" thread should have already started
+ * (if enabled); that thread must take care to not race with the main
+ * thread that is continuing to set up the NaCl module as well.
+ */
+int NaClAppLaunchServiceThreads(struct NaClApp *nap);
+
 /*
  * Used to launch the main thread.  NB: calling thread may in the
  * future become the main NaCl app thread, and this function will
@@ -427,7 +448,8 @@ int NaClWaitForMainThreadToExit(struct NaClApp  *nap);
  */
 void NaClSetIrtFileDesc(int fd);
 
-int NaClMainForChromium(int handle_count, const NaClHandle *handles, int debug);
+void NaClMainForChromium(int handle_count, const NaClHandle *handles,
+                         int debug);
 
 /*
  * Used by syscall code.
@@ -435,8 +457,8 @@ int NaClMainForChromium(int handle_count, const NaClHandle *handles, int debug);
 int32_t NaClCreateAdditionalThread(struct NaClApp *nap,
                                    uintptr_t      prog_ctr,
                                    uintptr_t      stack_ptr,
-                                   uintptr_t      sys_tdb,
-                                   size_t         tdb_size) NACL_WUR;
+                                   uintptr_t      sys_tls,
+                                   uint32_t       user_tls2) NACL_WUR;
 
 void NaClLoadTrampoline(struct NaClApp *nap);
 
@@ -519,6 +541,14 @@ void NaClSendServiceAddressTo(struct NaClApp  *nap,
 
 void NaClSecureCommandChannel(struct NaClApp  *nap);
 
+int NaClSecureReverseClientInsertHandler(
+    struct NaClSecureReverseClient  *self,
+    void                            (*handler)(
+        void                                   *handler_state,
+        struct NaClThreadInterface             *thread_if,
+        struct NaClDesc                        *new_conn),
+    void                            *handler_state) NACL_WUR;
+
 NaClErrorCode NaClWaitForLoadModuleStatus(struct NaClApp *nap) NACL_WUR;
 
 NaClErrorCode NaClWaitForStartModuleCommand(struct NaClApp *nap) NACL_WUR;
@@ -536,6 +566,13 @@ int NaClMakePcrelThunk(struct NaClApp *nap);
 
 #endif
 
+#if (NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 \
+     && NACL_BUILD_SUBARCH == 64)
+
+int NaClMakeDispatchThunk(struct NaClApp *nap);
+
+#endif
+
 void NaClPatchOneTrampoline(struct NaClApp *nap,
                             uintptr_t target_addr);
 /*
@@ -585,23 +622,9 @@ int NaClThreadContextCtor(struct NaClThreadContext  *ntcp,
 
 void NaClThreadContextDtor(struct NaClThreadContext *ntcp);
 
+void NaClVmHoleWaitToStartThread(struct NaClApp *nap);
 
-/*
- * Used in mmap code to prepare the untrusted module for mmap.
- * Specifically, on *x this is a no-op, but on Windows where mmap does
- * not atomically replace mappings and we have to unmap (VirtualFree
- * or UnmapViewOfFile) before installing new mappings, we stop all
- * other untrusted threads using SuspendThread in
- * NaClAppBeginMapRaceProtection and then ResumeThread in
- * NaClAppEndMapRaceProtection.  These functions must be called after
- * all the necessary locks for the bracketed intervening operations,
- * since no other thread that might otherwise release locks is able to
- * run.
- */
-void NaClAppBeginMapRaceProtection_AppMu(struct NaClAppThread *natp);
-
-void NaClAppEndMapRaceProtection_AppMu(struct NaClAppThread *natp);
-
+void NaClVmHoleThreadStackIsSafe(struct NaClApp *nap);
 
 void NaClGdbHook(struct NaClApp const *nap);
 
