@@ -34,6 +34,7 @@
 #include "native_client/src/shared/platform/nacl_sync_checked.h"
 #include "native_client/src/shared/srpc/nacl_srpc.h"
 
+#include "native_client/src/trusted/fault_injection/fault_injection.h"
 #include "native_client/src/trusted/perf_counter/nacl_perf_counter.h"
 #include "native_client/src/trusted/service_runtime/env_cleanser.h"
 #include "native_client/src/trusted/service_runtime/include/sys/fcntl.h"
@@ -47,6 +48,7 @@
 #include "native_client/src/trusted/service_runtime/outer_sandbox.h"
 #include "native_client/src/trusted/service_runtime/sel_ldr.h"
 #include "native_client/src/trusted/service_runtime/sel_qualify.h"
+#include "native_client/src/trusted/service_runtime/win/exception_patch/ntdll_patch.h"
 
 static void VmentryPrinter(void           *state,
                     struct NaClVmmapEntry *vmep) {
@@ -100,17 +102,6 @@ int ImportModeMap(char opt) {
   /* NOTREACHED */
 }
 
-int NaClDup2(int d_old, int d_new) {
-  if (d_old == d_new) {
-    return 0;
-  }
-  if (DUP2(d_old, d_new) == -1) {
-    fprintf(stderr, "sel_ldr: dup2 for log file failed\n");
-    exit(1);
-  }
-  return 1;
-}
-
 
 static void PrintUsage() {
   /* NOTE: this is broken up into multiple statements to work around
@@ -145,9 +136,8 @@ static void PrintUsage() {
           " -c ignore validator! dangerous! Repeating this option twice skips\n"
           "    validation completely.\n"
           " -F fuzz testing; quit after loading NaCl app\n"
-          " -S enable signal handling.\n"
-          " -g enable gdb debug stub.\n"
-          " -I disable ELF ABI version number check (safe)\n"
+          " -S enable signal handling.  Not secure on x86-64 Windows.\n"
+          " -g enable gdb debug stub.  Not secure on x86-64 Windows.\n"
           " -l <file>  write log output to the given file\n"
           " -s safely stub out non-validating instructions\n"
           " -Q disable platform qualification (dangerous!)\n"
@@ -155,11 +145,6 @@ static void PrintUsage() {
           );  /* easier to add new flags/lines */
 }
 
-/*
- * NB: do NOT return from main until our dependency on SDL is removed.
- * This is because the OSX version of SDL ignores main's return value
- * and always exits with an exit status of 0, breaking tests.
- */
 int main(int  argc,
          char **argv) {
   int                           opt;
@@ -174,7 +159,6 @@ int main(int  argc,
   char                          *blob_library_file = NULL;
   int                           rpc_supplies_nexe = 0;
   int                           export_addr_to = -2;
-  enum NaClAbiCheckOption       check_abi = NACL_ABI_CHECK_OPTION_CHECK;
 
   struct NaClApp                *nap;
 
@@ -239,12 +223,6 @@ int main(int  argc,
   redir_queue = NULL;
   redir_qend = &redir_queue;
 
-  /*
-   * Set an exception handler so Windows won't show a message box if
-   * an exception occurs
-   */
-  WINDOWS_EXCEPTION_TRY;
-
   NaClAllModulesInit();
 
   verbosity = NaClLogGetVerbosity();
@@ -336,9 +314,6 @@ int main(int  argc,
         *redir_qend = entry;
         redir_qend = &entry->next;
         break;
-      case 'I':
-        check_abi = NACL_ABI_CHECK_OPTION_SKIP;
-        break;
       case 'l':
         log_file = optarg;
         break;
@@ -527,7 +502,8 @@ int main(int  argc,
   /* We use the signal handler to verify a signal took place. */
   NaClSignalHandlerInit();
   if (!skip_qualification) {
-    NaClErrorCode pq_error = NaClRunSelQualificationTests();
+    NaClErrorCode pq_error = NACL_FI_VAL("pq", NaClErrorCode,
+                                         NaClRunSelQualificationTests());
     if (LOAD_OK != pq_error) {
       errcode = pq_error;
       nap->module_load_status = pq_error;
@@ -543,6 +519,16 @@ int main(int  argc,
     NaClSignalHandlerFini();
     /* Sanity check. */
     NaClSignalAssertNoHandlers();
+
+    /*
+     * Patch the Windows exception dispatcher to be safe in the case
+     * of faults inside x86-64 sandboxed code.  The sandbox is not
+     * secure on 64-bit Windows without this.
+     */
+#if (NACL_WINDOWS && NACL_ARCH(NACL_BUILD_ARCH) == NACL_x86 && \
+     NACL_BUILD_SUBARCH == 64)
+    NaClPatchWindowsExceptionDispatcher();
+#endif
   }
 
   /*
@@ -573,7 +559,8 @@ int main(int  argc,
     NaClPerfCounterIntervalLast(&time_all_main);
 
     if (LOAD_OK == errcode) {
-      errcode = NaClAppLoadFile((struct Gio *) &main_file, nap, check_abi);
+      NaClLog(2, "Loading nacl file %s (non-RPC)\n", nacl_file);
+      errcode = NaClAppLoadFile((struct Gio *) &main_file, nap);
       if (LOAD_OK != errcode) {
         fprintf(stderr, "Error while loading \"%s\": %s\n",
                 nacl_file,
@@ -700,6 +687,7 @@ int main(int  argc,
 
   if (NULL != blob_library_file) {
     if (LOAD_OK == errcode) {
+      NaClLog(2, "Loading blob file %s\n", blob_library_file);
       errcode = NaClAppLoadFileDynamically(nap, (struct Gio *) &blob_file);
       if (LOAD_OK != errcode) {
         fprintf(stderr, "Error while loading \"%s\": %s\n",
@@ -715,6 +703,10 @@ int main(int  argc,
       fprintf(stderr, "Error while closing \"%s\".\n", blob_library_file);
     }
     (*((struct Gio *) &blob_file)->vtbl->Dtor)((struct Gio *) &blob_file);
+    if (verbosity) {
+      gprintf((struct Gio *) &gout, "printing post-IRT NaClApp details\n");
+      NaClAppPrintDetails(nap, (struct Gio *) &gout);
+    }
   }
 
   /*
@@ -787,13 +779,17 @@ int main(int  argc,
    */
   fflush((FILE *) NULL);
 
-  if (NULL != nap->secure_service && LOAD_OK == errcode) {
+  if (NULL != nap->secure_service) {
+    NaClErrorCode start_result;
     /*
      * wait for start_module RPC call on secure channel thread.
      */
-    errcode = NaClWaitForStartModuleCommand(nap);
+    start_result = NaClWaitForStartModuleCommand(nap);
     NaClPerfCounterMark(&time_all_main, "WaitedForStartModuleCommand");
     NaClPerfCounterIntervalLast(&time_all_main);
+    if (LOAD_OK == errcode) {
+      errcode = start_result;
+    }
   }
 
   /*
@@ -817,10 +813,10 @@ int main(int  argc,
     NaClLog(LOG_FATAL, "Failed to initialise env cleanser\n");
   }
 
-  /*
-   * only nap->ehdrs.e_entry is usable, no symbol table is
-   * available.
-   */
+  if (!NaClAppLaunchServiceThreads(nap)) {
+    fprintf(stderr, "Launch service threads failed\n");
+    goto done;
+  }
   if (!NaClCreateMainThread(nap,
                             argc - optind,
                             argv + optind,
@@ -870,8 +866,6 @@ int main(int  argc,
   if (handle_signals) NaClSignalHandlerFini();
   NaClAllModulesFini();
 
-  WINDOWS_EXCEPTION_CATCH;
-
   NaClExit(ret_code);
 
   /* Unreachable, but having the return prevents a compiler error. */
