@@ -1,4 +1,5 @@
-/* Copyright (c) 2009 The Native Client Authors. All rights reserved.
+/*
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
  * Use of this source code is governed by a BSD-style license that can be
  * found in the LICENSE file.
  */
@@ -26,6 +27,7 @@
 #include "native_client/src/trusted/validator_x86/lock_insts.h"
 #include "native_client/src/trusted/validator_x86/long_mode.h"
 #include "native_client/src/trusted/validator_x86/nacl_illegal.h"
+#include "native_client/src/trusted/validator_x86/nc_def_jumps.h"
 #include "native_client/src/trusted/validator_x86/nc_rep_prefix.h"
 #include "native_client/src/trusted/validator_x86/ncdecode_st.h"
 #include "native_client/src/trusted/validator_x86/ncdecode_tablegen.h"
@@ -180,11 +182,19 @@ static NaClOpFlags NaClGetArg2PlusFlags(NaClInstCat icat,
       }
     case Exchange:
       if (op_index == 2) {
+        /* The second argument is always both a set and a use. */
         if (visible_index == 2) {
           return NACL_OPFLAG(OpSet) | NACL_OPFLAG(OpUse) | NACL_OPFLAG(OpDest);
         } else {
           return NACL_OPFLAG(OpSet) | NACL_OPFLAG(OpUse);
         }
+      } else if (op_index == 3) {
+        /* If it has a 3rd argument, it is like a cmpxchg. */
+        if (visible_index == 1) {
+          return NACL_OPFLAG(OpSet) | NACL_OPFLAG(OpUse) | NACL_OPFLAG(OpDest);
+        } else {
+          return NACL_OPFLAG(OpSet) | NACL_OPFLAG(OpUse);
+        }
       } else {
         return NACL_OPFLAG(OpUse);
       }
@@ -258,6 +268,7 @@ static void NaClAddMiscellaneousFlags() {
   NaClAddLongModeIfApplicable();
   NaClAddNaClIllegalIfApplicable();
   NaClAddRepPrefixFlagsIfApplicable();
+  NaClAddJumpFlagsIfApplicable();
   DEBUG(NaClLog(LOG_INFO, "<- Adding Miscellaneous Flags\n"));
 }
 
@@ -269,7 +280,7 @@ static void NaClSetInstCat(NaClInstCat icat) {
   int visible_count = 0;
   int i;
   int num_ops;
-  NaClInst* inst = NaClGetDefInst();
+  NaClModeledInst* inst = NaClGetDefInst();
   num_ops = inst->num_operands;
   for (i = 0; i < num_ops; ++i) {
     Bool is_visible = FALSE;
@@ -317,7 +328,7 @@ static uint8_t SLOpcode(int16_t val) {
  */
 static Bool NaClNameOpcodeSeqMatchesPrefixByte(
     int* prefix_count,
-    NaClInst* inst,
+    NaClModeledInst* inst,
     const NaClNameOpcodeSeq* name_and_opcode_seq,
     int index) {
   int16_t prefix;
@@ -350,7 +361,7 @@ Bool NaClInInstructionSet(const NaClMnemonic* names,
                           const NaClNameOpcodeSeq* name_and_opcode_seq,
                           size_t name_and_opcode_seq_size) {
   size_t i;
-  NaClInst* inst = NaClGetDefInst();
+  NaClModeledInst* inst = NaClGetDefInst();
 
   /* First handle cases where all instances of an instruction
    * mnemonic is in the set.
@@ -453,7 +464,7 @@ static void NaClOperandForm_Eb() {
    * specified yet, we go ahead and assume the effective operand size of the
    * instructruction as being a byte.
    */
-  NaClInst* inst = NaClGetDefInst();
+  NaClModeledInst* inst = NaClGetDefInst();
   if (inst->flags & (NACL_IFLAG(OperandSize_w) | NACL_IFLAG(OperandSize_v) |
                      NACL_IFLAG(OperandSize_o))) {
     NaClDefOp(Eb_Operand, NACL_EMPTY_OPFLAGS);
@@ -553,7 +564,7 @@ static void NaClOperandForm_Gw() {
 
 static void NaClOperandForm_Ib() {
   int i;
-  NaClInst* inst = NaClGetDefInst();
+  NaClModeledInst* inst = NaClGetDefInst();
   /* First look to see if 1st or 2nd instance of an immediate value,
    * since different opcode flags that must be used are different.
    */
