@@ -17,16 +17,13 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include "native_client/src/include/nacl_macros.h"
 #include "native_client/src/include/portability.h"
 #include "native_client/src/shared/gio/gio.h"
 #include "native_client/src/shared/platform/nacl_log.h"
-#include "native_client/src/trusted/validator_x86/ncvalidate.h"
-#include "native_client/src/trusted/validator_x86/ncdis_util.h"
-#include "native_client/src/trusted/validator_x86/ncvalidate_internaltypes.h"
-
-#if !defined(ARRAYSIZE)
-#define ARRAYSIZE(a) sizeof(a)/sizeof(a[0])
-#endif
+#include "native_client/src/trusted/validator/x86/ncval_seg_sfi/ncdecode_verbose.h"
+#include "native_client/src/trusted/validator/x86/ncval_seg_sfi/ncvalidate.h"
+#include "native_client/src/trusted/validator/x86/ncval_seg_sfi/ncvalidate_internaltypes.h"
 
 void Info(const char *fmt, ...)
 {
@@ -52,6 +49,11 @@ struct NCValTestCase {
 };
 
 struct NCValTestCase NCValTests[] = {
+  /* NOTE: Many of these tests are now in the textual testing structure in
+   * native_client/src/trusted/validator_x86/testdata/32 using
+   * files "test-n.hex", "test-n.ndis", "test-n.nvals", and
+   * "test-n.nvals16".
+   */
   {
     "test 1",
     "a first very simple test with an illegal inst.",
@@ -994,6 +996,22 @@ struct NCValTestCase NCValTests[] = {
     /* sawfailure= */ 1, /* illegalinst= */ 1,
     /* instructions= */ 3,
     /* vaddr= */ 0x80000000,
+    /* Note: Fixed so that this is a legal instruction,
+     * except for the prefix! (karl)
+     */
+    "2e 0f 3a 0f bb ab 00 00 00 00 \n"
+    "90 90 \n"
+    "f4 \n"                             /* hlt                             */
+  },
+  {
+    "test 87a",
+    "test 87a: bad NACLi_3BYTE instruction (not really an instruction)",
+    /* sawfailure= */ 1, /* illegalinst= */ 2,
+    /* instructions= */ 2,
+    /* vaddr= */ 0x80000000,
+    /* Note: Fixed so that this is a legal instruction,
+     * except for the prefix! (karl)
+     */
     "2e 0f 3a 7d bb ab 00 00 00 00 \n"
     "90 90 \n"
     "f4 \n"                             /* hlt                             */
@@ -1138,6 +1156,9 @@ static void TestValidator(struct NCValTestCase *vtest) {
   free(byte0);
   rc = NCValidateFinish(vstate);
   do {
+    printf("vtest->sawfailure = %d, vstate->stats.sawfailure = %d\n",
+           vtest->sawfailure, vstate->stats.sawfailure);
+    NCStatsPrint(vstate);
     if (vtest->sawfailure ^ vstate->stats.sawfailure) break;
     if (vtest->instructions != vstate->stats.instructions) break;
     if (vtest->illegalinst != vstate->stats.illegalinst) break;
@@ -1145,7 +1166,7 @@ static void TestValidator(struct NCValTestCase *vtest) {
     NCValidateFreeState(&vstate);
     return;
   } while (0);
-  Stats_Print(vstate);
+  NCStatsPrint(vstate);
   NCValidateFreeState(&vstate);
   Info("*** %s failed (%s)\n", vtest->name, vtest->description);
   exit(-1);
@@ -1170,7 +1191,7 @@ void test_fail_on_bad_alignment() {
 void ncvalidate_unittests() {
   size_t i;
 
-  for (i = 0; i < ARRAYSIZE(NCValTests); i++) {
+  for (i = 0; i < NACL_ARRAY_SIZE(NCValTests); i++) {
     TestValidator(&NCValTests[i]);
   }
 
