@@ -6,11 +6,14 @@
 #ifndef NATIVE_CLIENT_SRC_UNTRUSTED_IRT_IRT_INTERFACES_H_
 #define NATIVE_CLIENT_SRC_UNTRUSTED_IRT_IRT_INTERFACES_H_
 
+#include <stddef.h>
+
 extern size_t nacl_irt_interface(const char *interface_ident,
                                  void *table, size_t tablesize);
 
 extern const struct nacl_irt_basic nacl_irt_basic;
-extern const struct nacl_irt_file nacl_irt_file;
+extern const struct nacl_irt_fdio nacl_irt_fdio;
+extern const struct nacl_irt_filename nacl_irt_filename;
 extern const struct nacl_irt_memory nacl_irt_memory;
 extern const struct nacl_irt_dyncode nacl_irt_dyncode;
 extern const struct nacl_irt_thread nacl_irt_thread;
@@ -20,5 +23,6 @@ extern const struct nacl_irt_sem nacl_irt_sem;
 extern const struct nacl_irt_tls nacl_irt_tls;
 extern const struct nacl_irt_blockhook nacl_irt_blockhook;
 extern const struct nacl_irt_ppapihook nacl_irt_ppapihook;
+extern const struct nacl_irt_resource_open nacl_irt_resource_open;
 
 #endif  /* NATIVE_CLIENT_SRC_UNTRUSTED_IRT_IRT_INTERFACES_H_ */
