@@ -21,6 +21,7 @@
 #include "native_client/src/untrusted/irt/irt_interfaces.h"
 #include "native_client/src/untrusted/nacl/nacl_irt.h"
 #include "native_client/src/untrusted/nacl/tls.h"
+#include "native_client/src/untrusted/nacl/tls_params.h"
 
 #include "native_client/src/untrusted/pthread/nc_hash.h"
 #include "native_client/src/untrusted/pthread/pthread.h"
@@ -38,6 +39,9 @@ static struct nacl_irt_thread irt_thread;
 
 static const uint32_t kStackAlignment = 32;
 
+#define TDB_SIZE \
+  (sizeof(nc_thread_descriptor_t) + sizeof(nc_basic_thread_data_t))
+
 static inline char* align(uint32_t offset, uint32_t alignment) {
   return (char*) ((offset + alignment - 1) & ~(alignment - 1));
 }
@@ -62,10 +66,6 @@ pthread_cond_t __nc_last_thread_cond;
 /* number of threads currently running in this NaCl module */
 int __nc_running_threads_counter;
 
-/*
- * Array for TSD keys status management. */
-tsd_status_t __nc_thread_specific_storage[PTHREAD_KEYS_MAX];
-
 /* we have two queues of memory blocks - one for each type */
 STAILQ_HEAD(tailhead, entry) __nc_thread_memory_blocks[2];
 /* We need a counter for each queue to keep track of number of blocks */
@@ -82,7 +82,7 @@ static inline nc_thread_descriptor_t *nc_get_tdb(void) {
    * a wrapper around __libnacl_irt_tls.tls_get() but we don't use
    * that here so that the IRT build can override the definition.
    */
-  return __nacl_read_tp();
+  return (void *) ((char *) __nacl_read_tp() + __nacl_tp_tdb_offset(TDB_SIZE));
 }
 
 static int nc_allocate_thread_id_mu(nc_basic_thread_data_t *basic_data) {
@@ -136,7 +136,6 @@ void nc_thread_starter(nc_thread_function func,
   __newlib_thread_init();
   retval = tdb->start_func(tdb->state);
   /* if the function returns, terminate the thread */
-  tdb->exiting_without_returning = 0;
   pthread_exit(retval);
   /* NOTREACHED */
   /* TODO(gregoryd) - add assert */
@@ -249,7 +248,8 @@ static void nc_release_basic_data_mu(nc_basic_thread_data_t *basic_data) {
 static void nc_release_tls_node(nc_thread_memory_block_t *block) {
   if (block) {
     nc_thread_descriptor_t* tdb =
-      (nc_thread_descriptor_t *) __nacl_tls_tdb_start(NODE_TO_PAYLOAD(block));
+        (nc_thread_descriptor_t *) __nacl_tls_tdb_start(NODE_TO_PAYLOAD(block),
+                                                        TDB_SIZE);
 
     tdb->basic_data->tdb = NULL;
     block->is_used = 0;
@@ -257,34 +257,6 @@ static void nc_release_tls_node(nc_thread_memory_block_t *block) {
   }
 }
 
-/* Implements this, atomically:
- *  old = *ptr;
- *  if (*ptr == old_value)
- *    *ptr = new_value;
- *  return old;
- */
-static AtomicInt32 CompareAndSwap(volatile AtomicInt32* ptr,
-                                  AtomicInt32 old_value,
-                                  AtomicInt32 new_value) {
-  /* GCC intrinsic; see:
-   * http://gcc.gnu.org/onlinedocs/gcc/Atomic-Builtins.html.
-   * The x86-{32,64} compilers generate inline code.  The ARM
-   * implementation is external: instrincs/crt1_arm.S.
-   */
-  return __sync_val_compare_and_swap(ptr, old_value, new_value);
-}
-
-void nc_spinlock_lock(volatile int *lock) {
-  uint32_t val;
-  do {
-    val = CompareAndSwap(lock, 0, 1);
-  } while (val != 0);
-}
-
-void nc_spinlock_unlock(volatile int *lock) {
-  *lock = 0;
-}
-
 uint32_t __nacl_tdb_id_function(nc_hash_entry_t *entry) {
   nc_basic_thread_data_t *basic_data =
       HASH_ENTRY_TO_ENTRY_TYPE(entry, nc_basic_thread_data, hash_entry);
@@ -306,17 +278,11 @@ static int nc_tdb_init(nc_thread_descriptor_t *tdb,
   tdb->joinable = PTHREAD_CREATE_JOINABLE;
   tdb->join_waiting = 0;
 
-  tdb->thread_specific_data = NULL;
   tdb->tls_node = NULL;
   tdb->stack_node = NULL;
 
   tdb->start_func = NULL;
   tdb->state = NULL;
-  /*
-   * A call to pthread_exit without returning from the thread function
-   * should be recognized
-   */
-  tdb->exiting_without_returning = 1;
 
   /* Imitate PTHREAD_COND_INITIALIZER - we cannot use it directly here,
    * since this is not variable initialization.
@@ -325,7 +291,6 @@ static int nc_tdb_init(nc_thread_descriptor_t *tdb,
   return 0;
 }
 
-
 /* Will be called from the library startup code,
  * which always happens on the application's main thread
  */
@@ -335,29 +300,12 @@ int __pthread_initialize(void) {
   nc_basic_thread_data_t *basic_data;
   /* allocate TLS+TDB area */
   /* We allocate the basic data immediately after TDB */
-  __pthread_initialize_minimal(sizeof(nc_thread_descriptor_t) +
-                               sizeof(nc_basic_thread_data_t));
+  __pthread_initialize_minimal(TDB_SIZE);
 
   /*
-   * Fetch the ABI tables from the IRT.  We fall back to tables
-   * provided by libnacl, in case we have no IRT (or are the IRT).
+   * Fetch the ABI tables from the IRT.  If we don't have these, all is lost.
    */
-  if (NULL == __nacl_irt_query ||
-      __nacl_irt_query(NACL_IRT_THREAD_v0_1, &irt_thread,
-                       sizeof(irt_thread)) != sizeof(irt_thread))
-    irt_thread = nacl_irt_thread;
-  if (NULL == __nacl_irt_query ||
-      __nacl_irt_query(NACL_IRT_MUTEX_v0_1, &__nc_irt_mutex,
-                       sizeof(__nc_irt_mutex)) != sizeof(__nc_irt_mutex))
-    __nc_irt_mutex = nacl_irt_mutex;
-  if (NULL == __nacl_irt_query ||
-      __nacl_irt_query(NACL_IRT_COND_v0_1, &__nc_irt_cond,
-                       sizeof(__nc_irt_cond)) != sizeof(__nc_irt_cond))
-    __nc_irt_cond = nacl_irt_cond;
-  if (NULL == __nacl_irt_query ||
-      __nacl_irt_query(NACL_IRT_SEM_v0_1, &__nc_irt_sem,
-                       sizeof(__nc_irt_sem)) != sizeof(__nc_irt_sem))
-    __nc_irt_sem = nacl_irt_sem;
+  __nc_initialize_interfaces(&irt_thread);
 
   /* At this point GS is already initialized */
   tdb = nc_get_tdb();
@@ -401,43 +349,10 @@ int __pthread_initialize(void) {
 }
 
 
-/*
- * Initialize a thread that was not created by us.  This is for use in
- * the integrated runtime (IRT), for when a thread is created by the
- * user executable but IRT code must execute in its context.
- *
- * TODO(mseaborn): Currently the data structure we allocate here will
- * leak.  We will need to hook into the thread_exit syscall to
- * deallocate it.
- */
-void *__nc_adopt_thread(void) {
-  nc_thread_descriptor_t *tdb;
-  nc_basic_thread_data_t *basic_data;
-  __pthread_initialize_minimal(sizeof(nc_thread_descriptor_t) +
-                               sizeof(nc_basic_thread_data_t));
-  tdb = nc_get_tdb();
-  basic_data = (nc_basic_thread_data_t *) (tdb + 1);
-  nc_tdb_init(tdb, basic_data);
-  /*
-   * The thread is not ours to join or detach.  We set this just in
-   * case, but it is not strictly necessary.
-   */
-  tdb->joinable = 0;
-  /*
-   * There is no need to use a small integer for this thread's ID
-   * since it will never be looked up by pthread_join() or
-   * pthread_detach().  We do not insert the thread into the hash
-   * table.
-   */
-  basic_data->thread_id = (uintptr_t) basic_data;
-  return tdb;
-}
-
-
 /* pthread functions */
 
 int pthread_create(pthread_t *thread_id,
-                   pthread_attr_t *attr,
+                   const pthread_attr_t *attr,
                    void *(*start_routine) (void *),
                    void *arg) {
   int retval = EAGAIN;
@@ -449,6 +364,7 @@ int pthread_create(pthread_t *thread_id,
   nc_basic_thread_data_t *new_basic_data = NULL;
   nc_thread_memory_block_t *tls_node = NULL;
   size_t stacksize = PTHREAD_STACK_DEFAULT;
+  void *new_tp;
 
   /* TODO(gregoryd) - right now a single lock is used, try to optimize? */
   pthread_mutex_lock(&__nc_thread_management_lock);
@@ -456,16 +372,16 @@ int pthread_create(pthread_t *thread_id,
   do {
     /* Allocate the combined TLS + TDB block---see tls.h for explanation. */
 
-    tls_node = nc_allocate_memory_block_mu(
-        TLS_AND_TDB_MEMORY,
-         __nacl_tls_combined_size(sizeof(nc_thread_descriptor_t)));
+    tls_node = nc_allocate_memory_block_mu(TLS_AND_TDB_MEMORY,
+                                           __nacl_tls_combined_size(TDB_SIZE));
     if (NULL == tls_node)
       break;
 
-    __nacl_tls_data_bss_initialize_from_template(NODE_TO_PAYLOAD(tls_node));
+    __nacl_tls_data_bss_initialize_from_template(NODE_TO_PAYLOAD(tls_node),
+                                                 TDB_SIZE);
 
     new_tdb = (nc_thread_descriptor_t *)
-              __nacl_tls_tdb_start(NODE_TO_PAYLOAD(tls_node));
+              __nacl_tls_tdb_start(NODE_TO_PAYLOAD(tls_node), TDB_SIZE);
     /* TODO(gregoryd): consider creating a pool of basic_data structs,
      * similar to stack and TLS+TDB (probably when adding the support for
      * variable stack size).
@@ -524,21 +440,22 @@ int pthread_create(pthread_t *thread_id,
   pthread_mutex_unlock(&__nc_thread_management_lock);
 
   /*
-   * Calculate the top-of-stack location.  The very first location, is
-   * a zero address of architecture-dependent width, needed to satisfy
-   * the alignment requirement when we call nacl's thread_exit syscall
-   * when the thread terminates.
+   * Calculate the top-of-stack location.  The very first location is a
+   * zero address of architecture-dependent width, needed to satisfy the
+   * normal ABI alignment requirements for the stack.  (On some machines
+   * this is the dummy return address of the thread-start function.)
    *
    * Both thread_stack and stacksize are multiples of 16.
    */
-  size_t return_addr_size = __nacl_return_address_size();
-  esp = (void*) (thread_stack + stacksize - return_addr_size);
-  memset(esp, 0, return_addr_size);  /* NULL/0x00 pun is not strictly legal. */
+  size_t stack_padding = __nacl_thread_stack_padding();
+  esp = (void *) (thread_stack + stacksize - stack_padding);
+  memset(esp, 0, stack_padding);
+
+  new_tp = (char *) new_tdb - __nacl_tp_tdb_offset(TDB_SIZE);
 
   /* start the thread */
   retval = irt_thread.thread_create(
-      FUN_TO_VOID_PTR(nc_thread_starter), esp,
-      new_tdb, sizeof(nc_thread_descriptor_t));
+      FUN_TO_VOID_PTR(nc_thread_starter), esp, new_tp);
   if (0 != retval) {
     pthread_mutex_lock(&__nc_thread_management_lock);
     /* TODO(gregoryd) : replace with atomic decrement? */
@@ -570,7 +487,7 @@ ret:
   return retval;
 }
 
-int __pthread_shutdown(void) {
+static int wait_for_threads(void) {
   pthread_mutex_lock(&__nc_thread_management_lock);
 
   while (1 != __nc_running_threads_counter) {
@@ -600,36 +517,9 @@ void pthread_exit (void* retval) {
   nc_basic_thread_data_t    *basic_data = tdb->basic_data;
   pthread_t                 thread_id = basic_data->thread_id;
   int                       joinable = tdb->joinable;
-  unsigned int              generation;
 
   /* call the destruction functions for TSD */
-  if (tdb->exiting_without_returning) {
-    int destruction_iterations;
-    for (destruction_iterations = 0;
-        destruction_iterations < PTHREAD_DESTRUCTOR_ITERATIONS;
-        destruction_iterations++) {
-      int i = 0;
-      int nothing_to_do = 1;
-      for (i = 0; i < PTHREAD_KEYS_MAX; i++) {
-        void (*destructor)(void*) =
-            __nc_thread_specific_storage[i].destruction_callback;
-        if ((KEY_ALLOCATED(generation =
-                __nc_thread_specific_storage[i].generation)) &&
-            (tdb->thread_specific_data != NULL) &&
-            (tdb->thread_specific_data[i].generation == generation) &&
-            (tdb->thread_specific_data[i].ptr != NULL) &&
-            (NULL != destructor)) {
-          void *tmp = tdb->thread_specific_data[i].ptr;
-          tdb->thread_specific_data[i].ptr = NULL;
-          destructor(tmp);
-          nothing_to_do = 0;
-        }
-      }
-      if (nothing_to_do) {
-        break;
-      }
-    }
-  }
+  __nc_tsd_exit();
 
   __newlib_thread_exit();
 
@@ -645,8 +535,8 @@ void pthread_exit (void* retval) {
 
   if (0 == thread_id) {
     /* This is the main thread - wait for other threads to complete */
-    __pthread_shutdown();
-    _exit(0);
+    wait_for_threads();
+    exit(0);
   }
 
   pthread_mutex_lock(&__nc_thread_management_lock);
@@ -677,7 +567,7 @@ void pthread_exit (void* retval) {
 
   pthread_mutex_unlock(&__nc_thread_management_lock);
   irt_thread.thread_exit(is_used);
-  while (1) (*(void (*)(void)) 0)();  /* Crash.  */
+  while (1) *(volatile int *) 0 = 0;  /* Crash.  */
 }
 
 int pthread_join(pthread_t thread_id, void **thread_return) {
@@ -780,89 +670,6 @@ int pthread_equal (pthread_t thread1, pthread_t thread2) {
   return (thread1 == thread2);
 }
 
-/* Thread-specific data functions */
-
-int pthread_key_create (pthread_key_t *key,
-                        void (*destr_function) (void *)) {
-  int i;
-  unsigned int generation;
-  for (i=0; i<PTHREAD_KEYS_MAX; ++i) {
-    if (!KEY_ALLOCATED(generation =
-            __nc_thread_specific_storage[i].generation) &&
-        /* A key is not reusable after UINT_MAX-1 generations -
-         * the last even generation */
-        (__nc_thread_specific_storage[i].generation != UINT_MAX - 1) &&
-        (generation == CompareAndSwap(
-            (AtomicInt32*) &__nc_thread_specific_storage[i].generation,
-            generation,
-            generation + 1))) {
-      *key = i;
-      __nc_thread_specific_storage[i].destruction_callback = destr_function;
-      return 0;
-    }
-  }
-
-  return EAGAIN;
-}
-
-int pthread_key_delete (pthread_key_t key) {
-  unsigned int generation;
-  if ((key > PTHREAD_KEYS_MAX - 1) ||
-      !KEY_ALLOCATED(generation =
-          __nc_thread_specific_storage[key].generation)) {
-    return EINVAL;
-  }
-
-  if (generation !=
-      CompareAndSwap(
-          (AtomicInt32*) &__nc_thread_specific_storage[key].generation,
-          generation,
-          generation + 1)) {
-    /* Somebody incremented the generation counter before we did, i.e. the key
-     * has already been deleted.
-     */
-    return EINVAL;
-  }
-
-  __nc_thread_specific_storage[key].destruction_callback = NULL;
-  return 0;
-}
-
-int pthread_setspecific (pthread_key_t key,
-                         const void *pointer) {
-  nc_thread_descriptor_t *tdb = nc_get_tdb();
-  unsigned int generation;
-
-  if ((key > PTHREAD_KEYS_MAX - 1) ||
-      !KEY_ALLOCATED(generation =
-          __nc_thread_specific_storage[key].generation)) {
-      return EINVAL;
-  }
-
-  /* Allocate the memory for this thread's TSD - we actually need it */
-  if (NULL == tdb->thread_specific_data) {
-    tdb->thread_specific_data = calloc(PTHREAD_KEYS_MAX, sizeof(tsd_t));
-    if (tdb->thread_specific_data == NULL) {
-      return EINVAL;
-    }
-  }
-
-  tdb->thread_specific_data[key].generation = generation;
-  tdb->thread_specific_data[key].ptr = (void*) pointer;
-  return 0;
-}
-
-void *pthread_getspecific (pthread_key_t key) {
-  nc_thread_descriptor_t *tdb = nc_get_tdb();
-  if (!tdb->thread_specific_data ||
-      (key > PTHREAD_KEYS_MAX - 1) ||
-      (tdb->thread_specific_data[key].generation !=
-          __nc_thread_specific_storage[key].generation)) {
-    return NULL;
-  }
-  return tdb->thread_specific_data[key].ptr;
-}
-
 int pthread_setschedprio(pthread_t thread_id, int prio) {
   if (thread_id != pthread_self()) {
     /*
@@ -1028,3 +835,12 @@ void __local_lock_release(_LOCK_T* lock) {
 void __local_lock_release_recursive(_LOCK_T* lock) {
   __local_lock_release(lock);
 }
+
+/*
+ * We include this directly in this file rather than compiling it
+ * separately because there is some code (e.g. libstdc++) that uses weak
+ * references to all pthread functions, but conditionalizes its calls only
+ * on one symbol.  So if these functions are in another file in a library
+ * archive, they might not be linked in by static linking.
+ */
+#include "native_client/src/untrusted/pthread/nc_tsd.c"
