@@ -26,7 +26,6 @@
 
 #include <stdio.h>
 #include <assert.h>
-#include <pthread.h>
 #include <errno.h>
 #include <sched.h>
 #include <stdlib.h>
@@ -43,7 +42,17 @@
 
 #include "native_client/src/third_party/valgrind/ts_valgrind_client_requests.h"
 
-#ifndef __GLIBC__
+#ifdef __GLIBC__
+#include <pthread.h>
+#else
+/*
+ * Get pthread.h from the source tree rather than using the installed one
+ * in the newlib build.  It might not be installed yet when we're building.
+ * Since we're about to include its private header here anyway, we might
+ * as well consistently refer to the source rather than what's installed.
+ */
+#include "native_client/src/untrusted/pthread/pthread.h"
+
 /* For sizeof(nc_thread_memory_block_t) */
 #include "native_client/src/untrusted/pthread/pthread_types.h"
 #endif
@@ -57,8 +66,8 @@ int have_nacl_valgrind_interceptors;
 /* TSan interceptors. */
 
 #ifdef __GLIBC__
-#define VG_NACL_Z_LIBC_SONAME  NaClZulibcZdsoZa
-#define VG_NACL_Z_LIBPTHREAD_SONAME NaClZulibpthreadZdsoZd0
+#define VG_NACL_Z_LIBC_SONAME  NaClZulibcZdsoZdZa
+#define VG_NACL_Z_LIBPTHREAD_SONAME NaClZulibpthreadZdsoZdZa
 #define VG_NACL_Z_NONE_SONAME NaClZuNONE
 #define VG_NACL_Z_ANY_SONAME NaClZuZa
 #else
@@ -485,11 +494,11 @@ void VG_NACL_LIBPTHREAD_FUNC(ZuZupthreadZuinitializeZuminimal)(void) {
 void VG_NACL_NONE_FUNC(ncZuthreadZustarter)(size_t func, size_t state) {
   OrigFn fn;
   int local_stack_var = 0;
+  VALGRIND_GET_ORIG_FN(fn);
 
   /* Let the tool guess where the stack starts. */
   VG_CREQ_v_W(TSREQ_THR_STACK_TOP, (size_t)&local_stack_var);
 
-  VALGRIND_GET_ORIG_FN(fn);
   CALL_FN_v_WW(fn, func, state);
 }
 
