@@ -1,7 +1,7 @@
 /*
- * Copyright 2010 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 
@@ -29,6 +29,12 @@ extern char template_func_illegal_guard_replacement;
 extern char template_func_illegal_guard_replacement_end;
 extern char template_func_illegal_constant_replacement;
 extern char template_func_illegal_constant_replacement_end;
+#if defined(__i386__) || defined(__x86_64__)
+extern char template_instr;
+extern char template_instr_end;
+extern char template_instr_replace;
+extern char template_instr_replace_end;
+#endif
 
 extern char template_func_external_jump_target;
 extern char template_func_external_jump_target_end;
