@@ -1,12 +1,17 @@
 /*
- * Copyright 2011 The Native Client Authors.  All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 #include <stdio.h>
 #include <sys/nacl_syscalls.h>
 
+#if defined(IN_BROWSER)
+#include <nacl/nacl_srpc.h>
+#include "native_client/src/shared/srpc/nacl_srpc_ppapi_plugin_internal.h"
+#endif
+
 int main() {
   /*
    * There are two possible kinds of fault in trusted code:
@@ -29,6 +34,14 @@ int main() {
    * and the call fails to fault, the test runner will catch that.)
    * We don't use NULL because the syscall checks for NULL.
    */
+  fprintf(stderr, "Entered main\n"); fflush(stderr);
+#if defined(IN_BROWSER)
+  if (!NaClSrpcModuleInit())
+    return -1;
+  NaClPluginLowLevelInitializationComplete();
+#endif
+
+  fprintf(stderr, "About to crash\n"); fflush(stderr);
   int rc = imc_recvmsg(0, (struct NaClImcMsgHdr *) 0x1000, 0);
   if (rc < 0) {
     perror("imc_recvmsg");
