@@ -17,7 +17,7 @@ fi
 source ../../tools/llvm/common-tools.sh
 readonly NACL_ROOT="$(GetAbsolutePath "../../")"
 SetScriptPath "$(pwd)/run_all.sh"
-SetLogDirectory "."
+SetLogDirectory "${NACL_ROOT}/toolchain/test-log"
 
 ######################################################################
 # CONFIGURATION
@@ -41,7 +41,15 @@ SPEC2K_SCRIPT="./run.train.sh"
 # export VERIFY=no
 export VERIFY=${VERIFY:-yes}
 export MAKEOPTS=${MAKEOPTS:-}
+
+# Helper script to process timing / other perf data.
+# Export these paths to Makefile.common, which will be included by other
+# Makefiles in random sub-directories (where pwd will be different).
 export PERF_LOGGER="$(pwd)/emit_perf_log.sh"
+export COMPILE_REPEATER="$(pwd)/compile_repeater.sh"
+# Number of times to repeat a timed step.
+export SPEC_RUN_REPETITIONS=${SPEC_RUN_REPETITIONS:-1}
+export SPEC_COMPILE_REPETITIONS=${SPEC_COMPILE_REPETITIONS:-1}
 
 
 ######################################################################
@@ -54,7 +62,7 @@ readonly TC_ROOT="${NACL_ROOT}/toolchain"
 readonly ARM_TRUSTED_TC="${TC_ROOT}/linux_arm-trusted"
 readonly QEMU_TOOL="${ARM_TRUSTED_TC}/qemu_tool.sh"
 
-readonly PNACL_TC="${TC_ROOT}/pnacl_${BUILD_PLATFORM}_${BUILD_ARCH}"
+readonly PNACL_TC="${TC_ROOT}/pnacl_${BUILD_PLATFORM}_${BUILD_ARCH}_newlib"
 readonly NNACL_TC="${TC_ROOT}/${SCONS_BUILD_PLATFORM}_x86"
 readonly RUNNABLE_LD_X8632="${NNACL_TC}/nacl/lib/runnable-ld.so"
 readonly RUNNABLE_LD_X8664="${NNACL_TC}/nacl64/lib/runnable-ld.so"
@@ -119,12 +127,12 @@ SetupGccX8664Opt() {
 ######################################################################
 
 SetupNaclX8632Common() {
-  SEL_LDR=$(GetSelLdr "x86-32")
-  PREFIX="${SEL_LDR} -a -f"
+  basic-setup-nacl x86-32
+  SetupSelLdr x86-32
 }
 
 #@
-#@ SetupNaClX8632
+#@ SetupNaclX8632
 #@   use nacl-gcc compiler
 SetupNaclX8632() {
   SetupNaclX8632Common
@@ -132,7 +140,7 @@ SetupNaclX8632() {
 }
 
 #@
-#@ SetupNaClX8632Opt
+#@ SetupNaclX8632Opt
 #@   use nacl-gcc compiler with optimizations
 SetupNaclX8632Opt() {
   SetupNaclX8632Common
@@ -140,12 +148,12 @@ SetupNaclX8632Opt() {
 }
 
 SetupNaclX8664Common() {
-  SEL_LDR=$(GetSelLdr "x86-64")
-  PREFIX="${SEL_LDR} -a -f"
+  basic-setup-nacl x86-64
+  SetupSelLdr x86-64
 }
 
 #@
-#@ SetupNaClX8664
+#@ SetupNaclX8664
 #@   use nacl-gcc64 compiler
 SetupNaclX8664() {
   SetupNaclX8664Common
@@ -153,7 +161,7 @@ SetupNaclX8664() {
 }
 
 #@
-#@ SetupNaClX8664Opt
+#@ SetupNaclX8664Opt
 #@   use nacl-gcc64 compiler with optimizations
 SetupNaclX8664Opt() {
   SetupNaclX8664Common
@@ -161,8 +169,8 @@ SetupNaclX8664Opt() {
 }
 
 SetupNaclDynX8632Common() {
-  SEL_LDR=$(GetSelLdr "x86-32")
-  PREFIX="${SEL_LDR} -a -s -f ${RUNNABLE_LD_X8632}"
+  basic-setup-nacl x86-32
+  SetupSelLdr x86-32 "" "-s" "${RUNNABLE_LD_X8632}"
 }
 
 #@
@@ -182,8 +190,8 @@ SetupNaclDynX8632Opt() {
 }
 
 SetupNaclDynX8664Common() {
-  SEL_LDR=$(GetSelLdr "x86-64")
-  PREFIX="${SEL_LDR} -a -s -f ${RUNNABLE_LD_X8664}"
+  basic-setup-nacl x86-64
+  SetupSelLdr x86-64 "" "-s" "${RUNNABLE_LD_X8664}"
 }
 
 #@
@@ -205,8 +213,8 @@ SetupNaclDynX8664Opt() {
 ######################################################################
 
 SetupPnaclX8664Common() {
-  SEL_LDR=$(GetSelLdr "x86-64")
-  PREFIX="${SEL_LDR} -a -f"
+  basic-setup-pnacl x86-64
+  SetupSelLdr x86-64
 }
 
 #@
@@ -229,7 +237,6 @@ SetupPnaclX8664Opt() {
 #@ SetupPnaclTranslatorX8664
 #@    use pnacl x8664 translator (no lto)
 SetupPnaclTranslatorX8664() {
-  CheckSelUniversal "x86-64"
   SetupPnaclX8664Common
   SUFFIX=pnacl_translator.x8664
 }
@@ -238,14 +245,13 @@ SetupPnaclTranslatorX8664() {
 #@ SetupPnaclTranslatorX8664Opt
 #@    use pnacl x8664 translator (with lto)
 SetupPnaclTranslatorX8664Opt() {
-  CheckSelUniversal "x86-64"
   SetupPnaclX8664Common
   SUFFIX=pnacl_translator.opt.x8664
 }
 
 SetupPnaclX8632Common() {
-  SEL_LDR=$(GetSelLdr "x86-32")
-  PREFIX="${SEL_LDR} -a -f"
+  basic-setup-pnacl x86-32
+  SetupSelLdr x86-32
 }
 
 #@
@@ -269,7 +275,6 @@ SetupPnaclX8632Opt() {
 #@ SetupPnaclTranslatorX8632
 #@    use pnacl x8632 translator (no lto)
 SetupPnaclTranslatorX8632() {
-  CheckSelUniversal "x86-32"
   SetupPnaclX8632Common
   SUFFIX=pnacl_translator.x8632
 }
@@ -278,7 +283,6 @@ SetupPnaclTranslatorX8632() {
 #@ SetupPnaclTranslatorX8632Opt
 #@    use pnacl x8632 translator (with lto)
 SetupPnaclTranslatorX8632Opt() {
-  CheckSelUniversal "x86-32"
   SetupPnaclX8632Common
   SUFFIX=pnacl_translator.opt.x8632
 }
@@ -293,8 +297,8 @@ SetupGccArm() {
 
 
 SetupPnaclArmCommon() {
-  SEL_LDR=$(GetSelLdr "arm")
-  PREFIX="${QEMU_TOOL} run ${SEL_LDR} -a -Q -f"
+  basic-setup-pnacl arm
+  SetupSelLdr arm "${QEMU_TOOL} run" "-Q"
   SUFFIX=pnacl.arm
 }
 
@@ -318,7 +322,6 @@ SetupPnaclArm() {
 #@ SetupPnaclTranslatorArm
 #@    use pnacl arm translator (no lto)
 SetupPnaclTranslatorArm() {
-  CheckSelUniversal "arm"
   SetupPnaclArmCommon
   SUFFIX=pnacl_translator.arm
 }
@@ -327,14 +330,12 @@ SetupPnaclTranslatorArm() {
 #@ SetupPnaclTranslatorArmOpt
 #@    use pnacl arm translator (with lto)
 SetupPnaclTranslatorArmOpt() {
-  CheckSelUniversal "arm"
   SetupPnaclArmCommon
   SUFFIX=pnacl_translator.opt.arm
 }
 
 SetupPnaclArmCommonHW() {
-  SEL_LDR=$(GetSelLdr "arm")
-  PREFIX="${SEL_LDR} -a -f"
+  SetupSelLdr arm
   SUFFIX=pnacl.arm
 }
 
@@ -358,7 +359,6 @@ SetupPnaclArmHW() {
 #@ SetupPnaclTranslatorArmHW
 #@    use pnacl arm translator (no lto) -- run on ARM hardware
 SetupPnaclTranslatorArmHW() {
-  CheckSelUniversal "arm"
   SetupPnaclArmCommonHW
   SUFFIX=pnacl_translator.hw.arm
 }
@@ -367,7 +367,6 @@ SetupPnaclTranslatorArmHW() {
 #@ SetupPnaclTranslatorArmOptHW
 #@    use pnacl arm translator (with lto) -- run on ARM hardware
 SetupPnaclTranslatorArmOptHW() {
-  CheckSelUniversal "arm"
   SetupPnaclArmCommonHW
   SUFFIX=pnacl_translator.opt.hw.arm
 }
@@ -442,21 +441,88 @@ CheckFileBuilt() {
 }
 
 #+
-#+ GetSelLdr <arch>
+#+ SetupSelLdr <arch> <prefix> <extra_flags> <preload>
 #+
-#+   Get sel_ldr for the given arch.
-GetSelLdr() {
-  local arch=$1
+#+   Set up PREFIX to run sel_ldr on <arch>.
+#+   <prefix> precedes sel_ldr in the command.
+#+   <extra_flags> are additional flags to sel_ldr.
+#+   <preload> is used as the actual nexe to load, making the real nexe an arg.
+SetupSelLdr() {
+  local arch="$1"
+  local prefix="${2-}"
+  local extra_flags="${3-}"
+  local preload="${4-}"
+
   SEL_LDR="${SCONS_OUT}/opt-${SCONS_BUILD_PLATFORM}-${arch}/staging/sel_ldr"
   CheckFileBuilt "sel_ldr" "${SEL_LDR}"
-  echo "${SEL_LDR}"
+
+  IRT_IMAGE="${SCONS_OUT}/nacl_irt-${arch}/staging/irt_core.nexe"
+  CheckFileBuilt "IRT image" "${IRT_IMAGE}"
+
+  PREFIX="${prefix} ${SEL_LDR} -B ${IRT_IMAGE} -a ${extra_flags} -f ${preload}"
 }
 
-CheckSelUniversal() {
-  local arch=$1
-  SEL_UNIV="${SCONS_OUT}/opt-${SCONS_BUILD_PLATFORM}-${arch}/staging/\
-sel_universal"
-  CheckFileBuilt "sel_universal" "${SEL_UNIV}"
+# Flag to allow skipping building pre-requisite pieces if they were already
+# built in a previous run. Be careful that different platforms and
+# compilers have different prerequisites, so only set this to be "true"
+# If you are sure that the prerequisites are available.
+SPEC2K_PREREQS_BUILT=${SPEC2K_PREREQS_BUILT:-false}
+
+basic-setup-nacl() {
+  if ${SPEC2K_PREREQS_BUILT}; then
+    return 0
+  fi
+
+  local platforms=$1
+  build-runtime "${platforms}" "sel_ldr irt_core"
+  # libs may be unnecessary for the glibc build, but build it just in case.
+  build-libs-nacl "${platforms}"
+  # Set to true, assuming this is used via BuildAndRun() w/ the same
+  # platform and compiler setup.
+  SPEC2K_PREREQS_BUILT=true
+}
+
+basic-setup-pnacl() {
+  if ${SPEC2K_PREREQS_BUILT}; then
+    return 0
+  fi
+
+  local platforms=$1
+  # Sel universal is only used for the pnacl sandboxed translator,
+  # but prepare it just in case.
+  # IRT is used both to run the tests and to run the pnacl sandboxed translator.
+  build-runtime "${platforms}" "sel_ldr sel_universal irt_core"
+  build-libs-pnacl
+  # Set to true, assuming this is used via BuildAndRun() w/ the same
+  # platform and compiler setup.
+  SPEC2K_PREREQS_BUILT=true
+}
+
+SCONS_COMMON="./scons --mode=opt-host,nacl -j8 --verbose"
+
+build-runtime() {
+  local platforms=$1
+  local runtime_pieces=$2
+  for platform in ${platforms} ; do
+    echo "build-runtime: scons ${runtime_pieces} [${platform}]"
+    (cd ${NACL_ROOT};
+      ${SCONS_COMMON} platform=${platform} ${runtime_pieces})
+  done
+}
+
+build-libs-nacl() {
+  local platforms=$1
+  shift 1
+  for platform in ${platforms} ; do
+    echo "build-libs-nacl: scons build_lib [${platform}] $*"
+    (cd ${NACL_ROOT};
+      ${SCONS_COMMON} platform=${platform} build_lib "$@")
+  done
+}
+
+build-libs-pnacl() {
+  (cd ${NACL_ROOT};
+    ./tools/llvm/utman.sh sdk)
 }
 
 #@
@@ -483,8 +549,9 @@ CleanBenchmarks() {
 #@  Results are delivered to {execname}.compile_time
 BuildBenchmarks() {
   export PREFIX=
-  timeit=$1
-  "$2"
+  local timeit=$1
+  local setup_func=$2
+  "${setup_func}"
   shift 2
 
   local list=$(GetBenchmarkList "$@")
@@ -495,6 +562,8 @@ BuildBenchmarks() {
 
     make ${MAKEOPTS} measureit=${timeit} \
          PERF_LOGGER="${PERF_LOGGER}" \
+         REPETITIONS="${SPEC_COMPILE_REPETITIONS}" \
+         COMPILE_REPEATER="${COMPILE_REPEATER}" \
          BUILD_PLATFORM=${BUILD_PLATFORM} \
          SCONS_BUILD_PLATFORM=${SCONS_BUILD_PLATFORM} \
          BUILD_ARCH=${BUILD_ARCH} \
@@ -510,7 +579,7 @@ BuildBenchmarks() {
 TimedRunCmd() {
   target=$1
   shift
-  /usr/bin/time -f "%U %S %e %C" -o ${target} "$@"
+  /usr/bin/time -f "%U %S %e %C" --append -o ${target} "$@"
 }
 
 #@
@@ -519,7 +588,8 @@ TimedRunCmd() {
 #@  Run all benchmarks according to the setup.
 RunBenchmarks() {
   export PREFIX=
-  "$1"
+  local setup_func=$1
+  "${setup_func}"
   shift
   local list=$(GetBenchmarkList "$@")
   local script=$(GetInputSize "$@")
@@ -556,7 +626,13 @@ RunTimedBenchmarks() {
     local target_file=./${benchname}.${SUFFIX}
     local time_file=${target_file}.run_time
     gnu_size  ${target_file}
-    TimedRunCmd ${time_file} ${script} ${target_file}
+    # Clear out the previous times.
+    rm -f "${time_file}"
+    echo "Running benchmark ${SPEC_RUN_REPETITIONS} times"
+    for ((i=0; i<${SPEC_RUN_REPETITIONS}; i++))
+    do
+      TimedRunCmd ${time_file} ${script} ${target_file}
+    done
     # TODO(jvoung): split runtimes by arch as well
     # i.e., pull "arch" out of SUFFIX and add to the "runtime" label.
     "${PERF_LOGGER}" LogUserSysTime "${time_file}" "runtime" \
@@ -622,10 +698,10 @@ ArchivedRunOnArmHW() {
 
   # Switch to native_client directory (from tests/spec2k) so that
   # when we extract, the builder will have a more natural directory layout.
-  # Unfortunately, this means we can't reuse GetSelLdr()...
   local TAR_FROM="../.."
   tar -C ${TAR_FROM} -czvf ${TAR_TO}/${SEL_ARCHIVE_ZIPPED} \
-    scons-out/opt-${SCONS_BUILD_PLATFORM}-arm/staging/sel_ldr
+    scons-out/opt-${SCONS_BUILD_PLATFORM}-arm/staging/sel_ldr \
+    scons-out/nacl_irt-arm/staging/irt_core.nexe
   # Carefully tar only the parts of the spec harness that we need.
   (cd ${TAR_FROM} ;
     find tests/spec2k -type f -maxdepth 1 -print |
@@ -672,7 +748,7 @@ PopulateFromSpecHarness() {
   for i in ${list} ; do
     SubBanner "Populating: $i"
     # fix the dir with the same name inside spec harness
-    src=$(find ${harness} -name $i)
+    src=$(find -H ${harness} -name $i)
     # copy relevant dirs over
     echo "COPY"
     rm -rf src/ data/
