@@ -1,7 +1,7 @@
 /*
- * Copyright 2008 The Native Client Authors. All rights reserved.
- * Use of this source code is governed by a BSD-style license that can
- * be found in the LICENSE file.
+ * Copyright (c) 2011 The Native Client Authors. All rights reserved.
+ * Use of this source code is governed by a BSD-style license that can be
+ * found in the LICENSE file.
  */
 
 /*
@@ -342,7 +342,12 @@ void TestUnlockUninitializedReturnValue() {
 
 pthread_once_t once_control = PTHREAD_ONCE_INIT;
 
-typedef int AtomicInt32;  /* Why is this needed? We included pthread already. */
+/*
+ * The nacl-newlib pthread.h declares this type, but glibc's pthread.h does not.
+ */
+#ifdef __GLIBC__
+typedef int AtomicInt32;
+#endif
 
 void pthread_once_routine() {
   static AtomicInt32 count = 0;
@@ -471,15 +476,6 @@ pthread_key_t tsd_key;
 void* TsdThread(void *state) {
   CHECK_OK(pthread_setspecific(tsd_key, state));
 
-#ifndef __GLIBC__
-  /* This works around a bug in nacl-newlib's libpthread.  The TSD
-     destructor only gets run by explicit pthread_exit() calls, and
-     not by returning from the thread function.
-     See http://code.google.com/p/nativeclient/issues/detail?id=1085
-     TODO(mseaborn): This would be easy to fix. */
-  pthread_exit(NULL);
-#endif
-
   return 0;
 }
 
