@@ -8,6 +8,63 @@ function $(id) {
 }
 
 
+function createNaClEmbed(args) {
+  var fallback = function(value, default_value) {
+    return value !== undefined ? value : default_value;
+  };
+  var embed = document.createElement('embed');
+  embed.id = args.id;
+  embed.src = args.src;
+  embed.type = fallback(args.type, 'application/x-nacl');
+  // JavaScript inconsistency: this is equivalent to class=... in HTML.
+  embed.className = fallback(args.className, 'naclModule');
+  embed.width = fallback(args.width, 0);
+  embed.height = fallback(args.height, 0);
+  return embed;
+}
+
+
+function decodeURIArgs(encoded) {
+  var args = {};
+  if (encoded.length > 0) {
+    var pairs = encoded.replace(/\+/g, ' ').split('&');
+    for (var p = 0; p < pairs.length; p++) {
+      var pair = pairs[p].split('=');
+      if (pair.length != 2) {
+        throw "Malformed argument key/value pair: '" + pairs[p] + "'";
+      }
+      args[decodeURIComponent(pair[0])] = decodeURIComponent(pair[1]);
+    }
+  }
+  return args;
+}
+
+
+function addDefaultsToArgs(defaults, args) {
+  for (var key in defaults) {
+    if (!(key in args)) {
+      args[key] = defaults[key];
+    }
+  }
+}
+
+
+// Return a dictionary of arguments for the test.  These arguments are passed
+// in the query string of the main page's URL.  Any time this function is used,
+// default values should be provided for every argument.  In some cases a test
+// may be run without an expected query string (manual testing, for example.)
+// Careful: all the keys and values in the dictionary are strings.  You will
+// need to manually parse any non-string values you wish to use.
+function getTestArguments(defaults) {
+  var encoded = window.location.search.substring(1);
+  var args = decodeURIArgs(encoded);
+  if (defaults !== undefined) {
+    addDefaultsToArgs(defaults, args);
+  }
+  return args;
+}
+
+
 function exceptionToLogText(e) {
   if (typeof e == 'object' && 'message' in e && 'stack' in e) {
     return e.message + '\n' + e.stack.toString();
@@ -22,7 +79,7 @@ function exceptionToLogText(e) {
 // Logs test results to the server using URL-encoded RPC.
 // Also logs the same test results locally into the DOM.
 function RPCWrapper() {
-  // Workaround how JS binds 'this'
+  // Work around how JS binds 'this'
   var this_ = this;
   // It is assumed RPC will work unless proven otherwise.
   this.rpc_available = true;
@@ -43,8 +100,10 @@ function RPCWrapper() {
 
   function handleRPCResponse(name, req) {
     if (req.status == 200) {
-      // URL-encoded RPC always responds 'OK'.  If we get anything else, worry.
-      if (req.responseText != 'OK') {
+      if (req.responseText == 'Die, please') {
+        // TODO(eugenis): this does not end the browser process on Mac.
+        window.close();
+      } else if (req.responseText != 'OK') {
         this_.logLocal('Unexpected RPC response to ' + name + ': \'' +
                        req.responseText + '\' - If you are running this test ' +
                        'manually, this is not a problem.', 'gray');
@@ -131,6 +190,10 @@ function RPCWrapper() {
     rpcCall('Ping', {});
   }
 
+  this.heartbeat = function() {
+    rpcCall('JavaScriptIsAlive', {});
+  }
+
   this.client_error = function(message) {
     this.num_errors += 1;
     this.visualError();
@@ -148,16 +211,23 @@ function RPCWrapper() {
     this._log(full_message, 'blue');
   }
 
-  // TODO(ncbray) make this the main interface
-  this._log = function(message, color) {
+  this._log = function(message, color, from_completed_test) {
     if (typeof(message) != 'string') {
       message = toString(message);
     }
+
+    // For event-driven tests, output may come after the test has finished.
+    // Display this in a special way to assist debugging.
+    if (from_completed_test) {
+      color = 'orange';
+      message = 'completed test: ' + message;
+    }
+
     this.logLocal(message, color);
     rpcCall('TestLog', {message: message});
   }
 
-  this.log = function(test_name, message) {
+  this.log = function(test_name, message, from_completed_test) {
     if (message == undefined) {
       // This is a log message that is not assosiated with a test.
       // What we though was the test name is actually the message.
@@ -166,29 +236,30 @@ function RPCWrapper() {
       if (typeof(message) != 'string') {
         message = toString(message);
       }
-      this._log('[' + test_name + ' LOG] ' + message);
+      var full_message = '[' + test_name + ' LOG] ' + message;
+      this._log(full_message, 'black', from_completed_test);
     }
   }
 
-  this.fail = function(test_name, message) {
+  this.fail = function(test_name, message, from_completed_test) {
     this.num_failed += 1;
     this.visualError();
     var full_message = '[' + test_name + ' FAIL] ' + message
-    this._log(full_message, 'red');
+    this._log(full_message, 'red', from_completed_test);
   }
 
-  this.exception = function(test_name, err) {
+  this.exception = function(test_name, err, from_completed_test) {
     this.num_errors += 1;
     this.visualError();
     var message = exceptionToLogText(err);
     var full_message = '[' + test_name + ' EXCEPTION] ' + message;
-    this._log(full_message, 'purple');
+    this._log(full_message, 'purple', from_completed_test);
   }
 
-  this.pass = function(test_name) {
+  this.pass = function(test_name, from_completed_test) {
     this.num_passed += 1;
     var full_message = '[' + test_name + ' PASS]';
-    this._log(full_message, 'green');
+    this._log(full_message, 'green', from_completed_test);
   }
 
   this.blankLine = function() {
@@ -245,7 +316,7 @@ function RPCWrapper() {
 //
 
 
-function fail(message, info) {
+function fail(message, info, test_status) {
   var parts = [];
   if (message != undefined) {
     parts.push(message);
@@ -253,13 +324,21 @@ function fail(message, info) {
   if (info != undefined) {
     parts.push('(' + info + ')');
   }
-  throw {type: 'test_fail', message: parts.join(' ')};
+  var full_message = parts.join(' ');
+
+  if (test_status !== undefined) {
+    // New-style test
+    test_status.fail(full_message);
+  } else {
+    // Old-style test
+    throw {type: 'test_fail', message: full_message};
+  }
 }
 
 
-function assert(condition, message) {
+function assert(condition, message, test_status) {
   if (!condition) {
-    fail(message, toString(condition));
+    fail(message, toString(condition), test_status);
   }
 }
 
@@ -293,24 +372,30 @@ function toString(obj) {
 }
 
 
-function assertEqual(a, b, message) {
+// Old-style, but new-style tests use it indirectly.
+// (The use of the "test" parameter indicates a new-style test.  This is a
+// temporary hack to avoid code duplication.)
+function assertEqual(a, b, message, test_status) {
   if (isArray(a) && isArray(b)) {
-    assertArraysEqual(a, b, message);
+    assertArraysEqual(a, b, message, test_status);
   } else if (a !== b) {
-    fail(message, toString(a) + ' != ' + toString(b));
+    fail(message, toString(a) + ' != ' + toString(b), test_status);
   }
 }
 
 
-function assertArraysEqual(a, b, message) {
+// Old-style, but new-style tests use it indirectly.
+// (The use of the "test" parameter indicates a new-style test.  This is a
+// temporary hack to avoid code duplication.)
+function assertArraysEqual(a, b, message, test_status) {
   var dofail = function() {
-    fail(message, toString(a) + ' != ' + toString(b));
+    fail(message, toString(a) + ' != ' + toString(b), test_status);
   }
   if (a.length != b.length) {
     dofail();
   }
   for (var i = 0; i < a.length; i++) {
-    if(a[i] != b[i]) {
+    if (a[i] !== b[i]) {
       dofail();
     }
   }
@@ -320,23 +405,13 @@ function assertArraysEqual(a, b, message) {
 // Ideally there'd be some way to identify what exception was thrown, but JS
 // exceptions are fairly ad-hoc.
 // TODO(ncbray) allow manual validation of exception types?
-function assertRaises(func, message) {
+function assertRaises(func, message, test_status) {
   try {
     func();
   } catch (err) {
     return;
   }
-  fail(message, 'did not raise');
-}
-
-
-function halt_and_pass() {
-  throw {type: 'test_pass'};
-}
-
-
-function halt_and_callback(time, callback) {
-  throw {type: 'test_wait', time: time, callback: callback};
+  fail(message, 'did not raise', test_status);
 }
 
 
@@ -345,6 +420,11 @@ function halt_and_callback(time, callback) {
 //
 
 
+function haltAsyncTest() {
+  throw {type: 'test_halt'};
+}
+
+
 function begins_with(s, prefix) {
   if (s.length >= prefix.length) {
     return s.substr(0, prefix.length) == prefix;
@@ -473,9 +553,9 @@ function NaClWaiter(body_element) {
         this.pingCallback();
     }
   }
-
 }
 
+
 function logLoadStatus(rpc, load_errors_are_test_errors, loaded, waiting) {
   for (var i = 0; i < loaded.length; i++) {
     rpc.log(embed_name(loaded[i]) + ' loaded');
@@ -488,6 +568,8 @@ function logLoadStatus(rpc, load_errors_are_test_errors, loaded, waiting) {
       return '<exception>';
     }
   }
+
+  var errored = false;
   for (var j = 0; j < waiting.length; j++) {
     var name = getCarefully(function(){
         return embed_name(waiting[j]);
@@ -501,16 +583,179 @@ function logLoadStatus(rpc, load_errors_are_test_errors, loaded, waiting) {
         return toString(waiting[j].lastError);
       });
     var msg = (name + ' did not load. Status: ' + ready + ' / ' + last);
-    if(load_errors_are_test_errors) {
+    if (load_errors_are_test_errors) {
       rpc.client_error(msg);
+      errored = true;
     } else {
       rpc.log(msg);
     }
   }
+  return errored;
 }
 
+
+// Contains the state for a single test.
+function TestStatus(tester, name, async) {
+  // Work around how JS binds 'this'
+  var this_ = this;
+  this.tester = tester;
+  this.name = name;
+  this.async = async;
+  this.running = true;
+
+  this.log = function(message) {
+    this.tester.rpc.log(this.name, toString(message), !this.running);
+  }
+
+  this.pass = function() {
+    // TODO raise if not running.
+    this.tester.rpc.pass(this.name, !this.running);
+    this._done();
+    haltAsyncTest();
+  }
+
+  this.fail = function(message) {
+    this.tester.rpc.fail(this.name, message, !this.running);
+    this._done();
+    haltAsyncTest();
+  }
+
+  this._done = function() {
+    if (this.running) {
+      this.running = false;
+      this.tester.testDone(this);
+    }
+  }
+
+  this.assert = function(condition, message) {
+    assert(condition, message, this);
+  }
+
+  this.assertEqual = function(a, b, message) {
+    assertEqual(a, b, message, this);
+  }
+
+  this.callbackWrapper = function(callback, args) {
+    // A stale callback?
+    if (!this.running)
+      return;
+
+    if (args === undefined)
+      args = [];
+
+    try {
+      callback.apply(undefined, args);
+    } catch (err) {
+      if (typeof err == 'object' && 'type' in err) {
+        if (err.type == 'test_halt') {
+          // New-style test
+          // If we get this exception, we can assume any callbacks or next
+          // tests have already been scheduled.
+          return;
+        } else if (err.type == 'test_fail') {
+          // Old-style test
+          // A special exception that terminates the test with a failure
+          this.tester.rpc.fail(this.name, err.message, !this.running);
+          this._done();
+          return;
+        }
+      }
+      // This is not a special type of exception, it is an error.
+      this.tester.rpc.exception(this.name, err, !this.running);
+      this._done();
+      return;
+    }
+
+    // A normal exit.  Should we move on to the next test?
+    // Async tests do not move on without an explicit pass.
+    if (!this.async) {
+      this.tester.rpc.pass(this.name);
+      this._done();
+    }
+  }
+
+  // Async callbacks should be wrapped so the tester can catch unexpected
+  // exceptions.
+  this.wrap = function(callback) {
+    return function() {
+      this_.callbackWrapper(callback, arguments);
+    };
+  }
+
+  this.setTimeout = function(callback, time) {
+    setTimeout(this.wrap(callback), time);
+  }
+
+  this.waitForCallback = function(callbackName, expectedCalls) {
+    this.log('Waiting for ' + expectedCalls + ' invocations of callback: '
+               + callbackName);
+    var gotCallbacks = 0;
+
+    // Deliberately global - this is what the nexe expects.
+    // TODO(ncbray): consider returning this function, so the test has more
+    // flexibility. For example, in the test one could count to N
+    // using a different callback before calling _this_ callback, and
+    // continuing the test. Also, consider calling user-supplied callback
+    // when done waiting.
+    window[callbackName] = this.wrap(function() {
+      ++gotCallbacks;
+      this_.log('Received callback ' + gotCallbacks);
+      if (gotCallbacks == expectedCalls) {
+        this_.log("Done waiting");
+        this_.pass();
+      } else {
+        // HACK
+        haltAsyncTest();
+      }
+    });
+
+    // HACK if this function is used in a non-async test, make sure we don't
+    // spuriously pass.  Throwing this exception forces us to behave like an
+    // async test.
+    haltAsyncTest();
+  }
+
+  // This function takes an array of messages and asserts that the nexe
+  // calls PostMessage with each of these messages, in order.
+  // Arguments:
+  //   plugin - The DOM object for the NaCl plugin
+  //   messages - An array of expected responses
+  //   callback - An optional callback function that takes the current message
+  //              string as an argument
+  this.expectMessageSequence = function(plugin, messages, callback) {
+    this.assert(messages.length > 0, 'Must provide at least one message');
+    var local_messages = messages.slice();
+    var listener = function(message) {
+      if (message.data.indexOf('@:') == 0) {
+        // skip debug messages
+        this_.log('DEBUG: ' + message.data.substr(2));
+      } else {
+        this_.assertEqual(message.data, local_messages.shift());
+        if (callback !== undefined) {
+          callback(message.data);
+        }
+      }
+      if (local_messages.length == 0) {
+        this_.pass();
+      } else {
+        this_.expectEvent(plugin, 'message', listener);
+      }
+    }
+    this.expectEvent(plugin, 'message', listener);
+  }
+
+  this.expectEvent = function(src, event_type, listener) {
+    var wrapper = this.wrap(function(e) {
+      src.removeEventListener(event_type, wrapper, false);
+      listener(e);
+    });
+    src.addEventListener(event_type, wrapper, false);
+  }
+}
+
+
 function Tester(body_element) {
-  // Workaround how JS binds 'this'
+  // Work around how JS binds 'this'
   var this_ = this;
   // The tests being run.
   var tests = [];
@@ -519,6 +764,8 @@ function Tester(body_element) {
 
   var load_errors_are_test_errors = true;
 
+  var parallel = false;
+
   //
   // BEGIN public interface
   //
@@ -528,48 +775,33 @@ function Tester(body_element) {
   }
 
   this.log = function(message) {
-    this.rpc.log(this.currentTest, message);
+    this.rpc.log(message);
   }
 
+  // If this kind of test exits cleanly, it passes
   this.addTest = function(name, testFunction) {
-    tests.push({name: name, callback: testFunction});
+    tests.push({name: name, callback: testFunction, async: false});
   }
 
-  // TODO(ncbray): implement parallelized async test harness
-  this.waitForCallback = function(callbackName, expectedCalls) {
-    this_.log('Waiting for ' + expectedCalls + ' invocations of callback: '
-               + callbackName);
-    var gotCallbacks = 0;
-
-    // Deliberately global - this what the nexe expects.
-    // TODO(ncbray): consider returning this function, so the test has more
-    // flexibility. For example, in the test one could count to N
-    // using a different callback before calling _this_ callback, and
-    // continuing the test. Also, consider calling user-supplieed callback
-    // when done waiting.
-    window[callbackName] = function() {
-      ++gotCallbacks;
-      this_.log('Received callback ' + gotCallbacks);
-    };
-
-    function wait() {
-      if (gotCallbacks < expectedCalls) {
-        halt_and_callback(1, wait);
-      } else {
-        this_.log("Done waiting");
-      }
-    }
-
-    wait();
+  // This kind of test does not pass until "pass" is explicitly called.
+  this.addAsyncTest = function(name, testFunction) {
+    tests.push({name: name, callback: testFunction, async: true});
   }
 
   this.run = function() {
     this.rpc.startup();
-
+    this.startHeartbeat();
     this.waiter.run(
       function(loaded, waiting) {
-        logLoadStatus(this_.rpc, load_errors_are_test_errors, loaded, waiting);
-        this_.startTesting();
+        var errored = logLoadStatus(this_.rpc, load_errors_are_test_errors,
+                                    loaded, waiting);
+        if (errored) {
+          this_.rpc.blankLine();
+          this_.rpc.log('A nexe load error occured, aborting testing.');
+          this_._done();
+        } else {
+          this_.startTesting();
+        }
       },
       function() {
         this_.rpc.ping();
@@ -577,6 +809,11 @@ function Tester(body_element) {
     );
   }
 
+  this.runParallel = function() {
+    parallel = true;
+    this.run();
+  }
+
   // Takes an arbitrary number of arguments.
   this.waitFor = function() {
     for (var i = 0; i< arguments.length; i++) {
@@ -588,89 +825,58 @@ function Tester(body_element) {
   // END public interface
   //
 
-  this.initTest = function() {
-    if (this.testIndex < tests.length) {
-      // Setup for the next test.
-      var test = tests[this.testIndex];
-      this.currentTest = test.name;
-      this.rpc.blankLine();
-      this.rpc.begin(this.currentTest);
-      this.setNextStep(0, test.callback);
-    } else {
-      // There are no more test suites.
-      this.rpc.blankLine();
-      this.rpc.shutdown();
+  this.startHeartbeat = function() {
+    var rpc = this.rpc;
+    var heartbeat = function() {
+      rpc.heartbeat();
+      setTimeout(heartbeat, 500);
     }
+    heartbeat();
   }
 
-  this.setNextStep = function(time, callback) {
-    // Give up control
-    // There are three reasons for this:
-    // 1) It allows the render thread to update (can't see the log, otherwise)
-    // 2) It breaks up a "long running process" so the browser
-    //    won't think it crashed
-    // 3) It allows us to sleep for a specified interval
-    // TODO it might not be desirable to relinquish control between each test.
-    // Running several tests in a row might make things faster.
-    this.nextStep = callback;
-    setTimeout(function() { this_.runTestStep(); }, time);
+  this.launchTest = function(testIndex) {
+    var testDecl = tests[testIndex];
+    var currentTest = new TestStatus(this, testDecl.name, testDecl.async);
+    setTimeout(currentTest.wrap(function() {
+      this_.rpc.blankLine();
+      this_.rpc.begin(currentTest.name);
+      testDecl.callback(currentTest);
+    }), 0);
+  }
+
+  this._done = function() {
+    this.rpc.blankLine();
+    this.rpc.shutdown();
   }
 
   this.startTesting = function() {
-    // if tests[0] does not exist (no tests), initTest will shut down testing.
-    this.testIndex = 0;
-    this.initTest();
-  }
-
-  this.nextTest = function() {
-    // Advance to the next test suite
-    this.testIndex += 1;
-    this.initTest()
-  }
-
-  this.runTestStep = function() {
-    callback = this.nextStep;
-    // Be paranoid - avoid accidental infinite loops
-    this.nextStep = null;
-    // Should we move on to the next test?
-    var done = true;
-    // Did the text exit abnormally
-    var exceptionHandled = false;
-
-    try {
-      callback();
-    } catch (err) {
-      if (typeof err == 'object') {
-        if ('type' in err) {
-          if (err.type == 'test_fail') {
-            // A special exception that terminates the test with a failure
-            this.rpc.fail(this.currentTest, err.message);
-            exceptionHandled = true;
-          } else if (err.type == 'test_pass') {
-            // A special exception that terminates the test without failing
-            this.rpc.pass(this.currentTest);
-            exceptionHandled = true;
-          } else if (err.type == 'test_wait') {
-            // A special exception that indicates the test wants a callback
-            this.setNextStep(err.time, err.callback);
-            done = false;
-            exceptionHandled = true;
-          }
-        }
-      }
-      if (!exceptionHandled) {
-        // This is not a special type of exception, it is an error.
-        this.rpc.exception(this.currentTest, err);
-        exceptionHandled = true
-      }
+    if (tests.length == 0) {
+      // No tests specified.
+      this._done();
+      return;
     }
 
-    if (done) {
-      if (!exceptionHandled) {
-        // No exception, the test passed
-        this.rpc.pass(this.currentTest);
+    this.testCount = 0;
+    if (parallel) {
+      // Launch all tests.
+      for (var i = 0; i < tests.length; i++) {
+        this.launchTest(i);
       }
-      this.nextTest();
+    } else {
+      // Launch the first test.
+      this.launchTest(0);
+    }
+  }
+
+  this.testDone = function(test) {
+    this.testCount += 1;
+    if (this.testCount < tests.length) {
+      if (!parallel) {
+        // Move on to the next test if they're being run one at a time.
+        this.launchTest(this.testCount);
+      }
+    } else {
+      this._done();
     }
   }
 }
