@@ -12,7 +12,7 @@
 from driver_tools import *
 
 EXTRA_ENV = {
-  'ALLOW_TRANSLATE': '0',  # Allow .bc to be translated to .o (before linking).
+  'ALLOW_TRANSLATE': '0',  # Allow bitcode translation before linking.
                            # It doesn't normally make sense to do this.
 
   'ALLOW_NATIVE'   : '0',  # Allow native objects (.S,.s,.o) to be in the
@@ -36,8 +36,8 @@ EXTRA_ENV = {
   'STDINC'      : '1',    # Include standard headers (-nostdinc sets to 0)
   'STDLIB'      : '1',    # Include standard libraries (-nostdlib sets to 0)
   'DIAGNOSTIC'  : '0',    # Diagnostic flag detected
-  'STATIC'      : '${USE_GLIBC ? 0 : 1}', # -static (on by default for newlib)
-  'PIC'         : '0',   # Generate PIC
+  'STATIC'      : '${LIBMODE_NEWLIB ? 1 : 0}', # -static (on for newlib)
+  'PIC'         : '0',    # Generate PIC
 
   'INPUTS'      : '',    # Input files
   'OUTPUT'      : '',    # Output file
@@ -52,7 +52,7 @@ EXTRA_ENV = {
                   '-U__ARMEL__ -U__THUMBEL__ -U__SOFTFP__ ' +
                   '-U__VFP_FP__ -U__ARM_NEON__ -U__ARM_FP16 ' +
                   '-U__ARM_FP16__ -U__MAVERICK__ -U__XSCALE__ -U__IWMMXT__ ' +
-                  '-U__ARM_EABI__',
+                  '-U__ARM_EABI__ -U__ARM_ARCH_7A__',
 
   'BIAS_NONE'   : '${REMOVE_BIAS}',
   'BIAS_ARM'    : '',
@@ -61,101 +61,91 @@ EXTRA_ENV = {
   'BIAS_X8664'  : '${REMOVE_BIAS} ' +
                   '-D__amd64__ -D__amd64 -D__x86_64__ -D__x86_64 -D__core2__',
 
-  'OPT_LEVEL'   : '',
-  'CC_FLAGS'    : '${OPT_LEVEL} -fuse-llvm-va-arg -Werror-portable-llvm ' +
+  'OPT_LEVEL'   : '0',
+  'CC_FLAGS'    : '-O${OPT_LEVEL} -fuse-llvm-va-arg -Werror-portable-llvm ' +
                   '-nostdinc -DNACL_LINUX=1 -D__native_client__=1 ' +
                   '-D__pnacl__=1 ${BIAS_%BIAS%}',
-  'CC_STDINC'   :
-    # NOTE: the two competing approaches here
-    #       make the gcc driver "right" or
-    #       put all the logic/knowledge into this driver.
-    #       Currently, we have a messy mixture.
-    '-isystem ${BASE_ARM}/lib/gcc/arm-none-linux-gnueabi/4.2.1/include ' +
-    '-isystem ' +
-    '${BASE_ARM}/lib/gcc/arm-none-linux-gnueabi/4.2.1/install-tools/include ' +
-    '-isystem ${BASE_ARM}/include/c++/4.2.1 ' +
-    '-isystem ${BASE_ARM}/include/c++/4.2.1/arm-none-linux-gnueabi ' +
-    '-isystem ${BASE_ARM_INCLUDE} ' +
-    # NOTE: order important
-    # '-isystem',
-    # BASE + '/arm-newlib/arm-none-linux-gnueabi/usr/include/nacl/abi',
-    '-isystem ${BASE}/arm-newlib/arm-none-linux-gnueabi/include',
 
-  # ${LIBS} must come before ${LIBS_BC} so that the native glibc/libnacl
-  # takes precedence over bitcode libc.
-  'LD_FLAGS' : '${STATIC ? -static} ${SHARED ? -shared} ${PIC ? -fPIC} ' +
-               '-L${LIBS} -L${LIBS_BC}',
+  'ISYSTEM'        : '${ISYSTEM_USER} ${ISYSTEM_BUILTIN}',
+  'ISYSTEM_BUILTIN': '${STDINC ? ${ISYSTEM_%LIBMODE%}}',
+  'ISYSTEM_USER'   : '',  # System include directories specified by
+                          # using the -isystem flag.
+
+  'ISYSTEM_newlib' :
+    '${BASE_SDK}/include ' +
+    '${BASE_LLVM_GCC}/lib/gcc/arm-none-linux-gnueabi/4.2.1/include ' +
+    '${BASE_LLVM_GCC}/' +
+      'lib/gcc/arm-none-linux-gnueabi/4.2.1/install-tools/include ' +
+    '${BASE_LIBSTDCPP}/include/c++/4.2.1 ' +
+    '${BASE_LIBSTDCPP}/include/c++/4.2.1/arm-none-linux-gnueabi ' +
+    '${BASE_INCLUDE} ' +
+    '${BASE_NEWLIB}/arm-none-linux-gnueabi/include',
+
+  'ISYSTEM_glibc' :
+    '${BASE_SDK}/include ' +
+    '${BASE_GLIBC}/include ' +
+    '${BASE_LLVM_GCC}/lib/gcc/arm-none-linux-gnueabi/4.2.1/include ' +
+    '${BASE_LLVM_GCC}/' +
+      'lib/gcc/arm-none-linux-gnueabi/4.2.1/install-tools/include ' +
+    '${BASE_LIBSTDCPP}/include/c++/4.2.1 ' +
+    '${BASE_LIBSTDCPP}/include/c++/4.2.1/arm-none-linux-gnueabi',
+
+  'LD_FLAGS' : '-O${OPT_LEVEL} ${STATIC ? -static} ${SHARED ? -shared} ' +
+               '${PIC ? -fPIC} ${@AddPrefix:-L:SEARCH_DIRS}',
+
+  'SEARCH_DIRS'      : '${SEARCH_DIRS_USER} ${PREFIXES}',
+  'SEARCH_DIRS_USER' : '', # Directories specified using -L
+  'PREFIXES'         : '', # Prefixes specified by using the -B flag.
 
   # Library Strings
-  'LIBMODE'          : '${USE_GLIBC ? glibc : newlib}',
   'EMITMODE'         : '${STATIC ? static : ${SHARED ? shared : dynamic}}',
 
-  # Standard Library Directories
-  'LIBS_BC'          : '${BASE}/libs-bitcode',
-
-  'LIBS'             : '${LIBS_%ARCH%_%LIBMODE%}',
-  'LIBS_ARM_newlib'  : '${BASE}/libs-arm-newlib',
-  'LIBS_X8632_newlib': '${BASE}/libs-x8632-newlib',
-  'LIBS_X8664_newlib': '${BASE}/libs-x8664-newlib',
-
-  'LIBS_X8632_glibc': '${BASE}/libs-x8632-glibc',
-  'LIBS_X8664_glibc': '${BASE}/libs-x8664-glibc',
-  'LIBS_ARM_glibc'  : '${BASE}/libs-arm-glibc',
-
   'LD_ARGS' : '${STDLIB ? ${LD_ARGS_%LIBMODE%_%EMITMODE%}' +
                       ' : ${LD_ARGS_nostdlib}}',
 
-  # This information is also in pnacl-ld.
-  # This is currently needed here because we have to manually specify a linker
-  # script for glibc linking.
-  # TODO(pdox): Fix binutils so that the built-in linker scripts are correct.
-  'EMUL'        : '${EMUL_%ARCH%}',
-  'EMUL_ARM'    : 'armelf_nacl',
-  'EMUL_X8632'  : 'elf_nacl',
-  'EMUL_X8664'  : 'elf64_nacl',
-
   # ${ld_inputs} signifies where to place the objects and libraries
   # provided on the command-line.
-  'LD_ARGS_nostdlib': '${ld_inputs}',
+  'LD_ARGS_nostdlib': '-barebones-link -nostdlib ${ld_inputs}',
 
   'LD_ARGS_newlib_static':
-    '-static ${LIBS}/crt1.o ${LIBS_BC}/nacl_startup.bc ${ld_inputs} ' +
+    '-l:crt1.o -l:nacl_startup.bc ${ld_inputs} ' +
     '--start-group -lgcc_eh -lgcc -lc -lnacl ' +
-    '${LIBSTDCPP} ${LIBS}/libcrt_platform.a --end-group',
+    '${LIBSTDCPP} -l:libcrt_platform.a --end-group',
 
   # The next three are copied verbatim from nacl-gcc
   'LD_ARGS_glibc_static':
-    '-T ${LIBS}/${EMUL}.x.static ' +
-    '${LIBS}/crt1.o ${LIBS}/crti.o ${LIBS}/crtbeginT.o ' +
-    '${ld_inputs} ' +
-    '--start-group -lgcc -lgcc_eh -lc -lnacl ${LIBSTDCPP} -lcrt_platform ' +
-    '--end-group ${LIBS}/crtend.o ${LIBS}/crtn.o',
+    '-l:crt1.o -l:crti.o -l:crtbeginT.o ' +
+    '${ld_inputs} ${LIBSTDCPP} ' +
+    '--start-group -lgcc -lgcc_eh -lc ' +
+    '--end-group -l:crtend.o -l:crtn.o',
 
   'LD_ARGS_glibc_shared':
-    '-T ${LIBS}/${EMUL}.xs ' +
-    '--eh-frame-hdr -shared ${LIBS}/crti.o ${LIBS}/crtbeginS.o ' +
-    '${ld_inputs} ' +
-    '-lgcc --as-needed -lgcc_s --no-as-needed -lc -lnacl -lc -lcrt_platform ' +
-    '-lc -lgcc --as-needed -lgcc_s --no-as-needed ' +
-    '${LIBS}/crtendS.o ${LIBS}/crtn.o -rpath ${LIBS}',
+    '--eh-frame-hdr -shared -l:crti.o -l:crtbeginS.o ' +
+    '${ld_inputs} ${LIBSTDCPP} ' +
+    '-lgcc -lgcc_eh -lc -lgcc -lgcc_eh ' +
+    # When shared libgcc is ready, use this instead:
+    # '-lgcc --as-needed -lgcc_s --no-as-needed ' +
+    # '-lc -lgcc --as-needed -lgcc_s --no-as-needed ' +
+    '-l:crtendS.o -l:crtn.o',
 
   'LD_ARGS_glibc_dynamic':
-    '-T ${LIBS}/${EMUL}.x ' +
-    '--eh-frame-hdr ${LIBS}/crt1.o ${LIBS}/crti.o ' +
-    '${LIBS}/crtbegin.o ${ld_inputs} ' +
-    '-lgcc --as-needed -lgcc_s --no-as-needed ' +
-    '-lc -lnacl -lc ${LIBSTDCPP} -lcrt_platform -lc -lgcc --as-needed ' +
-    '-lgcc_s --no-as-needed ${LIBS}/crtend.o ${LIBS}/crtn.o -rpath ${LIBS}',
+    '--eh-frame-hdr -l:crt1.o -l:crti.o ' +
+    '-l:crtbegin.o ${ld_inputs} ${LIBSTDCPP} ' +
+    '-lgcc -lgcc_eh -lc -lgcc -lgcc_eh ' +
+    # When shared libgcc is ready, use this instead:
+    # '-lgcc --as-needed -lgcc_s --no-as-needed ' +
+    # '-lc -lgcc --as-needed -lgcc_s --no-as-needed ' +
+    '-l:crtend.o -l:crtn.o',
 
-  'LIBSTDCPP'       : '${USE_GXX ? -lstdc++ }',
+  'LIBSTDCPP'       : '${USE_GXX ? -lstdc++ -lm }',
 
   'CC' : '${USE_GXX ? ${LLVM_GXX} : ${LLVM_GCC}}',
 
   'RUN_CC': '${CC} -emit-llvm ${mode} ${CC_FLAGS} ' +
-            '${STDINC ? ${CC_STDINC}} ' +
+            '${@AddPrefix:-isystem :ISYSTEM} ' +
             '${input} -o ${output}',
 
-  'RUN_PP' : '${CC} -E ${CC_FLAGS} ${CC_STDINC} ' +
+  'RUN_PP' : '${CC} -E ${CC_FLAGS} ${@AddPrefix:-isystem :ISYSTEM} ' +
              '"${input}" -o "${output}"'
 }
 env.update(EXTRA_ENV)
@@ -166,19 +156,31 @@ def AddLDFlag(*args):
 def AddCCFlag(*args):
   env.append('CC_FLAGS', *args)
 
+def AddBPrefix(prefix):
+  prefix = pathtools.normalize(prefix)
+  if pathtools.isdir(prefix) and not prefix.endswith('/'):
+    prefix += '/'
+
+  env.append('PREFIXES', prefix)
+
+  # Add prefix/include to isystem if it exists
+  include_dir = prefix + 'include'
+  if pathtools.isdir(include_dir):
+    env.append('ISYSTEM_USER', include_dir)
+
 CustomPatterns = [
-  ( '--driver=(.+)',                   "env.set('CC', $0)\n"),
-  ( '--pnacl-skip-ll',                 "env.set('EMIT_LL', '0')"),
-  ( '--pnacl-gxx',                     "env.set('USE_GXX', '1')"),
-  ( '--pnacl-allow-native',            "env.set('ALLOW_NATIVE', '1')"),
-  ( '--pnacl-allow-translate',         "env.set('ALLOW_TRANSLATE', '1')"),
+  ( '--driver=(.+)',             "env.set('CC', pathtools.normalize($0))\n"),
+  ( '--pnacl-skip-ll',           "env.set('EMIT_LL', '0')"),
+  ( '--pnacl-gxx',               "env.set('USE_GXX', '1')"),
+  ( '--pnacl-allow-native',      "env.set('ALLOW_NATIVE', '1')"),
+  ( '--pnacl-allow-translate',   "env.set('ALLOW_TRANSLATE', '1')"),
   ( '--pnacl-native-hack', "env.append('LD_FLAGS', '--pnacl-native-hack')\n"
                            "env.set('ALLOW_NATIVE', '1')"),
 ]
 
 GCCPatterns = [
-  ( '-o(.+)',          "env.set('OUTPUT', $0)"),
-  ( ('-o', '(.+)'),    "env.set('OUTPUT', $0)"),
+  ( '-o(.+)',          "env.set('OUTPUT', pathtools.normalize($0))"),
+  ( ('-o', '(.+)'),    "env.set('OUTPUT', pathtools.normalize($0))"),
 
   ( '-E',              "env.set('GCC_MODE', '-E')"),
   ( '-S',              "env.set('GCC_MODE', '-S')"),
@@ -197,36 +199,41 @@ GCCPatterns = [
   ( '(-l.+)',             "env.append('INPUTS', $0)"),
   ( ('-Xlinker','(.*)'),  "env.append('INPUTS', '-Xlinker=' + $0)"),
   ( '(-Wl,.*)',           "env.append('INPUTS', $0)"),
+  ( '(-Bstatic)',         "env.append('INPUTS', $0)"),
+  ( '(-Bdynamic)',        "env.append('INPUTS', $0)"),
 
-  # This is currently only used for the front-end, but we may want to have
-  # this control LTO optimization level as well.
-  ( '(-O.+)',          "env.set('OPT_LEVEL', $0)"),
+  ( '-O([0-4s])',         "env.set('OPT_LEVEL', $0)\n"),
+  ( '-O',                 "env.set('OPT_LEVEL', 1)\n"),
+
+  ( ('-isystem', '(.*)'),
+                       "env.append('ISYSTEM_USER', pathtools.normalize($0))"),
+  ( ('-I', '(.+)'),    "env.append('CC_FLAGS', '-I'+pathtools.normalize($0))"),
+  ( '-I(.+)',          "env.append('CC_FLAGS', '-I'+pathtools.normalize($0))"),
 
   ( '(-g)',                   AddCCFlag),
   ( '(-W.*)',                 AddCCFlag),
   ( '(-std=.*)',              AddCCFlag),
-  ( '(-B.*)',                 AddCCFlag),
   ( '(-D.*)',                 AddCCFlag),
   ( '(-f.*)',                 AddCCFlag),
-  ( ('(-I)', '(.+)'),         AddCCFlag),
-  ( '(-I.+)',                 AddCCFlag),
   ( '(-pedantic)',            AddCCFlag),
-  ( ('(-isystem)', '(.+)'),   AddCCFlag),
   ( '(-g.*)',                 AddCCFlag),
   ( '(-xassembler-with-cpp)', AddCCFlag),
 
   ( '-shared',                "env.set('SHARED', '1')"),
   ( '-static',                "env.set('STATIC', '1')"),
 
-  ( ('(-L)','(.+)'),          AddLDFlag),
-  ( '(-L.+)',                 AddLDFlag),
-  ( '(-Wp,.*)',               AddCCFlag),
+  ( ('-B','(.*)'),            AddBPrefix),
+  ( ('-B(.+)'),               AddBPrefix),
 
-  ( '(-MMD)',                 AddCCFlag),
-  ( '(-MP)',                  AddCCFlag),
-  ( '(-MD)',                  AddCCFlag),
-  ( ('(-MF)','(.*)'),         AddCCFlag),
-  ( ('(-MT)','(.*)'),         AddCCFlag),
+  ( ('-L','(.+)'), "env.append('SEARCH_DIRS_USER', pathtools.normalize($0))"),
+  ( '-L(.+)',      "env.append('SEARCH_DIRS_USER', pathtools.normalize($0))"),
+
+  ( '(-Wp,.*)',       AddCCFlag),
+  ( '(-MMD)',         AddCCFlag),
+  ( '(-MP)',          AddCCFlag),
+  ( '(-MD)',          AddCCFlag),
+  ( ('(-MT)','(.*)'), AddCCFlag),
+  ( ('(-MF)','(.*)'), "env.append('CC_FLAGS', $0, pathtools.normalize($1))"),
 
   # With -xc, GCC considers future inputs to be an input source file.
   # The following is not the correct handling of -x,
@@ -258,7 +265,7 @@ GCCPatterns = [
   ( '(-print-.*)',            "env.set('DIAGNOSTIC', '1')"),
   ( '(--print.*)',            "env.set('DIAGNOSTIC', '1')"),
   ( '(-dumpspecs)',           "env.set('DIAGNOSTIC', '1')"),
-  ( '(-v|--version|--v|-V)',  "env.set('DIAGNOSTIC', '1')"),
+  ( '(-v|--v|--version|-V)',  "env.set('DIAGNOSTIC', '1')"),
   ( '(-d.*)',                 "env.set('DIAGNOSTIC', '1')"),
 
   # Catch all other command-line arguments
@@ -269,8 +276,8 @@ GCCPatterns = [
   # parsed on the command-line. This ensures that the gcc "-x"
   # setting is correctly applied.
 
-  ( '(.*)',                   "env.append('INPUTS', $0)\n"
-                              "ForceFileType($0)"),
+  ( '(.*)',  "env.append('INPUTS', pathtools.normalize($0))\n"
+             "ForceFileType(pathtools.normalize($0))"),
 ]
 
 
@@ -290,13 +297,12 @@ def main(argv):
   if len(unmatched) > 0:
     UnrecognizedOption(*unmatched)
 
-  use_glibc = env.getbool('USE_GLIBC')
+  libmode_newlib = env.getbool('LIBMODE_NEWLIB')
   is_static = env.getbool('STATIC')
   is_shared = env.getbool('SHARED')
 
-  if not use_glibc and (is_shared or not is_static):
-    Log.Fatal("Can't produce dynamic objects with newlib. Did you want "
-              "--pnacl-use-glibc ?")
+  if libmode_newlib and (is_shared or not is_static):
+    Log.Fatal("Can't produce dynamic objects with newlib.")
 
   if env.getbool('STATIC') and env.getbool('SHARED'):
     Log.Fatal("Can't handle both -static and -shared")
@@ -307,7 +313,9 @@ def main(argv):
   if len(inputs) == 0:
     Log.Fatal('No input files')
 
-  arch = GetArch()
+  # If -arch was given, we are compiling directly to native code
+  compiling_to_native = GetArch() is not None
+
   gcc_mode = env.getone('GCC_MODE')
 
   if env.getbool('SHARED'):
@@ -318,22 +326,22 @@ def main(argv):
     link_output = 'nexe'
 
   output_type_map = {
-    ('-E', True) : 'pp',
-    ('-E', False): 'pp',
-    ('-c', True) : 'bc',
-    ('-c', False): 'o',
-    ('-S', True) : 'll',
-    ('-S', False): 's',
-    ('',   True) : bclink_output,
-    ('',   False): link_output
+    ('-E', False) : 'pp',
+    ('-E', True)  : 'pp',
+    ('-c', False) : 'po',
+    ('-c', True)  : 'o',
+    ('-S', False) : 'll',
+    ('-S', True)  : 's',
+    ('',   False) : bclink_output,
+    ('',   True)  : link_output
   }
 
-  output_type = output_type_map[(gcc_mode, arch is None)]
-  NeedsLinking = (gcc_mode == '')
+  output_type = output_type_map[(gcc_mode, compiling_to_native)]
+  needs_linking = (gcc_mode == '')
 
   # There are multiple input files and no linking is being done.
   # There will be multiple outputs. Handle this case separately.
-  if not NeedsLinking:
+  if not needs_linking:
     if output != '' and len(inputs) > 1:
       Log.Fatal('Cannot have -o with -c, -S, or -E and multiple inputs')
 
@@ -343,10 +351,11 @@ def main(argv):
       intype = FileType(f)
       if ((output_type == 'pp' and intype not in ('src','S')) or
           (output_type == 'll' and intype not in 'src') or
-          (output_type == 'bc' and intype not in ('src','ll')) or
-          (output_type == 's' and intype not in ('src','ll','bc','S')) or
-          (output_type == 'o' and intype not in ('src','ll','bc','S','s'))):
-        Log.Fatal("%s: Unexpected type of file for '%s'", f, gcc_mode)
+          (output_type == 'po' and intype not in ('src','ll')) or
+          (output_type == 's' and intype not in ('src','ll','po','S')) or
+          (output_type == 'o' and intype not in ('src','ll','po','S','s'))):
+        Log.Fatal("%s: Unexpected type of file for '%s'",
+                  pathtools.touser(f), gcc_mode)
 
       if output == '':
         f_output = DefaultOutputName(f, output_type)
@@ -358,20 +367,20 @@ def main(argv):
     return 0
 
   # Linking case
-  assert(NeedsLinking)
+  assert(needs_linking)
   assert(output_type in ('pso','so','pexe','nexe'))
 
   if output == '':
-    output = 'a.out'
+    output = pathtools.normalize('a.out')
   namegen = TempNameGen(inputs, output)
 
-  # Compile all source files (c/c++/ll) to .bc
+  # Compile all source files (c/c++/ll) to .po
   for i in xrange(0, len(inputs)):
     if inputs[i].startswith('-'):
       continue
     intype = FileType(inputs[i])
     if intype in ('src','ll'):
-      inputs[i] = CompileOne(inputs[i], 'bc', namegen)
+      inputs[i] = CompileOne(inputs[i], 'po', namegen)
 
   # Compile all .s/.S to .o
   if env.getbool('ALLOW_NATIVE'):
@@ -382,15 +391,16 @@ def main(argv):
       if intype in ('s','S'):
         inputs[i] = CompileOne(inputs[i], 'o', namegen)
 
-  # We should only be left with .bc and .o and libraries
+  # We should only be left with .po and .o and libraries
   for f in inputs:
     if f.startswith('-'):
       continue
     intype = FileType(f)
-    if intype in ('o','nlib','s','S') and not env.getbool('ALLOW_NATIVE'):
+    if intype in ('o','s','S') or IsNativeArchive(f):
+      if not env.getbool('ALLOW_NATIVE'):
         Log.Fatal('%s: Native object files not allowed in link. '
-                  'Use --pnacl-allow-native to override.', f)
-    assert(intype in ('bc','o','bclib','nlib'))
+                  'Use --pnacl-allow-native to override.', pathtools.touser(f))
+    assert(intype in ('po','o','so','pso','ldscript') or IsArchive(f))
 
   # Fix the user-specified linker arguments
   ld_inputs = []
@@ -402,19 +412,15 @@ def main(argv):
     else:
       ld_inputs.append(f)
 
-  # There's an architecture bias here, since we have to link
-  # against native objects, even for bitcode linking.
-  if GetArch() is None:
-    env.set('ARCH', 'X8632')
-
   # Invoke the linker
   env.set('ld_inputs', *ld_inputs)
+
   ld_args = env.get('LD_ARGS')
   ld_flags = env.get('LD_FLAGS')
+
   RunDriver('pnacl-ld', ld_flags + ld_args + ['-o', output])
   return 0
 
-
 def CompileOne(input, output_type, namegen, output = None):
   if output is None:
     output = namegen.TempNameForInput(input, output_type)
@@ -440,14 +446,15 @@ def RunTranslate(input, output, mode):
   if not env.getbool('ALLOW_TRANSLATE'):
     Log.Fatal('%s: Trying to convert bitcode to an object file before '
               'bitcode linking. This is supposed to wait until '
-              'translation. Use --pnacl-allow-translate to override.', input)
+              'translation. Use --pnacl-allow-translate to override.',
+              pathtools.touser(input))
   args = [mode, input, '-o', output]
   if env.getbool('PIC'):
     args += ['-fPIC']
   RunDriver('pnacl-translate', args)
 
 def SetupChain(chain, input_type, output_type):
-  assert(output_type in ('pp','ll','bc','s','o'))
+  assert(output_type in ('pp','ll','po','s','o'))
   cur_type = input_type
 
   # src -> pp
@@ -464,29 +471,29 @@ def SetupChain(chain, input_type, output_type):
   if cur_type == output_type:
     return
 
-  # ll -> bc
+  # ll -> po
   if cur_type == 'll':
-    chain.add(RunLLVMAS, 'bc')
-    cur_type = 'bc'
+    chain.add(RunLLVMAS, 'po')
+    cur_type = 'po'
   if cur_type == output_type:
     return
 
-  # src -> bc
-  if cur_type == 'src' and output_type == 'bc':
-    chain.add(RunCC, 'bc', mode='-c')
-    cur_type = 'bc'
+  # src -> po
+  if cur_type == 'src' and output_type == 'po':
+    chain.add(RunCC, 'po', mode='-c')
+    cur_type = 'po'
   if cur_type == output_type:
     return
 
-  # bc -> o
-  if env.getbool('MC_DIRECT') and cur_type == 'bc' and output_type == 'o':
+  # po -> o
+  if env.getbool('MC_DIRECT') and cur_type == 'po' and output_type == 'o':
     chain.add(RunTranslate, 'o', mode='-c')
     cur_type = 'o'
   if cur_type == output_type:
     return
 
-  # bc -> s
-  if cur_type == 'bc':
+  # po -> s
+  if cur_type == 'po':
     chain.add(RunTranslate, 's', mode='-S')
     cur_type = 's'
   if cur_type == output_type:
