@@ -7,6 +7,7 @@
 
 // Used for global events which don't correspond to a particular entity.
 SOURCE_TYPE(NONE, 0)
+
 SOURCE_TYPE(URL_REQUEST, 1)
 SOURCE_TYPE(SOCKET_STREAM, 2)
 SOURCE_TYPE(INIT_PROXY_RESOLVER, 3)
@@ -19,5 +20,7 @@ SOURCE_TYPE(DISK_CACHE_ENTRY, 9)
 SOURCE_TYPE(MEMORY_CACHE_ENTRY, 10)
 SOURCE_TYPE(HTTP_STREAM_JOB, 11)
 SOURCE_TYPE(EXPONENTIAL_BACKOFF_THROTTLING, 12)
+SOURCE_TYPE(DNS_TRANSACTION, 13)
+SOURCE_TYPE(ASYNC_HOST_RESOLVER_REQUEST, 14)
 
-SOURCE_TYPE(COUNT, 13)  // Always keep this as the last entry.
+SOURCE_TYPE(COUNT, 15)  // Always keep this as the last entry.
