@@ -3,6 +3,9 @@
 // found in the LICENSE file.
 
 #include "net/base/transport_security_state.h"
+
+#include "base/base64.h"
+#include "base/sha1.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace net {
@@ -431,12 +434,6 @@ TEST_F(TransportSecurityStateTest, Preloaded) {
   EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
                                       "chrome.google.com",
                                       true));
-  EXPECT_FALSE(state->IsEnabledForHost(&domain_state,
-                                       "foo.latest.chrome.google.com",
-                                       true));
-  EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
-                                      "latest.chrome.google.com",
-                                      true));
   EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
                                       "checkout.google.com",
                                       true));
@@ -451,6 +448,7 @@ TEST_F(TransportSecurityStateTest, Preloaded) {
 
   EXPECT_TRUE(state->IsEnabledForHost(&domain_state, "docs.google.com", true));
   EXPECT_TRUE(state->IsEnabledForHost(&domain_state, "sites.google.com", true));
+  EXPECT_TRUE(state->IsEnabledForHost(&domain_state, "drive.google.com", true));
   EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
                                       "spreadsheets.google.com",
                                       true));
@@ -497,6 +495,11 @@ TEST_F(TransportSecurityStateTest, Preloaded) {
   EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
                                       "talkgadget.google.com",
                                       true));
+  EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
+                                      "hostedtalkgadget.google.com",
+                                      true));
+  EXPECT_TRUE(state->IsEnabledForHost(&domain_state, "talk.google.com", true));
+  EXPECT_TRUE(state->IsEnabledForHost(&domain_state, "plus.google.com", true));
 
   EXPECT_TRUE(state->IsEnabledForHost(&domain_state, "entropia.de", true));
   EXPECT_TRUE(state->IsEnabledForHost(&domain_state, "www.entropia.de", true));
@@ -615,6 +618,49 @@ TEST_F(TransportSecurityStateTest, Preloaded) {
   EXPECT_FALSE(state->IsEnabledForHost(&domain_state,
                                        "foo.squareup.com",
                                        false));
+
+  EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
+                                      "foo.cert.se",
+                                      false));
+
+  EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
+                                      "crypto.is",
+                                      false));
+  EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
+                                      "foo.crypto.is",
+                                      false));
+
+  EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
+                                      "simon.butcher.name",
+                                      false));
+  EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
+                                      "foo.simon.butcher.name",
+                                      false));
+
+  EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
+                                      "linx.net",
+                                      false));
+  EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
+                                      "foo.linx.net",
+                                      false));
+
+  EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
+                                      "dropcam.com",
+                                      false));
+
+  EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
+                                      "ebanking.indovinabank.com.vn",
+                                      false));
+  EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
+                                      "foo.ebanking.indovinabank.com.vn",
+                                      false));
+
+  EXPECT_TRUE(state->IsEnabledForHost(&domain_state,
+                                      "epoxate.com",
+                                      false));
+  EXPECT_FALSE(state->IsEnabledForHost(&domain_state,
+                                       "foo.epoxate.com",
+                                       false));
 }
 
 TEST_F(TransportSecurityStateTest, LongNames) {
@@ -679,6 +725,7 @@ TEST_F(TransportSecurityStateTest, BuiltinCertPins) {
   EXPECT_TRUE(state->HasPinsForHost(&domain_state, "docs.google.com", true));
   EXPECT_TRUE(state->HasPinsForHost(&domain_state, "1.docs.google.com", true));
   EXPECT_TRUE(state->HasPinsForHost(&domain_state, "sites.google.com", true));
+  EXPECT_TRUE(state->HasPinsForHost(&domain_state, "drive.google.com", true));
   EXPECT_TRUE(state->HasPinsForHost(&domain_state,
                                     "spreadsheets.google.com",
                                     true));
@@ -706,6 +753,11 @@ TEST_F(TransportSecurityStateTest, BuiltinCertPins) {
   EXPECT_TRUE(state->HasPinsForHost(&domain_state,
                                     "talkgadget.google.com",
                                     true));
+  EXPECT_TRUE(state->HasPinsForHost(&domain_state,
+                                    "hostedtalkgadget.google.com",
+                                    true));
+  EXPECT_TRUE(state->HasPinsForHost(&domain_state, "talk.google.com", true));
+  EXPECT_TRUE(state->HasPinsForHost(&domain_state, "plus.google.com", true));
   EXPECT_TRUE(state->HasPinsForHost(&domain_state, "ssl.gstatic.com", true));
   EXPECT_FALSE(state->HasPinsForHost(&domain_state, "www.gstatic.com", true));
   EXPECT_TRUE(state->HasPinsForHost(&domain_state,
@@ -713,6 +765,80 @@ TEST_F(TransportSecurityStateTest, BuiltinCertPins) {
                                     true));
 }
 
+static bool AddHash(const std::string& type_and_base64,
+                    std::vector<SHA1Fingerprint>* out) {
+  std::string hash_str;
+  if (type_and_base64.find("sha1/") == 0 &&
+      base::Base64Decode(type_and_base64.substr(5, type_and_base64.size() - 5),
+                         &hash_str) &&
+      hash_str.size() == base::SHA1_LENGTH) {
+    SHA1Fingerprint hash;
+    memcpy(hash.data, hash_str.data(), sizeof(hash.data));
+    out->push_back(hash);
+    return true;
+  }
+  return false;
+}
+
+TEST_F(TransportSecurityStateTest, PinValidationWithRejectedCerts) {
+  // kGoodPath is plus.google.com via Google Internet Authority.
+  static const char* kGoodPath[] = {
+    "sha1/4BjDjn8v2lWeUFQnqSs0BgbIcrU=",
+    "sha1/QMVAHW+MuvCLAO3vse6H0AWzuc0=",
+    "sha1/SOZo+SvSspXXR9gjIBBPM5iQn9Q=",
+    NULL,
+  };
+
+  // kBadPath is plus.google.com via Trustcenter, which contains a required
+  // certificate (Equifax root), but also an excluded certificate
+  // (Trustcenter).
+  static const char* kBadPath[] = {
+    "sha1/4BjDjn8v2lWeUFQnqSs0BgbIcrU=",
+    "sha1/gzuEEAB/bkqdQS3EIjk2by7lW+k=",
+    "sha1/SOZo+SvSspXXR9gjIBBPM5iQn9Q=",
+    NULL,
+  };
+
+  std::vector<net::SHA1Fingerprint> good_hashes, bad_hashes;
+
+  for (size_t i = 0; kGoodPath[i]; i++) {
+    EXPECT_TRUE(AddHash(kGoodPath[i], &good_hashes));
+  }
+  for (size_t i = 0; kBadPath[i]; i++) {
+    EXPECT_TRUE(AddHash(kBadPath[i], &bad_hashes));
+  }
+
+  TransportSecurityState state("");
+  TransportSecurityState::DomainState domain_state;
+  EXPECT_TRUE(state.HasPinsForHost(&domain_state, "plus.google.com", true));
+
+  EXPECT_TRUE(domain_state.IsChainOfPublicKeysPermitted(good_hashes));
+  EXPECT_FALSE(domain_state.IsChainOfPublicKeysPermitted(bad_hashes));
+}
+
+TEST_F(TransportSecurityStateTest, PinValidationWithoutRejectedCerts) {
+  // kBadPath is plus.google.com via Trustcenter, which is utterly wrong for
+  // torproject.org.
+  static const char* kBadPath[] = {
+    "sha1/4BjDjn8v2lWeUFQnqSs0BgbIcrU=",
+    "sha1/gzuEEAB/bkqdQS3EIjk2by7lW+k=",
+    "sha1/SOZo+SvSspXXR9gjIBBPM5iQn9Q=",
+    NULL,
+  };
+
+  std::vector<net::SHA1Fingerprint> bad_hashes;
+
+  for (size_t i = 0; kBadPath[i]; i++) {
+    EXPECT_TRUE(AddHash(kBadPath[i], &bad_hashes));
+  }
+
+  TransportSecurityState state("");
+  TransportSecurityState::DomainState domain_state;
+  EXPECT_TRUE(state.HasPinsForHost(&domain_state, "plus.google.com", true));
+
+  EXPECT_FALSE(domain_state.IsChainOfPublicKeysPermitted(bad_hashes));
+}
+
 TEST_F(TransportSecurityStateTest, OptionalHSTSCertPins) {
   scoped_refptr<TransportSecurityState> state(
       new TransportSecurityState(std::string()));
