@@ -6,6 +6,7 @@
 
 #include <stdlib.h>
 
+#include <algorithm>
 #include <map>
 #include <string>
 #include <vector>
@@ -18,23 +19,19 @@
 #include "base/sha1.h"
 #include "base/string_piece.h"
 #include "base/string_util.h"
+#include "base/synchronization/lock.h"
 #include "base/time.h"
+#include "googleurl/src/url_canon_ip.h"
+#include "net/base/cert_status_flags.h"
+#include "net/base/cert_verify_result.h"
+#include "net/base/net_errors.h"
+#include "net/base/net_util.h"
 #include "net/base/pem_tokenizer.h"
 
 namespace net {
 
 namespace {
 
-// Returns true if this cert fingerprint is the null (all zero) fingerprint.
-// We use this as a bogus fingerprint value.
-bool IsNullFingerprint(const SHA1Fingerprint& fingerprint) {
-  for (size_t i = 0; i < arraysize(fingerprint.data); ++i) {
-    if (fingerprint.data[i] != 0)
-      return false;
-  }
-  return true;
-}
-
 // Indicates the order to use when trying to decode binary data, which is
 // based on (speculation) as to what will be most common -> least common
 const X509Certificate::Format kFormatDecodePriority[] = {
@@ -47,29 +44,62 @@ const char kCertificateHeader[] = "CERTIFICATE";
 // The PEM block header used for PKCS#7 data
 const char kPKCS7Header[] = "PKCS7";
 
-// A thread-safe cache for X509Certificate objects.
+// A thread-safe cache for OS certificate handles.
 //
-// The cache does not hold a reference to the certificate objects.  The objects
-// must |Remove| themselves from the cache upon destruction (or else the cache
-// will be holding dead pointers to the objects).
-// TODO(rsleevi): There exists a chance of a use-after-free, due to a race
-// between Find() and Remove(). See http://crbug.com/49377
+// Within each of the supported underlying crypto libraries, a certificate
+// handle is represented as a ref-counted object that contains the parsed
+// data for the certificate. In addition, the underlying OS handle may also
+// contain a copy of the original ASN.1 DER used to constructed the handle.
+//
+// In order to reduce the memory usage when multiple SSL connections exist,
+// with each connection storing the server's identity certificate plus any
+// intermediates supplied, the certificate handles are cached. Any two
+// X509Certificates that were created from the same ASN.1 DER data,
+// regardless of where that data came from, will share the same underlying
+// OS certificate handle.
 class X509CertificateCache {
  public:
-  void Insert(X509Certificate* cert);
-  void Remove(X509Certificate* cert);
-  X509Certificate* Find(const SHA1Fingerprint& fingerprint);
+  // Performs a compare-and-swap like operation. If an OS certificate handle
+  // for the same certificate data as |*cert_handle| already exists in the
+  // cache, the original |*cert_handle| will be freed and |cert_handle|
+  // will be updated to point to a duplicated reference to the existing cached
+  // certificate, with the caller taking ownership of this duplicated handle.
+  // If an equivalent OS certificate handle is not found, a duplicated
+  // reference to |*cert_handle| will be added to the cache. In either case,
+  // upon return, the caller fully owns |*cert_handle| and is responsible for
+  // calling FreeOSCertHandle(), after first calling Remove().
+  void InsertOrUpdate(X509Certificate::OSCertHandle* cert_handle);
+
+  // Decrements the cache reference count for |cert_handle|, a handle that was
+  // previously obtained by calling InsertOrUpdate(). If this is the last
+  // cached reference held, this will remove the handle from the cache. The
+  // caller retains ownership of |cert_handle| and remains responsible for
+  // calling FreeOSCertHandle() to release the underlying OS certificate
+  void Remove(X509Certificate::OSCertHandle cert_handle);
 
  private:
-  typedef std::map<SHA1Fingerprint, X509Certificate*, SHA1FingerprintLessThan>
-      CertMap;
+  // A single entry in the cache. Certificates will be keyed by their SHA1
+  // fingerprints, but will not be considered equivalent unless the entire
+  // certificate data matches.
+  struct Entry {
+    Entry() : cert_handle(NULL), ref_count(0) {}
+
+    X509Certificate::OSCertHandle cert_handle;
+
+    // Increased by each call to InsertOrUpdate(), and balanced by each call
+    // to Remove(). When it equals 0, all references created by
+    // InsertOrUpdate() have been released, so the cache entry will be removed
+    // the cached OS certificate handle will be freed.
+    int ref_count;
+  };
+  typedef std::map<SHA1Fingerprint, Entry, SHA1FingerprintLessThan> CertMap;
 
   // Obtain an instance of X509CertificateCache via a LazyInstance.
   X509CertificateCache() {}
   ~X509CertificateCache() {}
   friend struct base::DefaultLazyInstanceTraits<X509CertificateCache>;
 
-  // You must acquire this lock before using any private data of this object.
+  // You must acquire this lock before using any private data of this object
   // You must not block while holding this lock.
   base::Lock lock_;
 
@@ -83,46 +113,116 @@ base::LazyInstance<X509CertificateCache,
                    base::LeakyLazyInstanceTraits<X509CertificateCache> >
     g_x509_certificate_cache(base::LINKER_INITIALIZED);
 
-// Insert |cert| into the cache.  The cache does NOT AddRef |cert|.
-// Any existing certificate with the same fingerprint will be replaced.
-void X509CertificateCache::Insert(X509Certificate* cert) {
+void X509CertificateCache::InsertOrUpdate(
+    X509Certificate::OSCertHandle* cert_handle) {
+  DCHECK(cert_handle);
+  SHA1Fingerprint fingerprint =
+      X509Certificate::CalculateFingerprint(*cert_handle);
+
+  X509Certificate::OSCertHandle old_handle = NULL;
+  {
+    base::AutoLock lock(lock_);
+    CertMap::iterator pos = cache_.find(fingerprint);
+    if (pos == cache_.end()) {
+      // A cached entry was not found, so initialize a new entry. The entry
+      // assumes ownership of the current |*cert_handle|.
+      Entry cache_entry;
+      cache_entry.cert_handle = *cert_handle;
+      cache_entry.ref_count = 0;
+      CertMap::value_type cache_value(fingerprint, cache_entry);
+      pos = cache_.insert(cache_value).first;
+    } else {
+      bool is_same_cert =
+          X509Certificate::IsSameOSCert(*cert_handle, pos->second.cert_handle);
+      if (!is_same_cert) {
+        // Two certificates don't match, due to a SHA1 hash collision. Given
+        // the low probability, the simplest solution is to not cache the
+        // certificate, which should not affect performance too negatively.
+        return;
+      }
+      // A cached entry was found and will be used instead of the caller's
+      // handle. Ensure the caller's original handle will be freed, since
+      // ownership is assumed.
+      old_handle = *cert_handle;
+    }
+    // Whether an existing cached handle or a new handle, increment the
+    // cache's reference count and return a handle that the caller can own.
+    ++pos->second.ref_count;
+    *cert_handle = X509Certificate::DupOSCertHandle(pos->second.cert_handle);
+  }
+  // If the caller's handle was replaced with a cached handle, free the
+  // original handle now. This is done outside of the lock because
+  // |old_handle| may be the only handle for this particular certificate, so
+  // freeing it may be complex or resource-intensive and does not need to
+  // be guarded by the lock.
+  if (old_handle) {
+    X509Certificate::FreeOSCertHandle(old_handle);
+    DHISTOGRAM_COUNTS("X509CertificateReuseCount", 1);
+  }
+}
+
+void X509CertificateCache::Remove(X509Certificate::OSCertHandle cert_handle) {
+  SHA1Fingerprint fingerprint =
+      X509Certificate::CalculateFingerprint(cert_handle);
   base::AutoLock lock(lock_);
 
-  DCHECK(!IsNullFingerprint(cert->fingerprint())) <<
-      "Only insert certs with real fingerprints.";
-  cache_[cert->fingerprint()] = cert;
-};
-
-// Remove |cert| from the cache.  The cache does not assume that |cert| is
-// already in the cache.
-void X509CertificateCache::Remove(X509Certificate* cert) {
-  base::AutoLock lock(lock_);
-
-  CertMap::iterator pos(cache_.find(cert->fingerprint()));
+  CertMap::iterator pos = cache_.find(fingerprint);
   if (pos == cache_.end())
-    return;  // It is not an error to remove a cert that is not in the cache.
-  cache_.erase(pos);
-};
+    return;  // A hash collision where the winning cert was already freed.
 
-// Find a certificate in the cache with the given fingerprint.  If one does
-// not exist, this method returns NULL.
-X509Certificate* X509CertificateCache::Find(
-    const SHA1Fingerprint& fingerprint) {
-  base::AutoLock lock(lock_);
+  bool is_same_cert = X509Certificate::IsSameOSCert(cert_handle,
+                                                    pos->second.cert_handle);
+  if (!is_same_cert)
+    return;  // A hash collision where the winning cert is still around.
 
-  CertMap::iterator pos(cache_.find(fingerprint));
-  if (pos == cache_.end())
-    return NULL;
-
-  return pos->second;
-};
+  if (--pos->second.ref_count == 0) {
+    // The last reference to |cert_handle| has been removed, so release the
+    // Entry's OS handle and remove the Entry. The caller still holds a
+    // reference to |cert_handle| and is responsible for freeing it.
+    X509Certificate::FreeOSCertHandle(pos->second.cert_handle);
+    cache_.erase(pos);
+  }
+}
 
 // CompareSHA1Hashes is a helper function for using bsearch() with an array of
 // SHA1 hashes.
-static int CompareSHA1Hashes(const void* a, const void* b) {
+int CompareSHA1Hashes(const void* a, const void* b) {
   return memcmp(a, b, base::SHA1_LENGTH);
 }
 
+// Utility to split |src| on the first occurrence of |c|, if any. |right| will
+// either be empty if |c| was not found, or will contain the remainder of the
+// string including the split character itself.
+void SplitOnChar(const base::StringPiece& src,
+                 char c,
+                 base::StringPiece* left,
+                 base::StringPiece* right) {
+  size_t pos = src.find(c);
+  if (pos == base::StringPiece::npos) {
+    *left = src;
+    right->clear();
+  } else {
+    *left = src.substr(0, pos);
+    *right = src.substr(pos);
+  }
+}
+
+#if defined(OS_WIN)
+X509Certificate::OSCertHandle CreateOSCert(base::StringPiece der_cert) {
+  X509Certificate::OSCertHandle cert_handle = NULL;
+  BOOL ok = CertAddEncodedCertificateToStore(
+      X509Certificate::cert_store(), X509_ASN_ENCODING | PKCS_7_ASN_ENCODING,
+      reinterpret_cast<const BYTE*>(der_cert.data()), der_cert.size(),
+      CERT_STORE_ADD_USE_EXISTING, &cert_handle);
+  return ok ? cert_handle : NULL;
+}
+#else
+X509Certificate::OSCertHandle CreateOSCert(base::StringPiece der_cert) {
+  return X509Certificate::CreateOSCertHandleFromBytes(
+      const_cast<char*>(der_cert.data()), der_cert.size());
+}
+#endif
+
 }  // namespace
 
 bool X509Certificate::LessThan::operator()(X509Certificate* lhs,
@@ -131,7 +231,7 @@ bool X509Certificate::LessThan::operator()(X509Certificate* lhs,
     return false;
 
   SHA1FingerprintLessThan fingerprint_functor;
-  return fingerprint_functor(lhs->fingerprint_, rhs->fingerprint_);
+  return fingerprint_functor(lhs->chain_fingerprint_, rhs->chain_fingerprint_);
 }
 
 X509Certificate::X509Certificate(const std::string& subject,
@@ -142,58 +242,19 @@ X509Certificate::X509Certificate(const std::string& subject,
       issuer_(issuer),
       valid_start_(start_date),
       valid_expiry_(expiration_date),
-      cert_handle_(NULL),
-      source_(SOURCE_UNUSED) {
+      cert_handle_(NULL) {
   memset(fingerprint_.data, 0, sizeof(fingerprint_.data));
+  memset(chain_fingerprint_.data, 0, sizeof(chain_fingerprint_.data));
 }
 
 // static
 X509Certificate* X509Certificate::CreateFromHandle(
     OSCertHandle cert_handle,
-    Source source,
     const OSCertHandles& intermediates) {
   DCHECK(cert_handle);
-  DCHECK_NE(source, SOURCE_UNUSED);
-
-  // Check if we already have this certificate in memory.
-  X509CertificateCache* cache = g_x509_certificate_cache.Pointer();
-  X509Certificate* cached_cert =
-      cache->Find(CalculateFingerprint(cert_handle));
-  if (cached_cert) {
-    DCHECK_NE(cached_cert->source_, SOURCE_UNUSED);
-    if (cached_cert->source_ > source ||
-        (cached_cert->source_ == source &&
-         cached_cert->HasIntermediateCertificates(intermediates))) {
-      // Return the certificate with the same fingerprint from our cache.
-      DHISTOGRAM_COUNTS("X509CertificateReuseCount", 1);
-      return cached_cert;
-    }
-    // Else the new cert is better and will replace the old one in the cache.
-  }
-
-  // Otherwise, allocate and cache a new object.
-  X509Certificate* cert = new X509Certificate(cert_handle, source,
-                                              intermediates);
-  cache->Insert(cert);
-  return cert;
+  return new X509Certificate(cert_handle, intermediates);
 }
 
-#if defined(OS_WIN)
-static X509Certificate::OSCertHandle CreateOSCert(base::StringPiece der_cert) {
-  X509Certificate::OSCertHandle cert_handle = NULL;
-  BOOL ok = CertAddEncodedCertificateToStore(
-      X509Certificate::cert_store(), X509_ASN_ENCODING | PKCS_7_ASN_ENCODING,
-      reinterpret_cast<const BYTE*>(der_cert.data()), der_cert.size(),
-      CERT_STORE_ADD_USE_EXISTING, &cert_handle);
-  return ok ? cert_handle : NULL;
-}
-#else
-static X509Certificate::OSCertHandle CreateOSCert(base::StringPiece der_cert) {
-  return X509Certificate::CreateOSCertHandleFromBytes(
-      const_cast<char*>(der_cert.data()), der_cert.size());
-}
-#endif
-
 // static
 X509Certificate* X509Certificate::CreateFromDERCertChain(
     const std::vector<base::StringPiece>& der_certs) {
@@ -203,15 +264,22 @@ X509Certificate* X509Certificate::CreateFromDERCertChain(
   X509Certificate::OSCertHandles intermediate_ca_certs;
   for (size_t i = 1; i < der_certs.size(); i++) {
     OSCertHandle handle = CreateOSCert(der_certs[i]);
-    DCHECK(handle);
+    if (!handle)
+      break;
     intermediate_ca_certs.push_back(handle);
   }
 
-  OSCertHandle handle = CreateOSCert(der_certs[0]);
-  DCHECK(handle);
-  X509Certificate* cert =
-      CreateFromHandle(handle, SOURCE_FROM_NETWORK, intermediate_ca_certs);
-  FreeOSCertHandle(handle);
+  OSCertHandle handle = NULL;
+  // Return NULL if we failed to parse any of the certs.
+  if (der_certs.size() - 1 == intermediate_ca_certs.size())
+    handle = CreateOSCert(der_certs[0]);
+
+  X509Certificate* cert = NULL;
+  if (handle) {
+    cert = CreateFromHandle(handle, intermediate_ca_certs);
+    FreeOSCertHandle(handle);
+  }
+
   for (size_t i = 0; i < intermediate_ca_certs.size(); i++)
     FreeOSCertHandle(intermediate_ca_certs[i]);
 
@@ -225,9 +293,7 @@ X509Certificate* X509Certificate::CreateFromBytes(const char* data,
   if (!cert_handle)
     return NULL;
 
-  X509Certificate* cert = CreateFromHandle(cert_handle,
-                                           SOURCE_LONE_CERT_IMPORT,
-                                           OSCertHandles());
+  X509Certificate* cert = CreateFromHandle(cert_handle, OSCertHandles());
   FreeOSCertHandle(cert_handle);
   return cert;
 }
@@ -259,7 +325,7 @@ X509Certificate* X509Certificate::CreateFromPickle(const Pickle& pickle,
 
   X509Certificate* cert = NULL;
   if (intermediates.size() == num_intermediates)
-    cert = CreateFromHandle(cert_handle, SOURCE_FROM_CACHE, intermediates);
+    cert = CreateFromHandle(cert_handle, intermediates);
   FreeOSCertHandle(cert_handle);
   for (size_t i = 0; i < intermediates.size(); ++i)
     FreeOSCertHandle(intermediates[i]);
@@ -336,8 +402,7 @@ CertificateList X509Certificate::CreateCertificateListFromBytes(
 
   for (OSCertHandles::iterator it = certificates.begin();
        it != certificates.end(); ++it) {
-    X509Certificate* result = CreateFromHandle(*it, SOURCE_LONE_CERT_IMPORT,
-                                               OSCertHandles());
+    X509Certificate* result = CreateFromHandle(*it, OSCertHandles());
     results.push_back(scoped_refptr<X509Certificate>(result));
     FreeOSCertHandle(*it);
   }
@@ -365,6 +430,12 @@ void X509Certificate::Persist(Pickle* pickle) {
   }
 }
 
+void X509Certificate::GetDNSNames(std::vector<std::string>* dns_names) const {
+  GetSubjectAltName(dns_names, NULL);
+  if (dns_names->empty())
+    dns_names->push_back(subject_.common_name);
+}
+
 bool X509Certificate::HasExpired() const {
   return base::Time::Now() > valid_expiry();
 }
@@ -392,148 +463,182 @@ bool X509Certificate::HasIntermediateCertificates(const OSCertHandles& certs) {
 // static
 bool X509Certificate::VerifyHostname(
     const std::string& hostname,
-    const std::vector<std::string>& cert_names) {
+    const std::string& cert_common_name,
+    const std::vector<std::string>& cert_san_dns_names,
+    const std::vector<std::string>& cert_san_ip_addrs) {
   DCHECK(!hostname.empty());
+  // Perform name verification following http://tools.ietf.org/html/rfc6125.
+  // The terminology used in this method is as per that RFC:-
+  // Reference identifier == the host the local user/agent is intending to
+  //                         access, i.e. the thing displayed in the URL bar.
+  // Presented identifier(s) == name(s) the server knows itself as, in its cert.
 
-  // Simple host name validation. A valid domain name must only contain
-  // alpha, digits, hyphens, and dots. An IP address may have digits and dots,
-  // and also square braces and colons for IPv6 addresses.
-  std::string reference_name;
-  reference_name.reserve(hostname.length());
-
-  bool found_alpha = false;
-  bool found_ip6_chars = false;
-  bool found_hyphen = false;
-  int dot_count = 0;
-
-  size_t first_dot_index = std::string::npos;
-  for (std::string::const_iterator it = hostname.begin();
-       it != hostname.end(); ++it) {
-    char c = *it;
-    if (IsAsciiAlpha(c)) {
-      found_alpha = true;
-      c = base::ToLowerASCII(c);
-    } else if (c == '.') {
-      ++dot_count;
-      if (first_dot_index == std::string::npos)
-        first_dot_index = reference_name.length();
-    } else if (c == ':') {
-      found_ip6_chars = true;
-    } else if (c == '-') {
-      found_hyphen = true;
-    } else if (!IsAsciiDigit(c)) {
-      LOG(WARNING) << "Invalid char " << c << " in hostname " << hostname;
-      return false;
-    }
-    reference_name.push_back(c);
-  }
-  DCHECK(!reference_name.empty());
-
-  if (found_ip6_chars || !found_alpha) {
-    // For now we just do simple localhost IP address support, primarily as
-    // it's needed by the test server. TODO(joth): Replace this with full IP
-    // address support. See http://crbug.com/62973
-    if (hostname == "127.0.0.1") {
-      for (size_t index = 0; index < cert_names.size(); ++index) {
-        if (cert_names[index] == hostname) {
-          DVLOG(1) << "Allowing localhost IP certificate: " << hostname;
-          return true;
-        }
-      }
-    }
-    NOTIMPLEMENTED() << hostname;  // See comment above.
+  // CanonicalizeHost requires surrounding brackets to parse an IPv6 address.
+  const std::string host_or_ip = hostname.find(':') != std::string::npos ?
+      "[" + hostname + "]" : hostname;
+  url_canon::CanonHostInfo host_info;
+  const std::string reference_name = CanonicalizeHost(host_or_ip, &host_info);
+  if (reference_name.empty())
     return false;
+
+  // Allow fallback to Common name matching?
+  const bool common_name_fallback = cert_san_dns_names.empty() &&
+                                    cert_san_ip_addrs.empty();
+
+  // Fully handle all cases where |hostname| contains an IP address.
+  if (host_info.IsIPAddress()) {
+    if (common_name_fallback &&
+        host_info.family == url_canon::CanonHostInfo::IPV4) {
+      // Fallback to Common name matching. As this is deprecated and only
+      // supported for compatibility refuse it for IPv6 addresses.
+      return reference_name == cert_common_name;
+    }
+    base::StringPiece ip_addr_string(
+        reinterpret_cast<const char*>(host_info.address),
+        host_info.AddressLength());
+    return std::find(cert_san_ip_addrs.begin(), cert_san_ip_addrs.end(),
+                     ip_addr_string) != cert_san_ip_addrs.end();
   }
 
-  // |wildcard_domain| is the remainder of |host| after the leading host
+  // |reference_domain| is the remainder of |host| after the leading host
   // component is stripped off, but includes the leading dot e.g.
   // "www.f.com" -> ".f.com".
-  // If there is no meaningful domain part to |host| (e.g. it is an IP address
-  // or contains no dots) then |wildcard_domain| will be empty.
-  // We required at least 3 components (i.e. 2 dots) as a basic protection
-  // against too-broad wild-carding.
-  base::StringPiece wildcard_domain;
-  if (found_alpha && !found_ip6_chars && dot_count >= 2) {
-    DCHECK(first_dot_index != std::string::npos);
-    wildcard_domain = reference_name;
-    wildcard_domain.remove_prefix(first_dot_index);
-    DCHECK(wildcard_domain.starts_with("."));
+  // If there is no meaningful domain part to |host| (e.g. it contains no dots)
+  // then |reference_domain| will be empty.
+  base::StringPiece reference_host, reference_domain;
+  SplitOnChar(reference_name, '.', &reference_host, &reference_domain);
+  bool allow_wildcards = false;
+  if (!reference_domain.empty()) {
+    DCHECK(reference_domain.starts_with("."));
+    // We required at least 3 components (i.e. 2 dots) as a basic protection
+    // against too-broad wild-carding.
+    // Also we don't attempt wildcard matching on a purely numerical hostname.
+    allow_wildcards = reference_domain.rfind('.') != 0 &&
+        reference_name.find_first_not_of("0123456789.") != std::string::npos;
   }
 
-  for (std::vector<std::string>::const_iterator it = cert_names.begin();
-       it != cert_names.end(); ++it) {
+  // Now step through the DNS names doing wild card comparison (if necessary)
+  // on each against the reference name. If subjectAltName is empty, then
+  // fallback to use the common name instead.
+  std::vector<std::string> common_name_as_vector;
+  const std::vector<std::string>* presented_names = &cert_san_dns_names;
+  if (common_name_fallback) {
+    // Note: there's a small possibility cert_common_name is an international
+    // domain name in non-standard encoding (e.g. UTF8String or BMPString
+    // instead of A-label). As common name fallback is deprecated we're not
+    // doing anything specific to deal with this.
+    common_name_as_vector.push_back(cert_common_name);
+    presented_names = &common_name_as_vector;
+  }
+  for (std::vector<std::string>::const_iterator it =
+           presented_names->begin();
+       it != presented_names->end(); ++it) {
     // Catch badly corrupt cert names up front.
     if (it->empty() || it->find('\0') != std::string::npos) {
-      LOG(WARNING) << "Bad name in cert: " << *it;
+      DVLOG(1) << "Bad name in cert: " << *it;
       continue;
     }
-    const std::string cert_name_string(StringToLowerASCII(*it));
-    base::StringPiece cert_match(cert_name_string);
+    std::string presented_name(StringToLowerASCII(*it));
 
     // Remove trailing dot, if any.
-    if (cert_match.ends_with("."))
-      cert_match.remove_suffix(1);
+    if (*presented_name.rbegin() == '.')
+      presented_name.resize(presented_name.length() - 1);
 
     // The hostname must be at least as long as the cert name it is matching,
     // as we require the wildcard (if present) to match at least one character.
-    if (cert_match.length() > reference_name.length())
+    if (presented_name.length() > reference_name.length())
       continue;
 
-    if (cert_match == reference_name)
-      return true;
+    base::StringPiece presented_host, presented_domain;
+    SplitOnChar(presented_name, '.', &presented_host, &presented_domain);
 
-    // Next see if this cert name starts with a wildcard, so long as the
-    // hostname we're matching against has a valid 'domain' part to match.
-    // Note the "-10" version of draft-saintandre-tls-server-id-check allows
-    // the wildcard to appear anywhere in the leftmost label, rather than
-    // requiring it to be the only character. See also http://crbug.com/60719
-    if (wildcard_domain.empty() || !cert_match.starts_with("*"))
+    if (presented_domain != reference_domain)
       continue;
 
-    // Erase the * but not the . from the domain, as we need to include the dot
-    // in the comparison.
-    cert_match.remove_prefix(1);
+    base::StringPiece pattern_begin, pattern_end;
+    SplitOnChar(presented_host, '*', &pattern_begin, &pattern_end);
 
-    // Do character by character comparison on the remainder to see
-    // if we have a wildcard match. This intentionally does no special handling
-    // for any other wildcard characters in |domain|; alternatively it could
-    // detect these and skip those candidate cert names.
-    if (cert_match == wildcard_domain)
+    if (pattern_end.empty()) {  // No '*' in the presented_host
+      if (presented_host == reference_host)
+        return true;
+      continue;
+    }
+    pattern_end.remove_prefix(1);  // move past the *
+
+    if (!allow_wildcards)
+      continue;
+
+    // * must not match a substring of an IDN A label; just a whole fragment.
+    if (reference_host.starts_with("xn--") &&
+        !(pattern_begin.empty() && pattern_end.empty()))
+      continue;
+
+    if (reference_host.starts_with(pattern_begin) &&
+        reference_host.ends_with(pattern_end))
       return true;
   }
-  DVLOG(1) << "Could not find any match for " << hostname
-           << " (canonicalized as " << reference_name
-           << ") in cert names " << JoinString(cert_names, '|');
   return false;
 }
 
+int X509Certificate::Verify(const std::string& hostname, int flags,
+                            CertVerifyResult* verify_result) const {
+  verify_result->Reset();
+  verify_result->verified_cert = const_cast<X509Certificate*>(this);
+
+  if (IsBlacklisted()) {
+    verify_result->cert_status |= CERT_STATUS_REVOKED;
+    return ERR_CERT_REVOKED;
+  }
+
+  int rv = VerifyInternal(hostname, flags, verify_result);
+
+  // This check is done after VerifyInternal so that VerifyInternal can fill in
+  // the list of public key hashes.
+  if (IsPublicKeyBlacklisted(verify_result->public_key_hashes)) {
+    verify_result->cert_status |= CERT_STATUS_REVOKED;
+    rv = MapCertStatusToNetError(verify_result->cert_status);
+  }
+
+  return rv;
+}
+
 #if !defined(USE_NSS)
 bool X509Certificate::VerifyNameMatch(const std::string& hostname) const {
-  std::vector<std::string> dns_names;
-  GetDNSNames(&dns_names);
-  return VerifyHostname(hostname, dns_names);
+  std::vector<std::string> dns_names, ip_addrs;
+  GetSubjectAltName(&dns_names, &ip_addrs);
+  return VerifyHostname(hostname, subject_.common_name, dns_names, ip_addrs);
 }
 #endif
 
 X509Certificate::X509Certificate(OSCertHandle cert_handle,
-                                 Source source,
                                  const OSCertHandles& intermediates)
-    : cert_handle_(DupOSCertHandle(cert_handle)),
-      source_(source) {
-  // Copy/retain the intermediate cert handles.
-  for (size_t i = 0; i < intermediates.size(); ++i)
-    intermediate_ca_certs_.push_back(DupOSCertHandle(intermediates[i]));
+    : cert_handle_(DupOSCertHandle(cert_handle)) {
+  X509CertificateCache* cache = g_x509_certificate_cache.Pointer();
+  cache->InsertOrUpdate(&cert_handle_);
+  for (size_t i = 0; i < intermediates.size(); ++i) {
+    // Duplicate the incoming certificate, as the caller retains ownership
+    // of |intermediates|.
+    OSCertHandle intermediate = DupOSCertHandle(intermediates[i]);
+    // Update the cache, which will assume ownership of the duplicated
+    // handle and return a suitable equivalent, potentially from the cache.
+    cache->InsertOrUpdate(&intermediate);
+    intermediate_ca_certs_.push_back(intermediate);
+  }
   // Platform-specific initialization.
   Initialize();
 }
 
 X509Certificate::~X509Certificate() {
   // We might not be in the cache, but it is safe to remove ourselves anyway.
-  g_x509_certificate_cache.Get().Remove(this);
-  if (cert_handle_)
+  X509CertificateCache* cache = g_x509_certificate_cache.Pointer();
+  if (cert_handle_) {
+    cache->Remove(cert_handle_);
     FreeOSCertHandle(cert_handle_);
-  for (size_t i = 0; i < intermediate_ca_certs_.size(); ++i)
+  }
+  for (size_t i = 0; i < intermediate_ca_certs_.size(); ++i) {
+    cache->Remove(intermediate_ca_certs_[i]);
     FreeOSCertHandle(intermediate_ca_certs_[i]);
+  }
 }
 
 bool X509Certificate::IsBlacklisted() const {
@@ -854,7 +959,7 @@ bool X509Certificate::IsBlacklisted() const {
 // static
 bool X509Certificate::IsPublicKeyBlacklisted(
     const std::vector<SHA1Fingerprint>& public_key_hashes) {
-  static const unsigned kNumHashes = 5;
+  static const unsigned kNumHashes = 7;
   static const uint8 kHashes[kNumHashes][base::SHA1_LENGTH] = {
     // Subject: CN=DigiNotar Root CA
     // Issuer: CN=Entrust.net x2 and self-signed
@@ -862,8 +967,8 @@ bool X509Certificate::IsPublicKeyBlacklisted(
      0x12, 0xce, 0x48, 0x63, 0xe4, 0x33, 0x43, 0x78, 0x06, 0xa8},
     // Subject: CN=DigiNotar Cyber CA
     // Issuer: CN=GTE CyberTrust Global Root
-    {0xba, 0x3e, 0x7b, 0xd3, 0x8c, 0xd7, 0xe1, 0xe6, 0xb9, 0xcd,
-     0x4c, 0x21, 0x99, 0x62, 0xe5, 0x9d, 0x7a, 0x2f, 0x4e, 0x37},
+    {0xc4, 0xf9, 0x66, 0x37, 0x16, 0xcd, 0x5e, 0x71, 0xd6, 0x95,
+     0x0b, 0x5f, 0x33, 0xce, 0x04, 0x1c, 0x95, 0xb4, 0x35, 0xd1},
     // Subject: CN=DigiNotar Services 1024 CA
     // Issuer: CN=Entrust.net
     {0xe2, 0x3b, 0x8d, 0x10, 0x5f, 0x87, 0x71, 0x0a, 0x68, 0xd9,
@@ -876,6 +981,16 @@ bool X509Certificate::IsPublicKeyBlacklisted(
     // Issuer: CN=Staat der Nederlanden Overheid CA
     {0xe8, 0xf9, 0x12, 0x00, 0xc6, 0x5c, 0xee, 0x16, 0xe0, 0x39,
      0xb9, 0xf8, 0x83, 0x84, 0x16, 0x61, 0x63, 0x5f, 0x81, 0xc5},
+    // Subject: O=Digicert Sdn. Bhd.
+    // Issuer: CN=GTE CyberTrust Global Root
+    // Expires: Jul 17 15:16:54 2012 GMT
+    {0x01, 0x29, 0xbc, 0xd5, 0xb4, 0x48, 0xae, 0x8d, 0x24, 0x96,
+     0xd1, 0xc3, 0xe1, 0x97, 0x23, 0x91, 0x90, 0x88, 0xe1, 0x52},
+    // Subject: O=Digicert Sdn. Bhd.
+    // Issuer: CN=Entrust.net Certification Authority (2048)
+    // Expires: Jul 16 17:53:37 2015 GMT
+    {0xd3, 0x3c, 0x5b, 0x41, 0xe4, 0x5c, 0xc4, 0xb3, 0xbe, 0x9a,
+     0xd6, 0x95, 0x2c, 0x4e, 0xcc, 0x25, 0x28, 0x03, 0x29, 0x81},
   };
 
   for (unsigned i = 0; i < kNumHashes; i++) {
@@ -889,7 +1004,6 @@ bool X509Certificate::IsPublicKeyBlacklisted(
   return false;
 }
 
-
 // static
 bool X509Certificate::IsSHA1HashInSortedArray(const SHA1Fingerprint& hash,
                                               const uint8* array,
