@@ -402,28 +402,6 @@ TEST(X509CertificateTest, PaypalNullCertParsing) {
 #endif
 }
 
-// A certificate whose AIA extension contains an LDAP URL without a host name.
-// This certificate will expire on 2011-09-08.
-TEST(X509CertificateTest, UnoSoftCertParsing) {
-  FilePath certs_dir = GetTestCertsDirectory();
-  scoped_refptr<X509Certificate> unosoft_hu_cert(
-      ImportCertFromFile(certs_dir, "unosoft_hu_cert.der"));
-
-  ASSERT_NE(static_cast<X509Certificate*>(NULL), unosoft_hu_cert);
-
-  const SHA1Fingerprint& fingerprint =
-      unosoft_hu_cert->fingerprint();
-  for (size_t i = 0; i < 20; ++i)
-    EXPECT_EQ(unosoft_hu_fingerprint[i], fingerprint.data[i]);
-
-  int flags = 0;
-  CertVerifyResult verify_result;
-  int error = unosoft_hu_cert->Verify("www.unosoft.hu", flags,
-                                      &verify_result);
-  EXPECT_EQ(ERR_CERT_AUTHORITY_INVALID, error);
-  EXPECT_NE(0, verify_result.cert_status & CERT_STATUS_AUTHORITY_INVALID);
-}
-
 TEST(X509CertificateTest, SerialNumbers) {
   scoped_refptr<X509Certificate> google_cert(
       X509Certificate::CreateFromBytes(
@@ -452,6 +430,47 @@ TEST(X509CertificateTest, SerialNumbers) {
                      paypal_null_serial, sizeof(paypal_null_serial)) == 0);
 }
 
+TEST(X509CertificateTest, ChainFingerprints) {
+  FilePath certs_dir = GetTestCertsDirectory();
+
+  scoped_refptr<X509Certificate> server_cert =
+      ImportCertFromFile(certs_dir, "salesforce_com_test.pem");
+  ASSERT_NE(static_cast<X509Certificate*>(NULL), server_cert);
+
+  scoped_refptr<X509Certificate> intermediate_cert1 =
+      ImportCertFromFile(certs_dir, "verisign_intermediate_ca_2011.pem");
+  ASSERT_NE(static_cast<X509Certificate*>(NULL), intermediate_cert1);
+
+  scoped_refptr<X509Certificate> intermediate_cert2 =
+      ImportCertFromFile(certs_dir, "verisign_intermediate_ca_2016.pem");
+  ASSERT_NE(static_cast<X509Certificate*>(NULL), intermediate_cert2);
+
+  X509Certificate::OSCertHandles intermediates;
+  intermediates.push_back(intermediate_cert1->os_cert_handle());
+  scoped_refptr<X509Certificate> cert_chain1 =
+      X509Certificate::CreateFromHandle(server_cert->os_cert_handle(),
+                                        intermediates);
+
+  intermediates.clear();
+  intermediates.push_back(intermediate_cert2->os_cert_handle());
+  scoped_refptr<X509Certificate> cert_chain2 =
+      X509Certificate::CreateFromHandle(server_cert->os_cert_handle(),
+                                        intermediates);
+
+  static const uint8 cert_chain1_fingerprint[20] = {
+    0x67, 0x78, 0x81, 0xd7, 0x78, 0xca, 0xd5, 0x04, 0x73, 0xf8,
+    0x95, 0xff, 0xf3, 0x39, 0xe4, 0xcd, 0x5e, 0xf0, 0x79, 0x76
+  };
+  static const uint8 cert_chain2_fingerprint[20] = {
+    0x8c, 0x93, 0x85, 0xb0, 0x15, 0xd3, 0xa3, 0x0e, 0xe7, 0x4f,
+    0x42, 0xf4, 0x30, 0xc3, 0xe9, 0x14, 0x12, 0x54, 0xb9, 0x9d
+  };
+  EXPECT_TRUE(memcmp(cert_chain1->chain_fingerprint().data,
+                     cert_chain1_fingerprint, 20) == 0);
+  EXPECT_TRUE(memcmp(cert_chain2->chain_fingerprint().data,
+                     cert_chain2_fingerprint, 20) == 0);
+}
+
 // A regression test for http://crbug.com/31497.
 // This certificate will expire on 2012-04-08.
 TEST(X509CertificateTest, IntermediateCARequireExplicitPolicy) {
@@ -475,7 +494,6 @@ TEST(X509CertificateTest, IntermediateCARequireExplicitPolicy) {
   intermediates.push_back(intermediate_cert->os_cert_handle());
   scoped_refptr<X509Certificate> cert_chain =
       X509Certificate::CreateFromHandle(server_cert->os_cert_handle(),
-                                        X509Certificate::SOURCE_FROM_NETWORK,
                                         intermediates);
 
   int flags = 0;
@@ -510,7 +528,6 @@ TEST(X509CertificateTest, DISABLED_GlobalSignR3EVTest) {
   intermediates.push_back(intermediate_cert->os_cert_handle());
   scoped_refptr<X509Certificate> cert_chain =
       X509Certificate::CreateFromHandle(server_cert->os_cert_handle(),
-                                        X509Certificate::SOURCE_FROM_NETWORK,
                                         intermediates);
 
   CertVerifyResult verify_result;
@@ -523,6 +540,70 @@ TEST(X509CertificateTest, DISABLED_GlobalSignR3EVTest) {
     EXPECT_EQ(ERR_CERT_DATE_INVALID, error);
 }
 
+// Test for bug 94673.
+TEST(X509CertificateTest, GoogleDigiNotarTest) {
+  FilePath certs_dir = GetTestCertsDirectory();
+
+  scoped_refptr<X509Certificate> server_cert =
+      ImportCertFromFile(certs_dir, "google_diginotar.pem");
+  ASSERT_NE(static_cast<X509Certificate*>(NULL), server_cert);
+
+  scoped_refptr<X509Certificate> intermediate_cert =
+      ImportCertFromFile(certs_dir, "diginotar_public_ca_2025.pem");
+  ASSERT_NE(static_cast<X509Certificate*>(NULL), intermediate_cert);
+
+  X509Certificate::OSCertHandles intermediates;
+  intermediates.push_back(intermediate_cert->os_cert_handle());
+  scoped_refptr<X509Certificate> cert_chain =
+      X509Certificate::CreateFromHandle(server_cert->os_cert_handle(),
+                                        intermediates);
+
+  CertVerifyResult verify_result;
+  int flags = X509Certificate::VERIFY_REV_CHECKING_ENABLED;
+  int error = cert_chain->Verify("mail.google.com", flags, &verify_result);
+  EXPECT_NE(OK, error);
+
+  // Now turn off revocation checking.  Certificate verification should still
+  // fail.
+  flags = 0;
+  error = cert_chain->Verify("mail.google.com", flags, &verify_result);
+  EXPECT_NE(OK, error);
+}
+
+TEST(X509CertificateTest, DigiNotarCerts) {
+  static const char* const kDigiNotarFilenames[] = {
+    "diginotar_root_ca.pem",
+    "diginotar_cyber_ca.pem",
+    "diginotar_services_1024_ca.pem",
+    "diginotar_pkioverheid.pem",
+    "diginotar_pkioverheid_g2.pem",
+    NULL,
+  };
+
+  FilePath certs_dir = GetTestCertsDirectory();
+
+  for (size_t i = 0; kDigiNotarFilenames[i]; i++) {
+    scoped_refptr<X509Certificate> diginotar_cert =
+        ImportCertFromFile(certs_dir, kDigiNotarFilenames[i]);
+    std::string der_bytes;
+    ASSERT_TRUE(diginotar_cert->GetDEREncoded(&der_bytes));
+
+    base::StringPiece spki;
+    ASSERT_TRUE(asn1::ExtractSPKIFromDERCert(der_bytes, &spki));
+
+    std::string spki_sha1 = base::SHA1HashString(spki.as_string());
+
+    std::vector<SHA1Fingerprint> public_keys;
+    SHA1Fingerprint fingerprint;
+    ASSERT_EQ(sizeof(fingerprint.data), spki_sha1.size());
+    memcpy(fingerprint.data, spki_sha1.data(), spki_sha1.size());
+    public_keys.push_back(fingerprint);
+
+    EXPECT_TRUE(X509Certificate::IsPublicKeyBlacklisted(public_keys)) <<
+        "Public key not blocked for " << kDigiNotarFilenames[i];
+  }
+}
+
 TEST(X509CertificateTest, TestKnownRoot) {
   FilePath certs_dir = GetTestCertsDirectory();
   scoped_refptr<X509Certificate> cert =
@@ -539,7 +620,6 @@ TEST(X509CertificateTest, TestKnownRoot) {
   intermediates.push_back(intermediate_cert->os_cert_handle());
   scoped_refptr<X509Certificate> cert_chain =
       X509Certificate::CreateFromHandle(cert->os_cert_handle(),
-                                        X509Certificate::SOURCE_FROM_NETWORK,
                                         intermediates);
 
   int flags = 0;
@@ -576,6 +656,25 @@ TEST(X509CertificateTest, ExtractSPKIFromDERCert) {
   EXPECT_TRUE(0 == memcmp(hash, nistSPKIHash, sizeof(hash)));
 }
 
+TEST(X509CertificateTest, ExtractCRLURLsFromDERCert) {
+  FilePath certs_dir = GetTestCertsDirectory();
+  scoped_refptr<X509Certificate> cert =
+      ImportCertFromFile(certs_dir, "nist.der");
+  ASSERT_NE(static_cast<X509Certificate*>(NULL), cert);
+
+  std::string derBytes;
+  EXPECT_TRUE(cert->GetDEREncoded(&derBytes));
+
+  std::vector<base::StringPiece> crl_urls;
+  EXPECT_TRUE(asn1::ExtractCRLURLsFromDERCert(derBytes, &crl_urls));
+
+  EXPECT_EQ(1u, crl_urls.size());
+  if (crl_urls.size() > 0) {
+    EXPECT_EQ("http://SVRSecure-G3-crl.verisign.com/SVRSecureG3.crl",
+              crl_urls[0].as_string());
+  }
+}
+
 TEST(X509CertificateTest, PublicKeyHashes) {
   FilePath certs_dir = GetTestCertsDirectory();
   // This is going to blow up in Feb 2012. Sorry! Disable and file a bug
@@ -596,7 +695,6 @@ TEST(X509CertificateTest, PublicKeyHashes) {
   intermediates.push_back(intermediate_cert->os_cert_handle());
   scoped_refptr<X509Certificate> cert_chain =
       X509Certificate::CreateFromHandle(cert->os_cert_handle(),
-                                        X509Certificate::SOURCE_FROM_NETWORK,
                                         intermediates);
 
   int flags = 0;
@@ -644,66 +742,54 @@ TEST(X509CertificateTest, InvalidKeyUsage) {
 #endif
 }
 
-// Tests X509Certificate::Cache via X509Certificate::CreateFromHandle.  We
+// Tests X509CertificateCache via X509Certificate::CreateFromHandle.  We
 // call X509Certificate::CreateFromHandle several times and observe whether
-// it returns a cached or new X509Certificate object.
-//
-// All the OS certificate handles in this test are actually from the same
-// source (the bytes of a lone certificate), but we pretend that some of them
-// come from the network.
+// it returns a cached or new OSCertHandle.
 TEST(X509CertificateTest, Cache) {
   X509Certificate::OSCertHandle google_cert_handle;
+  X509Certificate::OSCertHandle thawte_cert_handle;
 
-  // Add a certificate from the source SOURCE_LONE_CERT_IMPORT to our
-  // certificate cache.
+  // Add a single certificate to the certificate cache.
   google_cert_handle = X509Certificate::CreateOSCertHandleFromBytes(
       reinterpret_cast<const char*>(google_der), sizeof(google_der));
   scoped_refptr<X509Certificate> cert1(X509Certificate::CreateFromHandle(
-      google_cert_handle, X509Certificate::SOURCE_LONE_CERT_IMPORT,
-      X509Certificate::OSCertHandles()));
+      google_cert_handle, X509Certificate::OSCertHandles()));
   X509Certificate::FreeOSCertHandle(google_cert_handle);
 
-  // Add a certificate from the same source (SOURCE_LONE_CERT_IMPORT).  This
-  // should return the cached certificate (cert1).
+  // Add the same certificate, but as a new handle.
   google_cert_handle = X509Certificate::CreateOSCertHandleFromBytes(
       reinterpret_cast<const char*>(google_der), sizeof(google_der));
   scoped_refptr<X509Certificate> cert2(X509Certificate::CreateFromHandle(
-      google_cert_handle, X509Certificate::SOURCE_LONE_CERT_IMPORT,
-      X509Certificate::OSCertHandles()));
+      google_cert_handle, X509Certificate::OSCertHandles()));
   X509Certificate::FreeOSCertHandle(google_cert_handle);
 
-  EXPECT_EQ(cert1, cert2);
+  // A new X509Certificate should be returned.
+  EXPECT_NE(cert1.get(), cert2.get());
+  // But both instances should share the underlying OS certificate handle.
+  EXPECT_EQ(cert1->os_cert_handle(), cert2->os_cert_handle());
+  EXPECT_EQ(0u, cert1->GetIntermediateCertificates().size());
+  EXPECT_EQ(0u, cert2->GetIntermediateCertificates().size());
 
-  // Add a certificate from the network.  This should kick out the original
-  // cached certificate (cert1) and return a new certificate.
+  // Add the same certificate, but this time with an intermediate. This
+  // should result in the intermediate being cached. Note that this is not
+  // a legitimate chain, but is suitable for testing.
   google_cert_handle = X509Certificate::CreateOSCertHandleFromBytes(
       reinterpret_cast<const char*>(google_der), sizeof(google_der));
+  thawte_cert_handle = X509Certificate::CreateOSCertHandleFromBytes(
+      reinterpret_cast<const char*>(thawte_der), sizeof(thawte_der));
+  X509Certificate::OSCertHandles intermediates;
+  intermediates.push_back(thawte_cert_handle);
   scoped_refptr<X509Certificate> cert3(X509Certificate::CreateFromHandle(
-      google_cert_handle, X509Certificate::SOURCE_FROM_NETWORK,
-      X509Certificate::OSCertHandles()));
+      google_cert_handle, intermediates));
   X509Certificate::FreeOSCertHandle(google_cert_handle);
+  X509Certificate::FreeOSCertHandle(thawte_cert_handle);
 
-  EXPECT_NE(cert1, cert3);
-
-  // Add one certificate from each source.  Both should return the new cached
-  // certificate (cert3).
-  google_cert_handle = X509Certificate::CreateOSCertHandleFromBytes(
-      reinterpret_cast<const char*>(google_der), sizeof(google_der));
-  scoped_refptr<X509Certificate> cert4(X509Certificate::CreateFromHandle(
-      google_cert_handle, X509Certificate::SOURCE_FROM_NETWORK,
-      X509Certificate::OSCertHandles()));
-  X509Certificate::FreeOSCertHandle(google_cert_handle);
-
-  EXPECT_EQ(cert3, cert4);
-
-  google_cert_handle = X509Certificate::CreateOSCertHandleFromBytes(
-      reinterpret_cast<const char*>(google_der), sizeof(google_der));
-  scoped_refptr<X509Certificate> cert5(X509Certificate::CreateFromHandle(
-      google_cert_handle, X509Certificate::SOURCE_FROM_NETWORK,
-      X509Certificate::OSCertHandles()));
-  X509Certificate::FreeOSCertHandle(google_cert_handle);
-
-  EXPECT_EQ(cert3, cert5);
+  // Test that the new certificate, even with intermediates, results in the
+  // same underlying handle being used.
+  EXPECT_EQ(cert1->os_cert_handle(), cert3->os_cert_handle());
+  // Though they use the same OS handle, the intermediates should be different.
+  EXPECT_NE(cert1->GetIntermediateCertificates().size(),
+      cert3->GetIntermediateCertificates().size());
 }
 
 TEST(X509CertificateTest, Pickle) {
@@ -716,13 +802,8 @@ TEST(X509CertificateTest, Pickle) {
 
   X509Certificate::OSCertHandles intermediates;
   intermediates.push_back(thawte_cert_handle);
-  // Faking SOURCE_LONE_CERT_IMPORT so that when the pickled certificate is
-  // read, it successfully evicts |cert| from the X509Certificate::Cache.
-  // This will be fixed when http://crbug.com/49377 is fixed.
   scoped_refptr<X509Certificate> cert = X509Certificate::CreateFromHandle(
-      google_cert_handle,
-      X509Certificate::SOURCE_LONE_CERT_IMPORT,
-      intermediates);
+      google_cert_handle, intermediates);
   ASSERT_NE(static_cast<X509Certificate*>(NULL), cert.get());
 
   X509Certificate::FreeOSCertHandle(google_cert_handle);
@@ -736,7 +817,6 @@ TEST(X509CertificateTest, Pickle) {
       X509Certificate::CreateFromPickle(
           pickle, &iter, X509Certificate::PICKLETYPE_CERTIFICATE_CHAIN);
   ASSERT_NE(static_cast<X509Certificate*>(NULL), cert_from_pickle);
-  EXPECT_NE(cert.get(), cert_from_pickle.get());
   EXPECT_TRUE(X509Certificate::IsSameOSCert(
       cert->os_cert_handle(), cert_from_pickle->os_cert_handle()));
   EXPECT_TRUE(cert->HasIntermediateCertificates(
@@ -779,7 +859,6 @@ TEST(X509CertificateTest, Policy) {
   EXPECT_TRUE(policy.HasDeniedCert());
 }
 
-#if defined(OS_MACOSX) || defined(OS_WIN)
 TEST(X509CertificateTest, IntermediateCertificates) {
   scoped_refptr<X509Certificate> webkit_cert(
       X509Certificate::CreateFromBytes(
@@ -800,8 +879,7 @@ TEST(X509CertificateTest, IntermediateCertificates) {
       reinterpret_cast<const char*>(google_der), sizeof(google_der));
   X509Certificate::OSCertHandles intermediates1;
   scoped_refptr<X509Certificate> cert1;
-  cert1 = X509Certificate::CreateFromHandle(
-      google_handle, X509Certificate::SOURCE_FROM_NETWORK, intermediates1);
+  cert1 = X509Certificate::CreateFromHandle(google_handle, intermediates1);
   EXPECT_TRUE(cert1->HasIntermediateCertificates(intermediates1));
   EXPECT_FALSE(cert1->HasIntermediateCertificate(
       webkit_cert->os_cert_handle()));
@@ -811,11 +889,7 @@ TEST(X509CertificateTest, IntermediateCertificates) {
   intermediates2.push_back(webkit_cert->os_cert_handle());
   intermediates2.push_back(thawte_cert->os_cert_handle());
   scoped_refptr<X509Certificate> cert2;
-  cert2 = X509Certificate::CreateFromHandle(
-      google_handle, X509Certificate::SOURCE_FROM_NETWORK, intermediates2);
-
-  // The cache should have stored cert2 'cause it has more intermediates:
-  EXPECT_NE(cert1, cert2);
+  cert2 = X509Certificate::CreateFromHandle(google_handle, intermediates2);
 
   // Verify it has all the intermediates:
   EXPECT_TRUE(cert2->HasIntermediateCertificate(
@@ -825,20 +899,151 @@ TEST(X509CertificateTest, IntermediateCertificates) {
   EXPECT_FALSE(cert2->HasIntermediateCertificate(
       paypal_cert->os_cert_handle()));
 
-  // Create object with 1 intermediate:
-  X509Certificate::OSCertHandles intermediates3;
-  intermediates2.push_back(thawte_cert->os_cert_handle());
-  scoped_refptr<X509Certificate> cert3;
-  cert3 = X509Certificate::CreateFromHandle(
-      google_handle, X509Certificate::SOURCE_FROM_NETWORK, intermediates3);
-
-  // The cache should have returned cert2 'cause it has more intermediates:
-  EXPECT_EQ(cert3, cert2);
-
   // Cleanup
   X509Certificate::FreeOSCertHandle(google_handle);
 }
-#endif
+
+// Basic test for returning the chain in CertVerifyResult. Note that the
+// returned chain may just be a reflection of the originally supplied chain;
+// that is, if any errors occur, the default chain returned is an exact copy
+// of the certificate to be verified. The remaining VerifyReturn* tests are
+// used to ensure that the actual, verified chain is being returned by
+// Verify().
+TEST(X509CertificateTest, VerifyReturnChainBasic) {
+  FilePath certs_dir = GetTestCertsDirectory();
+  CertificateList certs = CreateCertificateListFromFile(
+      certs_dir, "x509_verify_results.chain.pem",
+      X509Certificate::FORMAT_AUTO);
+  ASSERT_EQ(3U, certs.size());
+
+  X509Certificate::OSCertHandles intermediates;
+  intermediates.push_back(certs[1]->os_cert_handle());
+  intermediates.push_back(certs[2]->os_cert_handle());
+
+  TestRootCerts::GetInstance()->Add(certs[2]);
+
+  scoped_refptr<X509Certificate> google_full_chain =
+      X509Certificate::CreateFromHandle(certs[0]->os_cert_handle(),
+                                        intermediates);
+  ASSERT_NE(static_cast<X509Certificate*>(NULL), google_full_chain);
+  ASSERT_EQ(2U, google_full_chain->GetIntermediateCertificates().size());
+
+  CertVerifyResult verify_result;
+  EXPECT_EQ(static_cast<X509Certificate*>(NULL), verify_result.verified_cert);
+  int error = google_full_chain->Verify("127.0.0.1", 0, &verify_result);
+  EXPECT_EQ(OK, error);
+  ASSERT_NE(static_cast<X509Certificate*>(NULL), verify_result.verified_cert);
+
+  EXPECT_NE(google_full_chain, verify_result.verified_cert);
+  EXPECT_TRUE(X509Certificate::IsSameOSCert(
+      google_full_chain->os_cert_handle(),
+      verify_result.verified_cert->os_cert_handle()));
+  const X509Certificate::OSCertHandles& return_intermediates =
+      verify_result.verified_cert->GetIntermediateCertificates();
+  ASSERT_EQ(2U, return_intermediates.size());
+  EXPECT_TRUE(X509Certificate::IsSameOSCert(return_intermediates[0],
+                                            certs[1]->os_cert_handle()));
+  EXPECT_TRUE(X509Certificate::IsSameOSCert(return_intermediates[1],
+                                            certs[2]->os_cert_handle()));
+
+  TestRootCerts::GetInstance()->Clear();
+}
+
+// Test that the certificate returned in CertVerifyResult is able to reorder
+// certificates that are not ordered from end-entity to root. While this is
+// a protocol violation if sent during a TLS handshake, if multiple sources
+// of intermediate certificates are combined, it's possible that order may
+// not be maintained.
+TEST(X509CertificateTest, VerifyReturnChainProperlyOrdered) {
+  FilePath certs_dir = GetTestCertsDirectory();
+  CertificateList certs = CreateCertificateListFromFile(
+      certs_dir, "x509_verify_results.chain.pem",
+      X509Certificate::FORMAT_AUTO);
+  ASSERT_EQ(3U, certs.size());
+
+  // Construct the chain out of order.
+  X509Certificate::OSCertHandles intermediates;
+  intermediates.push_back(certs[2]->os_cert_handle());
+  intermediates.push_back(certs[1]->os_cert_handle());
+
+  TestRootCerts::GetInstance()->Add(certs[2]);
+
+  scoped_refptr<X509Certificate> google_full_chain =
+      X509Certificate::CreateFromHandle(certs[0]->os_cert_handle(),
+                                        intermediates);
+  ASSERT_NE(static_cast<X509Certificate*>(NULL), google_full_chain);
+  ASSERT_EQ(2U, google_full_chain->GetIntermediateCertificates().size());
+
+  CertVerifyResult verify_result;
+  EXPECT_EQ(static_cast<X509Certificate*>(NULL), verify_result.verified_cert);
+  int error = google_full_chain->Verify("127.0.0.1", 0, &verify_result);
+  EXPECT_EQ(OK, error);
+  ASSERT_NE(static_cast<X509Certificate*>(NULL), verify_result.verified_cert);
+
+  EXPECT_NE(google_full_chain, verify_result.verified_cert);
+  EXPECT_TRUE(X509Certificate::IsSameOSCert(
+      google_full_chain->os_cert_handle(),
+      verify_result.verified_cert->os_cert_handle()));
+  const X509Certificate::OSCertHandles& return_intermediates =
+      verify_result.verified_cert->GetIntermediateCertificates();
+  ASSERT_EQ(2U, return_intermediates.size());
+  EXPECT_TRUE(X509Certificate::IsSameOSCert(return_intermediates[0],
+                                            certs[1]->os_cert_handle()));
+  EXPECT_TRUE(X509Certificate::IsSameOSCert(return_intermediates[1],
+                                            certs[2]->os_cert_handle()));
+
+  TestRootCerts::GetInstance()->Clear();
+}
+
+// Test that Verify() filters out certificates which are not related to
+// or part of the certificate chain being verified.
+TEST(X509CertificateTest, VerifyReturnChainFiltersUnrelatedCerts) {
+  FilePath certs_dir = GetTestCertsDirectory();
+  CertificateList certs = CreateCertificateListFromFile(
+      certs_dir, "x509_verify_results.chain.pem",
+      X509Certificate::FORMAT_AUTO);
+  ASSERT_EQ(3U, certs.size());
+  TestRootCerts::GetInstance()->Add(certs[2]);
+
+  scoped_refptr<X509Certificate> unrelated_dod_certificate =
+      ImportCertFromFile(certs_dir, "dod_ca_17_cert.der");
+  scoped_refptr<X509Certificate> unrelated_dod_certificate2 =
+      ImportCertFromFile(certs_dir, "dod_root_ca_2_cert.der");
+  ASSERT_NE(static_cast<X509Certificate*>(NULL), unrelated_dod_certificate);
+  ASSERT_NE(static_cast<X509Certificate*>(NULL), unrelated_dod_certificate2);
+
+  // Interject unrelated certificates into the list of intermediates.
+  X509Certificate::OSCertHandles intermediates;
+  intermediates.push_back(unrelated_dod_certificate->os_cert_handle());
+  intermediates.push_back(certs[1]->os_cert_handle());
+  intermediates.push_back(unrelated_dod_certificate2->os_cert_handle());
+  intermediates.push_back(certs[2]->os_cert_handle());
+
+  scoped_refptr<X509Certificate> google_full_chain =
+      X509Certificate::CreateFromHandle(certs[0]->os_cert_handle(),
+                                        intermediates);
+  ASSERT_NE(static_cast<X509Certificate*>(NULL), google_full_chain);
+  ASSERT_EQ(4U, google_full_chain->GetIntermediateCertificates().size());
+
+  CertVerifyResult verify_result;
+  EXPECT_EQ(static_cast<X509Certificate*>(NULL), verify_result.verified_cert);
+  int error = google_full_chain->Verify("127.0.0.1", 0, &verify_result);
+  EXPECT_EQ(OK, error);
+  ASSERT_NE(static_cast<X509Certificate*>(NULL), verify_result.verified_cert);
+
+  EXPECT_NE(google_full_chain, verify_result.verified_cert);
+  EXPECT_TRUE(X509Certificate::IsSameOSCert(
+      google_full_chain->os_cert_handle(),
+      verify_result.verified_cert->os_cert_handle()));
+  const X509Certificate::OSCertHandles& return_intermediates =
+      verify_result.verified_cert->GetIntermediateCertificates();
+  ASSERT_EQ(2U, return_intermediates.size());
+  EXPECT_TRUE(X509Certificate::IsSameOSCert(return_intermediates[0],
+                                            certs[1]->os_cert_handle()));
+  EXPECT_TRUE(X509Certificate::IsSameOSCert(return_intermediates[1],
+                                            certs[2]->os_cert_handle()));
+  TestRootCerts::GetInstance()->Clear();
+}
 
 #if defined(OS_MACOSX)
 TEST(X509CertificateTest, IsIssuedBy) {
@@ -1063,9 +1268,14 @@ struct CertificateNameVerifyTestData {
   bool expected;
   // The hostname to match.
   const char* hostname;
-  // '/' separated list of certificate names to match against. Any occurrence
+  // Common name, may be used if |dns_names| or |ip_addrs| are empty.
+  const char* common_name;
+  // Comma separated list of certificate names to match against. Any occurrence
   // of '#' will be replaced with a null character before processing.
-  const char* cert_names;
+  const char* dns_names;
+  // Comma separated list of certificate IP Addresses to match against. Each
+  // address is x prefixed 16 byte hex code for v6 or dotted-decimals for v4.
+  const char* ip_addrs;
 };
 
 // Required by valgrind on mac, otherwise it complains when using its default
@@ -1078,8 +1288,13 @@ struct CertificateNameVerifyTestData {
 // testing::internal2::TypeWithoutFormatter
 // ...
 void PrintTo(const CertificateNameVerifyTestData& data, std::ostream* os) {
-  *os << " expected: " << data.expected << ", hostname: "
-      << data.hostname << ", cert_names: " << data.cert_names;
+  ASSERT_TRUE(data.hostname && data.common_name);
+  // Using StringPiece to allow for optional fields being NULL.
+  *os << " expected: " << data.expected
+      << "; hostname: " << data.hostname
+      << "; common_name: " << data.common_name
+      << "; dns_names: " << base::StringPiece(data.dns_names)
+      << "; ip_addrs: " << base::StringPiece(data.ip_addrs);
 }
 
 const CertificateNameVerifyTestData kNameVerifyTestData[] = {
@@ -1087,51 +1302,104 @@ const CertificateNameVerifyTestData kNameVerifyTestData[] = {
     { true, "foo.com", "foo.com." },
     { true, "f", "f" },
     { true, "f", "f." },
+    { false, "h", "i" },
     { true, "bar.foo.com", "*.foo.com" },
     { true, "www-3.bar.foo.com", "*.bar.foo.com." },
-    { true, "www.test.fr", "*.test.com/*.test.co.uk/*.test.de/*.test.fr" },
-    { true, "wwW.tESt.fr", "//*.*/*.test.de/*.test.FR/www" },
+    { true, "www.test.fr", "common.name",
+        "*.test.com,*.test.co.uk,*.test.de,*.test.fr" },
+    { true, "wwW.tESt.fr",  "common.name",
+        ",*.*,*.test.de,*.test.FR,www" },
     { false, "foo.com", "*.com" },
     { false, "f.uk", ".uk" },
     { true,  "h.co.uk", "*.co.uk" },
-    { false, "192.168.1.11", "*.168.1.11" },
     { false, "foo.us", "*.us" },
-    { false, "www.bar.foo.com",
-      "*.foo.com/*.*.foo.com/*.*.bar.foo.com/*w*.bar.foo.com/*..bar.foo.com" },
     { false, "w.bar.foo.com", "?.bar.foo.com" },
     { false, "www.foo.com", "(www|ftp).foo.com" },
-    { false, "www.foo.com", "www.foo.com#*.foo.com/#" },  // # = null char.
+    { false, "www.foo.com", "www.foo.com#" }, // # = null char.
+    { false, "www.foo.com", "", "www.foo.com#*.foo.com,#,#" },
     { false, "foo", "*" },
     { false, "foo.", "*." },
-    { false, "test.org", "www.test.org/*.test.org/*.org" },
-    { false, "1.2.3.4.5.6", "*.2.3.4.5.6" },
+    { false, "www.house.example", "ww.house.example" },
+    { false, "test.org", "", "www.test.org,*.test.org,*.org" },
+    { false, "w.bar.foo.com", "w*.bar.foo.com" },
+    { false, "www.bar.foo.com", "ww*ww.bar.foo.com" },
+    { false, "wwww.bar.foo.com", "ww*ww.bar.foo.com" },
+    { true, "wwww.bar.foo.com", "w*w.bar.foo.com" },
+    { false, "wwww.bar.foo.com", "w*w.bar.foo.c0m" },
+    { true, "WALLY.bar.foo.com", "wa*.bar.foo.com" },
+    { true, "wally.bar.foo.com", "*Ly.bar.foo.com" },
+    { true, "ww%57.foo.com", "", "www.foo.com" },
+    { true, "www&.foo.com", "www%26.foo.com" },
+    // Common name must not be used if subject alternative name was provided.
+    { false, "www.test.co.jp",  "www.test.co.jp",
+        "*.test.de,*.jp,www.test.co.uk,www.*.co.jp" },
+    { false, "www.bar.foo.com", "www.bar.foo.com",
+      "*.foo.com,*.*.foo.com,*.*.bar.foo.com,*..bar.foo.com," },
+    { false, "www.bath.org", "www.bath.org", "", "20.30.40.50" },
+    { false, "66.77.88.99", "www.bath.org", "www.bath.org" },
     // IDN tests
     { true, "xn--poema-9qae5a.com.br", "xn--poema-9qae5a.com.br" },
     { true, "www.xn--poema-9qae5a.com.br", "*.xn--poema-9qae5a.com.br" },
-    { false, "xn--poema-9qae5a.com.br", "*.xn--poema-9qae5a.com.br" },
-    // The following are adapted from the examples in
-    // http://tools.ietf.org/html/draft-saintandre-tls-server-id-check-09#section-4.4.3
+    { false, "xn--poema-9qae5a.com.br", "", "*.xn--poema-9qae5a.com.br,"
+                                            "xn--poema-*.com.br,"
+                                            "xn--*-9qae5a.com.br,"
+                                            "*--poema-9qae5a.com.br" },
+    { true, "xn--poema-9qae5a.com.br", "*.com.br" },
+    // The following are adapted from the  examples quoted from
+    // http://tools.ietf.org/html/rfc6125#section-6.4.3
+    //  (e.g., *.example.com would match foo.example.com but
+    //   not bar.foo.example.com or example.com).
     { true, "foo.example.com", "*.example.com" },
     { false, "bar.foo.example.com", "*.example.com" },
     { false, "example.com", "*.example.com" },
-    { false, "baz1.example.net", "baz*.example.net" },
-    { false, "baz2.example.net", "baz*.example.net" },
-    { false, "bar.*.example.net", "bar.*.example.net" },
-    { false, "bar.f*o.example.net", "bar.f*o.example.net" },
-    // IP addresses currently not supported, except for the localhost.
+    //   (e.g., baz*.example.net and *baz.example.net and b*z.example.net would
+    //   be taken to match baz1.example.net and foobaz.example.net and
+    //   buzz.example.net, respectively
+    { true, "baz1.example.net", "baz*.example.net" },
+    { true, "foobaz.example.net", "*baz.example.net" },
+    { true, "buzz.example.net", "b*z.example.net" },
+    // IP addresses in common name; IPv4 only.
     { true, "127.0.0.1", "127.0.0.1" },
-    { false, "192.168.1.1", "192.168.1.1" },
-    { false, "FEDC:BA98:7654:3210:FEDC:BA98:7654:3210",
-      "FEDC:BA98:7654:3210:FEDC:BA98:7654:3210" },
-    { false, "FEDC:BA98:7654:3210:FEDC:BA98:7654:3210", "*.]" },
-    { false, "::192.9.5.5", "::192.9.5.5" },
+    { true, "192.168.1.1", "192.168.1.1" },
+    { true,  "676768", "0.10.83.160" },
+    { true,  "1.2.3", "1.2.0.3" },
+    { false, "192.169.1.1", "192.168.1.1" },
+    { false, "12.19.1.1", "12.19.1.1/255.255.255.0" },
+    { false, "FEDC:ba98:7654:3210:FEDC:BA98:7654:3210",
+      "FEDC:BA98:7654:3210:FEDC:ba98:7654:3210" },
+    { false, "1111:2222:3333:4444:5555:6666:7777:8888",
+      "1111:2222:3333:4444:5555:6666:7777:8888" },
+    { false, "::192.9.5.5", "[::192.9.5.5]" },
+    // No wildcard matching in valid IP addresses
     { false, "::192.9.5.5", "*.9.5.5" },
     { false, "2010:836B:4179::836B:4179", "*:836B:4179::836B:4179" },
+    { false, "192.168.1.11", "*.168.1.11" },
+    { false, "FEDC:BA98:7654:3210:FEDC:BA98:7654:3210", "*.]" },
+    // IP addresses in subject alternative name (common name ignored)
+    { true, "10.1.2.3", "", "", "10.1.2.3" },
+    { true,  "14.15", "", "", "14.0.0.15" },
+    { false, "10.1.2.7", "10.1.2.7", "", "10.1.2.6,10.1.2.8" },
+    { false, "10.1.2.8", "10.20.2.8", "foo" },
+    { true, "::4.5.6.7", "", "", "x00000000000000000000000004050607" },
+    { false, "::6.7.8.9", "::6.7.8.9", "::6.7.8.9",
+        "x00000000000000000000000006070808,x0000000000000000000000000607080a,"
+        "xff000000000000000000000006070809,6.7.8.9" },
+    { true, "FE80::200:f8ff:fe21:67cf", "no.common.name", "",
+        "x00000000000000000000000006070808,xfe800000000000000200f8fffe2167cf,"
+        "xff0000000000000000000000060708ff,10.0.0.1" },
+    // Numeric only hostnames (none of these are considered valid IP addresses).
+    { false,  "12345.6", "12345.6" },
+    { false, "121.2.3.512", "", "1*1.2.3.512,*1.2.3.512,1*.2.3.512,*.2.3.512",
+        "121.2.3.0"},
+    { false, "1.2.3.4.5.6", "*.2.3.4.5.6" },
+    { true, "1.2.3.4.5", "", "1.2.3.4.5" },
     // Invalid host names.
-    { false, "www%26.foo.com", "www%26.foo.com" },
+    { false, "junk)(£)$*!@~#", "junk)(£)$*!@~#" },
     { false, "www.*.com", "www.*.com" },
     { false, "w$w.f.com", "w$w.f.com" },
+    { false, "nocolonallowed:example", "", "nocolonallowed:example" },
     { false, "www-1.[::FFFF:129.144.52.38]", "*.[::FFFF:129.144.52.38]" },
+    { false, "[::4.5.6.9]", "", "", "x00000000000000000000000004050609" },
 };
 
 class X509CertificateNameVerifyTest
@@ -1141,15 +1409,54 @@ class X509CertificateNameVerifyTest
 TEST_P(X509CertificateNameVerifyTest, VerifyHostname) {
   CertificateNameVerifyTestData test_data = GetParam();
 
-  std::string cert_name_line(test_data.cert_names);
-  std::replace(cert_name_line.begin(), cert_name_line.end(), '#', '\0');
-  std::vector<std::string> cert_names;
-  base::SplitString(cert_name_line, '/', &cert_names);
+  std::string common_name(test_data.common_name);
+  ASSERT_EQ(std::string::npos, common_name.find(','));
+  std::replace(common_name.begin(), common_name.end(), '#', '\0');
 
-  EXPECT_EQ(test_data.expected,
-            X509Certificate::VerifyHostname(test_data.hostname, cert_names))
-      << "Host [" << test_data.hostname
-      << "], cert name [" << test_data.cert_names << "]";
+  std::vector<std::string> dns_names, ip_addressses;
+  if (test_data.dns_names) {
+    // Build up the certificate DNS names list.
+    std::string dns_name_line(test_data.dns_names);
+    std::replace(dns_name_line.begin(), dns_name_line.end(), '#', '\0');
+    base::SplitString(dns_name_line, ',', &dns_names);
+  }
+
+  if (test_data.ip_addrs) {
+    // Build up the certificate IP address list.
+    std::string ip_addrs_line(test_data.ip_addrs);
+    std::vector<std::string> ip_addressses_ascii;
+    base::SplitString(ip_addrs_line, ',', &ip_addressses_ascii);
+    for (size_t i = 0; i < ip_addressses_ascii.size(); ++i) {
+      std::string& addr_ascii = ip_addressses_ascii[i];
+      ASSERT_NE(0U, addr_ascii.length());
+      if (addr_ascii[0] == 'x') { // Hex encoded address
+        addr_ascii.erase(0, 1);
+        std::vector<uint8> bytes;
+        EXPECT_TRUE(base::HexStringToBytes(addr_ascii, &bytes))
+            << "Could not parse hex address " << addr_ascii << " i = " << i;
+        ip_addressses.push_back(std::string(reinterpret_cast<char*>(&bytes[0]),
+                                            bytes.size()));
+        ASSERT_EQ(16U, ip_addressses.back().size()) << i;
+      } else {  // Decimal groups
+        std::vector<std::string> decimals_ascii;
+        base::SplitString(addr_ascii, '.', &decimals_ascii);
+        EXPECT_EQ(4U, decimals_ascii.size()) << i;
+        std::string addr_bytes;
+        for (size_t j = 0; j < decimals_ascii.size(); ++j) {
+          int decimal_value;
+          EXPECT_TRUE(base::StringToInt(decimals_ascii[j], &decimal_value));
+          EXPECT_GE(decimal_value, 0);
+          EXPECT_LE(decimal_value, 255);
+          addr_bytes.push_back(static_cast<char>(decimal_value));
+        }
+        ip_addressses.push_back(addr_bytes);
+        ASSERT_EQ(4U, ip_addressses.back().size()) << i;
+      }
+    }
+  }
+
+  EXPECT_EQ(test_data.expected, X509Certificate::VerifyHostname(
+      test_data.hostname, common_name, dns_names, ip_addressses));
 }
 
 INSTANTIATE_TEST_CASE_P(, X509CertificateNameVerifyTest,
