@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -18,9 +18,6 @@
 
 using base::Time;
 
-extern volatile int g_cache_tests_received;
-extern volatile bool g_cache_tests_error;
-
 // Tests that can run with different types of caches.
 class DiskCacheEntryTest : public DiskCacheTestWithCache {
  public:
@@ -157,25 +154,21 @@ void DiskCacheEntryTest::InternalAsyncIO() {
   entry->Close();
   ASSERT_EQ(net::OK, OpenEntry("the first key", &entry));
 
-  // Let's verify that each IO goes to the right callback object.
-  CallbackTest callback1(false);
-  CallbackTest callback2(false);
-  CallbackTest callback3(false);
-  CallbackTest callback4(false);
-  CallbackTest callback5(false);
-  CallbackTest callback6(false);
-  CallbackTest callback7(false);
-  CallbackTest callback8(false);
-  CallbackTest callback9(false);
-  CallbackTest callback10(false);
-  CallbackTest callback11(false);
-  CallbackTest callback12(false);
-  CallbackTest callback13(false);
-
-  g_cache_tests_error = false;
-  g_cache_tests_received = 0;
-
   MessageLoopHelper helper;
+  // Let's verify that each IO goes to the right callback object.
+  CallbackTest callback1(&helper, false);
+  CallbackTest callback2(&helper, false);
+  CallbackTest callback3(&helper, false);
+  CallbackTest callback4(&helper, false);
+  CallbackTest callback5(&helper, false);
+  CallbackTest callback6(&helper, false);
+  CallbackTest callback7(&helper, false);
+  CallbackTest callback8(&helper, false);
+  CallbackTest callback9(&helper, false);
+  CallbackTest callback10(&helper, false);
+  CallbackTest callback11(&helper, false);
+  CallbackTest callback12(&helper, false);
+  CallbackTest callback13(&helper, false);
 
   const int kSize1 = 10;
   const int kSize2 = 5000;
@@ -264,8 +257,7 @@ void DiskCacheEntryTest::InternalAsyncIO() {
 
   EXPECT_TRUE(helper.WaitUntilCacheIoFinished(expected));
 
-  EXPECT_FALSE(g_cache_tests_error);
-  EXPECT_EQ(expected, g_cache_tests_received);
+  EXPECT_FALSE(helper.callback_reused_error());
 
   entry->Doom();
   entry->Close();
@@ -364,22 +356,19 @@ void DiskCacheEntryTest::ExternalAsyncIO() {
   disk_cache::Entry* entry;
   ASSERT_EQ(net::OK, CreateEntry("the first key", &entry));
 
-  // Let's verify that each IO goes to the right callback object.
-  CallbackTest callback1(false);
-  CallbackTest callback2(false);
-  CallbackTest callback3(false);
-  CallbackTest callback4(false);
-  CallbackTest callback5(false);
-  CallbackTest callback6(false);
-  CallbackTest callback7(false);
-  CallbackTest callback8(false);
-  CallbackTest callback9(false);
-
-  g_cache_tests_error = false;
-  g_cache_tests_received = 0;
   int expected = 0;
 
   MessageLoopHelper helper;
+  // Let's verify that each IO goes to the right callback object.
+  CallbackTest callback1(&helper, false);
+  CallbackTest callback2(&helper, false);
+  CallbackTest callback3(&helper, false);
+  CallbackTest callback4(&helper, false);
+  CallbackTest callback5(&helper, false);
+  CallbackTest callback6(&helper, false);
+  CallbackTest callback7(&helper, false);
+  CallbackTest callback8(&helper, false);
+  CallbackTest callback9(&helper, false);
 
   const int kSize1 = 17000;
   const int kSize2 = 25000;
@@ -448,8 +437,7 @@ void DiskCacheEntryTest::ExternalAsyncIO() {
   EXPECT_TRUE(helper.WaitUntilCacheIoFinished(expected));
   EXPECT_EQ(37000, entry->GetDataSize(1));
 
-  EXPECT_FALSE(g_cache_tests_error);
-  EXPECT_EQ(expected, g_cache_tests_received);
+  EXPECT_FALSE(helper.callback_reused_error());
 
   entry->Doom();
   entry->Close();
@@ -1944,3 +1932,29 @@ TEST_F(DiskCacheEntryTest, CancelSparseIO) {
   EXPECT_EQ(0, cb5.GetResult(rv));
   entry->Close();
 }
+
+// Tests that we perform sanity checks on an entry's key. Note that there are
+// other tests that exercise sanity checks by using saved corrupt files.
+TEST_F(DiskCacheEntryTest, KeySanityCheck) {
+  UseCurrentThread();
+  InitCache();
+  std::string key("the first key");
+  disk_cache::Entry* entry;
+  ASSERT_EQ(net::OK, CreateEntry(key, &entry));
+
+  disk_cache::EntryImpl* entry_impl =
+      static_cast<disk_cache::EntryImpl*>(entry);
+  disk_cache::EntryStore* store = entry_impl->entry()->Data();
+
+  // We have reserved space for a short key (one block), let's say that the key
+  // takes more than one block, and remove the NULLs after the actual key.
+  store->key_len = 800;
+  memset(store->key + key.size(), 'k', sizeof(store->key) - key.size());
+  entry_impl->entry()->set_modified();
+  entry->Close();
+
+  // We have a corrupt entry. Now reload it. We should NOT read beyond the
+  // allocated buffer here.
+  ASSERT_NE(net::OK, OpenEntry(key, &entry));
+  DisableIntegrityCheck();
+}
