@@ -12,12 +12,13 @@
 #include "base/basictypes.h"
 #include "base/string16.h"
 #include "base/time.h"
-#include "net/base/net_api.h"
+#include "net/base/net_export.h"
 
 namespace net {
 
 struct FtpDirectoryListingEntry {
   enum Type {
+    UNKNOWN,
     FILE,
     DIRECTORY,
     SYMLINK,
@@ -36,7 +37,7 @@ struct FtpDirectoryListingEntry {
 
 // Parses an FTP directory listing |text|. On success fills in |entries|.
 // Returns network error code.
-NET_API int ParseFtpDirectoryListing(
+NET_EXPORT int ParseFtpDirectoryListing(
     const std::string& text,
     const base::Time& current_time,
     std::vector<FtpDirectoryListingEntry>* entries);
