@@ -244,15 +244,10 @@ std::string DescribeOid(GSSAPILibrary* gssapi_lib, const gss_OID oid) {
     // Check if the first |kMaxCharsToPrint| characters
     // contain only printable characters and are NULL terminated.
     const char* str = reinterpret_cast<const char*>(oid);
-    bool is_printable = true;
     size_t str_length = 0;
     for ( ; str_length < kMaxCharsToPrint; ++str_length) {
-      if (!str[str_length])
+      if (!str[str_length] || !isprint(str[str_length]))
         break;
-      if (!isprint(str[str_length])) {
-        is_printable = false;
-        break;
-      }
     }
     if (!str[str_length]) {
       output += base::StringPrintf("\"%s\"", str);
@@ -415,9 +410,11 @@ bool GSSAPISharedLibrary::Init() {
 
 bool GSSAPISharedLibrary::InitImpl() {
   DCHECK(!initialized_);
+#if defined(DLOPEN_KERBEROS)
   gssapi_library_ = LoadSharedLibrary();
   if (gssapi_library_ == NULL)
     return false;
+#endif  // defined(DLOPEN_KERBEROS)
   initialized_ = true;
   return true;
 }
@@ -464,17 +461,20 @@ base::NativeLibrary GSSAPISharedLibrary::LoadSharedLibrary() {
   return NULL;
 }
 
+#if defined(DLOPEN_KERBEROS)
 #define BIND(lib, x)                                                    \
+  DCHECK(lib);                                                          \
   gss_##x##_type x = reinterpret_cast<gss_##x##_type>(                  \
       base::GetFunctionPointerFromNativeLibrary(lib, "gss_" #x));       \
   if (x == NULL) {                                                      \
     LOG(WARNING) << "Unable to bind function \"" << "gss_" #x << "\"";  \
     return false;                                                       \
   }
+#else
+#define BIND(lib, x) gss_##x##_type x = gss_##x
+#endif
 
 bool GSSAPISharedLibrary::BindMethods(base::NativeLibrary lib) {
-  DCHECK(lib != NULL);
-
   BIND(lib, import_name);
   BIND(lib, release_name);
   BIND(lib, release_buffer);
@@ -674,6 +674,10 @@ bool HttpAuthGSSAPI::NeedsIdentity() const {
   return decoded_server_auth_token_.empty();
 }
 
+bool HttpAuthGSSAPI::AllowsExplicitCredentials() const {
+  return false;
+}
+
 void HttpAuthGSSAPI::Delegate() {
   can_delegate_ = true;
 }
