@@ -114,6 +114,13 @@ int MapHttpResponseCode(int code) {
   return 0;
 }
 
+void CheckDoesNotHaveEmbededNulls(const std::string& str) {
+  // Care needs to be taken when adding values to the raw headers string to
+  // make sure it does not contain embeded NULLs. Any embeded '\0' may be
+  // understood as line terminators and change how header lines get tokenized.
+  CHECK(str.find('\0') == std::string::npos);
+}
+
 }  // namespace
 
 struct HttpResponseHeaders::ParsedHeader {
@@ -300,6 +307,7 @@ void HttpResponseHeaders::RemoveHeader(const std::string& name) {
 }
 
 void HttpResponseHeaders::AddHeader(const std::string& header) {
+  CheckDoesNotHaveEmbededNulls(header);
   DCHECK_EQ('\0', raw_headers_[raw_headers_.size() - 2]);
   DCHECK_EQ('\0', raw_headers_[raw_headers_.size() - 1]);
   // Don't copy the last null.
@@ -315,6 +323,7 @@ void HttpResponseHeaders::AddHeader(const std::string& header) {
 }
 
 void HttpResponseHeaders::ReplaceStatusLine(const std::string& new_status) {
+  CheckDoesNotHaveEmbededNulls(new_status);
   // Copy up to the null byte.  This just copies the status line.
   std::string new_raw_headers(new_status);
   new_raw_headers.push_back('\0');
@@ -329,7 +338,7 @@ void HttpResponseHeaders::Parse(const std::string& raw_input) {
   // ParseStatusLine adds a normalized status line to raw_headers_
   std::string::const_iterator line_begin = raw_input.begin();
   std::string::const_iterator line_end =
-      find(line_begin, raw_input.end(), '\0');
+      std::find(line_begin, raw_input.end(), '\0');
   // has_headers = true, if there is any data following the status line.
   // Used by ParseStatusLine() to decide if a HTTP/0.9 is really a HTTP/1.0.
   bool has_headers = (line_end != raw_input.end() &&
@@ -457,7 +466,7 @@ std::string HttpResponseHeaders::GetStatusText() const {
   std::string::const_iterator begin = status_text.begin();
   std::string::const_iterator end = status_text.end();
   for (int i = 0; i < 2; ++i)
-    begin = find(begin, end, ' ') + 1;
+    begin = std::find(begin, end, ' ') + 1;
   return std::string(begin, end);
 }
 
@@ -557,7 +566,7 @@ HttpVersion HttpResponseHeaders::ParseVersion(
     return HttpVersion();
   }
 
-  std::string::const_iterator dot = find(p, line_end, '.');
+  std::string::const_iterator dot = std::find(p, line_end, '.');
   if (dot == line_end) {
     DVLOG(1) << "malformed version";
     return HttpVersion();
@@ -604,7 +613,7 @@ void HttpResponseHeaders::ParseStatusLine(
   }
 
   // TODO(eroman): this doesn't make sense if ParseVersion failed.
-  std::string::const_iterator p = find(line_begin, line_end, ' ');
+  std::string::const_iterator p = std::find(line_begin, line_end, ' ');
 
   if (p == line_end) {
     DVLOG(1) << "missing response status; assuming 200 OK";
@@ -1047,8 +1056,7 @@ bool HttpResponseHeaders::GetTimeValuedHeader(const std::string& name,
   if (!EnumerateHeader(NULL, name, &value))
     return false;
 
-  std::wstring value_wide(value.begin(), value.end());  // inflate ascii
-  return Time::FromString(value_wide.c_str(), result);
+  return Time::FromString(value.c_str(), result);
 }
 
 bool HttpResponseHeaders::IsKeepAlive() const {
