@@ -35,7 +35,7 @@ class HttpResponseHeadersTest : public testing::Test {
 // Transform "normal"-looking headers (\n-separated) to the appropriate
 // input format for ParseRawHeaders (\0-separated).
 void HeadersToRaw(std::string* headers) {
-  replace(headers->begin(), headers->end(), '\n', '\0');
+  std::replace(headers->begin(), headers->end(), '\n', '\0');
   if (!headers->empty())
     *headers += '\0';
 }
@@ -51,10 +51,10 @@ void TestCommon(const TestData& test) {
   parsed->GetNormalizedHeaders(&headers);
 
   // Transform to readable output format (so it's easier to see diffs).
-  replace(headers.begin(), headers.end(), ' ', '_');
-  replace(headers.begin(), headers.end(), '\n', '\\');
-  replace(expected_headers.begin(), expected_headers.end(), ' ', '_');
-  replace(expected_headers.begin(), expected_headers.end(), '\n', '\\');
+  std::replace(headers.begin(), headers.end(), ' ', '_');
+  std::replace(headers.begin(), headers.end(), '\n', '\\');
+  std::replace(expected_headers.begin(), expected_headers.end(), ' ', '_');
+  std::replace(expected_headers.begin(), expected_headers.end(), '\n', '\\');
 
   EXPECT_EQ(expected_headers, headers);
 
@@ -188,6 +188,19 @@ TEST(HttpResponseHeadersTest, NormalizeHeadersBadStatus) {
   TestCommon(test);
 }
 
+TEST(HttpResponseHeadersTest, NormalizeHeadersInvalidStatusCode) {
+  TestData test = {
+    "HTTP/1.1 -1  Unknown\n",
+
+    "HTTP/1.1 200 OK\n",
+
+    200,
+    net::HttpVersion(1,1),
+    net::HttpVersion(1,1)
+  };
+  TestCommon(test);
+}
+
 TEST(HttpResponseHeadersTest, NormalizeHeadersEmpty) {
   TestData test = {
     "",
@@ -799,9 +812,9 @@ TEST(HttpResponseHeadersTest, RequiresValidation) {
     // TODO(darin): add many many more tests here
   };
   base::Time request_time, response_time, current_time;
-  base::Time::FromString(L"Wed, 28 Nov 2007 00:40:09 GMT", &request_time);
-  base::Time::FromString(L"Wed, 28 Nov 2007 00:40:12 GMT", &response_time);
-  base::Time::FromString(L"Wed, 28 Nov 2007 00:45:20 GMT", &current_time);
+  base::Time::FromString("Wed, 28 Nov 2007 00:40:09 GMT", &request_time);
+  base::Time::FromString("Wed, 28 Nov 2007 00:40:12 GMT", &response_time);
+  base::Time::FromString("Wed, 28 Nov 2007 00:45:20 GMT", &current_time);
 
   for (size_t i = 0; i < ARRAYSIZE_UNSAFE(tests); ++i) {
     std::string headers(tests[i].headers);
