@@ -6,6 +6,8 @@
 #define NET_OCSP_NSS_OCSP_H_
 #pragma once
 
+#include "net/base/net_export.h"
+
 namespace net {
 
 class URLRequestContext;
@@ -13,19 +15,19 @@ class URLRequestContext;
 // Sets the MessageLoop for OCSP to the current message loop.
 // This should be called before EnsureOCSPInit() if you want to
 // control the message loop for OCSP.
-void SetMessageLoopForOCSP();
+NET_EXPORT void SetMessageLoopForOCSP();
 
 // Initializes OCSP handlers for NSS.  This must be called before any
 // certificate verification functions.  This function is thread-safe, and OCSP
 // handlers will only ever be initialized once.  ShutdownOCSP() must be called
 // on shutdown.
-void EnsureOCSPInit();
+NET_EXPORT void EnsureOCSPInit();
 
 // This should be called once on shutdown to stop issuing URLRequests for OCSP.
-void ShutdownOCSP();
+NET_EXPORT void ShutdownOCSP();
 
 // Set URLRequestContext for OCSP handlers.
-void SetURLRequestContextForOCSP(URLRequestContext* request_context);
+NET_EXPORT void SetURLRequestContextForOCSP(URLRequestContext* request_context);
 
 }  // namespace net
 
