@@ -38,8 +38,8 @@
 #include "net/base/address_list.h"
 #include "net/base/completion_callback.h"
 #include "net/base/load_states.h"
-#include "net/base/net_api.h"
 #include "net/base/net_errors.h"
+#include "net/base/net_export.h"
 #include "net/base/net_log.h"
 #include "net/base/network_change_notifier.h"
 #include "net/base/request_priority.h"
@@ -50,12 +50,19 @@ namespace net {
 
 class ClientSocketHandle;
 
+// Returns the client socket reuse policy.
+NET_EXPORT_PRIVATE int GetSocketReusePolicy();
+
+// Sets the client socket reuse policy.
+// NOTE: 'policy' should be a valid ClientSocketReusePolicy enum value.
+NET_EXPORT void SetSocketReusePolicy(int policy);
+
 // ConnectJob provides an abstract interface for "connecting" a socket.
 // The connection may involve host resolution, tcp connection, ssl connection,
 // etc.
-class NET_TEST ConnectJob {
+class NET_EXPORT_PRIVATE ConnectJob {
  public:
-  class NET_TEST Delegate {
+  class NET_EXPORT_PRIVATE Delegate {
    public:
     Delegate() {}
     virtual ~Delegate() {}
@@ -155,7 +162,7 @@ namespace internal {
 // ClientSocketPoolBase adds templated definitions built on top of
 // ClientSocketPoolBaseHelper.  This class is not for external use, please use
 // ClientSocketPoolBase instead.
-class NET_TEST ClientSocketPoolBaseHelper
+class NET_EXPORT_PRIVATE ClientSocketPoolBaseHelper
     : public ConnectJob::Delegate,
       public NetworkChangeNotifier::IPAddressObserver {
  public:
@@ -167,7 +174,18 @@ class NET_TEST ClientSocketPoolBaseHelper
     NO_IDLE_SOCKETS = 0x1,  // Do not return an idle socket. Create a new one.
   };
 
-  class NET_TEST Request {
+  enum ClientSocketReusePolicy {
+    // Socket with largest amount of bytes transferred.
+    USE_WARMEST_SOCKET = 0,
+
+    // Socket which scores highest on large bytes transferred and low idle time.
+    USE_WARM_SOCKET = 1,
+
+    // Socket which was most recently used.
+    USE_LAST_ACCESSED_SOCKET = 2,
+  };
+
+  class NET_EXPORT_PRIVATE Request {
    public:
     Request(ClientSocketHandle* handle,
             CompletionCallback* callback,
@@ -280,8 +298,8 @@ class NET_TEST ClientSocketPoolBaseHelper
   void CleanupIdleSockets(bool force);
 
   // See ClientSocketPool::GetInfoAsValue for documentation on this function.
-  DictionaryValue* GetInfoAsValue(const std::string& name,
-                                  const std::string& type) const;
+  base::DictionaryValue* GetInfoAsValue(const std::string& name,
+                                        const std::string& type) const;
 
   base::TimeDelta ConnectionTimeout() const {
     return connect_job_factory_->ConnectionTimeout();
@@ -694,8 +712,8 @@ class ClientSocketPoolBase {
     return helper_.CleanupIdleSockets(force);
   }
 
-  DictionaryValue* GetInfoAsValue(const std::string& name,
-                                  const std::string& type) const {
+  base::DictionaryValue* GetInfoAsValue(const std::string& name,
+                                        const std::string& type) const {
     return helper_.GetInfoAsValue(name, type);
   }
 
