@@ -40,7 +40,7 @@ int g_max_sockets_per_group = 6;
 // The max number of sockets to allow per proxy server.  This applies both to
 // http and SOCKS proxies.  See http://crbug.com/12066 and
 // http://crbug.com/44501 for details about proxy server connection limits.
-int g_max_sockets_per_proxy_server = 32;
+int g_max_sockets_per_proxy_server = kDefaultMaxSocketsPerProxyServer;
 
 // Appends information about all |socket_pools| to the end of |list|.
 template <class MapType>
@@ -58,7 +58,10 @@ static void AddSocketPoolsToList(ListValue* list,
 
 // The meat of the implementation for the InitSocketHandleForHttpRequest,
 // InitSocketHandleForRawConnect and PreconnectSocketsForHttpRequest methods.
-int InitSocketPoolHelper(const HttpRequestInfo& request_info,
+int InitSocketPoolHelper(const GURL& request_url,
+                         const HttpRequestHeaders& request_extra_headers,
+                         int request_load_flags,
+                         RequestPriority request_priority,
                          HttpNetworkSession* session,
                          const ProxyInfo& proxy_info,
                          bool force_spdy_over_ssl,
@@ -75,18 +78,25 @@ int InitSocketPoolHelper(const HttpRequestInfo& request_info,
   scoped_refptr<SOCKSSocketParams> socks_params;
   scoped_ptr<HostPortPair> proxy_host_port;
 
-  bool using_ssl = request_info.url.SchemeIs("https") || force_spdy_over_ssl;
+  GURL request_referrer;
+  std::string request_referrer_str;
+  if (request_extra_headers.GetHeader(HttpRequestHeaders::kReferer,
+                                      &request_referrer_str)) {
+    request_referrer = GURL(request_referrer_str);
+  }
+
+  bool using_ssl = request_url.SchemeIs("https") || force_spdy_over_ssl;
 
   HostPortPair origin_host_port =
-      HostPortPair(request_info.url.HostNoBrackets(),
-                   request_info.url.EffectiveIntPort());
+      HostPortPair(request_url.HostNoBrackets(),
+                   request_url.EffectiveIntPort());
 
   bool disable_resolver_cache =
-      request_info.load_flags & LOAD_BYPASS_CACHE ||
-      request_info.load_flags & LOAD_VALIDATE_CACHE ||
-      request_info.load_flags & LOAD_DISABLE_CACHE;
+      request_load_flags & LOAD_BYPASS_CACHE ||
+      request_load_flags & LOAD_VALIDATE_CACHE ||
+      request_load_flags & LOAD_DISABLE_CACHE;
 
-  int load_flags = request_info.load_flags;
+  int load_flags = request_load_flags;
   if (HttpStreamFactory::ignore_certificate_errors())
     load_flags |= LOAD_IGNORE_ALL_CERT_ERRORS;
 
@@ -104,27 +114,27 @@ int InitSocketPoolHelper(const HttpRequestInfo& request_info,
     connection_group = prefix + connection_group;
   }
 
-  bool ignore_limits = (request_info.load_flags & LOAD_IGNORE_LIMITS) != 0;
+  bool ignore_limits = (request_load_flags & LOAD_IGNORE_LIMITS) != 0;
   if (proxy_info.is_direct()) {
     tcp_params = new TransportSocketParams(origin_host_port,
-                                     request_info.priority,
-                                     request_info.referrer,
-                                     disable_resolver_cache,
-                                     ignore_limits);
+                                           request_priority,
+                                           request_referrer,
+                                           disable_resolver_cache,
+                                           ignore_limits);
   } else {
     ProxyServer proxy_server = proxy_info.proxy_server();
     proxy_host_port.reset(new HostPortPair(proxy_server.host_port_pair()));
     scoped_refptr<TransportSocketParams> proxy_tcp_params(
         new TransportSocketParams(*proxy_host_port,
-                            request_info.priority,
-                            request_info.referrer,
-                            disable_resolver_cache,
-                            ignore_limits));
+                                  request_priority,
+                                  request_referrer,
+                                  disable_resolver_cache,
+                                  ignore_limits));
 
     if (proxy_info.is_http() || proxy_info.is_https()) {
       std::string user_agent;
-      request_info.extra_headers.GetHeader(HttpRequestHeaders::kUserAgent,
-                                           &user_agent);
+      request_extra_headers.GetHeader(HttpRequestHeaders::kUserAgent,
+                                      &user_agent);
       scoped_refptr<SSLSocketParams> ssl_params;
       if (proxy_info.is_https()) {
         // Set ssl_params, and unset proxy_tcp_params
@@ -143,7 +153,7 @@ int InitSocketPoolHelper(const HttpRequestInfo& request_info,
       http_proxy_params =
           new HttpProxySocketParams(proxy_tcp_params,
                                     ssl_params,
-                                    request_info.url,
+                                    request_url,
                                     user_agent,
                                     origin_host_port,
                                     session->http_auth_cache(),
@@ -163,8 +173,8 @@ int InitSocketPoolHelper(const HttpRequestInfo& request_info,
       socks_params = new SOCKSSocketParams(proxy_tcp_params,
                                            socks_version == '5',
                                            origin_host_port,
-                                           request_info.priority,
-                                           request_info.referrer);
+                                           request_priority,
+                                           request_referrer);
     }
   }
 
@@ -193,7 +203,7 @@ int InitSocketPoolHelper(const HttpRequestInfo& request_info,
     }
 
     return socket_handle->Init(connection_group, ssl_params,
-                               request_info.priority, callback, ssl_pool,
+                               request_priority, callback, ssl_pool,
                                net_log);
   }
 
@@ -208,7 +218,7 @@ int InitSocketPoolHelper(const HttpRequestInfo& request_info,
     }
 
     return socket_handle->Init(connection_group, http_proxy_params,
-                               request_info.priority, callback,
+                               request_priority, callback,
                                pool, net_log);
   }
 
@@ -222,7 +232,7 @@ int InitSocketPoolHelper(const HttpRequestInfo& request_info,
     }
 
     return socket_handle->Init(connection_group, socks_params,
-                               request_info.priority, callback, pool,
+                               request_priority, callback, pool,
                                net_log);
   }
 
@@ -236,7 +246,7 @@ int InitSocketPoolHelper(const HttpRequestInfo& request_info,
   }
 
   return socket_handle->Init(connection_group, tcp_params,
-                             request_info.priority, callback,
+                             request_priority, callback,
                              pool, net_log);
 }
 
@@ -247,6 +257,7 @@ ClientSocketPoolManager::ClientSocketPoolManager(
     ClientSocketFactory* socket_factory,
     HostResolver* host_resolver,
     CertVerifier* cert_verifier,
+    OriginBoundCertService* origin_bound_cert_service,
     DnsRRResolver* dnsrr_resolver,
     DnsCertProvenanceChecker* dns_cert_checker,
     SSLHostInfoFactory* ssl_host_info_factory,
@@ -256,6 +267,7 @@ ClientSocketPoolManager::ClientSocketPoolManager(
       socket_factory_(socket_factory),
       host_resolver_(host_resolver),
       cert_verifier_(cert_verifier),
+      origin_bound_cert_service_(origin_bound_cert_service),
       dnsrr_resolver_(dnsrr_resolver),
       dns_cert_checker_(dns_cert_checker),
       ssl_host_info_factory_(ssl_host_info_factory),
@@ -274,6 +286,7 @@ ClientSocketPoolManager::ClientSocketPoolManager(
           &ssl_pool_histograms_,
           host_resolver,
           cert_verifier,
+          origin_bound_cert_service,
           dnsrr_resolver,
           dns_cert_checker,
           ssl_host_info_factory,
@@ -478,6 +491,7 @@ HttpProxyClientSocketPool* ClientSocketPoolManager::GetSocketPoolForHTTPProxy(
                   &ssl_for_https_proxy_pool_histograms_,
                   host_resolver_,
                   cert_verifier_,
+                  origin_bound_cert_service_,
                   dnsrr_resolver_,
                   dns_cert_checker_,
                   ssl_host_info_factory_,
@@ -515,6 +529,7 @@ SSLClientSocketPool* ClientSocketPoolManager::GetSocketPoolForSSLWithProxy(
       &ssl_pool_histograms_,
       host_resolver_,
       cert_verifier_,
+      origin_bound_cert_service_,
       dnsrr_resolver_,
       dns_cert_checker_,
       ssl_host_info_factory_,
@@ -608,7 +623,10 @@ void ClientSocketPoolManager::OnCertTrustChanged(const X509Certificate* cert) {
 
 // static
 int ClientSocketPoolManager::InitSocketHandleForHttpRequest(
-    const HttpRequestInfo& request_info,
+    const GURL& request_url,
+    const HttpRequestHeaders& request_extra_headers,
+    int request_load_flags,
+    RequestPriority request_priority,
     HttpNetworkSession* session,
     const ProxyInfo& proxy_info,
     bool force_spdy_over_ssl,
@@ -619,7 +637,10 @@ int ClientSocketPoolManager::InitSocketHandleForHttpRequest(
     ClientSocketHandle* socket_handle,
     CompletionCallback* callback) {
   DCHECK(socket_handle);
-  return InitSocketPoolHelper(request_info,
+  return InitSocketPoolHelper(request_url,
+                              request_extra_headers,
+                              request_load_flags,
+                              request_priority,
                               session,
                               proxy_info,
                               force_spdy_over_ssl,
@@ -645,9 +666,15 @@ int ClientSocketPoolManager::InitSocketHandleForRawConnect(
     CompletionCallback* callback) {
   DCHECK(socket_handle);
   // Synthesize an HttpRequestInfo.
-  HttpRequestInfo request_info;
-  request_info.url = GURL("http://" + host_port_pair.ToString());
-  return InitSocketPoolHelper(request_info,
+  GURL request_url = GURL("http://" + host_port_pair.ToString());
+  HttpRequestHeaders request_extra_headers;
+  int request_load_flags = 0;
+  RequestPriority request_priority = MEDIUM;
+
+  return InitSocketPoolHelper(request_url,
+                              request_extra_headers,
+                              request_load_flags,
+                              request_priority,
                               session,
                               proxy_info,
                               false,
@@ -663,7 +690,10 @@ int ClientSocketPoolManager::InitSocketHandleForRawConnect(
 
 // static
 int ClientSocketPoolManager::PreconnectSocketsForHttpRequest(
-    const HttpRequestInfo& request_info,
+    const GURL& request_url,
+    const HttpRequestHeaders& request_extra_headers,
+    int request_load_flags,
+    RequestPriority request_priority,
     HttpNetworkSession* session,
     const ProxyInfo& proxy_info,
     bool force_spdy_over_ssl,
@@ -672,7 +702,10 @@ int ClientSocketPoolManager::PreconnectSocketsForHttpRequest(
     const SSLConfig& ssl_config_for_proxy,
     const BoundNetLog& net_log,
     int num_preconnect_streams) {
-  return InitSocketPoolHelper(request_info,
+  return InitSocketPoolHelper(request_url,
+                              request_extra_headers,
+                              request_load_flags,
+                              request_priority,
                               session,
                               proxy_info,
                               force_spdy_over_ssl,
