@@ -73,6 +73,8 @@
 #include "base/stringprintf.h"
 #include "base/threading/thread_restrictions.h"
 #include "base/values.h"
+#include "crypto/rsa_private_key.h"
+#include "crypto/scoped_nss_types.h"
 #include "net/base/address_list.h"
 #include "net/base/cert_status_flags.h"
 #include "net/base/cert_verifier.h"
@@ -305,96 +307,65 @@ enum DNSValidationResult {
   DNSVR_CONTINUE,  // perform CA validation as usual.
 };
 
-// VerifyTXTRecords processes the RRDATA for a number of DNS TXT records and
-// checks them against the given certificate.
-//   dnssec: if true then the TXT records are DNSSEC validated. In this case,
-//       DNSVR_SUCCESS may be returned.
-//    server_cert_nss: the certificate to validate
-//    rrdatas: the TXT records for the current domain.
-DNSValidationResult VerifyTXTRecords(
-    bool dnssec,
+// VerifyCAARecords processes DNSSEC validated RRDATA for a number of DNS CAA
+// records and checks them against the given chain.
+//    server_cert_nss: the server's leaf certificate.
+//    rrdatas: the CAA records for the current domain.
+//    port: the TCP port number that we connected to.
+DNSValidationResult VerifyCAARecords(
     CERTCertificate* server_cert_nss,
-    const std::vector<base::StringPiece>& rrdatas) {
-  bool found_well_formed_record = false;
-  bool matched_record = false;
+    const std::vector<base::StringPiece>& rrdatas,
+    uint16 port) {
+  DnsCAARecord::Policy policy;
+  const DnsCAARecord::ParseResult r = DnsCAARecord::Parse(rrdatas, &policy);
+  if (r == DnsCAARecord::SYNTAX_ERROR || r == DnsCAARecord::UNKNOWN_CRITICAL)
+    return DNSVR_FAILURE;
+  if (r == DnsCAARecord::DISCARD)
+    return DNSVR_CONTINUE;
+  DCHECK(r == DnsCAARecord::SUCCESS);
 
-  for (std::vector<base::StringPiece>::const_iterator
-       i = rrdatas.begin(); i != rrdatas.end(); ++i) {
-    std::map<std::string, std::string> m(
-        DNSSECChainVerifier::ParseTLSTXTRecord(*i));
-    if (m.empty())
-      continue;
+  for (std::vector<DnsCAARecord::Policy::Hash>::const_iterator
+       hash = policy.authorized_hashes.begin();
+       hash != policy.authorized_hashes.end();
+       ++hash) {
+    if (hash->target == DnsCAARecord::Policy::SUBJECT_PUBLIC_KEY_INFO &&
+        (hash->port == 0 || hash->port == port)) {
+      CHECK_LE(hash->data.size(), static_cast<unsigned>(SHA512_LENGTH));
+      uint8 calculated_hash[SHA512_LENGTH];  // SHA512 is the largest.
+      SECStatus rv = HASH_HashBuf(
+          static_cast<HASH_HashType>(hash->algorithm),
+          calculated_hash,
+          server_cert_nss->derPublicKey.data,
+          server_cert_nss->derPublicKey.len);
+      DCHECK(rv == SECSuccess);
+      const std::string actual_digest(reinterpret_cast<char*>(calculated_hash),
+                                      hash->data.size());
 
-    std::map<std::string, std::string>::const_iterator j;
-    j = m.find("v");
-    if (j == m.end() || j->second != "tls1")
-      continue;
-
-    j = m.find("ha");
-
-    HASH_HashType hash_algorithm;
-    unsigned hash_length;
-    if (j == m.end() || j->second == "sha1") {
-      hash_algorithm = HASH_AlgSHA1;
-      hash_length = SHA1_LENGTH;
-    } else if (j->second == "sha256") {
-      hash_algorithm = HASH_AlgSHA256;
-      hash_length = SHA256_LENGTH;
-    } else {
-      continue;
-    }
-
-    j = m.find("h");
-    if (j == m.end())
-      continue;
-
-    std::vector<uint8> given_hash;
-    if (!base::HexStringToBytes(j->second, &given_hash))
-      continue;
-
-    if (given_hash.size() != hash_length)
-      continue;
-
-    uint8 calculated_hash[SHA256_LENGTH];  // SHA256 is the largest.
-    SECStatus rv;
-
-    j = m.find("hr");
-    if (j == m.end() || j->second == "pubkey") {
-      rv = HASH_HashBuf(hash_algorithm, calculated_hash,
-                        server_cert_nss->derPublicKey.data,
-                        server_cert_nss->derPublicKey.len);
-    } else if (j->second == "cert") {
-      rv = HASH_HashBuf(hash_algorithm, calculated_hash,
-                        server_cert_nss->derCert.data,
-                        server_cert_nss->derCert.len);
-    } else {
-      continue;
-    }
-
-    if (rv != SECSuccess)
-      NOTREACHED();
-
-    found_well_formed_record = true;
-
-    if (memcmp(calculated_hash, &given_hash[0], hash_length) == 0) {
-      matched_record = true;
-      if (dnssec)
+      // Note that the parser ensures that hash->data.size() is correct for the
+      // given algorithm. An attacker cannot give a zero length hash that
+      // always matches.
+      if (actual_digest == hash->data) {
+        // A DNSSEC secure hash over the public key of the leaf-certificate
+        // is sufficient.
         return DNSVR_SUCCESS;
+      }
     }
   }
 
-  if (found_well_formed_record && !matched_record)
-    return DNSVR_FAILURE;
-
-  return DNSVR_CONTINUE;
+  // If a CAA record was found, but nothing matched, then we reject the
+  // certificate.
+  return DNSVR_FAILURE;
 }
 
 // CheckDNSSECChain tries to validate a DNSSEC chain embedded in
 // |server_cert_nss_|. It returns true iff a chain is found that proves the
-// value of a TXT record that contains a valid public key fingerprint.
+// value of a CAA record that contains a valid public key fingerprint.
+// |port| contains the TCP port number that we connected to as CAA records can
+// be specific to a given port.
 DNSValidationResult CheckDNSSECChain(
     const std::string& hostname,
-    CERTCertificate* server_cert_nss) {
+    CERTCertificate* server_cert_nss,
+    uint16 port) {
   if (!server_cert_nss)
     return DNSVR_CONTINUE;
 
@@ -439,12 +410,13 @@ DNSValidationResult CheckDNSSECChain(
     return DNSVR_CONTINUE;
   }
 
-  if (verifier.rrtype() != kDNS_TXT)
+  if (verifier.rrtype() != kDNS_CAA)
     return DNSVR_CONTINUE;
 
-  DNSValidationResult r = VerifyTXTRecords(
-      true /* DNSSEC verified */, server_cert_nss, verifier.rrdatas());
+  DNSValidationResult r = VerifyCAARecords(
+      server_cert_nss, verifier.rrdatas(), port);
   SECITEM_FreeItem(&dnssec_embedded_chain, PR_FALSE);
+
   return r;
 }
 
@@ -454,8 +426,7 @@ SSLClientSocketNSS::SSLClientSocketNSS(ClientSocketHandle* transport_socket,
                                        const HostPortPair& host_and_port,
                                        const SSLConfig& ssl_config,
                                        SSLHostInfo* ssl_host_info,
-                                       CertVerifier* cert_verifier,
-                                       DnsCertProvenanceChecker* dns_ctx)
+                                       const SSLClientSocketContext& context)
     : ALLOW_THIS_IN_INITIALIZER_LIST(buffer_send_callback_(
           this, &SSLClientSocketNSS::BufferSendComplete)),
       ALLOW_THIS_IN_INITIALIZER_LIST(buffer_recv_callback_(
@@ -477,19 +448,21 @@ SSLClientSocketNSS::SSLClientSocketNSS(ClientSocketHandle* transport_socket,
       server_cert_verify_result_(NULL),
       ssl_connection_status_(0),
       client_auth_cert_needed_(false),
-      cert_verifier_(cert_verifier),
+      cert_verifier_(context.cert_verifier),
+      ob_cert_xtn_negotiated_(false),
+      origin_bound_cert_service_(context.origin_bound_cert_service),
+      ob_cert_request_handle_(NULL),
       handshake_callback_called_(false),
       completed_handshake_(false),
       eset_mitm_detected_(false),
+      kaspersky_mitm_detected_(false),
       predicted_cert_chain_correct_(false),
-      peername_initialized_(false),
-      dnssec_provider_(NULL),
       next_handshake_state_(STATE_NONE),
       nss_fd_(NULL),
       nss_bufs_(NULL),
       net_log_(transport_socket->socket()->NetLog()),
       ssl_host_info_(ssl_host_info),
-      dns_cert_checker_(dns_ctx),
+      dns_cert_checker_(context.dns_cert_checker),
       valid_thread_id_(base::kInvalidThreadId) {
   EnterFunction("");
 }
@@ -508,12 +481,10 @@ void SSLClientSocketNSS::ClearSessionCache() {
 void SSLClientSocketNSS::GetSSLInfo(SSLInfo* ssl_info) {
   EnterFunction("");
   ssl_info->Reset();
-
-  if (!server_cert_)
+  if (!server_cert_nss_)
     return;
 
   ssl_info->cert_status = server_cert_verify_result_->cert_status;
-  DCHECK(server_cert_ != NULL);
   ssl_info->cert = server_cert_;
   ssl_info->connection_status = ssl_connection_status_;
   ssl_info->public_key_hashes = server_cert_verify_result_->public_key_hashes;
@@ -532,6 +503,17 @@ void SSLClientSocketNSS::GetSSLInfo(SSLInfo* ssl_info) {
     LOG(DFATAL) << "SSL_GetCipherSuiteInfo returned " << PR_GetError()
                 << " for cipherSuite " << cipher_suite;
   }
+
+  PRBool last_handshake_resumed;
+  ok = SSL_HandshakeResumedSession(nss_fd_, &last_handshake_resumed);
+  if (ok == SECSuccess) {
+    if (last_handshake_resumed) {
+      ssl_info->handshake_type = SSLInfo::HANDSHAKE_RESUME;
+    } else {
+      ssl_info->handshake_type = SSLInfo::HANDSHAKE_FULL;
+    }
+  }
+
   LeaveFunction("");
 }
 
@@ -544,6 +526,23 @@ void SSLClientSocketNSS::GetSSLCertRequestInfo(
   LeaveFunction(cert_request_info->client_certs.size());
 }
 
+int SSLClientSocketNSS::ExportKeyingMaterial(const base::StringPiece& label,
+                                             const base::StringPiece& context,
+                                             unsigned char *out,
+                                             unsigned int outlen) {
+  if (!IsConnected())
+    return ERR_SOCKET_NOT_CONNECTED;
+  SECStatus result = SSL_ExportKeyingMaterial(
+      nss_fd_, label.data(), label.size(),
+      reinterpret_cast<const unsigned char*>(context.data()),
+      context.length(), out, outlen);
+  if (result != SECSuccess) {
+    LogFailedNSSFunction(net_log_, "SSL_ExportKeyingMaterial", "");
+    return MapNSSError(PORT_GetError());
+  }
+  return OK;
+}
+
 SSLClientSocket::NextProtoStatus
 SSLClientSocketNSS::GetNextProto(std::string* proto) {
 #if defined(SSL_NEXT_PROTO_NEGOTIATED)
@@ -580,10 +579,6 @@ SSLClientSocketNSS::GetNextProto(std::string* proto) {
 #endif
 }
 
-void SSLClientSocketNSS::UseDNSSEC(DNSSECProvider* provider) {
-  dnssec_provider_ = provider;
-}
-
 int SSLClientSocketNSS::Connect(CompletionCallback* callback) {
   EnterFunction("");
   DCHECK(transport_.get());
@@ -610,17 +605,17 @@ int SSLClientSocketNSS::Connect(CompletionCallback* callback) {
     return rv;
   }
 
-  // Attempt to initialize the peer name.  In the case of TCP FastOpen,
-  // we don't have the peer yet.
-  if (!UsingTCPFastOpen()) {
-    rv = InitializeSSLPeerName();
-    if (rv != OK) {
-      net_log_.EndEventWithNetErrorCode(NetLog::TYPE_SSL_CONNECT, rv);
-      return rv;
-    }
+  rv = InitializeSSLPeerName();
+  if (rv != OK) {
+    net_log_.EndEventWithNetErrorCode(NetLog::TYPE_SSL_CONNECT, rv);
+    return rv;
   }
 
-  GotoState(STATE_HANDSHAKE);
+  if (ssl_config_.cached_info_enabled && ssl_host_info_.get()) {
+    GotoState(STATE_LOAD_SSL_HOST_INFO);
+  } else {
+    GotoState(STATE_HANDSHAKE);
+  }
 
   rv = DoHandshakeLoop(OK);
   if (rv == ERR_IO_PENDING) {
@@ -643,6 +638,11 @@ void SSLClientSocketNSS::Disconnect() {
   verifier_.reset();
   transport_->socket()->Disconnect();
 
+  if (ob_cert_request_handle_ != NULL) {
+    origin_bound_cert_service_->CancelRequest(ob_cert_request_handle_);
+    ob_cert_request_handle_ = NULL;
+  }
+
   // TODO(wtc): Send SSL close_notify alert.
   if (nss_fd_ != NULL) {
     PR_Close(nss_fd_);
@@ -669,12 +669,13 @@ void SSLClientSocketNSS::Disconnect() {
   ssl_connection_status_ = 0;
   completed_handshake_   = false;
   eset_mitm_detected_    = false;
+  kaspersky_mitm_detected_ = false;
   start_cert_verification_time_ = base::TimeTicks();
   predicted_cert_chain_correct_ = false;
-  peername_initialized_  = false;
   nss_bufs_              = NULL;
   client_certs_.clear();
   client_auth_cert_needed_ = false;
+  ob_cert_xtn_negotiated_ = false;
 
   LeaveFunction("");
 }
@@ -751,6 +752,22 @@ bool SSLClientSocketNSS::UsingTCPFastOpen() const {
   return false;
 }
 
+int64 SSLClientSocketNSS::NumBytesRead() const {
+  if (transport_.get() && transport_->socket()) {
+    return transport_->socket()->NumBytesRead();
+  }
+  NOTREACHED();
+  return -1;
+}
+
+base::TimeDelta SSLClientSocketNSS::GetConnectTimeMicros() const {
+  if (transport_.get() && transport_->socket()) {
+    return transport_->socket()->GetConnectTimeMicros();
+  }
+  NOTREACHED();
+  return base::TimeDelta::FromMicroseconds(-1);
+}
+
 int SSLClientSocketNSS::Read(IOBuffer* buf, int buf_len,
                              CompletionCallback* callback) {
   EnterFunction(buf_len);
@@ -821,10 +838,12 @@ int SSLClientSocketNSS::Init() {
   if (!NSS_IsInitialized())
     return ERR_UNEXPECTED;
 #if !defined(OS_MACOSX) && !defined(OS_WIN)
-  // We must call EnsureOCSPInit() here, on the IO thread, to get the IO loop
-  // by MessageLoopForIO::current().
-  // X509Certificate::Verify() runs on a worker thread of CertVerifier.
-  EnsureOCSPInit();
+  if (ssl_config_.rev_checking_enabled) {
+    // We must call EnsureOCSPInit() here, on the IO thread, to get the IO loop
+    // by MessageLoopForIO::current().
+    // X509Certificate::Verify() runs on a worker thread of CertVerifier.
+    EnsureOCSPInit();
+  }
 #endif
 
   LeaveFunction("");
@@ -951,11 +970,27 @@ int SSLClientSocketNSS::InitializeSSLOptions() {
 #ifdef SSL_ENABLE_OCSP_STAPLING
   if (IsOCSPStaplingSupported()) {
     rv = SSL_OptionSet(nss_fd_, SSL_ENABLE_OCSP_STAPLING, PR_TRUE);
-    if (rv != SECSuccess)
-      LogFailedNSSFunction(net_log_, "SSL_OptionSet (OCSP stapling)", "");
+    if (rv != SECSuccess) {
+      LogFailedNSSFunction(net_log_, "SSL_OptionSet",
+                           "SSL_ENABLE_OCSP_STAPLING");
+    }
   }
 #endif
 
+#ifdef SSL_ENABLE_CACHED_INFO
+  rv = SSL_OptionSet(nss_fd_, SSL_ENABLE_CACHED_INFO,
+                     ssl_config_.cached_info_enabled);
+  if (rv != SECSuccess)
+    LogFailedNSSFunction(net_log_, "SSL_OptionSet", "SSL_ENABLE_CACHED_INFO");
+#endif
+
+#ifdef SSL_ENABLE_OB_CERTS
+  rv = SSL_OptionSet(nss_fd_, SSL_ENABLE_OB_CERTS,
+                     ssl_config_.origin_bound_certs_enabled);
+  if (rv != SECSuccess)
+    LogFailedNSSFunction(net_log_, "SSL_OptionSet", "SSL_ENABLE_OB_CERTS");
+#endif
+
   rv = SSL_OptionSet(nss_fd_, SSL_HANDSHAKE_AS_CLIENT, PR_TRUE);
   if (rv != SECSuccess) {
     LogFailedNSSFunction(net_log_, "SSL_OptionSet", "SSL_HANDSHAKE_AS_CLIENT");
@@ -989,7 +1024,7 @@ int SSLClientSocketNSS::InitializeSSLOptions() {
   SSL_SetURL(nss_fd_, host_and_port_.host().c_str());
 
   // Tell SSL we're a client; needed if not letting NSPR do socket I/O
-  SSL_ResetHandshake(nss_fd_, 0);
+  SSL_ResetHandshake(nss_fd_, PR_FALSE);
 
   return OK;
 }
@@ -1026,24 +1061,23 @@ int SSLClientSocketNSS::InitializeSSLPeerName() {
   if (rv != SECSuccess)
     LogFailedNSSFunction(net_log_, "SSL_SetSockPeerID", peer_id.c_str());
 
-  peername_initialized_ = true;
   return OK;
 }
 
 
 // Sets server_cert_ and server_cert_nss_ if not yet set.
-// Returns server_cert_.
-X509Certificate *SSLClientSocketNSS::UpdateServerCert() {
+void SSLClientSocketNSS::UpdateServerCert() {
   // We set the server_cert_ from HandshakeCallback().
   if (server_cert_ == NULL) {
     server_cert_nss_ = SSL_PeerCertificate(nss_fd_);
     if (server_cert_nss_) {
       PeerCertificateChain certs(nss_fd_);
+      // This call may fail when SSL is used inside sandbox. In that
+      // case CreateFromDERCertChain() returns NULL.
       server_cert_ = X509Certificate::CreateFromDERCertChain(
           certs.AsStringPieceVector());
     }
   }
-  return server_cert_;
 }
 
 // Sets ssl_connection_status_.
@@ -1237,15 +1271,18 @@ int SSLClientSocketNSS::DoHandshakeLoop(int last_io_result) {
       case STATE_NONE:
         // we're just pumping data between the buffer and the network
         break;
+      case STATE_LOAD_SSL_HOST_INFO:
+        rv = DoLoadSSLHostInfo();
+        break;
       case STATE_HANDSHAKE:
         rv = DoHandshake();
         break;
+      case STATE_GET_OB_CERT_COMPLETE:
+        rv = DoGetOBCertComplete(rv);
+        break;
       case STATE_VERIFY_DNSSEC:
         rv = DoVerifyDNSSEC(rv);
         break;
-      case STATE_VERIFY_DNSSEC_COMPLETE:
-        rv = DoVerifyDNSSECComplete(rv);
-        break;
       case STATE_VERIFY_CERT:
         DCHECK(rv == OK);
         rv = DoVerifyCert(rv);
@@ -1321,31 +1358,93 @@ int SSLClientSocketNSS::DoWriteLoop(int result) {
   return rv;
 }
 
+bool SSLClientSocketNSS::LoadSSLHostInfo() {
+  const SSLHostInfo::State& state(ssl_host_info_->state());
+
+  if (state.certs.empty())
+    return true;
+
+  SECStatus rv;
+  const std::vector<std::string>& certs_in = state.certs;
+  scoped_array<CERTCertificate*> certs(new CERTCertificate*[certs_in.size()]);
+
+  for (size_t i = 0; i < certs_in.size(); i++) {
+    SECItem derCert;
+    derCert.data =
+      const_cast<uint8*>(reinterpret_cast<const uint8*>(certs_in[i].data()));
+    derCert.len = certs_in[i].size();
+    certs[i] = CERT_NewTempCertificate(
+        CERT_GetDefaultCertDB(), &derCert, NULL /* no nickname given */,
+        PR_FALSE /* not permanent */, PR_TRUE /* copy DER data */);
+    if (!certs[i]) {
+      DestroyCertificates(&certs[0], i);
+      NOTREACHED();
+      return false;
+    }
+  }
+
+  rv = SSL_SetPredictedPeerCertificates(nss_fd_, certs.get(), certs_in.size());
+  DestroyCertificates(&certs[0], certs_in.size());
+  DCHECK_EQ(SECSuccess, rv);
+
+  return true;
+}
+
+int SSLClientSocketNSS::DoLoadSSLHostInfo() {
+  EnterFunction("");
+  int rv = ssl_host_info_->WaitForDataReady(&handshake_io_callback_);
+  GotoState(STATE_HANDSHAKE);
+
+  if (rv == OK) {
+    if (!LoadSSLHostInfo())
+      LOG(WARNING) << "LoadSSLHostInfo failed: " << host_and_port_.ToString();
+  } else {
+    DCHECK_EQ(ERR_IO_PENDING, rv);
+    GotoState(STATE_LOAD_SSL_HOST_INFO);
+  }
+
+  LeaveFunction("");
+  return rv;
+}
+
 int SSLClientSocketNSS::DoHandshake() {
   EnterFunction("");
   int net_error = net::OK;
   SECStatus rv = SSL_ForceHandshake(nss_fd_);
 
+  // TODO(rkn): Handle the case in which origin-bound cert generation takes
+  // too long and the server has closed the connection. Report some new error
+  // code so that the higher level code will attempt to delete the socket and
+  // redo the handshake.
+
   if (client_auth_cert_needed_) {
-    net_error = ERR_SSL_CLIENT_AUTH_CERT_NEEDED;
-    net_log_.AddEvent(NetLog::TYPE_SSL_HANDSHAKE_ERROR,
-                      make_scoped_refptr(new SSLErrorParams(net_error, 0)));
-    // If the handshake already succeeded (because the server requests but
-    // doesn't require a client cert), we need to invalidate the SSL session
-    // so that we won't try to resume the non-client-authenticated session in
-    // the next handshake.  This will cause the server to ask for a client
-    // cert again.
-    if (rv == SECSuccess && SSL_InvalidateSession(nss_fd_) != SECSuccess) {
-      LOG(WARNING) << "Couldn't invalidate SSL session: " << PR_GetError();
+    if (ob_cert_xtn_negotiated_) {
+      GotoState(STATE_GET_OB_CERT_COMPLETE);
+      net_error = ERR_IO_PENDING;
+    } else {
+      net_error = ERR_SSL_CLIENT_AUTH_CERT_NEEDED;
+      net_log_.AddEvent(NetLog::TYPE_SSL_HANDSHAKE_ERROR,
+                        make_scoped_refptr(new SSLErrorParams(net_error, 0)));
+      // If the handshake already succeeded (because the server requests but
+      // doesn't require a client cert), we need to invalidate the SSL session
+      // so that we won't try to resume the non-client-authenticated session in
+      // the next handshake.  This will cause the server to ask for a client
+      // cert again.
+      if (rv == SECSuccess && SSL_InvalidateSession(nss_fd_) != SECSuccess) {
+        LOG(WARNING) << "Couldn't invalidate SSL session: " << PR_GetError();
+      }
     }
   } else if (rv == SECSuccess) {
     if (handshake_callback_called_) {
       if (eset_mitm_detected_) {
         net_error = ERR_ESET_ANTI_VIRUS_SSL_INTERCEPTION;
+      } else if (kaspersky_mitm_detected_) {
+        net_error = ERR_KASPERSKY_ANTI_VIRUS_SSL_INTERCEPTION;
       } else {
         // We need to see if the predicted certificate chain (in
         // |ssl_host_info_->state().certs) matches the actual certificate chain
-        // before we try to save it before we update |ssl_host_info_|.
+        // before we call SaveSSLHostInfo, as that will update
+        // |ssl_host_info_|.
         if (ssl_host_info_.get() && !ssl_host_info_->state().certs.empty()) {
           PeerCertificateChain certs(nss_fd_);
           const SSLHostInfo::State& state = ssl_host_info_->state();
@@ -1424,7 +1523,8 @@ int SSLClientSocketNSS::DoHandshake() {
     if (net_error == ERR_IO_PENDING) {
       GotoState(STATE_HANDSHAKE);
     } else {
-      LOG(ERROR) << "handshake failed; NSS error code " << prerr
+      LOG(ERROR) << "handshake with server " << host_and_port_.ToString()
+                 << " failed; NSS error code " << prerr
                  << ", net_error " << net_error;
       net_log_.AddEvent(
           NetLog::TYPE_SSL_HANDSHAKE_ERROR,
@@ -1436,6 +1536,79 @@ int SSLClientSocketNSS::DoHandshake() {
   return net_error;
 }
 
+#if defined(NSS_PLATFORM_CLIENT_AUTH)
+int SSLClientSocketNSS::ImportOBCertAndKey(CERTCertificate** cert,
+                                           SECKEYPrivateKey** key) {
+  NOTREACHED();
+  return ERR_NOT_IMPLEMENTED;
+}
+#else
+int SSLClientSocketNSS::ImportOBCertAndKey(CERTCertificate** cert,
+                                           SECKEYPrivateKey** key) {
+  // Set the certificate.
+  SECItem cert_item;
+  cert_item.data = (unsigned char*) ob_cert_.data();
+  cert_item.len = ob_cert_.size();
+  *cert = CERT_NewTempCertificate(CERT_GetDefaultCertDB(),
+                                  &cert_item,
+                                  NULL,
+                                  PR_FALSE,
+                                  PR_TRUE);
+  if (*cert == NULL)
+    return MapNSSError(PORT_GetError());
+
+  // Set the private key.
+  SECItem der_private_key_info;
+  der_private_key_info.data = (unsigned char*)ob_private_key_.data();
+  der_private_key_info.len = ob_private_key_.size();
+  const unsigned int key_usage = KU_DIGITAL_SIGNATURE;
+  crypto::ScopedPK11Slot slot(PK11_GetInternalSlot());
+  SECStatus rv = PK11_ImportDERPrivateKeyInfoAndReturnKey(
+      slot.get(), &der_private_key_info, NULL, NULL, PR_FALSE, PR_FALSE,
+      key_usage, key, NULL);
+
+  if (rv != SECSuccess) {
+    int error = MapNSSError(PORT_GetError());
+    CERT_DestroyCertificate(*cert);
+    *cert = NULL;
+    return error;
+  }
+
+  return OK;
+}
+#endif
+
+#if defined(NSS_PLATFORM_CLIENT_AUTH)
+int SSLClientSocketNSS::DoGetOBCertComplete(int result) {
+  NOTREACHED();
+  return ERR_NOT_IMPLEMENTED;
+}
+#else
+int SSLClientSocketNSS::DoGetOBCertComplete(int result) {
+  ob_cert_request_handle_ = NULL;
+
+  if (result != OK)
+    return result;
+
+  CERTCertificate* cert;
+  SECKEYPrivateKey* key;
+  int error = ImportOBCertAndKey(&cert, &key);
+  if (error != OK)
+    return error;
+
+  CERTCertificateList* cert_chain = CERT_CertChainFromCert(cert,
+                                                           certUsageSSLClient,
+                                                           PR_FALSE);
+  SECStatus rv;
+  rv = SSL_RestartHandshakeAfterCertReq(nss_fd_, cert, key, cert_chain);
+  if (rv != SECSuccess)
+    return MapNSSError(PORT_GetError());
+
+  GotoState(STATE_HANDSHAKE);
+  return OK;
+}
+#endif
+
 int SSLClientSocketNSS::DoVerifyDNSSEC(int result) {
   if (ssl_config_.dns_cert_provenance_checking_enabled &&
       dns_cert_checker_) {
@@ -1444,92 +1617,55 @@ int SSLClientSocketNSS::DoVerifyDNSSEC(int result) {
         host_and_port_.host(), certs.AsStringPieceVector());
   }
 
-  if (ssl_config_.dnssec_enabled) {
-    DNSValidationResult r = CheckDNSSECChain(host_and_port_.host(),
-                                             server_cert_nss_);
-    if (r == DNSVR_SUCCESS) {
-      local_server_cert_verify_result_.cert_status |= CERT_STATUS_IS_DNSSEC;
-      server_cert_verify_result_ = &local_server_cert_verify_result_;
-      GotoState(STATE_VERIFY_CERT_COMPLETE);
-      return OK;
-    }
-  }
-
-  if (dnssec_provider_ == NULL) {
-    GotoState(STATE_VERIFY_CERT);
+  DNSValidationResult r = CheckDNSSECChain(host_and_port_.host(),
+                                           server_cert_nss_,
+                                           host_and_port_.port());
+  if (r == DNSVR_SUCCESS) {
+    local_server_cert_verify_result_.cert_status |= CERT_STATUS_IS_DNSSEC;
+    local_server_cert_verify_result_.verified_cert = server_cert_;
+    server_cert_verify_result_ = &local_server_cert_verify_result_;
+    GotoState(STATE_VERIFY_CERT_COMPLETE);
     return OK;
   }
 
-  GotoState(STATE_VERIFY_DNSSEC_COMPLETE);
-  RRResponse* response;
-  dnssec_wait_start_time_ = base::Time::Now();
-  return dnssec_provider_->GetDNSSECRecords(&response, &handshake_io_callback_);
-}
-
-int SSLClientSocketNSS::DoVerifyDNSSECComplete(int result) {
-  RRResponse* response;
-  int err = dnssec_provider_->GetDNSSECRecords(&response, NULL);
-  DCHECK_EQ(err, OK);
-
-  const base::TimeDelta elapsed = base::Time::Now() - dnssec_wait_start_time_;
-  HISTOGRAM_TIMES("Net.DNSSECWaitTime", elapsed);
-
   GotoState(STATE_VERIFY_CERT);
-  if (!response || response->rrdatas.empty())
-    return OK;
-
-  std::vector<base::StringPiece> records;
-  records.resize(response->rrdatas.size());
-  for (unsigned i = 0; i < response->rrdatas.size(); i++)
-    records[i] = base::StringPiece(response->rrdatas[i]);
-  DNSValidationResult r =
-      VerifyTXTRecords(response->dnssec, server_cert_nss_, records);
-
-  if (!ssl_config_.dnssec_enabled) {
-    // If DNSSEC is not enabled we don't take any action based on the result,
-    // except to record the latency, above.
-    return OK;
-  }
-
-  switch (r) {
-    case DNSVR_FAILURE:
-      GotoState(STATE_VERIFY_CERT_COMPLETE);
-      local_server_cert_verify_result_.cert_status |= CERT_STATUS_NOT_IN_DNS;
-      server_cert_verify_result_ = &local_server_cert_verify_result_;
-      return ERR_CERT_NOT_IN_DNS;
-    case DNSVR_CONTINUE:
-      GotoState(STATE_VERIFY_CERT);
-      break;
-    case DNSVR_SUCCESS:
-      local_server_cert_verify_result_.cert_status |= CERT_STATUS_IS_DNSSEC;
-      server_cert_verify_result_ = &local_server_cert_verify_result_;
-      GotoState(STATE_VERIFY_CERT_COMPLETE);
-      break;
-    default:
-      NOTREACHED();
-      GotoState(STATE_VERIFY_CERT);
-  }
 
   return OK;
 }
 
 int SSLClientSocketNSS::DoVerifyCert(int result) {
-  DCHECK(server_cert_);
+  DCHECK(server_cert_nss_);
 
   GotoState(STATE_VERIFY_CERT_COMPLETE);
 
-  // If the certificate is expected to be bad we can use the expectation as the
-  // cert status.
+  // If the certificate is expected to be bad we can use the
+  // expectation as the cert status. Don't use |server_cert_| here
+  // because it can be set to NULL in case we failed to create
+  // X509Certificate in UpdateServerCert(). This may happen when this
+  // code is used inside sandbox.
+  base::StringPiece der_cert(
+      reinterpret_cast<char*>(server_cert_nss_->derCert.data),
+      server_cert_nss_->derCert.len);
   int cert_status;
-  if (ssl_config_.IsAllowedBadCert(server_cert_, &cert_status)) {
+  if (ssl_config_.IsAllowedBadCert(der_cert, &cert_status)) {
     DCHECK(start_cert_verification_time_.is_null());
     VLOG(1) << "Received an expected bad cert with status: " << cert_status;
     server_cert_verify_result_ = &local_server_cert_verify_result_;
     local_server_cert_verify_result_.Reset();
     local_server_cert_verify_result_.cert_status = cert_status;
+    local_server_cert_verify_result_.verified_cert = server_cert_;
     return OK;
   }
 
+  // We may have failed to create X509Certificate object if we are
+  // running inside sandbox.
+  if (!server_cert_) {
+    server_cert_verify_result_ = &local_server_cert_verify_result_;
+    local_server_cert_verify_result_.Reset();
+    local_server_cert_verify_result_.cert_status = CERT_STATUS_INVALID;
+    return ERR_CERT_INVALID;
+  }
+
   start_cert_verification_time_ = base::TimeTicks::Now();
 
   if (ssl_host_info_.get() && !ssl_host_info_->state().certs.empty() &&
@@ -1621,8 +1757,8 @@ int SSLClientSocketNSS::DoPayloadRead() {
     return rv;
   }
   if (rv >= 0) {
-    LogByteTransfer(net_log_, NetLog::TYPE_SSL_SOCKET_BYTES_RECEIVED, rv,
-                    user_read_buf_->data());
+    net_log_.AddByteTransferEvent(NetLog::TYPE_SSL_SOCKET_BYTES_RECEIVED, rv,
+                                  user_read_buf_->data());
     LeaveFunction("");
     return rv;
   }
@@ -1643,8 +1779,8 @@ int SSLClientSocketNSS::DoPayloadWrite() {
   DCHECK(user_write_buf_);
   int rv = PR_Write(nss_fd_, user_write_buf_->data(), user_write_buf_len_);
   if (rv >= 0) {
-    LogByteTransfer(net_log_, NetLog::TYPE_SSL_SOCKET_BYTES_SENT, rv,
-                    user_write_buf_->data());
+    net_log_.AddByteTransferEvent(NetLog::TYPE_SSL_SOCKET_BYTES_SENT, rv,
+                                  user_write_buf_->data());
     LeaveFunction("");
     return rv;
   }
@@ -1727,15 +1863,23 @@ void SSLClientSocketNSS::UncorkAfterTimeout() {
   } while (nsent > 0);
 }
 
-// Do network I/O between the given buffer and the given socket.
-// Return true if some I/O performed, false otherwise (error or ERR_IO_PENDING)
+// Do as much network I/O as possible between the buffer and the
+// transport socket. Return true if some I/O performed, false
+// otherwise (error or ERR_IO_PENDING).
 bool SSLClientSocketNSS::DoTransportIO() {
   EnterFunction("");
   bool network_moved = false;
   if (nss_bufs_ != NULL) {
-    int nsent = BufferSend();
-    int nreceived = BufferRecv();
-    network_moved = (nsent > 0 || nreceived >= 0);
+    int rv;
+    // Read and write as much data as we can. The loop is neccessary
+    // because Write() may return synchronously.
+    do {
+      rv = BufferSend();
+      if (rv > 0)
+        network_moved = true;
+    } while (rv > 0);
+    if (BufferRecv() >= 0)
+      network_moved = true;
   }
   LeaveFunction(network_moved);
   return network_moved;
@@ -1778,11 +1922,6 @@ int SSLClientSocketNSS::BufferSend(void) {
 
 void SSLClientSocketNSS::BufferSendComplete(int result) {
   EnterFunction(result);
-
-  // In the case of TCP FastOpen, connect is now finished.
-  if (!peername_initialized_ && UsingTCPFastOpen())
-    InitializeSSLPeerName();
-
   memio_PutWriteResult(nss_bufs_, MapErrorToNSS(result));
   transport_send_busy_ = false;
   OnSendComplete(result);
@@ -1881,39 +2020,42 @@ SECStatus SSLClientSocketNSS::OwnAuthCertHandler(void* arg,
   SECStatus rv = SSL_OptionGet(socket, SSL_ENABLE_FALSE_START, &false_start);
   DCHECK_EQ(SECSuccess, rv);
 
-  if (false_start) {
-    SSLClientSocketNSS* that = reinterpret_cast<SSLClientSocketNSS*>(arg);
-
-    // ESET anti-virus is capable of intercepting HTTPS connections on Windows.
-    // However, it is False Start intolerant and causes the connections to hang
-    // forever. We detect ESET by the issuer of the leaf certificate and set a
-    // flag to return a specific error, giving the user instructions for
-    // reconfiguring ESET.
-    CERTCertificate* cert = SSL_PeerCertificate(that->nss_fd_);
-    if (cert) {
-      char* common_name = CERT_GetCommonName(&cert->issuer);
-      if (common_name) {
-        if (strcmp(common_name, "ESET_RootSslCert") == 0)
-          that->eset_mitm_detected_ = true;
-        if (strcmp(common_name,
-                   "ContentWatch Root Certificate Authority") == 0) {
-          // This is NetNanny. NetNanny are updating their product so we
-          // silently disable False Start for now.
-          rv = SSL_OptionSet(socket, SSL_ENABLE_FALSE_START, PR_FALSE);
-          DCHECK_EQ(SECSuccess, rv);
-          false_start = 0;
-        }
-        PORT_Free(common_name);
+  SSLClientSocketNSS* that = reinterpret_cast<SSLClientSocketNSS*>(arg);
+  CERTCertificate* cert = SSL_PeerCertificate(that->nss_fd_);
+  if (cert) {
+    char* common_name = CERT_GetCommonName(&cert->issuer);
+    if (common_name) {
+      if (false_start && strcmp(common_name, "ESET_RootSslCert") == 0)
+        // ESET anti-virus is capable of intercepting HTTPS connections on
+        // Windows.  However, it is False Start intolerant and causes the
+        // connections to hang forever. We detect ESET by the issuer of the
+        // leaf certificate and set a flag to return a specific error, giving
+        // the user instructions for reconfiguring ESET.
+        that->eset_mitm_detected_ = true;
+      if (strcmp(common_name,
+                 "Kaspersky Anti-Virus personal root certificate") == 0) {
+        // Kaspersky has an unknown intolerance to our HTTPS handshakes and so
+        // we detect and give a more helpful error message.
+        that->kaspersky_mitm_detected_ = true;
       }
-      CERT_DestroyCertificate(cert);
+      if (false_start &&
+          strcmp(common_name, "ContentWatch Root Certificate Authority") == 0) {
+        // This is NetNanny. NetNanny are updating their product so we
+        // silently disable False Start for now.
+        rv = SSL_OptionSet(socket, SSL_ENABLE_FALSE_START, PR_FALSE);
+        DCHECK_EQ(SECSuccess, rv);
+        false_start = 0;
+      }
+      PORT_Free(common_name);
     }
+    CERT_DestroyCertificate(cert);
+  }
 
-    if (false_start && !that->handshake_callback_called_) {
-      that->corked_ = true;
-      that->uncork_timer_.Start(
-          base::TimeDelta::FromMilliseconds(kCorkTimeoutMs),
-          that, &SSLClientSocketNSS::UncorkAfterTimeout);
-    }
+  if (false_start && !that->handshake_callback_called_) {
+    that->corked_ = true;
+    that->uncork_timer_.Start(FROM_HERE,
+        base::TimeDelta::FromMilliseconds(kCorkTimeoutMs),
+        that, &SSLClientSocketNSS::UncorkAfterTimeout);
   }
 #endif
 
@@ -2066,8 +2208,7 @@ SECStatus SSLClientSocketNSS::PlatformClientAuthHandler(
     }
 
     scoped_refptr<X509Certificate> cert = X509Certificate::CreateFromHandle(
-        cert_context2, X509Certificate::SOURCE_LONE_CERT_IMPORT,
-        intermediates);
+        cert_context2, intermediates);
     that->client_certs_.push_back(cert);
 
     X509Certificate::FreeOSCertHandle(cert_context2);
@@ -2186,6 +2327,43 @@ SECStatus SSLClientSocketNSS::ClientAuthHandler(
     SECKEYPrivateKey** result_private_key) {
   SSLClientSocketNSS* that = reinterpret_cast<SSLClientSocketNSS*>(arg);
 
+  // Check if an origin-bound certificate is requested.
+  PRBool xtn_negotiated = PR_FALSE;
+  SECStatus rv = SSL_HandshakeNegotiatedExtension(
+      socket, ssl_ob_cert_xtn, &xtn_negotiated);
+  DCHECK_EQ(SECSuccess, rv);
+  that->ob_cert_xtn_negotiated_ = xtn_negotiated ? true : false;
+
+  if (that->ob_cert_xtn_negotiated_) {
+    // We have negotiated the origin-bound certificate extension.
+    std::string origin = "https://" + that->host_and_port_.ToString();
+    int error = that->origin_bound_cert_service_->GetOriginBoundCert(
+        origin,
+        &that->ob_private_key_,
+        &that->ob_cert_,
+        &that->handshake_io_callback_,
+        &that->ob_cert_request_handle_);
+
+    if (error == OK) {
+      // Synchronous success.
+      int result = that->ImportOBCertAndKey(result_certificate,
+                                            result_private_key);
+      if (result != OK)
+        return SECFailure;
+
+      return SECSuccess;
+    }
+
+    if (error == ERR_IO_PENDING) {
+      // Asynchronous case
+      that->client_auth_cert_needed_ = true;
+      return SECWouldBlock;
+    }
+
+    return SECFailure;  // Synchronous failure.
+  }
+
+  // Regular client certificate requested.
   that->client_auth_cert_needed_ = !that->ssl_config_.send_client_cert;
   void* wincx  = SSL_RevealPinArg(socket);
 
@@ -2230,8 +2408,7 @@ SECStatus SSLClientSocketNSS::ClientAuthHandler(
           NSS_CmpCertChainWCANames(node->cert, ca_names) != SECSuccess)
         continue;
       X509Certificate* x509_cert = X509Certificate::CreateFromHandle(
-          node->cert, X509Certificate::SOURCE_LONE_CERT_IMPORT,
-          net::X509Certificate::OSCertHandles());
+          node->cert, net::X509Certificate::OSCertHandles());
       that->client_certs_.push_back(x509_cert);
     }
     CERT_DestroyCertList(client_certs);
