@@ -31,7 +31,7 @@ static const char kUrl[] = "https://www.google.com/";
 static const char kOriginHost[] = "www.google.com";
 static const int kOriginPort = 443;
 static const char kOriginHostPort[] = "www.google.com:443";
-static const char kProxyUrl[] = "http://myproxy:6121/";
+static const char kProxyUrl[] = "https://myproxy:6121/";
 static const char kProxyHost[] = "myproxy";
 static const int kProxyPort = 6121;
 static const char kUserAgent[] = "Mozilla/1.0";
