@@ -25,6 +25,7 @@ import extension_specifics_pb2
 import nigori_specifics_pb2
 import password_specifics_pb2
 import preference_specifics_pb2
+import search_engine_specifics_pb2
 import session_specifics_pb2
 import sync_pb2
 import theme_specifics_pb2
@@ -44,9 +45,10 @@ ALL_TYPES = (
     NIGORI,
     PASSWORD,
     PREFERENCE,
+    SEARCH_ENGINE,
     SESSION,
     THEME,
-    TYPED_URL) = range(12)
+    TYPED_URL) = range(13)
 
 # Well-known server tag of the top level 'Google Chrome' folder.
 TOP_LEVEL_FOLDER_TAG = 'google_chrome'
@@ -62,6 +64,7 @@ SYNC_TYPE_TO_EXTENSION = {
     NIGORI: nigori_specifics_pb2.nigori,
     PASSWORD: password_specifics_pb2.password,
     PREFERENCE: preference_specifics_pb2.preference,
+    SEARCH_ENGINE: search_engine_specifics_pb2.search_engine,
     SESSION: session_specifics_pb2.session,
     THEME: theme_specifics_pb2.theme,
     TYPED_URL: typed_url_specifics_pb2.typed_url,
@@ -98,6 +101,10 @@ class StoreBirthdayError(Error):
   """The client sent a birthday that doesn't correspond to this server."""
 
 
+class TransientError(Error):
+  """The client would be sent a transient error."""
+
+
 def GetEntryType(entry):
   """Extract the sync type from a SyncEntry.
 
@@ -381,6 +388,8 @@ class SyncDataModel(object):
                     parent_tag='google_chrome', sync_type=EXTENSIONS),
       PermanentItem('google_chrome_passwords', name='Passwords',
                     parent_tag='google_chrome', sync_type=PASSWORD),
+      PermanentItem('google_chrome_search_engines', name='Search Engines',
+                    parent_tag='google_chrome', sync_type=SEARCH_ENGINE),
       PermanentItem('google_chrome_sessions', name='Sessions',
                     parent_tag='google_chrome', sync_type=SESSION),
       PermanentItem('google_chrome_themes', name='Themes',
@@ -402,8 +411,7 @@ class SyncDataModel(object):
     # SyncEntity protocol buffer.
     self._entries = {}
 
-    # TODO(nick): uuid.uuid1() is better, but python 2.5 only.
-    self.store_birthday = '%0.30f' % random.random()
+    self.ResetStoreBirthday()
 
     self.migration_history = MigrationHistory()
 
@@ -495,66 +503,26 @@ class SyncDataModel(object):
     migration_version_string, separator, inner_id = remainder.partition('^')
     return (int(datatype_string), int(migration_version_string), inner_id)
 
-  def _WritePosition(self, entry, parent_id, prev_id=None):
-    """Convert from a relative position into an absolute, numeric position.
+  def _WritePosition(self, entry, parent_id):
+    """Ensure the entry has an absolute, numeric position and parent_id.
 
-    Clients specify positions using the predecessor-based references; the
-    server stores and reports item positions using sparse integer values.
-    This method converts from the former to the latter.
+    Historically, clients would specify positions using the predecessor-based
+    references in the insert_after_item_id field; starting July 2011, this
+    was changed and Chrome now sends up the absolute position.  The server
+    must store a position_in_parent value and must not maintain
+    insert_after_item_id.
 
     Args:
-      entry: The entry for which to compute a position.  Its ID field are
+      entry: The entry for which to write a position.  Its ID field are
         assumed to be server IDs.  This entry will have its parent_id_string
         and position_in_parent fields updated; its insert_after_item_id field
         will be cleared.
       parent_id: The ID of the entry intended as the new parent.
-      prev_id: The ID of the entry intended as the new predecessor.  If this
-        is None, or an ID of an object which is not a child of the new parent,
-        the entry will be positioned at the end (right) of the ordering.  If
-        the empty ID (''), this will be positioned at the front (left) of the
-        ordering.  Otherwise, the entry will be given a position_in_parent
-        value placing it just after (to the right of) the new predecessor.
     """
-    preferred_gap = 2 ** 20
-
-    def ExtendRange(current_limit_entry, sign_multiplier):
-      """Compute values at the beginning or end."""
-      if current_limit_entry.id_string == entry.id_string:
-        step = 0
-      else:
-        step = sign_multiplier * preferred_gap
-      return current_limit_entry.position_in_parent + step
-
-    siblings = [x for x in self._entries.values()
-                if x.parent_id_string == parent_id and not x.deleted]
-    siblings = sorted(siblings, key=operator.attrgetter('position_in_parent'))
-    if prev_id == entry.id_string:
-      prev_id = ''
-    if not siblings:
-      # First item in this container; start in the middle.
-      entry.position_in_parent = 0
-    elif not prev_id:
-      # A special value in the protocol.  Insert at first position.
-      entry.position_in_parent = ExtendRange(siblings[0], -1)
-    else:
-      # Handle mid-insertion; consider items along with their successors.
-      for item, successor in zip(siblings, siblings[1:]):
-        if item.id_string != prev_id:
-          continue
-        elif successor.id_string == entry.id_string:
-          # We're already in place; don't change anything.
-          entry.position_in_parent = successor.position_in_parent
-        else:
-          # Interpolate new position between the previous item and its
-          # existing successor.
-          entry.position_in_parent = (item.position_in_parent * 7 +
-                                      successor.position_in_parent) / 8
-        break
-      else:
-        # Insert at end. Includes the case where prev_id is None.
-        entry.position_in_parent = ExtendRange(siblings[-1], +1)
 
     entry.parent_id_string = parent_id
+    if not entry.HasField('position_in_parent'):
+      entry.position_in_parent = 1337  # A debuggable, distinctive default.
     entry.ClearField('insert_after_item_id')
 
   def _ItemExists(self, id_string):
@@ -595,6 +563,15 @@ class SyncDataModel(object):
       if spec.sync_type in requested_types:
         self._CreatePermanentItem(spec)
 
+  def ResetStoreBirthday(self):
+    """Resets the store birthday to a random value."""
+    # TODO(nick): uuid.uuid1() is better, but python 2.5 only.
+    self.store_birthday = '%0.30f' % random.random()
+
+  def StoreBirthday(self):
+    """Gets the store birthday."""
+    return self.store_birthday
+
   def GetChanges(self, sieve):
     """Get entries which have changed, oldest first.
 
@@ -842,19 +819,13 @@ class SyncDataModel(object):
       entry = MakeTombstone(entry.id_string)
     else:
       # Comments in sync.proto detail how the representation of positional
-      # ordering works: the 'insert_after_item_id' field specifies a
-      # predecessor during Commit operations, but the 'position_in_parent'
-      # field provides an absolute ordering in GetUpdates contexts.  Here
-      # we convert from the former to the latter.  Specifically, we'll
-      # generate a numeric position placing the item just after the object
-      # identified by 'insert_after_item_id', and then clear the
-      # 'insert_after_item_id' field so that it's not sent back to the client
-      # during later GetUpdates requests.
-      if entry.HasField('insert_after_item_id'):
-        self._WritePosition(entry, entry.parent_id_string,
-                            entry.insert_after_item_id)
-      else:
-        self._WritePosition(entry, entry.parent_id_string)
+      # ordering works: either the 'insert_after_item_id' field or the
+      # 'position_in_parent' field may determine the sibling order during
+      # Commit operations.  The 'position_in_parent' field provides an absolute
+      # ordering in GetUpdates contexts.  Here we assume the client will
+      # always send a valid position_in_parent (this is the newer style), and
+      # we ignore insert_after_item_id (an older style).
+      self._WritePosition(entry, entry.parent_id_string)
 
     # Preserve the originator info, which the client is not required to send
     # when updating.
@@ -892,6 +863,23 @@ class SyncDataModel(object):
             entry.parent_id_string)
       self._entries[entry.id_string] = entry
 
+  def TriggerSyncTabs(self):
+    """Set the 'sync_tabs' field to this account's nigori node.
+
+    If the field is not currently set, will write a new nigori node entry
+    with the field set. Else does nothing.
+    """
+
+    nigori_tag = "google_chrome_nigori"
+    nigori_original = self._entries.get(self._ServerTagToId(nigori_tag))
+    if (nigori_original.specifics.Extensions[nigori_specifics_pb2.nigori].
+        sync_tabs):
+      return
+    nigori_new = copy.deepcopy(nigori_original)
+    nigori_new.specifics.Extensions[nigori_specifics_pb2.nigori].sync_tabs = (
+        True)
+    self._SaveEntry(nigori_new)
+
 
 class TestServer(object):
   """An object to handle requests for one (and only one) Chrome Sync account.
@@ -910,6 +898,7 @@ class TestServer(object):
     self.clients = {}
     self.client_name_generator = ('+' * times + chr(c)
         for times in xrange(0, sys.maxint) for c in xrange(ord('A'), ord('Z')))
+    self.transient_error = False
 
   def GetShortClientName(self, query):
     parsed = cgi.parse_qs(query[query.find('?')+1:])
@@ -925,9 +914,14 @@ class TestServer(object):
     """Raises StoreBirthdayError if the request's birthday is a mismatch."""
     if not request.HasField('store_birthday'):
       return
-    if self.account.store_birthday != request.store_birthday:
+    if self.account.StoreBirthday() != request.store_birthday:
       raise StoreBirthdayError
 
+  def CheckTransientError(self):
+    """Raises TransientError if transient_error variable is set."""
+    if self.transient_error:
+      raise TransientError
+
   def HandleMigrate(self, path):
     query = urlparse.urlparse(path)[4]
     code = 200
@@ -950,6 +944,25 @@ class TestServer(object):
     return (code, '<html><title>Migration: %d</title><H1>%d %s</H1></html>' %
                 (code, code, response))
 
+  def HandleCreateBirthdayError(self):
+    self.account.ResetStoreBirthday()
+    return (
+        200,
+        '<html><title>Birthday error</title><H1>Birthday error</H1></html>')
+
+  def HandleSetTransientError(self):
+    self.transient_error = True
+    return (
+        200,
+        '<html><title>Transient error</title><H1>Transient error</H1></html>')
+
+  def HandleSetSyncTabs(self):
+    """Set the 'sync_tab' field of the nigori node for this account."""
+    self.account.TriggerSyncTabs()
+    return (
+        200,
+        '<html><title>Sync Tabs</title><H1>Sync Tabs</H1></html>')
+
   def HandleCommand(self, query, raw_request):
     """Decode and handle a sync command from a raw input of bytes.
 
@@ -978,6 +991,7 @@ class TestServer(object):
       response.error_code = sync_pb2.ClientToServerResponse.SUCCESS
       self.CheckStoreBirthday(request)
       response.store_birthday = self.account.store_birthday
+      self.CheckTransientError();
 
       print_context('->')
 
@@ -999,7 +1013,7 @@ class TestServer(object):
         print 'Unrecognizable sync request!'
         return (400, None)  # Bad request.
       return (200, response.SerializeToString())
-    except MigrationDoneError as error:
+    except MigrationDoneError, error:
       print_context('<-')
       print 'MIGRATION_DONE: <%s>' % (ShortDatatypeListSummary(error.datatypes))
       response = sync_pb2.ClientToServerResponse()
@@ -1008,13 +1022,19 @@ class TestServer(object):
       response.migrated_data_type_id[:] = [
           SyncTypeToProtocolDataTypeId(x) for x in error.datatypes]
       return (200, response.SerializeToString())
-    except StoreBirthdayError as error:
+    except StoreBirthdayError, error:
       print_context('<-')
       print 'NOT_MY_BIRTHDAY'
       response = sync_pb2.ClientToServerResponse()
       response.store_birthday = self.account.store_birthday
       response.error_code = sync_pb2.ClientToServerResponse.NOT_MY_BIRTHDAY
       return (200, response.SerializeToString())
+    except TransientError as error:
+      print_context('<-')
+      print 'TRANSIENT_ERROR'
+      response.store_birthday = self.account.store_birthday
+      response.error_code = sync_pb2.ClientToServerResponse.TRANSIENT_ERROR
+      return (200, response.SerializeToString())
     finally:
       self.account_lock.release()
 
