@@ -15,28 +15,33 @@ The format of the file is JSON. The root dictionary contains a list under the
 key "managed_users". It contains auth tokens for which the server will claim
 that the user is managed. The token string "*" indicates that all users are
 claimed to be managed. Other keys in the root dictionary identify request
-scopes. Each request scope is described by a dictionary that holds two
+scopes. The user-request scope is described by a dictionary that holds two
 sub-dictionaries: "mandatory" and "recommended". Both these hold the policy
 definitions as key/value stores, their format is identical to what the Linux
 implementation reads from /etc.
+The device-scope holds the policy-definition directly as key/value stores in the
+protobuf-format.
 
 Example:
 
 {
-  "chromeos/device": {
-    "mandatory": {
-      "HomepageLocation" : "http://www.chromium.org"
-    },
-    "recommended": {
-      "JavascriptEnabled": false,
-    },
+  "google/chromeos/device" : {
+      "guest_mode_enabled" : false
   },
-  "managed_users": [
+  "google/chromeos/user" : {
+     "mandatory" : {
+      "HomepageLocation" : "http://www.chromium.org",
+      "IncognitoEnabled" : false
+    },
+     "recommended" : {
+      "JavascriptEnabled": false
+    }
+  },
+  "managed_users" : [
     "secret123456"
   ]
 }
 
-
 """
 
 import cgi
@@ -119,8 +124,10 @@ class RequestHandler(object):
     rmsg = dm.DeviceManagementRequest()
     rmsg.ParseFromString(self._request)
 
-    logging.debug('auth -> ' + self._headers.getheader('Authorization', ''))
-    logging.debug('deviceid -> ' + self.GetUniqueParam('deviceid'))
+    logging.debug('gaia auth token -> ' +
+                  self._headers.getheader('Authorization', ''))
+    logging.debug('oauth token -> ' + str(self.GetUniqueParam('oauth_token')))
+    logging.debug('deviceid -> ' + str(self.GetUniqueParam('deviceid')))
     self.DumpMessage('Request', rmsg)
 
     request_type = self.GetUniqueParam('request')
@@ -142,14 +149,21 @@ class RequestHandler(object):
       return (400, 'Invalid request parameter')
 
   def CheckGoogleLogin(self):
-    """Extracts the GoogleLogin auth token from the HTTP request, and
-    returns it. Returns None if the token is not present.
+    """Extracts the auth token from the request and returns it. The token may
+    either be a GoogleLogin token from an Authorization header, or an OAuth V2
+    token from the oauth_token query parameter. Returns None if no token is
+    present.
     """
+    oauth_token = self.GetUniqueParam('oauth_token')
+    if oauth_token:
+      return oauth_token
+
     match = re.match('GoogleLogin auth=(\\w+)',
                      self._headers.getheader('Authorization', ''))
-    if not match:
-      return None
-    return match.group(1)
+    if match:
+      return match.group(1)
+
+    return None
 
   def ProcessRegister(self, msg):
     """Handles a register request.
@@ -164,9 +178,15 @@ class RequestHandler(object):
       A tuple of HTTP status code and response data to send to the client.
     """
     # Check the auth token and device ID.
-    if not self.CheckGoogleLogin():
+    auth = self.CheckGoogleLogin()
+    if not auth:
       return (403, 'No authorization')
 
+    policy = self._server.GetPolicies()
+    if ('*' not in policy['managed_users'] and
+        auth not in policy['managed_users']):
+      return (403, 'Unmanaged')
+
     device_id = self.GetUniqueParam('deviceid')
     if not device_id:
       return (400, 'Missing device identifier')
@@ -213,100 +233,6 @@ class RequestHandler(object):
 
     return (200, response.SerializeToString())
 
-  def ProcessInitialPolicy(self, msg):
-    """Handles a 'preregister policy' request.
-
-    Queries the list of managed users and responds the client if their user
-    is managed or not.
-
-    Args:
-      msg: The PolicyFetchRequest message received from the client.
-
-    Returns:
-      A tuple of HTTP status code and response data to send to the client.
-    """
-    # Check the GAIA token.
-    auth = self.CheckGoogleLogin()
-    if not auth:
-      return (403, 'No authorization')
-
-    chrome_initial_settings = dm.ChromeInitialSettingsProto()
-    if ('*' in self._server.policy['managed_users'] or
-        auth in self._server.policy['managed_users']):
-      chrome_initial_settings.enrollment_provision = (
-          dm.ChromeInitialSettingsProto.MANAGED);
-    else:
-      chrome_initial_settings.enrollment_provision = (
-          dm.ChromeInitialSettingsProto.UNMANAGED);
-
-    policy_data = dm.PolicyData()
-    policy_data.policy_type = msg.policy_type
-    policy_data.policy_value = chrome_initial_settings.SerializeToString()
-
-    # Prepare and send the response.
-    response = dm.DeviceManagementResponse()
-    fetch_response = response.policy_response.response.add()
-    fetch_response.policy_data = (
-        policy_data.SerializeToString())
-
-    self.DumpMessage('Response', response)
-
-    return (200, response.SerializeToString())
-
-  def ProcessDevicePolicy(self, msg):
-    """Handles a policy request that uses the deprecated protcol.
-    TODO(gfeher): Remove this when we certainly don't need it.
-
-    Checks for authorization, encodes the policy into protobuf representation
-    and constructs the response.
-
-    Args:
-      msg: The DevicePolicyRequest message received from the client.
-
-    Returns:
-      A tuple of HTTP status code and response data to send to the client.
-    """
-
-    # Check the management token.
-    token, response = self.CheckToken()
-    if not token:
-      return response
-
-    # Stuff the policy dictionary into a response message and send it back.
-    response = dm.DeviceManagementResponse()
-    response.policy_response.CopyFrom(dm.DevicePolicyResponse())
-
-    # Respond only if the client requested policy for the cros/device scope,
-    # since that's where chrome policy is supposed to live in.
-    if msg.policy_scope == 'chromeos/device':
-      policy = self._server.policy['google/chromeos/user']['mandatory']
-      setting = response.policy_response.setting.add()
-      setting.policy_key = 'chrome-policy'
-      policy_value = dm.GenericSetting()
-      for (key, value) in policy.iteritems():
-        entry = policy_value.named_value.add()
-        entry.name = key
-        entry_value = dm.GenericValue()
-        if isinstance(value, bool):
-          entry_value.value_type = dm.GenericValue.VALUE_TYPE_BOOL
-          entry_value.bool_value = value
-        elif isinstance(value, int):
-          entry_value.value_type = dm.GenericValue.VALUE_TYPE_INT64
-          entry_value.int64_value = value
-        elif isinstance(value, str) or isinstance(value, unicode):
-          entry_value.value_type = dm.GenericValue.VALUE_TYPE_STRING
-          entry_value.string_value = value
-        elif isinstance(value, list):
-          entry_value.value_type = dm.GenericValue.VALUE_TYPE_STRING_ARRAY
-          for list_entry in value:
-            entry_value.string_array.append(str(list_entry))
-        entry.value.CopyFrom(entry_value)
-      setting.policy_value.CopyFrom(policy_value)
-
-    self.DumpMessage('Response', response)
-
-    return (200, response.SerializeToString())
-
   def ProcessPolicy(self, msg, request_type):
     """Handles a policy request.
 
@@ -319,22 +245,15 @@ class RequestHandler(object):
     Returns:
       A tuple of HTTP status code and response data to send to the client.
     """
-
-    if msg.request:
-      for request in msg.request:
-        if request.policy_type == 'google/chromeos/unregistered_user':
-          if request_type != 'ping':
-            return (400, 'Invalid request type')
-          return self.ProcessInitialPolicy(request)
-        elif (request.policy_type in
-              ('google/chromeos/user', 'google/chromeos/device')):
-          if request_type != 'policy':
-            return (400, 'Invalid request type')
-          return self.ProcessCloudPolicy(request)
+    for request in msg.request:
+      if (request.policy_type in
+             ('google/chromeos/user', 'google/chromeos/device')):
+        if request_type != 'policy':
+          return (400, 'Invalid request type')
         else:
-          return (400, 'Invalid policy_type')
-    else:
-      return self.ProcessDevicePolicy(msg)
+          return self.ProcessCloudPolicy(request)
+      else:
+        return (400, 'Invalid policy_type')
 
   def SetProtobufMessageField(self, group_message, field, field_value):
     '''Sets a field in a protobuf message.
@@ -444,18 +363,19 @@ class RequestHandler(object):
     # Response is only given if the scope is specified in the config file.
     # Normally 'google/chromeos/device' and 'google/chromeos/user' should be
     # accepted.
+    policy = self._server.GetPolicies()
     policy_value = ''
     if (msg.policy_type in token_info['allowed_policy_types'] and
-        msg.policy_type in self._server.policy):
+        msg.policy_type in policy):
       if msg.policy_type == 'google/chromeos/user':
         settings = cp.CloudPolicySettings()
         self.GatherUserPolicySettings(settings,
-                                      self._server.policy[msg.policy_type])
+                                      policy[msg.policy_type])
         policy_value = settings.SerializeToString()
       elif msg.policy_type == 'google/chromeos/device':
         settings = dp.ChromeDeviceSettingsProto()
         self.GatherDevicePolicySettings(settings,
-                                        self._server.policy[msg.policy_type])
+                                        policy[msg.policy_type])
         policy_value = settings.SerializeToString()
 
     # Figure out the key we want to use. If multiple keys are configured, the
@@ -526,7 +446,7 @@ class RequestHandler(object):
       if (not token_info or
           not request_device_id or
           token_info['device_id'] != request_device_id):
-        error = 901
+        error = 410
       else:
         return (token_info, None)
 
@@ -549,7 +469,7 @@ class TestServer(object):
       private_key_paths: List of paths to read private keys from.
     """
     self._registered_tokens = {}
-    self.policy = {}
+    self.policy_path = policy_path
 
     # There is no way to for the testserver to know the user name belonging to
     # the GAIA auth token we received (short of actually talking to GAIA). To
@@ -557,14 +477,6 @@ class TestServer(object):
     # the server should report to the client.
     self.username = policy_user
 
-    if json is None:
-      print 'No JSON module, cannot parse policy information'
-    else :
-      try:
-        self.policy = json.loads(open(policy_path).read())
-      except IOError:
-        print 'Failed to load policy from %s' % policy_path
-
     self.keys = []
     if private_key_paths:
       # Load specified keys from the filesystem.
@@ -595,6 +507,20 @@ class TestServer(object):
       pubkey = asn1der.Sequence([ algorithm, asn1der.Bitstring(rsa_pubkey) ])
       entry['public_key'] = pubkey;
 
+  def GetPolicies(self):
+    """Returns the policies to be used, reloaded form the backend file every
+       time this is called.
+    """
+    policy = {}
+    if json is None:
+      print 'No JSON module, cannot parse policy information'
+    else :
+      try:
+        policy = json.loads(open(self.policy_path).read())
+      except IOError:
+        print 'Failed to load policy from %s' % self.policy_path
+    return policy
+
   def HandleRequest(self, path, headers, request):
     """Handles a request.
 
