@@ -6,16 +6,23 @@
 #define NET_UDP_DATAGRAM_SOCKET_H_
 #pragma once
 
-#include "net/base/net_api.h"
+#include "net/base/net_export.h"
 
 namespace net {
 
+class BoundNetLog;
 class IPEndPoint;
 
 // A datagram socket is an interface to a protocol which exchanges
 // datagrams, like UDP.
-class NET_TEST DatagramSocket {
+class NET_EXPORT_PRIVATE DatagramSocket {
  public:
+  // Type of source port binding to use.
+  enum BindType {
+    RANDOM_BIND,
+    DEFAULT_BIND,
+  };
+
   virtual ~DatagramSocket() {}
 
   // Close the socket.
@@ -27,6 +34,9 @@ class NET_TEST DatagramSocket {
   // Copy the local udp address into |address| and return a network error code.
   // (similar to getsockname)
   virtual int GetLocalAddress(IPEndPoint* address) const = 0;
+
+  // Gets the NetLog for this socket.
+  virtual const BoundNetLog& NetLog() const = 0;
 };
 
 }  // namespace net
