@@ -7,6 +7,7 @@
 #pragma once
 
 #include "net/base/net_log.h"
+#include "net/base/rand_callback.h"
 #include "net/udp/datagram_client_socket.h"
 #include "net/udp/udp_socket.h"
 
@@ -15,9 +16,11 @@ namespace net {
 class BoundNetLog;
 
 // A client socket that uses UDP as the transport layer.
-class NET_TEST UDPClientSocket : public DatagramClientSocket {
+class NET_EXPORT_PRIVATE UDPClientSocket : public DatagramClientSocket {
  public:
-  UDPClientSocket(net::NetLog* net_log,
+  UDPClientSocket(DatagramSocket::BindType bind_type,
+                  const RandIntCallback& rand_int_cb,
+                  net::NetLog* net_log,
                   const net::NetLog::Source& source);
   virtual ~UDPClientSocket();
 
@@ -30,6 +33,7 @@ class NET_TEST UDPClientSocket : public DatagramClientSocket {
   virtual int GetLocalAddress(IPEndPoint* address) const;
   virtual bool SetReceiveBufferSize(int32 size);
   virtual bool SetSendBufferSize(int32 size);
+  virtual const BoundNetLog& NetLog() const;
 
  private:
   UDPSocket socket_;
