@@ -16,7 +16,7 @@ class IPEndPoint;
 class BoundNetLog;
 
 // A client socket that uses UDP as the transport layer.
-class NET_API UDPServerSocket : public DatagramServerSocket {
+class NET_EXPORT UDPServerSocket : public DatagramServerSocket {
  public:
   UDPServerSocket(net::NetLog* net_log,
                   const net::NetLog::Source& source);
@@ -35,6 +35,7 @@ class NET_API UDPServerSocket : public DatagramServerSocket {
   virtual void Close();
   virtual int GetPeerAddress(IPEndPoint* address) const;
   virtual int GetLocalAddress(IPEndPoint* address) const;
+  virtual const BoundNetLog& NetLog() const;
 
  private:
   UDPSocket socket_;
