@@ -46,6 +46,7 @@ void StripPostSpecificHeaders(HttpRequestHeaders* headers) {
   headers->RemoveHeader(HttpRequestHeaders::kOrigin);
 }
 
+// TODO(battre): Delete this, see http://crbug.com/89321:
 // This counter keeps track of the identifiers used for URL requests so far.
 // 0 is reserved to represent an invalid ID.
 uint64 g_next_url_request_identifier = 1;
@@ -61,6 +62,22 @@ uint64 GenerateURLRequestIdentifier() {
 
 }  // namespace
 
+URLRequest::ProtocolFactory*
+URLRequest::Deprecated::RegisterProtocolFactory(const std::string& scheme,
+                                                ProtocolFactory* factory) {
+  return URLRequest::RegisterProtocolFactory(scheme, factory);
+}
+
+void URLRequest::Deprecated::RegisterRequestInterceptor(
+    Interceptor* interceptor) {
+  URLRequest::RegisterRequestInterceptor(interceptor);
+}
+
+void URLRequest::Deprecated::UnregisterRequestInterceptor(
+    Interceptor* interceptor) {
+  URLRequest::UnregisterRequestInterceptor(interceptor);
+}
+
 ///////////////////////////////////////////////////////////////////////////////
 // URLRequest::Interceptor
 
@@ -100,13 +117,14 @@ void URLRequest::Delegate::OnSSLCertificateError(URLRequest* request,
   request->Cancel();
 }
 
-bool URLRequest::Delegate::CanGetCookies(URLRequest* request) {
+bool URLRequest::Delegate::CanGetCookies(const URLRequest* request,
+                                         const CookieList& cookie_list) const {
   return true;
 }
 
-bool URLRequest::Delegate::CanSetCookie(URLRequest* request,
+bool URLRequest::Delegate::CanSetCookie(const URLRequest* request,
                                         const std::string& cookie_line,
-                                        CookieOptions* options) {
+                                        CookieOptions* options) const {
   return true;
 }
 
@@ -123,8 +141,10 @@ URLRequest::URLRequest(const GURL& url, Delegate* delegate)
       final_upload_progress_(0),
       priority_(LOWEST),
       identifier_(GenerateURLRequestIdentifier()),
+      blocked_on_delegate_(false),
       ALLOW_THIS_IN_INITIALIZER_LIST(
-          before_request_callback_(this, &URLRequest::BeforeRequestComplete)) {
+          before_request_callback_(this, &URLRequest::BeforeRequestComplete)),
+      has_notified_completion_(false) {
   SIMPLE_STATS_COUNTER("URLRequestCount");
 
   // Sanity check out environment.
@@ -135,11 +155,11 @@ URLRequest::URLRequest(const GURL& url, Delegate* delegate)
 }
 
 URLRequest::~URLRequest() {
+  Cancel();
+
   if (context_ && context_->network_delegate())
     context_->network_delegate()->NotifyURLRequestDestroyed(this);
 
-  Cancel();
-
   if (job_)
     OrphanJob();
 
@@ -235,8 +255,13 @@ void URLRequest::SetExtraRequestHeaders(
   // for request headers are implemented.
 }
 
-LoadState URLRequest::GetLoadState() const {
-  return job_ ? job_->GetLoadState() : LOAD_STATE_IDLE;
+LoadStateWithParam URLRequest::GetLoadState() const {
+  if (blocked_on_delegate_) {
+    return LoadStateWithParam(LOAD_STATE_WAITING_FOR_DELEGATE,
+                              load_state_param_);
+  }
+  return LoadStateWithParam(job_ ? job_->GetLoadState() : LOAD_STATE_IDLE,
+                            string16());
 }
 
 uint64 URLRequest::GetUploadProgress() const {
@@ -369,16 +394,26 @@ GURL URLRequest::GetSanitizedReferrer() const {
   return ret;
 }
 
+void URLRequest::set_delegate(Delegate* delegate) {
+  delegate_ = delegate;
+}
+
 void URLRequest::Start() {
   response_info_.request_time = Time::Now();
 
   // Only notify the delegate for the initial request.
   if (context_ && context_->network_delegate()) {
-    if (context_->network_delegate()->NotifyBeforeURLRequest(
-            this, &before_request_callback_, &delegate_redirect_url_) ==
-            net::ERR_IO_PENDING) {
-      net_log_.BeginEvent(NetLog::TYPE_URL_REQUEST_BLOCKED_ON_EXTENSION, NULL);
-      return;  // paused
+    int error = context_->network_delegate()->NotifyBeforeURLRequest(
+        this, &before_request_callback_, &delegate_redirect_url_);
+    if (error != net::OK) {
+      if (error == net::ERR_IO_PENDING) {
+        // Paused on the delegate, will invoke |before_request_callback_| later.
+        SetBlockedOnDelegate();
+      } else {
+        // The delegate immediately returned some error code.
+        BeforeRequestComplete(error);
+      }
+      return;
     }
   }
 
@@ -391,8 +426,11 @@ void URLRequest::BeforeRequestComplete(int error) {
   DCHECK(!job_);
   DCHECK_NE(ERR_IO_PENDING, error);
 
-  net_log_.EndEvent(NetLog::TYPE_URL_REQUEST_BLOCKED_ON_EXTENSION, NULL);
+  if (blocked_on_delegate_)
+    SetUnblockedOnDelegate();
   if (error != OK) {
+    net_log_.AddEvent(NetLog::TYPE_CANCELLED,
+        make_scoped_refptr(new NetLogStringParameter("source", "delegate")));
     StartJob(new URLRequestErrorJob(this, error));
   } else if (!delegate_redirect_url_.is_empty()) {
     GURL new_url;
@@ -475,6 +513,11 @@ void URLRequest::DoCancel(int os_error, const SSLInfo& ssl_info) {
 
   job_->Kill();
 
+  // We need to notify about the end of this job here synchronously. The
+  // Job sends an asynchronous notification but by the time this is processed,
+  // our |context_| is NULL.
+  NotifyRequestCompleted();
+
   // The Job will call our NotifyDone method asynchronously.  This is done so
   // that the Delegate implementation can call Cancel without having to worry
   // about being called recursively.
@@ -497,7 +540,12 @@ bool URLRequest::Read(IOBuffer* dest, int dest_size, int* bytes_read) {
     return true;
   }
 
-  return job_->Read(dest, dest_size, bytes_read);
+  bool rv = job_->Read(dest, dest_size, bytes_read);
+  // If rv is false, the status cannot be success.
+  DCHECK(rv || status_.status() != URLRequestStatus::SUCCESS);
+  if (rv && *bytes_read <= 0 && status_.is_success())
+    NotifyRequestCompleted();
+  return rv;
 }
 
 void URLRequest::StopCaching() {
@@ -505,7 +553,8 @@ void URLRequest::StopCaching() {
   job_->StopCaching();
 }
 
-void URLRequest::ReceivedRedirect(const GURL& location, bool* defer_redirect) {
+void URLRequest::NotifyReceivedRedirect(const GURL& location,
+                                        bool* defer_redirect) {
   URLRequestJob* job =
       URLRequestJobManager::GetInstance()->MaybeInterceptRedirect(this,
                                                                   location);
@@ -516,7 +565,7 @@ void URLRequest::ReceivedRedirect(const GURL& location, bool* defer_redirect) {
   }
 }
 
-void URLRequest::ResponseStarted() {
+void URLRequest::NotifyResponseStarted() {
   scoped_refptr<NetLog::EventParameters> params;
   if (!status_.is_success())
     params = new NetLogIntegerParameter("net_error", status_.os_error());
@@ -527,10 +576,22 @@ void URLRequest::ResponseStarted() {
   if (job) {
     RestartWithJob(job);
   } else {
-    if (context_ && context_->network_delegate())
-      context_->network_delegate()->NotifyResponseStarted(this);
-    if (delegate_)
+    if (delegate_) {
+      // In some cases (e.g. an event was canceled), we might have sent the
+      // completion event and receive a NotifyResponseStarted() later.
+      if (!has_notified_completion_ && status_.is_success()) {
+        if (context_ && context_->network_delegate())
+          context_->network_delegate()->NotifyResponseStarted(this);
+      }
+
+      // Notify in case the entire URL Request has been finished.
+      if (!has_notified_completion_ && !status_.is_success())
+        NotifyRequestCompleted();
+
       delegate_->OnResponseStarted(this);
+      // Nothing may appear below this line as OnResponseStarted may delete
+      // |this|.
+    }
   }
 }
 
@@ -649,18 +710,23 @@ int URLRequest::Redirect(const GURL& location, int http_status_code) {
   return OK;
 }
 
-URLRequestContext* URLRequest::context() const {
+const URLRequestContext* URLRequest::context() const {
   return context_.get();
 }
 
-void URLRequest::set_context(URLRequestContext* context) {
-  scoped_refptr<URLRequestContext> prev_context = context_;
+void URLRequest::set_context(const URLRequestContext* context) {
+  scoped_refptr<const URLRequestContext> prev_context = context_;
 
   context_ = context;
 
   // If the context this request belongs to has changed, update the tracker.
   if (prev_context != context) {
-    net_log_.EndEvent(NetLog::TYPE_REQUEST_ALIVE, NULL);
+    int net_error = OK;
+    // Log error only on failure, not cancellation, as even successful requests
+    // are "cancelled" on destruction.
+    if (status_.status() == URLRequestStatus::FAILED)
+      net_error = status_.os_error();
+    net_log_.EndEventWithNetErrorCode(NetLog::TYPE_REQUEST_ALIVE, net_error);
     net_log_ = BoundNetLog();
 
     if (context) {
@@ -690,4 +756,81 @@ void URLRequest::SetUserData(const void* key, UserData* data) {
   user_data_[key] = linked_ptr<UserData>(data);
 }
 
+void URLRequest::NotifyAuthRequired(AuthChallengeInfo* auth_info) {
+  // TODO(battre): We could simulate a redirection there as follows:
+  // if (context_ && context_->network_delegate()) {
+  //  // We simulate a redirection.
+  //   context_->network_delegate()->NotifyBeforeRedirect(this, url());
+  //}
+  // This fixes URLRequestTestHTTP.BasicAuth but not
+  // URLRequestTestHTTP.BasicAuthWithCookies. In both cases we observe a
+  // call sequence of OnBeforeSendHeaders -> OnSendHeaders ->
+  // OnBeforeSendHeaders.
+  if (context_ && context_->network_delegate())
+    context_->network_delegate()->NotifyAuthRequired(this, *auth_info);
+
+  if (delegate_)
+    delegate_->OnAuthRequired(this, auth_info);
+}
+
+void URLRequest::NotifyCertificateRequested(
+    SSLCertRequestInfo* cert_request_info) {
+  if (delegate_)
+    delegate_->OnCertificateRequested(this, cert_request_info);
+}
+
+void URLRequest::NotifySSLCertificateError(int cert_error,
+                                           X509Certificate* cert) {
+  if (delegate_)
+    delegate_->OnSSLCertificateError(this, cert_error, cert);
+}
+
+bool URLRequest::CanGetCookies(const CookieList& cookie_list) const {
+  if (delegate_)
+    return delegate_->CanGetCookies(this, cookie_list);
+  return false;
+}
+
+bool URLRequest::CanSetCookie(const std::string& cookie_line,
+                              CookieOptions* options) const {
+  if (delegate_)
+    return delegate_->CanSetCookie(this, cookie_line, options);
+  return false;
+}
+
+
+void URLRequest::NotifyReadCompleted(int bytes_read) {
+  // Notify in case the entire URL Request has been finished.
+  if (bytes_read <= 0)
+    NotifyRequestCompleted();
+
+  if (delegate_)
+    delegate_->OnReadCompleted(this, bytes_read);
+
+  // Nothing below this line as OnReadCompleted may delete |this|.
+}
+
+void URLRequest::NotifyRequestCompleted() {
+  // TODO(battre): Get rid of this check, according to willchan it should
+  // not be needed.
+  if (has_notified_completion_)
+    return;
+
+  is_pending_ = false;
+  has_notified_completion_ = true;
+  if (context_ && context_->network_delegate())
+    context_->network_delegate()->NotifyCompleted(this);
+}
+
+void URLRequest::SetBlockedOnDelegate() {
+  blocked_on_delegate_ = true;
+  net_log_.BeginEvent(NetLog::TYPE_URL_REQUEST_BLOCKED_ON_DELEGATE, NULL);
+}
+
+void URLRequest::SetUnblockedOnDelegate() {
+  blocked_on_delegate_ = false;
+  load_state_param_.clear();
+  net_log_.EndEvent(NetLog::TYPE_URL_REQUEST_BLOCKED_ON_DELEGATE, NULL);
+}
+
 }  // namespace net
