@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -10,13 +10,15 @@
 
 #include "base/basictypes.h"
 #include "googleurl/src/gurl.h"
+#include "net/base/net_export.h"
 #include "net/base/net_log.h"
 #include "net/base/request_priority.h"
 
 namespace net {
 
 // Holds the parameters to emit to the NetLog when starting a URLRequest.
-class URLRequestStartEventParameters : public NetLog::EventParameters {
+class NET_EXPORT URLRequestStartEventParameters
+    : public NetLog::EventParameters {
  public:
   URLRequestStartEventParameters(const GURL& url,
                                  const std::string& method,
@@ -31,7 +33,7 @@ class URLRequestStartEventParameters : public NetLog::EventParameters {
     return load_flags_;
   }
 
-  virtual Value* ToValue() const;
+  virtual base::Value* ToValue() const;
 
  private:
   const GURL url_;
