@@ -19,7 +19,7 @@
 
 namespace net {
 
-class NET_TEST NetLogWebSocketHandshakeParameter
+class NET_EXPORT_PRIVATE NetLogWebSocketHandshakeParameter
     : public NetLog::EventParameters {
  public:
   explicit NetLogWebSocketHandshakeParameter(const std::string& headers);
