@@ -23,11 +23,12 @@ PP_COMPILE_ASSERT_SIZE_IN_BYTES(PP_ScrollBy_Dev, 4);
 
 #define PPB_SCROLLBAR_DEV_INTERFACE_0_3 "PPB_Scrollbar(Dev);0.3"
 #define PPB_SCROLLBAR_DEV_INTERFACE_0_4 "PPB_Scrollbar(Dev);0.4"
-#define PPB_SCROLLBAR_DEV_INTERFACE PPB_SCROLLBAR_DEV_INTERFACE_0_4
+#define PPB_SCROLLBAR_DEV_INTERFACE_0_5 "PPB_Scrollbar(Dev);0.5"
+#define PPB_SCROLLBAR_DEV_INTERFACE PPB_SCROLLBAR_DEV_INTERFACE_0_5
 
 // The interface for a scrollbar.  A scrollbar is a widget, so the functions
 // in PPB_Widget can also be used with scrollbar objects.
-struct PPB_Scrollbar_0_4_Dev {
+struct PPB_Scrollbar_0_5_Dev {
   // Create a new scrollbar.  Returns 0 if the instance is invalid.
   PP_Resource (*Create)(PP_Instance instance,
                         PP_Bool vertical);
@@ -39,6 +40,9 @@ struct PPB_Scrollbar_0_4_Dev {
   // Gets the thickness of a scrollbar.
   uint32_t (*GetThickness)(PP_Resource resource);
 
+  // Returns PP_TRUE if the system scrollbar style is an overlap scrollbar.
+  PP_Bool (*IsOverlay)(PP_Resource scrollbar);
+
   // Get/set the value of the scrollbar.
   uint32_t (*GetValue)(PP_Resource scrollbar);
 
@@ -64,6 +68,25 @@ struct PPB_Scrollbar_0_4_Dev {
                    int32_t multiplier);
 };
 
+// Old version without IsOverlay.
+struct PPB_Scrollbar_0_4_Dev {
+  PP_Resource (*Create)(PP_Instance instance,
+                        PP_Bool vertical);
+  PP_Bool (*IsScrollbar)(PP_Resource resource);
+  uint32_t (*GetThickness)(PP_Resource resource);
+  uint32_t (*GetValue)(PP_Resource scrollbar);
+  void (*SetValue)(PP_Resource scrollbar,
+                   uint32_t value);
+  void (*SetDocumentSize)(PP_Resource scrollbar,
+                          uint32_t size);
+  void (*SetTickMarks)(PP_Resource scrollbar,
+                       const struct PP_Rect* tick_marks,
+                       uint32_t count);
+  void (*ScrollBy)(PP_Resource scrollbar,
+                   PP_ScrollBy_Dev unit,
+                   int32_t multiplier);
+};
+
 // Old version with no resource argument to GetThickness.
 struct PPB_Scrollbar_0_3_Dev {
   PP_Resource (*Create)(PP_Instance instance,
@@ -83,7 +106,6 @@ struct PPB_Scrollbar_0_3_Dev {
                    int32_t multiplier);
 };
 
-typedef struct PPB_Scrollbar_0_4_Dev PPB_Scrollbar_Dev;
+typedef struct PPB_Scrollbar_0_5_Dev PPB_Scrollbar_Dev;
 
 #endif  /* PPAPI_C_DEV_PPB_SCROLLBAR_DEV_H_ */
-
