@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -14,12 +14,12 @@ namespace pp {
 
 class CompletionCallback;
 class DirectoryEntry_Dev;
-class FileRef_Dev;
+class FileRef;
 
 class DirectoryReader_Dev : public Resource {
  public:
   // Creates a DirectoryReader for the given directory.
-  DirectoryReader_Dev(const FileRef_Dev& directory_ref);
+  explicit DirectoryReader_Dev(const FileRef& directory_ref);
 
   DirectoryReader_Dev(const DirectoryReader_Dev& other);
 
