@@ -15,6 +15,7 @@
 namespace pp {
 
 class Context3D_Dev;
+class Graphics3D;
 class Instance;
 
 // C++ wrapper for the Pepper Video Decoder interface. For more detailed
@@ -23,19 +24,18 @@ class Instance;
 // C++ version of the PPB_VideoDecoder_Dev interface.
 class VideoDecoder_Dev : public Resource {
  public:
-  // Constructor for the video decoder. Calls the Create on the
-  // PPB_VideoDecoder_Dev interface.
-  //
-  // Parameters:
-  //  |instance| is the pointer to the plug-in instance.
-  explicit VideoDecoder_Dev(const Instance& instance);
+  // See PPB_VideoDecoder_Dev::Create.
+  VideoDecoder_Dev(const Instance* instance,
+                   const Context3D_Dev& context,
+                   PP_VideoDecoder_Profile profile);
+  VideoDecoder_Dev(const Instance* instance,
+                   const Graphics3D& context,
+                   PP_VideoDecoder_Profile profile);
+
   explicit VideoDecoder_Dev(PP_Resource resource);
   virtual ~VideoDecoder_Dev();
 
   // PPB_VideoDecoder_Dev implementation.
-  int32_t Initialize(const PP_VideoConfigElement* config,
-                     const Context3D_Dev& context,
-                     CompletionCallback callback);
   void AssignPictureBuffers(const std::vector<PP_PictureBuffer_Dev>& buffers);
   int32_t Decode(const PP_VideoBitstreamBuffer_Dev& bitstream_buffer,
                  CompletionCallback callback);
