@@ -13,7 +13,6 @@
 #include "base/process.h"
 #include "ppapi/c/pp_instance.h"
 #include "ppapi/proxy/dispatcher.h"
-#include "ppapi/proxy/plugin_var_tracker.h"
 #include "ppapi/shared_impl/function_group_base.h"
 
 struct PPB_Proxy_Private;
@@ -28,16 +27,15 @@ class SyncChannel;
 }
 
 namespace ppapi {
-struct Preferences;
-}
 
-namespace pp {
+struct Preferences;
+
 namespace proxy {
 
 class InterfaceProxy;
 class VarSerialization;
 
-class HostDispatcher : public Dispatcher {
+class PPAPI_PROXY_EXPORT HostDispatcher : public Dispatcher {
  public:
   // Constructor for the renderer side.
   //
@@ -53,7 +51,7 @@ class HostDispatcher : public Dispatcher {
   virtual bool InitHostWithChannel(Delegate* delegate,
                                    const IPC::ChannelHandle& channel_handle,
                                    bool is_client,
-                                   const ppapi::Preferences& preferences);
+                                   const Preferences& preferences);
 
   // The host side maintains a mapping from PP_Instance to Dispatcher so
   // that we can send the messages to the right channel.
@@ -122,8 +120,7 @@ class HostDispatcher : public Dispatcher {
   // Function proxies created for "new-style" FunctionGroups.
   // TODO(brettw) this is in progress. It should be merged with the target
   // proxies so there is one list to consult.
-  scoped_ptr< ::ppapi::FunctionGroupBase >
-      function_proxies_[INTERFACE_ID_COUNT];
+  scoped_ptr<FunctionGroupBase> function_proxies_[INTERFACE_ID_COUNT];
 
   // Guaranteed non-NULL.
   const PPB_Proxy_Private* ppb_proxy_;
@@ -156,6 +153,6 @@ class ScopedModuleReference {
 };
 
 }  // namespace proxy
-}  // namespace pp
+}  // namespace ppapi
 
 #endif  // PPAPI_PROXY_HOST_DISPATCHER_H_
