@@ -5,7 +5,7 @@
 #ifndef PPAPI_PROXY_INTERFACE_ID_H_
 #define PPAPI_PROXY_INTERFACE_ID_H_
 
-namespace pp {
+namespace ppapi {
 namespace proxy {
 
 // These numbers must be all small integers. They are used in a lookup table
@@ -51,13 +51,16 @@ enum InterfaceID {
   INTERFACE_ID_PPB_URL_UTIL,
   INTERFACE_ID_PPB_VAR,
   INTERFACE_ID_PPB_VAR_DEPRECATED,
+  INTERFACE_ID_PPB_VIDEO_CAPTURE_DEV,
   INTERFACE_ID_PPB_VIDEO_DECODER_DEV,
 
   INTERFACE_ID_PPP_CLASS,
-  INTERFACE_ID_PPP_GRAPHICS_3D_DEV,
+  INTERFACE_ID_PPP_GRAPHICS_3D,
   INTERFACE_ID_PPP_INPUT_EVENT,
   INTERFACE_ID_PPP_INSTANCE,
   INTERFACE_ID_PPP_INSTANCE_PRIVATE,
+  INTERFACE_ID_PPP_MESSAGING,
+  INTERFACE_ID_PPP_VIDEO_CAPTURE_DEV,
   INTERFACE_ID_PPP_VIDEO_DECODER_DEV,
 
   INTERFACE_ID_RESOURCE_CREATION,
@@ -67,6 +70,6 @@ enum InterfaceID {
 };
 
 }  // namespace proxy
-}  // namespace pp
+}  // namespace ppapi
 
 #endif  // PPAPI_PROXY_INTERFACE_ID_H_
