@@ -9,7 +9,7 @@
 #include "ppapi/proxy/ppapi_messages.h"
 #include "ppapi/proxy/ppapi_proxy_test.h"
 
-namespace pp {
+namespace ppapi {
 namespace proxy {
 
 namespace {
@@ -85,5 +85,5 @@ TEST_F(PluginDispatcherTest, PPBCreation) {
 }
 
 }  // namespace proxy
-}  // namespace pp
+}  // namespace ppapi
 
