@@ -12,34 +12,34 @@
 #include "ppapi/proxy/enter_proxy.h"
 #include "ppapi/proxy/host_dispatcher.h"
 #include "ppapi/proxy/plugin_dispatcher.h"
-#include "ppapi/proxy/plugin_resource.h"
 #include "ppapi/proxy/ppapi_messages.h"
 #include "ppapi/proxy/ppb_file_ref_proxy.h"
 #include "ppapi/proxy/serialized_var.h"
+#include "ppapi/shared_impl/var.h"
 #include "ppapi/thunk/thunk.h"
 
-using ::ppapi::thunk::PPB_FileChooser_API;
+using ppapi::thunk::PPB_FileChooser_API;
 
-namespace pp {
+namespace ppapi {
 namespace proxy {
 
-class FileChooser : public PluginResource,
+class FileChooser : public Resource,
                     public PPB_FileChooser_API {
  public:
   FileChooser(const HostResource& resource);
   virtual ~FileChooser();
 
-  // ResourceObjectBase overrides.
+  // Resource overrides.
   virtual PPB_FileChooser_API* AsPPB_FileChooser_API() OVERRIDE;
 
   // PPB_FileChooser_API implementation.
-  virtual int32_t Show(PP_CompletionCallback callback) OVERRIDE;
+  virtual int32_t Show(const PP_CompletionCallback& callback) OVERRIDE;
   virtual PP_Resource GetNextChosenFile() OVERRIDE;
 
   // Handles the choose complete notification from the host.
   void ChooseComplete(
     int32_t result_code,
-    const std::vector<PPBFileRef_CreateInfo>& chosen_files);
+    const std::vector<PPB_FileRef_CreateInfo>& chosen_files);
 
  private:
   PP_CompletionCallback current_show_callback_;
@@ -54,7 +54,7 @@ class FileChooser : public PluginResource,
 };
 
 FileChooser::FileChooser(const HostResource& resource)
-    : PluginResource(resource),
+    : Resource(resource),
       current_show_callback_(PP_MakeCompletionCallback(NULL, NULL)) {
 }
 
@@ -81,13 +81,14 @@ PPB_FileChooser_API* FileChooser::AsPPB_FileChooser_API() {
   return this;
 }
 
-int32_t FileChooser::Show(PP_CompletionCallback callback) {
+int32_t FileChooser::Show(const PP_CompletionCallback& callback) {
   if (current_show_callback_.func)
     return PP_ERROR_INPROGRESS;  // Can't show more than once.
 
   current_show_callback_ = callback;
-  GetDispatcher()->Send(new PpapiHostMsg_PPBFileChooser_Show(
-      INTERFACE_ID_PPB_FILE_CHOOSER, host_resource()));
+  PluginDispatcher::GetForResource(this)->Send(
+      new PpapiHostMsg_PPBFileChooser_Show(
+          INTERFACE_ID_PPB_FILE_CHOOSER, host_resource()));
   return PP_OK_COMPLETIONPENDING;
 }
 
@@ -105,7 +106,7 @@ PP_Resource FileChooser::GetNextChosenFile() {
 
 void FileChooser::ChooseComplete(
     int32_t result_code,
-    const std::vector<PPBFileRef_CreateInfo>& chosen_files) {
+    const std::vector<PPB_FileRef_CreateInfo>& chosen_files) {
   // Convert each of the passed in file infos to resources. These will be owned
   // by the FileChooser object until they're passed to the plugin.
   DCHECK(file_queue_.empty());
@@ -119,6 +120,37 @@ void FileChooser::ChooseComplete(
 
 namespace {
 
+PP_Resource Create0_4(PP_Instance instance,
+                      const PP_FileChooserOptions_0_4_Dev* options) {
+  PP_Var accept_var = PP_MakeUndefined();
+  if (options->accept_mime_types)
+    accept_var = StringVar::StringToPPVar(0, options->accept_mime_types);
+  PP_Resource result = thunk::GetPPB_FileChooser_Thunk()->Create(
+      instance, options->mode, accept_var);
+  if (accept_var.type == PP_VARTYPE_STRING)
+    PluginResourceTracker::GetInstance()->var_tracker().ReleaseVar(accept_var);
+  return result;
+}
+
+PP_Bool IsFileChooser0_4(PP_Resource resource) {
+  return thunk::GetPPB_FileChooser_Thunk()->IsFileChooser(resource);
+}
+
+int32_t Show0_4(PP_Resource chooser, PP_CompletionCallback callback) {
+  return thunk::GetPPB_FileChooser_Thunk()->Show(chooser, callback);
+}
+
+PP_Resource GetNextChosenFile0_4(PP_Resource chooser) {
+  return thunk::GetPPB_FileChooser_Thunk()->GetNextChosenFile(chooser);
+}
+
+PPB_FileChooser_0_4_Dev file_chooser_0_4_interface = {
+  &Create0_4,
+  &IsFileChooser0_4,
+  &Show0_4,
+  &GetNextChosenFile0_4
+};
+
 InterfaceProxy* CreateFileChooserProxy(Dispatcher* dispatcher,
                                        const void* target_interface) {
   return new PPB_FileChooser_Proxy(dispatcher, target_interface);
@@ -137,7 +169,7 @@ PPB_FileChooser_Proxy::~PPB_FileChooser_Proxy() {
 
 const InterfaceProxy::Info* PPB_FileChooser_Proxy::GetInfo() {
   static const Info info = {
-    ::ppapi::thunk::GetPPB_FileChooser_Thunk(),
+    thunk::GetPPB_FileChooser_Thunk(),
     PPB_FILECHOOSER_DEV_INTERFACE,
     INTERFACE_ID_PPB_FILE_CHOOSER,
     false,
@@ -146,10 +178,22 @@ const InterfaceProxy::Info* PPB_FileChooser_Proxy::GetInfo() {
   return &info;
 }
 
+const InterfaceProxy::Info* PPB_FileChooser_Proxy::GetInfo0_4() {
+  static const Info info = {
+    &file_chooser_0_4_interface,
+    PPB_FILECHOOSER_DEV_INTERFACE_0_4,
+    INTERFACE_ID_NONE,
+    false,
+    &CreateFileChooserProxy,
+  };
+  return &info;
+}
+
 // static
 PP_Resource PPB_FileChooser_Proxy::CreateProxyResource(
     PP_Instance instance,
-    const PP_FileChooserOptions_Dev* options) {
+    PP_FileChooserMode_Dev mode,
+    const PP_Var& accept_mime_types) {
   Dispatcher* dispatcher = PluginDispatcher::GetForInstance(instance);
   if (!dispatcher)
     return 0;
@@ -157,14 +201,12 @@ PP_Resource PPB_FileChooser_Proxy::CreateProxyResource(
   HostResource result;
   dispatcher->Send(new PpapiHostMsg_PPBFileChooser_Create(
       INTERFACE_ID_PPB_FILE_CHOOSER, instance,
-      options->mode,
-      options->accept_mime_types ? options->accept_mime_types : std::string(),
+      mode, SerializedVarSendInput(dispatcher, accept_mime_types),
       &result));
 
   if (result.is_null())
     return 0;
-  linked_ptr<FileChooser> object(new FileChooser(result));
-  return PluginResourceTracker::GetInstance()->AddResource(object);
+  return (new FileChooser(result))->GetReference();
 }
 
 bool PPB_FileChooser_Proxy::OnMessageReceived(const IPC::Message& msg) {
@@ -182,31 +224,28 @@ bool PPB_FileChooser_Proxy::OnMessageReceived(const IPC::Message& msg) {
   return handled;
 }
 
-void PPB_FileChooser_Proxy::OnMsgCreate(PP_Instance instance,
-                                        int mode,
-                                        const std::string& accept_mime_types,
-                                        HostResource* result) {
-  PP_FileChooserOptions_Dev options;
-  options.mode = static_cast<PP_FileChooserMode_Dev>(mode);
-  options.accept_mime_types = accept_mime_types.c_str();
-  result->SetHostResource(
-      instance, ppb_file_chooser_target()->Create(instance, &options));
+void PPB_FileChooser_Proxy::OnMsgCreate(
+    PP_Instance instance,
+    int mode,
+    SerializedVarReceiveInput accept_mime_types,
+    HostResource* result) {
+  result->SetHostResource(instance, ppb_file_chooser_target()->Create(
+      instance, static_cast<PP_FileChooserMode_Dev>(mode),
+      accept_mime_types.Get(dispatcher())));
 }
 
 void PPB_FileChooser_Proxy::OnMsgShow(const HostResource& chooser) {
-  CompletionCallback callback = callback_factory_.NewOptionalCallback(
-      &PPB_FileChooser_Proxy::OnShowCallback, chooser);
-
-  int32_t result = ppb_file_chooser_target()->Show(
-      chooser.host_resource(), callback.pp_completion_callback());
-  if (result != PP_OK_COMPLETIONPENDING)
-    callback.Run(result);
+  EnterHostFromHostResourceForceCallback<PPB_FileChooser_API> enter(
+      chooser, callback_factory_, &PPB_FileChooser_Proxy::OnShowCallback,
+      chooser);
+  if (enter.succeeded())
+    enter.SetResult(enter.object()->Show(enter.callback()));
 }
 
 void PPB_FileChooser_Proxy::OnMsgChooseComplete(
     const HostResource& chooser,
     int32_t result_code,
-    const std::vector<PPBFileRef_CreateInfo>& chosen_files) {
+    const std::vector<PPB_FileRef_CreateInfo>& chosen_files) {
   EnterPluginFromHostResource<PPB_FileChooser_API> enter(chooser);
   if (enter.succeeded()) {
     static_cast<FileChooser*>(enter.object())->ChooseComplete(
@@ -216,7 +255,7 @@ void PPB_FileChooser_Proxy::OnMsgChooseComplete(
 
 void PPB_FileChooser_Proxy::OnShowCallback(int32_t result,
                                            const HostResource& chooser) {
-  std::vector<PPBFileRef_CreateInfo> files;
+  std::vector<PPB_FileRef_CreateInfo> files;
   if (result == PP_OK) {
     // Jump through some hoops to get the FileRef proxy. Since we know we're
     // in the host at this point, we can ask the host dispatcher for it.
@@ -229,7 +268,7 @@ void PPB_FileChooser_Proxy::OnShowCallback(int32_t result,
     while (PP_Resource cur_file_resource =
                ppb_file_chooser_target()->GetNextChosenFile(
                    chooser.host_resource())) {
-      PPBFileRef_CreateInfo cur_create_info;
+      PPB_FileRef_CreateInfo cur_create_info;
       file_ref_proxy->SerializeFileRef(cur_file_resource, &cur_create_info);
       files.push_back(cur_create_info);
     }
@@ -240,4 +279,4 @@ void PPB_FileChooser_Proxy::OnShowCallback(int32_t result,
 }
 
 }  // namespace proxy
-}  // namespace pp
+}  // namespace ppapi
