@@ -18,11 +18,12 @@
 
 struct PPB_FileSystem_Dev;
 
-namespace pp {
-namespace proxy {
+namespace ppapi {
 
 class HostResource;
 
+namespace proxy {
+
 class PPB_FileSystem_Proxy : public InterfaceProxy {
  public:
   PPB_FileSystem_Proxy(Dispatcher* dispatcher, const void* target_interface);
@@ -40,22 +41,23 @@ class PPB_FileSystem_Proxy : public InterfaceProxy {
   // Message handlers.
   void OnMsgCreate(PP_Instance instance,
                    int type,
-                   HostResource* result);
-  void OnMsgOpen(const HostResource& filesystem,
+                   ppapi::HostResource* result);
+  void OnMsgOpen(const ppapi::HostResource& filesystem,
                  int64_t expected_size);
 
-  void OnMsgOpenComplete(const HostResource& filesystem,
+  void OnMsgOpenComplete(const ppapi::HostResource& filesystem,
                          int32_t result);
 
-  void OpenCompleteInHost(int32_t result, const HostResource& host_resource);
+  void OpenCompleteInHost(int32_t result,
+                          const ppapi::HostResource& host_resource);
 
-  CompletionCallbackFactory<PPB_FileSystem_Proxy,
-                            ProxyNonThreadSafeRefCount> callback_factory_;
+  pp::CompletionCallbackFactory<PPB_FileSystem_Proxy,
+                                ProxyNonThreadSafeRefCount> callback_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(PPB_FileSystem_Proxy);
 };
 
 }  // namespace proxy
-}  // namespace pp
+}  // namespace ppapi
 
 #endif  // PPAPI_PROXY_PPB_FILE_SYSTEM_PROXY_H_
