@@ -12,28 +12,29 @@
 #include "ppapi/proxy/ppb_image_data_proxy.h"
 #include "ppapi/proxy/serialized_var.h"
 #include "ppapi/shared_impl/ppapi_preferences.h"
-#include "ppapi/shared_impl/resource_object_base.h"
+#include "ppapi/shared_impl/resource.h"
+#include "ppapi/shared_impl/var.h"
 #include "ppapi/thunk/enter.h"
 #include "ppapi/thunk/ppb_image_data_api.h"
 #include "ppapi/thunk/thunk.h"
 
 using ppapi::thunk::EnterResourceNoLock;
+using ppapi::thunk::PPB_Font_API;
+using ppapi::thunk::PPB_Font_FunctionAPI;
 using ppapi::thunk::PPB_ImageData_API;
-using ppapi::WebKitForwarding;
 
-namespace pp {
+namespace ppapi {
 namespace proxy {
 
 namespace {
 
 bool PPTextRunToTextRun(const PP_TextRun_Dev* run,
                         WebKitForwarding::Font::TextRun* output) {
-  const std::string* str = PluginVarTracker::GetInstance()->GetExistingString(
-      run->text);
+  StringVar* str = StringVar::FromPPVar(run->text);
   if (!str)
     return false;
 
-  output->text = *str;
+  output->text = str->value();
   output->rtl = PP_ToBool(run->rtl);
   output->override_direction = PP_ToBool(run->override_direction);
   return true;
@@ -57,7 +58,7 @@ PPB_Font_Proxy::~PPB_Font_Proxy() {
 // static
 const InterfaceProxy::Info* PPB_Font_Proxy::GetInfo() {
   static const Info info = {
-    ::ppapi::thunk::GetPPB_Font_Thunk(),
+    thunk::GetPPB_Font_Thunk(),
     PPB_FONT_DEV_INTERFACE,
     INTERFACE_ID_PPB_FONT,
     false,
@@ -66,7 +67,7 @@ const InterfaceProxy::Info* PPB_Font_Proxy::GetInfo() {
   return &info;
 }
 
-::ppapi::thunk::PPB_Font_FunctionAPI* PPB_Font_Proxy::AsPPB_Font_FunctionAPI() {
+PPB_Font_FunctionAPI* PPB_Font_Proxy::AsPPB_Font_FunctionAPI() {
   return this;
 }
 
@@ -82,10 +83,7 @@ PP_Var PPB_Font_Proxy::GetFontFamilies(PP_Instance instance) {
         new PpapiHostMsg_PPBFont_GetFontFamilies(&families));
   }
 
-  PP_Var result;
-  result.type = PP_VARTYPE_STRING;
-  result.value.as_id = PluginVarTracker::GetInstance()->MakeString(families);
-  return result;
+  return StringVar::StringToPPVar(0, families);
 }
 
 bool PPB_Font_Proxy::OnMessageReceived(const IPC::Message& msg) {
@@ -96,32 +94,34 @@ bool PPB_Font_Proxy::OnMessageReceived(const IPC::Message& msg) {
 
 Font::Font(const HostResource& resource,
            const PP_FontDescription_Dev& desc)
-    : PluginResource(resource),
-      webkit_event_(false, false) {
+    : Resource(resource),
+      webkit_event_(false, false),
+      font_forwarding_(NULL) {
   TRACE_EVENT0("ppapi proxy", "Font::Font");
-  const std::string* face = PluginVarTracker::GetInstance()->GetExistingString(
-      desc.face);
+  StringVar* face = StringVar::FromPPVar(desc.face);
 
-  WebKitForwarding* forwarding = GetDispatcher()->GetWebKitForwarding();
+  PluginDispatcher* dispatcher = PluginDispatcher::GetForResource(this);
+  if (!dispatcher)
+    return;
+  WebKitForwarding* forwarding = dispatcher->GetWebKitForwarding();
 
-  WebKitForwarding::Font* result = NULL;
-  RunOnWebKitThread(base::Bind(&WebKitForwarding::CreateFontForwarding,
+  RunOnWebKitThread(true,
+                    base::Bind(&WebKitForwarding::CreateFontForwarding,
                                base::Unretained(forwarding),
                                &webkit_event_, desc,
-                               face ? *face : std::string(),
-                               GetDispatcher()->preferences(),
-                               &result));
-  font_forwarding_.reset(result);
+                               face ? face->value() : std::string(),
+                               dispatcher->preferences(),
+                               &font_forwarding_));
 }
 
 Font::~Font() {
+  if (font_forwarding_) {
+    RunOnWebKitThread(false,
+                      base::Bind(&DeleteFontForwarding, font_forwarding_));
+  }
 }
 
-ppapi::thunk::PPB_Font_API* Font::AsPPB_Font_API() {
-  return this;
-}
-
-Font* Font::AsFont() {
+PPB_Font_API* Font::AsPPB_Font_API() {
   return this;
 }
 
@@ -130,18 +130,18 @@ PP_Bool Font::Describe(PP_FontDescription_Dev* description,
   TRACE_EVENT0("ppapi proxy", "Font::Describe");
   std::string face;
   PP_Bool result = PP_FALSE;
-  RunOnWebKitThread(base::Bind(&WebKitForwarding::Font::Describe,
-                               base::Unretained(font_forwarding_.get()),
-                               &webkit_event_, description, &face, metrics,
-                               &result));
-
-  if (result == PP_TRUE) {
-    description->face.type = PP_VARTYPE_STRING;
-    description->face.value.as_id =
-        PluginVarTracker::GetInstance()->MakeString(face);
-  } else {
-    description->face.type = PP_VARTYPE_UNDEFINED;
+  if (font_forwarding_) {
+    RunOnWebKitThread(true,
+                      base::Bind(&WebKitForwarding::Font::Describe,
+                                 base::Unretained(font_forwarding_),
+                                 &webkit_event_, description, &face, metrics,
+                                 &result));
   }
+
+  if (PP_ToBool(result))
+    description->face = StringVar::StringToPPVar(0, face);
+  else
+    description->face = PP_MakeUndefined();
   return result;
 }
 
@@ -152,6 +152,9 @@ PP_Bool Font::DrawTextAt(PP_Resource pp_image_data,
                          const PP_Rect* clip,
                          PP_Bool image_data_is_opaque) {
   TRACE_EVENT0("ppapi proxy", "Font::DrawTextAt");
+  if (!font_forwarding_)
+    return PP_FALSE;
+
   // Convert to an ImageData object.
   EnterResourceNoLock<PPB_ImageData_API> enter(pp_image_data, true);
   if (enter.failed())
@@ -174,12 +177,13 @@ PP_Bool Font::DrawTextAt(PP_Resource pp_image_data,
       image_data->Unmap();
     return PP_FALSE;
   }
-  RunOnWebKitThread(base::Bind(
-      &WebKitForwarding::Font::DrawTextAt,
-      base::Unretained(font_forwarding_.get()),
-      &webkit_event_,
-      WebKitForwarding::Font::DrawTextParams(canvas, run, position, color,
-                                             clip, image_data_is_opaque)));
+  RunOnWebKitThread(
+      true,
+      base::Bind(&WebKitForwarding::Font::DrawTextAt,
+                 base::Unretained(font_forwarding_), &webkit_event_,
+                 WebKitForwarding::Font::DrawTextParams(canvas, run, position,
+                                                        color, clip,
+                                                        image_data_is_opaque)));
 
   if (needs_unmapping)
     image_data->Unmap();
@@ -189,11 +193,12 @@ PP_Bool Font::DrawTextAt(PP_Resource pp_image_data,
 int32_t Font::MeasureText(const PP_TextRun_Dev* text) {
   TRACE_EVENT0("ppapi proxy", "Font::MeasureText");
   WebKitForwarding::Font::TextRun run;
-  if (!PPTextRunToTextRun(text, &run))
+  if (!font_forwarding_ || !PPTextRunToTextRun(text, &run))
     return -1;
   int32_t result = -1;
-  RunOnWebKitThread(base::Bind(&WebKitForwarding::Font::MeasureText,
-                               base::Unretained(font_forwarding_.get()),
+  RunOnWebKitThread(true,
+                    base::Bind(&WebKitForwarding::Font::MeasureText,
+                               base::Unretained(font_forwarding_),
                                &webkit_event_, run, &result));
   return result;
 }
@@ -202,11 +207,12 @@ uint32_t Font::CharacterOffsetForPixel(const PP_TextRun_Dev* text,
                                        int32_t pixel_position) {
   TRACE_EVENT0("ppapi proxy", "Font::CharacterOffsetForPixel");
   WebKitForwarding::Font::TextRun run;
-  if (!PPTextRunToTextRun(text, &run))
+  if (!font_forwarding_ || !PPTextRunToTextRun(text, &run))
     return -1;
   uint32_t result = -1;
-  RunOnWebKitThread(base::Bind(&WebKitForwarding::Font::CharacterOffsetForPixel,
-                               base::Unretained(font_forwarding_.get()),
+  RunOnWebKitThread(true,
+                    base::Bind(&WebKitForwarding::Font::CharacterOffsetForPixel,
+                               base::Unretained(font_forwarding_),
                                &webkit_event_, run, pixel_position, &result));
   return result;
 }
@@ -215,19 +221,26 @@ int32_t Font::PixelOffsetForCharacter(const PP_TextRun_Dev* text,
                                       uint32_t char_offset) {
   TRACE_EVENT0("ppapi proxy", "Font::PixelOffsetForCharacter");
   WebKitForwarding::Font::TextRun run;
-  if (!PPTextRunToTextRun(text, &run))
+  if (!font_forwarding_ || !PPTextRunToTextRun(text, &run))
     return -1;
   int32_t result = -1;
-  RunOnWebKitThread(base::Bind(&WebKitForwarding::Font::PixelOffsetForCharacter,
-                               base::Unretained(font_forwarding_.get()),
+  RunOnWebKitThread(true,
+                    base::Bind(&WebKitForwarding::Font::PixelOffsetForCharacter,
+                               base::Unretained(font_forwarding_),
                                &webkit_event_, run, char_offset, &result));
   return result;
 }
 
-void Font::RunOnWebKitThread(const base::Closure& task) {
-  GetDispatcher()->PostToWebKitThread(FROM_HERE, task);
-  webkit_event_.Wait();
+void Font::RunOnWebKitThread(bool blocking, const base::Closure& task) {
+  PluginDispatcher::GetForResource(this)->PostToWebKitThread(FROM_HERE, task);
+  if (blocking)
+    webkit_event_.Wait();
+}
+
+// static
+void Font::DeleteFontForwarding(WebKitForwarding::Font* font_forwarding) {
+  delete font_forwarding;
 }
 
 }  // namespace proxy
-}  // namespace pp
+}  // namespace ppapi
