@@ -7,20 +7,21 @@
 
 #include "base/time.h"
 #include "ppapi/c/pp_time.h"
+#include "ppapi/shared_impl/ppapi_shared_export.h"
 
 namespace ppapi {
 
-PP_Time TimeToPPTime(base::Time t);
-base::Time PPTimeToTime(PP_Time t);
+PPAPI_SHARED_EXPORT PP_Time TimeToPPTime(base::Time t);
+PPAPI_SHARED_EXPORT base::Time PPTimeToTime(PP_Time t);
 
-PP_TimeTicks TimeTicksToPPTimeTicks(base::TimeTicks t);
+PPAPI_SHARED_EXPORT PP_TimeTicks TimeTicksToPPTimeTicks(base::TimeTicks t);
 
 // Converts between WebKit event times and time ticks. WebKit event times are
 // currently expressed in terms of wall clock time. This function does the
 // proper mapping to time ticks, assuming the wall clock time doesn't change
 // (which isn't necessarily the case).
-PP_TimeTicks EventTimeToPPTimeTicks(double event_time);
-double PPTimeTicksToEventTime(PP_TimeTicks t);
+PPAPI_SHARED_EXPORT PP_TimeTicks EventTimeToPPTimeTicks(double event_time);
+PPAPI_SHARED_EXPORT double PPTimeTicksToEventTime(PP_TimeTicks t);
 
 }  // namespace ppapi
 
