@@ -5,13 +5,14 @@
 #include "ppapi/tests/test_file_ref.h"
 
 #include <stdio.h>
+#include <vector>
 
 #include "ppapi/c/pp_errors.h"
-#include "ppapi/c/dev/ppb_file_io_dev.h"
+#include "ppapi/c/ppb_file_io.h"
 #include "ppapi/c/dev/ppb_testing_dev.h"
-#include "ppapi/cpp/dev/file_io_dev.h"
-#include "ppapi/cpp/dev/file_ref_dev.h"
-#include "ppapi/cpp/dev/file_system_dev.h"
+#include "ppapi/cpp/file_io.h"
+#include "ppapi/cpp/file_ref.h"
+#include "ppapi/cpp/file_system.h"
 #include "ppapi/cpp/instance.h"
 #include "ppapi/cpp/module.h"
 #include "ppapi/cpp/url_loader.h"
@@ -44,27 +45,58 @@ bool TestFileRef::Init() {
 }
 
 void TestFileRef::RunTest() {
-  RUN_TEST(GetFileSystemType);
-  RUN_TEST(GetName);
-  RUN_TEST(GetPath);
-  RUN_TEST(GetParent);
-  RUN_TEST(MakeDirectory);
-  RUN_TEST(QueryAndTouchFile);
-  RUN_TEST(DeleteFileAndDirectory);
-  RUN_TEST(RenameFileAndDirectory);
+  RUN_TEST_FORCEASYNC_AND_NOT(Create);
+  RUN_TEST_FORCEASYNC_AND_NOT(GetFileSystemType);
+  RUN_TEST_FORCEASYNC_AND_NOT(GetName);
+  RUN_TEST_FORCEASYNC_AND_NOT(GetPath);
+  RUN_TEST_FORCEASYNC_AND_NOT(GetParent);
+  RUN_TEST_FORCEASYNC_AND_NOT(MakeDirectory);
+  RUN_TEST_FORCEASYNC_AND_NOT(QueryAndTouchFile);
+  RUN_TEST_FORCEASYNC_AND_NOT(DeleteFileAndDirectory);
+  RUN_TEST_FORCEASYNC_AND_NOT(RenameFileAndDirectory);
+}
+
+std::string TestFileRef::TestCreate() {
+  std::vector<std::string> invalid_paths;
+  invalid_paths.push_back("invalid_path");  // no '/' at the first character
+  invalid_paths.push_back("");  // empty path
+  // The following are directory traversal checks
+  invalid_paths.push_back("..");
+  invalid_paths.push_back("/../invalid_path");
+  invalid_paths.push_back("/../../invalid_path");
+  invalid_paths.push_back("/invalid/../../path");
+  const size_t num_invalid_paths = invalid_paths.size();
+
+  pp::FileSystem file_system_pers(
+      instance_, PP_FILESYSTEMTYPE_LOCALPERSISTENT);
+  pp::FileSystem file_system_temp(
+      instance_, PP_FILESYSTEMTYPE_LOCALTEMPORARY);
+  for (size_t j = 0; j < num_invalid_paths; ++j) {
+    pp::FileRef file_ref_pers(file_system_pers, invalid_paths[j].c_str());
+    if (file_ref_pers.pp_resource() != 0) {
+      return "file_ref_pers expected to be invalid for path: " +
+          invalid_paths[j];
+    }
+    pp::FileRef file_ref_temp(file_system_temp, invalid_paths[j].c_str());
+    if (file_ref_temp.pp_resource() != 0) {
+      return "file_ref_temp expected to be invalid for path: " +
+          invalid_paths[j];
+    }
+  }
+  PASS();
 }
 
 std::string TestFileRef::TestGetFileSystemType() {
-  pp::FileSystem_Dev file_system_pers(
+  pp::FileSystem file_system_pers(
       instance_, PP_FILESYSTEMTYPE_LOCALPERSISTENT);
-  pp::FileSystem_Dev file_system_temp(
+  pp::FileSystem file_system_temp(
       instance_, PP_FILESYSTEMTYPE_LOCALTEMPORARY);
 
-  pp::FileRef_Dev file_ref_pers(file_system_pers, kPersFilePath);
+  pp::FileRef file_ref_pers(file_system_pers, kPersFilePath);
   if (file_ref_pers.GetFileSystemType() != PP_FILESYSTEMTYPE_LOCALPERSISTENT)
     return "file_ref_pers expected to be persistent.";
 
-  pp::FileRef_Dev file_ref_temp(file_system_temp, kTempFilePath);
+  pp::FileRef file_ref_temp(file_system_temp, kTempFilePath);
   if (file_ref_temp.GetFileSystemType() != PP_FILESYSTEMTYPE_LOCALTEMPORARY)
     return "file_ref_temp expected to be temporary.";
 
@@ -72,10 +104,12 @@ std::string TestFileRef::TestGetFileSystemType() {
   request.SetURL("test_url_loader_data/hello.txt");
   request.SetStreamToFile(true);
 
-  TestCompletionCallback callback(instance_->pp_instance());
+  TestCompletionCallback callback(instance_->pp_instance(), force_async_);
 
   pp::URLLoader loader(instance_);
   int32_t rv = loader.Open(request, callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("URLLoader::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
@@ -88,7 +122,7 @@ std::string TestFileRef::TestGetFileSystemType() {
   if (status_code != 200)
     return "Unexpected HTTP status code";
 
-  pp::FileRef_Dev file_ref_ext(response_info.GetBodyAsFileRef());
+  pp::FileRef file_ref_ext(response_info.GetBodyAsFileRef());
   if (file_ref_ext.GetFileSystemType() != PP_FILESYSTEMTYPE_EXTERNAL)
     return "file_ref_ext expected to be external.";
 
@@ -96,23 +130,23 @@ std::string TestFileRef::TestGetFileSystemType() {
 }
 
 std::string TestFileRef::TestGetName() {
-  pp::FileSystem_Dev file_system_pers(
+  pp::FileSystem file_system_pers(
       instance_, PP_FILESYSTEMTYPE_LOCALPERSISTENT);
-  pp::FileSystem_Dev file_system_temp(
+  pp::FileSystem file_system_temp(
       instance_, PP_FILESYSTEMTYPE_LOCALTEMPORARY);
 
-  pp::FileRef_Dev file_ref_pers(file_system_pers, kPersFilePath);
+  pp::FileRef file_ref_pers(file_system_pers, kPersFilePath);
   std::string name = file_ref_pers.GetName().AsString();
   if (name != kPersFileName)
     return ReportMismatch("FileRef::GetName", name, kPersFileName);
 
-  pp::FileRef_Dev file_ref_temp(file_system_temp, kTempFilePath);
+  pp::FileRef file_ref_temp(file_system_temp, kTempFilePath);
   name = file_ref_temp.GetName().AsString();
   if (name != kTempFileName)
     return ReportMismatch("FileRef::GetName", name, kTempFileName);
 
   // Test the "/" case.
-  pp::FileRef_Dev file_ref_slash(file_system_temp, "/");
+  pp::FileRef file_ref_slash(file_system_temp, "/");
   name = file_ref_slash.GetName().AsString();
   if (name != "/")
     return ReportMismatch("FileRef::GetName", name, "/");
@@ -121,10 +155,12 @@ std::string TestFileRef::TestGetName() {
   request.SetURL("test_url_loader_data/hello.txt");
   request.SetStreamToFile(true);
 
-  TestCompletionCallback callback(instance_->pp_instance());
+  TestCompletionCallback callback(instance_->pp_instance(), force_async_);
 
   pp::URLLoader loader(instance_);
   int32_t rv = loader.Open(request, callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("URLLoader::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
@@ -137,7 +173,7 @@ std::string TestFileRef::TestGetName() {
   if (status_code != 200)
     return "Unexpected HTTP status code";
 
-  pp::FileRef_Dev file_ref_ext(response_info.GetBodyAsFileRef());
+  pp::FileRef file_ref_ext(response_info.GetBodyAsFileRef());
   name = file_ref_ext.GetName().AsString();
   if (name == "")
     return ReportMismatch("FileRef::GetName", name, "<a temp file>");
@@ -146,17 +182,17 @@ std::string TestFileRef::TestGetName() {
 }
 
 std::string TestFileRef::TestGetPath() {
-  pp::FileSystem_Dev file_system_pers(
+  pp::FileSystem file_system_pers(
       instance_, PP_FILESYSTEMTYPE_LOCALPERSISTENT);
-  pp::FileSystem_Dev file_system_temp(
+  pp::FileSystem file_system_temp(
       instance_, PP_FILESYSTEMTYPE_LOCALTEMPORARY);
 
-  pp::FileRef_Dev file_ref_pers(file_system_pers, kPersFilePath);
+  pp::FileRef file_ref_pers(file_system_pers, kPersFilePath);
   std::string path = file_ref_pers.GetPath().AsString();
   if (path != kPersFilePath)
     return ReportMismatch("FileRef::GetPath", path, kPersFilePath);
 
-  pp::FileRef_Dev file_ref_temp(file_system_temp, kTempFilePath);
+  pp::FileRef file_ref_temp(file_system_temp, kTempFilePath);
   path = file_ref_temp.GetPath().AsString();
   if (path != kTempFilePath)
     return ReportMismatch("FileRef::GetPath", path, kTempFilePath);
@@ -165,10 +201,12 @@ std::string TestFileRef::TestGetPath() {
   request.SetURL("test_url_loader_data/hello.txt");
   request.SetStreamToFile(true);
 
-  TestCompletionCallback callback(instance_->pp_instance());
+  TestCompletionCallback callback(instance_->pp_instance(), force_async_);
 
   pp::URLLoader loader(instance_);
   int32_t rv = loader.Open(request, callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("URLLoader::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
@@ -181,7 +219,7 @@ std::string TestFileRef::TestGetPath() {
   if (status_code != 200)
     return "Unexpected HTTP status code";
 
-  pp::FileRef_Dev file_ref_ext(response_info.GetBodyAsFileRef());
+  pp::FileRef file_ref_ext(response_info.GetBodyAsFileRef());
   if (!file_ref_ext.GetPath().is_undefined())
     return "The path of an external FileRef should be void.";
 
@@ -189,29 +227,29 @@ std::string TestFileRef::TestGetPath() {
 }
 
 std::string TestFileRef::TestGetParent() {
-  pp::FileSystem_Dev file_system_pers(
+  pp::FileSystem file_system_pers(
       instance_, PP_FILESYSTEMTYPE_LOCALPERSISTENT);
-  pp::FileSystem_Dev file_system_temp(
+  pp::FileSystem file_system_temp(
       instance_, PP_FILESYSTEMTYPE_LOCALTEMPORARY);
 
-  pp::FileRef_Dev file_ref_pers(file_system_pers, kPersFilePath);
+  pp::FileRef file_ref_pers(file_system_pers, kPersFilePath);
   std::string parent_path = file_ref_pers.GetParent().GetPath().AsString();
   if (parent_path != kParentPath)
     return ReportMismatch("FileRef::GetParent", parent_path, kParentPath);
 
-  pp::FileRef_Dev file_ref_temp(file_system_temp, kTempFilePath);
+  pp::FileRef file_ref_temp(file_system_temp, kTempFilePath);
   parent_path = file_ref_temp.GetParent().GetPath().AsString();
   if (parent_path != kParentPath)
     return ReportMismatch("FileRef::GetParent", parent_path, kParentPath);
 
   // Test the "/" case.
-  pp::FileRef_Dev file_ref_slash(file_system_temp, "/");
+  pp::FileRef file_ref_slash(file_system_temp, "/");
   parent_path = file_ref_slash.GetParent().GetPath().AsString();
   if (parent_path != "/")
     return ReportMismatch("FileRef::GetParent", parent_path, "/");
 
   // Test the "/foo" case (the parent is "/").
-  pp::FileRef_Dev file_ref_with_root_parent(file_system_temp, "/foo");
+  pp::FileRef file_ref_with_root_parent(file_system_temp, "/foo");
   parent_path = file_ref_with_root_parent.GetParent().GetPath().AsString();
   if (parent_path != "/")
     return ReportMismatch("FileRef::GetParent", parent_path, "/");
@@ -220,10 +258,12 @@ std::string TestFileRef::TestGetParent() {
   request.SetURL("test_url_loader_data/hello.txt");
   request.SetStreamToFile(true);
 
-  TestCompletionCallback callback(instance_->pp_instance());
+  TestCompletionCallback callback(instance_->pp_instance(), force_async_);
 
   pp::URLLoader loader(instance_);
   int32_t rv = loader.Open(request, callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("URLLoader::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
@@ -236,7 +276,7 @@ std::string TestFileRef::TestGetParent() {
   if (status_code != 200)
     return "Unexpected HTTP status code";
 
-  pp::FileRef_Dev file_ref_ext(response_info.GetBodyAsFileRef());
+  pp::FileRef file_ref_ext(response_info.GetBodyAsFileRef());
   if (!file_ref_ext.GetParent().is_null())
     return "The parent of an external FileRef should be null.";
 
@@ -244,19 +284,23 @@ std::string TestFileRef::TestGetParent() {
 }
 
 std::string TestFileRef::TestMakeDirectory() {
-  TestCompletionCallback callback(instance_->pp_instance());
+  TestCompletionCallback callback(instance_->pp_instance(), force_async_);
 
   // Open.
-  pp::FileSystem_Dev file_system(instance_, PP_FILESYSTEMTYPE_LOCALTEMPORARY);
+  pp::FileSystem file_system(instance_, PP_FILESYSTEMTYPE_LOCALTEMPORARY);
   int32_t rv = file_system.Open(1024, callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileSystem::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
     return ReportError("FileSystem::Open", rv);
 
   // MakeDirectory.
-  pp::FileRef_Dev dir_ref(file_system, "/test_dir_make_directory");
+  pp::FileRef dir_ref(file_system, "/test_dir_make_directory");
   rv = dir_ref.MakeDirectory(callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileSystem::MakeDirectory force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
@@ -264,10 +308,12 @@ std::string TestFileRef::TestMakeDirectory() {
 
   // MakeDirectory aborted.
   callback.reset_run_count();
-  rv = pp::FileRef_Dev(file_system, "/test_dir_make_abort")
+  rv = pp::FileRef(file_system, "/test_dir_make_abort")
       .MakeDirectory(callback);
   if (callback.run_count() > 0)
     return "FileSystem::MakeDirectory ran callback synchronously.";
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileSystem::MakeDirectory force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING) {
     rv = callback.WaitForResult();
     if (rv != PP_ERROR_ABORTED)
@@ -277,8 +323,10 @@ std::string TestFileRef::TestMakeDirectory() {
   }
 
   // MakeDirectoryIncludingAncestors.
-  dir_ref = pp::FileRef_Dev(file_system, "/dir_make_dir_1/dir_make_dir_2");
+  dir_ref = pp::FileRef(file_system, "/dir_make_dir_1/dir_make_dir_2");
   rv = dir_ref.MakeDirectoryIncludingAncestors(callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileSystem::MakeDirectory force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
@@ -286,12 +334,15 @@ std::string TestFileRef::TestMakeDirectory() {
 
   // MakeDirectoryIncludingAncestors aborted.
   callback.reset_run_count();
-  rv = pp::FileRef_Dev(file_system, "/dir_make_abort_1/dir_make_abort_2")
+  rv = pp::FileRef(file_system, "/dir_make_abort_1/dir_make_abort_2")
       .MakeDirectoryIncludingAncestors(callback);
   if (callback.run_count() > 0) {
     return "FileSystem::MakeDirectoryIncludingAncestors "
            "ran callback synchronously.";
   }
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError(
+        "FileSystem::MakeDirectoryIncludingAncestors force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING) {
     rv = callback.WaitForResult();
     if (rv != PP_ERROR_ABORTED)
@@ -301,8 +352,10 @@ std::string TestFileRef::TestMakeDirectory() {
   }
 
   // MakeDirectory with nested path.
-  dir_ref = pp::FileRef_Dev(file_system, "/dir_make_dir_3/dir_make_dir_4");
+  dir_ref = pp::FileRef(file_system, "/dir_make_dir_3/dir_make_dir_4");
   rv = dir_ref.MakeDirectory(callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileSystem::MakeDirectory force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv == PP_OK) {
@@ -314,21 +367,25 @@ std::string TestFileRef::TestMakeDirectory() {
 }
 
 std::string TestFileRef::TestQueryAndTouchFile() {
-  TestCompletionCallback callback(instance_->pp_instance());
-  pp::FileSystem_Dev file_system(instance_, PP_FILESYSTEMTYPE_LOCALTEMPORARY);
+  TestCompletionCallback callback(instance_->pp_instance(), force_async_);
+  pp::FileSystem file_system(instance_, PP_FILESYSTEMTYPE_LOCALTEMPORARY);
   int32_t rv = file_system.Open(1024, callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileSystem::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
     return ReportError("FileSystem::Open", rv);
 
-  pp::FileRef_Dev file_ref(file_system, "/file_touch");
-  pp::FileIO_Dev file_io(instance_);
+  pp::FileRef file_ref(file_system, "/file_touch");
+  pp::FileIO file_io(instance_);
   rv = file_io.Open(file_ref,
                     PP_FILEOPENFLAG_CREATE |
                     PP_FILEOPENFLAG_TRUNCATE |
                     PP_FILEOPENFLAG_WRITE,
                     callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileIO::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
@@ -336,6 +393,8 @@ std::string TestFileRef::TestQueryAndTouchFile() {
 
   // Write some data to have a non-zero file size.
   rv = file_io.Write(0, "test", 4, callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileIO::Write force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != 4)
@@ -347,6 +406,8 @@ std::string TestFileRef::TestQueryAndTouchFile() {
   const PP_Time last_access_time = 123 * 24 * 3600.0;
   const PP_Time last_modified_time = 246.0;
   rv = file_ref.Touch(last_access_time, last_modified_time, callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileSystem::Touch force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
@@ -354,10 +415,12 @@ std::string TestFileRef::TestQueryAndTouchFile() {
 
   // Touch aborted.
   callback.reset_run_count();
-  rv = pp::FileRef_Dev(file_system, "/file_touch_abort")
+  rv = pp::FileRef(file_system, "/file_touch_abort")
       .Touch(last_access_time, last_modified_time, callback);
   if (callback.run_count() > 0)
     return "FileSystem::Touch ran callback synchronously.";
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileSystem::Touch force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING) {
     rv = callback.WaitForResult();
     if (rv != PP_ERROR_ABORTED)
@@ -367,8 +430,10 @@ std::string TestFileRef::TestQueryAndTouchFile() {
   }
 
   // Query.
-  PP_FileInfo_Dev info;
+  PP_FileInfo info;
   rv = file_io.Query(&info, callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileSystem::Query force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
@@ -385,10 +450,12 @@ std::string TestFileRef::TestQueryAndTouchFile() {
   // TODO(viettrungluu): this test causes a bunch of LOG(WARNING)s; investigate.
   callback.reset_run_count();
   // TODO(viettrungluu): check |info| for late writes.
-  rv = pp::FileRef_Dev(file_system, "/file_touch").Touch(
+  rv = pp::FileRef(file_system, "/file_touch").Touch(
       last_access_time, last_modified_time, callback);
   if (callback.run_count() > 0)
     return "FileSystem::Touch ran callback synchronously.";
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileSystem::Touch force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING) {
     rv = callback.WaitForResult();
     if (rv != PP_ERROR_ABORTED)
@@ -401,68 +468,86 @@ std::string TestFileRef::TestQueryAndTouchFile() {
 }
 
 std::string TestFileRef::TestDeleteFileAndDirectory() {
-  TestCompletionCallback callback(instance_->pp_instance());
-  pp::FileSystem_Dev file_system(instance_, PP_FILESYSTEMTYPE_LOCALTEMPORARY);
+  TestCompletionCallback callback(instance_->pp_instance(), force_async_);
+  pp::FileSystem file_system(instance_, PP_FILESYSTEMTYPE_LOCALTEMPORARY);
   int32_t rv = file_system.Open(1024, callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileSystem::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
     return ReportError("FileSystem::Open", rv);
 
-  pp::FileRef_Dev file_ref(file_system, "/file_delete");
-  pp::FileIO_Dev file_io(instance_);
+  pp::FileRef file_ref(file_system, "/file_delete");
+  pp::FileIO file_io(instance_);
   rv = file_io.Open(file_ref, PP_FILEOPENFLAG_CREATE, callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileIO::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
     return ReportError("FileIO::Open", rv);
 
   rv = file_ref.Delete(callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileRef::Delete force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
-    return ReportError("FileSystem::Delete", rv);
+    return ReportError("FileRef::Delete", rv);
 
-  pp::FileRef_Dev dir_ref(file_system, "/dir_delete");
+  pp::FileRef dir_ref(file_system, "/dir_delete");
   rv = dir_ref.MakeDirectory(callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileRef::MakeDirectory force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
-    return ReportError("FileSystem::MakeDirectory", rv);
+    return ReportError("FileRef::MakeDirectory", rv);
 
   rv = dir_ref.Delete(callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileRef::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
-    return ReportError("FileSystem::Delete", rv);
+    return ReportError("FileRef::Delete", rv);
 
-  pp::FileRef_Dev nested_dir_ref(file_system, "/dir_delete_1/dir_delete_2");
+  pp::FileRef nested_dir_ref(file_system, "/dir_delete_1/dir_delete_2");
   rv = nested_dir_ref.MakeDirectoryIncludingAncestors(callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileRef::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
-    return ReportError("FileSystem::MakeDirectoryIncludingAncestors", rv);
+    return ReportError("FileRef::MakeDirectoryIncludingAncestors", rv);
 
   // Hang on to a ref to the parent; otherwise the callback will be aborted.
-  pp::FileRef_Dev parent_dir_ref = nested_dir_ref.GetParent();
+  pp::FileRef parent_dir_ref = nested_dir_ref.GetParent();
   rv = parent_dir_ref.Delete(callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileRef::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_ERROR_FAILED)
-    return ReportError("FileSystem::Delete", rv);
+    return ReportError("FileRef::Delete", rv);
 
-  pp::FileRef_Dev nonexistent_file_ref(file_system, "/nonexistent_file_delete");
+  pp::FileRef nonexistent_file_ref(file_system, "/nonexistent_file_delete");
   rv = nonexistent_file_ref.Delete(callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileRef::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_ERROR_FILENOTFOUND)
-    return ReportError("FileSystem::Delete", rv);
+    return ReportError("FileRef::Delete", rv);
 
   // Delete aborted.
   {
-    pp::FileRef_Dev file_ref_abort(file_system, "/file_delete_abort");
-    pp::FileIO_Dev file_io_abort(instance_);
+    pp::FileRef file_ref_abort(file_system, "/file_delete_abort");
+    pp::FileIO file_io_abort(instance_);
     rv = file_io_abort.Open(file_ref_abort, PP_FILEOPENFLAG_CREATE, callback);
+    if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+      return ReportError("FileIO::Open force_async", rv);
     if (rv == PP_OK_COMPLETIONPENDING)
       rv = callback.WaitForResult();
     if (rv != PP_OK)
@@ -472,79 +557,97 @@ std::string TestFileRef::TestDeleteFileAndDirectory() {
     rv = file_ref_abort.Delete(callback);
   }
   if (callback.run_count() > 0)
-    return "FileSystem::Delete ran callback synchronously.";
+    return "FileRef::Delete ran callback synchronously.";
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileRef::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING) {
     rv = callback.WaitForResult();
     if (rv != PP_ERROR_ABORTED)
-      return "FileSystem::Delete not aborted.";
+      return "FileRef::Delete not aborted.";
   } else if (rv != PP_OK) {
-    return ReportError("FileSystem::Delete", rv);
+    return ReportError("FileRef::Delete", rv);
   }
 
   PASS();
 }
 
 std::string TestFileRef::TestRenameFileAndDirectory() {
-  TestCompletionCallback callback(instance_->pp_instance());
-  pp::FileSystem_Dev file_system(instance_, PP_FILESYSTEMTYPE_LOCALTEMPORARY);
+  TestCompletionCallback callback(instance_->pp_instance(), force_async_);
+  pp::FileSystem file_system(instance_, PP_FILESYSTEMTYPE_LOCALTEMPORARY);
   int32_t rv = file_system.Open(1024, callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileSystem::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
     return ReportError("FileSystem::Open", rv);
 
-  pp::FileRef_Dev file_ref(file_system, "/file_rename");
-  pp::FileIO_Dev file_io(instance_);
+  pp::FileRef file_ref(file_system, "/file_rename");
+  pp::FileIO file_io(instance_);
   rv = file_io.Open(file_ref, PP_FILEOPENFLAG_CREATE, callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileIO::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
     return ReportError("FileIO::Open", rv);
 
-  pp::FileRef_Dev target_file_ref(file_system, "/target_file_rename");
+  pp::FileRef target_file_ref(file_system, "/target_file_rename");
   rv = file_ref.Rename(target_file_ref, callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileRef::Rename force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
-    return ReportError("FileSystem::Rename", rv);
+    return ReportError("FileRef::Rename", rv);
 
-  pp::FileRef_Dev dir_ref(file_system, "/dir_rename");
+  pp::FileRef dir_ref(file_system, "/dir_rename");
   rv = dir_ref.MakeDirectory(callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileRef::MakeDirectory force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
-    return ReportError("FileSystem::MakeDirectory", rv);
+    return ReportError("FileRef::MakeDirectory", rv);
 
-  pp::FileRef_Dev target_dir_ref(file_system, "/target_dir_rename");
+  pp::FileRef target_dir_ref(file_system, "/target_dir_rename");
   rv = dir_ref.Rename(target_dir_ref, callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileRef::Rename force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
-    return ReportError("FileSystem::Rename", rv);
+    return ReportError("FileRef::Rename", rv);
 
-  pp::FileRef_Dev nested_dir_ref(file_system, "/dir_rename_1/dir_rename_2");
+  pp::FileRef nested_dir_ref(file_system, "/dir_rename_1/dir_rename_2");
   rv = nested_dir_ref.MakeDirectoryIncludingAncestors(callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileRef::MakeDirectory force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_OK)
-    return ReportError("FileSystem::MakeDirectoryIncludingAncestors", rv);
+    return ReportError("FileRef::MakeDirectoryIncludingAncestors", rv);
 
-  pp::FileRef_Dev target_nested_dir_ref(file_system, "/dir_rename_1");
+  pp::FileRef target_nested_dir_ref(file_system, "/dir_rename_1");
   rv = nested_dir_ref.Rename(target_nested_dir_ref, callback);
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileRef::Open force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING)
     rv = callback.WaitForResult();
   if (rv != PP_ERROR_FAILED)
-    return ReportError("FileSystem::Rename", rv);
+    return ReportError("FileRef::Rename", rv);
 
   // Rename aborted.
   // TODO(viettrungluu): Figure out what we want to do if the target file
   // resource is destroyed before completion.
-  pp::FileRef_Dev target_file_ref_abort(file_system,
+  pp::FileRef target_file_ref_abort(file_system,
                                         "/target_file_rename_abort");
   {
-    pp::FileRef_Dev file_ref_abort(file_system, "/file_rename_abort");
-    pp::FileIO_Dev file_io_abort(instance_);
+    pp::FileRef file_ref_abort(file_system, "/file_rename_abort");
+    pp::FileIO file_io_abort(instance_);
     rv = file_io_abort.Open(file_ref_abort, PP_FILEOPENFLAG_CREATE, callback);
+    if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+      return ReportError("FileIO::Open force_async", rv);
     if (rv == PP_OK_COMPLETIONPENDING)
       rv = callback.WaitForResult();
     if (rv != PP_OK)
@@ -555,6 +658,8 @@ std::string TestFileRef::TestRenameFileAndDirectory() {
   }
   if (callback.run_count() > 0)
     return "FileSystem::Rename ran callback synchronously.";
+  if (force_async_ && rv != PP_OK_COMPLETIONPENDING)
+    return ReportError("FileSystem::Rename force_async", rv);
   if (rv == PP_OK_COMPLETIONPENDING) {
     rv = callback.WaitForResult();
     if (rv != PP_ERROR_ABORTED)
