@@ -6,11 +6,15 @@
 #define PPAPI_THUNK_URL_REQUEST_INFO_API_H_
 
 #include "ppapi/c/ppb_url_request_info.h"
+#include "ppapi/thunk/ppapi_thunk_export.h"
 
 namespace ppapi {
+
+struct PPB_URLRequestInfo_Data;
+
 namespace thunk {
 
-class PPB_URLRequestInfo_API {
+class PPAPI_THUNK_EXPORT PPB_URLRequestInfo_API {
  public:
   virtual ~PPB_URLRequestInfo_API() {}
 
@@ -21,6 +25,9 @@ class PPB_URLRequestInfo_API {
                                    int64_t start_offset,
                                    int64_t number_of_bytes,
                                    PP_Time expected_last_modified_time) = 0;
+
+  // Internal-only function for retrieving the current config.
+  virtual const PPB_URLRequestInfo_Data& GetData() const = 0;
 };
 
 }  // namespace thunk
