@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -8,6 +8,8 @@
 
 #include <cups/cups.h>
 
+#include "printing/printing_export.h"
+
 class GURL;
 
 // These are helper functions for dealing with CUPS.
@@ -15,7 +17,7 @@ namespace printing {
 
 // Helper wrapper around http_t structure, with connection and cleanup
 // functionality.
-class HttpConnectionCUPS {
+class PRINTING_EXPORT HttpConnectionCUPS {
  public:
   explicit HttpConnectionCUPS(const GURL& print_server_url);
   ~HttpConnectionCUPS();
