@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -15,7 +15,7 @@ namespace printing {
 class PrintSettings;
 
 // Initializes a PrintSettings object from the provided device context.
-class PrintSettingsInitializerMac {
+class PRINTING_EXPORT PrintSettingsInitializerMac {
  public:
   static void InitPrintSettings(PMPrinter printer,
                                 PMPageFormat page_format,
