@@ -18,7 +18,7 @@ namespace printing {
 class Metafile;
 class PrintDialogGtkInterface;
 
-class PrintingContextCairo : public PrintingContext {
+class PRINTING_EXPORT PrintingContextCairo : public PrintingContext {
  public:
   explicit PrintingContextCairo(const std::string& app_locale);
   virtual ~PrintingContextCairo();
@@ -39,8 +39,9 @@ class PrintingContextCairo : public PrintingContext {
                                   bool has_selection,
                                   PrintSettingsCallback* callback);
   virtual Result UseDefaultSettings();
-  virtual Result UpdatePrintSettings(const base::DictionaryValue& job_settings,
-                                     const PageRanges& ranges);
+  virtual Result UpdatePrinterSettings(
+      const base::DictionaryValue& job_settings,
+      const PageRanges& ranges);
   virtual Result InitWithSettings(const PrintSettings& settings);
   virtual Result NewDocument(const string16& document_name);
   virtual Result NewPage();
