@@ -34,22 +34,20 @@ void InputHandler::SendKeyEvent(bool press, int keycode) {
       pressed_keys_.erase(keycode);
     }
 
-    KeyEvent* event = new KeyEvent();
-    event->set_keycode(keycode);
-    event->set_pressed(press);
-
-    stub->InjectKeyEvent(event, new DeleteTask<KeyEvent>(event));
+    KeyEvent event;
+    event.set_keycode(keycode);
+    event.set_pressed(press);
+    stub->InjectKeyEvent(event);
   }
 }
 
 void InputHandler::SendMouseMoveEvent(int x, int y) {
   protocol::InputStub* stub = connection_->input_stub();
   if (stub) {
-    MouseEvent* event = new MouseEvent();
-    event->set_x(x);
-    event->set_y(y);
-
-    stub->InjectMouseEvent(event, new DeleteTask<MouseEvent>(event));
+    MouseEvent event;
+    event.set_x(x);
+    event.set_y(y);
+    stub->InjectMouseEvent(event);
   }
 }
 
@@ -57,11 +55,10 @@ void InputHandler::SendMouseButtonEvent(bool button_down,
                                         MouseEvent::MouseButton button) {
   protocol::InputStub* stub = connection_->input_stub();
   if (stub) {
-    MouseEvent* event = new MouseEvent();
-    event->set_button(button);
-    event->set_button_down(button_down);
-
-    stub->InjectMouseEvent(event, new DeleteTask<MouseEvent>(event));
+    MouseEvent event;
+    event.set_button(button);
+    event.set_button_down(button_down);
+    stub->InjectMouseEvent(event);
   }
 }
 
