@@ -21,11 +21,9 @@
 #include "ppapi/cpp/private/instance_private.h"
 #include "remoting/client/client_context.h"
 #include "remoting/client/plugin/chromoting_scriptable_object.h"
-#include "remoting/client/plugin/pepper_client_logger.h"
+#include "remoting/client/plugin/pepper_plugin_thread_delegate.h"
 #include "remoting/protocol/connection_to_host.h"
 
-class MessageLoop;
-
 namespace base {
 class Thread;
 }  // namespace base
@@ -49,6 +47,7 @@ class JingleThread;
 class PepperView;
 class PepperViewProxy;
 class RectangleUpdateDecoder;
+class TaskThreadProxy;
 
 struct ClientConfig;
 
@@ -88,9 +87,6 @@ class ChromotingInstance : public pp::InstancePrivate {
   // Called by ChromotingScriptableObject to set scale-to-fit.
   void SetScaleToFit(bool scale_to_fit);
 
-  void Log(int severity, const char* format, ...);
-  void VLog(int verboselevel, const char* format, ...);
-
   // Return statistics record by ChromotingClient.
   // If no connection is currently active then NULL will be returned.
   ChromotingStats* GetStats();
@@ -99,6 +95,27 @@ class ChromotingInstance : public pp::InstancePrivate {
 
   bool DoScaling() const { return scale_to_fit_; }
 
+  // Registers a global log message handler that redirects the log output to
+  // our plugin instance.
+  // This is called by the plugin's PPP_InitializeModule.
+  // Note that no logging will be processed unless a ChromotingInstance has been
+  // registered for logging (see RegisterLoggingInstance).
+  static void RegisterLogMessageHandler();
+
+  // Registers this instance so it processes messages sent by the global log
+  // message handler. This overwrites any previously registered instance.
+  void RegisterLoggingInstance();
+
+  // Unregisters this instance so that debug log messages will no longer be sent
+  // to it. If this instance is not the currently registered logging instance,
+  // then the currently registered instance will stay in effect.
+  void UnregisterLoggingInstance();
+
+  // A Log Message Handler that is called after each LOG message has been
+  // processed. This must be of type LogMessageHandlerFunction defined in
+  // base/logging.h.
+  static bool LogToUI(int severity, const char* file, int line,
+                      size_t message_start, const std::string& str);
  private:
   FRIEND_TEST_ALL_PREFIXES(ChromotingInstanceTest, TestCaseSetup);
 
@@ -109,8 +126,12 @@ class ChromotingInstance : public pp::InstancePrivate {
   bool IsNatTraversalAllowed(const std::string& policy_json);
   void HandlePolicyUpdate(const std::string policy_json);
 
+  void ProcessLogToUI(const std::string& message);
+
   bool initialized_;
 
+  PepperPluginThreadDelegate plugin_thread_delegate_;
+  scoped_refptr<PluginMessageLoopProxy> plugin_message_loop_;
   ClientContext context_;
   scoped_ptr<protocol::ConnectionToHost> host_connection_;
   scoped_ptr<PepperView> view_;
@@ -118,13 +139,16 @@ class ChromotingInstance : public pp::InstancePrivate {
   // True if scale to fit is enabled.
   bool scale_to_fit_;
 
+  // A refcounted class to perform thread-switching for logging tasks.
+  scoped_refptr<TaskThreadProxy> log_proxy_;
+
   // PepperViewProxy is refcounted and used to interface between chromoting
   // objects and PepperView and perform thread switching. It wraps around
   // |view_| and receives method calls on chromoting threads. These method
   // calls are then delegates on the pepper thread. During destruction of
   // ChromotingInstance we need to detach PepperViewProxy from PepperView since
   // both ChromotingInstance and PepperView are destroyed and there will be
-  // outstanding tasks on the pepper message loo.
+  // outstanding tasks on the pepper message loop.
   scoped_refptr<PepperViewProxy> view_proxy_;
   scoped_refptr<RectangleUpdateDecoder> rectangle_decoder_;
   scoped_ptr<InputHandler> input_handler_;
@@ -136,8 +160,6 @@ class ChromotingInstance : public pp::InstancePrivate {
   // connection.
   scoped_refptr<PepperXmppProxy> xmpp_proxy_;
 
-  PepperClientLogger logger_;
-
   // JavaScript interface to control this instance.
   // This wraps a ChromotingScriptableObject in a pp::Var.
   pp::Var instance_object_;
