@@ -115,11 +115,15 @@
 #include <string>
 #include <vector>
 
+#include "base/task.h"
 #include "base/memory/weak_ptr.h"
-
 #include "ppapi/cpp/dev/scriptable_object_deprecated.h"
 #include "ppapi/cpp/var.h"
 
+namespace base {
+class MessageLoopProxy;
+};  // namespace base
+
 namespace remoting {
 
 class ChromotingInstance;
@@ -144,7 +148,9 @@ class ChromotingScriptableObject
     : public pp::deprecated::ScriptableObject,
       public base::SupportsWeakPtr<ChromotingScriptableObject> {
  public:
-  explicit ChromotingScriptableObject(ChromotingInstance* instance);
+  ChromotingScriptableObject(
+      ChromotingInstance* instance,
+      base::MessageLoopProxy* plugin_message_loop);
   virtual ~ChromotingScriptableObject();
 
   virtual void Init();
@@ -182,11 +188,11 @@ class ChromotingScriptableObject
   typedef pp::Var (ChromotingScriptableObject::*MethodHandler)(
       const std::vector<pp::Var>& args, pp::Var* exception);
   struct PropertyDescriptor {
-    explicit PropertyDescriptor(const std::string& n, pp::Var a)
+    PropertyDescriptor(const std::string& n, pp::Var a)
         : type(NONE), name(n), attribute(a), method(NULL) {
     }
 
-    explicit PropertyDescriptor(const std::string& n, MethodHandler m)
+    PropertyDescriptor(const std::string& n, MethodHandler m)
         : type(NONE), name(n), method(m) {
     }
 
@@ -205,13 +211,17 @@ class ChromotingScriptableObject
   void AddAttribute(const std::string& name, pp::Var attribute);
   void AddMethod(const std::string& name, MethodHandler handler);
 
-  // This should be called to signal the JS code that the connection status has
-  // changed.
   void SignalConnectionInfoChange();
-
-  // Signal the JS code that the desktop size has changed.
   void SignalDesktopSizeChange();
 
+  // Calls to these methods are posted to the plugin thread so that we
+  // call JavaScript with clean stack. This is necessary because
+  // JavaScript event handlers may destroy the plugin.
+  void DoSignalConnectionInfoChange();
+  void DoSignalDesktopSizeChange();
+  void DoSignalLoginChallenge();
+  void DoSendIq(const std::string& message_xml);
+
   pp::Var DoConnect(const std::vector<pp::Var>& args, pp::Var* exception);
   pp::Var DoDisconnect(const std::vector<pp::Var>& args, pp::Var* exception);
 
@@ -235,6 +245,11 @@ class ChromotingScriptableObject
   scoped_refptr<PepperXmppProxy> xmpp_proxy_;
 
   ChromotingInstance* instance_;
+
+  scoped_refptr<base::MessageLoopProxy> plugin_message_loop_;
+  ScopedRunnableMethodFactory<ChromotingScriptableObject> task_factory_;
+
+  DISALLOW_COPY_AND_ASSIGN(ChromotingScriptableObject);
 };
 
 }  // namespace remoting
