@@ -5,20 +5,39 @@
 #include "remoting/host/capturer.h"
 
 #include <ApplicationServices/ApplicationServices.h>
+#include <dlfcn.h>
 #include <OpenGL/CGLMacro.h>
 #include <OpenGL/OpenGL.h>
-
 #include <stddef.h>
 
 #include "base/logging.h"
 #include "base/mac/mac_util.h"
+#include "base/mac/scoped_cftyperef.h"
 #include "base/memory/scoped_ptr.h"
+#include "base/synchronization/waitable_event.h"
+#include "remoting/base/util.h"
 #include "remoting/host/capturer_helper.h"
+#include "skia/ext/skia_utils_mac.h"
+
 
 namespace remoting {
 
 namespace {
 
+#if (MAC_OS_X_VERSION_MIN_REQUIRED > MAC_OS_X_VERSION_10_5)
+// Possibly remove CapturerMac::CgBlitPreLion as well depending on performance
+// of CapturerMac::CgBlitPostLion on 10.6.
+#error No longer need to import CGDisplayCreateImage.
+#else
+// Declared here because CGDisplayCreateImage does not exist in the 10.5 SDK,
+// which we are currently compiling against, and it is required on 10.7 to do
+// screen capture.
+typedef CGImageRef (*CGDisplayCreateImageFunc)(CGDirectDisplayID displayID);
+#endif
+
+// The amount of time allowed for displays to reconfigure.
+const int64 kDisplayReconfigurationTimeoutInSeconds = 10;
+
 class scoped_pixel_buffer_object {
  public:
   scoped_pixel_buffer_object();
@@ -125,32 +144,33 @@ class CapturerMac : public Capturer {
   CapturerMac();
   virtual ~CapturerMac();
 
-  // Enable or disable capturing. Capturing should be disabled while a screen
-  // reconfiguration is in progress, otherwise reading from the screen base
-  // address is likely to segfault.
-  void EnableCapture(bool enable);
+  bool Init();
 
   // Capturer interface.
   virtual void ScreenConfigurationChanged() OVERRIDE;
   virtual media::VideoFrame::Format pixel_format() const OVERRIDE;
-  virtual void ClearInvalidRects() OVERRIDE;
-  virtual void InvalidateRects(const InvalidRects& inval_rects) OVERRIDE;
+  virtual void ClearInvalidRegion() OVERRIDE;
+  virtual void InvalidateRegion(const SkRegion& invalid_region) OVERRIDE;
   virtual void InvalidateScreen(const gfx::Size& size) OVERRIDE;
   virtual void InvalidateFullScreen() OVERRIDE;
-  virtual void CaptureInvalidRects(CaptureCompletedCallback* callback) OVERRIDE;
+  virtual void CaptureInvalidRegion(CaptureCompletedCallback* callback)
+      OVERRIDE;
   virtual const gfx::Size& size_most_recent() const OVERRIDE;
 
  private:
-  void GlBlitFast(const VideoFrameBuffer& buffer);
+  void GlBlitFast(const VideoFrameBuffer& buffer, const SkRegion& region);
   void GlBlitSlow(const VideoFrameBuffer& buffer);
-  void CgBlit(const VideoFrameBuffer& buffer, const InvalidRects& rects);
-  void CaptureRects(const InvalidRects& rects,
-                    CaptureCompletedCallback* callback);
+  void CgBlitPreLion(const VideoFrameBuffer& buffer, const SkRegion& region);
+  void CgBlitPostLion(const VideoFrameBuffer& buffer, const SkRegion& region);
+  void CaptureRegion(const SkRegion& region,
+                     CaptureCompletedCallback* callback);
 
   void ScreenRefresh(CGRectCount count, const CGRect *rect_array);
   void ScreenUpdateMove(CGScreenUpdateMoveDelta delta,
                                 size_t count,
                                 const CGRect *rect_array);
+  void DisplaysReconfigured(CGDirectDisplayID display,
+                            CGDisplayChangeSummaryFlags flags);
   static void ScreenRefreshCallback(CGRectCount count,
                                     const CGRect *rect_array,
                                     void *user_parameter);
@@ -175,14 +195,22 @@ class CapturerMac : public Capturer {
   // The current buffer with valid data for reading.
   int current_buffer_;
 
-  // The last buffer into which we captured, or NULL for the first capture for
-  // a particular screen resolution.
+  // The previous buffer into which we captured, or NULL for the first capture
+  // for a particular screen resolution.
   uint8* last_buffer_;
 
+  // Contains an invalid region from the previous capture.
+  SkRegion last_invalid_region_;
+
   // Format of pixels returned in buffer.
   media::VideoFrame::Format pixel_format_;
 
-  bool capturing_;
+  // Acts as a critical section around our display configuration data
+  // structures. Specifically cgl_context_ and pixel_buffer_object_.
+  base::WaitableEvent display_configuration_capture_event_;
+
+  // Will be non-null on lion.
+  CGDisplayCreateImageFunc display_create_image_func_;
 
   DISALLOW_COPY_AND_ASSIGN(CapturerMac);
 };
@@ -192,20 +220,8 @@ CapturerMac::CapturerMac()
       current_buffer_(0),
       last_buffer_(NULL),
       pixel_format_(media::VideoFrame::RGB32),
-      capturing_(true) {
-  // TODO(dmaclach): move this initialization out into session_manager,
-  // or at least have session_manager call into here to initialize it.
-  CGError err =
-      CGRegisterScreenRefreshCallback(CapturerMac::ScreenRefreshCallback,
-                                      this);
-  DCHECK_EQ(err, kCGErrorSuccess);
-  err = CGScreenRegisterMoveCallback(CapturerMac::ScreenUpdateMoveCallback,
-                                     this);
-  DCHECK_EQ(err, kCGErrorSuccess);
-  err = CGDisplayRegisterReconfigurationCallback(
-      CapturerMac::DisplaysReconfiguredCallback, this);
-  DCHECK_EQ(err, kCGErrorSuccess);
-  ScreenConfigurationChanged();
+      display_configuration_capture_event_(false, true),
+      display_create_image_func_(NULL) {
 }
 
 CapturerMac::~CapturerMac() {
@@ -216,8 +232,39 @@ CapturerMac::~CapturerMac() {
       CapturerMac::DisplaysReconfiguredCallback, this);
 }
 
-void CapturerMac::EnableCapture(bool enable) {
-  capturing_ = enable;
+bool CapturerMac::Init() {
+  CGError err =
+      CGRegisterScreenRefreshCallback(CapturerMac::ScreenRefreshCallback,
+                                      this);
+  if (err != kCGErrorSuccess) {
+    LOG(ERROR) << "CGRegisterScreenRefreshCallback " << err;
+    return false;
+  }
+
+  err = CGScreenRegisterMoveCallback(CapturerMac::ScreenUpdateMoveCallback,
+                                     this);
+  if (err != kCGErrorSuccess) {
+    LOG(ERROR) << "CGScreenRegisterMoveCallback " << err;
+    return false;
+  }
+  err = CGDisplayRegisterReconfigurationCallback(
+      CapturerMac::DisplaysReconfiguredCallback, this);
+  if (err != kCGErrorSuccess) {
+    LOG(ERROR) << "CGDisplayRegisterReconfigurationCallback " << err;
+    return false;
+  }
+
+  if (base::mac::IsOSLionOrLater()) {
+    display_create_image_func_ =
+        reinterpret_cast<CGDisplayCreateImageFunc>(
+            dlsym(RTLD_NEXT, "CGDisplayCreateImage"));
+    if (!display_create_image_func_) {
+      LOG(ERROR) << "Unable to load CGDisplayCreateImage on Lion";
+      return false;
+    }
+  }
+  ScreenConfigurationChanged();
+  return true;
 }
 
 void CapturerMac::ReleaseBuffers() {
@@ -236,8 +283,7 @@ void CapturerMac::ReleaseBuffers() {
 
 void CapturerMac::ScreenConfigurationChanged() {
   ReleaseBuffers();
-  InvalidRects rects;
-  helper_.SwapInvalidRects(rects);
+  helper_.ClearInvalidRegion();
   last_buffer_ = NULL;
 
   CGDirectDisplayID mainDevice = CGMainDisplayID();
@@ -245,11 +291,16 @@ void CapturerMac::ScreenConfigurationChanged() {
   int height = CGDisplayPixelsHigh(mainDevice);
   InvalidateScreen(gfx::Size(width, height));
 
-  if (CGDisplayIsBuiltin(mainDevice)) {
+  if (!CGDisplayUsesOpenGLAcceleration(mainDevice)) {
     VLOG(3) << "OpenGL support not available.";
     return;
   }
 
+  if (display_create_image_func_ != NULL) {
+    // No need for any OpenGL support on Lion
+    return;
+  }
+
   CGLPixelFormatAttribute attributes[] = {
     kCGLPFAFullScreen,
     kCGLPFADisplayMask,
@@ -276,12 +327,12 @@ media::VideoFrame::Format CapturerMac::pixel_format() const {
   return pixel_format_;
 }
 
-void CapturerMac::ClearInvalidRects() {
-  helper_.ClearInvalidRects();
+void CapturerMac::ClearInvalidRegion() {
+  helper_.ClearInvalidRegion();
 }
 
-void CapturerMac::InvalidateRects(const InvalidRects& inval_rects) {
-  helper_.InvalidateRects(inval_rects);
+void CapturerMac::InvalidateRegion(const SkRegion& invalid_region) {
+  helper_.InvalidateRegion(invalid_region);
 }
 
 void CapturerMac::InvalidateScreen(const gfx::Size& size) {
@@ -292,54 +343,91 @@ void CapturerMac::InvalidateFullScreen() {
   helper_.InvalidateFullScreen();
 }
 
-void CapturerMac::CaptureInvalidRects(CaptureCompletedCallback* callback) {
+void CapturerMac::CaptureInvalidRegion(CaptureCompletedCallback* callback) {
+  // Only allow captures when the display configuration is not occurring.
   scoped_refptr<CaptureData> data;
-  if (capturing_) {
-    InvalidRects rects;
-    helper_.SwapInvalidRects(rects);
-    VideoFrameBuffer& current_buffer = buffers_[current_buffer_];
-    current_buffer.Update();
 
-    bool flip = true;  // GL capturers need flipping.
-    if (cgl_context_) {
-      if (pixel_buffer_object_.get() != 0) {
-        GlBlitFast(current_buffer);
-      } else {
-        // See comment in scoped_pixel_buffer_object::Init about why the slow
-        // path is always used on 10.5.
-        GlBlitSlow(current_buffer);
-      }
+  // Critical section shared with DisplaysReconfigured(...).
+  CHECK(display_configuration_capture_event_.TimedWait(
+      base::TimeDelta::FromSeconds(kDisplayReconfigurationTimeoutInSeconds)));
+  SkRegion region;
+  helper_.SwapInvalidRegion(&region);
+  VideoFrameBuffer& current_buffer = buffers_[current_buffer_];
+  current_buffer.Update();
+
+  bool flip = false;  // GL capturers need flipping.
+  if (display_create_image_func_ != NULL) {
+    // Lion requires us to use their new APIs for doing screen capture.
+    CgBlitPostLion(current_buffer, region);
+  } else if (cgl_context_) {
+    flip = true;
+    if (pixel_buffer_object_.get() != 0) {
+      GlBlitFast(current_buffer, region);
     } else {
-      CgBlit(current_buffer, rects);
-      flip = false;
+      // See comment in scoped_pixel_buffer_object::Init about why the slow
+      // path is always used on 10.5.
+      GlBlitSlow(current_buffer);
     }
-
-    DataPlanes planes;
-    planes.data[0] = current_buffer.ptr();
-    planes.strides[0] = current_buffer.bytes_per_row();
-    if (flip) {
-      planes.strides[0] = -planes.strides[0];
-      planes.data[0] +=
-          (current_buffer.size().height() - 1) * current_buffer.bytes_per_row();
-    }
-
-    data = new CaptureData(planes, gfx::Size(current_buffer.size()),
-                           pixel_format());
-    data->mutable_dirty_rects() = rects;
-
-    current_buffer_ = (current_buffer_ + 1) % kNumBuffers;
-    helper_.set_size_most_recent(data->size());
+  } else {
+    CgBlitPreLion(current_buffer, region);
   }
 
+  DataPlanes planes;
+  planes.data[0] = current_buffer.ptr();
+  planes.strides[0] = current_buffer.bytes_per_row();
+  if (flip) {
+    planes.strides[0] = -planes.strides[0];
+    planes.data[0] +=
+        (current_buffer.size().height() - 1) * current_buffer.bytes_per_row();
+  }
+
+  data = new CaptureData(planes, gfx::Size(current_buffer.size()),
+                         pixel_format());
+  data->mutable_dirty_region() = region;
+
+  current_buffer_ = (current_buffer_ + 1) % kNumBuffers;
+  helper_.set_size_most_recent(data->size());
+  display_configuration_capture_event_.Signal();
+
   callback->Run(data);
   delete callback;
 }
 
-void CapturerMac::GlBlitFast(const VideoFrameBuffer& buffer) {
+void CapturerMac::GlBlitFast(const VideoFrameBuffer& buffer,
+           const SkRegion& region) {
+  const int buffer_height = buffer.size().height();
+  const int buffer_width = buffer.size().width();
+
+  // Clip to the size of our current screen.
+  SkIRect clip_rect = SkIRect::MakeWH(buffer_width, buffer_height);
+  if (last_buffer_) {
+    // We are doing double buffer for the capture data so we just need to copy
+    // the invalid region from the previous capture in the current buffer.
+    // TODO(hclam): We can reduce the amount of copying here by subtracting
+    // |capturer_helper_|s region from |last_invalid_region_|.
+    // http://crbug.com/92354
+
+    // Since the image obtained from OpenGL is upside-down, need to do some
+    // magic here to copy the correct rectangle.
+    const int y_offset = (buffer_height - 1) * buffer.bytes_per_row();
+    for(SkRegion::Iterator i(last_invalid_region_); !i.done(); i.next()) {
+      SkIRect copy_rect = i.rect();
+      if (copy_rect.intersect(clip_rect)) {
+        CopyRect(last_buffer_ + y_offset,
+                 -buffer.bytes_per_row(),
+                 buffer.ptr() + y_offset,
+                 -buffer.bytes_per_row(),
+                 4,  // Bytes for pixel for RGBA.
+                 copy_rect);
+      }
+    }
+  }
+  last_buffer_ = buffer.ptr();
+  last_invalid_region_ = region;
+
   CGLContextObj CGL_MACRO_CONTEXT = cgl_context_;
   glBindBufferARB(GL_PIXEL_PACK_BUFFER_ARB, pixel_buffer_object_.get());
-  glReadPixels(0, 0, buffer.size().width(), buffer.size().height(),
-               GL_BGRA, GL_UNSIGNED_BYTE, 0);
+  glReadPixels(0, 0, buffer_width, buffer_height, GL_BGRA, GL_UNSIGNED_BYTE, 0);
   GLubyte* ptr = static_cast<GLubyte*>(
       glMapBufferARB(GL_PIXEL_PACK_BUFFER_ARB, GL_READ_ONLY_ARB));
   if (ptr == NULL) {
@@ -347,7 +435,20 @@ void CapturerMac::GlBlitFast(const VideoFrameBuffer& buffer) {
     // release it.
     pixel_buffer_object_.Release();
   } else {
-    memcpy(buffer.ptr(), ptr, buffer.size().height() * buffer.bytes_per_row());
+    // Copy only from the dirty rects. Since the image obtained from OpenGL is
+    // upside-down we need to do some magic here to copy the correct rectangle.
+    const int y_offset = (buffer_height - 1) * buffer.bytes_per_row();
+    for(SkRegion::Iterator i(region); !i.done(); i.next()) {
+      SkIRect copy_rect = i.rect();
+      if (copy_rect.intersect(clip_rect)) {
+        CopyRect(ptr + y_offset,
+           -buffer.bytes_per_row(),
+           buffer.ptr() + y_offset,
+           -buffer.bytes_per_row(),
+           4,  // Bytes for pixel for RGBA.
+           copy_rect);
+      }
+    }
   }
   if (!glUnmapBufferARB(GL_PIXEL_PACK_BUFFER_ARB)) {
     // If glUnmapBuffer returns false, then the contents of the data store are
@@ -374,26 +475,73 @@ void CapturerMac::GlBlitSlow(const VideoFrameBuffer& buffer) {
   glPopClientAttrib();
 }
 
-void CapturerMac::CgBlit(const VideoFrameBuffer& buffer,
-                         const InvalidRects& rects) {
+void CapturerMac::CgBlitPreLion(const VideoFrameBuffer& buffer,
+                                const SkRegion& region) {
+  const int buffer_height = buffer.size().height();
+  const int buffer_width = buffer.size().width();
+
+    // Clip to the size of our current screen.
+  SkIRect clip_rect = SkIRect::MakeWH(buffer_width, buffer_height);
+
   if (last_buffer_)
-    memcpy(buffer.ptr(), last_buffer_,
-           buffer.bytes_per_row() * buffer.size().height());
+    memcpy(buffer.ptr(), last_buffer_, buffer.bytes_per_row() * buffer_height);
   last_buffer_ = buffer.ptr();
   CGDirectDisplayID main_display = CGMainDisplayID();
   uint8* display_base_address =
       reinterpret_cast<uint8*>(CGDisplayBaseAddress(main_display));
   int src_bytes_per_row = CGDisplayBytesPerRow(main_display);
   int src_bytes_per_pixel = CGDisplayBitsPerPixel(main_display) / 8;
-  for (InvalidRects::iterator i = rects.begin(); i != rects.end(); ++i) {
-    int src_row_offset =  i->x() * src_bytes_per_pixel;
-    int dst_row_offset = i->x() * sizeof(uint32_t);
-    int rect_width_in_bytes = i->width() * src_bytes_per_pixel;
-    int ymax = i->height() + i->y();
-    for (int y = i->y(); y < ymax; ++y) {
-      memcpy(buffer.ptr() + y * buffer.bytes_per_row() + dst_row_offset,
-             display_base_address + y * src_bytes_per_row + src_row_offset,
-             rect_width_in_bytes);
+  // TODO(hclam): We can reduce the amount of copying here by subtracting
+  // |capturer_helper_|s region from |last_invalid_region_|.
+  // http://crbug.com/92354
+  for(SkRegion::Iterator i(region); !i.done(); i.next()) {
+    SkIRect copy_rect = i.rect();
+    if (copy_rect.intersect(clip_rect)) {
+      CopyRect(display_base_address,
+               src_bytes_per_row,
+               buffer.ptr(),
+               buffer.bytes_per_row(),
+               src_bytes_per_pixel,
+               copy_rect);
+    }
+  }
+}
+
+void CapturerMac::CgBlitPostLion(const VideoFrameBuffer& buffer,
+                                 const SkRegion& region) {
+  const int buffer_height = buffer.size().height();
+  const int buffer_width = buffer.size().width();
+
+  // Clip to the size of our current screen.
+  SkIRect clip_rect = SkIRect::MakeWH(buffer_width, buffer_height);
+
+  if (last_buffer_)
+    memcpy(buffer.ptr(), last_buffer_, buffer.bytes_per_row() * buffer_height);
+  last_buffer_ = buffer.ptr();
+  CGDirectDisplayID display = CGMainDisplayID();
+  base::mac::ScopedCFTypeRef<CGImageRef> image(
+      display_create_image_func_(display));
+  if (image.get() == NULL)
+    return;
+  CGDataProviderRef provider = CGImageGetDataProvider(image);
+  base::mac::ScopedCFTypeRef<CFDataRef> data(CGDataProviderCopyData(provider));
+  if (data.get() == NULL)
+    return;
+  const uint8* display_base_address = CFDataGetBytePtr(data);
+  int src_bytes_per_row = CGImageGetBytesPerRow(image);
+  int src_bytes_per_pixel = CGImageGetBitsPerPixel(image) / 8;
+  // TODO(hclam): We can reduce the amount of copying here by subtracting
+  // |capturer_helper_|s region from |last_invalid_region_|.
+  // http://crbug.com/92354
+  for(SkRegion::Iterator i(region); !i.done(); i.next()) {
+    SkIRect copy_rect = i.rect();
+    if (copy_rect.intersect(clip_rect)) {
+      CopyRect(display_base_address,
+               src_bytes_per_row,
+               buffer.ptr(),
+               buffer.bytes_per_row(),
+               src_bytes_per_pixel,
+               copy_rect);
     }
   }
 }
@@ -403,24 +551,45 @@ const gfx::Size& CapturerMac::size_most_recent() const {
 }
 
 void CapturerMac::ScreenRefresh(CGRectCount count, const CGRect *rect_array) {
-  InvalidRects rects;
+  SkIRect skirect_array[count];
   for (CGRectCount i = 0; i < count; ++i) {
-    rects.insert(gfx::Rect(rect_array[i]));
+    skirect_array[i] = gfx::CGRectToSkIRect(rect_array[i]);
   }
-  InvalidateRects(rects);
+  SkRegion region;
+  region.setRects(skirect_array, count);
+  InvalidateRegion(region);
 }
 
 void CapturerMac::ScreenUpdateMove(CGScreenUpdateMoveDelta delta,
                                    size_t count,
                                    const CGRect *rect_array) {
-  InvalidRects rects;
+  SkIRect skirect_new_array[count];
   for (CGRectCount i = 0; i < count; ++i) {
     CGRect rect = rect_array[i];
-    rects.insert(gfx::Rect(rect));
     rect = CGRectOffset(rect, delta.dX, delta.dY);
-    rects.insert(gfx::Rect(rect));
+    skirect_new_array[i] = gfx::CGRectToSkIRect(rect);
+  }
+  SkRegion region;
+  region.setRects(skirect_new_array, count);
+  InvalidateRegion(region);
+}
+
+void CapturerMac::DisplaysReconfigured(CGDirectDisplayID display,
+                                       CGDisplayChangeSummaryFlags flags) {
+  if (display == CGMainDisplayID()) {
+    if (flags & kCGDisplayBeginConfigurationFlag) {
+      // Wait on |display_configuration_capture_event_| to prevent more
+      // captures from occurring on a different thread while the displays
+      // are being reconfigured.
+      CHECK(display_configuration_capture_event_.TimedWait(
+                base::TimeDelta::FromSeconds(
+                    kDisplayReconfigurationTimeoutInSeconds)));
+    } else {
+      ScreenConfigurationChanged();
+      // Now that the configuration has changed, signal the event.
+      display_configuration_capture_event_.Signal();
+    }
   }
-  InvalidateRects(rects);
 }
 
 void CapturerMac::ScreenRefreshCallback(CGRectCount count,
@@ -442,22 +611,20 @@ void CapturerMac::DisplaysReconfiguredCallback(
     CGDirectDisplayID display,
     CGDisplayChangeSummaryFlags flags,
     void *user_parameter) {
-  if (display == CGMainDisplayID()) {
-    CapturerMac *capturer = reinterpret_cast<CapturerMac *>(user_parameter);
-    if (flags & kCGDisplayBeginConfigurationFlag) {
-      capturer->EnableCapture(false);
-    } else {
-      capturer->EnableCapture(true);
-      capturer->ScreenConfigurationChanged();
-    }
-  }
+  CapturerMac *capturer = reinterpret_cast<CapturerMac *>(user_parameter);
+  capturer->DisplaysReconfigured(display, flags);
 }
 
 }  // namespace
 
 // static
 Capturer* Capturer::Create() {
-  return new CapturerMac();
+  CapturerMac* capturer = new CapturerMac();
+  if (!capturer->Init()) {
+    delete capturer;
+    capturer = NULL;
+  }
+  return capturer;
 }
 
 }  // namespace remoting
