@@ -1,4 +1,4 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
@@ -9,9 +9,11 @@
 
 namespace remoting {
 
-// Block size and format (BGRA 32 bit) are fixed.
-static const int kBlockWidth = 32;
-static const int kBlockHeight = 32;
+// Size (in pixels) of each square block used for diffing.
+// This must be a multiple of sizeof(uint64)/8.
+static const int kBlockSize = 32;
+
+// Format: BGRA 32 bit.
 static const int kBytesPerPixel = 4;
 
 // Low level functions to compare 2 blocks of pixels.
