@@ -8,9 +8,12 @@
 #include <string>
 
 namespace remoting {
-
 class SignalStrategy;
 
+namespace protocol {
+class ConnectionToClient;
+}
+
 class HostStatusObserver {
  public:
   HostStatusObserver() { }
@@ -28,7 +31,10 @@ class HostStatusObserver {
   // Called on the main thread when a client authenticates, or disconnects.
   // The observer must not tear-down ChromotingHost state on receipt of
   // this callback; it is purely informational.
-  virtual void OnAuthenticatedClientsChanged(int authenticated_clients) = 0;
+  virtual void OnClientAuthenticated(
+      remoting::protocol::ConnectionToClient* client) = 0;
+  virtual void OnClientDisconnected(
+      remoting::protocol::ConnectionToClient* client) = 0;
 
   // Called on the main thread when the host shuts down.
   virtual void OnShutdown() = 0;
