@@ -6,8 +6,10 @@
 
 #include "base/bind.h"
 #include "base/message_loop.h"
-#include "base/task.h"
+#include "base/message_loop_proxy.h"
+#include "base/sys_string_conversions.h"
 #include "base/threading/platform_thread.h"
+#include "base/utf_string_conversions.h"
 #include "remoting/base/auth_token_util.h"
 #include "remoting/host/chromoting_host.h"
 #include "remoting/host/chromoting_host_context.h"
@@ -15,7 +17,8 @@
 #include "remoting/host/host_config.h"
 #include "remoting/host/host_key_pair.h"
 #include "remoting/host/in_memory_host_config.h"
-#include "remoting/host/plugin/host_plugin_utils.h"
+#include "remoting/host/plugin/host_log_handler.h"
+#include "remoting/host/plugin/policy_hack/nat_policy.h"
 #include "remoting/host/register_support_host_request.h"
 #include "remoting/host/support_access_verifier.h"
 
@@ -24,14 +27,16 @@ namespace remoting {
 // Supported Javascript interface:
 // readonly attribute string accessCode;
 // readonly attribute int accessCodeLifetime;
+// readonly attribute string client;
 // readonly attribute int state;
 //
 // state: {
 //     DISCONNECTED,
+//     STARTING,
 //     REQUESTED_ACCESS_CODE,
 //     RECEIVED_ACCESS_CODE,
 //     CONNECTED,
-//     AFFIRMING_CONNECTION,
+//     DISCONNECTING,
 //     ERROR,
 // }
 //
@@ -42,83 +47,108 @@ namespace remoting {
 // // "auth_service:auth_token".  An example would be "oauth2:1/2a3912vd".
 // void connect(string uid, string auth_service_with_token);
 // void disconnect();
+// void localize(string (*localize_func)(string,...));
 
 namespace {
 
 const char* kAttrNameAccessCode = "accessCode";
 const char* kAttrNameAccessCodeLifetime = "accessCodeLifetime";
+const char* kAttrNameClient = "client";
 const char* kAttrNameState = "state";
 const char* kAttrNameLogDebugInfo = "logDebugInfo";
 const char* kAttrNameOnStateChanged = "onStateChanged";
 const char* kFuncNameConnect = "connect";
 const char* kFuncNameDisconnect = "disconnect";
+const char* kFuncNameLocalize = "localize";
 
 // States.
 const char* kAttrNameDisconnected = "DISCONNECTED";
+const char* kAttrNameStarting = "STARTING";
 const char* kAttrNameRequestedAccessCode = "REQUESTED_ACCESS_CODE";
 const char* kAttrNameReceivedAccessCode = "RECEIVED_ACCESS_CODE";
 const char* kAttrNameConnected = "CONNECTED";
-const char* kAttrNameAffirmingConnection = "AFFIRMING_CONNECTION";
+const char* kAttrNameDisconnecting = "DISCONNECTING";
 const char* kAttrNameError = "ERROR";
 
 const int kMaxLoginAttempts = 5;
 
 }  // namespace
 
-HostNPScriptObject::HostNPScriptObject(NPP plugin, NPObject* parent)
+HostNPScriptObject::HostNPScriptObject(
+    NPP plugin,
+    NPObject* parent,
+    PluginMessageLoopProxy::Delegate* plugin_thread_delegate)
     : plugin_(plugin),
       parent_(parent),
       state_(kDisconnected),
-      log_debug_info_func_(NULL),
-      on_state_changed_func_(NULL),
       np_thread_id_(base::PlatformThread::CurrentId()),
+      plugin_message_loop_proxy_(
+          new PluginMessageLoopProxy(plugin_thread_delegate)),
+      host_context_(plugin_message_loop_proxy_),
       failed_login_attempts_(0),
-      disconnected_event_(true, false) {
-  logger_.reset(new HostPluginLogger(this));
-  logger_->VLog(2, "HostNPScriptObject");
-  host_context_.SetUITaskPostFunction(base::Bind(
-      &HostNPScriptObject::PostTaskToNPThread, base::Unretained(this)));
+      disconnected_event_(true, false),
+      am_currently_logging_(false),
+      nat_traversal_enabled_(false),
+      policy_received_(false) {
 }
 
 HostNPScriptObject::~HostNPScriptObject() {
   CHECK_EQ(base::PlatformThread::CurrentId(), np_thread_id_);
 
+  // Shutdown DesktopEnvironment first so that it doesn't try to post
+  // tasks on the UI thread while we are stopping the host.
+  if (desktop_environment_.get()) {
+    desktop_environment_->Shutdown();
+  }
+
+  HostLogHandler::UnregisterLoggingScriptObject(this);
+
+  plugin_message_loop_proxy_->Detach();
+
+  // Stop listening for policy updates.
+  if (nat_policy_.get()) {
+    base::WaitableEvent nat_policy_stopped_(true, false);
+    nat_policy_->StopWatching(&nat_policy_stopped_);
+    nat_policy_stopped_.Wait();
+    nat_policy_.reset();
+  }
+
   // Disconnect synchronously. We cannot disconnect asynchronously
   // here because |host_context_| needs to be stopped on the plugin
   // thread, but the plugin thread may not exist after the instance
   // is destroyed.
-  destructing_.Set();
   disconnected_event_.Reset();
   DisconnectInternal();
   disconnected_event_.Wait();
 
+  // Stop all threads.
   host_context_.Stop();
-  if (log_debug_info_func_) {
-    g_npnetscape_funcs->releaseobject(log_debug_info_func_);
-  }
-  if (on_state_changed_func_) {
-    g_npnetscape_funcs->releaseobject(on_state_changed_func_);
-  }
 }
 
 bool HostNPScriptObject::Init() {
-  logger_->VLog(2, "Init");
+  VLOG(2) << "Init";
   // TODO(wez): This starts a bunch of threads, which might fail.
   host_context_.Start();
+  nat_policy_.reset(
+      policy_hack::NatPolicy::Create(host_context_.network_message_loop()));
+  nat_policy_->StartWatching(
+      base::Bind(&HostNPScriptObject::OnNatPolicyUpdate,
+                 base::Unretained(this)));
   return true;
 }
 
 bool HostNPScriptObject::HasMethod(const std::string& method_name) {
-  logger_->VLog(2, "HasMethod %s", method_name.c_str());
+  VLOG(2) << "HasMethod " << method_name;
   CHECK_EQ(base::PlatformThread::CurrentId(), np_thread_id_);
   return (method_name == kFuncNameConnect ||
-          method_name == kFuncNameDisconnect);
+          method_name == kFuncNameDisconnect ||
+          method_name == kFuncNameLocalize);
 }
 
 bool HostNPScriptObject::InvokeDefault(const NPVariant* args,
                                        uint32_t argCount,
                                        NPVariant* result) {
-  logger_->VLog(2, "InvokeDefault");
+  VLOG(2) << "InvokeDefault";
   CHECK_EQ(base::PlatformThread::CurrentId(), np_thread_id_);
   SetException("exception during default invocation");
   return false;
@@ -128,12 +158,14 @@ bool HostNPScriptObject::Invoke(const std::string& method_name,
                                 const NPVariant* args,
                                 uint32_t argCount,
                                 NPVariant* result) {
-  logger_->VLog(2, "Invoke %s", method_name.c_str());
+  VLOG(2) << "Invoke " << method_name;
   CHECK_EQ(base::PlatformThread::CurrentId(), np_thread_id_);
   if (method_name == kFuncNameConnect) {
     return Connect(args, argCount, result);
   } else if (method_name == kFuncNameDisconnect) {
     return Disconnect(args, argCount, result);
+  } else if (method_name == kFuncNameLocalize) {
+    return Localize(args, argCount, result);
   } else {
     SetException("Invoke: unknown method " + method_name);
     return false;
@@ -141,24 +173,26 @@ bool HostNPScriptObject::Invoke(const std::string& method_name,
 }
 
 bool HostNPScriptObject::HasProperty(const std::string& property_name) {
-  logger_->VLog(2, "HasProperty %s", property_name.c_str());
+  VLOG(2) << "HasProperty " << property_name;
   CHECK_EQ(base::PlatformThread::CurrentId(), np_thread_id_);
   return (property_name == kAttrNameAccessCode ||
           property_name == kAttrNameAccessCodeLifetime ||
+          property_name == kAttrNameClient ||
           property_name == kAttrNameState ||
           property_name == kAttrNameLogDebugInfo ||
           property_name == kAttrNameOnStateChanged ||
           property_name == kAttrNameDisconnected ||
+          property_name == kAttrNameStarting ||
           property_name == kAttrNameRequestedAccessCode ||
           property_name == kAttrNameReceivedAccessCode ||
           property_name == kAttrNameConnected ||
-          property_name == kAttrNameAffirmingConnection ||
+          property_name == kAttrNameDisconnecting ||
           property_name == kAttrNameError);
 }
 
 bool HostNPScriptObject::GetProperty(const std::string& property_name,
                                      NPVariant* result) {
-  logger_->VLog(2, "GetProperty %s", property_name.c_str());
+  VLOG(2) << "GetProperty " << property_name;
   CHECK_EQ(base::PlatformThread::CurrentId(), np_thread_id_);
   if (!result) {
     SetException("GetProperty: NULL result");
@@ -166,23 +200,31 @@ bool HostNPScriptObject::GetProperty(const std::string& property_name,
   }
 
   if (property_name == kAttrNameOnStateChanged) {
-    OBJECT_TO_NPVARIANT(on_state_changed_func_, *result);
+    OBJECT_TO_NPVARIANT(on_state_changed_func_.get(), *result);
     return true;
   } else if (property_name == kAttrNameLogDebugInfo) {
-    OBJECT_TO_NPVARIANT(log_debug_info_func_, *result);
+    OBJECT_TO_NPVARIANT(log_debug_info_func_.get(), *result);
     return true;
   } else if (property_name == kAttrNameState) {
     INT32_TO_NPVARIANT(state_, *result);
     return true;
   } else if (property_name == kAttrNameAccessCode) {
+    base::AutoLock auto_lock(access_code_lock_);
     *result = NPVariantFromString(access_code_);
     return true;
   } else if (property_name == kAttrNameAccessCodeLifetime) {
+    base::AutoLock auto_lock(access_code_lock_);
     INT32_TO_NPVARIANT(access_code_lifetime_.InSeconds(), *result);
     return true;
+  } else if (property_name == kAttrNameClient) {
+    *result = NPVariantFromString(client_username_);
+    return true;
   } else if (property_name == kAttrNameDisconnected) {
     INT32_TO_NPVARIANT(kDisconnected, *result);
     return true;
+  } else if (property_name == kAttrNameStarting) {
+    INT32_TO_NPVARIANT(kStarting, *result);
+    return true;
   } else if (property_name == kAttrNameRequestedAccessCode) {
     INT32_TO_NPVARIANT(kRequestedAccessCode, *result);
     return true;
@@ -192,8 +234,8 @@ bool HostNPScriptObject::GetProperty(const std::string& property_name,
   } else if (property_name == kAttrNameConnected) {
     INT32_TO_NPVARIANT(kConnected, *result);
     return true;
-  } else if (property_name == kAttrNameAffirmingConnection) {
-    INT32_TO_NPVARIANT(kAffirmingConnection, *result);
+  } else if (property_name == kAttrNameDisconnecting) {
+    INT32_TO_NPVARIANT(kDisconnecting, *result);
     return true;
   } else if (property_name == kAttrNameError) {
     INT32_TO_NPVARIANT(kError, *result);
@@ -206,18 +248,12 @@ bool HostNPScriptObject::GetProperty(const std::string& property_name,
 
 bool HostNPScriptObject::SetProperty(const std::string& property_name,
                                      const NPVariant* value) {
-  logger_->VLog(2, "SetProperty %s", property_name.c_str());
+  VLOG(2) <<  "SetProperty " << property_name;
   CHECK_EQ(base::PlatformThread::CurrentId(), np_thread_id_);
 
   if (property_name == kAttrNameOnStateChanged) {
     if (NPVARIANT_IS_OBJECT(*value)) {
-      if (on_state_changed_func_) {
-        g_npnetscape_funcs->releaseobject(on_state_changed_func_);
-      }
       on_state_changed_func_ = NPVARIANT_TO_OBJECT(*value);
-      if (on_state_changed_func_) {
-        g_npnetscape_funcs->retainobject(on_state_changed_func_);
-      }
       return true;
     } else {
       SetException("SetProperty: unexpected type for property " +
@@ -228,13 +264,8 @@ bool HostNPScriptObject::SetProperty(const std::string& property_name,
 
   if (property_name == kAttrNameLogDebugInfo) {
     if (NPVARIANT_IS_OBJECT(*value)) {
-      if (log_debug_info_func_) {
-        g_npnetscape_funcs->releaseobject(log_debug_info_func_);
-      }
       log_debug_info_func_ = NPVARIANT_TO_OBJECT(*value);
-      if (log_debug_info_func_) {
-        g_npnetscape_funcs->retainobject(log_debug_info_func_);
-      }
+      HostLogHandler::RegisterLoggingScriptObject(this);
       return true;
     } else {
       SetException("SetProperty: unexpected type for property " +
@@ -247,13 +278,13 @@ bool HostNPScriptObject::SetProperty(const std::string& property_name,
 }
 
 bool HostNPScriptObject::RemoveProperty(const std::string& property_name) {
-  logger_->VLog(2, "RemoveProperty %s", property_name.c_str());
+  VLOG(2) << "RemoveProperty " << property_name;
   CHECK_EQ(base::PlatformThread::CurrentId(), np_thread_id_);
   return false;
 }
 
 bool HostNPScriptObject::Enumerate(std::vector<std::string>* values) {
-  logger_->VLog(2, "Enumerate");
+  VLOG(2) << "Enumerate";
   CHECK_EQ(base::PlatformThread::CurrentId(), np_thread_id_);
   const char* entries[] = {
     kAttrNameAccessCode,
@@ -262,11 +293,13 @@ bool HostNPScriptObject::Enumerate(std::vector<std::string>* values) {
     kAttrNameOnStateChanged,
     kFuncNameConnect,
     kFuncNameDisconnect,
+    kFuncNameLocalize,
     kAttrNameDisconnected,
+    kAttrNameStarting,
     kAttrNameRequestedAccessCode,
     kAttrNameReceivedAccessCode,
     kAttrNameConnected,
-    kAttrNameAffirmingConnection,
+    kAttrNameDisconnecting,
     kAttrNameError
   };
   for (size_t i = 0; i < arraysize(entries); ++i) {
@@ -283,22 +316,47 @@ void HostNPScriptObject::OnSignallingDisconnected() {
 }
 
 void HostNPScriptObject::OnAccessDenied() {
-  DCHECK_EQ(MessageLoop::current(), host_context_.network_message_loop());
+  DCHECK(host_context_.network_message_loop()->BelongsToCurrentThread());
 
   ++failed_login_attempts_;
   if (failed_login_attempts_ == kMaxLoginAttempts)
     DisconnectInternal();
 }
 
-void HostNPScriptObject::OnAuthenticatedClientsChanged(int clients_connected) {
-  DCHECK_NE(base::PlatformThread::CurrentId(), np_thread_id_);
-  OnStateChanged(clients_connected ? kConnected : kDisconnected);
+void HostNPScriptObject::OnClientAuthenticated(
+    remoting::protocol::ConnectionToClient* client) {
+  DCHECK_EQ(MessageLoop::current(), host_context_.main_message_loop());
+
+  if (state_ == kDisconnecting) {
+    // Ignore the new connection if we are disconnecting.
+    return;
+  }
+
+  client_username_ = client->session()->jid();
+  size_t pos = client_username_.find('/');
+  if (pos != std::string::npos)
+    client_username_.replace(pos, std::string::npos, "");
+  LOG(INFO) << "Client " << client_username_ << " connected.";
+  SetState(kConnected);
+}
+
+void HostNPScriptObject::OnClientDisconnected(
+    remoting::protocol::ConnectionToClient* client) {
+  DCHECK_EQ(MessageLoop::current(), host_context_.main_message_loop());
+
+  client_username_.clear();
+
+  // Disconnect the host when a client disconnects.
+  DisconnectInternal();
 }
 
 void HostNPScriptObject::OnShutdown() {
   DCHECK_EQ(MessageLoop::current(), host_context_.main_message_loop());
 
-  OnStateChanged(kDisconnected);
+  host_ = NULL;
+  if (state_ != kDisconnected) {
+    SetState(kDisconnected);
+  }
 }
 
 // string uid, string auth_token
@@ -307,13 +365,18 @@ bool HostNPScriptObject::Connect(const NPVariant* args,
                                  NPVariant* result) {
   CHECK_EQ(base::PlatformThread::CurrentId(), np_thread_id_);
 
-  LogDebugInfo("Connecting...");
+  LOG(INFO) << "Connecting...";
 
   if (arg_count != 2) {
     SetException("connect: bad number of arguments");
     return false;
   }
 
+  if (state_ != kDisconnected) {
+    SetException("connect: can be called only when disconnected");
+    return false;
+  }
+
   std::string uid = StringFromNPVariant(args[0]);
   if (uid.empty()) {
     SetException("connect: bad uid argument");
@@ -330,22 +393,47 @@ bool HostNPScriptObject::Connect(const NPVariant* args,
     return false;
   }
 
-  ConnectInternal(uid, auth_token, auth_service);
+  ReadPolicyAndConnect(uid, auth_token, auth_service);
 
   return true;
 }
 
-void HostNPScriptObject::ConnectInternal(
+void HostNPScriptObject::ReadPolicyAndConnect(const std::string& uid,
+                                              const std::string& auth_token,
+                                              const std::string& auth_service) {
+  if (MessageLoop::current() != host_context_.main_message_loop()) {
+    host_context_.main_message_loop()->PostTask(
+        FROM_HERE, base::Bind(
+            &HostNPScriptObject::ReadPolicyAndConnect, base::Unretained(this),
+            uid, auth_token, auth_service));
+    return;
+  }
+
+  SetState(kStarting);
+
+  // Only proceed to FinishConnect() if at least one policy update has been
+  // received.
+  if (policy_received_) {
+    FinishConnect(uid, auth_token, auth_service);
+  } else {
+    // Otherwise, create the policy watcher, and thunk the connect.
+    pending_connect_ =
+        base::Bind(&HostNPScriptObject::FinishConnect,
+                   base::Unretained(this), uid, auth_token, auth_service);
+  }
+}
+
+void HostNPScriptObject::FinishConnect(
     const std::string& uid,
     const std::string& auth_token,
     const std::string& auth_service) {
-  if (MessageLoop::current() != host_context_.main_message_loop()) {
-    host_context_.main_message_loop()->PostTask(
-        FROM_HERE,
-        NewRunnableMethod(this, &HostNPScriptObject::ConnectInternal,
-                          uid, auth_token, auth_service));
+  DCHECK_EQ(MessageLoop::current(), host_context_.main_message_loop());
+
+  if (state_ != kStarting) {
+    // Host has been stopped while we were fetching policy.
     return;
   }
+
   // Store the supplied user ID and token to the Host configuration.
   scoped_refptr<MutableHostConfig> host_config = new InMemoryHostConfig();
   host_config->SetString(kXmppLoginConfigPath, uid);
@@ -370,30 +458,39 @@ void HostNPScriptObject::ConnectInternal(
           base::Bind(&HostNPScriptObject::OnReceivedSupportID,
                      base::Unretained(this),
                      access_verifier.get()))) {
-    OnStateChanged(kDisconnected);
+    SetState(kError);
     return;
   }
 
-  // Create the Host.
-  DesktopEnvironment* desktop_environment =
-      DesktopEnvironment::Create(&host_context_);
-  // TODO(sergeyu): Use firewall traversal policy settings here.
-  scoped_refptr<ChromotingHost> host =
-      ChromotingHost::Create(&host_context_, host_config, desktop_environment,
-                             access_verifier.release(), logger_.get(), false);
-  host->AddStatusObserver(this);
-  host->AddStatusObserver(register_request.get());
-  host->set_it2me(true);
+  // Create DesktopEnvironment.
+  desktop_environment_.reset(DesktopEnvironment::Create(&host_context_));
+  if (desktop_environment_.get() == NULL) {
+    SetState(kError);
+    return;
+  }
 
-  // Nothing went wrong, so lets save the host, config and request.
-  host_ = host;
+  // Beyond this point nothing can fail, so save the config and request.
   host_config_ = host_config;
   register_request_.reset(register_request.release());
 
+  // Create the Host.
+  LOG(INFO) << "Connecting with NAT state: " << nat_traversal_enabled_;
+  host_ = ChromotingHost::Create(
+      &host_context_, host_config_, desktop_environment_.get(),
+      access_verifier.release(), nat_traversal_enabled_);
+  host_->AddStatusObserver(this);
+  host_->AddStatusObserver(register_request_.get());
+  host_->set_it2me(true);
+
+  {
+    base::AutoLock auto_lock(ui_strings_lock_);
+    host_->SetUiStrings(ui_strings_);
+  }
+
   // Start the Host.
   host_->Start();
 
-  OnStateChanged(kRequestedAccessCode);
+  SetState(kRequestedAccessCode);
   return;
 }
 
@@ -411,98 +508,263 @@ bool HostNPScriptObject::Disconnect(const NPVariant* args,
   return true;
 }
 
+bool HostNPScriptObject::Localize(const NPVariant* args,
+                                  uint32_t arg_count,
+                                  NPVariant* result) {
+  CHECK_EQ(base::PlatformThread::CurrentId(), np_thread_id_);
+  if (arg_count != 1) {
+    SetException("localize: bad number of arguments");
+    return false;
+  }
+
+  if (NPVARIANT_IS_OBJECT(args[0])) {
+    ScopedRefNPObject localize_func(NPVARIANT_TO_OBJECT(args[0]));
+    LocalizeStrings(localize_func.get());
+    return true;
+  } else {
+    SetException("localize: unexpected type for argument 1");
+    return false;
+  }
+}
+
 void HostNPScriptObject::DisconnectInternal() {
   if (MessageLoop::current() != host_context_.main_message_loop()) {
     host_context_.main_message_loop()->PostTask(
-        FROM_HERE,
-        NewRunnableMethod(this, &HostNPScriptObject::DisconnectInternal));
+        FROM_HERE, base::Bind(&HostNPScriptObject::DisconnectInternal,
+                              base::Unretained(this)));
     return;
   }
 
-  if (!host_) {
-    disconnected_event_.Signal();
-    return;
-  }
+  switch (state_) {
+    case kDisconnected:
+      disconnected_event_.Signal();
+      return;
 
-  host_->Shutdown(
-      NewRunnableMethod(this, &HostNPScriptObject::OnShutdownFinished));
+    case kStarting:
+      SetState(kDisconnecting);
+      SetState(kDisconnected);
+      disconnected_event_.Signal();
+      return;
+
+    case kDisconnecting:
+      return;
+
+    default:
+      DCHECK(host_);
+      SetState(kDisconnecting);
+      host_->Shutdown(
+          NewRunnableMethod(this, &HostNPScriptObject::OnShutdownFinished));
+  }
 }
 
 void HostNPScriptObject::OnShutdownFinished() {
   DCHECK_EQ(MessageLoop::current(), host_context_.main_message_loop());
 
-  host_ = NULL;
-  register_request_.reset();
-  host_config_ = NULL;
   disconnected_event_.Signal();
 }
 
+void HostNPScriptObject::OnNatPolicyUpdate(bool nat_traversal_enabled) {
+  if (MessageLoop::current() != host_context_.main_message_loop()) {
+    host_context_.main_message_loop()->PostTask(
+        FROM_HERE,
+        base::Bind(&HostNPScriptObject::OnNatPolicyUpdate,
+                   base::Unretained(this), nat_traversal_enabled));
+    return;
+  }
+
+  VLOG(2) << "OnNatPolicyUpdate: " << nat_traversal_enabled;
+
+  // When transitioning from enabled to disabled, force disconnect any
+  // existing session.
+  if (nat_traversal_enabled_ && !nat_traversal_enabled) {
+    DisconnectInternal();
+  }
+
+  policy_received_ = true;
+  nat_traversal_enabled_ = nat_traversal_enabled;
+
+  if (!pending_connect_.is_null()) {
+    pending_connect_.Run();
+    pending_connect_.Reset();
+  }
+}
+
 void HostNPScriptObject::OnReceivedSupportID(
     SupportAccessVerifier* access_verifier,
     bool success,
     const std::string& support_id,
     const base::TimeDelta& lifetime) {
-  CHECK_NE(base::PlatformThread::CurrentId(), np_thread_id_);
+  DCHECK(host_context_.network_message_loop()->BelongsToCurrentThread());
 
   if (!success) {
-    // TODO(wez): Replace the success/fail flag with full error reporting.
+    host_context_.main_message_loop()->PostTask(FROM_HERE, base::Bind(
+        &HostNPScriptObject::NotifyAccessCode, base::Unretained(this), false));
     DisconnectInternal();
     return;
   }
 
-  // Inform the AccessVerifier of our Support-Id, for authentication.
   access_verifier->OnIT2MeHostRegistered(success, support_id);
+  std::string access_code = support_id + access_verifier->host_secret();
+  host_->set_access_code(access_code);
 
-  // Combine the Support Id with the Host Id to make the Access Code.
-  // TODO(wez): Locking, anyone?
-  access_code_ = support_id + access_verifier->host_secret();
-  access_code_lifetime_ = lifetime;
+  {
+    base::AutoLock lock(access_code_lock_);
+    access_code_ = access_code;
+    access_code_lifetime_ = lifetime;
+  }
 
-  // Tell the ChromotingHost the access code, to use as shared-secret.
-  host_->set_access_code(access_code_);
-
-  // Let the caller know that life is good.
-  OnStateChanged(kReceivedAccessCode);
+  host_context_.main_message_loop()->PostTask(FROM_HERE, base::Bind(
+      &HostNPScriptObject::NotifyAccessCode, base::Unretained(this), true));
 }
 
-void HostNPScriptObject::OnStateChanged(State state) {
-  if (destructing_.IsSet()) {
-    return;
+void HostNPScriptObject::NotifyAccessCode(bool success) {
+  DCHECK_EQ(MessageLoop::current(), host_context_.main_message_loop());
+  if (state_ == kRequestedAccessCode) {
+    SetState(success ? kReceivedAccessCode : kError);
   }
+}
 
-  if (!host_context_.IsUIThread()) {
-    host_context_.PostToUIThread(
-        FROM_HERE,
-        NewRunnableMethod(this, &HostNPScriptObject::OnStateChanged, state));
+void HostNPScriptObject::SetState(State state) {
+  DCHECK_EQ(MessageLoop::current(), host_context_.main_message_loop());
+  switch (state_) {
+    case kDisconnected:
+      DCHECK(state == kStarting ||
+             state == kError) << state;
+      break;
+    case kStarting:
+      DCHECK(state == kRequestedAccessCode ||
+             state == kDisconnecting ||
+             state == kError) << state;
+      break;
+    case kRequestedAccessCode:
+      DCHECK(state == kReceivedAccessCode ||
+             state == kDisconnecting ||
+             state == kError) << state;
+      break;
+    case kReceivedAccessCode:
+      DCHECK(state == kConnected ||
+             state == kDisconnecting ||
+             state == kError) << state;
+      break;
+    case kConnected:
+      DCHECK(state == kDisconnecting ||
+             state == kDisconnected ||
+             state == kError) << state;
+      break;
+    case kDisconnecting:
+      DCHECK(state == kDisconnected) << state;
+      break;
+    case kError:
+      DCHECK(state == kDisconnecting) << state;
+      break;
+  };
+  state_ = state;
+  NotifyStateChanged(state);
+}
+
+void HostNPScriptObject::NotifyStateChanged(State state) {
+  if (!plugin_message_loop_proxy_->BelongsToCurrentThread()) {
+    plugin_message_loop_proxy_->PostTask(
+        FROM_HERE, base::Bind(&HostNPScriptObject::NotifyStateChanged,
+                              base::Unretained(this), state));
     return;
   }
-  state_ = state;
-  if (on_state_changed_func_) {
-    logger_->VLog(2, "Calling state changed %s", state);
-    bool is_good = InvokeAndIgnoreResult(on_state_changed_func_, NULL, 0);
+  if (on_state_changed_func_.get()) {
+    VLOG(2) << "Calling state changed " << state;
+    bool is_good = InvokeAndIgnoreResult(on_state_changed_func_.get(),
+                                         NULL, 0);
     LOG_IF(ERROR, !is_good) << "OnStateChanged failed";
   }
 }
 
-void HostNPScriptObject::LogDebugInfo(const std::string& message) {
-  if (!host_context_.IsUIThread()) {
-    host_context_.PostToUIThread(
-        FROM_HERE,
-        NewRunnableMethod(this, &HostNPScriptObject::LogDebugInfo, message));
-    return;
+void HostNPScriptObject::PostLogDebugInfo(const std::string& message) {
+  if (plugin_message_loop_proxy_->BelongsToCurrentThread()) {
+    // Make sure we're not currently processing a log message.
+    // We only need to check this if we're on the plugin thread.
+    if (am_currently_logging_)
+      return;
   }
-  if (log_debug_info_func_) {
-    NPVariant* arg = new NPVariant();
-    STRINGZ_TO_NPVARIANT(message.c_str(), *arg);
-    bool is_good = InvokeAndIgnoreResult(log_debug_info_func_, arg, 1);
-    LOG_IF(ERROR, !is_good) << "LogDebugInfo failed";
+
+  // Always post (even if we're already on the correct thread) so that debug
+  // log messages are shown in the correct order.
+  plugin_message_loop_proxy_->PostTask(
+      FROM_HERE, base::Bind(&HostNPScriptObject::LogDebugInfo,
+                            base::Unretained(this), message));
+}
+
+void HostNPScriptObject::LogDebugInfo(const std::string& message) {
+  if (log_debug_info_func_.get()) {
+    am_currently_logging_ = true;
+    NPVariant log_message;
+    STRINGZ_TO_NPVARIANT(message.c_str(), log_message);
+    bool is_good = InvokeAndIgnoreResult(log_debug_info_func_.get(),
+                                         &log_message, 1);
+    if (!is_good) {
+      LOG(ERROR) << "ERROR - LogDebugInfo failed\n";
+    }
+    am_currently_logging_ = false;
   }
 }
 
 void HostNPScriptObject::SetException(const std::string& exception_string) {
   CHECK_EQ(base::PlatformThread::CurrentId(), np_thread_id_);
   g_npnetscape_funcs->setexception(parent_, exception_string.c_str());
-  LogDebugInfo(exception_string);
+  LOG(INFO) << exception_string;
+}
+
+void HostNPScriptObject::LocalizeStrings(NPObject* localize_func) {
+  DCHECK(plugin_message_loop_proxy_->BelongsToCurrentThread());
+
+  UiStrings ui_strings;
+  string16 direction;
+  LocalizeString(localize_func, "@@bidi_dir", &direction);
+  ui_strings.direction = UTF16ToUTF8(direction) == "rtl" ?
+      remoting::UiStrings::RTL : remoting::UiStrings::LTR;
+  LocalizeString(localize_func, /*i18n-content*/"PRODUCT_NAME",
+                 &ui_strings.product_name);
+  LocalizeString(localize_func, /*i18n-content*/"DISCONNECT_BUTTON",
+                 &ui_strings.disconnect_button_text);
+  LocalizeString(localize_func,
+#if defined(OS_WIN)
+      /*i18n-content*/"DISCONNECT_BUTTON_PLUS_SHORTCUT_WINDOWS",
+#elif defined(OS_MACOSX)
+      /*i18n-content*/"DISCONNECT_BUTTON_PLUS_SHORTCUT_MAC_OS_X",
+#else
+      /*i18n-content*/"DISCONNECT_BUTTON_PLUS_SHORTCUT_LINUX",
+#endif
+      &ui_strings.disconnect_button_text_plus_shortcut);
+  LocalizeString(localize_func, /*i18n-content*/"CONTINUE_PROMPT",
+                 &ui_strings.continue_prompt);
+  LocalizeString(localize_func, /*i18n-content*/"CONTINUE_BUTTON",
+                 &ui_strings.continue_button_text);
+  LocalizeString(localize_func, /*i18n-content*/"STOP_SHARING_BUTTON",
+                 &ui_strings.stop_sharing_button_text);
+  LocalizeString(localize_func, /*i18n-content*/"MESSAGE_SHARED",
+                 &ui_strings.disconnect_message);
+
+  base::AutoLock auto_lock(ui_strings_lock_);
+  ui_strings_ = ui_strings;
+}
+
+bool HostNPScriptObject::LocalizeString(NPObject* localize_func,
+                                        const char* tag, string16* result) {
+  NPVariant args[2];
+  STRINGZ_TO_NPVARIANT(tag, args[0]);
+  NPVariant np_result;
+  bool is_good = g_npnetscape_funcs->invokeDefault(
+      plugin_, localize_func, &args[0], 1, &np_result);
+  if (!is_good) {
+    LOG(ERROR) << "Localization failed for " << tag;
+    return false;
+  }
+  std::string translation = StringFromNPVariant(np_result);
+  g_npnetscape_funcs->releasevariantvalue(&np_result);
+  if (translation.empty()) {
+    LOG(ERROR) << "Missing translation for " << tag;
+    return false;
+  }
+  *result = UTF8ToUTF16(translation);
+  return true;
 }
 
 bool HostNPScriptObject::InvokeAndIgnoreResult(NPObject* func,
@@ -516,22 +778,4 @@ bool HostNPScriptObject::InvokeAndIgnoreResult(NPObject* func,
   return is_good;
 }
 
-void HostNPScriptObject::PostTaskToNPThread(
-    const tracked_objects::Location& from_here, Task* task) {
-  // The NPAPI functions cannot make use of |from_here|, but this method is
-  // passed as a callback to ChromotingHostContext, so it needs to have the
-  // appropriate signature.
-
-  // Can be called from any thread.
-  g_npnetscape_funcs->pluginthreadasynccall(plugin_,
-                                            &NPTaskSpringboard,
-                                            task);
-}
-
-void HostNPScriptObject::NPTaskSpringboard(void* task) {
-  Task* real_task = reinterpret_cast<Task*>(task);
-  real_task->Run();
-  delete real_task;
-}
-
 }  // namespace remoting
