@@ -59,7 +59,10 @@ class RegisterSupportHostRequest : public HostStatusObserver {
   virtual void OnSignallingConnected(SignalStrategy* signal_strategy,
                                      const std::string& full_jid) OVERRIDE;
   virtual void OnSignallingDisconnected() OVERRIDE;
-  virtual void OnAuthenticatedClientsChanged(int clients_connected) OVERRIDE;
+  virtual void OnClientAuthenticated(
+      remoting::protocol::ConnectionToClient* client) OVERRIDE;
+  virtual void OnClientDisconnected(
+      remoting::protocol::ConnectionToClient* client) OVERRIDE;
   virtual void OnAccessDenied() OVERRIDE;
   virtual void OnShutdown() OVERRIDE;
 
