@@ -23,8 +23,8 @@ class InputStub {
   InputStub() {};
   virtual ~InputStub() {};
 
-  virtual void InjectKeyEvent(const KeyEvent* event, Task* done) = 0;
-  virtual void InjectMouseEvent(const MouseEvent* event, Task* done) = 0;
+  virtual void InjectKeyEvent(const KeyEvent& event) = 0;
+  virtual void InjectMouseEvent(const MouseEvent& event) = 0;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(InputStub);
