@@ -1,10 +1,12 @@
-// Copyright (c) 2010 The Chromium Authors. All rights reserved.
+// Copyright (c) 2011 The Chromium Authors. All rights reserved.
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
 #include <vector>
 
+#include "base/bind.h"
 #include "base/message_loop.h"
+#include "base/message_loop_proxy.h"
 #include "base/string_number_conversions.h"
 #include "net/base/io_buffer.h"
 #include "remoting/proto/video.pb.h"
@@ -64,11 +66,18 @@ class RtpVideoReaderTest : public testing::Test,
 
   void Reset() {
     session_.reset(new FakeSession());
-    reader_.reset(new RtpVideoReader());
-    reader_->Init(session_.get(), this);
+    reader_.reset(new RtpVideoReader(
+        base::MessageLoopProxy::current()));
+    reader_->Init(session_.get(), this,
+                  base::Bind(&RtpVideoReaderTest::OnReaderInitialized,
+                             base::Unretained(this)));
     received_packets_.clear();
   }
 
+  void OnReaderInitialized(bool success) {
+    ASSERT_TRUE(success);
+  }
+
   void InitData(int size) {
     data_.resize(size);
     for (int i = 0; i < size; ++i) {
