@@ -5,9 +5,14 @@
 #ifndef REMOTING_PROTOCOL_RTP_VIDEO_WRITER_H_
 #define REMOTING_PROTOCOL_RTP_VIDEO_WRITER_H_
 
+#include "base/memory/scoped_ptr.h"
 #include "remoting/protocol/rtp_writer.h"
 #include "remoting/protocol/video_writer.h"
 
+namespace base {
+class MessageLoopProxy;
+}  // namespace base
+
 namespace remoting {
 namespace protocol {
 
@@ -15,11 +20,12 @@ class Session;
 
 class RtpVideoWriter : public VideoWriter {
  public:
-  RtpVideoWriter();
+  RtpVideoWriter(base::MessageLoopProxy* message_loop);
   virtual ~RtpVideoWriter();
 
   // VideoWriter interface.
-  virtual void Init(protocol::Session* session) OVERRIDE;
+  virtual void Init(protocol::Session* session,
+                    const InitializedCallback& callback) OVERRIDE;
   virtual void Close() OVERRIDE;
 
   // VideoStub interface.
@@ -28,7 +34,14 @@ class RtpVideoWriter : public VideoWriter {
   virtual int GetPendingPackets() OVERRIDE;
 
  private:
+  void OnChannelReady(bool rtp, net::Socket* socket);
+
+  bool initialized_;
+  InitializedCallback initialized_callback_;
+
+  scoped_ptr<net::Socket> rtp_channel_;
   RtpWriter rtp_writer_;
+  scoped_ptr<net::Socket> rtcp_channel_;
 
   DISALLOW_COPY_AND_ASSIGN(RtpVideoWriter);
 };
