@@ -2,12 +2,14 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <asm/prctl.h>
 #include <asm/unistd.h>
 #include <dirent.h>
 #include <errno.h>
 #include <poll.h>
 #include <pthread.h>
 #include <pty.h>
+#include <setjmp.h>
 #include <sys/param.h>
 #include <sys/prctl.h>
 #include <sys/ptrace.h>
@@ -24,6 +26,7 @@
 
 #include "debug.h"
 #include "sandbox_impl.h"
+#include "tls_setup.h"
 
 #ifdef DEBUG
 #define MSG(fmt, ...) printf(fmt, ##__VA_ARGS__)
@@ -31,6 +34,8 @@
 #define MSG(fmt, ...) do { } while (0)
 #endif
 
+#pragma GCC diagnostic ignored "-Wunused-value"
+
 // Checks that "expr" evaluates to "true". Returns value of "expr".
 #define CHECK(expr)                                                           \
   ({ typeof (expr) check_res = (expr);                                        \
@@ -201,6 +206,10 @@ static int clone_func(void *x) {
 }
 
 #if defined(__i386__)
+static void set_gs(int gs) {
+  asm volatile("mov %0, %%gs" : : "r"(gs));
+}
+
 static int get_gs() {
   int gs;
   asm volatile("mov %%gs, %0" : "=r"(gs));
@@ -654,10 +663,6 @@ TEST(test_stat_enabled) {
   CHECK_ERRNO(stat("path-that-does-not-exist", &st), ENOENT);
 }
 
-// TODO(mseaborn): It would be good to test the error cases for the
-// sandbox's treatment of SysV SHM.  However, that leads to leaks of
-// SysV shared memory segments, because the sandbox prevents us from
-// freeing a segment with shmctl() if it gets a SysV-related error.
 TEST(test_sysv_shared_memory) {
   StartSeccompSandbox();
   int shmid;
@@ -670,6 +675,60 @@ TEST(test_sysv_shared_memory) {
   CHECK_SUCCEEDS(shmctl(shmid, IPC_RMID, NULL) == 0);
 }
 
+TEST(test_preallocated_sysv_shared_memory) {
+  int shmid;
+  CHECK_SUCCEEDS((shmid = shmget(IPC_PRIVATE, 0x1000, 0700)) != -1);
+  pid_t pid;
+  int status;
+  CHECK_SUCCEEDS((pid = fork()) != -1);
+
+  if (pid == 0) {
+    // The sandbox should disallow attaching to our segment, as it has no
+    // knowledge of this particular shmid.
+    CHECK_SUCCEEDS((pid = fork()) != -1);
+    if (pid == 0) {
+      StartSeccompSandbox();
+      void *addr;
+      CHECK_ERRNO((addr = shmat(shmid, NULL, 0)) == MAP_FAILED, EINVAL);
+      _exit(0);
+    }
+    CHECK_SUCCEEDS(waitpid(pid, &status, 0) == pid);
+    CHECK(WIFEXITED(status) && !WEXITSTATUS(status));
+  
+    // Of course, without the sandbox, all of this should work. Even from a
+    // child process. SysV shared memory is accessible to any process owned
+    // by the correct uid/gid combination.
+    CHECK_SUCCEEDS((pid = fork()) != -1);
+    if (pid == 0) {
+      void *addr;
+      CHECK_SUCCEEDS((addr = shmat(shmid, NULL, 0)) != MAP_FAILED);
+      _exit(0);
+    }
+    CHECK_SUCCEEDS(waitpid(pid, &status, 0) == pid);
+    CHECK(WIFEXITED(status) && !WEXITSTATUS(status));
+  
+    // And if we disable SysV checking, it should also work in the sandbox.
+    CHECK_SUCCEEDS((pid = fork()) != -1);
+    if (pid == 0) {
+      playground::g_policy.unrestricted_sysv_mem = true;
+      StartSeccompSandbox();
+      void *addr;
+      CHECK_SUCCEEDS((addr = shmat(shmid, NULL, 0)) != MAP_FAILED);
+      _exit(0);
+    }
+    CHECK_SUCCEEDS(waitpid(pid, &status, 0) == pid);
+    CHECK(WIFEXITED(status) && !WEXITSTATUS(status));
+    _exit(0);
+  }
+
+  // We use child processes to make sure that we can clean up even in the
+  // presence of unittest failures. This way, we will not leak any SysV
+  // memory segments.
+  CHECK_SUCCEEDS(waitpid(pid, &status, 0) == pid);
+  CHECK_SUCCEEDS(shmctl(shmid, IPC_RMID, NULL) == 0);
+  CHECK(WIFEXITED(status) && !WEXITSTATUS(status));
+}
+
 static int g_value;
 
 static void signal_handler(int sig) {
@@ -763,6 +822,7 @@ TEST(test_signal_handler) {
 
 TEST(test_sigaction_handler) {
   struct sigaction act;
+  memset(&act, 0, sizeof(act));
   act.sa_sigaction = sigaction_handler;
   sigemptyset(&act.sa_mask);
   act.sa_flags = SA_SIGINFO;
@@ -1102,8 +1162,21 @@ TEST(test_backtrace) {
           // of the child. We don't want to be confused if the sandbox tried
           // to rewrite the code (this shouldn't happen for
           // linux_syscall_support.h, but will happen for other system calls).
+          unsigned short insn = 0;
           #if defined(__x86_64__)
-          void *insn_addr = (void *)(ip + 2);
+          // Clang's built-in assembler can't generate short jumps. So, we need
+          // to inspect the instruction to tell whether it is a short or a long
+          // jump.
+          if (!safe_memcpy(&insn, (void *)ip, 1) ||
+              (insn != 0xEB /* jmp */ && insn != 0xE9 /* jmpq */)) {
+            break;
+          }
+          void *insn_addr;
+          if (insn == 0xEB /* jmp */) {
+            insn_addr = (void *)(ip + 2);
+          } else {
+            insn_addr = (void *)(ip + 5);
+          }
           unsigned short expected_insn = 0x050F; /* SYSCALL */
           #elif defined(__i386__)
           void *insn_addr = (void *)(ip - 2);
@@ -1111,7 +1184,6 @@ TEST(test_backtrace) {
           #else
           #error Unsupported target platform
           #endif
-          unsigned short insn = 0;
           if (!safe_memcpy(&insn, insn_addr, sizeof(insn)) ||
               insn != expected_insn) {
             break;
@@ -1231,6 +1303,134 @@ TEST(test_concurrent_sendmsg_and_recvmsg) {
   CHECK(err == 0);
 }
 
+TEST(test_tls_setup_via_arch_prctl) {
+#if defined(__x86_64__)
+  playground::AddTlsSetupSyscall();
+  StartSeccompSandbox();
+  void *new_tls = (void *) &new_tls;
+  void *original = get_tls_base();
+  // If this call partly succeeds but still returns an error, we will
+  // probably get a segfault, because libc's syscall() sets errno on
+  // error.
+  CHECK_SUCCEEDS(syscall(__NR_arch_prctl, ARCH_SET_FS, &new_tls) == 0);
+  void *checked = get_tls_base();
+  // Restore libc's original TLS before making any further libc calls.
+  CHECK_SUCCEEDS(syscall(__NR_arch_prctl, ARCH_SET_FS, original) == 0);
+  CHECK(checked == new_tls);
+#endif
+}
+
+TEST(test_tls_setup_via_set_thread_area) {
+#if defined(__i386__)
+  playground::AddTlsSetupSyscall();
+  StartSeccompSandbox();
+  int orig_gs = get_gs();
+  void *new_tls = (void *) &new_tls;
+  struct user_desc tls_desc;
+  tls_desc.entry_number = -1; // Allocate new segment selector
+  tls_desc.base_addr = (long) &new_tls;
+  tls_desc.limit = 0xfffff;
+  tls_desc.seg_32bit = 1;
+  tls_desc.contents = 0;
+  tls_desc.read_exec_only = 0;
+  tls_desc.limit_in_pages = 1;
+  tls_desc.seg_not_present = 0;
+  tls_desc.useable = 1;
+  // If this call partly succeeds but still returns an error, we will
+  // probably get a segfault, because libc's syscall() sets errno on
+  // error.
+  CHECK_SUCCEEDS(syscall(__NR_set_thread_area, &tls_desc) == 0);
+  CHECK(tls_desc.entry_number != (unsigned) -1);
+
+  set_gs((tls_desc.entry_number << 3) | 3);
+  void *checked = get_tls_base();
+  set_gs(orig_gs);
+  CHECK(checked == new_tls);
+#endif
+}
+
+static sigjmp_buf segv_env;
+static void segv_handler(int signo) {
+  siglongjmp(segv_env, 42);
+}
+
+TEST(test_syscall_table_is_readonly) {
+  playground::SyscallTable *entry =
+    (playground::SyscallTable *)&playground::SyscallTable::syscallTable[0];
+  StartSeccompSandbox();
+  struct sigaction act;
+  memset(&act, 0, sizeof(act));
+  act.sa_handler = segv_handler;
+  sigaction(SIGSEGV, &act, NULL);
+  int rc = sigsetjmp(segv_env, 1);
+  if (!rc) {
+    entry->handler = 0;
+    CHECK(0); // We should never get here
+  }
+  CHECK(rc == 42);
+}
+
+static void (*alignment_test_syscall_handler(void))(int *status) {
+  void (*fnc)(int *);
+#if !defined(__i386__) && !defined(__x86_64__)
+#error Unsupported target platform
+#else
+  asm(
+      // Verify that prior to pushing the return address, the stack was
+      // aligned to a 16 byte boundary. This is needed by clang when
+      // performing SSE operations on variables stored on the stack.
+      "lea  0f, %0\n"
+      "jmp  999f\n"
+#if defined(__i386__)
+    "0:movl 4(%%esp), %%eax\n"
+      "movl $0, 0(%%eax)\n"
+      "lea  4(%%esp), %%eax\n"
+      "and  $15, %%eax\n"
+      "test %%eax, %%eax\n"
+      "jne  1f\n"
+      "movl 4(%%esp), %%eax\n"
+      "movl $1, 0(%%eax)\n"
+    "1:xor  %%eax, %%eax\n"
+      "ret\n"
+#elif defined(__x86_64__)
+    "0:lea  8(%%rsp), %%rax\n"
+      "and  $15, %%rax\n"
+      "movl $0, 0(%%rdi)\n"
+      "test %%rax, %%rax\n"
+      "jne  1f\n"
+      "movl $1, 0(%%rdi)\n"
+    "1:xor  %%rax, %%rax\n"
+      "ret\n"
+#endif
+  "999:\n"
+      : "=r"(fnc));
+#endif
+  return fnc;
+}
+
+TEST(test_stack_alignment) {
+  // Temporarily install a handler for sched_yield() that checks stack
+  // alignment upon intercepting system calls. This makes sure we don't get
+  // segmentations faults in C code that is invoked as a system call wrapper.
+  playground::SyscallTable::initializeSyscallTable();
+  playground::SyscallTable *entry = (playground::SyscallTable *)
+    &playground::SyscallTable::syscallTable[__NR_sched_yield];
+  mprotect((void *)((uintptr_t)entry & -4096), 4096, PROT_READ|PROT_WRITE);
+  entry->handler = (void (*)())alignment_test_syscall_handler();
+
+  // TODO(markus): This should actually be PROT_READ, instead of
+  // PROT_READ|PROT_EXEC.
+  mprotect((void *)((uintptr_t)entry & -4096), 4096, PROT_READ|PROT_EXEC);
+
+  // In the seccomp sandbox, we should now be able to call sched_yield(&status)
+  // and if everything worked correctly, "status" will be set to "1"
+  int status = -1;
+  StartSeccompSandbox();
+  syscall(__NR_sched_yield, &status);
+  CHECK(status != -1); // syscall never got intercepted correctly
+  CHECK(status != 0);  // stack was unaligned
+}
+
 struct testcase {
   const char *test_name;
   void (*test_func)();
