@@ -7,7 +7,8 @@
 #pragma once
 
 #include "base/basictypes.h"
-#include "skia/ext/platform_device_mac.h"
+#include "base/compiler_specific.h"
+#include "skia/ext/platform_device.h"
 
 namespace skia {
 
@@ -25,7 +26,7 @@ namespace skia {
 // For us, that other bitmap will become invalid as soon as the device becomes
 // invalid, which may lead to subtle bugs. Therefore, DO NOT ASSIGN THE
 // DEVICE'S PIXEL DATA TO ANOTHER BITMAP, make sure you copy instead.
-class BitmapPlatformDevice : public PlatformDevice {
+class BitmapPlatformDevice : public PlatformDevice, public SkDevice {
  public:
   // |context| may be NULL.
   static BitmapPlatformDevice* Create(CGContextRef context,
@@ -42,12 +43,11 @@ class BitmapPlatformDevice : public PlatformDevice {
   // PlatformDevice overrides
   virtual CGContextRef GetBitmapContext();
   virtual void DrawToNativeContext(CGContextRef context, int x, int y,
-                                   const CGRect* src_rect);
-  virtual void MakeOpaque(int x, int y, int width, int height);
+                                   const CGRect* src_rect) OVERRIDE;
 
   // SkDevice overrides
   virtual void setMatrixClip(const SkMatrix& transform, const SkRegion& region,
-                             const SkClipStack&);
+                             const SkClipStack&) OVERRIDE;
 
  protected:
   // Reference counted data that can be shared between multiple devices. This
@@ -61,11 +61,11 @@ class BitmapPlatformDevice : public PlatformDevice {
   // Flushes the CoreGraphics context so that the pixel data can be accessed
   // directly by Skia. Overridden from SkDevice, this is called when Skia
   // starts accessing pixel data.
-  virtual void onAccessBitmap(SkBitmap*);
+  virtual void onAccessBitmap(SkBitmap*) OVERRIDE;
 
   virtual SkDevice* onCreateCompatibleDevice(SkBitmap::Config, int width,
                                              int height, bool isOpaque,
-                                             Usage usage);
+                                             Usage usage) OVERRIDE;
 
   // Data associated with this device, guaranteed non-null. We hold a reference
   // to this object.
