@@ -34,6 +34,15 @@ SkCanvas* CreateBitmapCanvas(int width, int height, bool is_opaque) {
   return new PlatformCanvas(width, height, is_opaque);
 }
 
+SkCanvas* TryCreateBitmapCanvas(int width, int height, bool is_opaque) {
+  PlatformCanvas* canvas = new PlatformCanvas();
+  if (!canvas->initialize(width, height, is_opaque)) {
+    delete canvas;
+    canvas = NULL;
+  }
+  return canvas;
+}
+
 SkDevice* GetTopDevice(const SkCanvas& canvas) {
   SkCanvas::LayerIter iter(const_cast<SkCanvas*>(&canvas), false);
   return iter.device();
@@ -67,10 +76,23 @@ void DrawToNativeContext(SkCanvas* canvas, PlatformSurface context, int x,
     platform_device->DrawToNativeContext(context, x, y, src_rect);
 }
 
+static SkPMColor MakeOpaqueXfermodeProc(SkPMColor src, SkPMColor dst) {
+    return dst | (0xFF << SK_A32_SHIFT);
+}
+
 void MakeOpaque(SkCanvas* canvas, int x, int y, int width, int height) {
-  PlatformDevice* platform_device = GetPlatformDevice(GetTopDevice(*canvas));
-  if (platform_device)
-    platform_device->MakeOpaque(x, y, width, height);
+  if (width <= 0 || height <= 0)
+    return;
+
+  SkRect rect;
+  rect.setXYWH(SkIntToScalar(x), SkIntToScalar(y),
+               SkIntToScalar(width), SkIntToScalar(height));
+  SkPaint paint;
+  // so we don't draw anything on a device that ignores xfermodes
+  paint.setColor(0);
+  // install our custom mode
+  paint.setXfermode(new SkProcXfermode(MakeOpaqueXfermodeProc))->unref();
+  canvas->drawRect(rect, paint);
 }
 
 }  // namespace skia
