@@ -7,8 +7,7 @@
 
 namespace skia {
 
-VectorCanvas::VectorCanvas(SkDevice* device)
-    : PlatformCanvas(device->getDeviceFactory()) {
+VectorCanvas::VectorCanvas(SkDevice* device) {
   setDevice(device)->unref(); // Created with refcount 1, and setDevice refs.
 }
 
