@@ -19,12 +19,13 @@ namespace skia {
 // cooresponding Cairo APIs and outputs to a Cairo surface. Please NOTE that
 // since it is completely vectorial, the bitmap content in it is thus
 // meaningless.
-class SK_API VectorPlatformDeviceCairo : public PlatformDevice {
+class SK_API VectorPlatformDeviceCairo : public PlatformDevice,
+    public SkDevice {
  public:
   virtual ~VectorPlatformDeviceCairo();
 
-  static PlatformDevice* CreateDevice(cairo_t* context, int width, int height,
-                                      bool isOpaque);
+  static SkDevice* CreateDevice(cairo_t* context, int width, int height,
+                                bool isOpaque);
 
   // Clean up cached fonts. It is an error to call this while some
   // VectorPlatformDeviceCairo callee is still using fonts created for it by
@@ -67,18 +68,19 @@ class SK_API VectorPlatformDeviceCairo : public PlatformDevice {
                           const SkPaint&) OVERRIDE;
 
   virtual void setMatrixClip(const SkMatrix& transform, const SkRegion& region,
-                             const SkClipStack&);
+                             const SkClipStack&) OVERRIDE;
 
   // Overridden from PlatformDevice
-  virtual PlatformSurface BeginPlatformPaint();
+  virtual PlatformSurface BeginPlatformPaint() OVERRIDE;
+  virtual void DrawToNativeContext(PlatformSurface surface, int x, int y,
+                                   const PlatformRect* src_rect) OVERRIDE;
 
  protected:
-  explicit VectorPlatformDeviceCairo(PlatformSurface context,
-                                     const SkBitmap& bitmap);
+  VectorPlatformDeviceCairo(PlatformSurface context, const SkBitmap& bitmap);
 
   virtual SkDevice* onCreateCompatibleDevice(SkBitmap::Config, int width,
                                              int height, bool isOpaque,
-                                             Usage usage);
+                                             Usage usage) OVERRIDE;
 
  private:
   // Apply paint's color in the context.
