@@ -15,9 +15,8 @@
 namespace skia {
 
 //static
-PlatformDevice* VectorPlatformDeviceEmf::CreateDevice(int width, int height,
-                                                      bool is_opaque,
-                                                      HANDLE shared_section) {
+SkDevice* VectorPlatformDeviceEmf::CreateDevice(
+    int width, int height, bool is_opaque, HANDLE shared_section) {
   if (!is_opaque) {
     // TODO(maruel):  http://crbug.com/18382 When restoring a semi-transparent
     // layer, i.e. merging it, we need to rasterize it because GDI doesn't
@@ -40,7 +39,7 @@ PlatformDevice* VectorPlatformDeviceEmf::CreateDevice(int width, int height,
   // SkScalarRound(value) as SkScalarRound(value * 10). Safari is already
   // doing the same for text rendering.
   SkASSERT(shared_section);
-  PlatformDevice* device = VectorPlatformDeviceEmf::create(
+  SkDevice* device = VectorPlatformDeviceEmf::create(
       reinterpret_cast<HDC>(shared_section), width, height);
   return device;
 }
@@ -59,9 +58,7 @@ static void FillBitmapInfoHeader(int width, int height, BITMAPINFOHEADER* hdr) {
   hdr->biClrImportant = 0;
 }
 
-VectorPlatformDeviceEmf* VectorPlatformDeviceEmf::create(HDC dc,
-                                                         int width,
-                                                         int height) {
+SkDevice* VectorPlatformDeviceEmf::create(HDC dc, int width, int height) {
   InitializeDC(dc);
 
   // Link the SkBitmap to the current selected bitmap in the device context.
@@ -96,12 +93,13 @@ VectorPlatformDeviceEmf* VectorPlatformDeviceEmf::create(HDC dc,
 }
 
 VectorPlatformDeviceEmf::VectorPlatformDeviceEmf(HDC dc, const SkBitmap& bitmap)
-    : PlatformDevice(bitmap),
+    : SkDevice(bitmap),
       hdc_(dc),
       previous_brush_(NULL),
       previous_pen_(NULL),
       alpha_blend_used_(false) {
   transform_.reset();
+  SetPlatformDevice(this, this);
 }
 
 VectorPlatformDeviceEmf::~VectorPlatformDeviceEmf() {
@@ -865,4 +863,3 @@ void VectorPlatformDeviceEmf::InternalDrawBitmap(const SkBitmap& bitmap,
 }
 
 }  // namespace skia
-
